@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2022 by the Widelands Development Team
+ * Copyright (C) 2009-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,6 +16,7 @@
  *
  */
 
+#include <cstddef>
 #include <cstdlib>
 
 #include "ai/defaultai.h"
@@ -29,13 +30,8 @@ bool DefaultAI::check_enemy_sites(const Time& gametime) {
 
 	const Widelands::Map& map = game().map();
 
-	Widelands::PlayerNumber const nr_players = map.get_nrplayers();
-	uint32_t plr_in_game = 0;
 	Widelands::PlayerNumber const pn = player_number();
 
-	iterate_players_existing_novar(p, nr_players, game())++ plr_in_game;
-
-	update_player_stat(gametime);
 	// defining treshold ratio of own_strength/enemy's strength
 	uint32_t treshold_ratio = 90 + management_data.get_military_number_at(5) / 5;
 	if (type_ == AiType::kNormal) {
@@ -242,7 +238,9 @@ bool DefaultAI::check_enemy_sites(const Time& gametime) {
 					int32_t strength = calculate_strength(attackers);
 					observer.second.attack_soldiers_strength = strength;
 					assert(!attackers.empty());
-					observer.second.attack_soldiers_competency = strength * 10 / attackers.size();
+					assert(strength >= 0);
+					observer.second.attack_soldiers_competency =
+					   strength * 10 / static_cast<int32_t>(attackers.size());
 				}
 			} else {
 				observer.second.attack_soldiers_strength = 0;
@@ -646,36 +644,86 @@ void DefaultAI::count_military_vacant_positions() {
 	int32_t vacant_mil_positions_ = 0;
 	int32_t understaffed_ = 0;
 	int32_t on_stock_ = 0;
+
+	// count all soldiers too
+	int32_t soldiers_counted = 0;
+
 	for (TrainingSiteObserver tso : trainingsites) {
 		vacant_mil_positions_ +=
 		   5 * std::min<int32_t>((tso.site->soldier_control()->soldier_capacity() -
-		                          tso.site->soldier_control()->stationed_soldiers().size()),
+		                          tso.site->soldier_control()->associated_soldiers().size()),
 		                         2);
+		soldiers_counted += tso.site->soldier_control()->associated_soldiers().size();
 	}
 	for (const MilitarySiteObserver& mso : militarysites) {
 		vacant_mil_positions_ += mso.site->soldier_control()->soldier_capacity() -
-		                         mso.site->soldier_control()->stationed_soldiers().size();
+		                         mso.site->soldier_control()->associated_soldiers().size();
 		understaffed_ += mso.understaffed;
+		soldiers_counted += mso.site->soldier_control()->associated_soldiers().size();
 	}
-	vacant_mil_positions_ += understaffed_;
+
+	int32_t garrisons_count = militarysites.size();
 
 	// also available in warehouses
 	for (auto wh : warehousesites) {
-		on_stock_ += wh.site->soldier_control()->stationed_soldiers().size();
+		// TODO(tothxa): If HQ garrison handling is added, use the condition from below assert
+		//               instead.
+		if (wh.bo->is(BuildingAttribute::kPort)) {
+			assert(wh.site->soldier_control()->max_soldier_capacity() > 0);
+			++garrisons_count;
+			// TODO(tothxa): use a common constextpr with manage_ports() for maximum
+			assert(wh.site->get_desired_soldier_count() <= kPortDefaultGarrison * 3);
+			// Do the same as military sites
+			understaffed_ += kPortDefaultGarrison * 3 - wh.site->get_desired_soldier_count();
+		}
+
+		uint32_t wh_associated = wh.site->soldier_control()->associated_soldiers().size();
+		if (wh_associated < wh.site->get_desired_soldier_count()) {
+			vacant_mil_positions_ += wh.site->get_desired_soldier_count() - wh_associated;
+		} else {
+			on_stock_ += wh_associated - wh.site->get_desired_soldier_count();
+		}
+		soldiers_counted += wh_associated;
 	}
 
-	vacant_mil_positions_ += on_stock_;
+	// Unassociated soldiers on the roads are counted as stock.
+	// TODO(tothxa): Soldiers fighting for ports are still not associated, so they are counted as
+	//               on stock...
+	// TODO(tothxa): If AI warships are allowed to hold soldiers, the soldiers associated to
+	//               them will have to be counted too
+	int32_t total_soldiers = player_->count_soldiers();
+	if (soldiers_counted < total_soldiers) {
+		int32_t remaining_soldiers = total_soldiers - soldiers_counted;
+		verb_log_dbg_time(game().get_gametime(),
+		                  "AI %d: soldiers: total: %d, associated: %d, difference: %d",
+		                  player_number(), total_soldiers, soldiers_counted, remaining_soldiers);
+		on_stock_ += remaining_soldiers;
+	} else if (soldiers_counted > total_soldiers) {
+		// This shouldn't happen
+		log_warn_time(game().get_gametime(),
+		              "AI %d: soldiers: total: %d, associated: %d, unexpected: %d", player_number(),
+		              total_soldiers, soldiers_counted, soldiers_counted - total_soldiers);
+	}
 
-	// to avoid floats this is actual number * 100
-	vacant_mil_positions_average_ =
-	   vacant_mil_positions_average_ * 8 / 10 + 20 * vacant_mil_positions_;
+	vacant_mil_positions_ += understaffed_;
 
+	// May become negative, but that doesn't seem to be a problem for the below checks,
+	// and the variable is local and of a signed type.
+	vacant_mil_positions_ -= on_stock_;
+
+	// In practice kFull and kEnough only depend on small values of on_stock_.
+	// Military site garrisons are increased or decreased based on this, maintaining a kind of
+	// dynamic equilibrium, controlled by genetic parameters.
+	// TODO(tothxa): The names are thus confusing and should be changed. Also all consumers
+	//               should be checked whether they use the correct interpretation.
+	//               Possibly the values here should be tweaked too to make the intermediate
+	//               states actually matter, and the whole garrison handling more stable,
+	//               but that probably needs re-training the AI afterwards.
 	if (vacant_mil_positions_ <= 1 || on_stock_ > 4) {
 		soldier_status_ = SoldiersStatus::kFull;
-	} else if (vacant_mil_positions_ * 4 <= static_cast<int32_t>(militarysites.size()) ||
-	           on_stock_ > 2) {
+	} else if (vacant_mil_positions_ * 4 <= garrisons_count || on_stock_ > 2) {
 		soldier_status_ = SoldiersStatus::kEnough;
-	} else if (vacant_mil_positions_ > static_cast<int32_t>(militarysites.size())) {
+	} else if (vacant_mil_positions_ > garrisons_count) {
 		soldier_status_ = SoldiersStatus::kBadShortage;
 	} else {
 		soldier_status_ = SoldiersStatus::kShortage;
@@ -953,9 +1001,6 @@ bool DefaultAI::check_militarysites(const Time& gametime) {
 		return false;
 	}
 
-	// Make sure we have statistics about our enemies up-to-date
-	update_player_stat(gametime);
-
 	// Inform if we are above ai type limit.
 	if (militarysites.front().bo->total_count() > militarysites.front().bo->cnt_limit_by_aimode) {
 		verb_log_warn_time(gametime, "AI check_militarysites: Too many %s: %d, ai limit: %d\n",
@@ -1056,8 +1101,7 @@ bool DefaultAI::check_militarysites(const Time& gametime) {
 			changed = true;
 		}
 		if (ms->get_soldier_preference() == Widelands::SoldierPreference::kRookies) {
-			game().send_player_militarysite_set_soldier_preference(
-			   *ms, Widelands::SoldierPreference::kHeroes);
+			game().send_player_set_soldier_preference(*ms, Widelands::SoldierPreference::kHeroes);
 			changed = true;
 		}
 	} else if (should_be_dismantled && can_be_dismantled) {
@@ -1075,8 +1119,7 @@ bool DefaultAI::check_militarysites(const Time& gametime) {
 			changed = true;
 		}
 		if (ms->get_soldier_preference() == Widelands::SoldierPreference::kHeroes) {
-			game().send_player_militarysite_set_soldier_preference(
-			   *ms, Widelands::SoldierPreference::kRookies);
+			game().send_player_set_soldier_preference(*ms, Widelands::SoldierPreference::kRookies);
 			changed = true;
 		}
 	}
@@ -1412,7 +1455,7 @@ BuildingNecessity DefaultAI::check_building_necessity(BuildingObserver& bo, cons
 	inputs[107] = stocked_wood_level < 25 ? -3 * (size - 1) : 0;
 	inputs[108] = stocked_wood_level < 25 ? -5 * size : 0;
 	inputs[109] = stocked_wood_level < 25 ? -5 * (size - 1) : 0;
-	if (!bo.critical_building_material.empty() && buil_material_mines_count == 0) {
+	if (!bo.critical_building_material.empty() && build_material_mines_count == 0) {
 		inputs[110] = -5;
 		inputs[111] = -2;
 		inputs[111] = -10;
@@ -1430,6 +1473,18 @@ BuildingNecessity DefaultAI::check_building_necessity(BuildingObserver& bo, cons
 	}
 	inputs[118] = -mine_fields_stat.count_types();
 	inputs[119] = -mine_fields_stat.count_types() * 3;
+	if (!basic_economy_established) {  // discourage big milsites if no basic economy
+		inputs[120] = size * -5;
+		inputs[121] = size * -3;
+		inputs[122] = size * -4;
+		inputs[123] = size * -2;
+	}
+	if (!basic_economy_established && !bo.critical_building_material.empty()) {
+		inputs[124] = (size - build_material_mines_count) * -5;
+		inputs[125] = (size - build_material_mines_count) * -3;
+		inputs[126] = (size - build_material_mines_count) * -4;
+		inputs[127] = (size - build_material_mines_count) * -2;
+	}
 
 	for (int i = 0; i < 4 * kFNeuronBitSize; i = i + 1) {
 		if (inputs[i] < -35 || inputs[i] > 6) {
@@ -1484,7 +1539,7 @@ void DefaultAI::soldier_trained(const Widelands::TrainingSite& site) {
 		}
 	}
 
-	verb_log_err_time(
-	   gametime, " %d: Computer player error - trainingsite not found\n", player_number());
+	verb_log_warn_time(
+	   gametime, "AI %d: soldier_trained(): trainingsite not found\n", player_number());
 }
 }  // namespace AI
