@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2022 by the Widelands Development Team
+ * Copyright (C) 2006-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -46,10 +46,6 @@
 #endif
 #endif
 
-CLANG_DIAG_RESERVED_IDENTIFIER_OFF
-extern int _nl_msg_cat_cntr;
-CLANG_DIAG_RESERVED_IDENTIFIER_ON
-
 namespace i18n {
 
 /// A stack of textdomains. On entering a new textdomain, the old one gets
@@ -374,9 +370,6 @@ void set_locale(const std::string& name) {
 		// maybe, do another check with the return value (?)
 		locale = lang;
 	}
-
-	/* Finally make changes known.  */
-	++_nl_msg_cat_cntr;
 #endif
 
 	SETLOCALE(LC_ALL, "");  //  call to libintl
@@ -434,4 +427,18 @@ std::string join_sentences(const std::string& sentence1, const std::string& sent
 	return format(pgettext("sentence_separator", "%1% %2%"), sentence1, sentence2);
 }
 
+bool is_translation_of(const std::string& input,
+                       const std::string& base,
+                       const std::string& textdomain_name) {
+	if (input == base) {
+		return true;
+	}
+	if (textdomain_name.empty()) {
+		return input == translate(base);
+	}
+
+	Textdomain td(textdomain_name);
+	return input == translate(base);
+}
+
 }  // namespace i18n
