@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019-2022 by the Widelands Development Team
+ * Copyright (C) 2019-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
 
 #include "economy/shipping_schedule.h"
 
+#include <cstddef>
 #include <memory>
 #include <set>
 
@@ -48,8 +49,8 @@ constexpr Duration kActualDurationsRecalculationInterval(60 * 1000);
 // Give a ship the highest score for assigning wares to it if it will
 // arrive at the port of interest within the next 10 s gametime,
 // or the lowest score if this will take longer than 10 min gametime
-constexpr uint64_t kWonderfullyShortDuration = 10 * 1000;   // 10 s
-constexpr uint64_t kHorriblyLongDuration = 10 * 60 * 1000;  // 10 min
+constexpr uint64_t kWonderfullyShortDuration = 10ULL * 1000;   // 10 s
+constexpr uint64_t kHorriblyLongDuration = 10ULL * 60 * 1000;  // 10 min
 
 // Only assign wares to a ship in 5.1 if it's score is higher than a certain
 // threshold based on minimal distance. Ships with lower scores will only be
@@ -78,24 +79,18 @@ ShippingSchedule::ShippingSchedule(ShipFleet& f) : fleet_(f), last_updated_(0),
 }
 
 bool ShippingSchedule::empty() const {
-	for (const auto& pair : plans_) {
-		if (!pair.second.empty()) {
-			return false;
-		}
-	}
-	return true;
+	return std::all_of(
+	   plans_.begin(), plans_.end(), [](const auto& pair) { return pair.second.empty(); });
 }
 
 bool ShippingSchedule::is_busy(const Ship& ship) const {
 	if (ship.get_nritems() != 0u) {
 		return true;
 	}
-	for (const SchedulingState& ss : plans_.at(const_cast<Ship*>(&ship))) {
-		if (ss.expedition || !ss.load_there.empty()) {
-			return true;
-		}
-	}
-	return false;
+	const auto& plan = plans_.at(const_cast<Ship*>(&ship));
+	return std::any_of(plan.begin(), plan.end(), [](const SchedulingState& ss) {
+		return ss.expedition || !ss.load_there.empty();
+	});
 }
 
 void ShippingSchedule::start_expedition(Game& game, Ship& ship, PortDock& port) {
@@ -124,8 +119,8 @@ void ShippingSchedule::start_expedition(Game& game, Ship& ship, PortDock& port)
 }
 
 void ShippingSchedule::ship_arrived(Game& game, Ship& ship, PortDock& port) {
-	sslog(
-	   "\nShippingSchedule::ship_arrived(%s at %u)\n", ship.get_shipname().c_str(), port.serial());
+	sslog("\nShippingSchedule::ship_arrived(%s at %u %s)\n", ship.get_shipname().c_str(),
+	      port.serial(), port.get_warehouse()->get_warehouse_name().c_str());
 	auto plan = plans_.find(&ship);
 	assert(plan != plans_.end());
 	const size_t plan_size = plan->second.size();
@@ -159,8 +154,10 @@ void ShippingSchedule::ship_arrived(Game& game, Ship& ship, PortDock& port) {
 
 	plan->second.pop_front();
 	if (plan_size > 1) {
-		ship.set_destination(game, plan->second.front().dock.get(game));
-		sslog("Loaded cargo and sending to %u\n\n", plan->second.front().dock.serial());
+		PortDock* dest = plan->second.front().dock.get(game);
+		ship.set_destination(game, dest);
+		sslog("Loaded cargo and sending to %u %s\n\n", dest->serial(),
+		      dest->get_warehouse()->get_warehouse_name().c_str());
 	} else {
 		ship.set_destination(game, nullptr);
 		assert(ship.get_nritems() == 0);
@@ -214,8 +211,9 @@ bool ShippingSchedule::do_remove_port_from_plan(Game& game,
 				}
 			}
 			if (closest != nullptr) {
-				sslog("Ship %s is carrying %u items, rerouting to NEW destination %u\n",
-				      ship.get_shipname().c_str(), ship.get_nritems(), closest->serial());
+				sslog("Ship %s is carrying %u items, rerouting to NEW destination %u %s\n",
+				      ship.get_shipname().c_str(), ship.get_nritems(), closest->serial(),
+				      closest->get_warehouse()->get_warehouse_name().c_str());
 				ship_plan.push_back(SchedulingState(closest, false, Duration(dist)));
 				ship.set_destination(game, closest);
 			} else {
@@ -228,8 +226,10 @@ bool ShippingSchedule::do_remove_port_from_plan(Game& game,
 				ship.set_destination(game, nullptr);
 			}
 		} else {  // the ships has more destinations in its plan, just reroute to the next one
-			ship.set_destination(game, ship_plan.front().dock.get(game));
-			sslog("Rerouted %s to %u\n", ship.get_shipname().c_str(), ship_plan.front().dock.serial());
+			PortDock* dest = ship_plan.front().dock.get(game);
+			ship.set_destination(game, dest);
+			sslog("Rerouted %s to %u %s\n", ship.get_shipname().c_str(), dest->serial(),
+			      dest->get_warehouse()->get_warehouse_name().c_str());
 			Path path;
 			ship.calculate_sea_route(game, *ship_plan.front().dock.get(game), &path);
 			int32_t d = -1;
@@ -269,7 +269,8 @@ bool ShippingSchedule::do_remove_port_from_plan(Game& game,
 // `dock` is not a dangling reference yet, but this function is called
 // via `ShipFleet::remove_port()` from `PortDock::cleanup()`
 void ShippingSchedule::port_removed(Game& game, PortDock* dock) {
-	sslog("\nShippingSchedule::port_removed (%u)\n", dock->serial());
+	sslog("\nShippingSchedule::port_removed (%u %s)\n", dock->serial(),
+	      dock->get_warehouse()->get_warehouse_name().c_str());
 	// Find all ships planning to visit this dock and reroute them.
 	std::vector<Ship*> ships_heading_there;
 	for (auto& ship_and_plan : plans_) {
@@ -287,7 +288,8 @@ void ShippingSchedule::port_removed(Game& game, PortDock* dock) {
 	for (PortDock* pd : fleet_.get_ports()) {
 		for (auto it = pd->waiting_.begin(); it != pd->waiting_.end();) {
 			if (it->destination_dock_.serial() == dock->serial()) {
-				sslog("found a shippingitem in port %u\n", pd->serial());
+				sslog("found a shippingitem in port %u %s\n", pd->serial(),
+				      pd->get_warehouse()->get_warehouse_name().c_str());
 				it->set_location(game, pd->warehouse_);
 				it->end_shipping(game);
 				it = pd->waiting_.erase(it);
@@ -301,7 +303,7 @@ void ShippingSchedule::port_removed(Game& game, PortDock* dock) {
 		for (ShippingItem& si : ship->items_) {
 			if (si.destination_dock_.serial() == dock->serial()) {
 				sslog("found a shippingitem on %s\n", ship->get_shipname().c_str());
-				si.destination_dock_ = ship->get_destination();
+				si.destination_dock_ = ship->get_destination_port(game);
 			}
 		}
 	}
@@ -309,13 +311,16 @@ void ShippingSchedule::port_removed(Game& game, PortDock* dock) {
 }
 
 void ShippingSchedule::ship_removed(const Game& /* game */, Ship* ship) {
-	auto it = plans_.find(ship);
-	assert(it != plans_.end());
-	plans_.erase(it);
+	if (auto it = plans_.find(ship); it != plans_.end()) {
+		plans_.erase(it);
+	} else if (ship->get_ship_type() != ShipType::kWarship) {  // Warships might not have a schedule.
+		throw wexception(
+		   "Removing ship %s from fleet which did not have a schedule", ship->get_shipname().c_str());
+	}
 
-	auto i = last_actual_duration_recalculation_.find(ship);
-	if (i != last_actual_duration_recalculation_.end()) {
-		last_actual_duration_recalculation_.erase(i);
+	if (auto it = last_actual_duration_recalculation_.find(ship);
+	    it != last_actual_duration_recalculation_.end()) {
+		last_actual_duration_recalculation_.erase(it);
 	}
 
 	// Handling any items that were intended to be transported by this ship
@@ -324,7 +329,11 @@ void ShippingSchedule::ship_removed(const Game& /* game */, Ship* ship) {
 
 void ShippingSchedule::ship_added(Game& game, Ship& s) {
 	sslog("\nShippingSchedule::ship_added (%s)\n", s.get_shipname().c_str());
-	assert(!s.get_destination());
+	if (s.get_ship_type() == ShipType::kWarship) {
+		sslog("Is a warship, won't create a plan.\n\n");
+		return;
+	}
+	assert(s.get_destination_port(game) == nullptr);
 	plans_[&s] = ShipPlan();
 	last_actual_duration_recalculation_[&s] = game.get_gametime();
 	if (fleet_.get_ports().empty()) {
@@ -357,11 +366,13 @@ void ShippingSchedule::ship_added(Game& game, Ship& s) {
 	plans_[&s].push_back(SchedulingState(closest, false, Duration(dist)));
 	s.set_destination(game, closest);
 	// Check for closest to make clang-tidy happy
-	sslog("Sent to %u\n\n", closest ? closest->serial() : 0);
+	sslog("Sent to %u %s\n\n", closest ? closest->serial() : 0,
+	      closest ? closest->get_warehouse()->get_warehouse_name().c_str() : "");
 }
 
 void ShippingSchedule::port_added(Game& game, PortDock& dock) {
-	sslog("\nShippingSchedule::port_added (%u)\n", dock.serial());
+	sslog("\nShippingSchedule::port_added (%u %s)\n", dock.serial(),
+	      dock.get_warehouse()->get_warehouse_name().c_str());
 	if (fleet_.count_ports() > 1) {
 		// nothing to do currently
 		sslog("nothing to do\n\n");
@@ -370,7 +381,7 @@ void ShippingSchedule::port_added(Game& game, PortDock& dock) {
 	// All ships are most likely panicking because they have
 	// no destination. Send them all to the new port.
 	for (Ship* ship : fleet_.get_ships()) {
-		assert(!ship->get_destination());
+		assert(ship->get_destination_port(game) == nullptr);
 		sslog("Rerouting %s there\n", ship->get_shipname().c_str());
 		ship->set_destination(game, &dock);
 		for (ShippingItem& si : ship->items_) {
@@ -591,9 +602,11 @@ void ShippingSchedule::load_on_ship(Game& game,
                                     std::list<PrioritisedPortPair>& all_ppps) {
 	const uint32_t take = std::min(ppp.open_count, ppp.ships.front().capacity);
 	Ship* ship = ppp.ships.front().ship;
-	sslog("load_on_ship: PPP %u –> %u (open_count %u): assigning %u items (capacity %u) to %s\n",
-	      ppp.start->serial(), ppp.end->serial(), ppp.open_count, take, ppp.ships.front().capacity,
-	      ship->get_shipname().c_str());
+	sslog(
+	   "load_on_ship: PPP %u %s –> %u %s (open_count %u): assigning %u items (capacity %u) to %s\n",
+	   ppp.start->serial(), ppp.start->get_warehouse()->get_warehouse_name().c_str(),
+	   ppp.end->serial(), ppp.end->get_warehouse()->get_warehouse_name().c_str(), ppp.open_count,
+	   take, ppp.ships.front().capacity, ship->get_shipname().c_str());
 	assert(take);
 	// We assume that EITHER both end points are already part of the plan,
 	// or that the start point is the last entry in the plan
@@ -655,10 +668,12 @@ void ShippingSchedule::load_on_ship(Game& game,
 		p.ships.erase(ship_it);
 		if ((free_capacity != 0u) && found_start && !expedition && (found_end || start_is_last)) {
 			ScoredShip updated_ship(ship, free_capacity, arrival_time, detour_start_end);
-			sslog("load_on_ship: PPP %u –> %u (open_count %u) UPDATED: may assign up to %u items "
-			      "to %s (score %" PRIu64 ")\n",
-			      p.start->serial(), p.end->serial(), ppp.open_count, free_capacity,
-			      ship->get_shipname().c_str(), updated_ship.score);
+			sslog(
+			   "load_on_ship: PPP %u %s –> %u %s (open_count %u) UPDATED: may assign up to %u items "
+			   "to %s (score %" PRIu64 ")\n",
+			   p.start->serial(), p.start->get_warehouse()->get_warehouse_name().c_str(),
+			   p.end->serial(), p.end->get_warehouse()->get_warehouse_name().c_str(), ppp.open_count,
+			   free_capacity, ship->get_shipname().c_str(), updated_ship.score);
 			for (auto it = p.ships.begin();; ++it) {
 				if (it == p.ships.end() || updated_ship < *it) {
 					p.ships.insert(it, updated_ship);
@@ -666,8 +681,10 @@ void ShippingSchedule::load_on_ship(Game& game,
 				}
 			}
 		} else {
-			sslog("load_on_ship: PPP %u –> %u (open_count %u) REMOVED %s\n", p.start->serial(),
-			      p.end->serial(), ppp.open_count, ship->get_shipname().c_str());
+			sslog("load_on_ship: PPP %u %s –> %u %s (open_count %u) REMOVED %s\n", p.start->serial(),
+			      p.start->get_warehouse()->get_warehouse_name().c_str(), p.end->serial(),
+			      p.end->get_warehouse()->get_warehouse_name().c_str(), ppp.open_count,
+			      ship->get_shipname().c_str());
 		}
 	}
 }
@@ -798,7 +815,8 @@ Duration ShippingSchedule::update(Game& game) {
 
 	for (PortDock* dock : fleet_.get_ports()) {
 		const bool expedition_ready = dock->is_expedition_ready();
-		sslog("Iteration: dock %u (expedition ready %s)\n", dock->serial(),
+		sslog("Iteration: dock %u %s (expedition ready %s)\n", dock->serial(),
+		      dock->get_warehouse()->get_warehouse_name().c_str(),
 		      expedition_ready ? "true" : "false");
 		Ship* expedition_ship_coming = nullptr;
 		Map_ToDestination_ShipsWithInfoAndCapacity map;
@@ -845,7 +863,8 @@ Duration ShippingSchedule::update(Game& game) {
 
 		for (PortDock* dest : fleet_.get_ports()) {
 			const int32_t waiting_items = dock->count_waiting(dest);
-			sslog("Iteration: Iteration: dest %u, waiting %d\n", dest->serial(), waiting_items);
+			sslog("Iteration: Iteration: dest %u %s, waiting %d\n", dest->serial(),
+			      dest->get_warehouse()->get_warehouse_name().c_str(), waiting_items);
 			std::multiset<Duration> arrival_times;  // one entry per item that will be picked up
 			for (const auto& pair : map[dest].first) {
 				for (uint32_t i = pair.second.second; i != 0u; --i) {
@@ -972,9 +991,11 @@ Duration ShippingSchedule::update(Game& game) {
 		if (plans_[ship].empty()) {
 			sslog("No orders left, setting to idle\n");
 			ship->set_destination(game, nullptr);
-		} else if (plans_[ship].front().dock != ship->get_destination()) {
-			ship->set_destination(game, plans_[ship].front().dock.get(game));
-			sslog("Rerouted to %u\n", ship->get_destination()->serial());
+		} else if (plans_[ship].front().dock != ship->get_destination_port(game)) {
+			PortDock* dest = plans_[ship].front().dock.get(game);
+			ship->set_destination(game, dest);
+			sslog("Rerouted to %u %s\n", dest->serial(),
+			      dest->get_warehouse()->get_warehouse_name().c_str());
 		}
 	}
 
@@ -991,7 +1012,8 @@ Duration ShippingSchedule::update(Game& game) {
 	 */
 	for (auto dock = ports_with_unserviced_expeditions.begin();
 	     dock != ports_with_unserviced_expeditions.end();) {
-		sslog("FOURTH PASS: Iteration %u\n", (*dock)->serial());
+		sslog("FOURTH PASS: Iteration %u %s\n", (*dock)->serial(),
+		      (*dock)->get_warehouse()->get_warehouse_name().c_str());
 		bool assigned = false;
 		for (auto& plan : plans_) {
 			bool has_further_plans = false;
@@ -1064,7 +1086,8 @@ Duration ShippingSchedule::update(Game& game) {
 					closest = dock;
 				}
 			}
-			sslog("assigned %s to dock %u\n", ship->get_shipname().c_str(), (*closest)->serial());
+			sslog("assigned %s to dock %u %s\n", ship->get_shipname().c_str(), (*closest)->serial(),
+			      (*closest)->get_warehouse()->get_warehouse_name().c_str());
 			ship->set_destination(game, *closest);
 			plans_[ship].clear();
 			plans_[ship].push_back(SchedulingState(*closest, true, Duration(dist)));
@@ -1139,10 +1162,13 @@ Duration ShippingSchedule::update(Game& game) {
 			                          detour_start_end, free_capacity);
 			if ((free_capacity != 0u) && found_start && !expedition && (found_end || start_is_last)) {
 				ScoredShip ss(plan.first.get(game), free_capacity, arrival_time, detour_start_end);
-				sslog("Phase 5.0: PPP %u –> %u (open_count %u): may assign up to %u items to %s (score "
+				sslog("Phase 5.0: PPP %u %s –> %u %s (open_count %u): may assign up to %u items to %s "
+				      "(score "
 				      "%" PRIu64 ")\n",
-				      ppp.start->serial(), ppp.end->serial(), ppp.open_count, free_capacity,
-				      plan.first.get(game)->get_shipname().c_str(), ss.score);
+				      ppp.start->serial(), ppp.start->get_warehouse()->get_warehouse_name().c_str(),
+				      ppp.end->serial(), ppp.end->get_warehouse()->get_warehouse_name().c_str(),
+				      ppp.open_count, free_capacity, plan.first.get(game)->get_shipname().c_str(),
+				      ss.score);
 				_ships.insert(ss);
 			}
 		}
@@ -1160,8 +1186,9 @@ Duration ShippingSchedule::update(Game& game) {
 			}
 			load_on_ship(game, ppp, open_pairs);
 		}
-		sslog("Phase 5.1: PPP %u –> %u: %u open_count remaining\n", ppp.start->serial(),
-		      ppp.end->serial(), ppp.open_count);
+		sslog("Phase 5.1: PPP %u %s –> %u %s: %u open_count remaining\n", ppp.start->serial(),
+		      ppp.start->get_warehouse()->get_warehouse_name().c_str(), ppp.end->serial(),
+		      ppp.end->get_warehouse()->get_warehouse_name().c_str(), ppp.open_count);
 	}
 
 	// 2) assign idle ships
@@ -1192,9 +1219,10 @@ Duration ShippingSchedule::update(Game& game) {
 			assert(closest);
 			const uint32_t take = std::min(ppp.open_count, closest->get_capacity());
 			assert(take);
-			sslog("Phase 5.2: PPP %u –> %u (open_count %u): assigning %u items to %s\n",
-			      ppp.start->serial(), ppp.end->serial(), ppp.open_count, take,
-			      closest->get_shipname().c_str());
+			sslog("Phase 5.2: PPP %u %s –> %u %s (open_count %u): assigning %u items to %s\n",
+			      ppp.start->serial(), ppp.start->get_warehouse()->get_warehouse_name().c_str(),
+			      ppp.end->serial(), ppp.end->get_warehouse()->get_warehouse_name().c_str(),
+			      ppp.open_count, take, closest->get_shipname().c_str());
 			plans_[closest].clear();
 			plans_[closest].push_back(SchedulingState(ppp.start, false, Duration(dist)));
 			plans_[closest].front().load_there[ppp.end] = take;
@@ -1222,8 +1250,10 @@ Duration ShippingSchedule::update(Game& game) {
 	std::list<PortDock*> open_count_left;
 	for (PrioritisedPortPair& ppp : open_pairs) {
 		while ((ppp.open_count != 0u) && !ppp.ships.empty()) {
-			sslog("Phase 5.3: PPP %u –> %u (open_count %u): assigning items…\n", ppp.start->serial(),
-			      ppp.end->serial(), ppp.open_count);
+			sslog("Phase 5.3: PPP %u %s –> %u %s (open_count %u): assigning items…\n",
+			      ppp.start->serial(), ppp.start->get_warehouse()->get_warehouse_name().c_str(),
+			      ppp.end->serial(), ppp.end->get_warehouse()->get_warehouse_name().c_str(),
+			      ppp.open_count);
 			load_on_ship(game, ppp, open_pairs);
 		}
 		sslog("%u open_count remaining\n", ppp.open_count);
@@ -1324,9 +1354,11 @@ Duration ShippingSchedule::update(Game& game) {
 							continue;
 						}
 						const uint32_t take = std::min(capacity, ppp.open_count);
-						sslog("Phase 5.4.A: PPP %u –> %u (open_count %u): assigning %u items to %s\n",
-						      ppp.start->serial(), ppp.end->serial(), ppp.open_count, take,
-						      plan.first.get(game)->get_shipname().c_str());
+						sslog(
+						   "Phase 5.4.A: PPP %u %s –> %u %s (open_count %u): assigning %u items to %s\n",
+						   ppp.start->serial(), ppp.start->get_warehouse()->get_warehouse_name().c_str(),
+						   ppp.end->serial(), ppp.end->get_warehouse()->get_warehouse_name().c_str(),
+						   ppp.open_count, take, plan.first.get(game)->get_shipname().c_str());
 						assert(take);
 						ppp.open_count -= take;
 						// c
@@ -1362,9 +1394,11 @@ Duration ShippingSchedule::update(Game& game) {
 						// → a) add items for start and b) push a State to end
 						const uint32_t take =
 						   std::min(plan.first.get(game)->get_capacity(), ppp.open_count);
-						sslog("Phase 5.4.B: PPP %u –> %u (open_count %u): assigning %u items to %s\n",
-						      ppp.start->serial(), ppp.end->serial(), ppp.open_count, take,
-						      plan.first.get(game)->get_shipname().c_str());
+						sslog(
+						   "Phase 5.4.B: PPP %u %s –> %u %s (open_count %u): assigning %u items to %s\n",
+						   ppp.start->serial(), ppp.start->get_warehouse()->get_warehouse_name().c_str(),
+						   ppp.end->serial(), ppp.end->get_warehouse()->get_warehouse_name().c_str(),
+						   ppp.open_count, take, plan.first.get(game)->get_shipname().c_str());
 						assert(take);
 						ppp.open_count -= take;
 						// a
@@ -1389,9 +1423,10 @@ Duration ShippingSchedule::update(Game& game) {
 					// or end)
 					// → a) insert a new state with items for start and b) push a State to end
 					const uint32_t take = std::min(plan.first.get(game)->get_capacity(), ppp.open_count);
-					sslog("Phase 5.4.C: PPP %u –> %u (open_count %u): assigning %u items to %s\n",
-					      ppp.start->serial(), ppp.end->serial(), ppp.open_count, take,
-					      plan.first.get(game)->get_shipname().c_str());
+					sslog("Phase 5.4.C: PPP %u %s –> %u %s (open_count %u): assigning %u items to %s\n",
+					      ppp.start->serial(), ppp.start->get_warehouse()->get_warehouse_name().c_str(),
+					      ppp.end->serial(), ppp.end->get_warehouse()->get_warehouse_name().c_str(),
+					      ppp.open_count, take, plan.first.get(game)->get_shipname().c_str());
 					assert(take);
 					ppp.open_count -= take;
 					// a
@@ -1430,9 +1465,10 @@ Duration ShippingSchedule::update(Game& game) {
 						continue;
 					}
 					const uint32_t take = std::min(capacity, ppp.open_count);
-					sslog("Phase 5.4.D: PPP %u –> %u (open_count %u): assigning %u items to %s\n",
-					      ppp.start->serial(), ppp.end->serial(), ppp.open_count, take,
-					      plan.first.get(game)->get_shipname().c_str());
+					sslog("Phase 5.4.D: PPP %u %s –> %u %s (open_count %u): assigning %u items to %s\n",
+					      ppp.start->serial(), ppp.start->get_warehouse()->get_warehouse_name().c_str(),
+					      ppp.end->serial(), ppp.end->get_warehouse()->get_warehouse_name().c_str(),
+					      ppp.open_count, take, plan.first.get(game)->get_shipname().c_str());
 					assert(take);
 					ppp.open_count -= take;
 					// b
@@ -1473,9 +1509,13 @@ Duration ShippingSchedule::update(Game& game) {
 								continue;
 							}
 							const uint32_t take = std::min(capacity, ppp.open_count);
-							sslog("Phase 5.4.E: PPP %u –> %u (open_count %u): assigning %u items to %s\n",
-							      ppp.start->serial(), ppp.end->serial(), ppp.open_count, take,
-							      plan.first.get(game)->get_shipname().c_str());
+							sslog("Phase 5.4.E: PPP %u %s –> %u %s (open_count %u): assigning %u items to "
+							      "%s\n",
+							      ppp.start->serial(),
+							      ppp.start->get_warehouse()->get_warehouse_name().c_str(),
+							      ppp.end->serial(),
+							      ppp.end->get_warehouse()->get_warehouse_name().c_str(), ppp.open_count,
+							      take, plan.first.get(game)->get_shipname().c_str());
 							assert(take);
 							ppp.open_count -= take;
 							// c
@@ -1577,12 +1617,14 @@ Duration ShippingSchedule::update(Game& game) {
 		assert(closest);
 		if (dist < kNearbyDockMaxDistanceFactor) {
 			// Check for closest to make clang-tidy happy
-			sslog("%s is already near %u\n", ship->get_shipname().c_str(),
-			      closest ? closest->serial() : 0);
+			sslog("%s is already near %u %s\n", ship->get_shipname().c_str(),
+			      closest ? closest->serial() : 0,
+			      closest ? closest->get_warehouse()->get_warehouse_name().c_str() : "");
 		} else {
 			plans_[ship].push_back(SchedulingState(closest, false, Duration(dist)));
 			ship->set_destination(game, closest);
-			sslog("Sending %s to %u\n", ship->get_shipname().c_str(), closest->serial());
+			sslog("Sending %s to %u %s\n", ship->get_shipname().c_str(), closest->serial(),
+			      closest->get_warehouse()->get_warehouse_name().c_str());
 		}
 		for (auto it = ships_per_port.begin(); it != ships_per_port.end(); ++it) {
 			if (it->first == closest) {
@@ -1603,7 +1645,7 @@ Duration ShippingSchedule::update(Game& game) {
 void ShippingSchedule::log_general_info(const EditorGameBase& e) const {
 	const Time& t = e.get_gametime();
 	for (const auto& plan : plans_) {
-		log_dbg_time(t, "· %s: carrying %u items (capacity %u)\n",
+		log_dbg_time(t, "• %s: carrying %u items (capacity %u)\n",
 		             plan.first.get(e)->get_shipname().c_str(), plan.first.get(e)->get_nritems(),
 		             plan.first.get(e)->get_capacity());
 		std::map<Serial, uint32_t> dests;
@@ -1621,7 +1663,7 @@ void ShippingSchedule::log_general_info(const EditorGameBase& e) const {
 		}
 		log_dbg_time(t, "  SCHEDULE: %" PRIuS " stations\n", plan.second.size());
 		for (const SchedulingState& ss : plan.second) {
-			log_dbg_time(t, "          · in %u ms at %u\n", ss.duration_from_previous_location.get(),
+			log_dbg_time(t, "          • in %u ms at %u\n", ss.duration_from_previous_location.get(),
 			             ss.dock.serial());
 			log_dbg_time(t, "            load there: ");
 			if (ss.expedition) {
