@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2022 by the Widelands Development Team
+ * Copyright (C) 2004-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -53,25 +53,26 @@ struct IdleWareSupply : public Supply {
 
 	//  implementation of Supply
 	PlayerImmovable* get_position(Game& /*game*/) override;
-	bool is_active() const override;
+	[[nodiscard]] bool is_active() const override;
 	SupplyProviders provider_type(Game* /*game*/) const override;
-	bool has_storage() const override;
+	[[nodiscard]] bool has_storage() const override;
 	void get_ware_type(WareWorker& type, DescriptionIndex& ware) const override;
 	void send_to_storage(Game& /*game*/, Warehouse* wh) override;
 
-	uint32_t nr_supplies(const Game& /* game */, const Request& /* req */) const override;
+	[[nodiscard]] uint32_t nr_supplies(const Game& /* game */,
+	                                   const Request& /* req */) const override;
 	WareInstance& launch_ware(Game& /* game */, const Request& /* req */) override;
 	Worker& launch_worker(Game& /* game */, const Request& /* req */) override;
 
 private:
 	WareInstance& ware_;
-	Economy* economy_;
+	Economy* economy_{nullptr};
 };
 
 /**
  * Initialize the Supply and update the economy.
  */
-IdleWareSupply::IdleWareSupply(WareInstance& ware) : ware_(ware), economy_(nullptr) {
+IdleWareSupply::IdleWareSupply(WareInstance& ware) : ware_(ware) {
 	set_economy(ware.get_economy());
 }
 
@@ -104,17 +105,24 @@ void IdleWareSupply::set_economy(Economy* const e) {
  */
 PlayerImmovable* IdleWareSupply::get_position(Game& game) {
 	MapObject* const loc = ware_.get_location(game);
+	assert(loc != nullptr);
 
-	if (upcast(PlayerImmovable, playerimmovable, loc)) {
-		return playerimmovable;
+	if (loc->descr().type() > MapObjectType::IMMOVABLE) {
+		if (upcast(PlayerImmovable, playerimmovable, loc); playerimmovable != nullptr) {
+			return playerimmovable;
+		}
 	}
 
-	if (upcast(Worker, worker, loc)) {
-		return worker->get_location(game);
+	if (loc->descr().type() >= MapObjectType::WORKER) {
+		if (upcast(Worker, worker, loc); worker != nullptr) {
+			return worker->get_location(game);
+		}
 	}
 
-	if (upcast(Ship, ship, loc)) {
-		if (PortDock* pd = ship->get_destination()) {
+	if (loc->descr().type() == MapObjectType::SHIP) {
+		upcast(Ship, ship, loc);
+		assert(ship != nullptr);
+		if (PortDock* pd = ship->get_destination_port(game); pd != nullptr) {
 			return pd;
 		}
 		return ship->get_fleet()->get_arbitrary_dock();
@@ -187,11 +195,7 @@ void IdleWareSupply::send_to_storage(Game& game, Warehouse* wh) {
 /*                     Ware Instance Implementation                      */
 /*************************************************************************/
 WareInstance::WareInstance(DescriptionIndex const i, const WareDescr* const ware_descr)
-   : MapObject(ware_descr),
-     economy_(nullptr),
-     descr_index_(i),
-     supply_(nullptr),
-     transfer_(nullptr) {
+   : MapObject(ware_descr), descr_index_(i) {
 }
 
 WareInstance::~WareInstance() {
