@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -52,48 +52,52 @@
 #include "editor/ui_menus/toolsize_menu.h"
 #include "graphic/mouse_cursor.h"
 #include "graphic/playercolor.h"
-#include "graphic/style_manager.h"
 #include "graphic/text_layout.h"
 #include "logic/addons.h"
+#include "logic/generic_save_handler.h"
 #include "logic/map.h"
 #include "logic/map_objects/descriptions.h"
 #include "logic/map_objects/map_object_type.h"
 #include "logic/map_objects/world/resource_description.h"
 #include "logic/mapregion.h"
 #include "logic/maptriangleregion.h"
+#include "logic/mutable_addon.h"
 #include "logic/player.h"
 #include "map_io/map_loader.h"
+#include "map_io/map_saver.h"
 #include "map_io/widelands_map_loader.h"
+#include "network/net_addons.h"
 #include "scripting/lua_interface.h"
 #include "scripting/lua_table.h"
 #include "sound/sound_handler.h"
 #include "ui_basic/messagebox.h"
 #include "ui_basic/progresswindow.h"
+#include "ui_basic/toolbar_setup.h"
+#include "ui_fsmenu/addons/login_box.h"
+#include "ui_fsmenu/addons/progress.h"
 #include "wlapplication_mousewheel_options.h"
 #include "wlapplication_options.h"
 #include "wui/interactive_base.h"
 #include "wui/toolbar.h"
 
-std::string editor_splash_image() {
-	return template_dir() + "loadscreens/editor.jpg";
-}
+const std::string kEditorSplashImage("loadscreens/editor.jpg");
 
 EditorInteractive::EditorInteractive(Widelands::EditorGameBase& e)
    : InteractiveBase(e, get_config_section(), nullptr),
-     need_save_(false),
+
      realtime_(SDL_GetTicks()),
-     is_painting_(false),
+
      mainmenu_(toolbar(),
                "dropdown_menu_main",
                0,
                0,
-               MainToolbar::kButtonSize,
+               UI::main_toolbar_button_size(),
                10,
-               MainToolbar::kButtonSize,
+               UI::main_toolbar_button_size(),
                as_tooltip_text_with_hotkey(
                   /** TRANSLATORS: Title for the main menu button in the editor */
                   _("Main Menu"),
-                  shortcut_string_for(KeyboardShortcut::kEditorMenu, true),
+                  pgettext("hotkey", "Esc"),
                   UI::PanelStyle::kWui),
                UI::DropdownType::kPictorialMenu,
                UI::PanelStyle::kWui,
@@ -102,12 +106,12 @@ EditorInteractive::EditorInteractive(Widelands::EditorGameBase& e)
                "dropdown_menu_tools",
                0,
                0,
-               MainToolbar::kButtonSize,
+               UI::main_toolbar_button_size(),
                12,
-               MainToolbar::kButtonSize,
+               UI::main_toolbar_button_size(),
                as_tooltip_text_with_hotkey(
                   /** TRANSLATORS: Title for the tool menu button in the editor */
-                  _("Tools"),
+                  pgettext("editor", "Tools"),
                   shortcut_string_for(KeyboardShortcut::kEditorTools, true),
                   UI::PanelStyle::kWui),
                UI::DropdownType::kPictorialMenu,
@@ -118,9 +122,9 @@ EditorInteractive::EditorInteractive(Widelands::EditorGameBase& e)
                    "dropdown_menu_showhide",
                    0,
                    0,
-                   MainToolbar::kButtonSize,
+                   UI::main_toolbar_button_size(),
                    10,
-                   MainToolbar::kButtonSize,
+                   UI::main_toolbar_button_size(),
                    /** TRANSLATORS: Title for a menu button in the editor. This menu will show/hide
                       building spaces, animals, immovables, resources */
                    _("Show / Hide"),
@@ -128,8 +132,7 @@ EditorInteractive::EditorInteractive(Widelands::EditorGameBase& e)
                    UI::PanelStyle::kWui,
                    UI::ButtonStyle::kWuiPrimary,
                    [this](ShowHideEntry t) { showhide_menu_selected(t); }),
-     undo_(nullptr),
-     redo_(nullptr),
+
      tools_(new Tools(*this, e.map())),
      history_(nullptr)  // history needs the undo/redo buttons
 {
@@ -177,6 +180,8 @@ EditorInteractive::EditorInteractive(Widelands::EditorGameBase& e)
 		new EditorHelp(*this, menu_windows_.help, &egbase().lua());
 	};
 
+	add_plugin_menu();
+
 	finalize_toolbar();
 
 	set_display_flags(EditorInteractive::dfShowResources | EditorInteractive::dfShowImmovables |
@@ -200,7 +205,8 @@ void EditorInteractive::add_main_menu() {
 	menu_windows_.newmap.open_window = [this] { new MainMenuNewMap(*this, menu_windows_.newmap); };
 	/** TRANSLATORS: An entry in the editor's main menu */
 	mainmenu_.add(_("New Map"), MainMenuEntry::kNewMap,
-	              g_image_cache->get("images/wui/editor/menus/new_map.png"));
+	              g_image_cache->get("images/wui/editor/menus/new_map.png"), false, "",
+	              shortcut_string_for(KeyboardShortcut::kEditorNewMap, false));
 
 	menu_windows_.newrandommap.open_window = [this] {
 		new MainMenuNewRandomMap(*this, menu_windows_.newrandommap, egbase().map().get_width(),
@@ -208,7 +214,8 @@ void EditorInteractive::add_main_menu() {
 	};
 	/** TRANSLATORS: An entry in the editor's main menu */
 	mainmenu_.add(_("New Random Map"), MainMenuEntry::kNewRandomMap,
-	              g_image_cache->get("images/wui/editor/menus/new_random_map.png"));
+	              g_image_cache->get("images/wui/editor/menus/new_random_map.png"), false, "",
+	              shortcut_string_for(KeyboardShortcut::kEditorNewRandomMap, false));
 
 	menu_windows_.loadmap.open_window = [this] {
 		new MainMenuLoadMap(*this, menu_windows_.loadmap);
@@ -216,7 +223,7 @@ void EditorInteractive::add_main_menu() {
 	/** TRANSLATORS: An entry in the editor's main menu */
 	mainmenu_.add(_("Load Map"), MainMenuEntry::kLoadMap,
 	              g_image_cache->get("images/wui/editor/menus/load_map.png"), false, "",
-	              shortcut_string_for(KeyboardShortcut::kEditorLoad, false));
+	              shortcut_string_for(KeyboardShortcut::kCommonLoad, false));
 
 	menu_windows_.savemap.open_window = [this] {
 		new MainMenuSaveMap(*this, menu_windows_.savemap, menu_windows_.mapoptions);
@@ -224,18 +231,26 @@ void EditorInteractive::add_main_menu() {
 	/** TRANSLATORS: An entry in the editor's main menu */
 	mainmenu_.add(_("Save Map"), MainMenuEntry::kSaveMap,
 	              g_image_cache->get("images/wui/editor/menus/save_map.png"), false, "",
-	              shortcut_string_for(KeyboardShortcut::kEditorSave, false));
+	              shortcut_string_for(KeyboardShortcut::kCommonSave, false));
+
+	/** TRANSLATORS: An entry in the editor's main menu */
+	mainmenu_.add(_("Publish Map Online…"), MainMenuEntry::kUploadAsAddOn,
+	              g_image_cache->get("images/wui/editor/menus/upload.png"), false, "",
+	              shortcut_string_for(KeyboardShortcut::kEditorUploadMap, false));
 
 	menu_windows_.mapoptions.open_window = [this] {
 		new MainMenuMapOptions(*this, menu_windows_.mapoptions);
 	};
 	/** TRANSLATORS: An entry in the editor's main menu */
 	mainmenu_.add(_("Map Options"), MainMenuEntry::kMapOptions,
-	              g_image_cache->get("images/wui/editor/menus/map_options.png"));
+	              g_image_cache->get("images/wui/editor/menus/map_options.png"), false, "",
+	              shortcut_string_for(KeyboardShortcut::kEditorMapOptions, false));
 
 	/** TRANSLATORS: An entry in the editor's main menu */
 	mainmenu_.add(_("Exit Editor"), MainMenuEntry::kExitEditor,
-	              g_image_cache->get("images/wui/menus/exit.png"));
+	              g_image_cache->get("images/wui/menus/exit.png"), false, "",
+	              shortcut_string_for(KeyboardShortcut::kCommonExit, false));
+
 	mainmenu_.selected.connect([this] { main_menu_selected(mainmenu_.get_selected()); });
 	toolbar()->add(&mainmenu_);
 }
@@ -257,8 +272,11 @@ void EditorInteractive::main_menu_selected(MainMenuEntry entry) {
 	case MainMenuEntry::kMapOptions: {
 		menu_windows_.mapoptions.toggle();
 	} break;
+	case MainMenuEntry::kUploadAsAddOn: {
+		publish_map();
+	} break;
 	case MainMenuEntry::kExitEditor: {
-		exit();
+		exit((SDL_GetModState() & KMOD_CTRL) != 0);
 	}
 	}
 }
@@ -273,7 +291,8 @@ void EditorInteractive::add_tool_menu() {
 	toolmenu_.add(_("Change height"), ToolMenuEntry::kChangeHeight,
 	              g_image_cache->get("images/wui/editor/tools/height.png"), false,
 	              /** TRANSLATORS: Tooltip for the change height tool in the editor */
-	              _("Change the terrain height"));
+	              _("Change the terrain height"),
+	              shortcut_string_for(KeyboardShortcut::kEditorChangeHeight, false));
 
 	tool_windows_.noiseheight.open_window = [this] {
 		new EditorToolNoiseHeightOptionsMenu(*this, tools()->noise_height, tool_windows_.noiseheight);
@@ -282,7 +301,8 @@ void EditorInteractive::add_tool_menu() {
 	toolmenu_.add(_("Random height"), ToolMenuEntry::kRandomHeight,
 	              g_image_cache->get("images/wui/editor/tools/noise_height.png"), false,
 	              /** TRANSLATORS: Tooltip for the random height tool in the editor */
-	              _("Set the terrain height to random values"));
+	              _("Set the terrain height to random values"),
+	              shortcut_string_for(KeyboardShortcut::kEditorRandomHeight, false));
 
 	tool_windows_.terrain.open_window = [this] {
 		new EditorToolSetTerrainOptionsMenu(*this, tools()->set_terrain, tool_windows_.terrain);
@@ -291,7 +311,8 @@ void EditorInteractive::add_tool_menu() {
 	toolmenu_.add(_("Terrain"), ToolMenuEntry::kTerrain,
 	              g_image_cache->get("images/wui/editor/tools/terrain.png"), false,
 	              /** TRANSLATORS: Tooltip for the terrain tool in the editor */
-	              _("Change the map’s terrain"));
+	              _("Change the map’s terrain"),
+	              shortcut_string_for(KeyboardShortcut::kEditorTerrain, false));
 
 	tool_windows_.immovables.open_window = [this] {
 		new EditorToolPlaceImmovableOptionsMenu(
@@ -301,7 +322,8 @@ void EditorInteractive::add_tool_menu() {
 	toolmenu_.add(_("Immovables"), ToolMenuEntry::kImmovables,
 	              g_image_cache->get("images/wui/editor/tools/immovables.png"), false,
 	              /** TRANSLATORS: Tooltip for the immovables tool in the editor */
-	              _("Add or remove immovables"));
+	              _("Add or remove immovables"),
+	              shortcut_string_for(KeyboardShortcut::kEditorImmovables, false));
 
 	tool_windows_.critters.open_window = [this] {
 		new EditorToolPlaceCritterOptionsMenu(*this, tools()->place_critter, tool_windows_.critters);
@@ -310,7 +332,8 @@ void EditorInteractive::add_tool_menu() {
 	toolmenu_.add(_("Animals"), ToolMenuEntry::kAnimals,
 	              g_image_cache->get("images/wui/editor/tools/critters.png"), false,
 	              /** TRANSLATORS: Tooltip for the animals tool in the editor */
-	              _("Add or remove animals"));
+	              _("Add or remove animals"),
+	              shortcut_string_for(KeyboardShortcut::kEditorAnimals, false));
 
 	tool_windows_.resources.open_window = [this] {
 		new EditorToolChangeResourcesOptionsMenu(
@@ -320,13 +343,15 @@ void EditorInteractive::add_tool_menu() {
 	toolmenu_.add(_("Resources"), ToolMenuEntry::kResources,
 	              g_image_cache->get("images/wui/editor/tools/resources.png"), false,
 	              /** TRANSLATORS: Tooltip for the resources tool in the editor */
-	              _("Set or change resources"));
+	              _("Set or change resources"),
+	              shortcut_string_for(KeyboardShortcut::kEditorResources, false));
 
 	/** TRANSLATORS: An entry in the editor's tool menu */
 	toolmenu_.add(_("Port spaces"), ToolMenuEntry::kPortSpace,
 	              g_image_cache->get("images/wui/editor/tools/port_spaces.png"), false,
 	              /** TRANSLATORS: Tooltip for the port spaces tool in the editor */
-	              _("Add or remove port spaces"));
+	              _("Add or remove port spaces"),
+	              shortcut_string_for(KeyboardShortcut::kEditorPortSpaces, false));
 
 	tool_windows_.players.open_window = [this] {
 		new EditorPlayerMenu(*this, tools()->set_starting_pos, tool_windows_.players);
@@ -344,7 +369,8 @@ void EditorInteractive::add_tool_menu() {
 	              g_image_cache->get("images/wui/editor/tools/map_origin.png"), false,
 	              /** TRANSLATORS: Tooltip for the map origin tool in the editor */
 	              _("Set the position that will have the coordinates (0, 0). This will be the "
-	                "top-left corner of a generated minimap."));
+	                "top-left corner of a generated minimap."),
+	              shortcut_string_for(KeyboardShortcut::kEditorMapOrigin, false));
 
 	tool_windows_.resizemap.open_window = [this] {
 		new EditorToolResizeOptionsMenu(*this, tools()->resize, tool_windows_.resizemap);
@@ -353,7 +379,8 @@ void EditorInteractive::add_tool_menu() {
 	toolmenu_.add(_("Map size"), ToolMenuEntry::kMapSize,
 	              g_image_cache->get("images/wui/editor/tools/resize_map.png"), false,
 	              /** TRANSLATORS: Tooltip for the map size tool in the editor */
-	              _("Change the map’s size"));
+	              _("Change the map’s size"),
+	              shortcut_string_for(KeyboardShortcut::kEditorMapSize, false));
 
 	/** TRANSLATORS: An entry in the editor's tool menu */
 	toolmenu_.add(_("Information"), ToolMenuEntry::kFieldInfo,
@@ -370,6 +397,7 @@ void EditorInteractive::add_tool_menu() {
 	              shortcut_string_for(KeyboardShortcut::kEditorToolHistory, false));
 	tool_windows_.toolhistory.open_window = [this] {
 		new EditorToolhistoryOptionsMenu(*this, tools()->tool_history, tool_windows_.toolhistory);
+		select_tool(tools()->info, EditorTool::First);
 	};
 
 	toolmenu_.selected.connect([this] { tool_menu_selected(toolmenu_.get_selected()); });
@@ -540,16 +568,9 @@ void EditorInteractive::cleanup_for_load() {
 	if (cleaning_up_) {
 		return;
 	}
+
 	cleaning_up_ = true;
-
-	// TODO(unknown): get rid of cleanup_for_load, it tends to be very messy
-	// Instead, delete and re-create the egbase.
-	// TODO(Nordfriese): …and then we can get rid of delete_world_and_tribes() as well
-	egbase().cleanup_for_load();
-	// This is needed so add-ons are configured correctly if current
-	// and previous map had different world add-on settings
-	egbase().delete_world_and_tribes();
-
+	egbase().full_cleanup();
 	cleaning_up_ = false;
 }
 
@@ -584,17 +605,15 @@ void EditorInteractive::think() {
 	egbase().get_gametime_pointer().increment(Duration(realtime_ - lasttime));
 }
 
-void EditorInteractive::exit() {
-	if (need_save_) {
-		if ((SDL_GetModState() & KMOD_CTRL) != 0) {
-			end_modal<UI::Panel::Returncodes>(UI::Panel::Returncodes::kBack);
-		} else {
-			UI::WLMessageBox mmb(this, UI::WindowStyle::kWui, _("Unsaved Map"),
-			                     _("The map has not been saved, do you really want to quit?"),
-			                     UI::WLMessageBox::MBoxType::kOkCancel);
-			if (mmb.run<UI::Panel::Returncodes>() == UI::Panel::Returncodes::kBack) {
-				return;
-			}
+void EditorInteractive::exit(const bool force) {
+	if (!force) {
+		UI::WLMessageBox mmb(
+		   this, UI::WindowStyle::kWui, need_save_ ? _("Unsaved Map") : _("Exit Editor Confirmation"),
+		   need_save_ ? _("The map has not been saved, do you really want to quit?") :
+                      _("Are you sure you wish to exit the editor?"),
+		   UI::WLMessageBox::MBoxType::kOkCancel);
+		if (mmb.run<UI::Panel::Returncodes>() == UI::Panel::Returncodes::kBack) {
+			return;
 		}
 	}
 	g_sh->change_music(Songset::kMenu, 200);
@@ -660,6 +679,17 @@ void EditorInteractive::draw(RenderTarget& dst) {
 	const float scale = 1.f / map_view()->view().zoom;
 	const Time& gametime = ebase.get_gametime();
 
+	float sel_alpha;
+	if (get_sel_gap_percent() < 10) {
+		sel_alpha = 1.f;
+	} else if (get_sel_gap_percent() <= 50) {
+		sel_alpha = 0.8f;
+	} else if (get_sel_gap_percent() < 100) {
+		sel_alpha = 0.5f;
+	} else {
+		sel_alpha = 0.f;
+	}
+
 	// The map provides a mapping from player number to Coords, while we require
 	// the inverse here. We construct this, but this is done on every frame and
 	// therefore potentially expensive - though it never showed up in any of my
@@ -689,8 +719,15 @@ void EditorInteractive::draw(RenderTarget& dst) {
 		}
 	}
 
+	std::map<std::pair<Widelands::DescriptionIndex, Widelands::ResourceAmount>, const Image*>
+	   resource_images_cache;
+
 	for (size_t idx = 0; idx < fields_to_draw->size(); ++idx) {
 		const FieldsToDraw::Field& field = fields_to_draw->at(idx);
+		if (field.obscured_by_slope) {
+			continue;
+		}
+
 		if (get_display_flag(dfShowImmovables)) {
 			Widelands::BaseImmovable* const imm = field.fcoords.field->get_immovable();
 			if (imm != nullptr && imm->get_positions(ebase).front() == field.fcoords) {
@@ -708,16 +745,20 @@ void EditorInteractive::draw(RenderTarget& dst) {
 		}
 
 		// Draw resource overlay.
-		uint8_t const amount = field.fcoords.field->get_resources_amount();
-		if (get_display_flag(dfShowResources) && amount > 0) {
-			const std::string& immname = ebase.descriptions()
-			                                .get_resource_descr(field.fcoords.field->get_resources())
-			                                ->editor_image(amount);
-			if (!immname.empty()) {
-				const auto* pic = g_image_cache->get(immname);
-				blit_field_overlay(
-				   &dst, field, pic, Vector2i(pic->width() / 2, pic->height() / 2), scale);
+		if (get_display_flag(dfShowResources) && field.fcoords.field->get_resources_amount() > 0) {
+			std::pair<Widelands::DescriptionIndex, Widelands::ResourceAmount> cachekey(
+			   field.fcoords.field->get_resources(), field.fcoords.field->get_resources_amount());
+			auto it = resource_images_cache.find(cachekey);
+			if (it == resource_images_cache.end()) {
+				it = resource_images_cache
+				        .emplace(cachekey, g_image_cache->get(ebase.descriptions()
+				                                                 .get_resource_descr(cachekey.first)
+				                                                 ->editor_image(cachekey.second)))
+				        .first;
 			}
+
+			blit_field_overlay(&dst, field, it->second,
+			                   Vector2i(it->second->width() / 2, it->second->height() / 2), scale);
 		}
 
 		const Widelands::NodeCaps nodecaps =
@@ -727,7 +768,7 @@ void EditorInteractive::draw(RenderTarget& dst) {
 		// Draw build help for maximum building spaces.
 		bool buildhelp_drawn = false;
 		if (get_display_flag(dfShowMaximumBuildhelp)) {
-			const auto* overlay = get_buildhelp_overlay(maxcaps);
+			const auto* overlay = get_buildhelp_overlay(maxcaps, scale);
 			if (overlay != nullptr) {
 				float opacity = 1.0f;
 				if (nodecaps != maxcaps) {
@@ -735,13 +776,14 @@ void EditorInteractive::draw(RenderTarget& dst) {
 				} else {
 					buildhelp_drawn = true;
 				}
-				blit_field_overlay(&dst, field, overlay->pic, overlay->hotspot, scale, opacity);
+				blit_field_overlay(
+				   &dst, field, overlay->pic, overlay->hotspot, scale / overlay->scale, opacity);
 			}
 		}
 
 		// Draw build help for actual building spaces.
 		if (buildhelp() && !buildhelp_drawn) {
-			const auto* overlay = get_buildhelp_overlay(nodecaps);
+			const auto* overlay = get_buildhelp_overlay(nodecaps, scale);
 			if (overlay != nullptr) {
 				// If maximum cap buildhelp is also on and is different,
 				// and current space is medium (ie. maximum is big or port),
@@ -751,7 +793,8 @@ void EditorInteractive::draw(RenderTarget& dst) {
 				         ((nodecaps & Widelands::BUILDCAPS_SIZEMASK) == Widelands::BUILDCAPS_MEDIUM) ?
                   0.9f :
                   1.0f;
-				blit_field_overlay(&dst, field, overlay->pic, overlay->hotspot, scale * scaling);
+				blit_field_overlay(
+				   &dst, field, overlay->pic, overlay->hotspot, scale / overlay->scale * scaling);
 			}
 		}
 
@@ -771,7 +814,7 @@ void EditorInteractive::draw(RenderTarget& dst) {
 			if (selected_nodes.count(field.fcoords) > 0) {
 				const Image* pic = get_sel_picture();
 				blit_field_overlay(
-				   &dst, field, pic, Vector2i(pic->width() / 2, pic->height() / 2), scale);
+				   &dst, field, pic, Vector2i(pic->width() / 2, pic->height() / 2), scale, sel_alpha);
 			}
 
 			// Draw selection markers on the triangles.
@@ -788,8 +831,8 @@ void EditorInteractive::draw(RenderTarget& dst) {
 					                       brn.rendertarget_pixel.y) /
 					                         3);
 					const Image* pic = get_sel_picture();
-					blit_overlay(
-					   &dst, tripos, pic, Vector2i(pic->width() / 2, pic->height() / 2), scale, 1.f);
+					blit_overlay(&dst, tripos, pic, Vector2i(pic->width() / 2, pic->height() / 2), scale,
+					             sel_alpha);
 				}
 				if (selected_triangles.count(
 				       Widelands::TCoords<>(field.fcoords, Widelands::TriangleIndex::D)) != 0u) {
@@ -800,8 +843,8 @@ void EditorInteractive::draw(RenderTarget& dst) {
 					                       brn.rendertarget_pixel.y) /
 					                         3);
 					const Image* pic = get_sel_picture();
-					blit_overlay(
-					   &dst, tripos, pic, Vector2i(pic->width() / 2, pic->height() / 2), scale, 1.f);
+					blit_overlay(&dst, tripos, pic, Vector2i(pic->width() / 2, pic->height() / 2), scale,
+					             sel_alpha);
 				}
 			}
 		}
@@ -819,15 +862,15 @@ void EditorInteractive::set_sel_pos(Widelands::NodeAndTriangle<> const sel) {
 	}
 }
 
-void EditorInteractive::set_sel_radius_and_update_menu(uint32_t const val) {
+void EditorInteractive::set_sel_radius_and_update_menu(uint32_t radius, uint16_t gap) {
 	if (tools_->current().has_size_one()) {
-		set_sel_radius(0);
+		set_sel_radius(0, 0);
 		return;
 	}
-	if (UI::UniqueWindow* const w = menu_windows_.toolsize.window) {
-		dynamic_cast<EditorToolsizeMenu&>(*w).update(val);
+	if (UI::UniqueWindow* const w = menu_windows_.toolsize.window; w != nullptr) {
+		dynamic_cast<EditorToolsizeMenu&>(*w).update(radius, gap);
 	} else {
-		set_sel_radius(val);
+		set_sel_radius(radius, gap);
 	}
 }
 
@@ -870,26 +913,78 @@ bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
 			menu_windows_.help.toggle();
 			return true;
 		}
-		if (matches_shortcut(KeyboardShortcut::kEditorMenu, code)) {
-			mainmenu_.toggle();
+		if (matches_shortcut(KeyboardShortcut::kCommonExit, code)) {
+			exit();
 			return true;
 		}
-		if (matches_shortcut(KeyboardShortcut::kEditorSave, code)) {
+		if (matches_shortcut(KeyboardShortcut::kCommonSave, code)) {
 			menu_windows_.savemap.toggle();
 			return true;
 		}
-		if (matches_shortcut(KeyboardShortcut::kEditorLoad, code)) {
+		if (matches_shortcut(KeyboardShortcut::kCommonLoad, code)) {
 			menu_windows_.loadmap.toggle();
 			return true;
 		}
+		if (matches_shortcut(KeyboardShortcut::kEditorNewMap, code)) {
+			menu_windows_.newmap.toggle();
+			return true;
+		}
+		if (matches_shortcut(KeyboardShortcut::kEditorNewRandomMap, code)) {
+			menu_windows_.newrandommap.toggle();
+			return true;
+		}
+		if (matches_shortcut(KeyboardShortcut::kEditorUploadMap, code)) {
+			publish_map();
+			return true;
+		}
+		if (matches_shortcut(KeyboardShortcut::kEditorMapOptions, code)) {
+			menu_windows_.mapoptions.toggle();
+			return true;
+		}
 		if (matches_shortcut(KeyboardShortcut::kEditorTools, code)) {
 			toolmenu_.toggle();
 			return true;
 		}
+		if (matches_shortcut(KeyboardShortcut::kEditorChangeHeight, code)) {
+			tool_windows_.height.toggle();
+			return true;
+		}
+		if (matches_shortcut(KeyboardShortcut::kEditorRandomHeight, code)) {
+			tool_windows_.noiseheight.toggle();
+			return true;
+		}
+		if (matches_shortcut(KeyboardShortcut::kEditorTerrain, code)) {
+			tool_windows_.terrain.toggle();
+			return true;
+		}
+		if (matches_shortcut(KeyboardShortcut::kEditorImmovables, code)) {
+			tool_windows_.immovables.toggle();
+			return true;
+		}
+		if (matches_shortcut(KeyboardShortcut::kEditorAnimals, code)) {
+			tool_windows_.critters.toggle();
+			return true;
+		}
+		if (matches_shortcut(KeyboardShortcut::kEditorResources, code)) {
+			tool_windows_.resources.toggle();
+			return true;
+		}
+		if (matches_shortcut(KeyboardShortcut::kEditorPortSpaces, code)) {
+			select_tool(tools()->set_port_space, EditorTool::First);
+			return true;
+		}
 		if (matches_shortcut(KeyboardShortcut::kEditorInfo, code)) {
 			select_tool(tools_->info, EditorTool::First);
 			return true;
 		}
+		if (matches_shortcut(KeyboardShortcut::kEditorMapOrigin, code)) {
+			select_tool(tools()->set_origin, EditorTool::First);
+			return true;
+		}
+		if (matches_shortcut(KeyboardShortcut::kEditorMapSize, code)) {
+			tool_windows_.resizemap.toggle();
+			return true;
+		}
 		if (matches_shortcut(KeyboardShortcut::kEditorPlayers, code)) {
 			tool_windows_.players.toggle();
 			return true;
@@ -928,10 +1023,15 @@ bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
 		}
 
 		for (int i = 0; i < 10; ++i) {
-			if (matches_shortcut(static_cast<KeyboardShortcut>(
-			                        static_cast<uint16_t>(KeyboardShortcut::kEditorToolsize1) + i),
-			                     code)) {
-				set_sel_radius_and_update_menu(i);
+			if (matches_shortcut(KeyboardShortcut::kEditorToolsize1 + i, code)) {
+				set_sel_radius_and_update_menu(i, get_sel_gap_percent());
+				return true;
+			}
+		}
+
+		for (int i = 0; i < 10; ++i) {
+			if (matches_shortcut(KeyboardShortcut::kEditorToolgap0 + i, code)) {
+				set_sel_radius_and_update_menu(get_sel_radius(), i * 10);
 				return true;
 			}
 		}
@@ -946,10 +1046,6 @@ bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
 
 		case SDLK_LCTRL:
 		case SDLK_RCTRL:
-		// TODO(GunChleoc): Keeping ALT and MODE to make the transition easier. Remove for Build 20.
-		case SDLK_LALT:
-		case SDLK_RALT:
-		case SDLK_MODE:
 			if (tools_->use_tool == EditorTool::First) {
 				select_tool(tools_->current(), EditorTool::Third);
 			}
@@ -968,10 +1064,6 @@ bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
 		case SDLK_RSHIFT:
 		case SDLK_LCTRL:
 		case SDLK_RCTRL:
-		// TODO(GunChleoc): Keeping ALT and MODE to make the transition easier. Remove for Build 20.
-		case SDLK_LALT:
-		case SDLK_RALT:
-		case SDLK_MODE:
 			if (tools_->use_tool != EditorTool::First) {
 				select_tool(tools_->current(), EditorTool::First);
 			}
@@ -985,20 +1077,25 @@ bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
 }
 
 bool EditorInteractive::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
-	int32_t change =
+	const int32_t change_size =
 	   get_mousewheel_change(MousewheelHandlerConfigID::kEditorToolsize, x, y, modstate);
-	if (change == 0) {
+	const int32_t change_gap =
+	   get_mousewheel_change(MousewheelHandlerConfigID::kEditorToolgap, x, y, modstate);
+
+	if (change_size == 0 && change_gap == 0) {
 		return false;
 	}
+
 	set_sel_radius_and_update_menu(
-	   std::max(0, std::min(static_cast<int32_t>(get_sel_radius()) + change, MAX_TOOL_AREA)));
+	   std::max(0, std::min(static_cast<int32_t>(get_sel_radius()) + change_size, MAX_TOOL_AREA)),
+	   std::max(0, std::min(static_cast<int32_t>(get_sel_gap_percent()) + change_gap, 100)));
 	return true;
 }
 
 void EditorInteractive::select_tool(EditorTool& primary, EditorTool::ToolIndex const which) {
 	if (which == EditorTool::First && &primary != tools_->current_pointer) {
 		if (primary.has_size_one()) {
-			set_sel_radius(0);
+			set_sel_radius(0, 0);
 			if (UI::UniqueWindow* const w = menu_windows_.toolsize.window) {
 				EditorToolsizeMenu& toolsize_menu = dynamic_cast<EditorToolsizeMenu&>(*w);
 				toolsize_menu.set_buttons_enabled(false);
@@ -1006,7 +1103,7 @@ void EditorInteractive::select_tool(EditorTool& primary, EditorTool::ToolIndex c
 		} else {
 			if (UI::UniqueWindow* const w = menu_windows_.toolsize.window) {
 				EditorToolsizeMenu& toolsize_menu = dynamic_cast<EditorToolsizeMenu&>(*w);
-				toolsize_menu.update(toolsize_menu.value());
+				toolsize_menu.update(toolsize_menu.radius(), toolsize_menu.gap_percent());
 			}
 		}
 		egbase().mutable_map()->recalc_whole_map(egbase());
@@ -1023,7 +1120,7 @@ void EditorInteractive::select_tool(EditorTool& primary, EditorTool::ToolIndex c
 	tool_settings_changed_ = true;
 }
 
-void EditorInteractive::run_editor(UI::Panel* error_message_parent,
+bool EditorInteractive::run_editor(UI::Panel* error_message_parent,
                                    const EditorInteractive::Init init,
                                    const std::string& filename,
                                    const std::string& script_to_run) {
@@ -1035,7 +1132,7 @@ void EditorInteractive::run_editor(UI::Panel* error_message_parent,
 		        "##############################\n",
 		        e.what());
 		if (error_message_parent == nullptr) {
-			return;
+			return false;
 		}
 		// Note: We don't necessarily want a bug report here, but the wording must
 		// be EXACTLY LIKE THIS in v1.0 to avoid adding a new translatable string
@@ -1050,7 +1147,9 @@ void EditorInteractive::run_editor(UI::Panel* error_message_parent,
 		      e.what(), build_ver_details()),
 		   UI::WLMessageBox::MBoxType::kOk);
 		m.run<UI::Panel::Returncodes>();
+		return false;
 	}
+	return true;
 }
 
 void EditorInteractive::do_run_editor(const EditorInteractive::Init init,
@@ -1069,7 +1168,7 @@ void EditorInteractive::do_run_editor(const EditorInteractive::Init init,
 		}
 	}
 
-	egbase.create_loader_ui({"editor"}, true, "", editor_splash_image());
+	egbase.create_loader_ui({"editor"}, true, "", kEditorSplashImage, false);
 	EditorInteractive::load_world_units(&eia, egbase);
 
 	if (init == EditorInteractive::Init::kLoadMapDirectly) {
@@ -1194,7 +1293,7 @@ void EditorInteractive::map_changed(const MapWas& action) {
 
 		tools_.reset(new Tools(*this, egbase().map()));
 		select_tool(tools_->info, EditorTool::First);
-		set_sel_radius(0);
+		set_sel_radius(0, 0);
 
 		set_need_save(false);
 		show_buildhelp(true);
@@ -1301,7 +1400,178 @@ UI::UniqueWindow::Registry& EditorInteractive::get_registry_for_window(WindowID
 	NEVER_HERE();
 }
 
-void EditorInteractive::set_sel_radius(const uint32_t n) {
-	InteractiveBase::set_sel_radius(n);
+void EditorInteractive::set_sel_radius(uint32_t radius, uint16_t gap) {
+	InteractiveBase::set_sel_radius(radius);
+	InteractiveBase::set_sel_gap_percent(gap);
 	tool_settings_changed_ = true;
+	set_tooltip(gap > 0 ? format(_("Tool gaps: %u%%"), gap) : "");
+}
+
+void EditorInteractive::publish_map() {
+	/* Create the add-on name. */
+	const std::string& map_name = egbase().map().get_name();
+	std::string sanitized_name;
+	sanitized_name.resize(map_name.size());
+	{
+		size_t i = 0;
+		for (const char* c = map_name.c_str(); *c != '\0'; ++c, ++i) {
+			if ((*c >= 'a' && *c <= 'z') || (*c >= 'A' && *c <= 'Z') || (*c >= '0' && *c <= '9') ||
+			    *c == '-' || *c == '_') {
+				sanitized_name[i] = *c;
+			} else {
+				sanitized_name[i] = '_';
+			}
+		}
+	}
+
+	/* Ask for confirmation. */
+	if ((SDL_GetModState() & KMOD_CTRL) == 0) {
+		UI::WLMessageBox w(
+		   this, UI::WindowStyle::kWui, _("Publish Map"),
+		   format(_("Are you certain that you want to publish this map online?\n\n"
+		            "Name: %1$s\n"
+		            "Internal name: %2$s\n"
+		            "Author: %3$s\n"
+		            "Description: %4$s\n"
+		            "Hint: %5$s\n"),
+		          map_name, sanitized_name + kAddOnExtension, egbase().map().get_author(),
+		          egbase().map().get_description(), egbase().map().get_hint()),
+		   UI::WLMessageBox::MBoxType::kOkCancel);
+		if (w.run<UI::Panel::Returncodes>() != UI::Panel::Returncodes::kOk) {
+			return;
+		}
+	}
+
+	/* Ask the user to log in. */
+	AddOns::NetAddons net;
+	AddOnsUI::AddOnsLoginBox login(*this, UI::WindowStyle::kWui);
+	if (login.run<UI::Panel::Returncodes>() != UI::Panel::Returncodes::kOk) {
+		return;
+	}
+	try {
+		net.set_login(login.get_username(), login.get_password());
+	} catch (const std::exception& e) {
+		UI::WLMessageBox mbox(
+		   this, UI::WindowStyle::kWui, _("Login Error"), e.what(), UI::WLMessageBox::MBoxType::kOk);
+		mbox.run<UI::Panel::Returncodes>();
+		return;
+	}
+
+	AddOnsUI::ProgressIndicatorWindow progress(this, UI::WindowStyle::kWui, _("Uploading Add-On…"));
+	progress.set_message_1(_("Saving map…"));
+
+	/* Save the map to a temp file. */
+	const std::string temp_file =
+	   kTempFileDir + FileSystem::file_separator() + sanitized_name + kTempFileExtension;
+	g_fs->ensure_directory_exists(kTempFileDir);
+	GenericSaveHandler gsh(
+	   [this](FileSystem& fs) {
+		   Widelands::MapSaver wms(fs, egbase());
+		   wms.save();
+	   },
+	   temp_file, FileSystem::ZIP);
+	GenericSaveHandler::Error error = gsh.save();
+
+	if (error != GenericSaveHandler::Error::kSuccess &&
+	    error != GenericSaveHandler::Error::kDeletingBackupFailed) {
+		progress.set_visible(false);
+		std::string msg = gsh.localized_formatted_result_message();
+		UI::WLMessageBox mbox(
+		   this, UI::WindowStyle::kWui, _("Error Saving Map!"), msg, UI::WLMessageBox::MBoxType::kOk);
+		mbox.run<UI::Panel::Returncodes>();
+		return;
+	}
+
+	/* Create the add-on directory, overwriting a previous add-on installation. */
+	progress.set_message_1(_("Packaging add-on…"));
+	const std::string addon_name = sanitized_name + kAddOnExtension;
+	const std::string addon_dir = kAddOnDir + FileSystem::file_separator() + addon_name;
+	const std::string map_dir = addon_dir + FileSystem::file_separator() + kAddOnMapsDir;
+	const std::string map_file =
+	   map_dir + FileSystem::file_separator() + sanitized_name + kWidelandsMapExtension;
+	const AddOns::AddOnInfo* existing = AddOns::find_addon(addon_name);
+
+	AddOns::AddOnVersion version;
+	if (existing != nullptr) {
+		version = existing->version;
+	}
+
+	if (g_fs->is_directory(addon_dir)) {
+		g_fs->fs_unlink(addon_dir);
+		assert(!g_fs->is_directory(addon_dir));
+	}
+	g_fs->ensure_directory_exists(map_dir);
+	g_fs->fs_rename(temp_file, map_file);
+
+	if (version.empty()) {
+		version = {1};
+	} else {
+		version = {version.at(0) + 1};
+	}
+
+	AddOns::AddOnInfo info;
+	info.internal_name = addon_name;
+	info.unlocalized_descname = map_name;
+	info.unlocalized_description = egbase().map().get_description();
+	info.unlocalized_author = egbase().map().get_author();
+	info.version = version;
+	info.category = AddOns::AddOnCategory::kMaps;
+	info.min_wl_version = egbase().map().version().minimum_required_widelands_version;
+
+	AddOns::NetAddons::CallbackFn fnn = [this](const std::string& /* f */, int64_t /* l */) {
+		do_redraw_now();
+	};
+	AddOns::MutableAddOn::ProgressFunction fnm = [this](size_t /* p */) { do_redraw_now(); };
+
+	AddOns::MapsAddon mutable_addon(info);
+	mutable_addon.set_dirname(kAddOnMapsDir, "Add-On Maps");
+	{
+		std::unique_ptr<FileSystem> map_fs(g_fs->make_sub_file_system(map_file));
+		mutable_addon.set_force_sync_safe(!map_fs->is_directory("scripting") ||
+		                                  map_fs->list_directory("scripting").empty());
+	}
+
+	mutable_addon.set_callbacks(fnm, fnm);
+	if (!mutable_addon.write_to_disk()) {
+		progress.set_visible(false);
+		UI::WLMessageBox mbox(this, UI::WindowStyle::kWui, _("Add-On Generation Error"),
+		                      _("The add-on could not be written to the disk."),
+		                      UI::WLMessageBox::MBoxType::kOk);
+		mbox.run<UI::Panel::Returncodes>();
+		return;
+	}
+
+	/* Upload to the server. */
+	progress.set_message_1(_("Uploading…"));
+	try {
+		net.upload_addon(addon_name, fnn, fnn);
+	} catch (const std::exception& e) {
+		progress.set_visible(false);
+		UI::WLMessageBox mbox(
+		   this, UI::WindowStyle::kWui, _("Upload Error"), e.what(), UI::WLMessageBox::MBoxType::kOk);
+		mbox.run<UI::Panel::Returncodes>();
+		return;
+	}
+
+	/* Finally, reload the local add-on. */
+	progress.set_message_1("");
+	{
+		bool found = false;
+		for (auto& pair : AddOns::g_addons) {
+			if (pair.first->internal_name == addon_name) {
+				pair.first = AddOns::preload_addon(addon_name);
+				found = true;
+				break;
+			}
+		}
+		if (!found) {
+			AddOns::g_addons.emplace_back(AddOns::preload_addon(addon_name), true);
+		}
+	}
+
+	progress.set_visible(false);
+	UI::WLMessageBox mbox(this, UI::WindowStyle::kWui, _("Success"),
+	                      _("The add-on was uploaded successfully."),
+	                      UI::WLMessageBox::MBoxType::kOk);
+	mbox.run<UI::Panel::Returncodes>();
 }
