@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,6 +40,8 @@ enum class WindowID {
 	ToolHistory,
 };
 
+constexpr int32_t kAutoTreesIndex = std::numeric_limits<uint16_t>::max() - 4;
+
 /**
  * An editor tool is a tool that can be selected in the editor. Examples are:
  * modify height, place immovable, place critter, place building. A Tool only
@@ -51,8 +53,7 @@ public:
 	EditorTool(EditorInteractive& parent, EditorTool& second, EditorTool& third, bool uda = true)
 	   : parent_(parent), second_(second), third_(third), undoable_(uda) {
 	}
-	virtual ~EditorTool() {
-	}
+	virtual ~EditorTool() = default;
 
 	enum ToolIndex { First, Second, Third };
 	int32_t handle_click(ToolIndex i,
@@ -83,10 +84,10 @@ public:
 		return (i == First ? *this : i == Second ? second_ : third_).format_args_impl();
 	}
 
-	bool is_undoable() {
+	[[nodiscard]] bool is_undoable() const {
 		return undoable_;
 	}
-	virtual bool has_size_one() const {
+	[[nodiscard]] virtual bool has_size_one() const {
 		return false;
 	}
 	virtual EditorActionArgs format_args_impl() {
@@ -104,7 +105,7 @@ public:
 	handle_undo_impl(const Widelands::NodeAndTriangle<>&, EditorActionArgs*, Widelands::Map*) {
 		return 0;
 	}  // non unduable tools don't need to implement this.
-	virtual const Image* get_sel_impl() const = 0;
+	[[nodiscard]] virtual const Image* get_sel_impl() const = 0;
 
 	// Gives the tool the chance to modify the nodecaps to change what will be
 	// displayed as build help.
@@ -120,7 +121,7 @@ public:
 		return fcoords.field->maxcaps();
 	}
 
-	virtual bool operates_on_triangles() const {
+	[[nodiscard]] virtual bool operates_on_triangles() const {
 		return false;
 	}
 
