@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -39,22 +39,48 @@ MainMenuLoadOrSaveMap::MainMenuLoadOrSaveMap(EditorInteractive& parent,
    : UI::UniqueWindow(
         &parent, UI::WindowStyle::kWui, name, &registry, parent.get_w(), parent.get_h(), title),
 
-     // Values for alignment and size
-     padding_(4),
-
      show_empty_dirs_(show_empty_dirs),
 
-     main_box_(this, UI::PanelStyle::kWui, padding_, padding_, UI::Box::Vertical, 0, 0, padding_),
+     main_box_(this,
+               UI::PanelStyle::kWui,
+               "main_box",
+               padding_,
+               padding_,
+               UI::Box::Vertical,
+               0,
+               0,
+               padding_),
 
-     table_and_details_box_(
-        &main_box_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal, 0, 0, padding_),
-     table_box_(
-        &table_and_details_box_, UI::PanelStyle::kWui, 0, 0, UI::Box::Vertical, 0, 0, padding_),
+     table_and_details_box_(&main_box_,
+                            UI::PanelStyle::kWui,
+                            "table_details_box",
+                            0,
+                            0,
+                            UI::Box::Horizontal,
+                            0,
+                            0,
+                            padding_),
+     table_box_(&table_and_details_box_,
+                UI::PanelStyle::kWui,
+                "table_box",
+                0,
+                0,
+                UI::Box::Vertical,
+                0,
+                0,
+                padding_),
 
      table_(&table_box_, 0, 0, 200, 200, UI::PanelStyle::kWui),
      egbase_(nullptr),
-     map_details_box_(
-        &table_and_details_box_, UI::PanelStyle::kWui, 0, 0, UI::Box::Vertical, 0, 0, padding_),
+     map_details_box_(&table_and_details_box_,
+                      UI::PanelStyle::kWui,
+                      "map_details_box",
+                      0,
+                      0,
+                      UI::Box::Vertical,
+                      0,
+                      0,
+                      padding_),
      map_details_(&map_details_box_, 0, 0, 100, 100, UI::PanelStyle::kWui, egbase_),
 
      display_mode_(&table_box_,
@@ -70,12 +96,21 @@ MainMenuLoadOrSaveMap::MainMenuLoadOrSaveMap(EditorInteractive& parent,
                    UI::PanelStyle::kWui,
                    UI::ButtonStyle::kWuiSecondary),
 
-     table_footer_box_(&main_box_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal, 0, 0, padding_),
+     table_footer_box_(
+        &main_box_, UI::PanelStyle::kWui, "footer_box", 0, 0, UI::Box::Horizontal, 0, 0, padding_),
 
-     directory_info_(&main_box_, UI::PanelStyle::kWui, UI::FontStyle::kWuiLabel, 0, 0, 0, 0),
+     directory_info_(&main_box_,
+                     UI::PanelStyle::kWui,
+                     "label_directory_info",
+                     UI::FontStyle::kWuiLabel,
+                     0,
+                     0,
+                     0,
+                     0),
 
      // Bottom button row
-     button_box_(&main_box_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal, 0, 0, padding_),
+     button_box_(
+        &main_box_, UI::PanelStyle::kWui, "buttons_box", 0, 0, UI::Box::Horizontal, 0, 0, padding_),
      ok_(&button_box_, "ok", 0, 0, 0, 0, UI::ButtonStyle::kWuiPrimary, _("OK")),
      cancel_(&button_box_, "cancel", 0, 0, 0, 0, UI::ButtonStyle::kWuiSecondary, _("Cancel")),
 
@@ -84,7 +119,7 @@ MainMenuLoadOrSaveMap::MainMenuLoadOrSaveMap(EditorInteractive& parent,
      include_addon_maps_(addons) {
 
 	g_fs->ensure_directory_exists(basedir_);
-	curdir_ = basedir_;
+	curdir_ = {basedir_};
 
 	main_box_.add(&table_and_details_box_, UI::Box::Resizing::kExpandBoth);
 	main_box_.add_space(padding_);
@@ -124,7 +159,7 @@ MainMenuLoadOrSaveMap::MainMenuLoadOrSaveMap(EditorInteractive& parent,
 	display_mode_.selected.connect([this]() { fill_table(); });
 	table_.cancel.connect([this]() { die(); });
 
-	move_to_top();
+	set_z(UI::Panel::ZOrder::kFullscreenWindow);
 }
 
 bool MainMenuLoadOrSaveMap::compare_players(uint32_t rowa, uint32_t rowb) {
@@ -151,20 +186,26 @@ void MainMenuLoadOrSaveMap::layout() {
 /**
  * fill the file list
  */
+// TODO(Nordfriese): Code duplication with FsMenu::MapSelect::fill_table
 void MainMenuLoadOrSaveMap::fill_table() {
 	table_.clear();
 	maps_data_.clear();
 
 	//  Fill it with all files we find.
-	FilenameSet files = g_fs->list_directory(curdir_);
+	assert(!curdir_.empty());
+	FilenameSet files;
+	for (const std::string& dir : curdir_) {
+		FilenameSet f = g_fs->list_directory(dir);
+		files.insert(f.begin(), f.end());
+	}
 
 	// If we are not at the top of the map directory hierarchy (we're not talking
 	// about the absolute filesystem top!) we manually add ".."
-	if (curdir_ != basedir_) {
-		maps_data_.push_back(MapData::create_parent_dir(curdir_));
+	if (curdir_.at(0) != basedir_) {
+		maps_data_.push_back(MapData::create_parent_dir(curdir_.at(0)));
 	} else {
 		if (files.empty()) {
-			maps_data_.push_back(MapData::create_empty_dir(curdir_));
+			maps_data_.push_back(MapData::create_empty_dir(curdir_.at(0)));
 		}
 		// In the toplevel directory we also need to include add-on maps –
 		// but only in the load screen, not in the save screen!
@@ -207,7 +248,8 @@ void MainMenuLoadOrSaveMap::fill_table() {
 				}
 
 				maps_data_.emplace_back(map, mapfilename, maptype, display_type);
-			} catch (const WException&) {
+			} catch (const WException& e) {
+				log_warn("Map list: Skip %s due to preload error: %s\n", mapfilename.c_str(), e.what());
 			}  //  we simply skip illegal entries
 		} else if (g_fs->is_directory(mapfilename) &&
 		           (show_empty_dirs_ || !g_fs->list_directory(mapfilename).empty())) {
@@ -216,7 +258,20 @@ void MainMenuLoadOrSaveMap::fill_table() {
 			if ((strcmp(fs_filename, ".") == 0) || (strcmp(fs_filename, "..") == 0)) {
 				continue;
 			}
-			maps_data_.push_back(MapData::create_directory(mapfilename));
+
+			MapData new_md = MapData::create_directory(mapfilename);
+			bool found = false;
+			for (MapData& md : maps_data_) {
+				if (md.maptype == MapData::MapType::kDirectory &&
+				    md.localized_name == new_md.localized_name) {
+					found = true;
+					md.add(new_md);
+					break;
+				}
+			}
+			if (!found) {
+				maps_data_.push_back(new_md);
+			}
 		}
 	}
 
