@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,7 +37,11 @@
 
 namespace Widelands {
 
-constexpr uint16_t kCurrentPacketVersion = 9;
+/* Changelog:
+ * 9: v1.1
+ * 10: Added configurable win condition duration
+ */
+constexpr uint16_t kCurrentPacketVersion = 10;
 constexpr const char* kMinimapFilename = "minimap.png";
 
 // Win condition localization can come from the 'widelands' or 'win_conditions' textdomain.
@@ -56,13 +60,12 @@ void GamePreloadPacket::read(FileSystem& fs, Game& /* game */, MapObjectLoader*
 		Section& s = prof.get_safe_section("global");
 		int32_t const packet_version = s.get_int("packet_version");
 
-		if (packet_version >= 6 && packet_version <= kCurrentPacketVersion) {
+		if (packet_version >= 9 && packet_version <= kCurrentPacketVersion) {
 			gametime_ = Time(s.get_safe_int("gametime"));
 			mapname_ = s.get_safe_string("mapname");
 
 			background_ = s.get_safe_string("background");
-			// TODO(Nordfriese): Savegame compatibility
-			background_theme_ = (packet_version < 7 ? "" : s.get_safe_string("theme"));
+			background_theme_ = s.get_safe_string("theme");
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0. `kCurrentPacketVersion` will
        // then need to be increased, and the minimum version for the following two values updated
        // accordingly.
@@ -73,6 +76,9 @@ void GamePreloadPacket::read(FileSystem& fs, Game& /* game */, MapObjectLoader*
 #endif
 			player_nr_ = s.get_safe_int("player_nr");
 			win_condition_ = s.get_safe_string("win_condition");
+			// TODO(Nordfriese): Savegame compatibility v1.1
+			win_condition_duration_ = (packet_version < 10 ? kDefaultWinConditionDuration :
+                                                          s.get_safe_int("win_condition_duration"));
 			number_of_players_ = s.get_safe_int("player_amount");
 			version_ = s.get_safe_string("widelands_version");
 			if (fs.file_exists(kMinimapFilename)) {
@@ -137,6 +143,7 @@ void GamePreloadPacket::write(FileSystem& fs, Game& game, MapObjectSaver* const
 	s.set_string("background", map.get_background());
 	s.set_string("theme", map.get_background_theme());
 	s.set_string("win_condition", game.get_win_condition_displayname());
+	s.set_int("win_condition_duration", game.get_win_condition_duration());
 	s.set_int("savetimestamp", static_cast<uint32_t>(time(nullptr)));
 	s.set_int("gametype", static_cast<int32_t>(game.game_controller() != nullptr ?
                                                  game.game_controller()->get_game_type() :
