@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,7 +21,6 @@
 #include <cassert>
 #include <memory>
 
-#include "base/log.h"
 #include "base/math.h"
 #include "base/vector.h"
 #include "io/filesystem/layered_filesystem.h"
@@ -33,19 +32,14 @@
 const std::map<float, std::string> Animation::kSupportedScales{
    {0.5, "_0.5"}, {1, "_1"}, {2, "_2"}, {4, "_4"}};
 
-Animation::MipMapEntry::MipMapEntry() : has_playercolor_masks(false) {
-}
-
 Animation::Animation(const LuaTable& table)
-   : nr_frames_(0),
-     representative_frame_(
+   : representative_frame_(
         table.has_key("representative_frame") ? table.get_int("representative_frame") : 0),
      hotspot_(table.get_vector<std::string, int>("hotspot")),
      frametime_(table.has_key("fps") ? (1000 / get_positive_int(table, "fps")) : kFrameLength),
      play_once_(table.has_key("play_once") ? table.get_bool("play_once") : false),
      sound_effect_(kNoSoundEffect),
-     sound_priority_(kFxPriorityLowest),
-     sound_allow_multiple_(false) {
+     sound_priority_(kFxPriorityLowest) {
 	try {
 		// Sound
 		if (table.has_key("sound_effect")) {
@@ -53,14 +47,7 @@ Animation::Animation(const LuaTable& table)
 			sound_effect_ =
 			   SoundHandler::register_fx(SoundType::kAmbient, sound_effects->get_string("path"));
 
-			try {
-				sound_priority_ = math::read_percent_to_int(sound_effects->get_string("priority"));
-			} catch (const std::exception&) {
-				// TODO(GunChleoc): Compatibility, remove try-catch after v1.0
-				log_warn("Animation sound effect priority '%.2f' without percent symbol is deprecated",
-				         sound_effects->get_double("priority"));
-				sound_priority_ = std::round(100 * sound_effects->get_double("priority"));
-			}
+			sound_priority_ = math::read_percent_to_int(sound_effects->get_string("priority"));
 
 			if (sound_effects->has_key<std::string>("allow_multiple")) {
 				sound_allow_multiple_ = sound_effects->get_bool("allow_multiple");
