@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2022 by the Widelands Development Team
+ * Copyright (C) 2017-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@
 #include <memory>
 
 #include "graphic/graphic.h"
+#include "graphic/hyperlink.h"
 
 namespace UI {
 // RenderedRect
@@ -30,51 +31,63 @@ RenderedRect::RenderedRect(const Recti& init_rect,
                            bool visited,
                            const RGBColor& color,
                            bool is_background_color_set,
-                           DrawMode init_mode)
+                           DrawMode init_mode,
+                           const TextClickTarget* click_target)
    : rect_(init_rect),
      transient_image_(std::move(init_image)),
      permanent_image_(nullptr),
      visited_(visited),
      background_color_(color),
      is_background_color_set_(is_background_color_set),
-     mode_(init_mode) {
+     mode_(init_mode),
+     click_target_(click_target) {
 }
 RenderedRect::RenderedRect(const Recti& init_rect,
                            const Image* init_image,
                            bool visited,
                            const RGBColor& color,
                            bool is_background_color_set,
-                           DrawMode init_mode)
+                           DrawMode init_mode,
+                           const TextClickTarget* click_target)
    : rect_(init_rect),
      transient_image_(nullptr),
      permanent_image_(init_image),
      visited_(visited),
      background_color_(color),
      is_background_color_set_(is_background_color_set),
-     mode_(init_mode) {
+     mode_(init_mode),
+     click_target_(click_target) {
 }
 
-RenderedRect::RenderedRect(const Recti& init_rect, const Image* init_image)
-   : RenderedRect(init_rect, init_image, false, RGBColor(0, 0, 0), false, DrawMode::kTile) {
+RenderedRect::RenderedRect(const Recti& init_rect,
+                           const Image* init_image,
+                           const TextClickTarget* click_target)
+   : RenderedRect(
+        init_rect, init_image, false, RGBColor(0, 0, 0), false, DrawMode::kTile, click_target) {
 }
-RenderedRect::RenderedRect(const Recti& init_rect, const RGBColor& color)
-   : RenderedRect(init_rect, nullptr, false, color, true, DrawMode::kTile) {
+RenderedRect::RenderedRect(const Recti& init_rect,
+                           const RGBColor& color,
+                           const TextClickTarget* click_target)
+   : RenderedRect(init_rect, nullptr, false, color, true, DrawMode::kTile, click_target) {
 }
-RenderedRect::RenderedRect(const std::shared_ptr<const Image>& init_image)
+RenderedRect::RenderedRect(const std::shared_ptr<const Image>& init_image,
+                           const TextClickTarget* click_target)
    : RenderedRect(Recti(0, 0, init_image->width(), init_image->height()),
                   init_image,
                   false,
                   RGBColor(0, 0, 0),
                   false,
-                  DrawMode::kBlit) {
+                  DrawMode::kBlit,
+                  click_target) {
 }
-RenderedRect::RenderedRect(const Image* init_image)
+RenderedRect::RenderedRect(const Image* init_image, const TextClickTarget* click_target)
    : RenderedRect(Recti(0, 0, init_image->width(), init_image->height()),
                   init_image,
                   false,
                   RGBColor(0, 0, 0),
                   false,
-                  DrawMode::kBlit) {
+                  DrawMode::kBlit,
+                  click_target) {
 }
 
 const Image* RenderedRect::image() const {
@@ -82,6 +95,10 @@ const Image* RenderedRect::image() const {
 	return permanent_image_ == nullptr ? transient_image_.get() : permanent_image_;
 }
 
+const Recti& RenderedRect::rect() const {
+	return rect_;
+}
+
 int RenderedRect::x() const {
 	return rect_.x;
 }
@@ -97,6 +114,13 @@ int RenderedRect::height() const {
 	return rect_.h;
 }
 
+bool RenderedRect::handle_mousepress(int32_t x, int32_t y) const {
+	return click_target_->handle_mousepress(x - rect_.x, y - rect_.y);
+}
+const std::string* RenderedRect::get_tooltip(int32_t x, int32_t y) const {
+	return click_target_->get_tooltip(x - rect_.x, y - rect_.y);
+}
+
 void RenderedRect::set_origin(const Vector2i& new_origin) {
 	rect_.x = new_origin.x;
 	rect_.y = new_origin.y;
@@ -120,6 +144,16 @@ RenderedRect::DrawMode RenderedRect::mode() const {
 }
 
 // RenderedText
+RenderedText::~RenderedText() {
+	delete memory_tree_root_;
+}
+
+void RenderedText::set_memory_tree_root(TextClickTarget* t) {
+	assert(t != nullptr);
+	assert(memory_tree_root_ == nullptr);
+	memory_tree_root_ = t;
+}
+
 int RenderedText::width() const {
 	int result = 0;
 	for (const auto& rect : rects) {
@@ -135,6 +169,26 @@ int RenderedText::height() const {
 	return result;
 }
 
+bool RenderedText::handle_mousepress(int32_t x, int32_t y) const {
+	for (const auto& r : rects) {
+		if (r->rect().contains(Vector2i(x, y)) && r->handle_mousepress(x, y)) {
+			return true;
+		}
+	}
+	return false;
+}
+const std::string* RenderedText::get_tooltip(int32_t x, int32_t y) const {
+	for (const auto& r : rects) {
+		if (r->rect().contains(Vector2i(x, y))) {
+			const std::string* tt = r->get_tooltip(x, y);
+			if (tt != nullptr && !tt->empty()) {
+				return tt;
+			}
+		}
+	}
+	return nullptr;
+}
+
 void RenderedText::draw(RenderTarget& dst,
                         const Vector2i& position,
                         const Recti& region,
