@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2022 by the Widelands Development Team
+ * Copyright (C) 2006-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -500,7 +500,7 @@ struct RealFSStreamRead : public StreamRead {
 		return fread(read_data, 1, bufsize, file_);
 	}
 
-	bool end_of_file() const override {
+	[[nodiscard]] bool end_of_file() const override {
 		return feof(file_) != 0;
 	}
 
