@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,6 +43,7 @@
 #include "base/i18n.h"
 #include "base/log.h"
 #include "base/string.h"
+#include "base/time_string.h"
 #include "config.h"
 #include "io/filesystem/disk_filesystem.h"
 #include "io/filesystem/filesystem_exceptions.h"
@@ -82,7 +83,7 @@ private:
 	std::string template_;
 	std::string format_;
 	std::string to_replace_;
-	uint32_t current_;
+	uint32_t current_{0U};
 	uint32_t max_;
 
 	DISALLOW_COPY_AND_ASSIGN(NumberGlob);
@@ -91,7 +92,7 @@ private:
 /**
  * Implementation for NumberGlob.
  */
-NumberGlob::NumberGlob(const std::string& file_template) : template_(file_template), current_(0) {
+NumberGlob::NumberGlob(const std::string& file_template) : template_(file_template) {
 	int nchars = count(file_template.begin(), file_template.end(), '?');
 	format_ = "%0" + as_string(nchars) + "i";
 
@@ -349,6 +350,30 @@ std::vector<std::string> FileSystem::get_xdgdatadirs() {
 }
 #endif
 
+std::string FileSystem::create_unique_temp_file_path(std::string prefix,
+                                                     const std::string& suffix) const {
+	prefix += file_separator();
+	prefix += timestring();
+
+	std::string complete_filename = prefix + suffix;
+	if (!file_exists(complete_filename)) {
+		return complete_filename;
+	}
+
+	prefix += '-';
+	for (int i = 0; i <= 9; ++i) {
+		complete_filename = prefix;
+		complete_filename += std::to_string(i);
+		complete_filename += suffix;
+		if (!file_exists(complete_filename)) {
+			return complete_filename;
+		}
+	}
+
+	throw wexception(
+	   "Could not create a unique filename similar near %s", complete_filename.c_str());
+}
+
 // Returning a vector rather than a set because animations need the indices
 std::vector<std::string> FileSystem::get_sequential_files(const std::string& directory,
                                                           const std::string& basename,
