@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2022 by the Widelands Development Team
+ * Copyright (C) 2020-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,7 @@
 
 #include <list>
 #include <memory>
+#include <regex>
 #include <set>
 
 #include "base/log.h"
@@ -28,6 +29,7 @@
 #include "build_info.h"
 #include "graphic/image_cache.h"
 #include "graphic/style_manager.h"
+#include "graphic/text_layout.h"
 #include "io/filesystem/layered_filesystem.h"
 #include "io/profile.h"
 #include "wlapplication_options.h"
@@ -43,29 +45,35 @@ const std::unordered_map<std::string, std::string> kDifficultyIcons = {
 
 const std::map<AddOnCategory, AddOnCategoryInfo> kAddOnCategories = {
    {AddOnCategory::kNone,
-    AddOnCategoryInfo{"", []() { return _("Error"); }, "images/ui_basic/stop.png"}},
-   {AddOnCategory::kTribes, AddOnCategoryInfo{"tribes", []() { return _("Tribes"); },
-                                              "images/wui/stats/menu_tab_wares_warehouse.png"}},
+    AddOnCategoryInfo{"", []() { return _("Error"); }, "images/ui_basic/stop.png", false}},
+   {AddOnCategory::kTribes,
+    AddOnCategoryInfo{"tribes", []() { return _("Tribes"); },
+                      "images/wui/stats/menu_tab_wares_warehouse.png", true}},
    {AddOnCategory::kWorld, AddOnCategoryInfo{"world", []() { return _("World"); },
-                                             "images/wui/menus/toggle_immovables.png"}},
-   {AddOnCategory::kScript,
-    AddOnCategoryInfo{"script", []() { return _("Script"); }, "images/logos/WL-Editor-32.png"}},
+                                             "images/wui/menus/toggle_immovables.png", true}},
+   {AddOnCategory::kScript, AddOnCategoryInfo{"script", []() { return _("Script"); },
+                                              "images/logos/WL-Editor-32.png", true}},
    {AddOnCategory::kMaps, AddOnCategoryInfo{"maps", []() { return _("Map Set"); },
-                                            "images/wui/menus/toggle_minimap.png"}},
+                                            "images/wui/menus/toggle_minimap.png", true}},
+   {AddOnCategory::kMapGenerator,
+    AddOnCategoryInfo{"map_generator", []() { return _("Map Generator"); },
+                      "images/wui/editor/menus/new_random_map.png", false}},
    {AddOnCategory::kCampaign, AddOnCategoryInfo{"campaign", []() { return _("Campaign"); },
-                                                "images/wui/messages/messages_warfare.png"}},
+                                                "images/wui/messages/messages_warfare.png", false}},
    {AddOnCategory::kWinCondition,
-    AddOnCategoryInfo{
-       "win_condition", []() { return _("Win Condition"); }, "images/wui/menus/objectives.png"}},
+    AddOnCategoryInfo{"win_condition", []() { return _("Win Condition"); },
+                      "images/wui/menus/objectives.png", true}},
    {AddOnCategory::kStartingCondition,
     AddOnCategoryInfo{"starting_condition", []() { return _("Starting Condition"); },
-                      "tribes/buildings/warehouses/atlanteans/headquarters/menu.png"}},
-   {AddOnCategory::kTheme,
-    AddOnCategoryInfo{"theme", []() { return _("Theme"); }, "images/wui/menus/main_menu.png"}}};
+                      "tribes/buildings/warehouses/atlanteans/headquarters/menu.png", true}},
+   {AddOnCategory::kTheme, AddOnCategoryInfo{"theme", []() { return _("Theme"); },
+                                             "images/wui/menus/main_menu.png", false}},
+   {AddOnCategory::kUIPlugin,
+    AddOnCategoryInfo{"ui_plugin", []() { return _("UI Plugin"); }, "images/plugin.png", false}}};
 
 std::vector<AddOnState> g_addons;
 
-static const AddOnInfo* find_addon(const std::string& name) {
+const AddOnInfo* find_addon(const std::string& name) {
 	for (const auto& pair : g_addons) {
 		if (pair.first->internal_name == name) {
 			return pair.first.get();
@@ -111,16 +119,35 @@ std::string version_to_string(const AddOnVersion& version, const bool localize)
 }
 AddOnVersion string_to_version(std::string input) {
 	AddOnVersion result;
-	for (;;) {
-		const size_t pos = input.find('.');
-		if (pos == std::string::npos) {
-			result.push_back(math::to_long(input));
-			return result;
-		}
-		result.push_back(math::to_long(input.substr(0, pos)));
-		input = input.substr(pos + 1);
+	if (input.empty()) {
+		return result;
 	}
-	NEVER_HERE();
+
+	static std::regex kNewStyleRegex("[0-9]+(\\.[0-9]+)*");
+	static std::regex kOldStyleRegex("build ([0-9]+)");
+
+	if (std::regex_match(input, kNewStyleRegex)) {
+		for (;;) {
+			const size_t pos = input.find('.');
+			if (pos == std::string::npos) {
+				result.push_back(math::to_long(input));
+				return result;
+			}
+
+			result.push_back(math::to_long(input.substr(0, pos)));
+			input = input.substr(pos + 1);
+		}
+
+		NEVER_HERE();
+	}
+
+	if (std::smatch match; std::regex_match(input, match, kOldStyleRegex)) {
+		result.push_back(0);
+		result.push_back(math::to_long(match.str(1)));
+		return result;
+	}
+
+	throw wexception("Malformed version string: '%s'", input.c_str());
 }
 
 bool is_newer_version(const AddOnVersion& base, const AddOnVersion& compare) {
@@ -146,6 +173,17 @@ bool order_matters(AddOnCategory base, AddOnCategory dependency) {
 	}
 }
 
+bool require_enabled(AddOnCategory base, AddOnCategory dependency) {
+	switch (base) {
+	case AddOnCategory::kMaps:
+	case AddOnCategory::kCampaign:
+		// Maps enable their own world add-ons automatically
+		return dependency != AddOnCategory::kWorld;
+	default:
+		return true;
+	}
+}
+
 static AddOnConflict check_requirements_conflicts(const AddOnRequirements& required_addons) {
 	std::set<std::string> addons_missing;
 	std::map<std::string, std::pair<AddOnVersion, AddOnVersion>> addons_wrong_version;
@@ -234,7 +272,7 @@ AddOnConflict check_requirements(const AddOnRequirements& required_addons) {
 	}
 	AddOnConflict result = check_requirements_conflicts(required_addons);
 	for (const auto& pair : required_addons) {
-		result.first = format(_("%1$s<br>· %2$s (version %3$s)"), result.first, pair.first,
+		result.first = format(_("%1$s<br>• %2$s (version %3$s)"), result.first, pair.first,
 		                      version_to_string(pair.second));
 	}
 	return result;
@@ -254,6 +292,30 @@ unsigned count_all_dependencies(const std::string& info,
 	return deps;
 }
 
+std::string list_game_relevant_addons() {
+	std::vector<std::string> addons;
+	for (const auto& pair : AddOns::g_addons) {
+		if (pair.second && AddOns::kAddOnCategories.at(pair.first->category).network_relevant) {
+			addons.push_back(pair.first->descname());
+		}
+	}
+
+	if (addons.empty()) {
+		return as_richtext(g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
+		                      .as_font_tag(_("No game-relevant add-ons in use.")));
+	}
+
+	std::string addons_text =
+	   g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelHeading)
+	      .as_font_tag(format(ngettext("%u game-relevant add-on in use:",
+	                                   "%u game-relevant add-ons in use:", addons.size()),
+	                          addons.size()));
+	for (const std::string& a : addons) {
+		addons_text += as_listitem(a, UI::FontStyle::kFsMenuInfoPanelParagraph);
+	}
+	return as_richtext(addons_text);
+}
+
 AddOnCategory get_category(const std::string& name) {
 	for (const auto& pair : kAddOnCategories) {
 		if (pair.second.internal_name == name) {
@@ -281,16 +343,13 @@ double AddOnInfo::average_rating() const {
 }
 
 static bool contains_png(const std::string& dir) {
-	for (const std::string& f : g_fs->list_directory(dir)) {
+	const auto dirs = g_fs->list_directory(dir);
+	return std::any_of(dirs.begin(), dirs.end(), [](const std::string& f) {
 		if (g_fs->is_directory(f)) {
-			if (contains_png(f)) {
-				return true;
-			}
-		} else if (FileSystem::filename_ext(f) == ".png") {
-			return true;
+			return contains_png(f);
 		}
-	}
-	return false;
+		return FileSystem::filename_ext(f) == ".png";
+	});
 }
 // Rebuilding the texture atlas is required if an add-on defines new terrain, flag, or road
 // textures.
@@ -365,7 +424,14 @@ void update_ui_theme(const UpdateThemeAction action, std::string arg) {
 	NEVER_HERE();
 }
 
-bool AddOnInfo::matches_widelands_version() const {
+bool AddOnInfo::matches_widelands_version(const bool warn_future) const {
+	return AddOns::matches_widelands_version(
+	   min_wl_version, max_wl_version, warn_future ? internal_name : std::string());
+}
+
+bool matches_widelands_version(const std::string& min_wl_version,
+                               const std::string& max_wl_version,
+                               const std::string& warn_future_name) {
 	if (min_wl_version.empty() && max_wl_version.empty()) {
 		return true;
 	}
@@ -373,26 +439,27 @@ bool AddOnInfo::matches_widelands_version() const {
 	const std::string& wl_version = build_id();
 	// Two cases. Either we have a release version such as "1.0".
 	// Or we have a development version such as "1.0~git25169[9d77594@master]" –
-	// which is then considered older than the version string before the '~'
-	// but newer than any version less than that.
+	// which is then considered equal to the version string before the '~'.
+	// It is the responsibility of testers of development versions to always
+	// use the latest one, and to be prepared for failures in any case.
 	const size_t tilde = wl_version.find('~');
-	if (tilde == std::string::npos) {
-		AddOnVersion wl = string_to_version(wl_version);
-		if (!min_wl_version.empty() && wl < string_to_version(min_wl_version)) {
-			return false;
-		}
-		if (!max_wl_version.empty() && wl > string_to_version(max_wl_version)) {
-			return false;
-		}
-	} else {
-		AddOnVersion next_wl = string_to_version(wl_version.substr(0, tilde));
-		if (!min_wl_version.empty() && next_wl <= string_to_version(min_wl_version)) {
-			return false;
-		}
-		if (!max_wl_version.empty() && next_wl > string_to_version(max_wl_version)) {
-			return false;
-		}
+	const AddOnVersion wl =
+	   string_to_version(tilde == std::string::npos ? wl_version : wl_version.substr(0, tilde));
+
+	if (!min_wl_version.empty() && wl < string_to_version(min_wl_version)) {
+		return false;
 	}
+	if (!max_wl_version.empty() && wl > string_to_version(max_wl_version)) {
+		return false;
+	}
+
+	if (!warn_future_name.empty() && !min_wl_version.empty() && tilde != std::string::npos &&
+	    wl == string_to_version(min_wl_version)) {
+		log_warn("Add-on '%s' requires minimum Widelands version %s from the future.\n"
+		         "Make sure your development build is up to date!",
+		         warn_future_name.c_str(), min_wl_version.c_str());
+	}
+
 	return true;
 }
 
@@ -441,6 +508,10 @@ std::shared_ptr<AddOnInfo> preload_addon(const std::string& name) {
 	if (i->version.empty()) {
 		throw wexception("preload_addon (%s): version string is empty", name.c_str());
 	}
+	if (!i->matches_widelands_version(true)) {
+		throw wexception(
+		   "preload_addon (%s): incompatible with this Widelands version", name.c_str());
+	}
 
 	for (std::string req(s.get_safe_string("requires")); !req.empty();) {
 		const size_t commapos = req.find(',');
