@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,7 +24,6 @@
 #include "base/log.h"
 #include "base/macros.h"
 #include "base/scoped_timer.h"
-#include "base/time_string.h"
 #include "base/wexception.h"
 #include "economy/flag.h"
 #include "economy/road.h"
@@ -68,18 +67,11 @@ initialization
 ============
 */
 EditorGameBase::EditorGameBase(LuaInterface* lua_interface)
-   : did_postload_addons_(false),
-     did_postload_tribes_(false),
-     gametime_(0),
-     // TODO(SirVer): this is sooo ugly, I can't say
+   :  // TODO(SirVer): this is sooo ugly, I can't say
      lua_(lua_interface != nullptr ? lua_interface : new LuaEditorInterface(this)),
      player_manager_(new PlayersManager(*this)),
-     ibase_(nullptr),
-     loader_ui_(nullptr),
-     game_tips_(nullptr),
      loading_message_subscriber_(Notifications::subscribe<UI::NoteLoadingMessage>(
-        [this](const UI::NoteLoadingMessage& note) { step_loader_ui(note.message); })),
-     tmp_fs_(nullptr) {
+        [this](const UI::NoteLoadingMessage& note) { step_loader_ui(note.message); })) {
 
 	init_addons(false);
 
@@ -145,26 +137,8 @@ void EditorGameBase::create_tempfile_and_save_mapdata(FileSystem::Type const typ
 	try {
 		g_fs->ensure_directory_exists(kTempFileDir);
 
-		std::string filename =
-		   kTempFileDir + FileSystem::file_separator() + timestring() + "_mapdata";
-		std::string complete_filename = filename + kTempFileExtension;
-
-		// if a file with that name already exists, then try a few name modifications
-		if (g_fs->file_exists(complete_filename)) {
-			int suffix;
-			for (suffix = 0; suffix <= 9; suffix++) {
-				complete_filename =
-				   filename.append("-").append(std::to_string(suffix)).append(kTempFileExtension);
-				if (!g_fs->file_exists(complete_filename)) {
-					break;
-				}
-			}
-			if (suffix > 9) {
-				throw wexception(
-				   "EditorGameBase::create_tempfile_and_save_mapdata(): for all considered "
-				   "filenames a file already existed");
-			}
-		}
+		std::string complete_filename = g_fs->create_unique_temp_file_path(
+		   kTempFileDir, std::string("_mapdata") + kTempFileExtension);
 
 		// create tmp_fs_
 		tmp_fs_.reset(g_fs->create_sub_file_system(complete_filename, type));
@@ -230,6 +204,16 @@ Descriptions* EditorGameBase::mutable_descriptions() {
 
 void EditorGameBase::set_ibase(InteractiveBase* const b) {
 	ibase_.reset(b);
+
+	// Now that the IBase and the EGBase are linked into each other, load the UI plugins.
+	if (b != nullptr) {
+		for (const auto& pair : AddOns::g_addons) {
+			if (pair.second && pair.first->category == AddOns::AddOnCategory::kUIPlugin) {
+				lua().run_script(kAddOnDir + FileSystem::file_separator() + pair.first->internal_name +
+				                 FileSystem::file_separator() + "init.lua");
+			}
+		}
+	}
 }
 
 InteractiveGameBase* EditorGameBase::get_igbase() const {
@@ -262,7 +246,7 @@ const Player& EditorGameBase::player(const int32_t n) const {
 }
 
 void EditorGameBase::inform_players_about_ownership(MapIndex const i,
-                                                    PlayerNumber const new_owner) {
+                                                    PlayerNumber const new_owner) const {
 	iterate_players_existing_const(plnum, kMaxPlayers, *this, p) {
 		Player::Field& player_field = p->fields_[i];
 		if (VisibleState::kVisible == player_field.vision) {
@@ -271,7 +255,7 @@ void EditorGameBase::inform_players_about_ownership(MapIndex const i,
 	}
 }
 void EditorGameBase::inform_players_about_immovable(MapIndex const i,
-                                                    MapObjectDescr const* const descr) {
+                                                    MapObjectDescr const* const descr) const {
 	if (!Road::is_road_descr(descr) && !Waterway::is_waterway_descr(descr)) {
 		iterate_players_existing_const(plnum, kMaxPlayers, *this, p) {
 			Player::Field& player_field = p->fields_[i];
@@ -294,7 +278,7 @@ void EditorGameBase::load_all_tribes() {
 	}
 }
 
-void EditorGameBase::allocate_player_maps() {
+void EditorGameBase::allocate_player_maps() const {
 	iterate_players_existing(plnum, kMaxPlayers, *this, p) {
 		p->allocate_map();
 	}
@@ -363,9 +347,10 @@ UI::ProgressWindow& EditorGameBase::create_loader_ui(const std::vector<std::stri
                                                      bool show_game_tips,
                                                      const std::string& theme,
                                                      const std::string& background,
+                                                     bool crop,
                                                      UI::Panel* parent) {
 	assert(!has_loader_ui());
-	loader_ui_.reset(new UI::ProgressWindow(parent, theme, background));
+	loader_ui_.reset(new UI::ProgressWindow(parent, theme, background, crop));
 	registered_game_tips_ = tipstexts;
 	if (show_game_tips) {
 		game_tips_.reset(registered_game_tips_.empty() ?
@@ -609,7 +594,7 @@ void EditorGameBase::full_cleanup() {
 
 void EditorGameBase::set_road(const FCoords& f,
                               uint8_t const direction,
-                              RoadSegment const roadtype) {
+                              RoadSegment const roadtype) const {
 	const Map& m = map();
 	const Field& first_field = m[0];
 	assert(0 <= f.x);
@@ -662,8 +647,7 @@ void EditorGameBase::set_road(const FCoords& f,
 	}
 }
 
-/// This unconquers an area. This is only possible, when there is a building
-/// placed on this node.
+/// This unconquers an area.
 void EditorGameBase::unconquer_area(PlayerArea<Area<FCoords>> player_area,
                                     PlayerNumber const destroying_player) {
 	assert(0 <= player_area.x);
@@ -675,11 +659,6 @@ void EditorGameBase::unconquer_area(PlayerArea<Area<FCoords>> player_area,
 	assert(0 < player_area.player_number);
 	assert(player_area.player_number <= map().get_nrplayers());
 
-	//  Here must be a building.
-	assert(
-	   dynamic_cast<const Building&>(*map().get_immovable(player_area)).owner().player_number() ==
-	   player_area.player_number);
-
 	//  step 1: unconquer area of this building
 	do_conquer_area(player_area, false, destroying_player);
 }
@@ -700,7 +679,7 @@ void EditorGameBase::conquer_area(PlayerArea<Area<FCoords>> player_area,
 	do_conquer_area(player_area, true, 0, conquer_guarded_location);
 }
 
-void EditorGameBase::change_field_owner(const FCoords& fc, PlayerNumber const new_owner) {
+void EditorGameBase::change_field_owner(const FCoords& fc, PlayerNumber const new_owner) const {
 	const Field& first_field = map()[0];
 
 	PlayerNumber const old_owner = fc.field->get_owned_by();
