@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,7 +38,6 @@
 #include "build_info.h"
 #include "economy/economy.h"
 #include "economy/portdock.h"
-#include "editor/editorinteractive.h"
 #include "game_io/game_loader.h"
 #include "game_io/game_preload_packet.h"
 #include "io/fileread.h"
@@ -62,6 +61,7 @@
 #include "logic/player.h"
 #include "logic/playercommand.h"
 #include "logic/replay.h"
+#include "logic/replay_game_controller.h"
 #include "logic/single_player_game_controller.h"
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 #include "logic/training_wheels.h"
@@ -72,6 +72,7 @@
 #include "ui_basic/progresswindow.h"
 #include "wlapplication_options.h"
 #include "wui/interactive_player.h"
+#include "wui/interactive_spectator.h"
 
 namespace Widelands {
 
@@ -102,11 +103,11 @@ void Game::SyncWrapper::start_dump(const std::string& fname) {
 void Game::SyncWrapper::data(void const* const sync_data, size_t const size) {
 #ifdef SYNC_DEBUG
 	const Time& time = game_.get_gametime();
-	log_dbg_time(game_.get_gametime(), "[sync:%08u t=%6u]", counter_, time.get());
-	for (size_t i = 0; i < size; ++i) {
-		log_dbg_time(game_.get_gametime(), " %02x", (static_cast<uint8_t const*>(sync_data))[i]);
+	std::string logtext = format("[sync:%08u t=%6u]", counter_, time.get());
+	for (size_t i = size; i > 0; --i) {
+		logtext += format(" %02x", (static_cast<uint8_t const*>(sync_data))[i - 1]);
 	}
-	log_dbg_time(game_.get_gametime(), "\n");
+	log_dbg_time(game_.get_gametime(), "%s", logtext.c_str());
 #endif
 
 	if (dump_ != nullptr && static_cast<int32_t>(counter_ - next_diskspacecheck_) >= 0) {
@@ -138,24 +139,12 @@ void Game::SyncWrapper::data(void const* const sync_data, size_t const size) {
 
 Game::Game()
    : EditorGameBase(new LuaGameInterface(this)),
-     did_postload_addons_before_loading_(false),
      syncwrapper_(*this, synchash_),
-     ctrl_(nullptr),
-     writereplay_(true),
-     writesyncstream_(false),
-     ai_training_mode_(false),
-     auto_speed_(false),
-     state_(gs_notrunning),
      cmdqueue_(*this),
-     scenario_difficulty_(kScenarioDifficultyNotSet),
-     diplomacy_allowed_(true),
      /** TRANSLATORS: Win condition for this game has not been set. */
-     win_condition_displayname_(_("Not set")),
-#if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
-     training_wheels_wanted_(false),
-#endif
-     replay_(false) {
+     win_condition_displayname_(_("Not set")) {
 	Economy::initialize_serial();
+	DetectedPortSpace::initialize_serial();
 }
 
 Game::~Game() {  // NOLINT
@@ -177,6 +166,9 @@ void Game::sync_reset() {
 InteractivePlayer* Game::get_ipl() {
 	return dynamic_cast<InteractivePlayer*>(get_ibase());
 }
+const InteractivePlayer* Game::get_ipl() const {
+	return dynamic_cast<const InteractivePlayer*>(get_ibase());
+}
 
 void Game::set_game_controller(std::shared_ptr<GameController> c) {
 	ctrl_ = c;
@@ -228,34 +220,6 @@ void Game::postload_addons_before_loading() {
 	postload_addons();
 }
 
-// TODO(Nordfriese): Needed for v1.0 savegame compatibility, remove after v1.1
-void Game::check_legacy_addons_desync_magic() {
-	bool needed = false;
-	for (const auto& a : enabled_addons()) {
-		if (a->category == AddOns::AddOnCategory::kWorld ||
-		    a->category == AddOns::AddOnCategory::kTribes) {
-			needed = true;
-			break;
-		}
-	}
-	if (!needed) {
-		postload_addons();
-		return;
-	}
-
-	did_postload_addons_before_loading_ = true;
-	did_postload_addons_ = false;
-
-	delete_world_and_tribes();
-	descriptions();
-
-	// Cyclic dependency. Can and must be gotten rid of when fixing the above TO-DO.
-	EditorInteractive::load_world_units(nullptr, *this);
-	load_all_tribes();
-
-	postload_addons();
-}
-
 bool Game::run_splayer_scenario_direct(const std::list<std::string>& list_of_scenarios,
                                        const std::string& script_to_run) {
 	full_cleanup();
@@ -277,7 +241,7 @@ bool Game::run_splayer_scenario_direct(const std::list<std::string>& list_of_sce
 	maploader->preload_map(true, &enabled_addons());
 
 	create_loader_ui({"general_game"}, false /* no game tips in scenarios */,
-	                 map().get_background_theme(), map().get_background());
+	                 map().get_background_theme(), map().get_background(), true);
 
 	Notifications::publish(UI::NoteLoadingMessage(_("Preloading map…")));
 
@@ -307,8 +271,7 @@ bool Game::run_splayer_scenario_direct(const std::list<std::string>& list_of_sce
 
 	set_game_controller(std::make_shared<SinglePlayerGameController>(*this, true, 1));
 	try {
-		bool const result =
-		   run(StartGameType::kSinglePlayerScenario, script_to_run, false, "single_player");
+		bool const result = run(StartGameType::kSinglePlayerScenario, script_to_run, "single_player");
 		ctrl_.reset();
 		return result;
 	} catch (...) {
@@ -380,7 +343,7 @@ void Game::init_newgame(const GameSettings& settings) {
 	// Check for win_conditions
 	if (!settings.scenario) {
 		Notifications::publish(UI::NoteLoadingMessage(_("Initializing game…")));
-		if (settings.peaceful) {
+		if ((settings.flags & GameSettings::Flags::kPeaceful) != 0) {
 			for (uint32_t i = 1; i < settings.players.size(); ++i) {
 				if (Player* p1 = get_player(i)) {
 					for (uint32_t j = i + 1; j <= settings.players.size(); ++j) {
@@ -392,7 +355,7 @@ void Game::init_newgame(const GameSettings& settings) {
 				}
 			}
 		}
-		if (settings.custom_starting_positions) {
+		if ((settings.flags & GameSettings::Flags::kCustomStartingPositions) != 0) {
 			iterate_players_existing(p, map().get_nrplayers(), *this, pl) {
 				if (settings.get_tribeinfo(pl->tribe().name())
 				       .initializations[pl->initialization_index()]
@@ -402,6 +365,22 @@ void Game::init_newgame(const GameSettings& settings) {
 			}
 		}
 
+		if ((settings.flags & GameSettings::Flags::kFogless) != 0) {
+			Coords c;
+			iterate_players_existing(p, map().get_nrplayers(), *this, pl) {
+				c.x = 0;
+				for (; c.x < map().get_width(); ++c.x) {
+					c.y = 0;
+					for (; c.y < map().get_height(); ++c.y) {
+						pl->hide_or_reveal_field(c, HideOrRevealFieldMode::kReveal);
+					}
+				}
+			}
+		}
+
+		diplomacy_allowed_ = ((settings.flags & GameSettings::Flags::kForbidDiplomacy) == 0);
+		naval_warfare_allowed_ = ((settings.flags & GameSettings::Flags::kAllowNavalWarfare) != 0);
+		win_condition_duration_ = settings.win_condition_duration;
 		std::unique_ptr<LuaTable> table(lua().run_script(settings.win_condition_script));
 		table->do_not_warn_about_unaccessed_keys();
 		win_condition_displayname_ = table->get_string("name");
@@ -433,6 +412,7 @@ void Game::init_savegame(const GameSettings& settings) {
 		gl.preload_game(gpdp);
 
 		win_condition_displayname_ = gpdp.get_win_condition();
+		win_condition_duration_ = gpdp.get_win_condition_duration();
 
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 		training_wheels_wanted_ =
@@ -478,10 +458,11 @@ bool Game::run_load_game(const std::string& filename, const std::string& script_
 		gl.preload_game(gpdp);
 
 		create_loader_ui({"general_game", "singleplayer"}, true, gpdp.get_background_theme(),
-		                 gpdp.get_background());
+		                 gpdp.get_background(), true);
 		Notifications::publish(UI::NoteLoadingMessage(_("Preloading map…")));
 
 		win_condition_displayname_ = gpdp.get_win_condition();
+		win_condition_duration_ = gpdp.get_win_condition_duration();
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 		training_wheels_wanted_ =
 		   gpdp.get_training_wheels_wanted() && get_config_bool("training_wheels", true);
@@ -511,7 +492,7 @@ bool Game::run_load_game(const std::string& filename, const std::string& script_
 
 	set_game_controller(std::make_shared<SinglePlayerGameController>(*this, true, player_nr));
 	try {
-		bool const result = run(StartGameType::kSaveGame, script_to_run, false, "single_player");
+		bool const result = run(StartGameType::kSaveGame, script_to_run, "single_player");
 		ctrl_.reset();
 		return result;
 	} catch (...) {
@@ -588,11 +569,9 @@ void Game::postload() {
  */
 bool Game::run(StartGameType const start_game_type,
                const std::string& script_to_run,
-               bool replay,
                const std::string& prefix_for_replays) {
 	assert(has_loader_ui());
 
-	replay_ = replay;
 	postload();
 
 	InteractivePlayer* ipl = get_ipl();
@@ -776,6 +755,7 @@ bool Game::run(StartGameType const start_game_type,
 	g_sh->change_music(Songset::kMenu, 1000);
 
 	cleanup_objects();
+	delete_pending_player_commands();
 	set_ibase(nullptr);
 
 	state_ = gs_notrunning;
@@ -784,9 +764,14 @@ bool Game::run(StartGameType const start_game_type,
 		return true;
 	}
 
-	create_loader_ui({"general_game"}, false, map().get_background_theme(), map().get_background());
 	const std::string load = next_game_to_load_;  // Pass-by-reference does have its disadvantages…
+
+	if (FileSystem::filename_ext(load) == kReplayExtension) {
+		return run_replay(load, script_to_run);
+	}
 	if (FileSystem::filename_ext(load) == kSavegameExtension) {
+		create_loader_ui(
+		   {"general_game"}, false, map().get_background_theme(), map().get_background(), true);
 		return run_load_game(load, script_to_run);
 	}
 
@@ -801,6 +786,21 @@ bool Game::run(StartGameType const start_game_type,
 	return run_splayer_scenario_direct(list, script_to_run);
 }
 
+bool Game::run_replay(const std::string& filename, const std::string& script_to_run) {
+	full_cleanup();
+	replay_filename_ = filename;
+
+	create_loader_ui(
+	   {"general_game"}, false, map().get_background_theme(), map().get_background(), true);
+	set_ibase(new InteractiveSpectator(*this, get_config_section()));
+
+	set_write_replay(false);
+	set_game_controller(std::make_shared<ReplayGameController>(*this));
+	save_handler().set_allow_saving(false);
+
+	return run(Widelands::Game::StartGameType::kSaveGame, script_to_run, "replay");
+}
+
 void Game::set_next_game_to_load(const std::string& file) {
 	next_game_to_load_ = file;
 }
@@ -860,10 +860,12 @@ void Game::cleanup_for_load() {
 
 	EditorGameBase::cleanup_for_load();
 
+	delete_pending_player_commands();
 	cmdqueue().flush();
 
 	pending_diplomacy_actions_.clear();
 	diplomacy_allowed_ = true;
+	naval_warfare_allowed_ = false;
 
 	// Statistics
 	general_stats_.clear();
@@ -872,19 +874,31 @@ void Game::cleanup_for_load() {
 void Game::full_cleanup() {
 	EditorGameBase::full_cleanup();
 
+	delete_pending_player_commands();
 	did_postload_addons_before_loading_ = false;
 	ctrl_.reset();
 	replaywriter_.reset();
 	writereplay_ = true;  // Not using `set_write_replay()` on purpose.
 	next_game_to_load_.clear();
 	list_of_scenarios_.clear();
+	replay_filename_.clear();
+	forester_cache_.clear();
 	Economy::initialize_serial();
+	DetectedPortSpace::initialize_serial();
 
 	if (has_loader_ui()) {
 		remove_loader_ui();
 	}
 }
 
+void Game::delete_pending_player_commands() {
+	MutexLock m(MutexLock::ID::kCommands);
+	for (PlayerCommand* pc : pending_player_commands_) {
+		delete pc;
+	}
+	pending_player_commands_.clear();
+}
+
 /**
  * Game logic code may write to the synchronization
  * token stream. All written data will be hashed and can be used to
@@ -1040,10 +1054,14 @@ void Game::send_player_start_stop_building(Building& building) {
 	   new CmdStartStopBuilding(get_gametime(), building.owner().player_number(), building));
 }
 
-void Game::send_player_militarysite_set_soldier_preference(Building& building,
-                                                           SoldierPreference my_preference) {
-	send_player_command(new CmdMilitarySiteSetSoldierPreference(
-	   get_gametime(), building.owner().player_number(), building, my_preference));
+void Game::send_player_toggle_infinite_production(Building& building) {
+	send_player_command(
+	   new CmdToggleInfiniteProduction(get_gametime(), building.owner().player_number(), building));
+}
+
+void Game::send_player_set_soldier_preference(MapObject& mo, SoldierPreference my_preference) {
+	send_player_command(
+	   new CmdSetSoldierPreference(get_gametime(), mo.owner().player_number(), mo, my_preference));
 }
 
 void Game::send_player_start_or_cancel_expedition(Building& building) {
@@ -1089,7 +1107,7 @@ void Game::send_player_change_training_options(TrainingSite& ts,
 	   new CmdChangeTrainingOptions(get_gametime(), ts.owner().player_number(), ts, attr, val));
 }
 
-void Game::send_player_drop_soldier(Building& b, int32_t const ser) {
+void Game::send_player_drop_soldier(MapObject& b, int32_t const ser) {
 	assert(ser != -1);
 	send_player_command(new CmdDropSoldier(get_gametime(), b.owner().player_number(), b, ser));
 }
@@ -1127,11 +1145,33 @@ void Game::send_player_ship_explore_island(const Ship& ship, IslandExploreDirect
 	   get_gametime(), ship.get_owner()->player_number(), ship.serial(), direction));
 }
 
+void Game::send_player_ship_set_destination(const Ship& ship, const MapObject* dest) {
+	send_player_command(new CmdShipSetDestination(get_gametime(), ship.get_owner()->player_number(),
+	                                              ship.serial(),
+	                                              dest == nullptr ? 0 : dest->serial()));
+}
+void Game::send_player_ship_set_destination(const Ship& ship, const DetectedPortSpace& dest) {
+	send_player_command(new CmdShipSetDestination(
+	   get_gametime(), ship.get_owner()->player_number(), ship.serial(), dest));
+}
+
 void Game::send_player_sink_ship(const Ship& ship) {
 	send_player_command(
 	   new CmdShipSink(get_gametime(), ship.get_owner()->player_number(), ship.serial()));
 }
 
+void Game::send_player_refit_ship(const Ship& ship, const ShipType t) {
+	send_player_command(
+	   new CmdShipRefit(get_gametime(), ship.get_owner()->player_number(), ship.serial(), t));
+}
+
+void Game::send_player_warship_command(const Ship& ship,
+                                       const WarshipCommand cmd,
+                                       const std::vector<uint32_t>& parameters) {
+	send_player_command(new CmdWarshipCommand(
+	   get_gametime(), ship.get_owner()->player_number(), ship.serial(), cmd, parameters));
+}
+
 void Game::send_player_cancel_expedition_ship(const Ship& ship) {
 	send_player_command(new CmdShipCancelExpedition(
 	   get_gametime(), ship.get_owner()->player_number(), ship.serial()));
@@ -1173,6 +1213,19 @@ void Game::send_player_mark_object_for_removal(PlayerNumber p, Immovable& mo, bo
 	send_player_command(new CmdMarkMapObjectForRemoval(get_gametime(), p, mo, mark));
 }
 
+void Game::send_player_pinned_note(
+   PlayerNumber p, Coords pos, const std::string& text, const RGBColor& rgb, bool del) {
+	send_player_command(new CmdPinnedNote(get_gametime(), p, text, pos, rgb, del));
+}
+
+void Game::send_player_ship_port_name(PlayerNumber p, Serial s, const std::string& name) {
+	send_player_command(new CmdShipPortName(get_gametime(), p, s, name));
+}
+
+void Game::send_player_fleet_targets(PlayerNumber p, Serial i, Quantity q) {
+	send_player_command(new CmdFleetTargets(get_gametime(), p, i, q));
+}
+
 int Game::propose_trade(const Trade& trade) {
 	// TODO(sirver,trading): Check if a trade is possible (i.e. if there is a
 	// path between the two markets);
@@ -1256,6 +1309,12 @@ const std::string& Game::get_win_condition_displayname() const {
 void Game::set_win_condition_displayname(const std::string& name) {
 	win_condition_displayname_ = name;
 }
+int32_t Game::get_win_condition_duration() const {
+	return win_condition_duration_;
+}
+void Game::set_win_condition_duration(int32_t d) {
+	win_condition_duration_ = d;
+}
 
 /**
  * Sample global statistics for the game.
@@ -1265,13 +1324,16 @@ void Game::sample_statistics() {
 	PlayerNumber const nr_plrs = map().get_nrplayers();
 	std::vector<uint32_t> land_size;
 	std::vector<uint32_t> nr_buildings;
+	std::vector<uint32_t> nr_ships;
+	std::vector<uint32_t> nr_naval_victories;
+	std::vector<uint32_t> nr_naval_losses;
 	std::vector<uint32_t> nr_casualties;
 	std::vector<uint32_t> nr_kills;
 	std::vector<uint32_t> nr_msites_lost;
 	std::vector<uint32_t> nr_msites_defeated;
 	std::vector<uint32_t> nr_civil_blds_lost;
 	std::vector<uint32_t> nr_civil_blds_defeated;
-	std::vector<uint32_t> miltary_strength;
+	std::vector<float> miltary_strength;
 	std::vector<uint32_t> nr_workers;
 	std::vector<uint32_t> nr_wares;
 	std::vector<uint32_t> productivity;
@@ -1279,6 +1341,9 @@ void Game::sample_statistics() {
 	std::vector<uint32_t> custom_statistic;
 	land_size.resize(nr_plrs);
 	nr_buildings.resize(nr_plrs);
+	nr_ships.resize(nr_plrs);
+	nr_naval_victories.resize(nr_plrs);
+	nr_naval_losses.resize(nr_plrs);
 	nr_casualties.resize(nr_plrs);
 	nr_kills.resize(nr_plrs);
 	nr_msites_lost.resize(nr_plrs);
@@ -1301,36 +1366,46 @@ void Game::sample_statistics() {
 		}
 
 		// Get the immovable
-		if (upcast(Building, building, fc.field->get_immovable())) {
+		if (fc.field->get_immovable() != nullptr &&
+		    fc.field->get_immovable()->descr().type() >= MapObjectType::BUILDING) {
+			upcast(Building, building, fc.field->get_immovable());
 			if (building->get_position() == fc) {  // only count main location
 				uint8_t const player_index = building->owner().player_number() - 1;
 				++nr_buildings[player_index];
 
 				//  If it is a productionsite, add its productivity.
-				if (upcast(ProductionSite, productionsite, building)) {
+				if (building->descr().type() == MapObjectType::PRODUCTIONSITE ||
+				    building->descr().type() == MapObjectType::TRAININGSITE) {
 					++nr_production_sites[player_index];
-					productivity[player_index] += productionsite->get_statistics_percent();
+					productivity[player_index] +=
+					   dynamic_cast<const ProductionSite&>(*building).get_statistics_percent();
 				}
 			}
 		}
 
 		// Now, walk the bobs
 		for (Bob const* b = fc.field->get_first_bob(); b != nullptr; b = b->get_next_bob()) {
-			if (upcast(Soldier const, s, b)) {
-				miltary_strength[s->owner().player_number() - 1] +=
-				   s->get_level(TrainingAttribute::kTotal) + 1;  //  So that level 0 also counts.
+			if (b->descr().type() != MapObjectType::SOLDIER) {
+				continue;
 			}
+
+			constexpr int kHeroValue = 19;
+			upcast(Soldier const, soldier, b);
+			assert(soldier != nullptr);
+			const float total_level = soldier->get_level(TrainingAttribute::kTotal);
+			const float max_level = std::max<float>(soldier->descr().get_max_total_level(), 1);
+			miltary_strength[soldier->owner().player_number() - 1] +=
+			   (total_level * total_level * kHeroValue) / (max_level * max_level) + 1.f;
 		}
 	}
 
 	//  Number of workers / wares / casualties / kills.
 	iterate_players_existing(p, nr_plrs, *this, plr) {
+		const TribeDescr& tribe = plr->tribe();
 		uint32_t wostock = 0;
 		uint32_t wastock = 0;
 
 		for (const auto& economy : plr->economies()) {
-			const TribeDescr& tribe = plr->tribe();
-
 			switch (economy.second->type()) {
 			case wwWARE:
 				for (const DescriptionIndex& ware_index : tribe.wares()) {
@@ -1339,15 +1414,23 @@ void Game::sample_statistics() {
 				break;
 			case wwWORKER:
 				for (const DescriptionIndex& worker_index : tribe.workers()) {
-					if (tribe.get_worker_descr(worker_index)->type() != MapObjectType::CARRIER) {
-						wostock += economy.second->stock_ware_or_worker(worker_index);
+					if (worker_index == tribe.soldier()) {
+						continue;  // Ignore soldiers.
 					}
+					const WorkerDescr& d = *tribe.get_worker_descr(worker_index);
+					if (d.is_buildable() && d.buildcost().empty()) {
+						continue;  // Don't count free workers.
+					}
+					wostock += economy.second->stock_ware_or_worker(worker_index);
 				}
 				break;
 			}
 		}
 		nr_wares[p - 1] = wastock;
 		nr_workers[p - 1] = wostock;
+		nr_ships[p - 1] = plr->ships().size();
+		nr_naval_victories[p - 1] = plr->naval_victories();
+		nr_naval_losses[p - 1] = plr->naval_losses();
 		nr_casualties[p - 1] = plr->casualties();
 		nr_kills[p - 1] = plr->kills();
 		nr_msites_lost[p - 1] = plr->msites_lost();
@@ -1381,13 +1464,16 @@ void Game::sample_statistics() {
 	for (uint32_t i = 0; i < map().get_nrplayers(); ++i) {
 		general_stats_[i].land_size.push_back(land_size[i]);
 		general_stats_[i].nr_buildings.push_back(nr_buildings[i]);
+		general_stats_[i].nr_ships.push_back(nr_ships[i]);
+		general_stats_[i].nr_naval_victories.push_back(nr_naval_victories[i]);
+		general_stats_[i].nr_naval_losses.push_back(nr_naval_losses[i]);
 		general_stats_[i].nr_casualties.push_back(nr_casualties[i]);
 		general_stats_[i].nr_kills.push_back(nr_kills[i]);
 		general_stats_[i].nr_msites_lost.push_back(nr_msites_lost[i]);
 		general_stats_[i].nr_msites_defeated.push_back(nr_msites_defeated[i]);
 		general_stats_[i].nr_civil_blds_lost.push_back(nr_civil_blds_lost[i]);
 		general_stats_[i].nr_civil_blds_defeated.push_back(nr_civil_blds_defeated[i]);
-		general_stats_[i].miltary_strength.push_back(miltary_strength[i]);
+		general_stats_[i].miltary_strength.push_back(lroundf(miltary_strength[i]));
 		general_stats_[i].nr_workers.push_back(nr_workers[i]);
 		general_stats_[i].nr_wares.push_back(nr_wares[i]);
 		general_stats_[i].productivity.push_back(productivity[i]);
@@ -1403,8 +1489,9 @@ void Game::sample_statistics() {
  * Read statistics data from a file.
  *
  * \param fr file to read from
+ * \param packet_version from GamePlayerInfoPacket in game_io/game_player_info_packet.cc
  */
-void Game::read_statistics(FileRead& fr) {
+void Game::read_statistics(FileRead& fr, uint16_t packet_version) {
 	fr.unsigned_32();  // used to be last stats update time
 
 	// Read general statistics
@@ -1418,6 +1505,9 @@ void Game::read_statistics(FileRead& fr) {
 		general_stats_[p - 1].nr_buildings.resize(entries);
 		general_stats_[p - 1].nr_wares.resize(entries);
 		general_stats_[p - 1].productivity.resize(entries);
+		general_stats_[p - 1].nr_ships.resize(entries);
+		general_stats_[p - 1].nr_naval_victories.resize(entries);
+		general_stats_[p - 1].nr_naval_losses.resize(entries);
 		general_stats_[p - 1].nr_casualties.resize(entries);
 		general_stats_[p - 1].nr_kills.resize(entries);
 		general_stats_[p - 1].nr_msites_lost.resize(entries);
@@ -1443,6 +1533,10 @@ void Game::read_statistics(FileRead& fr) {
 		general_stats_[p - 1].nr_civil_blds_defeated[j] = fr.unsigned_32();
 		general_stats_[p - 1].miltary_strength[j] = fr.unsigned_32();
 		general_stats_[p - 1].custom_statistic[j] = fr.unsigned_32();
+		// TODO(Nordfriese): Savegame compatibility v1.1
+		general_stats_[p - 1].nr_ships[j] = packet_version >= 32 ? fr.unsigned_32() : 0;
+		general_stats_[p - 1].nr_naval_victories[j] = packet_version >= 32 ? fr.unsigned_32() : 0;
+		general_stats_[p - 1].nr_naval_losses[j] = packet_version >= 32 ? fr.unsigned_32() : 0;
 	}
 }
 
@@ -1478,6 +1572,9 @@ void Game::write_statistics(FileWrite& fw) {
 		fw.unsigned_32(general_stats_[p - 1].nr_civil_blds_defeated[j]);
 		fw.unsigned_32(general_stats_[p - 1].miltary_strength[j]);
 		fw.unsigned_32(general_stats_[p - 1].custom_statistic[j]);
+		fw.unsigned_32(general_stats_[p - 1].nr_ships[j]);
+		fw.unsigned_32(general_stats_[p - 1].nr_naval_victories[j]);
+		fw.unsigned_32(general_stats_[p - 1].nr_naval_losses[j]);
 	}
 }
 }  // namespace Widelands
