@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,7 +26,9 @@
 #include "economy/flag_job.h"
 #include "io/streamwrite.h"
 #include "logic/cmd_queue.h"
+#include "logic/detected_port_space.h"
 #include "logic/editor_game_base.h"
+#include "logic/map_objects/tribes/shipstates.h"
 #include "logic/save_handler.h"
 #include "logic/trade_agreement.h"
 #include "scripting/logic.h"
@@ -50,7 +52,6 @@ struct PlayerImmovable;
 enum class IslandExploreDirection;
 class PortDock;
 enum class ScoutingDirection;
-enum class SoldierPreference : uint8_t;
 struct Ship;
 class TrainingSite;
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
@@ -138,6 +139,9 @@ public:
 		std::vector<uint32_t> nr_civil_blds_lost;
 		std::vector<uint32_t> nr_civil_blds_defeated;
 		std::vector<uint32_t> miltary_strength;
+		std::vector<uint32_t> nr_ships;
+		std::vector<uint32_t> nr_naval_victories;
+		std::vector<uint32_t> nr_naval_losses;
 
 		std::vector<uint32_t> custom_statistic;
 	};
@@ -181,10 +185,7 @@ public:
 
 	enum class StartGameType { kMap, kSinglePlayerScenario, kMultiPlayerScenario, kSaveGame };
 
-	bool run(StartGameType,
-	         const std::string& script_to_run,
-	         bool replay,
-	         const std::string& prefix_for_replays);
+	bool run(StartGameType, const std::string& script_to_run, const std::string& prefix_for_replays);
 
 	// Returns the upcasted lua interface.
 	LuaGameInterface& lua() override;
@@ -202,6 +203,8 @@ public:
 	// Returns the result of run().
 	bool run_load_game(const std::string& filename, const std::string& script_to_run);
 
+	bool run_replay(const std::string& filename, const std::string& script_to_run);
+
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 	bool acquire_training_wheel_lock(const std::string& objective);
 	void release_training_wheel_lock();
@@ -215,7 +218,6 @@ public:
 
 	void postload() override;
 	void postload_addons_before_loading();
-	void check_legacy_addons_desync_magic();
 
 	void think() override;
 
@@ -264,7 +266,7 @@ public:
 	void report_desync(int32_t playernumber);
 	Md5Checksum get_sync_hash() const;
 
-	void enqueue_command(Command* const);
+	void enqueue_command(Command*);
 
 	void send_player_command(Widelands::PlayerCommand*);
 
@@ -276,7 +278,8 @@ public:
 	void send_player_build_waterway(int32_t, Path&);
 	void send_player_flagaction(Flag&, FlagJob::Type);
 	void send_player_start_stop_building(Building&);
-	void send_player_militarysite_set_soldier_preference(Building&, SoldierPreference preference);
+	void send_player_toggle_infinite_production(Building&);
+	void send_player_set_soldier_preference(MapObject&, SoldierPreference preference);
 	void send_player_start_or_cancel_expedition(Building&);
 	void send_player_expedition_config(PortDock&, WareWorker, DescriptionIndex, bool);
 
@@ -288,7 +291,7 @@ public:
 	void send_player_set_input_max_fill(
 	   PlayerImmovable&, DescriptionIndex index, WareWorker type, uint32_t, bool is_cs = false);
 	void send_player_change_training_options(TrainingSite&, TrainingAttribute, int32_t);
-	void send_player_drop_soldier(Building&, int32_t);
+	void send_player_drop_soldier(MapObject&, int32_t);
 	void send_player_change_soldier_capacity(Building&, int32_t);
 	void send_player_enemyflagaction(const Flag&,
 	                                 PlayerNumber,
@@ -296,16 +299,27 @@ public:
 	                                 bool allow_conquer);
 	void send_player_mark_object_for_removal(PlayerNumber, Immovable&, bool);
 
-	void send_player_ship_scouting_direction(const Ship&, WalkingDir);
-	void send_player_ship_construct_port(const Ship&, Coords);
-	void send_player_ship_explore_island(const Ship&, IslandExploreDirection);
-	void send_player_sink_ship(const Ship&);
+	void send_player_ship_scouting_direction(const Ship& ship, WalkingDir direction);
+	void send_player_ship_construct_port(const Ship& ship, Coords coords);
+	void send_player_ship_explore_island(const Ship& ship, IslandExploreDirection direction);
+	void send_player_ship_set_destination(const Ship& ship, const MapObject* dest);
+	void send_player_ship_set_destination(const Ship& ship, const DetectedPortSpace& dest);
+	void send_player_sink_ship(const Ship& ship);
+	void send_player_refit_ship(const Ship& ship, ShipType t);
+	void send_player_warship_command(const Ship& ship,
+	                                 WarshipCommand cmd,
+	                                 const std::vector<uint32_t>& parameters);
 	void send_player_cancel_expedition_ship(const Ship&);
 	void send_player_propose_trade(const Trade& trade);
 	void send_player_toggle_mute(const Building&, bool all);
 	void send_player_diplomacy(PlayerNumber, DiplomacyAction, PlayerNumber);
+	void send_player_pinned_note(
+	   PlayerNumber p, Coords pos, const std::string& text, const RGBColor& rgb, bool del);
+	void send_player_ship_port_name(PlayerNumber p, Serial s, const std::string& name);
+	void send_player_fleet_targets(PlayerNumber p, Serial i, Quantity q);
 
 	InteractivePlayer* get_ipl();
+	const InteractivePlayer* get_ipl() const;
 
 	SaveHandler& save_handler() {
 		return savehandler_;
@@ -325,16 +339,22 @@ public:
 		return general_stats_;
 	}
 
-	void read_statistics(FileRead&);
-	void write_statistics(FileWrite&);
+	void read_statistics(FileRead& fr, uint16_t packet_version);
+	void write_statistics(FileWrite& fw);
 
 	void sample_statistics();
 
 	const std::string& get_win_condition_displayname() const;
 	void set_win_condition_displayname(const std::string& name);
+	int32_t get_win_condition_duration() const;
+	void set_win_condition_duration(int32_t);
 
 	bool is_replay() const {
-		return replay_;
+		return !replay_filename_.empty();
+	}
+	const std::string& replay_filename() const {
+		assert(is_replay());
+		return replay_filename_;
 	}
 
 	bool is_ai_training_mode() const {
@@ -366,13 +386,17 @@ public:
 	void cancel_trade(int trade_id);
 
 	struct PendingDiplomacyAction {
-		const PlayerNumber sender;     ///< The player who initiated the action.
-		const DiplomacyAction action;  ///< The action to perform.
-		const PlayerNumber other;      ///< The other player affected, if any.
+		PlayerNumber sender;     ///< The player who initiated the action.
+		DiplomacyAction action;  ///< The action to perform.
+		PlayerNumber other;      ///< The other player affected, if any.
 
 		PendingDiplomacyAction(PlayerNumber p1, DiplomacyAction a, PlayerNumber p2)
 		   : sender(p1), action(a), other(p2) {
 		}
+
+		inline bool operator==(const PendingDiplomacyAction& pda) const {
+			return sender == pda.sender && action == pda.action && other == pda.other;
+		}
 	};
 	const std::list<PendingDiplomacyAction>& pending_diplomacy_actions() const {
 		return pending_diplomacy_actions_;
@@ -381,28 +405,29 @@ public:
 		return pending_diplomacy_actions_;
 	}
 
-	bool diplomacy_allowed() const {
+	[[nodiscard]] bool diplomacy_allowed() const {
 		return diplomacy_allowed_;
 	}
-	void set_diplomacy_allowed(bool d) {
-		diplomacy_allowed_ = d;
+	void set_diplomacy_allowed(bool allow) {
+		diplomacy_allowed_ = allow;
+	}
+
+	[[nodiscard]] bool naval_warfare_allowed() const {
+		return naval_warfare_allowed_;
+	}
+	void set_naval_warfare_allowed(bool allow) {
+		naval_warfare_allowed_ = allow;
 	}
 
 private:
-	bool did_postload_addons_before_loading_;
+	bool did_postload_addons_before_loading_{false};
 
 	void sync_reset();
 
 	MD5Checksum<StreamWrite> synchash_;
 
 	struct SyncWrapper : public StreamWrite {
-		SyncWrapper(Game& game, StreamWrite& target)
-		   : game_(game),
-		     target_(target),
-		     counter_(0),
-		     next_diskspacecheck_(0),
-		     syncstreamsave_(false),
-		     current_excerpt_id_(0) {
+		SyncWrapper(Game& game, StreamWrite& target) : game_(game), target_(target) {
 		}
 
 		~SyncWrapper() override;
@@ -422,14 +447,14 @@ private:
 	public:
 		Game& game_;
 		StreamWrite& target_;
-		uint32_t counter_;
-		uint32_t next_diskspacecheck_;
+		uint32_t counter_{0U};
+		uint32_t next_diskspacecheck_{0U};
 		std::unique_ptr<StreamWrite> dump_;
 		std::string dumpfname_;
-		bool syncstreamsave_;
+		bool syncstreamsave_{false};
 		// Use a cyclic buffer for storing parts of the syncstream
 		// Currently used buffer
-		size_t current_excerpt_id_;
+		size_t current_excerpt_id_{0U};
 		// (Arbitrary) count of buffers
 		// Syncreports seem to be requested from the network clients every game second so this
 		// buffer should be big enough to store the last 32 seconds of the game actions leading
@@ -440,34 +465,35 @@ private:
 		std::string excerpts_buffer_[kExcerptSize];
 	} syncwrapper_;
 
+	void delete_pending_player_commands();
 	void do_send_player_command(PlayerCommand*);
 	std::shared_ptr<GameController> ctrl_;
 
 	/// Whether a replay writer should be created.
 	/// Defaults to \c true, and should only be set to \c false for playing back
 	/// replays.
-	bool writereplay_;
+	bool writereplay_{true};
 
 	/// Whether a syncsteam file should be created.
 	/// Defaults to \c false, and can be set to true for network games. The file
 	/// is written only if \ref writereplay_ is true too.
-	bool writesyncstream_;
+	bool writesyncstream_{false};
 
-	bool ai_training_mode_;
-	bool auto_speed_;
+	bool ai_training_mode_{false};
+	bool auto_speed_{false};
 
-	int32_t state_;
+	int32_t state_{gs_notrunning};
 
 	RNG rng_;
 
 	CmdQueue cmdqueue_;
-	std::list<PlayerCommand*> pending_player_commands_;
+	std::deque<PlayerCommand*> pending_player_commands_;
 
 	SaveHandler savehandler_;
 
 	std::unique_ptr<ReplayWriter> replaywriter_;
 
-	uint32_t scenario_difficulty_;
+	uint32_t scenario_difficulty_{kScenarioDifficultyNotSet};
 
 	GeneralStatsVector general_stats_;
 	int next_trade_agreement_id_ = 1;
@@ -475,17 +501,21 @@ private:
 	std::map<int, TradeAgreement> trade_agreements_;
 
 	std::list<PendingDiplomacyAction> pending_diplomacy_actions_;
-	bool diplomacy_allowed_;
+	bool diplomacy_allowed_{true};
+	bool naval_warfare_allowed_{false};
 
 	/// For save games and statistics generation
 	std::string win_condition_displayname_;
 
+	int32_t win_condition_duration_{kDefaultWinConditionDuration};
+
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 	std::unique_ptr<TrainingWheels> training_wheels_;
-	bool training_wheels_wanted_;
+	bool training_wheels_wanted_{false};
 #endif
 
-	bool replay_;
+	/** Filename of the replay represented by this game, or empty if this is not a replay. */
+	std::string replay_filename_;
 
 	std::string next_game_to_load_;
 	std::list<std::string> list_of_scenarios_;
@@ -495,6 +525,7 @@ inline Coords Game::random_location(Coords location, uint8_t radius) {
 	const uint16_t s = radius * 2 + 1;
 	location.x += logic_rand() % s - radius;
 	location.y += logic_rand() % s - radius;
+	map().normalize_coords(location);
 	return location;
 }
 }  // namespace Widelands
