@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2022 by the Widelands Development Team
+ * Copyright (C) 2008-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -109,21 +109,19 @@ struct NoteGameSettings {
 	}
 };
 
-/**
- * Holds all settings about a game that can be configured before the
- * game actually starts.
- *
- * Think of it as the Model in MVC.
- */
+/** Holds all settings about a game that can be configured before the game actually starts. */
 struct GameSettings {
-	GameSettings()
-	   : playernum(0),
-	     usernum(0),
-	     scenario(false),
-	     multiplayer(false),
-	     savegame(false),
-	     peaceful(false),
-	     custom_starting_positions(false) {
+	enum Flags : uint16_t {
+		kDefaultSettings = 0,               ///< No special options; the default.
+		kPeaceful = 1 << 0,                 ///< No fighting between players.
+		kCustomStartingPositions = 1 << 1,  ///< Players may choose their own starting positions.
+		kFogless = 1 << 2,                  ///< The entire map is always visible.
+		kForbidDiplomacy = 1 << 3,          ///< Players may not change their alliances.
+		kAllowNavalWarfare = 1 << 4,        ///< Allow naval warfare (experimental in v1.2);
+		                                    ///< TODO(Nordfriese): Enable by default in v1.3
+	};
+
+	GameSettings() {
 		std::unique_ptr<LuaInterface> lua(new LuaInterface);
 		std::unique_ptr<LuaTable> win_conditions(
 		   lua->run_script("scripting/win_conditions/init.lua"));
@@ -137,9 +135,9 @@ struct GameSettings {
 		}
 		for (const auto& pair : AddOns::g_addons) {
 			if (pair.first->category == AddOns::AddOnCategory::kWinCondition && pair.second) {
-				const std::string filename = kAddOnDir + g_fs->file_separator() +
-				                             pair.first->internal_name + g_fs->file_separator() +
-				                             "init.lua";
+				const std::string filename = kAddOnDir + LayeredFileSystem::file_separator() +
+				                             pair.first->internal_name +
+				                             LayeredFileSystem::file_separator() + "init.lua";
 				if (g_fs->file_exists(filename)) {
 					win_condition_scripts.push_back(filename);
 				} else {
@@ -151,22 +149,22 @@ struct GameSettings {
 	}
 
 	/// Returns the basic preload info for a tribe.
-	Widelands::TribeBasicInfo get_tribeinfo(const std::string& tribename) const;
+	[[nodiscard]] Widelands::TribeBasicInfo get_tribeinfo(const std::string& tribename) const;
 
 	/// Find a player number that the slot could share in. Does not guarantee that a viable slot was
 	/// actually found.
-	Widelands::PlayerNumber find_shared(PlayerSlot slot) const;
+	[[nodiscard]] Widelands::PlayerNumber find_shared(PlayerSlot slot) const;
 	/// Check if the player number returned by find_shared is usable
-	bool is_shared_usable(PlayerSlot slot, Widelands::PlayerNumber shared) const;
+	[[nodiscard]] bool is_shared_usable(PlayerSlot slot, Widelands::PlayerNumber shared) const;
 	/// Savegame slots and certain scenario slots can't be closed
-	bool uncloseable(PlayerSlot slot) const;
+	[[nodiscard]] bool uncloseable(PlayerSlot slot) const;
 	/// AIs cannot be changed in scenarios
-	bool allows_ais(PlayerSlot slot) const;
+	[[nodiscard]] bool allows_ais(PlayerSlot slot) const;
 
 	/// Number of player position of the host player
-	int16_t playernum;
+	int16_t playernum{0};
 	/// Number of users entry
-	int8_t usernum;
+	int8_t usernum{0};
 
 	/// Name of the selected map
 	std::string mapname;
@@ -176,21 +174,20 @@ struct GameSettings {
 	std::string win_condition_script;
 	/// An ordered list of all win condition script files.
 	std::vector<std::string> win_condition_scripts;
+	/// User-configured win condition time limit, in minutes.
+	int32_t win_condition_duration{Widelands::kDefaultWinConditionDuration};
 
 	/// Is map a scenario
-	bool scenario;
+	bool scenario{false};
 
 	/// Is this a multiplayer game
-	bool multiplayer;
+	bool multiplayer{false};
 
 	/// Is a savegame selected for loading?
-	bool savegame;
+	bool savegame{false};
 
-	/// Is all fighting forbidden?
-	bool peaceful;
-
-	// Whether players may pick their own starting positions
-	bool custom_starting_positions;
+	/// Additional flags to control game behaviour.
+	uint16_t flags{Flags::kDefaultSettings};
 
 	std::string map_theme;
 	std::string map_background;
@@ -214,8 +211,7 @@ struct GameSettings {
  * Think of it as a mix of Model and Controller in MVC.
  */
 struct GameSettingsProvider {
-	virtual ~GameSettingsProvider() {
-	}
+	virtual ~GameSettingsProvider() = default;
 
 	virtual const GameSettings& settings() = 0;
 
@@ -236,12 +232,11 @@ struct GameSettingsProvider {
 	                     uint32_t maxplayers,
 	                     bool savegame = false) = 0;
 	virtual void set_player_state(uint8_t number, PlayerSettings::State) = 0;
-	virtual void set_player_ai(uint8_t number, const std::string&, bool const random_ai = false) = 0;
+	virtual void set_player_ai(uint8_t number, const std::string&, bool random_ai = false) = 0;
 	// Multiplayer no longer toggles per button
 	virtual void next_player_state(uint8_t /* number */) {
 	}
-	virtual void
-	set_player_tribe(uint8_t number, const std::string&, bool const random_tribe = false) = 0;
+	virtual void set_player_tribe(uint8_t number, const std::string&, bool random_tribe = false) = 0;
 	virtual void set_player_init(uint8_t number, uint8_t index) = 0;
 	virtual void set_player_name(uint8_t number, const std::string&) = 0;
 	virtual void set_player(uint8_t number, const PlayerSettings&) = 0;
@@ -252,21 +247,21 @@ struct GameSettingsProvider {
 	virtual void set_player_shared(PlayerSlot number, Widelands::PlayerNumber shared) = 0;
 	virtual void set_win_condition_script(const std::string& wc) = 0;
 	virtual std::string get_win_condition_script() = 0;
+	virtual void set_win_condition_duration(int32_t duration) = 0;
+	virtual int32_t get_win_condition_duration() = 0;
 
-	virtual void set_peaceful_mode(bool peace) = 0;
-	virtual bool is_peaceful_mode() = 0;
-
-	virtual void set_custom_starting_positions(bool) = 0;
-	virtual bool get_custom_starting_positions() = 0;
+	virtual void set_flag(GameSettings::Flags flag, bool state) = 0;
+	virtual bool get_flag(GameSettings::Flags flag) = 0;
 
 	bool has_players_tribe() {
 		return UserSettings::highest_playernum() >= settings().playernum;
 	}
 	// For retrieving tips texts
-	struct NoTribe {};
+	struct NoTribe : public std::exception {};
 	const std::string& get_players_tribe() {
-		if (!has_players_tribe())
+		if (!has_players_tribe()) {
 			throw NoTribe();
+		}
 		return settings().players[settings().playernum].tribe;
 	}
 };
