@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
 
 #include "logic/map.h"
 
+#include <cstddef>
 #include <cstdlib>
 #include <memory>
 
@@ -132,15 +133,7 @@ Map IMPLEMENTATION
  * This really identifies a map like it is in the game
  */
 
-Map::Map()
-   : nrplayers_(0),
-     scenario_types_(NO_SCENARIO),
-     width_(0),
-     height_(0),
-     localize_author_(false),
-     pathfieldmgr_(new PathfieldManager),
-     allows_seafaring_(false),
-     waterway_max_length_(0) {
+Map::Map() : pathfieldmgr_(new PathfieldManager) {
 }
 
 Map::~Map() {
@@ -554,7 +547,7 @@ void Map::set_origin(const Coords& new_origin) {
 	assert(0 <= new_origin.y);
 	assert(new_origin.y < height_);
 
-	const size_t field_size = width_ * height_;
+	const size_t field_size = static_cast<size_t>(width_) * height_;
 
 	for (uint8_t i = get_nrplayers(); i != 0u;) {
 		starting_pos_[--i].reorigin(new_origin, extent());
@@ -652,9 +645,9 @@ void Map::resize(EditorGameBase& egbase, const Coords split, const int32_t w, co
 
 	// Generate the new fields. Does not modify the actual map yet.
 
-	std::unique_ptr<Field[]> new_fields(new Field[w * h]());
-	std::unique_ptr<bool[]> was_preserved(new bool[width_ * height_]());
-	std::unique_ptr<bool[]> was_created(new bool[w * h]());
+	std::unique_ptr<Field[]> new_fields(new Field[static_cast<uint64_t>(w) * h]());
+	std::unique_ptr<bool[]> was_preserved(new bool[static_cast<uint64_t>(width_) * height_]());
+	std::unique_ptr<bool[]> was_created(new bool[static_cast<uint64_t>(w) * h]());
 
 	for (int16_t x = 0; x < width_; ++x) {
 		for (int16_t y = 0; y < height_; ++y) {
@@ -795,7 +788,7 @@ void Map::set_to(EditorGameBase& egbase, ResizeHistory rh) {
 	// Reset the fields to blank
 	width_ = rh.size.w;
 	height_ = rh.size.h;
-	fields_.reset(new Field[width_ * height_]);
+	fields_.reset(new Field[static_cast<uint64_t>(width_) * height_]);
 	egbase.allocate_player_maps();
 
 	// Overwrite starting locations and port spaces
@@ -967,6 +960,57 @@ void Map::set_nrplayers(PlayerNumber const nrplayers) {
 	scenario_tribes_.resize(nrplayers);
 
 	nrplayers_ = nrplayers;  // in case the number players got less
+
+	sanitize_suggested_teams();
+}
+
+/*
+===============
+Remove invalid and duplicate players from suggested teams and remove empty teams and lineups.
+===============
+*/
+void Map::sanitize_suggested_teams() {
+	for (size_t lineup_index = 0; lineup_index < suggested_teams_.size();) {
+		SuggestedTeamLineup& stl = suggested_teams_.at(lineup_index);
+		std::set<PlayerNumber> used;
+		for (size_t team_index = 0; team_index < stl.size();) {
+			SuggestedTeam& team = stl.at(team_index);
+			for (size_t player_index = 0; player_index < team.size();) {
+				PlayerNumber pn = team.at(player_index);  // Zero-based
+				if (pn >= nrplayers_ || used.count(pn) > 0) {
+					team.erase(team.begin() + player_index);
+				} else {
+					used.insert(pn);
+					++player_index;
+				}
+			}
+			if (team.empty()) {
+				stl.erase(stl.begin() + team_index);
+			} else {
+				++team_index;
+			}
+		}
+		if (stl.empty()) {
+			suggested_teams_.erase(suggested_teams_.begin() + lineup_index);
+		} else {
+			++lineup_index;
+		}
+	}
+
+	for (auto it = suggested_teams_.begin(); it != suggested_teams_.end();) {
+		bool remove = false;
+		for (auto other = suggested_teams_.begin(); other != it; ++other) {
+			if (*other == *it) {
+				remove = true;
+				break;
+			}
+		}
+		if (remove) {
+			it = suggested_teams_.erase(it);
+		} else {
+			++it;
+		}
+	}
 }
 
 /*
@@ -1049,9 +1093,10 @@ void Map::find_reachable(const EditorGameBase& egbase,
                          const Area<FCoords>& area,
                          const CheckStep& checkstep,
                          functorT& functor) const {
-	std::vector<Coords> queue;
+	MutexLock m(MutexLock::ID::kPathfinding);
 	std::shared_ptr<Pathfields> pathfields = pathfieldmgr_->allocate();
 
+	std::vector<Coords> queue;
 	queue.push_back(area);
 
 	while (!queue.empty()) {
@@ -1107,7 +1152,7 @@ The actual logic behind find_bobs and find_reachable_bobs.
 */
 struct FindBobsCallback {
 	FindBobsCallback(std::vector<Bob*>* const list, const FindBob& functor)
-	   : list_(list), functor_(functor), found_(0) {
+	   : list_(list), functor_(functor) {
 	}
 
 	void operator()(const EditorGameBase& /* egbase */, const FCoords& cur) {
@@ -1127,7 +1172,7 @@ struct FindBobsCallback {
 
 	std::vector<Bob*>* list_;
 	const FindBob& functor_;
-	uint32_t found_;
+	uint32_t found_{0U};
 };
 
 /*
@@ -1184,7 +1229,7 @@ The actual logic behind find_immovables and find_reachable_immovables.
 */
 struct FindImmovablesCallback {
 	FindImmovablesCallback(std::vector<ImmovableFound>* const list, const FindImmovable& functor)
-	   : list_(list), functor_(functor), found_(0) {
+	   : list_(list), functor_(functor) {
 	}
 
 	void operator()(const EditorGameBase& /* egbase */, const FCoords& cur) {
@@ -1208,7 +1253,7 @@ struct FindImmovablesCallback {
 
 	std::vector<ImmovableFound>* list_;
 	const FindImmovable& functor_;
-	uint32_t found_;
+	uint32_t found_{0U};
 };
 
 /*
@@ -1290,7 +1335,7 @@ The actual logic behind find_fields and find_reachable_fields.
 */
 struct FindNodesCallback {
 	FindNodesCallback(std::vector<Coords>* const list, const FindNode& functor)
-	   : list_(list), functor_(functor), found_(0) {
+	   : list_(list), functor_(functor) {
 	}
 
 	void operator()(const EditorGameBase& egbase, const FCoords& cur) {
@@ -1304,7 +1349,7 @@ struct FindNodesCallback {
 
 	std::vector<Coords>* list_;
 	const FindNode& functor_;
-	uint32_t found_;
+	uint32_t found_{0U};
 };
 
 /*
@@ -1800,18 +1845,19 @@ bool Map::is_cycle_connected(const FCoords& start, const std::vector<WalkingDir>
 	return true;
 }
 
+static constexpr WalkingDir kPortSpaceWalkingDirs[16] = {
+   WALK_NE, WALK_NE, WALK_NE, WALK_NW, WALK_NW, WALK_W, WALK_W, WALK_W,
+   WALK_SW, WALK_SW, WALK_SW, WALK_SE, WALK_SE, WALK_E, WALK_E, WALK_E};
+
 /**
  * Returns a list of portdock fields (if any) that a port built at \p c should have.
  */
 std::vector<Coords> Map::find_portdock(const Coords& c, bool force) const {
-	static const WalkingDir cycledirs[16] = {WALK_NE, WALK_NE, WALK_NE, WALK_NW, WALK_NW, WALK_W,
-	                                         WALK_W,  WALK_W,  WALK_SW, WALK_SW, WALK_SW, WALK_SE,
-	                                         WALK_SE, WALK_E,  WALK_E,  WALK_E};
 	const FCoords start = br_n(br_n(get_fcoords(c)));
 	const Widelands::PlayerNumber owner = start.field->get_owned_by();
 	FCoords f = start;
 	std::vector<Coords> portdock;
-	for (uint32_t i = 0; i < 16; ++i) {
+	for (WalkingDir next_direction : kPortSpaceWalkingDirs) {
 		if (force) {
 			if ((f.field->maxcaps() & MOVECAPS_SWIM) != 0) {
 				return {f};
@@ -1845,14 +1891,33 @@ std::vector<Coords> Map::find_portdock(const Coords& c, bool force) const {
 			}
 		}
 
-		if (i < 15) {
-			f = get_neighbour(f, cycledirs[i]);
-		}
+		f = get_neighbour(f, next_direction);
 	}
 
 	return portdock;
 }
 
+/**
+ * Finds a port space field where a port, if built, could have the provided coords in its portdock.
+ * Returns Coords::null() if no such port space exists.
+ */
+Coords Map::find_portspace_for_dockpoint(Coords dockpoint) const {
+	if (port_spaces_.empty()) {
+		return Coords::null();
+	}
+
+	dockpoint = tl_n(tl_n(dockpoint));
+	for (WalkingDir next_direction : kPortSpaceWalkingDirs) {
+		if (is_port_space(dockpoint)) {
+			return dockpoint;
+		}
+
+		dockpoint = get_neighbour(dockpoint, get_backward_dir(next_direction));
+	}
+
+	return Coords::null();
+}
+
 bool Map::is_port_space_allowed(const EditorGameBase& egbase, const FCoords& fc) const {
 	return (get_max_nodecaps(egbase, fc) & BUILDCAPS_SIZEMASK) == BUILDCAPS_BIG &&
 	       !find_portdock(fc, false).empty();
@@ -1890,7 +1955,7 @@ bool Map::set_port_space(const EditorGameBase& egbase,
  * Calculate the (Manhattan) distance from a to b
  * a and b are expected to be normalized!
  */
-uint32_t Map::calc_distance(const Coords& a, const Coords& b) const {
+uint32_t Map::calc_distance(const Coords a, const Coords b) const {
 	uint32_t dist;
 	int32_t dy;
 
@@ -2217,6 +2282,7 @@ int32_t Map::findpath(Coords instart,
 	}
 
 	// Actual pathfinding
+	MutexLock m(MutexLock::ID::kPathfinding);
 	std::shared_ptr<Pathfields> pathfields = pathfieldmgr_->allocate();
 	Pathfield::Queue Open(type);
 	Pathfield* curpf = &pathfields->fields[start.field - fields_.get()];
@@ -2328,8 +2394,6 @@ bool Map::can_reach_by_water(const Coords& field) const {
 		return false;
 	}
 
-	FCoords neighb;
-
 	for (Direction dir = FIRST_DIRECTION; dir <= LAST_DIRECTION; ++dir) {
 		if ((get_neighbour(fc, dir).field->nodecaps() & MOVECAPS_SWIM) != 0) {
 			return true;
@@ -2364,8 +2428,9 @@ int32_t Map::change_terrain(const EditorGameBase& egbase,
 		clear_resources(f_sw_e);
 	}
 
-	Notifications::publish(
-	   NoteFieldTerrainChanged{c.node, static_cast<MapIndex>(c.node.field - &fields_[0])});
+	Notifications::publish(NoteFieldTerrainChanged{c.node,
+	                                               static_cast<MapIndex>(c.node.field - &fields_[0]),
+	                                               NoteFieldTerrainChanged::Change::kTerrain});
 
 	// Changing the terrain can affect ports, which can be up to 3 fields away.
 	constexpr int kPotentiallyAffectedNeighbors = 3;
@@ -2455,11 +2520,17 @@ uint32_t Map::set_height(const EditorGameBase& egbase, const FCoords fc, uint8_t
 	return radius;
 }
 
-uint32_t
-Map::change_height(const EditorGameBase& egbase, Area<FCoords> area, int16_t const difference) {
+uint32_t Map::change_height(const EditorGameBase& egbase,
+                            Area<FCoords> area,
+                            int16_t const difference,
+                            std::vector<bool>::const_iterator gap_iterator) {
 	{
 		MapRegion<Area<FCoords>> mr(*this, area);
 		do {
+			if (kEditorGapAffectsHeightTool && *gap_iterator++) {
+				continue;
+			}
+
 			if (difference < 0 && mr.location().field->height < static_cast<uint8_t>(-difference)) {
 				mr.location().field->height = 0;
 			} else if (static_cast<int16_t>(MAX_FIELD_HEIGHT) - difference <
@@ -2482,13 +2553,19 @@ Map::change_height(const EditorGameBase& egbase, Area<FCoords> area, int16_t con
 	return area.radius;
 }
 
-uint32_t
-Map::set_height(const EditorGameBase& egbase, Area<FCoords> area, HeightInterval height_interval) {
+uint32_t Map::set_height(const EditorGameBase& egbase,
+                         Area<FCoords> area,
+                         HeightInterval height_interval,
+                         std::vector<bool>::const_iterator gap_iterator) {
 	assert(height_interval.valid());
 	assert(height_interval.max <= MAX_FIELD_HEIGHT);
 	{
 		MapRegion<Area<FCoords>> mr(*this, area);
 		do {
+			if (kEditorGapAffectsHeightTool && *gap_iterator++) {
+				continue;
+			}
+
 			if (mr.location().field->height < height_interval.min) {
 				mr.location().field->height = height_interval.min;
 			} else if (height_interval.max < mr.location().field->height) {
@@ -2502,11 +2579,11 @@ Map::set_height(const EditorGameBase& egbase, Area<FCoords> area, HeightInterval
 		bool changed;
 		do {
 			changed = false;
-			height_interval.min = height_interval.min < MAX_FIELD_HEIGHT_DIFF ?
+			height_interval.min = height_interval.min < max_field_height_diff() ?
                                   0 :
-                                  height_interval.min - MAX_FIELD_HEIGHT_DIFF;
-			height_interval.max = height_interval.max < MAX_FIELD_HEIGHT - MAX_FIELD_HEIGHT_DIFF ?
-                                  height_interval.max + MAX_FIELD_HEIGHT_DIFF :
+                                  height_interval.min - max_field_height_diff();
+			height_interval.max = height_interval.max < MAX_FIELD_HEIGHT - max_field_height_diff() ?
+                                  height_interval.max + max_field_height_diff() :
                                   MAX_FIELD_HEIGHT;
 			do {
 				if (mr.location().field->height < height_interval.min) {
@@ -2548,14 +2625,14 @@ void Map::check_neighbour_heights(FCoords coords, uint32_t& area) {
 	for (uint8_t i = 0; i < 6; ++i) {
 		Field& f = *n[i].field;
 		const int32_t diff = height - f.get_height();
-		if (diff > MAX_FIELD_HEIGHT_DIFF) {
+		if (diff > max_field_height_diff()) {
 			++area;
-			f.set_height(height - MAX_FIELD_HEIGHT_DIFF);
+			f.set_height(height - max_field_height_diff());
 			check[i] = true;
 		}
-		if (diff < -MAX_FIELD_HEIGHT_DIFF) {
+		if (diff < -max_field_height_diff()) {
 			++area;
-			f.set_height(height + MAX_FIELD_HEIGHT_DIFF);
+			f.set_height(height + max_field_height_diff());
 			check[i] = true;
 		}
 	}
