@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +23,7 @@
 #include <memory>
 
 #include "base/log.h"
-#include "base/multithreading.h"
+#include "base/mutex.h"
 #include "base/wexception.h"
 #include "graphic/animation/animation_manager.h"
 #include "graphic/font_handler.h"
@@ -262,16 +262,8 @@ MapObjectDescr::MapObjectDescr(const MapObjectType init_type,
                                const std::string& init_descname,
                                const LuaTable& table)
    : MapObjectDescr(init_type, init_name, init_descname) {
-	if (table.has_key("helptext_script")) {
-		// TODO(GunChleoc): Compatibility - remove after v1.0
-		log_warn("Helptexts script for %s is obsolete - please move strings to "
-		         "tribes/initializations/<tribename>/units.lua",
-		         name().c_str());
-	}
 
 	bool has_animations = false;
-	// TODO(GunChleoc): When all animations have been converted, require that animation_directory is
-	// not empty if the map object has animations.
 	const std::string animation_directory(
 	   table.has_key("animation_directory") ? table.get_string("animation_directory") : "");
 	if (table.has_key("animations")) {
@@ -297,11 +289,6 @@ MapObjectDescr::MapObjectDescr(const MapObjectType init_type,
 		}
 	}
 	check_representative_image();
-
-	// TODO(GunChleoc): Compatibility, remove after v1.0
-	if (table.has_key("attributes")) {
-		throw GameDataError("Attributes need to be defined in 'register.lua' now");
-	}
 }
 MapObjectDescr::~MapObjectDescr() {
 	anims_.clear();
@@ -451,12 +438,8 @@ const std::string& MapObjectDescr::icon_filename() const {
  * Search for the attribute in the attribute list
  */
 bool MapObjectDescr::has_attribute(AttributeIndex attr) const {
-	for (const uint32_t& attrib : attribute_ids_) {
-		if (attrib == attr) {
-			return true;
-		}
-	}
-	return false;
+	return std::any_of(attribute_ids_.begin(), attribute_ids_.end(),
+	                   [attr](const uint32_t atrribute) { return atrribute == attr; });
 }
 
 /**
@@ -530,8 +513,7 @@ MapObject IMPLEMENTATION
 /**
  * Zero-initialize a map object
  */
-MapObject::MapObject(const MapObjectDescr* const the_descr)
-   : descr_(the_descr), serial_(0), logsink_(nullptr), owner_(nullptr), reserved_by_worker_(false) {
+MapObject::MapObject(const MapObjectDescr* const the_descr) : descr_(the_descr) {
 }
 
 /**
@@ -726,7 +708,7 @@ constexpr uint8_t kCurrentPacketVersionMapObject = 2;
  *
  * Derived functions must call ancestor's function in the appropriate place.
  */
-void MapObject::Loader::load(FileRead& fr) {
+void MapObject::Loader::load(FileRead& fr) const {
 	try {
 		uint8_t const header = fr.unsigned_8();
 		if (header != HeaderMapObject) {
