@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,7 @@
 
 #include "base/log.h"
 #include "base/macros.h"
-#include "base/multithreading.h"
+#include "base/mutex.h"
 #include "base/string.h"
 #include "base/wexception.h"
 #include "economy/flag.h"
@@ -58,14 +58,7 @@ BuildingDescr::BuildingDescr(const std::string& init_descname,
      buildable_(table.has_key("buildcost")),
      can_be_dismantled_(table.has_key("return_on_dismantle")),
      destructible_(table.has_key("destructible") ? table.get_bool("destructible") : true),
-     size_(BaseImmovable::SMALL),
-     mine_(false),
-     port_(false),
-     enhancement_(INVALID_INDEX),
-     enhanced_from_(INVALID_INDEX),
-     enhanced_building_(false),
-     hints_(table.get_table("aihints"), name()),
-     vision_range_(0) {
+     hints_(table.get_table("aihints"), name()) {
 	if (!is_animation_known("idle")) {
 		throw GameDataError("Building %s has no idle animation", name().c_str());
 	}
@@ -106,37 +99,8 @@ BuildingDescr::BuildingDescr(const std::string& init_descname,
 
 	// Parse build options
 	if (table.has_key("enhancement")) {
-		// TODO(GunChleoc): Compatibility code, remove "if" section after v1.0.
-		// The "else" branch with get_table is the current code.
-		if (table.get_datatype("enhancement") == LuaTable::DataType::kString) {
-			const std::string enh = table.get_string("enhancement");
-			log_warn("Deprecated enhancement code found in building '%s'", name().c_str());
-
-			if (enh == name()) {
-				throw wexception("enhancement to same type");
-			}
-			DescriptionIndex const en_i = descriptions.load_building(enh);
-			if (descriptions.building_exists(en_i)) {
-				enhancement_ = en_i;
-
-				//  Merge the enhancements workarea info into this building's
-				//  workarea info.
-				const BuildingDescr* tmp_enhancement = descriptions.get_building_descr(en_i);
-				for (const auto& area : tmp_enhancement->workarea_info_) {
-					std::set<std::string>& strs = workarea_info_[area.first];
-					for (const std::string& str : area.second) {
-						strs.insert(str);
-					}
-				}
-			} else {
-				throw GameDataError(
-				   "\"%s\" has not been defined as a building type (wrong declaration order?)",
-				   enh.c_str());
-			}
-		} else {
-			std::unique_ptr<LuaTable> enhancement_table = table.get_table("enhancement");
-			set_enhancement(descriptions, *enhancement_table);
-		}
+		std::unique_ptr<LuaTable> enhancement_table = table.get_table("enhancement");
+		set_enhancement(descriptions, *enhancement_table);
 	}
 
 	// We define a building as buildable if it has a "buildcost" table.
@@ -154,19 +118,6 @@ BuildingDescr::BuildingDescr(const std::string& init_descname,
 		buildcost_ = Buildcost(table.get_table("buildcost"), descriptions);
 	}
 
-	if (table.has_key("enhancement_cost")) {
-		// TODO(GunChleoc): Compatibility code, remove after v1.0
-		log_warn("Deprecated enhancement_cost code found in building '%s'", name().c_str());
-		if (!table.has_key("return_on_dismantle_on_enhanced")) {
-			throw GameDataError("The enhanced building '%s' has an \"enhancement_cost\" but no "
-			                    "\"return_on_dismantle_on_enhanced\"",
-			                    name().c_str());
-		}
-		set_enhancement_cost(
-		   Buildcost(table.get_table("enhancement_cost"), descriptions),
-		   Buildcost(table.get_table("return_on_dismantle_on_enhanced"), descriptions));
-	}
-
 	needs_seafaring_ = false;
 	needs_waterways_ = false;
 	if (table.has_key("map_check")) {
@@ -362,19 +313,7 @@ Implementation
 ==============================
 */
 
-Building::Building(const BuildingDescr& building_descr)
-   : PlayerImmovable(building_descr),
-     flag_(nullptr),
-     anim_(0),
-     animstart_(0),
-     leave_time_(0),
-     defeating_player_(0),
-     seeing_(false),
-     was_immovable_(nullptr),
-     attack_target_(nullptr),
-     soldier_control_(nullptr),
-     mute_messages_(false),
-     is_destruction_blocked_(false) {
+Building::Building(const BuildingDescr& building_descr) : PlayerImmovable(building_descr) {
 }
 
 void Building::load_finish(EditorGameBase& egbase) {
@@ -882,6 +821,15 @@ void Building::set_soldier_control(SoldierControl* new_soldier_control) {
 	soldier_control_ = new_soldier_control;
 }
 
+/**
+ * \return \c true if the worker is currently present and idle in the building.
+ */
+bool Building::is_present(Worker& worker) const {
+	return worker.get_location(get_owner()->egbase()) == this &&
+	       worker.get_state() == worker.get_state(Worker::taskBuildingwork) &&
+	       worker.get_position() == get_position();
+}
+
 /**
  * Change whether this building sees its vision range based on workers
  * inside the building.
