@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -45,8 +45,6 @@
 
 namespace Widelands {
 
-constexpr Duration ConstructionSite::kConstructionsiteStepTime;
-
 void ConstructionsiteInformation::draw(const Vector2f& point_on_dst,
                                        const Widelands::Coords& coords,
                                        float scale,
@@ -163,10 +161,7 @@ IMPLEMENTATION
 */
 
 ConstructionSite::ConstructionSite(const ConstructionSiteDescr& cs_descr)
-   : PartiallyFinishedBuilding(cs_descr),
-     fetchfromflag_(0),
-     builder_idle_(false),
-     settings_(nullptr) {
+   : PartiallyFinishedBuilding(cs_descr) {
 }
 
 void ConstructionSite::update_statistics_string(std::string* s) {
@@ -354,8 +349,7 @@ void ConstructionSite::cleanup(EditorGameBase& egbase) {
 				if (ms->desired_capacity != b.soldier_control()->soldier_capacity()) {
 					b.mutable_soldier_control()->set_soldier_capacity(ms->desired_capacity);
 				}
-				dynamic_cast<MilitarySite&>(b).set_soldier_preference(
-				   ms->prefer_heroes ? SoldierPreference::kHeroes : SoldierPreference::kRookies);
+				dynamic_cast<MilitarySite&>(b).set_soldier_preference(ms->soldier_preference);
 			} else if (upcast(WarehouseSettings, ws, settings_.get())) {
 				Warehouse& site = dynamic_cast<Warehouse&>(b);
 				for (const auto& pair : ws->ware_preferences) {
@@ -367,6 +361,8 @@ void ConstructionSite::cleanup(EditorGameBase& egbase) {
 				if (ws->launch_expedition) {
 					get_owner()->start_or_cancel_expedition(site);
 				}
+				site.mutable_soldier_control()->set_soldier_capacity(ws->desired_capacity);
+				site.set_soldier_preference(ws->soldier_preference);
 			} else {
 				NEVER_HERE();
 			}
@@ -398,7 +394,7 @@ void ConstructionSite::cleanup(EditorGameBase& egbase) {
 Start building the next enhancement even before the base building is completed.
 ===============
 */
-void ConstructionSite::enhance(const Game& game) {
+void ConstructionSite::enhance(const EditorGameBase& egbase) {
 	MutexLock m(MutexLock::ID::kObjects);
 	assert(building_->enhancement() != INVALID_INDEX);
 	Notifications::publish(NoteImmovable(this, NoteImmovable::Ownership::LOST));
@@ -468,6 +464,8 @@ void ConstructionSite::enhance(const Game& game) {
 			new_settings->worker_preferences[pair.first] = pair.second;
 		}
 		new_settings->launch_expedition = ws->launch_expedition && building_->get_isport();
+		new_settings->desired_capacity = ws->desired_capacity;
+		new_settings->soldier_preference = ws->soldier_preference;
 	} break;
 	case Widelands::MapObjectType::TRAININGSITE: {
 		upcast(const TrainingSiteDescr, td, building_);
@@ -536,11 +534,11 @@ void ConstructionSite::enhance(const Game& game) {
 		new_settings->desired_capacity = std::max<uint32_t>(
 		   1,
 		   new_desired_capacity(ms->max_capacity, ms->desired_capacity, new_settings->max_capacity));
-		new_settings->prefer_heroes = ms->prefer_heroes;
+		new_settings->soldier_preference = ms->soldier_preference;
 	} break;
 	default:
 		// TODO(Nordfriese): Add support for markets when trading is implemented
-		log_warn_time(game.get_gametime(),
+		log_warn_time(egbase.get_gametime(),
 		              "Enhanced constructionsite to a %s, which is not of any known building type\n",
 		              building_->name().c_str());
 	}
