@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,14 +22,15 @@
 #include <memory>
 
 #include "base/i18n.h"
-#include "base/log.h"
 #include "base/macros.h"
 #include "base/math.h"
 #include "base/wexception.h"
 #include "config.h"
 #include "economy/economy.h"
+#include "economy/ferry_fleet.h"
 #include "economy/flag.h"
 #include "economy/input_queue.h"
+#include "economy/ship_fleet.h"
 #include "economy/wares_queue.h"
 #include "io/filesystem/layered_filesystem.h"
 #include "logic/game.h"
@@ -171,10 +172,11 @@ Available actions are:
 */
 
 ProductionProgram::ActReturn::Condition* create_economy_condition(
-   const std::string& item, const ProductionSiteDescr& descr, const Descriptions& descriptions) {
+   const std::string& item, ProductionSiteDescr& descr, const Descriptions& descriptions) {
 	try {
 		const std::pair<WareWorker, DescriptionIndex> wareworker =
 		   descriptions.load_ware_or_worker(item);
+		descr.set_infinite_production_useful(true);
 		switch (wareworker.first) {
 		case WareWorker::wwWARE: {
 			descr.ware_demand_checks()->insert(wareworker.second);
@@ -190,25 +192,6 @@ ProductionProgram::ActReturn::Condition* create_economy_condition(
 		throw GameDataError("economy condition: %s", e.what());
 	}
 }
-
-// TODO(GunChleoc): Incorporate this into TrainingParameters constructor when we drop compatibility
-// after v1.0
-TrainingAttribute parse_training_attribute(const std::string& argument) {
-	if (argument == "health") {
-		return TrainingAttribute::kHealth;
-	}
-	if (argument == "attack") {
-		return TrainingAttribute::kAttack;
-	}
-	if (argument == "defense") {
-		return TrainingAttribute::kDefense;
-	}
-	if (argument == "evade") {
-		return TrainingAttribute::kEvade;
-	}
-	throw GameDataError(
-	   "Expected health|attack|defense|evade after 'soldier' but found '%s'", argument.c_str());
-}
 }  // namespace
 
 bool ProductionProgram::Action::get_building_work(Game& /* game */,
@@ -300,23 +283,32 @@ BillOfMaterials ProductionProgram::parse_bill_of_materials(
 	return result;
 }
 
-ProductionProgram::Action::TrainingParameters
-ProductionProgram::Action::TrainingParameters::parse(const std::vector<std::string>& arguments,
-                                                     const std::string& action_name) {
-	ProductionProgram::Action::TrainingParameters result;
+ProductionProgram::Action::TrainingParameters::TrainingParameters(
+   const std::vector<std::string>& arguments, const std::string& action_name) {
 	for (const std::string& argument : arguments) {
 		const std::pair<std::string, std::string> item = read_key_value_pair(argument, ':');
 		if (item.first == "soldier") {
-			result.attribute = parse_training_attribute(item.second);
+			if (item.second == "health") {
+				attribute = TrainingAttribute::kHealth;
+			} else if (item.second == "attack") {
+				attribute = TrainingAttribute::kAttack;
+			} else if (item.second == "defense") {
+				attribute = TrainingAttribute::kDefense;
+			} else if (item.second == "evade") {
+				attribute = TrainingAttribute::kEvade;
+			} else {
+				throw GameDataError(
+				   "Expected health|attack|defense|evade after 'soldier' but found '%s'",
+				   argument.c_str());
+			}
 		} else if (item.first == "level") {
-			result.level = read_int(item.second, 0);
+			level = read_int(item.second, 0);
 		} else {
 			throw GameDataError(
 			   "Unknown argument '%s'. Usage: %s=soldier:attack|defense|evade|health level:<number>",
 			   item.first.c_str(), action_name.c_str());
 		}
 	}
-	return result;
 }
 
 /* RST
@@ -405,7 +397,7 @@ Examples for ``return=skipped``:
    -- If the economy has sufficient 'coal', run anyway even if none of these two wares are needed.
    return=skipped unless economy needs iron or economy needs gold or not economy needs coal
 
-   -- If the building has no 'fur_garment_old' in its input queues, skip running this progam.
+   -- If the building has no 'fur_garment_old' in its input queues, skip running this program.
    return=skipped unless site has fur_garment_old
 
    -- If the building has 'wheat' in its input queue and if the economy needs the ware
@@ -427,7 +419,7 @@ Examples for ``return=skipped``:
 ProductionProgram::ActReturn::Negation::Negation(const std::vector<std::string>& arguments,
                                                  std::vector<std::string>::const_iterator& begin,
                                                  std::vector<std::string>::const_iterator& end,
-                                                 const ProductionSiteDescr& descr,
+                                                 ProductionSiteDescr& descr,
                                                  const Descriptions& descriptions)
    : operand(create_condition(arguments, begin, end, descr, descriptions)) {
 }
@@ -451,7 +443,7 @@ ProductionProgram::ActReturn::Negation::description_negation(const Descriptions&
 }
 
 bool ProductionProgram::ActReturn::EconomyNeedsWare::evaluate(const ProductionSite& ps) const {
-	return ps.get_economy(wwWARE)->needs_ware_or_worker(ware_type);
+	return ps.infinite_production() || ps.get_economy(wwWARE)->needs_ware_or_worker(ware_type);
 }
 std::string ProductionProgram::ActReturn::EconomyNeedsWare::description(
    const Descriptions& descriptions) const {
@@ -471,7 +463,7 @@ std::string ProductionProgram::ActReturn::EconomyNeedsWare::description_negation
 }
 
 bool ProductionProgram::ActReturn::EconomyNeedsWorker::evaluate(const ProductionSite& ps) const {
-	return ps.get_economy(wwWORKER)->needs_ware_or_worker(worker_type);
+	return ps.infinite_production() || ps.get_economy(wwWORKER)->needs_ware_or_worker(worker_type);
 }
 std::string ProductionProgram::ActReturn::EconomyNeedsWorker::description(
    const Descriptions& descriptions) const {
@@ -588,11 +580,47 @@ std::string ProductionProgram::ActReturn::WorkersNeedExperience::description_neg
 	return _("the workers need no experience");
 }
 
+bool ProductionProgram::ActReturn::FleetNeeds::evaluate(const ProductionSite& ps) const {
+	if (type_ == Type::kShip) {
+		for (ShipFleetYardInterface* interface : ps.get_ship_fleet_interfaces()) {
+			if (interface->get_fleet()->lacks_ship() || ps.infinite_production()) {
+				BaseImmovable* immo = interface->get_position().field->get_immovable();
+				if (immo == nullptr || immo->get_size() == BaseImmovable::Size::NONE) {
+					return true;
+				}
+			}
+		}
+	} else {
+		for (FerryFleetYardInterface* interface : ps.get_ferry_fleet_interfaces()) {
+			if (interface->get_fleet()->lacks_ferry() || ps.infinite_production()) {
+				BaseImmovable* immo = interface->get_position().field->get_immovable();
+				if (immo == nullptr || immo->get_size() == BaseImmovable::Size::NONE) {
+					return true;
+				}
+			}
+		}
+	}
+
+	return false;
+}
+std::string ProductionProgram::ActReturn::FleetNeeds::description(
+   const Descriptions& /* descriptions */) const {
+	/** TRANSLATORS: 'Completed/Skipped/Did not start ... because the fleet needs a ship/ferry'. */
+	return type_ == Type::kShip ? _("the fleet needs a ship") : _("the fleet needs a ferry");
+}
+
+std::string ProductionProgram::ActReturn::FleetNeeds::description_negation(
+   const Descriptions& /* descriptions */) const {
+	/** TRANSLATORS: 'Completed/Skipped/Did not start ... because the fleet needs no ships/ferries'.
+	 */
+	return type_ == Type::kShip ? _("the fleet needs no ships") : _("the fleet needs no ferries");
+}
+
 ProductionProgram::ActReturn::Condition*
 ProductionProgram::ActReturn::create_condition(const std::vector<std::string>& arguments,
                                                std::vector<std::string>::const_iterator& begin,
                                                std::vector<std::string>::const_iterator& end,
-                                               const ProductionSiteDescr& descr,
+                                               ProductionSiteDescr& descr,
                                                const Descriptions& descriptions) {
 	if (begin == end) {
 		throw GameDataError("Expected a condition after '%s'", (begin - 1)->c_str());
@@ -624,6 +652,25 @@ ProductionProgram::ActReturn::create_condition(const std::vector<std::string>& a
 			}
 			return new ProductionProgram::ActReturn::WorkersNeedExperience();
 		}
+		if (match_and_skip(arguments, begin, "fleet")) {
+			if (!match_and_skip(arguments, begin, "needs")) {
+				throw GameDataError(
+				   "Expected 'needs ship|ferry' after 'fleet' but found '%s'", begin->c_str());
+			}
+			descr.set_infinite_production_useful(true);
+			if (match_and_skip(arguments, begin, "ship")) {
+				descr.has_ship_fleet_check_ = true;
+				return new ProductionProgram::ActReturn::FleetNeeds(
+				   ProductionProgram::ActReturn::FleetNeeds::Type::kShip);
+			}
+			if (match_and_skip(arguments, begin, "ferry")) {
+				descr.has_ferry_fleet_check_ = true;
+				return new ProductionProgram::ActReturn::FleetNeeds(
+				   ProductionProgram::ActReturn::FleetNeeds::Type::kFerry);
+			}
+			throw GameDataError("Expected 'ship' or 'ferry' after 'fleet needs' but found '%s'",
+			                    begin == end ? "" : begin->c_str());
+		}
 		throw GameDataError("Expected not|economy|site|workers after '%s' but found '%s'",
 		                    (begin - 1)->c_str(), begin->c_str());
 	} catch (const WException& e) {
@@ -632,7 +679,7 @@ ProductionProgram::ActReturn::create_condition(const std::vector<std::string>& a
 }
 
 ProductionProgram::ActReturn::ActReturn(const std::vector<std::string>& arguments,
-                                        const ProductionSiteDescr& descr,
+                                        ProductionSiteDescr& descr,
                                         const Descriptions& descriptions) {
 	if (arguments.empty()) {
 		throw GameDataError("Usage: return=failed|completed|skipped [when|unless <conditions>]");
@@ -919,8 +966,7 @@ Calls a program of the productionsite's main worker. Example:
 ProductionProgram::ActCallWorker::ActCallWorker(const std::vector<std::string>& arguments,
                                                 const std::string& production_program_name,
                                                 ProductionSiteDescr* descr,
-                                                const Descriptions& descriptions)
-   : on_failure_(ProgramResult::kFailed) {
+                                                const Descriptions& descriptions) {
 	const size_t nr_args = arguments.size();
 	if (nr_args != 1 && nr_args != 4) {
 		throw GameDataError(
@@ -1043,20 +1089,13 @@ Blocks the execution of the program for the specified duration. Example:
          "callworker=scout"
       }
 */
-ProductionProgram::ActSleep::ActSleep(const std::vector<std::string>& arguments,
-                                      const ProductionSiteDescr& psite) {
+ProductionProgram::ActSleep::ActSleep(const std::vector<std::string>& arguments) {
 	if (arguments.size() != 1) {
 		throw GameDataError("Usage: sleep=duration:<duration>");
 	}
 	const std::pair<std::string, std::string> item = read_key_value_pair(arguments.front(), ':');
 	if (item.first == "duration") {
-		duration_ = read_duration(item.second, psite);
-	} else if (item.second.empty()) {
-		// TODO(GunChleoc): Compatibility, remove after v1.0
-		duration_ = read_duration(item.first, psite);
-		log_warn("'sleep' program without parameter name is deprecated, please use "
-		         "'sleep=duration:<duration>' in %s\n",
-		         psite.name().c_str());
+		duration_ = read_duration(item.second);
 	} else {
 		throw GameDataError(
 		   "Unknown argument '%s'. Usage: duration:<duration>", arguments.front().c_str());
@@ -1481,50 +1520,34 @@ mine
 ProductionProgram::ActMine::ActMine(const std::vector<std::string>& arguments,
                                     Descriptions& descriptions,
                                     const std::string& production_program_name,
-                                    ProductionSiteDescr* descr)
-   : resource_(INVALID_INDEX), notify_on_failure_(true) {
+                                    ProductionSiteDescr* descr) {
 	if (arguments.size() > 6 || arguments.size() < 4) {
 		throw GameDataError("Usage: mine=<resource name> radius:<number> yield:<percent> "
 		                    "when_empty:<percent> [experience:<percent>] [no_notify]");
 	}
 	experience_chance_ = 0U;
 
-	if (read_key_value_pair(arguments.at(2), ':').second.empty()) {
-		// TODO(GunChleoc): Savegame compatibility, remove after v1.0
-		log_warn("Using old syntax in %s. Please use 'mine=<resource name> radius:<number> "
-		         "yield:<percent> when_empty:<percent> [experience_on_fail:<percent>] [no_notify]'",
-		         descr->name().c_str());
-		resource_ = descriptions.load_resource(arguments.front());
-		workarea_ = read_positive(arguments.at(1));
-		max_resources_ = read_positive(arguments.at(2)) * 100U;
-		depleted_chance_ = read_positive(arguments.at(3)) * 100U;
-		if (arguments.size() == 5) {
-			experience_chance_ = read_positive(arguments.at(4)) * 100U;
-		}
-	} else {
-		for (const std::string& argument : arguments) {
-			const std::pair<std::string, std::string> item = read_key_value_pair(argument, ':');
-			if (item.second.empty()) {
-				// The safeguard is for the case that someone creates a resource called "no_notify"
-				if (item.first == "no_notify" && resource_ != INVALID_INDEX) {
-					notify_on_failure_ = false;
-				} else {
-					resource_ = descriptions.load_resource(item.first);
-				}
-			} else if (item.first == "radius") {
-				workarea_ = read_positive(item.second);
-			} else if (item.first == "yield") {
-				max_resources_ = math::read_percent_to_int(item.second);
-			} else if (item.first == "when_empty") {
-				depleted_chance_ = math::read_percent_to_int(item.second);
-			} else if (item.first == "experience_on_fail") {
-				experience_chance_ = math::read_percent_to_int(item.second);
+	for (const std::string& argument : arguments) {
+		const std::pair<std::string, std::string> item = read_key_value_pair(argument, ':');
+		if (item.second.empty()) {
+			// The safeguard is for the case that someone creates a resource called "no_notify"
+			if (item.first == "no_notify" && resource_ != INVALID_INDEX) {
+				notify_on_failure_ = false;
 			} else {
-				throw GameDataError(
-				   "Unknown argument '%s'. Usage: mine=<resource name> radius:<number> "
-				   "yield:<percent> when_empty:<percent> [experience_on_fail:<percent>]",
-				   item.first.c_str());
+				resource_ = descriptions.load_resource(item.first);
 			}
+		} else if (item.first == "radius") {
+			workarea_ = read_positive(item.second);
+		} else if (item.first == "yield") {
+			max_resources_ = math::read_percent_to_int(item.second);
+		} else if (item.first == "when_empty") {
+			depleted_chance_ = math::read_percent_to_int(item.second);
+		} else if (item.first == "experience_on_fail") {
+			experience_chance_ = math::read_percent_to_int(item.second);
+		} else {
+			throw GameDataError("Unknown argument '%s'. Usage: mine=<resource name> radius:<number> "
+			                    "yield:<percent> when_empty:<percent> [experience_on_fail:<percent>]",
+			                    item.first.c_str());
 		}
 	}
 
@@ -1699,24 +1722,11 @@ ProductionProgram::ActCheckSoldier::ActCheckSoldier(const std::vector<std::strin
 		                    "only available to trainingsites.",
 		                    descr.name().c_str());
 	}
-	if (arguments.size() != 2 && arguments.size() != 3) {
+	if (arguments.size() != 2) {
 		throw GameDataError("Usage: checksoldier=soldier:attack|defense|evade|health level:<number>");
 	}
 
-	if (arguments.size() == 3) {
-		// TODO(GunChleoc): Savegame compatibility, remove after v1.0
-		log_warn("Using old syntax in %s. Please use "
-		         "'checksoldier=soldier:attack|defense|evade|health level:<number>'\n",
-		         descr.name().c_str());
-
-		if (arguments.front() != "soldier") {
-			throw GameDataError("Expected 'soldier' but found '%s'", arguments.front().c_str());
-		}
-		training_.attribute = parse_training_attribute(arguments.at(1));
-		training_.level = read_int(arguments.at(2), 0);
-	} else {
-		training_ = TrainingParameters::parse(arguments, "checksoldier");
-	}
+	training_ = TrainingParameters(arguments, "checksoldier");
 }
 
 void ProductionProgram::ActCheckSoldier::execute(Game& game, ProductionSite& ps) const {
@@ -1807,25 +1817,11 @@ ProductionProgram::ActTrain::ActTrain(const std::vector<std::string>& arguments,
 		                    descr.name().c_str());
 	}
 
-	if (arguments.size() != 2 && arguments.size() != 4) {
+	if (arguments.size() != 2) {
 		throw GameDataError("Usage: train=soldier:attack|defense|evade|health level:<number>");
 	}
 
-	if (arguments.size() == 4) {
-		// TODO(GunChleoc): Savegame compatibility, remove after v1.0
-		log_warn("Using old syntax in %s. Please use train=soldier:attack|defense|evade|health "
-		         "level:<number>\n",
-		         descr.name().c_str());
-
-		if (arguments.front() != "soldier") {
-			throw GameDataError("Expected 'soldier' but found '%s'", arguments.front().c_str());
-		}
-
-		training_.attribute = parse_training_attribute(arguments.at(1));
-		training_.level = read_positive(arguments.at(3));
-	} else {
-		training_ = TrainingParameters::parse(arguments, "train");
-	}
+	training_ = TrainingParameters(arguments, "train");
 }
 
 void ProductionProgram::ActTrain::execute(Game& game, ProductionSite& ps) const {
@@ -1900,9 +1896,8 @@ playsound
 ---------
 Plays a sound effect. See :ref:`map_object_programs_playsound`.
 */
-ProductionProgram::ActPlaySound::ActPlaySound(const std::vector<std::string>& arguments,
-                                              const ProductionSiteDescr& descr)
-   : parameters(MapObjectProgram::parse_act_play_sound(arguments, descr)) {
+ProductionProgram::ActPlaySound::ActPlaySound(const std::vector<std::string>& arguments)
+   : parameters(MapObjectProgram::parse_act_play_sound(arguments)) {
 }
 
 void ProductionProgram::ActPlaySound::execute(Game& game, ProductionSite& ps) const {
@@ -1956,28 +1951,18 @@ ProductionProgram::ActConstruct::ActConstruct(const std::vector<std::string>& ar
 		   "Usage: construct=<immovable_name> worker:<program_name> radius:<number>");
 	}
 
-	if (read_key_value_pair(arguments.at(2), ':').second.empty()) {
-		// TODO(GunChleoc): Compatibility, remove this argument option after v1.0
-		log_warn("'construct' program without parameter names is deprecated, please use "
-		         "'construct=<immovable_name> worker:<program_name> radius:<number>' in %s\n",
-		         descr->name().c_str());
-		objectname = arguments.at(0);
-		workerprogram = arguments.at(1);
-		radius = read_positive(arguments.at(2));
-	} else {
-		for (const std::string& argument : arguments) {
-			const std::pair<std::string, std::string> item = read_key_value_pair(argument, ':');
-			if (item.first == "worker") {
-				workerprogram = item.second;
-			} else if (item.first == "radius") {
-				radius = read_positive(item.second);
-			} else if (item.second.empty()) {
-				objectname = item.first;
-			} else {
-				throw GameDataError("Unknown parameter '%s'. Usage: construct=<immovable_name> "
-				                    "worker:<program_name> radius:<number>",
-				                    item.first.c_str());
-			}
+	for (const std::string& argument : arguments) {
+		const std::pair<std::string, std::string> item = read_key_value_pair(argument, ':');
+		if (item.first == "worker") {
+			workerprogram = item.second;
+		} else if (item.first == "radius") {
+			radius = read_positive(item.second);
+		} else if (item.second.empty()) {
+			objectname = item.first;
+		} else {
+			throw GameDataError("Unknown parameter '%s'. Usage: construct=<immovable_name> "
+			                    "worker:<program_name> radius:<number>",
+			                    item.first.c_str());
 		}
 	}
 
@@ -1992,6 +1977,7 @@ ProductionProgram::ActConstruct::ActConstruct(const std::vector<std::string>& ar
 	     main_worker_descr.get_program(workerprogram)->created_attributes()) {
 		descr->add_created_attribute(attribute_info);
 	}
+	descr->has_ship_fleet_check_ = true;
 }
 
 const ImmovableDescr&
@@ -2030,9 +2016,11 @@ void ProductionProgram::ActConstruct::execute(Game& game, ProductionSite& psite)
 	std::vector<ImmovableFound> immovables;
 	CheckStepWalkOn cstep(MOVECAPS_WALK, true);
 	Area<FCoords> area(map.get_fcoords(psite.get_position()), radius);
-	if (map.find_reachable_immovables(game, area, &immovables, cstep, FindImmovableByDescr(descr)) !=
-	    0u) {
-		state.objvar = immovables[0].object;
+	FindImmovableAnd finder;
+	finder.add(FindImmovableByDescr(descr));
+	finder.add(FindImmovableNotReserved());
+	if (map.find_reachable_immovables(game, area, &immovables, cstep, finder) != 0U) {
+		state.objvar = immovables.at(game.logic_rand() % immovables.size()).object;
 
 		psite.working_positions_.at(psite.main_worker_)
 		   .worker.get(game)
@@ -2040,46 +2028,31 @@ void ProductionProgram::ActConstruct::execute(Game& game, ProductionSite& psite)
 		return;
 	}
 
-	// No object found, look for a field where we can build
-	std::vector<Coords> fields;
-	FindNodeAnd fna;
-	// 10 is custom value to make sure the "water" is at least 10 nodes big
-	fna.add(FindNodeShore(10));
-	fna.add(FindNodeImmovableSize(FindNodeImmovableSize::sizeNone));
-	if (map.find_reachable_fields(game, area, &fields, cstep, fna) != 0u) {
-		// Testing received fields to get one with less immovables nearby
-		Coords best_coords = fields.back();  // Just to initialize it
-		uint32_t best_score = std::numeric_limits<uint32_t>::max();
-		while (!fields.empty()) {
-			Coords coords = fields.back();
-
-			// Counting immovables nearby
-			std::vector<ImmovableFound> found_immovables;
-			const uint32_t imm_count =
-			   map.find_immovables(game, Area<FCoords>(map.get_fcoords(coords), 2), &found_immovables);
-			if (best_score > imm_count) {
-				best_score = imm_count;
-				best_coords = coords;
+	std::vector<ShipFleetYardInterface*> candidates;
+	bool ships_needed = false;
+	for (ShipFleetYardInterface* interface : psite.get_ship_fleet_interfaces()) {
+		if (interface->get_fleet()->lacks_ship()) {
+			ships_needed = true;
+			BaseImmovable* immo = interface->get_position().field->get_immovable();
+			if (immo == nullptr || immo->get_size() == BaseImmovable::Size::NONE) {
+				candidates.push_back(interface);
 			}
-
-			// No need to go on, it cannot be better
-			if (imm_count == 0) {
-				break;
-			}
-
-			fields.pop_back();
 		}
+	}
 
-		state.coord = best_coords;
-
-		psite.working_positions_.at(psite.main_worker_)
-		   .worker.get(game)
-		   ->update_task_buildingwork(game);
+	if (candidates.empty()) {
+		if (ships_needed || psite.get_ship_fleet_interfaces().empty()) {
+			psite.molog(game.get_gametime(), "construct: no space for ships\n");
+			psite.program_end(game, ProgramResult::kFailed);
+		} else {
+			psite.molog(game.get_gametime(), "construct: no ships needed\n");
+			psite.program_end(game, ProgramResult::kSkipped);
+		}
 		return;
 	}
 
-	psite.molog(game.get_gametime(), "construct: no object or buildable field\n");
-	psite.program_end(game, ProgramResult::kFailed);
+	state.coord = candidates.at(game.logic_rand() % candidates.size())->get_position();
+	psite.working_positions_.at(psite.main_worker_).worker.get(game)->update_task_buildingwork(game);
 }
 
 bool ProductionProgram::ActConstruct::get_building_work(Game& game,
@@ -2097,7 +2070,7 @@ bool ProductionProgram::ActConstruct::get_building_work(Game& game,
 
 	Immovable* construction = dynamic_cast<Immovable*>(state.objvar.get(game));
 	if (construction != nullptr) {
-		if (!construction->construct_remaining_buildcost(game, &remaining)) {
+		if (!construction->construct_remaining_buildcost(&remaining)) {
 			psite.molog(game.get_gametime(), "construct: immovable %u not under construction",
 			            construction->serial());
 			psite.program_end(game, ProgramResult::kFailed);
@@ -2132,6 +2105,9 @@ bool ProductionProgram::ActConstruct::get_building_work(Game& game,
 	worker.start_task_program(game, workerprogram);
 	worker.top_state().objvar1 = construction;
 	worker.top_state().coords = state.coord;
+	if (construction != nullptr) {
+		construction->set_reserved_by_worker(true);
+	}
 
 	state.phase = ProgramResult::kFailed;
 	return true;
@@ -2165,8 +2141,8 @@ ProductionProgram::ProductionProgram(const std::string& init_name,
 				actions_.push_back(
 				   std::unique_ptr<ProductionProgram::Action>(new ActCall(parseinput.arguments)));
 			} else if (parseinput.name == "sleep") {
-				actions_.push_back(std::unique_ptr<ProductionProgram::Action>(
-				   new ActSleep(parseinput.arguments, *building)));
+				actions_.push_back(
+				   std::unique_ptr<ProductionProgram::Action>(new ActSleep(parseinput.arguments)));
 			} else if (parseinput.name == "animate") {
 				actions_.push_back(std::unique_ptr<ProductionProgram::Action>(
 				   new ActAnimate(parseinput.arguments, building)));
@@ -2192,8 +2168,8 @@ ProductionProgram::ProductionProgram(const std::string& init_name,
 				actions_.push_back(std::unique_ptr<ProductionProgram::Action>(
 				   new ActTrain(parseinput.arguments, *building)));
 			} else if (parseinput.name == "playsound") {
-				actions_.push_back(std::unique_ptr<ProductionProgram::Action>(
-				   new ActPlaySound(parseinput.arguments, *building)));
+				actions_.push_back(
+				   std::unique_ptr<ProductionProgram::Action>(new ActPlaySound(parseinput.arguments)));
 			} else if (parseinput.name == "construct") {
 				actions_.push_back(std::unique_ptr<ProductionProgram::Action>(
 				   new ActConstruct(parseinput.arguments, name(), building, descriptions)));
@@ -2224,6 +2200,23 @@ ProductionProgram::ProductionProgram(const std::string& init_name,
 					recruited_workers_.insert(worker);
 				}
 			}
+			// Add trained attributes
+			if (upcast(const ActCheckSoldier, act_cs, &action)) {
+				const auto& train = act_cs->training();
+				train_from_level_ = train.level;
+				if (train.attribute == Widelands::TrainingAttribute::kHealth) {
+					trained_attribute_ = "Health";
+				} else if (train.attribute == Widelands::TrainingAttribute::kAttack) {
+					trained_attribute_ = "Attack";
+				} else if (train.attribute == Widelands::TrainingAttribute::kDefense) {
+					trained_attribute_ = "Defense";
+				} else if (train.attribute == Widelands::TrainingAttribute::kEvade) {
+					trained_attribute_ = "Evade";
+				}
+			} else if (upcast(const ActTrain, act_tr, &action)) {
+				const auto& train = act_tr->training();
+				train_to_level_ = train.level;
+			}
 		} catch (const std::exception& e) {
 			throw GameDataError("Error reading line '%s': %s", line.c_str(), e.what());
 		}
