@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,8 +25,10 @@
 #include "base/macros.h"
 #include "base/wexception.h"
 #include "economy/economy.h"
+#include "economy/ferry_fleet.h"
 #include "economy/input_queue.h"
 #include "economy/request.h"
+#include "economy/ship_fleet.h"
 #include "economy/ware_instance.h"
 #include "economy/wares_queue.h"
 #include "economy/workers_queue.h"
@@ -35,7 +37,9 @@
 #include "logic/game.h"
 #include "logic/game_data_error.h"
 #include "logic/map.h"
+#include "logic/map_objects/checkstep.h"
 #include "logic/map_objects/descriptions.h"
+#include "logic/map_objects/findnode.h"
 #include "logic/map_objects/tribes/carrier.h"
 #include "logic/map_objects/tribes/soldier.h"
 #include "logic/map_objects/tribes/tribe_descr.h"
@@ -94,6 +98,7 @@ ProductionSiteDescr::ProductionSiteDescr(const std::string& init_descname,
    : BuildingDescr(init_descname, init_type, table, descriptions),
      ware_demand_checks_(new std::set<DescriptionIndex>()),
      worker_demand_checks_(new std::set<DescriptionIndex>()),
+     is_infinite_production_useful_(false),
      out_of_resource_productivity_threshold_(100) {
 
 	std::unique_ptr<LuaTable> items_table;
@@ -284,19 +289,19 @@ IMPLEMENTATION
 */
 
 ProductionSite::ProductionSite(const ProductionSiteDescr& ps_descr)
-   : Building(ps_descr),
-     working_positions_(ps_descr.nr_working_positions()),
-     fetchfromflag_(0),
-     program_timer_(false),
-     program_time_(0),
-     post_timer_(50),
-     last_stat_percent_(0),
-     actual_percent_(0),
-     last_program_end_time(0),
-     is_stopped_(false),
-     default_anim_("idle"),
-     main_worker_(-1) {
+   : Building(ps_descr), working_positions_(ps_descr.nr_working_positions()) {
 	format_statistics_string();
+
+	if (descr().has_ship_fleet_check() || descr().has_ferry_fleet_check()) {
+		field_terrain_changed_subscriber_ = Notifications::subscribe<NoteFieldTerrainChanged>(
+		   [this](const NoteFieldTerrainChanged& note) {
+			   if (note.action == NoteFieldTerrainChanged::Change::kTerrain &&
+			       owner().egbase().map().calc_distance(note.fc, get_position()) <=
+			          descr().workarea_info().rbegin()->first + 1) {
+				   init_yard_interfaces(get_owner()->egbase());
+			   }
+		   });
+	}
 }
 
 void ProductionSite::load_finish(EditorGameBase& egbase) {
@@ -304,25 +309,69 @@ void ProductionSite::load_finish(EditorGameBase& egbase) {
 	format_statistics_string();
 }
 
+void ProductionSite::postload(EditorGameBase& egbase) {
+	Building::postload(egbase);
+
+	// TODO(Nordfriese): This is only needed for v1.1 savegame compatibility
+	if ((descr().has_ship_fleet_check() && ship_fleet_interfaces_.empty()) ||
+	    (descr().has_ferry_fleet_check() && ferry_fleet_interfaces_.empty())) {
+		init_yard_interfaces(egbase);
+	}
+}
+
 /**
  * Display whether we're occupied.
  */
 void ProductionSite::update_statistics_string(std::string* s) {
+	uint32_t nr_xp_requests = 0;
 	uint32_t nr_requests = 0;
 	uint32_t nr_coming = 0;
+
+	auto worker_descr_it = descr().working_positions().begin();
+	uint32_t worker_descr_it_counter = worker_descr_it->second;
+
 	for (uint32_t i = 0; i < descr().nr_working_positions(); ++i) {
 		const Widelands::Request* request = working_positions_.at(i).worker_request;
 		// Check whether a request is being fulfilled or not
+		bool vacant = false;
 		if (request != nullptr) {
 			if (request->is_open()) {
-				++nr_requests;
+				vacant = true;
 			} else {
 				++nr_coming;
 			}
 		} else if (working_positions_.at(i).worker == nullptr) {
 			// We might have no request, but no worker either
-			++nr_requests;
+			vacant = true;
 		}
+
+		if (vacant) {
+			// Check if this worker can be built directly
+			const WorkerDescr& d =
+			   *owner().egbase().descriptions().get_worker_descr(worker_descr_it->first);
+			if (d.is_buildable() || d.has_demand_check()) {
+				++nr_requests;
+			} else {
+				++nr_xp_requests;
+			}
+		}
+
+		--worker_descr_it_counter;
+		if (worker_descr_it_counter == 0) {
+			++worker_descr_it;
+			if (worker_descr_it != descr().working_positions().end()) {
+				worker_descr_it_counter = worker_descr_it->second;
+			}
+		}
+	}
+
+	if (nr_xp_requests > 0) {
+		*s = StyleManager::color_tag(
+		   (nr_xp_requests == 1 ?
+             owner().tribe().get_productionsite_experienced_worker_missing_string() :
+             owner().tribe().get_productionsite_experienced_workers_missing_string()),
+		   g_style_manager->building_statistics_style().low_color());
+		return;
 	}
 
 	if (nr_requests > 0) {
@@ -488,6 +537,8 @@ bool ProductionSite::init(EditorGameBase& egbase) {
 		}
 	}
 
+	init_yard_interfaces(egbase);
+
 	if (upcast(Game, game, &egbase)) {
 		try_start_working(*game);
 	}
@@ -530,6 +581,8 @@ void ProductionSite::set_economy(Economy* const e, WareWorker type) {
  * Cleanup after a production site is removed
  */
 void ProductionSite::cleanup(EditorGameBase& egbase) {
+	field_terrain_changed_subscriber_.reset();
+
 	for (uint32_t i = descr().nr_working_positions(); i != 0u;) {
 		--i;
 		delete working_positions_.at(i).worker_request;
@@ -552,6 +605,13 @@ void ProductionSite::cleanup(EditorGameBase& egbase) {
 	}
 	input_queues_.clear();
 
+	while (!ship_fleet_interfaces_.empty()) {
+		ship_fleet_interfaces_.front()->remove(egbase);
+	}
+	while (!ferry_fleet_interfaces_.empty()) {
+		ferry_fleet_interfaces_.front()->remove(egbase);
+	}
+
 	Building::cleanup(egbase);
 }
 
@@ -822,12 +882,9 @@ void ProductionSite::set_next_program_override(Game& game,
 }
 
 bool ProductionSite::has_forced_state() const {
-	for (const State& s : stack_) {
-		if ((s.flags & State::StateFlags::kStateFlagIgnoreStopped) != 0u) {
-			return true;
-		}
-	}
-	return false;
+	return std::any_of(stack_.begin(), stack_.end(), [](const State& s) {
+		return (s.flags & State::StateFlags::kStateFlagIgnoreStopped) != 0;
+	});
 }
 
 void ProductionSite::find_and_start_next_program(Game& game) {
@@ -884,6 +941,11 @@ void ProductionSite::set_stopped(bool const stopped) {
 	Notifications::publish(NoteBuilding(serial(), NoteBuilding::Action::kChanged));
 }
 
+void ProductionSite::set_infinite_production(const bool i) {
+	infinite_production_ = i;
+	Notifications::publish(NoteBuilding(serial(), NoteBuilding::Action::kChanged));
+}
+
 /**
  * \return True if this production site could theoretically start working (if
  * all workers are present)
@@ -1155,6 +1217,83 @@ void ProductionSite::unnotify_player() {
 	set_production_result("");
 }
 
+void ProductionSite::init_yard_interfaces(EditorGameBase& egbase) {
+	const Map& map = egbase.map();
+
+	while (!ship_fleet_interfaces_.empty()) {
+		ship_fleet_interfaces_.front()->remove(egbase);
+	}
+	while (!ferry_fleet_interfaces_.empty()) {
+		ferry_fleet_interfaces_.front()->remove(egbase);
+	}
+
+	if (descr().has_ship_fleet_check()) {
+		std::vector<Coords> result;
+		// 10 is a custom value to make sure the "ocean" is at least 10 nodes big.
+		constexpr int kMinOceanSize = 10;
+		map.find_reachable_fields(
+		   egbase,
+		   Area<FCoords>(map.get_fcoords(get_position()), descr().workarea_info().rbegin()->first),
+		   &result, CheckStepDefault(MOVECAPS_WALK), FindNodeShore(kMinOceanSize));
+
+		for (const Coords& coords : result) {
+			ship_fleet_interfaces_.push_back(ShipFleetYardInterface::create(egbase, *this, coords));
+		}
+
+		if (ship_fleet_interfaces_.empty()) {
+			if (upcast(Game, game, &egbase)) {
+				send_message(*game, Message::Type::kEconomy, pgettext("building", "No Shore"),
+				             descr().icon_filename(), _("Shipyard Without Shore"),
+				             _("Your shipyard has not been built close enough to a shore. It will not "
+				               "be able to build ships."));
+			}
+		}
+	}
+
+	if (descr().has_ferry_fleet_check()) {
+		std::vector<Coords> result;
+		map.find_reachable_fields(
+		   egbase,
+		   Area<FCoords>(map.get_fcoords(get_position()), descr().workarea_info().rbegin()->first),
+		   &result, CheckStepDefault(MOVECAPS_WALK), FindNodeFerry(0));
+
+		for (const Coords& coords : result) {
+			ferry_fleet_interfaces_.push_back(FerryFleetYardInterface::create(egbase, *this, coords));
+		}
+
+		if (ferry_fleet_interfaces_.empty()) {
+			if (upcast(Game, game, &egbase)) {
+				send_message(*game, Message::Type::kEconomy, pgettext("building", "No Shore"),
+				             descr().icon_filename(), _("Ferry Yard Without Shore"),
+				             _("Your ferry yard has not been built close enough to a shore. It will "
+				               "not be able to build ferries."));
+			}
+		}
+	}
+}
+
+void ProductionSite::remove_fleet_interface(EditorGameBase& /*egbase*/,
+                                            const ShipFleetYardInterface* interface) {
+	auto it = std::find(ship_fleet_interfaces_.begin(), ship_fleet_interfaces_.end(), interface);
+	if (it == ship_fleet_interfaces_.end()) {
+		throw wexception("Attempt to remove unknown ship fleet interface %u",
+		                 interface == nullptr ? 0 : interface->serial());
+	}
+	*it = ship_fleet_interfaces_.back();
+	ship_fleet_interfaces_.pop_back();
+}
+
+void ProductionSite::remove_fleet_interface(EditorGameBase& /*egbase*/,
+                                            const FerryFleetYardInterface* interface) {
+	auto it = std::find(ferry_fleet_interfaces_.begin(), ferry_fleet_interfaces_.end(), interface);
+	if (it == ferry_fleet_interfaces_.end()) {
+		throw wexception("Attempt to remove unknown ferry fleet interface %u",
+		                 interface == nullptr ? 0 : interface->serial());
+	}
+	*it = ferry_fleet_interfaces_.back();
+	ferry_fleet_interfaces_.pop_back();
+}
+
 std::unique_ptr<const BuildingSettings> ProductionSite::create_building_settings() const {
 	std::unique_ptr<ProductionsiteSettings> settings(
 	   new ProductionsiteSettings(descr(), owner().tribe()));
