@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
 
 #include "logic/map_objects/tribes/soldier.h"
 
+#include <algorithm>
 #include <memory>
 
 #include "base/log.h"
@@ -43,6 +44,7 @@
 #include "logic/map_objects/tribes/militarysite.h"
 #include "logic/map_objects/tribes/tribe_descr.h"
 #include "logic/map_objects/tribes/warehouse.h"
+#include "logic/mapregion.h"
 #include "logic/message_queue.h"
 #include "logic/player.h"
 #include "map_io/map_object_loader.h"
@@ -52,20 +54,14 @@
 namespace Widelands {
 
 namespace {
+constexpr int kPortSpaceRadius = 2;
+constexpr int kPortSpaceGeneralAreaRadius = 5;
+
 constexpr int kSoldierHealthBarWidth = 13;
 constexpr int kRetreatWhenHealthDropsBelowThisPercentage = 50;
-}  // namespace
 
-SoldierLevelRange::SoldierLevelRange()
-   : min_health(-1),
-     min_attack(-1),
-     min_defense(-1),
-     min_evade(-1),
-     max_health(-1),
-     max_attack(-1),
-     max_defense(-1),
-     max_evade(-1) {
-}
+NavalInvasionBaseDescr g_naval_invasion_base_descr("naval_invasion_base", "Naval Invasion Base");
+}  // namespace
 
 SoldierLevelRange::SoldierLevelRange(const LuaTable& t) {
 	min_health = t.get_int("min_health");
@@ -99,8 +95,7 @@ SoldierDescr::SoldierDescr(const std::string& init_descname,
      health_(table.get_table("health")),
      attack_(table.get_table("attack")),
      defense_(table.get_table("defense")),
-     evade_(table.get_table("evade")),
-     max_anim_height_(0) {
+     evade_(table.get_table("evade")) {
 
 	// Battle animations
 	// attack_success_*-> soldier is attacking and hit his opponent
@@ -739,7 +734,8 @@ void Soldier::start_animation(EditorGameBase& egbase,
  * \return \c true if this soldier is considered to be on the battlefield
  */
 bool Soldier::is_on_battlefield() {
-	return (get_state(taskAttack) != nullptr) || (get_state(taskDefense) != nullptr);
+	return (get_state(taskAttack) != nullptr) || (get_state(taskDefense) != nullptr) ||
+	       (get_state(taskNavalInvasion) != nullptr);
 }
 
 /**
@@ -806,7 +802,7 @@ void Soldier::init_auto_task(Game& game) {
 struct FindNodeOwned {
 	explicit FindNodeOwned(PlayerNumber owner) : owner_(owner) {
 	}
-	bool accept(const EditorGameBase& /* egbase */, const FCoords& coords) const {
+	[[nodiscard]] bool accept(const EditorGameBase& /* egbase */, const FCoords& coords) const {
 		return (coords.field->get_owned_by() == owner_);
 	}
 
@@ -824,15 +820,15 @@ private:
 Bob::Task const Soldier::taskAttack = {"attack", static_cast<Bob::Ptr>(&Soldier::attack_update),
                                        nullptr, static_cast<Bob::Ptr>(&Soldier::attack_pop), true};
 
-void Soldier::start_task_attack(Game& game, Building& building) {
+void Soldier::start_task_attack(Game& game, Building& building, CombatFlags flags) {
 	push_task(game, taskAttack);
 	State& state = top_state();
 	state.objvar1 = &building;
 	state.coords = building.get_position();
+	state.ivar1 = flags;
 	state.ivar2 = 0;  // The return state 1=go home 2=go back in known land
 	state.ivar3 = 0;  // Counts how often the soldier is blocked in a row
 
-	state.ivar1 |= CF_RETREAT_WHEN_INJURED;
 	set_retreat_health(kRetreatWhenHealthDropsBelowThisPercentage * get_max_health() / 100);
 
 	// Injured soldiers are not allowed to attack
@@ -974,6 +970,11 @@ void Soldier::attack_update(Game& game, State& state) {
 		}
 	}
 
+	if (location != nullptr && get_position() == location->get_position()) {
+		// Still at home, need to go outside first
+		return start_task_leavebuilding(game, false);
+	}
+
 	if (battle_ != nullptr) {
 		return start_task_battle(game);
 	}
@@ -988,6 +989,13 @@ void Soldier::attack_update(Game& game, State& state) {
 
 	// Count remaining defenders
 	if (enemy != nullptr) {
+		if (!owner().is_hostile(enemy->owner()) && !enemy->owner().is_hostile(owner())) {
+			/* The players agreed on a truce. */
+			molog(game.get_gametime(), "[attack] opponent is an ally, cancel attack");
+			combat_walking_ = CD_NONE;
+			return pop_task(game);
+		}
+
 		if (enemy->soldier_control() != nullptr) {
 			defenders = enemy->soldier_control()->present_soldiers().size();
 		}
@@ -1003,16 +1011,15 @@ void Soldier::attack_update(Game& game, State& state) {
 		defenders += soldiers.size();
 	}
 
-	if ((enemy == nullptr) || (get_retreat_health() > get_current_health() && defenders > 0)) {
+	const bool consider_retreat = (state.ivar1 & CF_RETREAT_WHEN_INJURED) != 0 && defenders > 0 &&
+	                              get_retreat_health() > get_current_health();
+	if (enemy == nullptr || consider_retreat) {
 		// Injured soldiers will try to return to safe site at home.
-		if (get_retreat_health() > get_current_health()) {
-			assert(state.ivar1 & CF_RETREAT_WHEN_INJURED);
-			if (defenders != 0u) {
-				molog(game.get_gametime(), " [attack] badly injured (%d), retreating...\n",
-				      get_current_health());
-				state.coords = Coords::null();
-				state.objvar1 = nullptr;
-			}
+		if (consider_retreat) {
+			molog(game.get_gametime(), " [attack] badly injured (%d), retreating...\n",
+			      get_current_health());
+			state.coords = Coords::null();
+			state.objvar1 = nullptr;
 		}
 		// The old militarysite gets replaced by a new one, so if "enemy" is not
 		// valid anymore, we either "conquered" the new building, or it was
@@ -1030,7 +1037,11 @@ void Soldier::attack_update(Game& game, State& state) {
 				    ((location == nullptr) ||
 				     location->base_flag().get_position() != newsite->base_flag().get_position())) {
 					molog(game.get_gametime(), "[attack] enemy belongs to us now, move in\n");
-					pop_task(game);
+
+					// Remain at the conquered site.
+					reset_tasks(game);
+					start_task_buildingwork(game);
+
 					set_location(newsite);
 					newsite->update_soldier_request();
 					return schedule_act(game, Duration(10));
@@ -1440,6 +1451,12 @@ void Soldier::battle_update(Game& game, State& /* state */) {
 
 	const Map& map = game.map();
 	Soldier& opponent = *battle_->opponent(*this);
+	if (!owner().is_hostile(opponent.owner()) && !opponent.owner().is_hostile(owner())) {
+		/* The players agreed on a truce. */
+		molog(game.get_gametime(), "[battle] opponent is an ally, cancel battle");
+		combat_walking_ = CD_NONE;
+		return pop_task(game);
+	}
 	if (opponent.get_position() != get_position()) {
 		const MapObject* mo = map[get_position()].get_immovable();
 		if ((mo != nullptr) && mo->descr().type() >= MapObjectType::BUILDING) {
@@ -1596,6 +1613,202 @@ void Soldier::die_pop(Game& game, State& /* state */) {
 	schedule_destroy(game);
 }
 
+Bob::Task const Soldier::taskNavalInvasion = {
+   "naval_invasion", static_cast<Bob::Ptr>(&Soldier::naval_invasion_update), nullptr, nullptr,
+   true};
+
+void Soldier::start_task_naval_invasion(Game& game, const Coords& coords) {
+	push_task(game, taskNavalInvasion);
+	State& invasion_state = top_state();
+
+	invasion_state.coords = coords;
+	invasion_state.ivar1 = 0;
+	invasion_state.ivar2 =
+	   game.descriptions().get_building_descr(owner().tribe().port())->get_conquers();
+	invasion_state.ivar3 = game.descriptions().get_largest_workarea();
+
+	molog(game.get_gametime(), "[naval_invasion] Starting at %3dx%3d\n", coords.x, coords.y);
+}
+
+void Soldier::naval_invasion_update(Game& game, State& state) {
+	std::string signal = get_signal();
+	if (!signal.empty()) {
+		signal_handled();
+	}
+
+	if (battle_ != nullptr) {
+		// We are being attacked
+		return start_task_battle(game);
+	}
+
+	/*
+	 * Soldiers in a naval invasion are based around `state.coords` and seek to eliminate
+	 * all enemy influence within a radius of `state.ivar2 + state.ivar3` (where ivar2 is the
+	 * conquer radius of our planned port and ivar3 the biggest possible enemy conquer radius).
+	 * When such influence is found, the target building serial is stored in `state.ivar1`.
+	 * Otherwise, they stay at the flag and wait indefinitely until an own port
+	 * has been built at their location.
+	 */
+
+	if (state.ivar1 != 0) {
+		// Heading to enemy building
+		Building* enemy = dynamic_cast<Building*>(game.objects().get_object(state.ivar1));
+		state.ivar1 = 0;
+
+		if (enemy == nullptr || !owner().is_hostile(enemy->owner()) ||
+		    enemy->attack_target() == nullptr || !enemy->attack_target()->can_be_attacked()) {
+			molog(game.get_gametime(), "[naval_invasion] Attack target vanished or not hostile\n");
+			return schedule_act(game, Duration(10));
+		}
+
+		start_task_attack(game, *enemy, CF_NONE);
+		return schedule_act(game, Duration(10));
+	}
+
+	// No enemy, look for one
+	const Map& map = game.map();
+	CheckStepWalkOn checkstep(descr().movecaps(), false);
+	const FCoords portspace_fcoords = map.get_fcoords(state.coords);
+
+	std::vector<ImmovableFound> results;
+	map.find_reachable_immovables(game, Area<FCoords>(portspace_fcoords, state.ivar2 + state.ivar3),
+	                              &results, checkstep, FindImmovableAttackTarget());
+	// Consider closest targets first (estimate by air distance to us, not to the port space)
+	std::sort(results.begin(), results.end(),
+	          [this, &map](const ImmovableFound& a, const ImmovableFound& b) {
+		          const uint32_t distance_a = map.calc_distance(get_position(), a.coords);
+		          const uint32_t distance_b = map.calc_distance(get_position(), b.coords);
+		          if (distance_a != distance_b) {
+			          return distance_a < distance_b;
+		          }
+		          return a.object->serial() < b.object->serial();
+	          });
+
+	for (const ImmovableFound& result : results) {
+		Building& bld = dynamic_cast<Building&>(*result.object);
+		if (!owner().is_hostile(bld.owner()) || bld.attack_target() == nullptr ||
+		    !bld.attack_target()->can_be_attacked() ||
+		    bld.descr().get_conquers() + state.ivar2 <
+		       map.calc_distance(bld.get_position(), state.coords)) {
+			continue;
+		}
+
+		// Attack in next cycle
+		molog(game.get_gametime(), "[naval_invasion] Attack target selected (%s at %3dx%3d)\n",
+		      bld.descr().name().c_str(), bld.get_position().x, bld.get_position().y);
+		state.ivar1 = bld.serial();
+		return schedule_act(game, Duration(10));
+	}
+
+	// Check if another enemy wants our port space as well
+	if (get_battle() == nullptr) {
+		std::vector<Bob*> hostile_soldiers;
+		map.find_reachable_bobs(game, Area<FCoords>(portspace_fcoords, state.ivar2),
+		                        &hostile_soldiers, checkstep, FindBobEnemySoldier(get_owner()));
+		std::sort(hostile_soldiers.begin(), hostile_soldiers.end(), [this, &map](Bob* a, Bob* b) {
+			const uint32_t distance_a = map.calc_distance(get_position(), a->get_position());
+			const uint32_t distance_b = map.calc_distance(get_position(), b->get_position());
+			if (distance_a != distance_b) {
+				return distance_a < distance_b;
+			}
+			return a->serial() < b->serial();
+		});
+		for (Bob* bob : hostile_soldiers) {
+			upcast(Soldier, soldier, bob);
+			if (soldier->can_be_challenged() && soldier->get_battle() == nullptr &&
+			    soldier->get_state(taskNavalInvasion) != nullptr) {
+				molog(game.get_gametime(),
+				      "[naval_invasion] Hostile soldier selected (%s at %3dx%3d)\n",
+				      soldier->descr().name().c_str(), soldier->get_position().x,
+				      soldier->get_position().y);
+				new Battle(game, this, soldier);
+				return start_task_battle(game);
+			}
+		}
+	}
+
+	// No targets found, return to our temporary camp and wait there.
+
+	if (portspace_fcoords.field->get_immovable() != nullptr &&
+	    portspace_fcoords.field->get_immovable()->descr().type() == MapObjectType::WAREHOUSE &&
+	    portspace_fcoords.field->get_immovable()->get_owner() == get_owner()) {
+		Warehouse& wh = dynamic_cast<Warehouse&>(*portspace_fcoords.field->get_immovable());
+		if (wh.descr().get_isport()) {
+			// A port has been built. Our work here is done.
+			pop_task(game);
+			set_location(&wh);
+
+			if (get_position().field->get_immovable() == &wh) {
+				molog(game.get_gametime(), "[naval_invasion] Entering port\n");
+				wh.incorporate_worker(game, this);
+				return;
+			}
+
+			molog(game.get_gametime(), "[naval_invasion] Heading to port\n");
+			return start_task_return(game, false);
+		}
+	}
+
+	// The player doesn't have to build the port exactly where we envision it.
+	if (map.find_reachable_immovables(
+	       game, Area<FCoords>(get_position(), kPortSpaceGeneralAreaRadius), nullptr, checkstep,
+	       FindFlagWithPlayersWarehouse(*get_owner())) > 0) {
+		molog(game.get_gametime(), "[naval_invasion] Nearby economy found, join it\n");
+		return pop_task(game);
+	}
+
+	if (map.calc_distance(get_position(), state.coords) <= kPortSpaceGeneralAreaRadius) {
+		// The target should be unguarded now, conquer the port space.
+
+		bool has_invasion_base = false;
+		for (Bob* bob = map[state.coords].get_first_bob(); bob != nullptr;
+		     bob = bob->get_next_bob()) {
+			if (bob->descr().type() == MapObjectType::NAVAL_INVASION_BASE &&
+			    bob->get_owner() == get_owner()) {
+				upcast(NavalInvasionBase, invasion, bob);
+				assert(invasion != nullptr);
+				has_invasion_base = true;
+				invasion->add_soldier(game, this);  // add ourself if not present already
+				break;
+			}
+		}
+
+		if (!has_invasion_base) {
+			NavalInvasionBase::create(game, *this, state.coords);
+			return start_task_idle(game, descr().get_animation("idle", this), 1000);
+		}
+
+		// Stationed soldiers heal very slowly.
+		if (current_health_ < get_max_health()) {
+			constexpr unsigned kStationaryHealPerSecond = 10;
+			heal(kStationaryHealPerSecond);
+		}
+
+		return start_task_idle(game, descr().get_animation("idle", this), 1000);
+	}
+
+	if (start_task_movepath(game, state.coords, 0,
+	                        descr().get_right_walk_anims(does_carry_ware(), this), false,
+	                        kPortSpaceRadius)) {
+		molog(game.get_gametime(), "[naval_invasion] Return to camp\n");
+		return;
+	}
+
+	// Camp is blocked, try to get as close as we can.
+	for (int i = 10; i > 0; --i) {
+		Coords coords = game.random_location(state.coords, kPortSpaceGeneralAreaRadius);
+		if (start_task_movepath(game, coords, 0,
+		                        descr().get_right_walk_anims(does_carry_ware(), this), false,
+		                        kPortSpaceRadius)) {
+			molog(game.get_gametime(), "[naval_invasion] Approach camp\n");
+			return;
+		}
+	}
+
+	molog(game.get_gametime(), "[naval_invasion] Could not find a way to camp!\n");
+	return pop_task(game);  // Become fugitive
+}
+
 /**
  * Accept a Bob if it is a Soldier, is not dead and is running taskAttack or
  * taskDefense.
@@ -1616,13 +1829,20 @@ struct FindBobSoldierOnBattlefield : public FindBob {
  */
 bool Soldier::check_node_blocked(Game& game, const FCoords& field, bool const commit) {
 	State* attackdefense = get_state(taskAttack);
+	bool is_retreating = false;
 
 	if (attackdefense == nullptr) {
 		attackdefense = get_state(taskDefense);
 	}
 
-	if ((attackdefense == nullptr) || (((attackdefense->ivar1 & CF_RETREAT_WHEN_INJURED) != 0) &&
-	                                   get_retreat_health() > get_current_health())) {
+	if (attackdefense == nullptr) {
+		attackdefense = get_state(taskNavalInvasion);
+	} else {
+		is_retreating = ((attackdefense->ivar1 & CF_RETREAT_WHEN_INJURED) != 0) &&
+		                get_retreat_health() > get_current_health();
+	}
+
+	if (attackdefense == nullptr || is_retreating) {
 		// Retreating or non-combatant soldiers act like normal bobs
 		return Bob::check_node_blocked(game, field, commit);
 	}
@@ -1641,18 +1861,28 @@ bool Soldier::check_node_blocked(Game& game, const FCoords& field, bool const co
 	bool multiplesoldiers = false;
 
 	for (Bob* bob = field.field->get_first_bob(); bob != nullptr; bob = bob->get_next_on_field()) {
+		if (bob == this || bob->descr().type() != MapObjectType::SOLDIER) {
+			continue;
+		}
 		if (upcast(Soldier, soldier, bob)) {
 			if (!soldier->is_on_battlefield() || (soldier->get_current_health() == 0u)) {
 				continue;
 			}
 
+			if (soldier->get_state() != nullptr && soldier->get_state()->task == &taskNavalInvasion &&
+			    get_state() != nullptr && get_state()->task == &taskNavalInvasion &&
+			    soldier->get_owner() == get_owner()) {
+				continue;
+			}
+
 			if (foundsoldier == nullptr) {
 				foundsoldier = soldier;
 			} else {
 				multiplesoldiers = true;
 			}
 
-			if ((soldier->get_battle() != nullptr) &&
+			if (soldier->get_battle() != nullptr && soldier->get_battle()->first() != nullptr &&
+			    soldier->get_battle()->second() != nullptr &&
 			    game.map().calc_distance(soldier->get_battle()->first()->get_position(),
 			                             soldier->get_battle()->second()->get_position()) < 2) {
 				foundbattle = true;
@@ -1752,9 +1982,6 @@ Load/save support
 
 constexpr uint8_t kCurrentPacketVersion = 3;
 
-Soldier::Loader::Loader() : battle_(0) {
-}
-
 void Soldier::Loader::load(FileRead& fr) {
 	Worker::Loader::load(fr);
 
@@ -1824,6 +2051,9 @@ const Bob::Task* Soldier::Loader::get_task(const std::string& name) {
 	if (name == "die") {
 		return &taskDie;
 	}
+	if (name == "naval_invasion") {
+		return &taskNavalInvasion;
+	}
 	return Worker::Loader::get_task(name);
 }
 
@@ -1850,4 +2080,138 @@ void Soldier::do_save(EditorGameBase& egbase, MapObjectSaver& mos, FileWrite& fw
 
 	fw.unsigned_32(mos.get_object_file_index_or_zero(battle_));
 }
+
+// Naval invasion base implementation
+
+const NavalInvasionBaseDescr& NavalInvasionBase::descr() const {
+	return g_naval_invasion_base_descr;
+}
+
+Bob& NavalInvasionBaseDescr::create_object() const {
+	return *new NavalInvasionBase();
+}
+
+NavalInvasionBase::NavalInvasionBase() : Bob(g_naval_invasion_base_descr) {
+}
+
+NavalInvasionBase*
+NavalInvasionBase::create(EditorGameBase& egbase, Soldier& soldier, const Coords& pos) {
+	NavalInvasionBase* invasion = new NavalInvasionBase();
+	invasion->set_owner(soldier.get_owner());
+	invasion->set_position(egbase, pos);
+
+	invasion->add_soldier(egbase, &soldier);
+	invasion->init(egbase);
+
+	return invasion;
+}
+
+void NavalInvasionBase::init_auto_task(Game& game) {
+	for (auto it = soldiers_.begin(); it != soldiers_.end();) {
+		Soldier* soldier = it->get(game);
+		if (soldier == nullptr) {
+			molog(game.get_gametime(), "Soldier %u no longer exists", it->serial());
+			it = soldiers_.erase(it);
+			continue;
+		}
+
+		State* state = soldier->get_state(Soldier::taskNavalInvasion);
+		if (state == nullptr) {
+			molog(game.get_gametime(), "Soldier %u no longer an invasion member", soldier->serial());
+			it = soldiers_.erase(it);
+			continue;
+		}
+		if (state->coords != get_position()) {
+			molog(game.get_gametime(), "Soldier %u at different invasion location %dx%d!",
+			      soldier->serial(), state->coords.x, state->coords.y);
+			it = soldiers_.erase(it);
+			continue;
+		}
+
+		++it;
+	}
+
+	if (soldiers_.empty()) {
+		schedule_destroy(game);
+	}
+	start_task_idle(game, 0, 1000);
+}
+
+void NavalInvasionBase::cleanup(EditorGameBase& egbase) {
+	egbase.unconquer_area(PlayerArea<Area<FCoords>>(
+	   owner().player_number(), Area<FCoords>(get_position(), kPortSpaceRadius)));
+
+	Bob::cleanup(egbase);
+}
+
+void NavalInvasionBase::log_general_info(const EditorGameBase& egbase) const {
+	Bob::log_general_info(egbase);
+	molog(egbase.get_gametime(), "Invasion at %3dx%3d with %" PRIuS " soldiers\n", get_position().x,
+	      get_position().y, soldiers_.size());
+	for (auto soldier : soldiers_) {
+		molog(egbase.get_gametime(), "Soldier %u", soldier.serial());
+	}
+}
+
+void NavalInvasionBase::add_soldier(EditorGameBase& egbase, Soldier* soldier) {
+	assert(soldier != nullptr);
+
+	if (soldiers_.empty()) {
+		egbase.conquer_area(PlayerArea<Area<FCoords>>(
+		   owner().player_number(), Area<FCoords>(get_position(), kPortSpaceRadius)));
+	}
+
+	soldiers_.insert(soldier);
+}
+
+constexpr uint8_t kCurrentPacketVersionInvasion = 1;
+
+void NavalInvasionBase::Loader::load(FileRead& fr) {
+	Bob::Loader::load(fr);
+	for (size_t i = fr.unsigned_32(); i > 0; --i) {
+		soldiers_.insert(fr.unsigned_32());
+	}
+}
+
+void NavalInvasionBase::Loader::load_pointers() {
+	Bob::Loader::load_pointers();
+
+	NavalInvasionBase& invasion = get<NavalInvasionBase>();
+	for (Serial s : soldiers_) {
+		invasion.add_soldier(egbase(), &mol().get<Soldier>(s));
+	}
+}
+
+Bob::Loader* NavalInvasionBase::load(EditorGameBase& egbase, MapObjectLoader& mol, FileRead& fr) {
+	std::unique_ptr<Loader> loader(new Loader);
+
+	try {
+		// The header has been peeled away by the caller
+		uint8_t const packet_version = fr.unsigned_8();
+		if (packet_version == kCurrentPacketVersionInvasion) {
+			loader->init(egbase, mol, *new NavalInvasionBase);
+			loader->load(fr);
+		} else {
+			throw UnhandledVersionError(
+			   "NavalInvasionBase", packet_version, kCurrentPacketVersionInvasion);
+		}
+	} catch (const std::exception& e) {
+		throw wexception("loading naval invasion base: %s", e.what());
+	}
+
+	return loader.release();
+}
+
+void NavalInvasionBase::save(EditorGameBase& egbase, MapObjectSaver& mos, FileWrite& fw) {
+	fw.unsigned_8(HeaderNavalInvasionBase);
+	fw.unsigned_8(kCurrentPacketVersionInvasion);
+
+	Bob::save(egbase, mos, fw);
+
+	fw.unsigned_32(soldiers_.size());
+	for (const auto& soldier : soldiers_) {
+		fw.unsigned_32(mos.get_object_file_index(*soldier.get(egbase)));
+	}
+}
+
 }  // namespace Widelands
