@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,6 +43,7 @@
 #include "logic/game_data_error.h"
 #include "logic/map_objects/checkstep.h"
 #include "logic/map_objects/findimmovable.h"
+#include "logic/map_objects/pinned_note.h"
 #include "logic/map_objects/tribes/building.h"
 #include "logic/map_objects/tribes/constructionsite.h"
 #include "logic/map_objects/tribes/dismantlesite.h"
@@ -68,25 +69,55 @@ void terraform_for_building(Widelands::EditorGameBase& egbase,
 	const Widelands::Map& map = egbase.map();
 	Widelands::FCoords c[4];  //  Big buildings occupy 4 locations.
 	c[0] = map.get_fcoords(location);
-	map.get_brn(c[0], &c[1]);
-	if (Widelands::BaseImmovable* const immovable = c[0].field->get_immovable()) {
-		immovable->remove(egbase);
+
+	Widelands::DescriptionIndex const required_immovable_attr = descr->get_built_over_immovable();
+	Widelands::BaseImmovable* main_immovable = c[0].field->get_immovable();
+
+	// Remove blocking immovable
+	if (main_immovable != nullptr && (required_immovable_attr == Widelands::INVALID_INDEX ||
+	                                  !main_immovable->has_attribute(required_immovable_attr))) {
+		main_immovable->remove(egbase);
+		main_immovable = nullptr;
+	}
+
+	// Place required immovable if not present
+	if (required_immovable_attr != Widelands::INVALID_INDEX && main_immovable == nullptr) {
+		for (Widelands::DescriptionIndex i = 0; i < egbase.descriptions().nr_immovables(); ++i) {
+			const Widelands::ImmovableDescr* imdesc = egbase.descriptions().get_immovable_descr(i);
+			if (imdesc->has_attribute(required_immovable_attr)) {
+				main_immovable = &egbase.create_immovable(c[0], i, nullptr);
+				break;
+			}
+		}
+		if (main_immovable == nullptr) {
+			throw wexception("Attempting to build %s at %dx%d - cannot place any immovable with "
+			                 "required attribute %i",
+			                 descr->name().c_str(), location.x, location.y, required_immovable_attr);
+		}
 	}
 
 	size_t nr_locations = 1;
 	if ((descr->get_size() & Widelands::BUILDCAPS_SIZEMASK) == Widelands::BUILDCAPS_BIG) {
+		// Fortunately we don't have any big buildings that are built over immovables,
+		// so we can just clear the other fields.
+		assert(required_immovable_attr == Widelands::INVALID_INDEX);
 		nr_locations = 4;
 		map.get_trn(c[0], &c[1]);
 		map.get_tln(c[0], &c[2]);
 		map.get_ln(c[0], &c[3]);
 	}
+
 	for (size_t i = 0; i < nr_locations; ++i) {
 		//  Make sure that the player owns the area around.
 		egbase.conquer_area_no_building(Widelands::PlayerArea<Widelands::Area<Widelands::FCoords>>(
 		   player_number, Widelands::Area<Widelands::FCoords>(c[i], 1)));
 
-		if (Widelands::BaseImmovable* const immovable = c[i].field->get_immovable()) {
-			immovable->remove(egbase);
+		// We've just made sure that the main field is as the building needs it, so skip it.
+		// Only big buildings have more.
+		if (i > 0) {
+			if (Widelands::BaseImmovable* const immovable = c[i].field->get_immovable()) {
+				immovable->remove(egbase);
+			}
 		}
 	}
 }
@@ -140,27 +171,15 @@ Player::Player(EditorGameBase& the_egbase,
                const TribeDescr& tribe_descr,
                const std::string& name)
    : egbase_(the_egbase),
-     initialization_index_(initialization_index),
-     playercolor_(pc),
-     team_number_(0),
-     see_all_(false),
-     player_number_(plnum),
      tribe_(tribe_descr),
-     random_tribe_(false),
-     casualties_(0),
-     kills_(0),
-     msites_lost_(0),
-     msites_defeated_(0),
-     civil_blds_lost_(0),
-     civil_blds_defeated_(0),
-     ship_name_counter_(0),
-     fields_(nullptr),
-     is_picking_custom_starting_position_(false),
-     allow_additional_expedition_items_(true),
-     hidden_from_general_statistics_(false),
+
      message_fx_(SoundHandler::register_fx(SoundType::kMessage, "sound/message")),
      attack_fx_(SoundHandler::register_fx(SoundType::kMessage, "sound/military/under_attack")),
-     occupied_fx_(SoundHandler::register_fx(SoundType::kMessage, "sound/military/site_occupied")) {
+     occupied_fx_(SoundHandler::register_fx(SoundType::kMessage, "sound/military/site_occupied")),
+
+     playercolor_(pc),
+     player_number_(plnum),
+     initialization_index_(initialization_index) {
 	set_name(name);
 
 	init_statistics();
@@ -193,11 +212,13 @@ Player::Player(EditorGameBase& the_egbase,
 		   }
 	   });
 
-	// Populating remaining_shipnames vector
-	for (const std::string& shipname :
-	     egbase_.descriptions().get_ship_descr(tribe().ship())->get_ship_names()) {
-		remaining_shipnames_.push_back(shipname);
-	}
+	// Populating list of remaining ship/port names
+	const Widelands::ShipDescr& ship = *egbase_.descriptions().get_ship_descr(tribe().ship());
+	remaining_shipnames_.insert(
+	   remaining_shipnames_.begin(), ship.get_ship_names().begin(), ship.get_ship_names().end());
+	remaining_warehousenames_.insert(remaining_warehousenames_.begin(),
+	                                 tribe().get_warehouse_names().begin(),
+	                                 tribe().get_warehouse_names().end());
 
 	update_team_players();
 }
@@ -258,6 +279,7 @@ bool Player::pick_custom_starting_position(const Coords& c) {
 	if (!get_starting_position_suitability(c)) {
 		return false;
 	}
+	local_player_starting_position_is_pending_ = true;
 	dynamic_cast<Game&>(egbase()).send_player_command(
 	   new CmdPickCustomStartingPosition(egbase().get_gametime(), player_number(), c));
 	return true;
@@ -268,6 +290,7 @@ void Player::do_pick_custom_starting_position(const Coords& c) {
 		return;
 	}
 	is_picking_custom_starting_position_ = false;
+	local_player_starting_position_is_pending_ = false;
 	egbase().mutable_map()->set_starting_pos(player_number(), c);
 	create_default_infrastructure();
 }
@@ -331,6 +354,27 @@ void Player::set_team_number(TeamNumber team) {
 	update_team_players();
 }
 
+bool Player::may_approve_request(DiplomacyAction action, PlayerNumber from, PlayerNumber to) const {
+	if (from == player_number()) {
+		return false;
+	}
+	if (to == player_number()) {
+		return true;
+	}
+
+	switch (action) {
+	case DiplomacyAction::kInvite:
+		return false;  // Only the invited player may do that.
+
+	case DiplomacyAction::kJoin:
+		/* If we are on the same team as the recipient, we may decide this too. */
+		return team_number() != 0 && team_number() == egbase().player(to).team_number();
+
+	default:
+		NEVER_HERE();
+	}
+}
+
 /**
  * Returns whether this player and the given other player can attack
  * each other.
@@ -950,15 +994,6 @@ void Player::start_or_cancel_expedition(const Warehouse& wh) {
 	}
 }
 
-void Player::military_site_set_soldier_preference(PlayerImmovable& imm,
-                                                  SoldierPreference soldier_preference) {
-	if (imm.get_owner() == this) {
-		if (upcast(MilitarySite, milsite, &imm)) {
-			milsite->set_soldier_preference(soldier_preference);
-		}
-	}
-}
-
 /*
  * enhance this building, remove it, but give the constructionsite
  * an idea of enhancing
@@ -1126,6 +1161,8 @@ void Player::allow_building_type(DescriptionIndex const i, bool const allow) {
  * Economy stuff below
  */
 Economy* Player::create_economy(WareWorker type) {
+	MutexLock m(MutexLock::ID::kObjects);
+
 	std::unique_ptr<Economy> eco(new Economy(*this, type));
 	const Serial serial = eco->serial();
 
@@ -1138,6 +1175,8 @@ Economy* Player::create_economy(WareWorker type) {
 }
 
 Economy* Player::create_economy(Serial serial, WareWorker type) {
+	MutexLock m(MutexLock::ID::kObjects);
+
 	std::unique_ptr<Economy> eco(new Economy(*this, serial, type));
 
 	assert(economies_.count(serial) == 0);
@@ -1149,6 +1188,8 @@ Economy* Player::create_economy(Serial serial, WareWorker type) {
 }
 
 void Player::remove_economy(Serial serial) {
+	MutexLock m(MutexLock::ID::kObjects);
+
 	assert(has_economy(serial));
 	economies_.erase(economies_.find(serial));
 	assert(!has_economy(serial));
@@ -1159,6 +1200,8 @@ const std::map<Serial, std::unique_ptr<Economy>>& Player::economies() const {
 }
 
 Economy* Player::get_economy(Widelands::Serial serial) const {
+	MutexLock m(MutexLock::ID::kObjects);
+
 	if (economies_.count(serial) == 0) {
 		return nullptr;
 	}
@@ -1166,6 +1209,7 @@ Economy* Player::get_economy(Widelands::Serial serial) const {
 }
 
 bool Player::has_economy(Widelands::Serial serial) const {
+	MutexLock m(MutexLock::ID::kObjects);
 	return economies_.count(serial) != 0;
 }
 
@@ -1274,10 +1318,28 @@ void Player::enemyflagaction(const Flag& flag,
 			if (const AttackTarget* attack_target = building->attack_target()) {
 				if (attack_target->can_be_attacked()) {
 					attack_target->set_allow_conquer(player_number(), allow_conquer);
+					std::map<OPtr<Ship>, std::vector<uint32_t>> soldiers_on_warships;
+
 					for (Soldier* temp_attacker : soldiers) {
-						assert(temp_attacker);
+						assert(temp_attacker != nullptr);
 						assert(temp_attacker->get_owner() == this);
-						if (upcast(MilitarySite, ms, temp_attacker->get_location(egbase()))) {
+						if (temp_attacker->is_shipping()) {
+							if (Ship* ship = dynamic_cast<Ship*>(
+							       egbase().objects().get_object(temp_attacker->get_ship_serial()));
+							    ship != nullptr) {
+								std::vector<uint32_t>& parameters_vector = soldiers_on_warships[ship];
+								if (parameters_vector.empty()) {
+									parameters_vector.push_back(building->get_position().x);
+									parameters_vector.push_back(building->get_position().y);
+								}
+								parameters_vector.push_back(temp_attacker->serial());
+							} else {
+								verb_log_warn_time(egbase().get_gametime(),
+								                   "Player(%u)::enemyflagaction: Not sending soldier %u "
+								                   "because his warship has vanished\n",
+								                   player_number(), temp_attacker->serial());
+							}
+						} else if (upcast(MilitarySite, ms, temp_attacker->get_location(egbase()))) {
 							assert(ms->get_owner() == this);
 							ms->send_attacker(*temp_attacker, *building);
 						} else {
@@ -1285,11 +1347,15 @@ void Player::enemyflagaction(const Flag& flag,
 							// in the short delay between sending and executing a playercommand
 							verb_log_warn_time(egbase().get_gametime(),
 							                   "Player(%u)::enemyflagaction: Not sending soldier %u "
-							                   "because he left the "
-							                   "building\n",
+							                   "because he left the building\n",
 							                   player_number(), temp_attacker->serial());
 						}
 					}
+
+					for (auto& pair : soldiers_on_warships) {
+						pair.first.get(egbase())->warship_command(
+						   dynamic_cast<Game&>(egbase()), WarshipCommand::kAttack, pair.second);
+					}
 				}
 			}
 		}
@@ -1852,6 +1918,15 @@ bool Player::is_attack_forbidden(PlayerNumber who) const {
 	return forbid_attack_.find(who) != forbid_attack_.end();
 }
 
+void Player::register_pinned_note(PinnedNote* note) {
+	assert(pinned_notes_.count(note) == 0);
+	pinned_notes_.insert(note);
+}
+void Player::unregister_pinned_note(PinnedNote* note) {
+	assert(pinned_notes_.count(note) == 1);
+	pinned_notes_.erase(note);
+}
+
 void Player::add_soldier(unsigned h, unsigned a, unsigned d, unsigned e) {
 	SoldierStatistics ss(h, a, d, e);
 	auto it = std::find(soldier_stats_.begin(), soldier_stats_.end(), ss);
@@ -1873,6 +1948,13 @@ uint32_t Player::count_soldiers(unsigned h, unsigned a, unsigned d, unsigned e)
 	   std::find(soldier_stats_.begin(), soldier_stats_.end(), SoldierStatistics(h, a, d, e));
 	return it == soldier_stats_.end() ? 0 : it->total;
 }
+uint32_t Player::count_soldiers() const {
+	uint32_t s = 0;
+	for (const SoldierStatistics& ss : soldier_stats_) {
+		s += ss.total;
+	}
+	return s;
+}
 uint32_t Player::count_soldiers_h(unsigned value) const {
 	uint32_t s = 0;
 	for (const SoldierStatistics& ss : soldier_stats_) {
@@ -1931,24 +2013,121 @@ void Player::set_hidden_from_general_statistics(const bool hide) {
 /**
  * Pick random name from remaining names (if any)
  */
-const std::string Player::pick_shipname() {
+std::string Player::pick_shipname() {
 	++ship_name_counter_;
 
 	if (remaining_shipnames_.empty()) {
 		return format(pgettext("shipname", "Ship %d"), ship_name_counter_);
 	}
 
-	Game& game = dynamic_cast<Game&>(egbase());
-	const size_t index = game.logic_rand() % remaining_shipnames_.size();
-	auto it = remaining_shipnames_.begin();
-	std::advance(it, index);
-	std::string new_name = *it;
-	remaining_shipnames_.erase(it);
+	const size_t index =
+	   egbase().is_game() ?
+         (dynamic_cast<Game&>(egbase()).logic_rand() % remaining_shipnames_.size()) :
+         RNG::static_rand(remaining_shipnames_.size());
+	std::string new_name = remaining_shipnames_.at(index);
+	remaining_shipnames_.at(index) = remaining_shipnames_.back();
+	remaining_shipnames_.pop_back();
+	return new_name;
+}
+
+std::string Player::pick_warehousename(bool port) {
+	++warehouse_name_counter_;
+
+	if (remaining_warehousenames_.empty()) {
+		return format(port ? pgettext("warehouse", "Port %d") : pgettext("warehouse", "Warehouse %d"),
+		              warehouse_name_counter_);
+	}
+
+	const size_t index =
+	   egbase().is_game() ?
+         (dynamic_cast<Game&>(egbase()).logic_rand() % remaining_warehousenames_.size()) :
+         RNG::static_rand(remaining_warehousenames_.size());
+	std::string new_name = remaining_warehousenames_.at(index);
+	remaining_warehousenames_.at(index) = remaining_warehousenames_.back();
+	remaining_warehousenames_.pop_back();
 	return new_name;
 }
 
 /**
- * Read remaining ship indexes to the give file
+ * Remove name from remaining names (if present)
+ */
+void Player::reserve_shipname(const std::string& name) {
+	for (auto it = remaining_shipnames_.begin(); it != remaining_shipnames_.end(); ++it) {
+		if (*it == name) {
+			remaining_shipnames_.erase(it);
+			return;
+		}
+	}
+}
+
+void Player::reserve_warehousename(const std::string& name) {
+	for (auto it = remaining_warehousenames_.begin(); it != remaining_warehousenames_.end(); ++it) {
+		if (*it == name) {
+			remaining_warehousenames_.erase(it);
+			return;
+		}
+	}
+}
+
+void Player::set_shipnames(const std::set<std::string>& names) {
+	if (!names.empty()) {
+		remaining_shipnames_.clear();
+		remaining_shipnames_.insert(remaining_shipnames_.begin(), names.begin(), names.end());
+	}
+}
+
+void Player::set_warehousenames(const std::set<std::string>& names) {
+	if (!names.empty()) {
+		remaining_warehousenames_.clear();
+		remaining_warehousenames_.insert(
+		   remaining_warehousenames_.begin(), names.begin(), names.end());
+	}
+}
+
+DetectedPortSpace* Player::has_detected_port_space(const Coords& coords) {
+	for (auto& dps : detected_port_spaces_) {
+		if (dps->coords == coords) {
+			return dps.get();
+		}
+	}
+	return nullptr;
+}
+
+void Player::detect_port_space(std::unique_ptr<DetectedPortSpace> new_dps) {
+	detected_port_spaces_.emplace_back(std::move(new_dps));
+}
+
+const DetectedPortSpace& Player::get_detected_port_space(Serial serial) const {
+	for (const auto& dps : detected_port_spaces_) {
+		if (dps->serial == serial) {
+			return *dps;
+		}
+	}
+	throw wexception("Player %u has no detected port space with serial %u", player_number(), serial);
+}
+
+bool Player::remove_detected_port_space(const Coords& coords, PortDock* replaced_by) {
+	for (size_t i = 0; i < detected_port_spaces_.size(); ++i) {
+		if (detected_port_spaces_[i]->coords == coords) {
+			for (Serial serial : ships()) {
+				Ship* ship = dynamic_cast<Ship*>(egbase().objects().get_object(serial));
+				if (ship != nullptr &&
+				    ship->get_destination_detected_port_space() == detected_port_spaces_[i].get()) {
+					ship->set_destination(
+					   egbase(), replaced_by, ship->get_send_message_at_destination());
+				}
+			}
+
+			detected_port_spaces_[i] = std::move(detected_port_spaces_.back());
+			detected_port_spaces_.pop_back();
+			return true;
+		}
+	}
+	return false;
+}
+
+/**
+ * Read remaining ship names from the given file
  *
  * \param fr source stream
  */
@@ -1961,6 +2140,14 @@ void Player::read_remaining_shipnames(FileRead& fr) {
 	}
 	ship_name_counter_ = fr.unsigned_32();
 }
+void Player::read_remaining_warehousenames(FileRead& fr) {
+	remaining_warehousenames_.clear();
+	const uint16_t count = fr.unsigned_16();
+	for (uint16_t i = 0; i < count; ++i) {
+		remaining_warehousenames_.push_back(fr.string());
+	}
+	warehouse_name_counter_ = fr.unsigned_32();
+}
 
 void Player::init_statistics() {
 	current_produced_statistics_.clear();
@@ -1986,8 +2173,10 @@ void Player::init_statistics() {
  * Read statistics data from a file.
  *
  * \param fr source stream
+ * \param packet_version from GamePlayerInfoPacket in game_io/game_player_info_packet.cc
+ *                       currently unused, but may be needed for savegame compatibility
  */
-void Player::read_statistics(FileRead& fr, const uint16_t packet_version) {
+void Player::read_statistics(FileRead& fr, const uint16_t /* packet_version */) {
 	uint16_t nr_wares = fr.unsigned_16();
 	size_t nr_entries = fr.unsigned_16();
 	assert(tribe().wares().size() >= nr_wares);
@@ -2084,9 +2273,8 @@ void Player::read_statistics(FileRead& fr, const uint16_t packet_version) {
 	}
 
 	// Read worker stock statistics
-	// TODO(Nordfriese): Savegame compatibility
-	uint16_t nr_workers = packet_version >= 29 ? fr.unsigned_16() : 0;
-	nr_entries = packet_version >= 29 ? fr.unsigned_16() : 0;
+	uint16_t nr_workers = fr.unsigned_16();
+	nr_entries = fr.unsigned_16();
 	assert(tribe().workers().size() >= nr_workers);
 
 	for (DescriptionIndex idx : tribe().workers()) {
@@ -2116,7 +2304,7 @@ void Player::read_statistics(FileRead& fr, const uint16_t packet_version) {
 }
 
 /**
- * Write remaining ship indexes to the given file
+ * Write remaining ship names to the given file
  */
 void Player::write_remaining_shipnames(FileWrite& fw) const {
 	fw.unsigned_16(remaining_shipnames_.size());
@@ -2125,6 +2313,13 @@ void Player::write_remaining_shipnames(FileWrite& fw) const {
 	}
 	fw.unsigned_32(ship_name_counter_);
 }
+void Player::write_remaining_warehousenames(FileWrite& fw) const {
+	fw.unsigned_16(remaining_warehousenames_.size());
+	for (const auto& warehousename : remaining_warehousenames_) {
+		fw.string(warehousename);
+	}
+	fw.unsigned_32(warehouse_name_counter_);
+}
 
 /**
  * Write statistics data to the given file
@@ -2188,5 +2383,45 @@ void Player::write_statistics(FileWrite& fw) const {
 	}
 }
 
-constexpr Time Player::AiPersistentState::kNoExpedition;
+std::pair<std::set<std::string>, std::set<std::string>> read_custom_warehouse_ship_names() {
+	std::pair<std::set<std::string>, std::set<std::string>> result;
+
+	const std::string* filenames[2] = {&kCustomShipNamesFile, &kCustomWarehouseNamesFile};
+	std::set<std::string>* result_sets[2] = {&result.first, &result.second};
+
+	for (int i = 0; i < 2; ++i) {  // To deduplicate this a bit
+		if (FileRead fr; fr.try_open(*g_fs, *filenames[i])) {
+			for (;;) {
+				const char* line = nullptr;
+				try {
+					line = fr.read_line();
+				} catch (const std::exception& e) {
+					log_warn(
+					   "Naming list in '%s' file is malformed: %s", filenames[i]->c_str(), e.what());
+					break;
+				}
+				if (line == nullptr) {
+					break;  // End of file
+				}
+
+				std::string name(line);
+
+				// Cleanup and sanitizing
+				constexpr const char kDoubleWhitespace[] = "  ";
+				replace_all(name, "\t", " ");
+				trim(name);
+				while (contains(name, kDoubleWhitespace)) {
+					replace_all(name, kDoubleWhitespace, " ");
+				}
+
+				if (!name.empty()) {
+					result_sets[i]->insert(name);
+				}
+			}
+		}
+	}
+
+	return result;
+}
+
 }  // namespace Widelands
