@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2022 by the Widelands Development Team
+ * Copyright (C) 2004-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,13 +25,17 @@
 #include "base/wexception.h"
 #include "economy/economy.h"
 #include "economy/expedition_bootstrap.h"
+#include "economy/ferry_fleet.h"
 #include "economy/input_queue.h"
+#include "economy/ship_fleet.h"
 #include "io/fileread.h"
 #include "io/filewrite.h"
 #include "io/streamwrite.h"
 #include "logic/game.h"
 #include "logic/game_controller.h"
+#include "logic/map_objects/pinned_note.h"
 #include "logic/map_objects/tribes/market.h"
+#include "logic/map_objects/tribes/militarysite.h"
 #include "logic/map_objects/tribes/soldier.h"
 #include "logic/map_objects/tribes/tribe_descr.h"
 #include "logic/player.h"
@@ -39,6 +43,7 @@
 #include "logic/widelands_geometry_io.h"
 #include "map_io/map_object_loader.h"
 #include "map_io/map_object_saver.h"
+#include "wui/interactive_spectator.h"
 
 namespace Widelands {
 
@@ -106,6 +111,8 @@ PlayerCommand* PlayerCommand::deserialize(StreamRead& des) {
 		return new CmdFlagAction(des);
 	case QueueCommandTypes::kStartStopBuilding:
 		return new CmdStartStopBuilding(des);
+	case QueueCommandTypes::kToggleInfiniteProduction:
+		return new CmdToggleInfiniteProduction(des);
 	case QueueCommandTypes::kEnhanceBuilding:
 		return new CmdEnhanceBuilding(des);
 
@@ -138,8 +145,8 @@ PlayerCommand* PlayerCommand::deserialize(StreamRead& des) {
 		return new CmdDismantleBuilding(des);
 	case QueueCommandTypes::kEvictWorker:
 		return new CmdEvictWorker(des);
-	case QueueCommandTypes::kMilitarysiteSetSoldierPreference:
-		return new CmdMilitarySiteSetSoldierPreference(des);
+	case QueueCommandTypes::kSetSoldierPreference:
+		return new CmdSetSoldierPreference(des);
 	case QueueCommandTypes::kToggleMuteMessages:
 		return new CmdToggleMuteMessages(des);
 
@@ -149,6 +156,8 @@ PlayerCommand* PlayerCommand::deserialize(StreamRead& des) {
 		return new CmdShipScoutDirection(des);
 	case QueueCommandTypes::kShipExploreIsland:
 		return new CmdShipExploreIsland(des);
+	case QueueCommandTypes::kShipSetDestination:
+		return new CmdShipSetDestination(des);
 	case QueueCommandTypes::kShipConstructPort:
 		return new CmdShipConstructPort(des);
 	case QueueCommandTypes::kShipSink:
@@ -163,6 +172,16 @@ PlayerCommand* PlayerCommand::deserialize(StreamRead& des) {
 		return new CmdMarkMapObjectForRemoval(des);
 	case QueueCommandTypes::kDiplomacy:
 		return new CmdDiplomacy(des);
+	case QueueCommandTypes::kPinnedNote:
+		return new CmdPinnedNote(des);
+	case QueueCommandTypes::kShipPortName:
+		return new CmdShipPortName(des);
+	case QueueCommandTypes::kFleetTargets:
+		return new CmdFleetTargets(des);
+	case QueueCommandTypes::kShipRefit:
+		return new CmdShipRefit(des);
+	case QueueCommandTypes::kWarshipCommand:
+		return new CmdWarshipCommand(des);
 
 	default:
 		throw wexception("PlayerCommand::deserialize(): Encountered invalid command id: %d",
@@ -519,18 +538,15 @@ void CmdFlagAction::serialize(StreamWrite& ser) {
 	ser.unsigned_32(serial_);
 }
 
-constexpr uint16_t kCurrentPacketVersionCmdFlagAction = 3;
+constexpr uint16_t kCurrentPacketVersionCmdFlagAction = 3;  // since v1.0
 
 void CmdFlagAction::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mol) {
 	try {
 		const uint16_t packet_version = fr.unsigned_16();
-		// TODO(Nordfriese): Savegame compatibility
-		if (packet_version >= 2 && packet_version <= kCurrentPacketVersionCmdFlagAction) {
+		if (packet_version >= 3 && packet_version <= kCurrentPacketVersionCmdFlagAction) {
 			PlayerCommand::read(fr, egbase, mol);
 			serial_ = get_object_serial_or_zero<Flag>(fr.unsigned_32(), mol);
-			// TODO(Nordfriese): Savegame compatibility
-			type_ = packet_version < 3 ? FlagJob::Type::kGeologist :
-                                      static_cast<FlagJob::Type>(fr.unsigned_8());
+			type_ = static_cast<FlagJob::Type>(fr.unsigned_8());
 		} else {
 			throw UnhandledVersionError(
 			   "CmdFlagAction", packet_version, kCurrentPacketVersionCmdFlagAction);
@@ -598,34 +614,82 @@ void CmdStartStopBuilding::write(FileWrite& fw, EditorGameBase& egbase, MapObjec
 	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(serial)));
 }
 
-CmdMilitarySiteSetSoldierPreference::CmdMilitarySiteSetSoldierPreference(StreamRead& des)
+/*** Cmd_ToggleInfiniteProduction ***/
+
+CmdToggleInfiniteProduction::CmdToggleInfiniteProduction(StreamRead& des)
+   : PlayerCommand(Time(0), des.unsigned_8()) {
+	serial_ = des.unsigned_32();
+}
+
+void CmdToggleInfiniteProduction::execute(Game& game) {
+	if (upcast(ProductionSite, ps, game.objects().get_object(serial_))) {
+		ps->set_infinite_production(!ps->infinite_production());
+	}
+}
+
+void CmdToggleInfiniteProduction::serialize(StreamWrite& ser) {
+	write_id_and_sender(ser);
+	ser.unsigned_32(serial_);
+}
+
+constexpr uint16_t kCurrentPacketVersionCmdToggleInfiniteProduction = 1;
+
+void CmdToggleInfiniteProduction::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mol) {
+	try {
+		const uint16_t packet_version = fr.unsigned_16();
+		if (packet_version == kCurrentPacketVersionCmdToggleInfiniteProduction) {
+			PlayerCommand::read(fr, egbase, mol);
+			serial_ = get_object_serial_or_zero<Building>(fr.unsigned_32(), mol);
+		} else {
+			throw UnhandledVersionError("CmdToggleInfiniteProduction", packet_version,
+			                            kCurrentPacketVersionCmdToggleInfiniteProduction);
+		}
+	} catch (const WException& e) {
+		throw GameDataError("toggle infinite production: %s", e.what());
+	}
+}
+void CmdToggleInfiniteProduction::write(FileWrite& fw,
+                                        EditorGameBase& egbase,
+                                        MapObjectSaver& mos) {
+	fw.unsigned_16(kCurrentPacketVersionCmdToggleInfiniteProduction);
+	PlayerCommand::write(fw, egbase, mos);
+	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(serial_)));
+}
+
+/*** Cmd_SetSoldierPreference ***/
+
+CmdSetSoldierPreference::CmdSetSoldierPreference(StreamRead& des)
    : PlayerCommand(Time(0), des.unsigned_8()) {
 	serial = des.unsigned_32();
 	preference = static_cast<Widelands::SoldierPreference>(des.unsigned_8());
 }
 
-void CmdMilitarySiteSetSoldierPreference::serialize(StreamWrite& ser) {
+void CmdSetSoldierPreference::serialize(StreamWrite& ser) {
 	write_id_and_sender(ser);
 	ser.unsigned_32(serial);
 	ser.unsigned_8(static_cast<uint8_t>(preference));
 }
 
-void CmdMilitarySiteSetSoldierPreference::execute(Game& game) {
+void CmdSetSoldierPreference::execute(Game& game) {
 	MapObject* mo = game.objects().get_object(serial);
 	if (upcast(ConstructionSite, cs, mo)) {
-		if (upcast(MilitarysiteSettings, s, cs->get_settings())) {
-			s->prefer_heroes = preference == SoldierPreference::kHeroes;
+		if (upcast(MilitarysiteSettings, ms, cs->get_settings())) {
+			ms->soldier_preference = preference;
+		} else if (upcast(WarehouseSettings, wh, cs->get_settings())) {
+			wh->soldier_preference = preference;
 		}
-	} else if (upcast(MilitarySite, building, mo)) {
-		game.get_player(sender())->military_site_set_soldier_preference(*building, preference);
+	} else if (upcast(MilitarySite, ms, mo)) {
+		ms->set_soldier_preference(preference);
+	} else if (upcast(Warehouse, wh, mo)) {
+		wh->set_soldier_preference(preference);
+	} else if (upcast(Ship, ship, mo)) {
+		ship->set_soldier_preference(preference);
 	}
 }
 
 constexpr uint16_t kCurrentPacketVersionSoldierPreference = 1;
 
-void CmdMilitarySiteSetSoldierPreference::write(FileWrite& fw,
-                                                EditorGameBase& egbase,
-                                                MapObjectSaver& mos) {
+void CmdSetSoldierPreference::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver& mos) {
 	// First, write version
 	fw.unsigned_16(kCurrentPacketVersionSoldierPreference);
 	// Write base classes
@@ -637,21 +701,19 @@ void CmdMilitarySiteSetSoldierPreference::write(FileWrite& fw,
 	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(serial)));
 }
 
-void CmdMilitarySiteSetSoldierPreference::read(FileRead& fr,
-                                               EditorGameBase& egbase,
-                                               MapObjectLoader& mol) {
+void CmdSetSoldierPreference::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mol) {
 	try {
 		const uint16_t packet_version = fr.unsigned_16();
 		if (packet_version == kCurrentPacketVersionSoldierPreference) {
 			PlayerCommand::read(fr, egbase, mol);
 			preference = static_cast<Widelands::SoldierPreference>(fr.unsigned_8());
-			serial = get_object_serial_or_zero<MilitarySite>(fr.unsigned_32(), mol);
+			serial = get_object_serial_or_zero<MapObject>(fr.unsigned_32(), mol);
 		} else {
-			throw UnhandledVersionError("CmdMilitarySiteSetSoldierPreference", packet_version,
-			                            kCurrentPacketVersionSoldierPreference);
+			throw UnhandledVersionError(
+			   "CmdSetSoldierPreference", packet_version, kCurrentPacketVersionSoldierPreference);
 		}
 	} catch (const WException& e) {
-		throw GameDataError("start/stop building: %s", e.what());
+		throw GameDataError("cmd soldier preference: %s", e.what());
 	}
 }
 
@@ -691,7 +753,7 @@ void CmdStartOrCancelExpedition::read(FileRead& fr, EditorGameBase& egbase, MapO
 			   "CmdStartOrCancelExpedition", packet_version, kCurrentPacketVersionExpedition);
 		}
 	} catch (const WException& e) {
-		throw GameDataError("start/stop building: %s", e.what());
+		throw GameDataError("cmd start/cancel expedition: %s", e.what());
 	}
 }
 void CmdStartOrCancelExpedition::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver& mos) {
@@ -901,6 +963,118 @@ void CmdEvictWorker::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver
 	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(serial)));
 }
 
+/*** Cmd_ShipRefit ***/
+CmdShipRefit::CmdShipRefit(StreamRead& des) : PlayerCommand(Time(0), des.unsigned_8()) {
+	serial_ = des.unsigned_32();
+	type_ = static_cast<ShipType>(des.unsigned_8());
+}
+
+void CmdShipRefit::execute(Game& game) {
+	if (!game.naval_warfare_allowed()) {
+		log_warn("Received a refit command but naval warfare is disabled, ignoring.");
+		return;
+	}
+
+	upcast(Ship, ship, game.objects().get_object(serial_));
+	if (ship != nullptr && ship->get_owner()->player_number() == sender()) {
+		ship->refit(game, type_);
+	}
+}
+
+void CmdShipRefit::serialize(StreamWrite& ser) {
+	write_id_and_sender(ser);
+	ser.unsigned_32(serial_);
+	ser.unsigned_8(static_cast<uint8_t>(type_));
+}
+
+constexpr uint16_t kCurrentPacketVersionShipRefit = 1;
+
+void CmdShipRefit::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mol) {
+	try {
+		const uint16_t packet_version = fr.unsigned_16();
+		if (packet_version == kCurrentPacketVersionShipRefit) {
+			PlayerCommand::read(fr, egbase, mol);
+			serial_ = get_object_serial_or_zero<Ship>(fr.unsigned_32(), mol);
+			type_ = static_cast<ShipType>(fr.unsigned_8());
+		} else {
+			throw UnhandledVersionError(
+			   "CmdShipRefit", packet_version, kCurrentPacketVersionShipRefit);
+		}
+	} catch (const WException& e) {
+		throw GameDataError("Ship refit: %s", e.what());
+	}
+}
+void CmdShipRefit::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver& mos) {
+	fw.unsigned_16(kCurrentPacketVersionShipRefit);
+	PlayerCommand::write(fw, egbase, mos);
+
+	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(serial_)));
+	fw.unsigned_8(static_cast<uint8_t>(type_));
+}
+
+/*** Cmd_WarshipCommand ***/
+CmdWarshipCommand::CmdWarshipCommand(StreamRead& des) : PlayerCommand(Time(0), des.unsigned_8()) {
+	serial_ = des.unsigned_32();
+	cmd_ = static_cast<WarshipCommand>(des.unsigned_8());
+	for (uint32_t i = des.unsigned_32(); i > 0U; --i) {
+		parameters_.push_back(des.signed_32());
+	}
+}
+
+void CmdWarshipCommand::execute(Game& game) {
+	if (!game.naval_warfare_allowed()) {
+		log_warn("Received a warship command but naval warfare is disabled, ignoring.");
+		return;
+	}
+
+	upcast(Ship, ship, game.objects().get_object(serial_));
+	if (ship != nullptr && ship->get_owner()->player_number() == sender()) {
+		ship->warship_command(game, cmd_, parameters_);
+	}
+}
+
+void CmdWarshipCommand::serialize(StreamWrite& ser) {
+	write_id_and_sender(ser);
+	ser.unsigned_32(serial_);
+	ser.unsigned_8(static_cast<uint8_t>(cmd_));
+	ser.unsigned_32(parameters_.size());
+	for (uint32_t p : parameters_) {
+		ser.unsigned_32(p);
+	}
+}
+
+constexpr uint16_t kCurrentPacketVersionWarshipCommand = 1;
+
+void CmdWarshipCommand::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mol) {
+	try {
+		const uint16_t packet_version = fr.unsigned_16();
+		if (packet_version == kCurrentPacketVersionWarshipCommand) {
+			PlayerCommand::read(fr, egbase, mol);
+			serial_ = get_object_serial_or_zero<Ship>(fr.unsigned_32(), mol);
+			cmd_ = static_cast<WarshipCommand>(fr.unsigned_8());
+			for (uint32_t i = fr.unsigned_32(); i > 0U; --i) {
+				parameters_.push_back(fr.signed_32());
+			}
+		} else {
+			throw UnhandledVersionError(
+			   "CmdWarshipCommand", packet_version, kCurrentPacketVersionWarshipCommand);
+		}
+	} catch (const WException& e) {
+		throw GameDataError("Warship command: %s", e.what());
+	}
+}
+void CmdWarshipCommand::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver& mos) {
+	fw.unsigned_16(kCurrentPacketVersionWarshipCommand);
+	PlayerCommand::write(fw, egbase, mos);
+
+	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(serial_)));
+	fw.unsigned_8(static_cast<uint8_t>(cmd_));
+	fw.unsigned_32(parameters_.size());
+	for (uint32_t p : parameters_) {
+		fw.unsigned_32(p);
+	}
+}
+
 /*** Cmd_ShipScoutDirection ***/
 CmdShipScoutDirection::CmdShipScoutDirection(StreamRead& des)
    : PlayerCommand(Time(0), des.unsigned_8()) {
@@ -910,10 +1084,10 @@ CmdShipScoutDirection::CmdShipScoutDirection(StreamRead& des)
 
 void CmdShipScoutDirection::execute(Game& game) {
 	upcast(Ship, ship, game.objects().get_object(serial));
-	if ((ship != nullptr) && ship->get_owner()->player_number() == sender()) {
-		if (!(ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionWaiting ||
-		      ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionPortspaceFound ||
-		      ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionScouting)) {
+	if (ship != nullptr && ship->get_owner()->player_number() == sender()) {
+		if (!(ship->get_ship_state() == Widelands::ShipStates::kExpeditionWaiting ||
+		      ship->get_ship_state() == Widelands::ShipStates::kExpeditionPortspaceFound ||
+		      ship->get_ship_state() == Widelands::ShipStates::kExpeditionScouting)) {
 			log_warn_time(
 			   game.get_gametime(),
 			   " %1d:ship on %3dx%3d received scout command but not in "
@@ -973,8 +1147,8 @@ CmdShipConstructPort::CmdShipConstructPort(StreamRead& des)
 
 void CmdShipConstructPort::execute(Game& game) {
 	upcast(Ship, ship, game.objects().get_object(serial));
-	if ((ship != nullptr) && ship->get_owner()->player_number() == sender()) {
-		if (ship->get_ship_state() != Widelands::Ship::ShipStates::kExpeditionPortspaceFound) {
+	if (ship != nullptr && ship->get_owner()->player_number() == sender()) {
+		if (ship->get_ship_state() != Widelands::ShipStates::kExpeditionPortspaceFound) {
 			log_warn_time(game.get_gametime(),
 			              " %1d:ship on %3dx%3d received build port command but "
 			              "not in kExpeditionPortspaceFound status (expedition: %s), ignoring...\n",
@@ -1032,10 +1206,10 @@ CmdShipExploreIsland::CmdShipExploreIsland(StreamRead& des)
 
 void CmdShipExploreIsland::execute(Game& game) {
 	upcast(Ship, ship, game.objects().get_object(serial));
-	if ((ship != nullptr) && ship->get_owner()->player_number() == sender()) {
-		if (!(ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionWaiting ||
-		      ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionPortspaceFound ||
-		      ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionScouting)) {
+	if (ship != nullptr && ship->get_owner()->player_number() == sender()) {
+		if (!(ship->get_ship_state() == Widelands::ShipStates::kExpeditionWaiting ||
+		      ship->get_ship_state() == Widelands::ShipStates::kExpeditionPortspaceFound ||
+		      ship->get_ship_state() == Widelands::ShipStates::kExpeditionScouting)) {
 			log_warn_time(
 			   game.get_gametime(),
 			   " %1d:ship on %3dx%3d received explore island command "
@@ -1085,6 +1259,62 @@ void CmdShipExploreIsland::write(FileWrite& fw, EditorGameBase& egbase, MapObjec
 	fw.unsigned_8(static_cast<uint8_t>(island_explore_direction));
 }
 
+/*** Cmd_ShipSetDestination ***/
+CmdShipSetDestination::CmdShipSetDestination(StreamRead& des)
+   : PlayerCommand(Time(0), des.unsigned_8()) {
+	serial_ = des.unsigned_32();
+	destination_object_ = des.unsigned_32();
+	destination_coords_ = des.unsigned_32();
+}
+
+void CmdShipSetDestination::execute(Game& game) {
+	upcast(Ship, ship, game.objects().get_object(serial_));
+	if (ship != nullptr && ship->get_owner()->player_number() == sender()) {
+		if (destination_coords_ != 0U) {
+			ship->set_destination(
+			   game, ship->owner().get_detected_port_space(destination_coords_), true);
+		} else {
+			ship->set_destination(game, game.objects().get_object(destination_object_), true);
+		}
+	}
+}
+
+void CmdShipSetDestination::serialize(StreamWrite& ser) {
+	write_id_and_sender(ser);
+	ser.unsigned_32(serial_);
+	ser.unsigned_32(destination_object_);
+	ser.unsigned_32(destination_coords_);
+}
+
+constexpr uint16_t kCurrentPacketVersionShipSetDestination = 2;
+
+void CmdShipSetDestination::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mol) {
+	try {
+		const uint16_t packet_version = fr.unsigned_16();
+		// TODO(Nordfriese): Savegame compatibility v1.1
+		if (packet_version >= 1 && packet_version <= kCurrentPacketVersionShipSetDestination) {
+			PlayerCommand::read(fr, egbase, mol);
+			serial_ = get_object_serial_or_zero<Ship>(fr.unsigned_32(), mol);
+			destination_object_ = get_object_serial_or_zero<MapObject>(fr.unsigned_32(), mol);
+			destination_coords_ = packet_version >= 2 ? fr.unsigned_32() : 0;
+		} else {
+			throw UnhandledVersionError(
+			   "CmdShipSetDestination", packet_version, kCurrentPacketVersionShipSetDestination);
+		}
+	} catch (const WException& e) {
+		throw GameDataError("Ship set destination: %s", e.what());
+	}
+}
+void CmdShipSetDestination::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver& mos) {
+	fw.unsigned_16(kCurrentPacketVersionShipSetDestination);
+	PlayerCommand::write(fw, egbase, mos);
+
+	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(serial_)));
+	fw.unsigned_32(
+	   mos.get_object_file_index_or_zero(egbase.objects().get_object(destination_object_)));
+	fw.unsigned_32(destination_coords_);
+}
+
 /*** Cmd_ShipSink ***/
 CmdShipSink::CmdShipSink(StreamRead& des) : PlayerCommand(Time(0), des.unsigned_8()) {
 	serial = des.unsigned_32();
@@ -1307,18 +1537,12 @@ void CmdSetInputMaxFill::execute(Game& game) {
 		}
 	} else if (upcast(Building, b, mo)) {
 		if (b->owner().player_number() == sender()) {
-			try {
-				b->inputqueue(index_, type_, nullptr).set_max_fill(max_fill_);
-				if (upcast(Warehouse, wh, b)) {
-					if (PortDock* p = wh->get_portdock()) {
-						// Update in case the expedition was ready previously and now lacks a ware again
-						p->expedition_bootstrap()->check_is_ready(game);
-					}
+			b->inputqueue(index_, type_, nullptr).set_max_fill(max_fill_);
+			if (upcast(Warehouse, wh, b)) {
+				if (PortDock* p = wh->get_portdock()) {
+					// Update in case the expedition was ready previously and now lacks a ware again
+					p->expedition_bootstrap()->check_is_ready(game);
 				}
-			} catch (const std::exception& e) {
-				// TODO(matthiakl): This exception is only caught to ensure b21 savegame compatibility
-				// and should be removed after v1.0
-				log_err("Skipped CmdSetInputMaxFill command: %s", e.what());
 			}
 		}
 	}
@@ -1588,6 +1812,8 @@ void CmdDropSoldier::execute(Game& game) {
 		if (upcast(Soldier, s, game.objects().get_object(soldier))) {
 			game.get_player(sender())->drop_soldier(*player_imm, *s);
 		}
+	} else if (upcast(Ship, ship, game.objects().get_object(serial))) {
+		ship->drop_soldier(game, soldier);
 	}
 }
 
@@ -1604,7 +1830,7 @@ void CmdDropSoldier::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader&
 		const uint16_t packet_version = fr.unsigned_16();
 		if (packet_version == kCurrentPacketVersionCmdDropSoldier) {
 			PlayerCommand::read(fr, egbase, mol);
-			serial = get_object_serial_or_zero<PlayerImmovable>(fr.unsigned_32(), mol);
+			serial = get_object_serial_or_zero<MapObject>(fr.unsigned_32(), mol);
 			soldier = get_object_serial_or_zero<Soldier>(fr.unsigned_32(), mol);
 		} else {
 			throw UnhandledVersionError(
@@ -1645,6 +1871,8 @@ void CmdChangeSoldierCapacity::execute(Game& game) {
 			ms->desired_capacity = std::max(1, std::min<int32_t>(ms->max_capacity, val));
 		} else if (upcast(TrainingsiteSettings, ts, cs->get_settings())) {
 			ts->desired_capacity = std::max(0, std::min<int32_t>(ts->max_capacity, val));
+		} else if (upcast(WarehouseSettings, ws, cs->get_settings())) {
+			ws->desired_capacity = std::max(0, std::min<int32_t>(ws->max_garrison, val));
 		}
 	} else if (upcast(Building, building, mo)) {
 		if (building->get_owner() == game.get_player(sender()) &&
@@ -1753,40 +1981,21 @@ void CmdEnemyFlagAction::serialize(StreamWrite& ser) {
 	ser.unsigned_8(allow_conquer_ ? 1 : 0);
 }
 
-constexpr uint16_t kCurrentPacketVersionCmdEnemyFlagAction = 5;
+constexpr uint16_t kCurrentPacketVersionCmdEnemyFlagAction = 5;  // since v1.0
 
 void CmdEnemyFlagAction::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mol) {
 	try {
 		const uint16_t packet_version = fr.unsigned_16();
-		// TODO(Nordfriese): Savegame compatibility
-		if (packet_version <= kCurrentPacketVersionCmdEnemyFlagAction && packet_version >= 3) {
+		if (packet_version <= kCurrentPacketVersionCmdEnemyFlagAction && packet_version >= 5) {
 			PlayerCommand::read(fr, egbase, mol);
-			if (packet_version < 5) {
-				fr.unsigned_8();
-			}
 			serial_ = get_object_serial_or_zero<Flag>(fr.unsigned_32(), mol);
-			if (packet_version < 5) {
-				fr.unsigned_8();
-			}
 
 			soldiers_.clear();
-			if (packet_version == kCurrentPacketVersionCmdEnemyFlagAction) {
-				const uint32_t number = fr.unsigned_32();
-				for (uint32_t i = 0; i < number; ++i) {
-					soldiers_.push_back(mol.get<Soldier>(fr.unsigned_32()).serial());
-				}
-			} else {
-				const uint8_t number = fr.unsigned_8();
-				upcast(Flag, flag, egbase.objects().get_object(serial_));
-				assert(flag);
-				std::vector<Soldier*> result;
-				egbase.get_player(sender())->find_attack_soldiers(*flag, &result, number);
-				assert(result.size() == number);
-				for (const auto& s : result) {
-					soldiers_.push_back(s->serial());
-				}
+			const uint32_t number = fr.unsigned_32();
+			for (uint32_t i = 0; i < number; ++i) {
+				soldiers_.push_back(mol.get<Soldier>(fr.unsigned_32()).serial());
 			}
-			allow_conquer_ = packet_version < 5 || (fr.unsigned_8() != 0u);
+			allow_conquer_ = (fr.unsigned_8() != 0u);
 		} else {
 			throw UnhandledVersionError(
 			   "CmdEnemyFlagAction", packet_version, kCurrentPacketVersionCmdEnemyFlagAction);
@@ -1828,8 +2037,8 @@ void PlayerMessageCommand::read(FileRead& fr, EditorGameBase& egbase, MapObjectL
 		if (packet_version == kCurrentPacketVersionPlayerMessageCommand) {
 			PlayerCommand::read(fr, egbase, mol);
 			message_id_ = MessageId(fr.unsigned_32());
-			if (!message_id_) {
-				throw GameDataError("(player %u): message id is null", sender());
+			if (!static_cast<bool>(message_id_)) {
+				verb_log_warn("PlayerMessageCommand (player %u): message ID is null", sender());
 			}
 		} else {
 			throw UnhandledVersionError(
@@ -2148,15 +2357,16 @@ void CmdDiplomacy::execute(Game& game) {
 			                             Message::Type::kScenario, game.get_gametime(), _("Diplomacy"),
 			                             "images/players/team.png", heading, text)));
 		}
+		if (upcast(InteractiveSpectator, is, game.get_ibase())) {
+			is->log_message(heading, text);
+		}
 	};
 
 	switch (action_) {
 	case DiplomacyAction::kResign:
-		for (const auto& status : game.player_manager()->get_players_end_status()) {
-			if (status.player == sender()) {
-				// Ignore if the player lost a moment ago
-				return;
-			}
+		if (game.player_manager()->get_player_end_status(sender()) != nullptr) {
+			// Ignore if the player lost a moment ago
+			return;
 		}
 		broadcast_message(_("Player Resigned"), format(_("%s has resigned and is now a spectator."),
 		                                               sending_player.get_name()));
@@ -2192,9 +2402,21 @@ void CmdDiplomacy::execute(Game& game) {
 
 	case DiplomacyAction::kAcceptJoin:
 	case DiplomacyAction::kRefuseJoin:
+	case DiplomacyAction::kRetractJoin:
 	case DiplomacyAction::kAcceptInvite:
-	case DiplomacyAction::kRefuseInvite: {
-		assert(other_player_ != sender());
+	case DiplomacyAction::kRefuseInvite:
+	case DiplomacyAction::kRetractInvite: {
+		PlayerNumber cmd_sender = sender();
+		bool retract = false;
+		if (action_ == DiplomacyAction::kRetractJoin || action_ == DiplomacyAction::kRetractInvite) {
+			/* Retracting is treated just like rejection but with a different message. */
+			std::swap(other_player_, cmd_sender);
+			retract = true;
+			action_ = (action_ == DiplomacyAction::kRetractJoin) ? DiplomacyAction::kRefuseJoin :
+                                                                DiplomacyAction::kRefuseInvite;
+		}
+		assert(other_player_ != cmd_sender);
+
 		const DiplomacyAction original_action =
 		   (action_ == DiplomacyAction::kAcceptJoin || action_ == DiplomacyAction::kRefuseJoin) ?
             DiplomacyAction::kJoin :
@@ -2204,24 +2426,39 @@ void CmdDiplomacy::execute(Game& game) {
 			// Note that in the response the numbers of the two players
 			// are swapped compared to the original message.
 			if (it->action == original_action && it->sender == other_player_ &&
-			    it->other == sender()) {
+			    it->other == cmd_sender) {
 				const bool accept =
 				   action_ == DiplomacyAction::kAcceptJoin || action_ == DiplomacyAction::kAcceptInvite;
+				std::string fmt_message;
+				if (accept) {
+					if (original_action == DiplomacyAction::kJoin) {
+						fmt_message = _("%1$s has accepted %2$s into their team.");
+					} else {
+						fmt_message = _("%1$s has accepted the invitation to join the team of %2$s.");
+					}
+				} else {
+					if (original_action == DiplomacyAction::kJoin) {
+						if (retract) {
+							fmt_message = _("%1$s has retracted the request to join the team of %2$s.");
+						} else {
+							fmt_message = _("%1$s has denied %2$s membership in their team.");
+						}
+					} else if (retract) {
+						fmt_message = _("%1$s has retracted the invitation to %2$s to join their team.");
+					} else {
+						fmt_message = _("%1$s has rejected the invitation to join the team of %2$s.");
+					}
+				}
 				broadcast_message(
 				   accept ? _("Team Change Accepted") : _("Team Change Rejected"),
-				   format(accept ? original_action == DiplomacyAction::kJoin ?
-                               _("%1$s has accepted %2$s into their team.") :
-                               _("%1$s has accepted the invitation to join the team of %2$s.") :
-				          original_action == DiplomacyAction::kJoin ?
-                               _("%1$s has denied %2$s membership in their team.") :
-                               _("%1$s has rejected the invitation to join the team of %2$s."),
-				          sending_player.get_name(), game.get_safe_player(other_player_)->get_name()));
+				   format(fmt_message, sending_player.get_name(),
+				          game.get_safe_player(retract ? cmd_sender : other_player_)->get_name()));
 
 				if (accept) {
 					Player* joiner = game.get_safe_player(
-					   original_action == DiplomacyAction::kJoin ? other_player_ : sender());
+					   original_action == DiplomacyAction::kJoin ? other_player_ : cmd_sender);
 					Player* other = game.get_safe_player(
-					   original_action != DiplomacyAction::kJoin ? other_player_ : sender());
+					   original_action != DiplomacyAction::kJoin ? other_player_ : cmd_sender);
 					if (other->team_number() == 0) {
 						// Assign both players to a previously unused team slot
 						std::set<TeamNumber> teams;
@@ -2287,6 +2524,200 @@ void CmdDiplomacy::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver&
 	fw.unsigned_8(other_player_);
 }
 
+// CmdPinnedNote
+void CmdPinnedNote::execute(Game& game) {
+	for (Bob* b = game.map()[pos_].get_first_bob(); b != nullptr; b = b->get_next_bob()) {
+		if (b->descr().type() == MapObjectType::PINNED_NOTE &&
+		    b->owner().player_number() == sender()) {
+			PinnedNote& pn = dynamic_cast<PinnedNote&>(*b);
+			if (delete_) {
+				pn.remove(game);
+			} else {
+				pn.set_text(text_);
+				pn.set_rgb(rgb_);
+			}
+			return;
+		}
+	}
+
+	if (!delete_) {
+		PinnedNote::create(game, *game.get_player(sender()), pos_, text_, rgb_);
+	}
+}
+
+CmdPinnedNote::CmdPinnedNote(StreamRead& des) : PlayerCommand(Time(0), des.unsigned_8()) {
+	text_ = des.string();
+	pos_.x = des.unsigned_16();
+	pos_.y = des.unsigned_16();
+	rgb_.r = des.unsigned_8();
+	rgb_.g = des.unsigned_8();
+	rgb_.b = des.unsigned_8();
+	delete_ = des.unsigned_8() != 0;
+}
+
+void CmdPinnedNote::serialize(StreamWrite& ser) {
+	write_id_and_sender(ser);
+	ser.string(text_);
+	ser.unsigned_16(pos_.x);
+	ser.unsigned_16(pos_.y);
+	ser.unsigned_8(rgb_.r);
+	ser.unsigned_8(rgb_.g);
+	ser.unsigned_8(rgb_.b);
+	ser.unsigned_8(delete_ ? 1 : 0);
+}
+
+constexpr uint8_t kCurrentPacketVersionCmdPinnedNote = 1;
+
+void CmdPinnedNote::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mol) {
+	try {
+		uint8_t packet_version = fr.unsigned_8();
+		if (packet_version == kCurrentPacketVersionCmdPinnedNote) {
+			PlayerCommand::read(fr, egbase, mol);
+			text_ = fr.string();
+			pos_.x = fr.unsigned_16();
+			pos_.y = fr.unsigned_16();
+			rgb_.r = fr.unsigned_8();
+			rgb_.g = fr.unsigned_8();
+			rgb_.b = fr.unsigned_8();
+			delete_ = fr.unsigned_8() != 0;
+		} else {
+			throw UnhandledVersionError(
+			   "CmdPinnedNote", packet_version, kCurrentPacketVersionCmdPinnedNote);
+		}
+	} catch (const std::exception& e) {
+		throw GameDataError("Cmd_PinnedNote: %s", e.what());
+	}
+}
+
+void CmdPinnedNote::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver& mos) {
+	fw.unsigned_8(kCurrentPacketVersionCmdPinnedNote);
+	PlayerCommand::write(fw, egbase, mos);
+	fw.string(text_);
+	fw.unsigned_16(pos_.x);
+	fw.unsigned_16(pos_.y);
+	fw.unsigned_8(rgb_.r);
+	fw.unsigned_8(rgb_.g);
+	fw.unsigned_8(rgb_.b);
+	fw.unsigned_8(delete_ ? 1 : 0);
+}
+
+// CmdShipPortName
+void CmdShipPortName::execute(Game& game) {
+	MapObject* mo = game.objects().get_object(serial_);
+	if (mo == nullptr) {
+		return;
+	}
+
+	switch (mo->descr().type()) {
+	case MapObjectType::SHIP:
+		dynamic_cast<Ship&>(*mo).set_shipname(name_);
+		return;
+
+	case MapObjectType::WAREHOUSE:
+		dynamic_cast<Warehouse&>(*mo).set_warehouse_name(name_);
+		return;
+
+	default:
+		throw wexception("CmdShipPortName for object %u of type %s", serial_,
+		                 to_string(mo->descr().type()).c_str());
+	}
+}
+
+CmdShipPortName::CmdShipPortName(StreamRead& des) : PlayerCommand(Time(0), des.unsigned_8()) {
+	serial_ = des.unsigned_32();
+	name_ = des.string();
+}
+
+void CmdShipPortName::serialize(StreamWrite& ser) {
+	write_id_and_sender(ser);
+	ser.unsigned_32(serial_);
+	ser.string(name_);
+}
+
+constexpr uint8_t kCurrentPacketVersionCmdShipPortName = 1;
+
+void CmdShipPortName::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mol) {
+	try {
+		uint8_t packet_version = fr.unsigned_8();
+		if (packet_version == kCurrentPacketVersionCmdShipPortName) {
+			PlayerCommand::read(fr, egbase, mol);
+			serial_ = fr.unsigned_32();
+			name_ = fr.string();
+		} else {
+			throw UnhandledVersionError(
+			   "CmdShipPortName", packet_version, kCurrentPacketVersionCmdShipPortName);
+		}
+	} catch (const std::exception& e) {
+		throw GameDataError("Cmd_ShipPortName: %s", e.what());
+	}
+}
+
+void CmdShipPortName::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver& mos) {
+	fw.unsigned_8(kCurrentPacketVersionCmdShipPortName);
+	PlayerCommand::write(fw, egbase, mos);
+	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(serial_)));
+	fw.string(name_);
+}
+
+// CmdFleetTargets
+void CmdFleetTargets::execute(Game& game) {
+	MapObject* mo = game.objects().get_object(interface_);
+	if (mo == nullptr) {
+		return;
+	}
+
+	switch (mo->descr().type()) {
+	case MapObjectType::SHIP_FLEET_YARD_INTERFACE:
+		dynamic_cast<ShipFleetYardInterface*>(mo)->get_fleet()->set_ships_target(game, target_);
+		return;
+
+	case MapObjectType::FERRY_FLEET_YARD_INTERFACE:
+		dynamic_cast<FerryFleetYardInterface*>(mo)->get_fleet()->set_idle_ferries_target(
+		   game, target_);
+		return;
+
+	default:
+		throw wexception("CmdFleetTargets for object %u of type %s", interface_,
+		                 to_string(mo->descr().type()).c_str());
+	}
+}
+
+CmdFleetTargets::CmdFleetTargets(StreamRead& des) : PlayerCommand(Time(0), des.unsigned_8()) {
+	interface_ = des.unsigned_32();
+	target_ = des.unsigned_32();
+}
+
+void CmdFleetTargets::serialize(StreamWrite& ser) {
+	write_id_and_sender(ser);
+	ser.unsigned_32(interface_);
+	ser.unsigned_32(target_);
+}
+
+constexpr uint8_t kCurrentPacketVersionCmdFleetTargets = 1;
+
+void CmdFleetTargets::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mol) {
+	try {
+		uint8_t packet_version = fr.unsigned_8();
+		if (packet_version == kCurrentPacketVersionCmdFleetTargets) {
+			PlayerCommand::read(fr, egbase, mol);
+			interface_ = fr.unsigned_32();
+			target_ = fr.unsigned_32();
+		} else {
+			throw UnhandledVersionError(
+			   "CmdFleetTargets", packet_version, kCurrentPacketVersionCmdFleetTargets);
+		}
+	} catch (const std::exception& e) {
+		throw GameDataError("Cmd_FleetTargets: %s", e.what());
+	}
+}
+
+void CmdFleetTargets::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver& mos) {
+	fw.unsigned_8(kCurrentPacketVersionCmdFleetTargets);
+	PlayerCommand::write(fw, egbase, mos);
+	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(interface_)));
+	fw.unsigned_32(target_);
+}
+
 // CmdPickCustomStartingPosition
 void CmdPickCustomStartingPosition::execute(Game& game) {
 	game.get_player(sender())->do_pick_custom_starting_position(coords_);
