@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2022 by the Widelands Development Team
+ * Copyright (C) 2008-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -97,6 +97,8 @@ struct GameClientImpl {
 	 */
 	uint32_t desiredspeed;
 
+	bool should_write_replay;
+
 	/// Backlog of chat messages
 	std::vector<ChatMessage> chatmessages;
 
@@ -109,6 +111,7 @@ struct GameClientImpl {
 	bool disconnect_called_;
 
 	void send_hello() const;
+	void send_custom_naming_lists() const;
 	void send_player_command(Widelands::PlayerCommand* /*pc*/) const;
 
 	void run_game(InteractiveGameBase* igb);
@@ -133,6 +136,23 @@ void GameClientImpl::send_player_command(Widelands::PlayerCommand* pc) const {
 	net->send(s);
 }
 
+void GameClientImpl::send_custom_naming_lists() const {
+	SendPacket s;
+	s.unsigned_8(NETCMD_CUSTOM_NAMING_LISTS);
+
+	auto names = Widelands::read_custom_warehouse_ship_names();
+	s.unsigned_32(names.first.size());
+	for (const std::string& name : names.first) {
+		s.string(name);
+	}
+	s.unsigned_32(names.second.size());
+	for (const std::string& name : names.second) {
+		s.string(name);
+	}
+
+	net->send(s);
+}
+
 /**
  * Show progress dialog and load map or saved game.
  */
@@ -174,7 +194,7 @@ void GameClientImpl::run_game(InteractiveGameBase* igb) {
 	game->run(settings.savegame ? Widelands::Game::StartGameType::kSaveGame :
 	          settings.scenario ? Widelands::Game::StartGameType::kMultiPlayerScenario :
                                  Widelands::Game::StartGameType::kMap,
-	          "", false, format("netclient_%d", static_cast<int>(settings.usernum)));
+	          "", format("netclient_%d", static_cast<int>(settings.usernum)));
 
 	// if this is an internet game, tell the metaserver that the game is done.
 	if (internet_) {
@@ -220,6 +240,7 @@ GameClient::GameClient(FsMenu::MenuCapsule& c,
 	d->localplayername = playername;
 	d->modal = nullptr;
 	d->panel_whose_mutex_needs_resetting_on_game_start = nullptr;
+	d->should_write_replay = true;
 	d->game = nullptr;
 	d->realspeed = 0;
 	d->desiredspeed = 1000;
@@ -246,12 +267,10 @@ GameClient::~GameClient() {
 
 void GameClient::run() {
 	d->send_hello();
+	d->send_custom_naming_lists();
 	d->settings.multiplayer = true;
+	d->modal = new FsMenu::LaunchMPG(capsule_, *this, *this, *this, d->internet_);
 
-	// Fill the list of possible system messages
-	NetworkGamingMessages::fill_map();
-
-	d->modal = new FsMenu::LaunchMPG(capsule_, *this, *this, *this, *d->game, d->internet_);
 	// The main menu's think() loop creates a mutex lock that would permanently block the Game
 	// from locking this mutex. So when the game starts we'll need to break the lock by force.
 	for (UI::Panel* p = d->modal; p != nullptr; p = p->get_parent()) {
@@ -263,11 +282,21 @@ void GameClient::run() {
 	NEVER_HERE();
 }
 
+const std::string& GameClient::get_local_playername() const {
+	return d->localplayername;
+}
+
+void GameClient::set_write_replay(bool replay) {
+	d->should_write_replay = replay;
+}
+
 void GameClient::do_run(RecvPacket& packet) {
 	d->server_is_waiting = true;
 
 	Widelands::Game game;
-	game.set_write_syncstream(get_config_bool("write_syncstreams", true));
+	game.set_write_replay(d->should_write_replay);
+	game.set_write_syncstream(g_write_syncstreams);
+	game.logic_rand_seed(packet.unsigned_32());
 
 	game.enabled_addons().clear();
 	for (size_t i = packet.unsigned_32(); i != 0u; --i) {
@@ -285,29 +314,53 @@ void GameClient::do_run(RecvPacket& packet) {
 		}
 	}
 
+	std::map<Widelands::PlayerNumber, std::pair<std::set<std::string>, std::set<std::string>>>
+	   custom_naming_lists;
+	for (;;) {
+		Widelands::PlayerNumber number = packet.unsigned_8();
+		if (number == 0) {
+			break;
+		}
+
+		for (size_t i = packet.unsigned_32(); i > 0; --i) {
+			custom_naming_lists[number].first.insert(packet.string());
+		}
+		for (size_t i = packet.unsigned_32(); i > 0; --i) {
+			custom_naming_lists[number].second.insert(packet.string());
+		}
+	}
+
 	capsule_.set_visible(false);
 	try {
 		std::vector<std::string> tipstexts{"general_game", "multiplayer"};
 		if (has_players_tribe()) {
 			tipstexts.push_back(get_players_tribe());
 		}
-		UI::ProgressWindow& loader_ui =
-		   game.create_loader_ui(tipstexts, true, d->settings.map_theme, d->settings.map_background);
+		UI::ProgressWindow& loader_ui = game.create_loader_ui(
+		   tipstexts, true, d->settings.map_theme, d->settings.map_background, true);
 
 		d->game = &game;
 		InteractiveGameBase* igb = d->init_game(this, loader_ui);
+
+		for (const auto& lists : custom_naming_lists) {
+			Widelands::Player* player = game.get_safe_player(lists.first);
+			player->set_shipnames(lists.second.first);
+			player->set_warehousenames(lists.second.second);
+		}
+
 		if (d->panel_whose_mutex_needs_resetting_on_game_start != nullptr) {
 			d->panel_whose_mutex_needs_resetting_on_game_start->clear_current_think_mutex();
 			d->panel_whose_mutex_needs_resetting_on_game_start = nullptr;
 		}
+
 		d->run_game(igb);
 
 	} catch (const WLWarning& e) {
-		WLApplication::emergency_save(&capsule_.menu(), game, e.what(), 1, true, false);
+		WLApplication::emergency_save(&capsule_.menu(), game, e.what(), 1, false, false);
 		d->game = nullptr;
 	} catch (const std::exception& e) {
 		FsMenu::MainMenu& parent = capsule_.menu();  // make includes script happy
-		WLApplication::emergency_save(&parent, game, e.what());
+		WLApplication::emergency_save(&parent, game, e.what(), 1, false);
 		d->game = nullptr;
 		disconnect("CLIENT_CRASHED");
 		if (d->internet_) {
@@ -453,12 +506,12 @@ void GameClient::set_map(const std::string& /*mapname*/,
 	// client is not allowed to do this
 }
 
-void GameClient::send_cheating_info() {
+void GameClient::send_cheating_info(const std::string& code, const std::string& arg2) {
 	SendPacket packet;
 	packet.unsigned_8(NETCMD_SYSTEM_MESSAGE_CODE);
-	packet.string("CHEAT");
+	packet.string(code);
 	packet.string(d->localplayername);
-	packet.string("");
+	packet.string(arg2);
 	packet.string("");
 	d->net->send(packet);
 }
@@ -542,27 +595,32 @@ void GameClient::set_player(uint8_t /*number*/, const PlayerSettings& /* setting
 	// set_player_number(uint8_t) to the host.
 }
 
-void GameClient::set_peaceful_mode(bool peace) {
-	d->settings.peaceful = peace;
+void GameClient::set_flag(GameSettings::Flags flag, bool state) {
+	if (state) {
+		d->settings.flags |= flag;
+	} else {
+		d->settings.flags &= ~flag;
+	}
 }
 
-bool GameClient::is_peaceful_mode() {
-	return d->settings.peaceful;
-}
-
-void GameClient::set_custom_starting_positions(bool c) {
-	d->settings.custom_starting_positions = c;
-}
-
-bool GameClient::get_custom_starting_positions() {
-	return d->settings.custom_starting_positions;
+bool GameClient::get_flag(GameSettings::Flags flag) {
+	return (d->settings.flags & flag) != 0;
 }
 
 std::string GameClient::get_win_condition_script() {
 	return d->settings.win_condition_script;
 }
 
-void GameClient::set_win_condition_script(const std::string& /*wc*/) {
+int32_t GameClient::get_win_condition_duration() {
+	return d->settings.win_condition_duration;
+}
+
+void GameClient::set_win_condition_script(const std::string& /* wc */) {
+	// Clients are not allowed to change this
+	NEVER_HERE();
+}
+
+void GameClient::set_win_condition_duration(const int32_t /* duration */) {
 	// Clients are not allowed to change this
 	NEVER_HERE();
 }
@@ -755,10 +813,10 @@ void GameClient::handle_hello(RecvPacket& packet) {
 		std::string message = format(
 		   ngettext("%u add-on mismatch detected:\n", "%u add-on mismatches detected:\n", nr), nr);
 		for (const std::string& name : missing_addons) {
-			message = format(_("%1$s\n· ‘%2$s’ required by host not found"), message, name);
+			message = format(_("%1$s\n• ‘%2$s’ required by host not found"), message, name);
 		}
 		for (const auto& pair : wrong_version_addons) {
-			message = format(_("%1$s\n· ‘%2$s’ installed at version %3$s but host uses version %4$s"),
+			message = format(_("%1$s\n• ‘%2$s’ installed at version %3$s but host uses version %4$s"),
 			                 message, pair.first, pair.second.first, pair.second.second);
 		}
 		throw AddOnsMismatchException(message);
@@ -1091,6 +1149,16 @@ void GameClient::handle_system_message(RecvPacket& packet) {
 	                                   // c.sender remains empty to indicate a system message
 	d->chatmessages.push_back(c);
 	Notifications::publish(c);
+
+	if (g_allow_script_console && code == "CLIENT_HAS_JOINED_GAME") {
+		// Warn others
+		// TODO(tothxa): It would be better to only broadcast if we are the new user, otherwise send
+		//               it to the new user only, but
+		//                 1. We can only send commands to the host
+		//                 2. System messages are assembled and translated on each client, so
+		//                    individual players can't be @-addressed
+		send_cheating_info("CAN_CHEAT");
+	}
 }
 
 /**
@@ -1165,11 +1233,11 @@ void GameClient::handle_packet(RecvPacket& packet) {
 	case NETCMD_WIN_CONDITION:
 		d->settings.win_condition_script = g_fs->FileSystem::fix_cross_file(packet.string());
 		break;
-	case NETCMD_PEACEFUL_MODE:
-		d->settings.peaceful = (packet.unsigned_8() != 0u);
+	case NETCMD_WIN_CONDITION_DURATION:
+		d->settings.win_condition_duration = packet.signed_32();
 		break;
-	case NETCMD_CUSTOM_STARTING_POSITIONS:
-		d->settings.custom_starting_positions = (packet.unsigned_8() != 0u);
+	case NETCMD_GAMEFLAGS:
+		d->settings.flags = packet.unsigned_16();
 		break;
 	case NETCMD_LAUNCH:
 		if ((d->game != nullptr) || ((d->modal != nullptr) && d->modal->is_modal())) {
@@ -1227,15 +1295,19 @@ void GameClient::handle_network() {
 	} catch (const AddOnsMismatchException& e) {
 		disconnect("SOMETHING_WRONG", e.what());
 	} catch (const WLWarning& e) {
-		// disconnect() should have been called already, but just in case:
+		if (d->disconnect_called_) {
+			// the Warning is intended for the caller
+			throw;
+		}
 		disconnect("SOMETHING_WRONG", e.what());
-		// the Warning is intended for the caller
-		throw;
 	} catch (const DisconnectException& e) {
 		disconnect(e.what());
 	} catch (const ProtocolException& e) {
 		disconnect("PROTOCOL_EXCEPTION", e.what());
 	} catch (const std::exception& e) {
+		if (d->disconnect_called_) {
+			throw;
+		}
 		disconnect("SOMETHING_WRONG", e.what());
 	}
 }
