@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2022 by the Widelands Development Team
+ * Copyright (C) 2008-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -65,7 +65,7 @@
 #include "wui/interactive_spectator.h"
 
 struct HostChatProvider : public ChatProvider {
-	explicit HostChatProvider(GameHost* const init_host) : h(init_host), kickClient(0) {
+	explicit HostChatProvider(GameHost* const init_host) : h(init_host) {
 	}
 
 	// TODO(k.halfmann): this deserves a refactoring
@@ -228,7 +228,7 @@ struct HostChatProvider : public ChatProvider {
 		h->send(c);
 	}
 
-	const std::vector<ChatMessage>& get_messages() const override {
+	[[nodiscard]] const std::vector<ChatMessage>& get_messages() const override {
 		return messages;
 	}
 
@@ -237,7 +237,7 @@ struct HostChatProvider : public ChatProvider {
 		Notifications::publish(msg);
 	}
 
-	bool has_been_set() const override {
+	[[nodiscard]] bool has_been_set() const override {
 		return true;
 	}
 
@@ -245,7 +245,7 @@ private:
 	GameHost* h;
 	std::vector<ChatMessage> messages;
 	std::string kickUser;
-	uint32_t kickClient;
+	uint32_t kickClient{0U};
 	std::string kickReason;
 };
 
@@ -266,12 +266,15 @@ struct Client {
 	/// relative
 	/// to when the last answer of the client was received.
 	time_t lastdelta;
+
+	std::set<std::string> custom_ship_names;
+	std::set<std::string> custom_warehouse_names;
 };
 
 struct GameHostImpl {
 	GameSettings settings;
 	std::string localplayername;
-	uint32_t localdesiredspeed;
+	uint32_t localdesiredspeed{0U};
 	// unique_ptr instead of object to break cyclic dependency
 	std::unique_ptr<ParticipantList> participants;
 	HostChatProvider chat;
@@ -286,59 +289,41 @@ struct GameHostImpl {
 	std::vector<Client> clients;
 
 	/// The game itself; only non-null while game is running
-	Widelands::Game* game;
+	Widelands::Game* game{nullptr};
 
 	/// If we were to send out a plain networktime packet, this would be the
 	/// time. However, we have not yet committed to this networktime.
-	Time pseudo_networktime;
-	int32_t last_heartbeat;
+	Time pseudo_networktime{0U};
+	int32_t last_heartbeat{0};
 
 	/// The networktime we committed to by sending it across the network.
-	Time committed_networktime;
+	Time committed_networktime{0U};
 
 	/// This is the time for local simulation
 	NetworkTime time;
 
 	/// Whether we're waiting for all clients to report back.
-	bool waiting;
-	uint32_t lastframe;
+	bool waiting{false};
+	uint32_t lastframe{0U};
 
 	/**
 	 * The speed, in milliseconds per second, that is effective as long
 	 * as we're not \ref waiting.
 	 */
-	uint32_t networkspeed;
-	time_t lastpauseping;
+	uint32_t networkspeed{0U};
+	time_t lastpauseping{0U};
 
 	/// All currently running computer players, *NOT* in one-one correspondence
 	/// with \ref Player objects
 	std::vector<AI::ComputerPlayer*> computerplayers;
 
 	/// \c true if a syncreport is currently in flight
-	bool syncreport_pending;
-	Time syncreport_time;
+	bool syncreport_pending{false};
+	Time syncreport_time{0U};
 	Md5Checksum syncreport;
-	bool syncreport_arrived;
+	bool syncreport_arrived{false};
 
-	explicit GameHostImpl(GameHost* const h)
-	   : localdesiredspeed(0),
-	     participants(nullptr),
-	     chat(h),
-	     hp(h),
-	     npsb(&hp),
-
-	     game(nullptr),
-	     pseudo_networktime(0),
-	     last_heartbeat(0),
-	     committed_networktime(0),
-	     waiting(false),
-	     lastframe(0),
-	     networkspeed(0),
-	     lastpauseping(0),
-	     syncreport_pending(false),
-	     syncreport_time(0),
-	     syncreport(),
-	     syncreport_arrived(false) {
+	explicit GameHostImpl(GameHost* const h) : participants(nullptr), chat(h), hp(h), npsb(&hp) {
 	}
 
 	/// Takes ownership of the given pointer
@@ -353,11 +338,7 @@ GameHost::GameHost(FsMenu::MenuCapsule* c,
                    const std::string& playername,
                    std::vector<Widelands::TribeBasicInfo> tribeinfos,
                    bool internet)
-   : capsule_(c),
-     pointer_(ptr),
-     d(new GameHostImpl(this)),
-     internet_(internet),
-     forced_pause_(false) {
+   : capsule_(c), pointer_(ptr), d(new GameHostImpl(this)), internet_(internet) {
 	verb_log_info("[Host]: starting up.");
 
 	d->localplayername = playername;
@@ -487,19 +468,18 @@ void GameHost::init_computer_players() {
 
 void GameHost::run() {
 	game_.reset(new Widelands::Game());
-	// Fill the list of possible system messages
-	NetworkGamingMessages::fill_map();
-	new FsMenu::LaunchMPG(
-	   *capsule_, d->hp, *this, d->chat, *game_, internet_, [this]() { run_callback(); });
+	new FsMenu::LaunchMPG(*capsule_, d->hp, *this, d->chat, internet_, [this]() { run_callback(); });
 }
 
 void GameHost::run_direct() {
 	game_.reset(new Widelands::Game());
-	// Fill the list of possible system messages
-	NetworkGamingMessages::fill_map();
 	run_callback();
 }
 
+void GameHost::set_write_replay(bool replay) {
+	game_->set_write_replay(replay);
+}
+
 // TODO(k.halfmann): refactor into smaller functions
 void GameHost::run_callback() {
 	// if this is an internet game, tell the metaserver that the game started
@@ -517,17 +497,55 @@ void GameHost::run_callback() {
 		}
 	}
 
+	const uint32_t rng_seed = RNG::static_rand();
 	SendPacket packet;
 	packet.unsigned_8(NETCMD_LAUNCH);
+	packet.unsigned_32(rng_seed);
+
 	packet.unsigned_32(game_->enabled_addons().size());
 	for (const auto& a : game_->enabled_addons()) {
 		packet.string(a->internal_name);
 	}
+
+	std::map<Widelands::PlayerNumber, std::pair<std::set<std::string>, std::set<std::string>>>
+	   custom_naming_lists;
+	if (!d->settings.savegame) {  // Naming lists don't make sense in savegames
+		int playernumber = d->settings.playernum + 1;
+		if (playernumber > 0 && playernumber <= UserSettings::highest_playernum()) {
+			custom_naming_lists.emplace(playernumber, Widelands::read_custom_warehouse_ship_names());
+		}
+
+		for (Client& client : d->clients) {
+			playernumber = client.playernum + 1;
+			if (playernumber > 0 && playernumber <= UserSettings::highest_playernum()) {
+				// Merge instead of overwrite - multiple clients can share a player
+				custom_naming_lists[playernumber].first.insert(
+				   client.custom_ship_names.begin(), client.custom_ship_names.end());
+				custom_naming_lists[playernumber].second.insert(
+				   client.custom_warehouse_names.begin(), client.custom_warehouse_names.end());
+			}
+		}
+
+		for (const auto& lists : custom_naming_lists) {
+			packet.unsigned_8(lists.first);
+			packet.unsigned_32(lists.second.first.size());
+			for (const std::string& name : lists.second.first) {
+				packet.string(name);
+			}
+			packet.unsigned_32(lists.second.second.size());
+			for (const std::string& name : lists.second.second) {
+				packet.string(name);
+			}
+		}
+	}
+	packet.unsigned_8(0);  // End of naming lists section
+
 	broadcast(packet);
 
+	game_->logic_rand_seed(rng_seed);
 	game_->set_ai_training_mode(get_config_bool("ai_training", false));
 	game_->set_auto_speed(get_config_bool("auto_speed", false));
-	game_->set_write_syncstream(get_config_bool("write_syncstreams", true));
+	game_->set_write_syncstream(g_write_syncstreams);
 
 	if (capsule_ != nullptr) {
 		capsule_->set_visible(false);
@@ -538,7 +556,8 @@ void GameHost::run_callback() {
 		if (d->hp.has_players_tribe()) {
 			tipstexts.push_back(d->hp.get_players_tribe());
 		}
-		game_->create_loader_ui(tipstexts, true, d->settings.map_theme, d->settings.map_background);
+		game_->create_loader_ui(
+		   tipstexts, true, d->settings.map_theme, d->settings.map_background, true);
 		Notifications::publish(UI::NoteLoadingMessage(_("Preparing game…")));
 
 		d->game = game_.get();
@@ -569,6 +588,13 @@ void GameHost::run_callback() {
 		} else {  // savegame
 			game_->init_savegame(d->settings);
 		}
+
+		for (const auto& lists : custom_naming_lists) {
+			Widelands::Player* player = game_->get_safe_player(lists.first);
+			player->set_shipnames(lists.second.first);
+			player->set_warehousenames(lists.second.second);
+		}
+
 		d->pseudo_networktime = game_->get_gametime();
 		d->time.reset(d->pseudo_networktime);
 		d->lastframe = SDL_GetTicks();
@@ -588,7 +614,7 @@ void GameHost::run_callback() {
 		game_->run(d->settings.savegame ? Widelands::Game::StartGameType::kSaveGame :
 		           d->settings.scenario ? Widelands::Game::StartGameType::kMultiPlayerScenario :
                                         Widelands::Game::StartGameType::kMap,
-		           script_to_run_, false, "nethost");
+		           script_to_run_, "nethost");
 
 		// if this is an internet game, tell the metaserver that the game is done.
 		if (internet_) {
@@ -598,7 +624,7 @@ void GameHost::run_callback() {
 	} catch (const std::exception& e) {
 		FsMenu::MainMenu* parent =
 		   capsule_ != nullptr ? &capsule_->menu() : nullptr;  // make includes script happy
-		WLApplication::emergency_save(parent, *game_, e.what(), player_number);
+		WLApplication::emergency_save(parent, *game_, e.what(), player_number, false);
 		clear_computer_players();
 
 		while (!d->clients.empty()) {
@@ -1109,9 +1135,9 @@ void GameHost::set_map(const std::string& mapname,
 		}
 		std::vector<char> complete(file_->bytes);
 		fr.set_file_pos(0);
-		fr.data_complete(&complete[0], file_->bytes);
+		fr.data_complete(complete.data(), file_->bytes);
 		SimpleMD5Checksum md5sum;
-		md5sum.data(&complete[0], file_->bytes);
+		md5sum.data(complete.data(), file_->bytes);
 		md5sum.finish_checksum();
 		file_->md5sum = md5sum.get_checksum().str();
 	} else {
@@ -1308,10 +1334,12 @@ bool GameHost::remove_player_name(uint8_t const number, const std::string& name)
 	PlayerSettings& p = d->settings.players.at(number);
 	std::string temp(p.name);
 	temp.erase(p.name.find(name), name.size());
-	if (temp.back() == ' ') {
-		temp.erase(temp.end() - 1);
-	} else if (temp.front() == ' ') {
-		temp.erase(temp.begin());
+	if (!temp.empty()) {
+		if (temp.back() == ' ') {
+			temp.pop_back();
+		} else if (temp.front() == ' ') {
+			temp.erase(temp.begin());
+		}
 	}
 	set_player_name(number, temp);
 	return temp.empty();
@@ -1399,23 +1427,27 @@ void GameHost::set_win_condition_script(const std::string& wc) {
 	broadcast(packet);
 }
 
-void GameHost::set_peaceful_mode(bool peace) {
-	d->settings.peaceful = peace;
+void GameHost::set_win_condition_duration(const int32_t duration) {
+	d->settings.win_condition_duration = duration;
 
 	// Broadcast changes
 	SendPacket packet;
-	packet.unsigned_8(NETCMD_PEACEFUL_MODE);
-	packet.unsigned_8(peace ? 1 : 0);
+	packet.unsigned_8(NETCMD_WIN_CONDITION_DURATION);
+	packet.signed_32(duration);
 	broadcast(packet);
 }
 
-void GameHost::set_custom_starting_positions(bool c) {
-	d->settings.custom_starting_positions = c;
+void GameHost::set_flag(GameSettings::Flags flag, bool state) {
+	if (state) {
+		d->settings.flags |= flag;
+	} else {
+		d->settings.flags &= ~flag;
+	}
 
 	// Broadcast changes
 	SendPacket packet;
-	packet.unsigned_8(NETCMD_CUSTOM_STARTING_POSITIONS);
-	packet.unsigned_8(c ? 1 : 0);
+	packet.unsigned_8(NETCMD_GAMEFLAGS);
+	packet.unsigned_16(d->settings.flags);
 	broadcast(packet);
 }
 
@@ -1720,7 +1752,7 @@ void GameHost::welcome_client(uint32_t const number, std::string& playername) {
 	{
 		std::vector<const AddOns::AddOnInfo*> enabled_addons;
 		for (const auto& pair : AddOns::g_addons) {
-			if (pair.second && pair.first->category != AddOns::AddOnCategory::kTheme) {
+			if (pair.second && AddOns::kAddOnCategories.at(pair.first->category).network_relevant) {
 				enabled_addons.push_back(pair.first.get());
 			}
 		}
@@ -1780,13 +1812,13 @@ void GameHost::welcome_client(uint32_t const number, std::string& playername) {
 	d->net->send(client.sock_id, packet);
 
 	packet.reset();
-	packet.unsigned_8(NETCMD_PEACEFUL_MODE);
-	packet.unsigned_8(d->settings.peaceful ? 1 : 0);
+	packet.unsigned_8(NETCMD_WIN_CONDITION_DURATION);
+	packet.signed_32(d->settings.win_condition_duration);
 	d->net->send(client.sock_id, packet);
 
 	packet.reset();
-	packet.unsigned_8(NETCMD_CUSTOM_STARTING_POSITIONS);
-	packet.unsigned_8(d->settings.custom_starting_positions ? 1 : 0);
+	packet.unsigned_8(NETCMD_GAMEFLAGS);
+	packet.unsigned_16(d->settings.flags);
 	d->net->send(client.sock_id, packet);
 
 	// Broadcast new information about the player to everybody
@@ -1804,6 +1836,16 @@ void GameHost::welcome_client(uint32_t const number, std::string& playername) {
 	}
 
 	send_system_message_code("CLIENT_HAS_JOINED_GAME", effective_name);
+
+	if (g_allow_script_console) {
+		// TODO(tothxa): The host could warn only the new client, but other clients can only
+		//               broadcast:
+		//                 1. They can only send commands to the host
+		//                 2. System messages are assembled and translated on each client, so
+		//                    individual players can't be @-addressed
+		//               Until this is solved, it's better if the host broadcasts too.
+		send_system_message_code("CAN_CHEAT", d->localplayername);
+	}
 }
 
 void GameHost::committed_network_time(const Time& time) {
@@ -1919,6 +1961,16 @@ void GameHost::broadcast_real_speed(uint32_t const speed) {
 	broadcast(packet);
 }
 
+bool GameHost::client_may_change_speed(uint8_t playernum) const {
+	if (playernum > UserSettings::highest_playernum()) {
+		return false;
+	}
+	const Widelands::PlayerEndStatus* pes =
+	   d->game->player_manager()->get_player_end_status(playernum + 1);
+	return pes == nullptr || (pes->result != Widelands::PlayerEndResult::kLost &&
+	                          pes->result != Widelands::PlayerEndResult::kResigned);
+}
+
 /**
  * This is the algorithm that decides upon the effective network speed,
  * given the desired speed of all clients.
@@ -1950,13 +2002,14 @@ void GameHost::update_network_speed() {
 		// No pause was forced - normal speed calculation
 		std::vector<uint32_t> speeds;
 
-		speeds.push_back(d->localdesiredspeed);
 		for (const Client& client : d->clients) {
-			if (client.playernum <= UserSettings::highest_playernum()) {
+			if (client_may_change_speed(client.playernum)) {
 				speeds.push_back(client.desiredspeed);
 			}
 		}
-		assert(!speeds.empty());
+		if (speeds.empty() || client_may_change_speed(d->settings.playernum)) {
+			speeds.push_back(d->localdesiredspeed);
+		}
 
 		std::sort(speeds.begin(), speeds.end());
 
@@ -2268,7 +2321,7 @@ void GameHost::handle_nettime(uint32_t const client_num, RecvPacket& r) {
 	receive_client_time(client_num, Time(r.unsigned_32()));
 }
 
-void GameHost::handle_playercommmand(uint32_t const client_num, Client& client, RecvPacket& r) {
+void GameHost::handle_playercommand(uint32_t const client_num, Client& client, RecvPacket& r) {
 	if (d->game == nullptr) {
 		throw DisconnectException("PLAYERCMD_WO_GAME");
 	}
@@ -2283,6 +2336,17 @@ void GameHost::handle_playercommmand(uint32_t const client_num, Client& client,
 	do_send_player_command(plcmd);
 }
 
+void GameHost::handle_custom_naming_lists(Client& client, RecvPacket& r) {
+	client.custom_ship_names.clear();
+	client.custom_warehouse_names.clear();
+	for (size_t i = r.unsigned_32(); i > 0; --i) {
+		client.custom_ship_names.insert(r.string());
+	}
+	for (size_t i = r.unsigned_32(); i > 0; --i) {
+		client.custom_warehouse_names.insert(r.string());
+	}
+}
+
 void GameHost::handle_syncreport(uint32_t const client_num, Client& client, RecvPacket& r) {
 	if ((d->game == nullptr) || !d->syncreport_pending || client.syncreport_arrived) {
 		throw DisconnectException("UNEXPECTED_SYNC_REP");
@@ -2376,7 +2440,9 @@ void GameHost::handle_packet(uint32_t const client_num, RecvPacket& r) {
 	case NETCMD_TIME:
 		return handle_nettime(client_num, r);
 	case NETCMD_PLAYERCOMMAND:
-		return handle_playercommmand(client_num, client, r);
+		return handle_playercommand(client_num, client, r);
+	case NETCMD_CUSTOM_NAMING_LISTS:
+		return handle_custom_naming_lists(client, r);
 	case NETCMD_SYNCREPORT:
 		return handle_syncreport(client_num, client, r);
 	case NETCMD_CHAT:
@@ -2393,8 +2459,8 @@ void GameHost::handle_packet(uint32_t const client_num, RecvPacket& r) {
 	case NETCMD_SETTING_MAP:
 	case NETCMD_SETTING_PLAYER:
 	case NETCMD_WIN_CONDITION:
-	case NETCMD_PEACEFUL_MODE:
-	case NETCMD_CUSTOM_STARTING_POSITIONS:
+	case NETCMD_WIN_CONDITION_DURATION:
+	case NETCMD_GAMEFLAGS:
 	case NETCMD_LAUNCH:
 		if (d->game ==
 		    nullptr) {  // not expected while game is in progress -> something is wrong here
@@ -2409,12 +2475,15 @@ void GameHost::handle_packet(uint32_t const client_num, RecvPacket& r) {
 	}
 }
 
+static const std::set<std::string> cheating_message_codes = {
+   "CHEAT", "CAN_CHEAT", "SWITCHED_PLAYER", "CHEAT_OTHER"};
+
 void GameHost::handle_system_message(RecvPacket& packet) {
 	const std::string code = packet.string();
 	const std::string arg1 = packet.string();
 	const std::string arg2 = packet.string();
 	const std::string arg3 = packet.string();
-	if (code != "CHEAT") {
+	if (cheating_message_codes.count(code) == 0) {
 		log_err("[Host]: Received system command %s(%s,%s,%s) from client", code.c_str(),
 		        arg1.c_str(), arg2.c_str(), arg3.c_str());
 		throw DisconnectException("MALFORMED_COMMANDS");
@@ -2501,7 +2570,11 @@ void GameHost::disconnect_client(uint32_t const client_number,
 				if (!forced_pause()) {
 					force_pause();
 				}
-				WLApplication::emergency_save(nullptr, *d->game, reason, 1, false);
+				NoteThreadSafeFunction::instantiate(
+				   [&game = *d->game, &reason]() {
+					   WLApplication::emergency_save(nullptr, game, reason, 1, false);
+				   },
+				   true);
 			}
 			// Client was active but is a winner of the game: Replace with normal AI
 		} else if (d->settings.users.at(client.usernum).result == Widelands::PlayerEndResult::kWon) {
