@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2022 by the Widelands Development Team
+ * Copyright (C) 2020-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,6 +43,7 @@
 #include "base/i18n.h"
 #include "base/math.h"
 #include "base/md5.h"
+#include "base/time_string.h"
 #include "base/warning.h"
 #include "build_info.h"
 #include "graphic/image_cache.h"
@@ -192,6 +193,24 @@ void NetAddons::init(std::string username, std::string password) {
 		throw WLWarning("", "Unable to create socket");
 	}
 
+	constexpr uint32_t kTimeout = 4;
+#ifdef _WIN32
+	DWORD timeout_val = kTimeout * 1000;
+	const char* timeout_ptr = reinterpret_cast<const char*>(&timeout_val);
+	// Set timeout of read()
+	setsockopt(client_socket_, SOL_SOCKET, SO_RCVTIMEO, timeout_ptr, sizeof(timeout_val));
+	// Set timeout of write()
+	setsockopt(client_socket_, SOL_SOCKET, SO_SNDTIMEO, timeout_ptr, sizeof(timeout_val));
+#else
+	struct timeval timeout_val;
+	timeout_val.tv_sec = kTimeout;
+	timeout_val.tv_usec = 0;
+	// Set timeout of read()
+	setsockopt(client_socket_, SOL_SOCKET, SO_RCVTIMEO, &timeout_val, sizeof(timeout_val));
+	// Set timeout of write()
+	setsockopt(client_socket_, SOL_SOCKET, SO_SNDTIMEO, &timeout_val, sizeof(timeout_val));
+#endif
+
 	const std::string target_ip = get_config_string("addon_server_ip", "widelands.org");
 	const int target_port = get_config_int("addon_server_port", 7388);
 	sockaddr_in server;
@@ -290,7 +309,7 @@ void NetAddons::write_to_server(const char* send, const size_t length) {
 	}
 
 	std::string message;
-	for (; initialized_;) {
+	while (initialized_) {
 		std::string line = read_line();
 		if (line.empty()) {
 			break;
@@ -300,6 +319,12 @@ void NetAddons::write_to_server(const char* send, const size_t length) {
 	}
 
 	if (message.empty()) {
+		if (is_uploading_addon_) {
+			throw WLWarning("",
+			                "Connection interrupted (%s). Please note that you can not upload updates "
+			                "for an add-on more often than every three days.",
+			                strerror(errno));
+		}
 		throw WLWarning("", "Connection interrupted (%s)", strerror(errno));
 	}
 	throw WLWarning("", "Connection interrupted (%s). Reason: %s", strerror(errno), message.c_str());
@@ -346,7 +371,7 @@ void NetAddons::check_endofstream() {
 // reading or writing random leftover bytes. Create it before doing some
 // networking stuff and call `ok()` after everything has gone well.
 struct CrashGuard {
-	explicit CrashGuard(NetAddons& n, bool uses_cache = false) : net_(n), ok_(false) {
+	explicit CrashGuard(NetAddons& n, bool uses_cache = false) : net_(n) {
 		assert(net_.initialized_);
 		if (net_.network_active_) {
 			throw WLWarning("", "Network is already active");
@@ -366,6 +391,7 @@ struct CrashGuard {
 		assert(net_.initialized_);
 		assert(net_.network_active_);
 		net_.network_active_ = false;
+		net_.is_uploading_addon_ = false;
 		if (!ok_) {
 			net_.quit_connection();
 		}
@@ -373,7 +399,7 @@ struct CrashGuard {
 
 private:
 	NetAddons& net_;
-	bool ok_;
+	bool ok_{false};
 };
 
 std::vector<std::string> NetAddons::refresh_remotes(const bool all) {
@@ -497,9 +523,8 @@ AddOnInfo NetAddons::fetch_one_remote(const std::string& name) {
 		a.icon = g_image_cache->get(kAddOnCategories.at(a.category).icon);
 	} else {
 		g_fs->ensure_directory_exists(kTempFileDir);
-		const std::string path =
-		   kTempFileDir + FileSystem::file_separator() + a.internal_name + ".icon" +
-		   std::to_string(std::time(nullptr)) /* for disambiguation */ + kTempFileExtension;
+		const std::string path = kTempFileDir + FileSystem::file_separator() + timestring() +
+		                         ".icon." + a.internal_name + kTempFileExtension;
 		read_file(icon_file_size, path);
 		check_checksum(path, icon_checksum);
 		a.icon = g_image_cache->get(path);
@@ -740,6 +765,7 @@ void NetAddons::upload_addon(const std::string& name,
 		init_fn("", gather_addon_content(dir, "", content));
 	}
 
+	is_uploading_addon_ = true;
 	CrashGuard guard(*this);
 	std::string send = kCmdSubmit;
 	send += ' ';
@@ -884,8 +910,8 @@ std::string NetAddons::download_screenshot(const std::string& name, const std::s
 		send += '\n';
 		write_to_server(send);
 
-		std::string temp_dirname =
-		   kTempFileDir + FileSystem::file_separator() + name + ".screenshots" + kTempFileExtension;
+		std::string temp_dirname = kTempFileDir + FileSystem::file_separator() + timestring() +
+		                           ".screenshots." + name + kTempFileExtension;
 		g_fs->ensure_directory_exists(temp_dirname);
 		std::string output = temp_dirname + FileSystem::file_separator() + screenie;
 
