@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2022 by the Widelands Development Team
+ * Copyright (C) 2006-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -622,11 +622,17 @@ PlayerBase
 
 const char LuaPlayerBase::className[] = "PlayerBase";
 const MethodType<LuaPlayerBase> LuaPlayerBase::Methods[] = {
-   METHOD(LuaPlayerBase, __eq),        METHOD(LuaPlayerBase, __tostring),
-   METHOD(LuaPlayerBase, conquer),     METHOD(LuaPlayerBase, get_wares),
-   METHOD(LuaPlayerBase, get_workers), METHOD(LuaPlayerBase, place_building),
-   METHOD(LuaPlayerBase, place_flag),  METHOD(LuaPlayerBase, place_road),
-   METHOD(LuaPlayerBase, place_ship),  {nullptr, nullptr},
+   METHOD(LuaPlayerBase, __eq),
+   METHOD(LuaPlayerBase, __tostring),
+   METHOD(LuaPlayerBase, conquer),
+   METHOD(LuaPlayerBase, get_wares),
+   METHOD(LuaPlayerBase, get_workers),
+   METHOD(LuaPlayerBase, place_building),
+   METHOD(LuaPlayerBase, place_flag),
+   METHOD(LuaPlayerBase, place_road),
+   METHOD(LuaPlayerBase, place_ship),
+   METHOD(LuaPlayerBase, place_pinned_note),
+   {nullptr, nullptr},
 };
 const PropertyType<LuaPlayerBase> LuaPlayerBase::Properties[] = {
    PROP_RO(LuaPlayerBase, number),
@@ -728,18 +734,18 @@ int LuaPlayerBase::place_flag(lua_State* L) {  // NOLINT - can not be made const
 }
 
 /* RST
-   .. method:: place_road(roadtype, start_field, dirs, ...[, force=false])
+   .. method:: place_road(roadtype, start_flag, dirs, ...[, force=false])
 
-      Start a road or waterway at the given field, then walk the **dirs**
+      Start a road or waterway at the given flag, then walk the **dirs**
       given. Places a flag at the last field.
 
       See also the convenience function :meth:`connected_road` in *infrastructure.lua*.
 
       :arg roadtype: One of: :const:`"normal"`, :const:`"busy"`, or :const:`"waterway"`.
       :type roadtype: :class:`string`
-      :arg start_field: The field where to start the road.
-      :type start_field: :class:`~wl.map.Field`
-      :arg dirs: Comma separated list of directions of the road. Allowed values for each
+      :arg start_flag: The flag where to start the road.
+      :type start_flag: :class:`~wl.map.Flag`
+      :arg dirs: Comma separated directions of the road. Allowed values for each
         direction are either:
 
                * directions: :const:`"r"`, :const:`"l"`, :const:`"br"`, :const:`"bl"`,
@@ -751,6 +757,16 @@ int LuaPlayerBase::place_flag(lua_State* L) {  // NOLINT - can not be made const
          in the way are removed and land is conquered. Defaults to :const:`false`.
       :type force: :class:`boolean`
       :returns: The :class:`~wl.map.Road` created.
+
+      Example:
+
+      .. code-block:: lua
+
+         local plr = wl.Game().players[1]
+         local start_field = wl.Game().map:get_field(20,10)
+         local start_flag = pl:place_flag(start_field, true)
+         plr:place_road("normal", start_flag, "br", "br", "r", "r", true)
+
 */
 int LuaPlayerBase::place_road(lua_State* L) {  // NOLINT - can not be made const
 	Widelands::EditorGameBase& egbase = get_egbase(L);
@@ -902,24 +918,31 @@ int LuaPlayerBase::place_building(lua_State* L) {  // NOLINT - can not be made c
 		// If the building belongs to a tribe that no player is playing, we need to load it now
 		const Widelands::DescriptionIndex building_index =
 		   egbase.mutable_descriptions()->load_building(name);
+		const Widelands::BuildingDescr* building_description =
+		   descriptions.get_building_descr(building_index);
 
 		if (!player.tribe().has_building(building_index) &&
-		    descriptions.get_building_descr(building_index)->type() !=
-		       Widelands::MapObjectType::MILITARYSITE) {
+		    building_description->type() != Widelands::MapObjectType::MILITARYSITE) {
 			report_error(L, "Building: '%s' is not available for Player %d's tribe '%s'", name.c_str(),
 			             player.player_number(), player.tribe().name().c_str());
 		}
 
 		Widelands::FormerBuildings former_buildings;
 		find_former_buildings(descriptions, building_index, &former_buildings);
+		Widelands::DescriptionIndex base_building = former_buildings.front().first;
+		uint8_t steps = former_buildings.size() - 1;
 		if (constructionsite) {
-			former_buildings.pop_back();
+			former_buildings.clear();
+			if (building_description->is_buildable()) {
+				steps = 0;
+				base_building = building_index;
+			}
 		}
 
 		Widelands::Building* b = nullptr;
 		if (force) {
 			if (constructionsite) {
-				b = &player.force_csite(c->coords(), building_index, former_buildings);
+				b = &player.force_csite(c->coords(), base_building, former_buildings);
 			} else {
 				b = &player.force_building(c->coords(), former_buildings);
 			}
@@ -933,6 +956,12 @@ int LuaPlayerBase::place_building(lua_State* L) {  // NOLINT - can not be made c
 			report_error(L, "Couldn't %s '%s' at (%d, %d)!", tempname.c_str(), name.c_str(),
 			             c->coords().x, c->coords().y);
 		}
+		if (constructionsite && steps > 0) {
+			Widelands::ConstructionSite* cs = dynamic_cast<Widelands::ConstructionSite*>(b);
+			for (uint8_t i = 0; i < steps; ++i) {
+				cs->enhance(egbase);
+			}
+		}
 
 		LuaMaps::upcasted_map_object_to_lua(L, b);
 	} catch (const Widelands::GameDataError&) {
@@ -968,6 +997,47 @@ int LuaPlayerBase::place_ship(lua_State* L) {  // NOLINT - can not be made const
 	return 1;
 }
 
+/* RST
+   .. method:: place_pinned_note(field, text[, r, g, b])
+
+      .. versionadded:: 1.2
+
+      Place a pinned note on the map for this player.
+
+      :arg field: The field where the note should be placed.
+      :type field: :class:`~wl.map.Field`
+      :arg text: The text of the note.
+      :type text: :class:`string`
+      :arg r: The Red component of the note's color.
+      :type r: :class:`integer`
+      :arg g: The Green component of the note's color.
+      :type g: :class:`integer`
+      :arg b: The Blue component of the note's color.
+      :type b: :class:`integer`
+
+      :returns: The new :class:`~wl.map.PinnedNote` that was created.
+*/
+int LuaPlayerBase::place_pinned_note(lua_State* L) {  // NOLINT - can not be made const
+	if (lua_gettop(L) != 3 && lua_gettop(L) != 6) {
+		report_error(L, "Takes either 2 or 5 arguments");
+	}
+
+	LuaMaps::LuaField* c = *get_user_class<LuaMaps::LuaField>(L, 2);
+
+	Widelands::EditorGameBase& egbase = get_egbase(L);
+	Widelands::Player& player = get(L, egbase);
+
+	Widelands::PinnedNote& note = Widelands::PinnedNote::create(
+	   egbase, player, c->coords(), luaL_checkstring(L, 3),
+	   lua_gettop(L) < 6 ?
+         get(L, get_egbase(L)).get_playercolor() :
+         RGBColor(luaL_checkuint32(L, 4), luaL_checkuint32(L, 5), luaL_checkuint32(L, 6)));
+
+	LuaMaps::upcasted_map_object_to_lua(L, &note);
+
+	return 1;
+}
+
 /* RST
    .. method:: conquer(f[, radius=1])
 
