@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2022 by the Widelands Development Team
+ * Copyright (C) 2006-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -78,7 +78,13 @@ Player
 
       local plr = wl.Game().players[1]                            -- the first player (usually blue)
       local plr = wl.Game().players[2]                            -- the second player
-      local plr = wl.Game().players[wl.Game().interactive_player] -- the interactive player
+      local i_plr
+      for p_idx, player in pairs(wl.Game().players) do
+         if player.number == wl.Game().interactive_player then
+            i_plr = player                                        -- the interactive player
+            break
+         end
+      end
 
 */
 const char LuaPlayer::className[] = "Player";
@@ -117,6 +123,7 @@ const PropertyType<LuaPlayer> LuaPlayer::Properties[] = {
    PROP_RO(LuaPlayer, objectives),
    PROP_RO(LuaPlayer, defeated),
    PROP_RO(LuaPlayer, resigned),
+   PROP_RO(LuaPlayer, end_result),
    PROP_RO(LuaPlayer, messages),
    PROP_RO(LuaPlayer, inbox),
    PROP_RO(LuaPlayer, color),
@@ -173,9 +180,15 @@ int LuaPlayer::get_allowed_buildings(lua_State* L) {
       (RO) A :class:`table` of :class:`objectives <wl.game.Objective>` in form of
       ``{objectivename=objective}``. You can change the objectives in this :class:`table`
       and it will be reflected in the game. To add a new item, use :meth:`add_objective`.
+
+      .. note:: The actual implementation of objectives has a single list that is shared
+         by all players, so it's not possible to have different objectives for different
+         players.
 */
 int LuaPlayer::get_objectives(lua_State* L) {
 	lua_newtable(L);
+	// TODO(tothxa): either make each player have their own objectives vector, or move this
+	//               to LuaMap
 	for (const auto& pair : get_egbase(L).map().objectives()) {
 		lua_pushstring(L, pair.second->name());
 		to_lua<LuaObjective>(L, new LuaObjective(*pair.second));
@@ -202,13 +215,24 @@ int LuaPlayer::get_defeated(lua_State* L) {
       (RO) :const:`true` if this player has resigned, :const:`false` otherwise
 */
 int LuaPlayer::get_resigned(lua_State* L) {
-	for (const auto& s : get_egbase(L).player_manager()->get_players_end_status()) {
-		if (s.player == player_number() && s.result == Widelands::PlayerEndResult::kResigned) {
-			lua_pushboolean(L, 1);
-			return 1;
-		}
-	}
-	lua_pushboolean(L, 0);
+	const Widelands::PlayerEndStatus* p =
+	   get_egbase(L).player_manager()->get_player_end_status(player_number());
+	lua_pushboolean(L, p != nullptr && p->result == Widelands::PlayerEndResult::kResigned ? 1 : 0);
+	return 1;
+}
+
+/* RST
+   .. attribute:: end_result
+
+      .. versionadded:: 1.2
+
+      (RO) This player's end status: :const:`0`: lost, :const:`1`: won, :const:`2`: resigned,
+           :const:`255`: end status is not set
+*/
+int LuaPlayer::get_end_result(lua_State* L) {
+	const Widelands::PlayerEndStatus* p =
+	   get_egbase(L).player_manager()->get_player_end_status(player_number());
+	lua_pushinteger(L, static_cast<int>(p->result));
 	return 1;
 }
 
@@ -555,11 +579,16 @@ int LuaPlayer::message_box(lua_State* L) {
 	}
 #undef CHECK_UINT
 
+	InteractivePlayer* ipl = game.get_ipl();
+	if (ipl != nullptr) {
+		ipl->map_view()->stop_dragging();
+	}
+
 	if (is_modal) {
 		std::unique_ptr<StoryMessageBox> mb(new StoryMessageBox(&game, coords, luaL_checkstring(L, 2),
 		                                                        luaL_checkstring(L, 3), posx, posy, w,
 		                                                        h, is_modal, allow_next_scenario));
-		mb->run<UI::Panel::Returncodes>();
+		NoteThreadSafeFunction::instantiate([&mb]() { mb->run<UI::Panel::Returncodes>(); }, true);
 	} else {
 		new StoryMessageBox(&game, coords, luaL_checkstring(L, 2), luaL_checkstring(L, 3), posx, posy,
 		                    w, h, is_modal, allow_next_scenario);
@@ -641,10 +670,17 @@ int LuaPlayer::forbid_buildings(lua_State* L) {
 /* RST
    .. method:: add_objective(name, title, body)
 
-      Add a new :class:`~wl.game.Objective` for this player. Will report an error, if an
-      Objective with the same name is already registered - note that the names
-      for the objectives are shared internally for all players, so not even
-      another player can have an objective with the same name.
+      Add a new :class:`~wl.game.Objective` for the game. Will report an error, if an objective
+      with the same name is already registered.
+
+      .. note:: Objectives are shared by all players, so it's not possible to have
+         different objectives for different players.
+
+         This function has two kind of users currently:
+
+         1. Scenarios where the objective is only relevant for the interactive player,
+            but it doesn't matter that it's also set for the AI players
+         2. Win conditions where it is assumed that all players have the same goal
 
       :arg name: The name of the objective. Has to be unique.
       :type name: :class:`string`
@@ -657,6 +693,9 @@ int LuaPlayer::forbid_buildings(lua_State* L) {
       :rtype: :class:`wl.game.Objective`
 */
 int LuaPlayer::add_objective(lua_State* L) {
+	// TODO(tothxa): either make each player have their own objectives vector, or move this
+	//               to LuaMap
+
 	Widelands::Game& game = get_game(L);
 
 	Widelands::Map::Objectives* objectives = game.mutable_map()->mutable_objectives();
@@ -714,10 +753,6 @@ int LuaPlayer::reveal_fields(lua_State* L) {
 
       See also :ref:`field_animations` for animated hiding.
 
-      .. note:: Passing a :class:`boolean` as the **state** argument is deprecated.
-         Use :const:`"permanent"` instead of :const:`true` and :const:`"seen"` instead of
-         :const:`false`.
-
       :arg fields: The fields to hide.
       :type fields: :class:`array` of :class:`fields <wl.map.Field>`.
 
@@ -738,11 +773,7 @@ int LuaPlayer::hide_fields(lua_State* L) {
 	Widelands::Player& p = get(L, game);
 
 	luaL_checktype(L, 2, LUA_TTABLE);
-	// TODO(hessenfarmer): Boolean check for compatibility. Remove after v1.0
-	const std::string state = lua_isnone(L, 3)        ? "seen" :
-	                          !lua_isboolean(L, 3)    ? luaL_checkstring(L, 3) :
-	                          luaL_checkboolean(L, 3) ? "permanent" :
-                                                       "seen";
+	const std::string state = lua_isnone(L, 3) ? "seen" : luaL_checkstring(L, 3);
 	const Widelands::HideOrRevealFieldMode mode =
 	   (state == "permanent")  ? Widelands::HideOrRevealFieldMode::kHide :
 	   (state == "explorable") ? Widelands::HideOrRevealFieldMode::kUnexplore :
@@ -909,8 +940,8 @@ int LuaPlayer::get_ships(lua_State* L) {
 
       **which** can be either a single name or an :class:`array` of names. In the first
       case, the method returns an :class:`array` of all Buildings that the player has of
-      this kind. If **which** is an :class:`array`, the function returns a :class:`table` of
-      ``{name=array_of_buildings}`` pairs.
+      this kind. If **which** is an :class:`array` or "all", the function returns a
+      :class:`table` of ``{name=array_of_buildings}`` pairs.
 
       :arg which: The name of a building or an :class:`array` of building names.
       :type which: :class:`string` or :class:`array`
@@ -927,11 +958,11 @@ int LuaPlayer::get_buildings(lua_State* L) {
 
       **which** can be either a single name or an :class:`array` of names. In the first
       case, the method returns an :class:`array` of all constructionsites that the
-      player has of this kind. If which is an :class:`array`, the function returns a
+      player has of this kind. If which is an :class:`array` or "all", the function returns a
       :class:`table` of ``{name=array_of_constructionsites}`` pairs.
 
       :arg which: The internal name of a constructionsites building or an :class:`array` of
-         building names.
+         building names or "all".
       :type which: :class:`string` or :class:`array`
       :returns: Information about the player's constructionsites,
          see :class:`wl.map.ConstructionSite`.
@@ -1028,7 +1059,7 @@ int LuaPlayer::allow_workers(lua_State* L) {
       full control over the player given by **playernumber** and loosing control over the
       formerly interactive player.
 
-      :arg playernumber: An index in the :class:`array` of :attr:`~wl.bases.EditorGameBase.players`.
+      :arg playernumber: The :attr:`wl.bases.PlayerBase.number` of the player to switch to.
       :type playernumber: :class:`integer`
 */
 int LuaPlayer::switchplayer(lua_State* L) {
@@ -1092,7 +1123,7 @@ int LuaPlayer::get_produced_wares_count(lua_State* L) {
       necessarily mean that this player *can* attack the other player, as they might for
       example be in the same team.
 
-      :arg playernumber: An index in the :class:`array` of :attr:`~wl.bases.EditorGameBase.players`.
+      :arg playernumber: The value of :attr:`wl.bases.PlayerBase.number` of the other player.
       :type playernumber: :class:`int`
       :rtype: :class:`boolean`
 */
@@ -1109,7 +1140,7 @@ int LuaPlayer::is_attack_forbidden(lua_State* L) {
       **playernumber**. Note that setting this to :const:`false` does not necessarily mean that this
       player *can* attack the other player, as they might for example be in the same team.
 
-      :arg playernumber: An index in the :class:`array` of :attr:`~wl.bases.EditorGameBase.players`.
+      :arg playernumber: The value of :attr:`wl.bases.PlayerBase.number` of the other player.
       :type playernumber: :class:`int`
       :arg forbid: If this is :const:`true` forbids attacking, :const:`false` allows
          attacking (if the player is not in the same team).
@@ -1185,13 +1216,15 @@ int LuaPlayer::do_get_buildings(lua_State* L, const bool csites) {
 	// parse_building_list
 	bool return_array = true;
 	if (lua_isstring(L, -1) != 0) {
-		const char* name = luaL_checkstring(L, -1);
-		lua_pop(L, 1);
-		lua_newtable(L);
-		lua_pushuint32(L, 1);
-		lua_pushstring(L, name);
-		lua_rawset(L, -3);
-		return_array = false;
+		const std::string name = luaL_checkstring(L, -1);
+		if (name != "all") {
+			lua_pop(L, 1);
+			lua_newtable(L);
+			lua_pushuint32(L, 1);
+			lua_pushstring(L, name);
+			lua_rawset(L, -3);
+			return_array = false;
+		}
 	}
 
 	std::vector<Widelands::DescriptionIndex> houses;
@@ -1232,10 +1265,10 @@ Objective
 
 .. class:: Objective
 
-   This represents an Objective, a goal for the player in the game. This is
+   This represents an Objective, a goal for the player(s) in the game. This is
    mainly for displaying to the user, but each objective also has an attribute
    :attr:`done` which can be set by the scripter to define if this is done. Use
-   :attr:`visible` to hide it from the user.
+   :attr:`visible` to hide it from the users.
 */
 const char LuaObjective::className[] = "Objective";
 const MethodType<LuaObjective> LuaObjective::Methods[] = {
@@ -1268,6 +1301,9 @@ void LuaObjective::__unpersist(lua_State* L) {
 
       (RO) The internal name. You can reference this object via
       :attr:`wl.game.Player.objectives` with :attr:`name` as key.
+
+      .. note:: Although objectives are accessible through :class:`~wl.game.Player`, they are
+         actually shared by all players.
 */
 int LuaObjective::get_name(lua_State* L) {
 	const Widelands::Objective& o = get(L, get_game(L));
@@ -1325,7 +1361,7 @@ int LuaObjective::set_visible(lua_State* L) {
    .. attribute:: done
 
       (RW) Defines if this objective is already fulfilled. If done is
-      :const:`true`, the objective will not be shown to the user, no matter what
+      :const:`true`, the objective will not be shown to the users, no matter what
       :attr:`visible` is set to. A savegame will be created when this attribute
       is changed to :const:`true`.
 
