@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -49,7 +49,7 @@ Button::Button  //  Common constructor
     const std::string& tooltip_text,
     UI::Button::VisualState init_state,
     ImageMode mode)
-   : NamedPanel(parent, to_panel_style(init_style), name, x, y, w, h, tooltip_text),
+   : Panel(parent, to_panel_style(init_style), name, x, y, w, h, tooltip_text),
      highlighted_(false),
      pressed_(false),
      enabled_(true),
@@ -62,6 +62,7 @@ Button::Button  //  Common constructor
      title_image_(title_image),
      button_style_(init_style) {
 	set_thinks(false);
+	set_snap_target(true);
 	set_can_focus(enabled_);
 }
 
@@ -320,12 +321,12 @@ void Button::think() {
 }
 
 bool Button::handle_key(bool down, SDL_Keysym code) {
-	if (down && code.sym == SDLK_RETURN && (SDL_GetModState() & KMOD_CTRL) == 0) {
+	if (down && code.sym == SDLK_RETURN && (code.mod & KMOD_CTRL) == 0) {
 		play_click();
 		sigclicked();
 		return true;
 	}
-	return NamedPanel::handle_key(down, code);
+	return Panel::handle_key(down, code);
 }
 
 /**
