@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
 
 #include "ui_basic/checkbox.h"
 
+#include <cassert>
+
 #include <SDL_mouse.h>
 
 #include "graphic/font_handler.h"
@@ -40,10 +42,11 @@ namespace UI {
  */
 Statebox::Statebox(Panel* const parent,
                    PanelStyle s,
+                   const std::string& name,
                    Vector2i const p,
                    const Image* pic,
                    const std::string& tooltip_text)
-   : Panel(parent, s, p.x, p.y, kStateboxSize, kStateboxSize, tooltip_text),
+   : Panel(parent, s, name, p.x, p.y, kStateboxSize, kStateboxSize, tooltip_text),
      flags_(Is_Enabled),
      pic_graphics_(pic),
      rendered_text_(nullptr) {
@@ -62,11 +65,12 @@ static inline std::string get_checkbox_graphics(const PanelStyle& s) {
 
 Statebox::Statebox(Panel* const parent,
                    PanelStyle s,
+                   const std::string& name,
                    Vector2i const p,
                    const std::string& label_text,
                    const std::string& tooltip_text,
                    int width)
-   : Panel(parent, s, p.x, p.y, std::max(width, kStateboxSize), kStateboxSize, tooltip_text),
+   : Panel(parent, s, name, p.x, p.y, std::max(width, kStateboxSize), kStateboxSize, tooltip_text),
      flags_(Is_Enabled),
      pic_graphics_(g_image_cache->get(get_checkbox_graphics(panel_style_))),
      rendered_text_(nullptr),
@@ -86,11 +90,7 @@ void Statebox::layout() {
 		int w = get_w();
 		int h = kStateboxSize;
 		int pic_width = kStateboxSize;
-		if (pic_graphics_ != nullptr) {
-			w = std::max(pic_graphics_->width(), w);
-			h = pic_graphics_->height();
-			pic_width = pic_graphics_->width();
-		}
+		assert((flags_ & Has_Custom_Picture) == 0);
 		rendered_text_ = label_text_.empty() ?
                           nullptr :
                           UI::g_fh->render(as_richtext_paragraph(
@@ -221,7 +221,7 @@ bool Statebox::handle_mousemove(
 }
 
 bool Statebox::handle_key(bool down, SDL_Keysym code) {
-	if (down && code.sym == SDLK_RETURN && (SDL_GetModState() & KMOD_CTRL) == 0) {
+	if (down && code.sym == SDLK_RETURN && (code.mod & KMOD_CTRL) == 0) {
 		button_clicked();
 		return true;
 	}
