@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2022 by the Widelands Development Team
+ * Copyright (C) 2016-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -62,24 +62,22 @@ BaseDropdown::BaseDropdown(UI::Panel* parent,
                            const DropdownType type,
                            UI::PanelStyle style,
                            ButtonStyle button_style)
-   : UI::NamedPanel(parent,
-                    style,
-                    name,
-                    x,
-                    y,
-                    (type == DropdownType::kPictorial || type == DropdownType::kPictorialMenu) ?
-                       button_dimension :
-                       w,
-                    // Height only to fit the button, so we can use this in Box layout.
-                    base_height(button_dimension, style)),
+   : UI::Panel(parent,
+               style,
+               name,
+               x,
+               y,
+               (type == DropdownType::kPictorial || type == DropdownType::kPictorialMenu) ?
+                  button_dimension :
+                  w,
+               // Height only to fit the button, so we can use this in Box layout.
+               base_height(button_dimension, style)),
      id_(next_id_++),
      max_list_items_(max_list_items),
-     max_list_height_(std::numeric_limits<uint32_t>::max()),
-     list_offset_x_(0),
-     list_offset_y_(0),
+
      base_height_(base_height(button_dimension, style)),
-     mouse_tolerance_(50),
-     button_box_(this, style, 0, 0, UI::Box::Horizontal, w, get_h()),
+
+     button_box_(this, style, "dropdown_button_box", 0, 0, UI::Box::Horizontal, w, get_h()),
      push_button_(type == DropdownType::kTextual ?
                      new UI::Button(&button_box_,
                                     "dropdown_select",
@@ -105,10 +103,10 @@ BaseDropdown::BaseDropdown(UI::Panel* parent,
                      label),
      label_(label),
      type_(type),
-     is_enabled_(true),
-     button_style_(button_style),
 
-     autoexpand_display_button_(false) {
+     button_style_(button_style)
+
+{
 	if (label.empty()) {
 		set_tooltip(pgettext("dropdown", "Select Item"));
 	} else {
@@ -130,11 +128,12 @@ BaseDropdown::BaseDropdown(UI::Panel* parent,
 	while (list_parent->get_parent() != nullptr) {
 		list_parent = list_parent->get_parent();
 	}
-	list_ =
-	   new UI::Listselect<uintptr_t>(list_parent, 0, 0, w, 0, style, ListselectLayout::kDropdown);
+	list_ = new UI::Listselect<uintptr_t>(
+	   list_parent, "list", 0, 0, w, 0, style, ListselectLayout::kDropdown);
 	list_->set_linked_dropdown(this);
 
 	list_->set_visible(false);
+	list_->set_z(UI::Panel::ZOrder::kDropdown);
 	button_box_.add(&display_button_, UI::Box::Resizing::kExpandBoth);
 	display_button_.sigclicked.connect([this]() {
 		toggle_list();
@@ -365,6 +364,10 @@ void BaseDropdown::set_errored(const std::string& error_message) {
 }
 
 void BaseDropdown::set_enabled(bool on) {
+	if (is_enabled_ == on) {
+		return;
+	}
+
 	is_enabled_ = on;
 	set_can_focus(on);
 	if (push_button_ != nullptr) {
@@ -502,7 +505,7 @@ bool BaseDropdown::is_mouse_away() const {
 }
 
 bool BaseDropdown::handle_key(bool down, SDL_Keysym code) {
-	if (down && (SDL_GetModState() & KMOD_CTRL) == 0) {
+	if (down && (code.mod & KMOD_CTRL) == 0) {
 		switch (code.sym) {
 		case SDLK_RETURN:
 			if (list_->is_visible()) {
@@ -532,7 +535,7 @@ bool BaseDropdown::handle_key(bool down, SDL_Keysym code) {
 	if (is_expanded()) {
 		return list_->handle_key(down, code);
 	}
-	return NamedPanel::handle_key(down, code);
+	return Panel::handle_key(down, code);
 }
 void BaseDropdown::delete_last_of_filter() {
 	if (is_filtered()) {
