@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2022 by the Widelands Development Team
+ * Copyright (C) 2016-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,8 +26,8 @@
 
 namespace UI {
 
-FileViewPanel::FileViewPanel(Panel* parent, TabPanelStyle background_style)
-   : TabPanel(parent, background_style), padding_(5), contents_width_(0), contents_height_(0) {
+FileViewPanel::FileViewPanel(Panel* parent, TabPanelStyle background_style, const std::string& name)
+   : TabPanel(parent, background_style, name) {
 	layout();
 
 	sigclicked.connect([this]() { load_tab_contents(); });
@@ -51,13 +51,13 @@ void FileViewPanel::load_tab_contents() {
 }
 
 void FileViewPanel::add_tab(const std::string& title, const std::string& lua_script) {
+	size_t index = boxes_.size();
 	script_paths_.push_back(lua_script);
-	boxes_.push_back(std::unique_ptr<UI::Box>(
-	   new UI::Box(this, panel_style_, 0, 0, UI::Box::Vertical, 0, 0, padding_)));
-	size_t index = boxes_.size() - 1;
+	boxes_.push_back(std::unique_ptr<UI::Box>(new UI::Box(
+	   this, panel_style_, format("box_%" PRIuS, index), 0, 0, UI::Box::Vertical, 0, 0, padding_)));
 
-	UI::MultilineTextarea* textarea =
-	   new UI::MultilineTextarea(boxes_.at(index).get(), 0, 0, Scrollbar::kSize, 0, panel_style_);
+	UI::MultilineTextarea* textarea = new UI::MultilineTextarea(
+	   boxes_.at(index).get(), "text_pane", 0, 0, Scrollbar::kSize, 0, panel_style_);
 
 	textviews_.push_back(std::unique_ptr<UI::MultilineTextarea>(textarea));
 	add(format("about_%" PRIuS, index), title, boxes_.at(index).get(), "");
