@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -77,23 +77,22 @@ inline BaseListselect::EntryRecord::~EntryRecord() {
  *       h
  */
 BaseListselect::BaseListselect(Panel* const parent,
+                               const std::string& name,
                                const int32_t x,
                                const int32_t y,
                                const uint32_t w,
                                const uint32_t h,
                                UI::PanelStyle style,
                                const ListselectLayout selection_mode)
-   : Panel(parent, style, x, y, w, h),
-     widest_text_(0),
-     widest_hotkey_(0),
-     scrollbar_(this, get_w() - Scrollbar::kSize, 0, 0, h, style),
-     scrollpos_(0),
+   : Panel(parent, style, name, x, y, w, h),
+
+     scrollbar_(this, "scrollbar", get_w() - Scrollbar::kSize, 0, 0, h, style),
+
      selection_(no_selection_index()),
-     last_click_time_(-10000),
+
      last_selection_(no_selection_index()),
      selection_mode_(selection_mode),
-     lineheight_(text_height(table_style().enabled()) + kMargin),
-     linked_dropdown(nullptr) {
+     lineheight_(text_height(table_style().enabled()) + kMargin) {
 	set_thinks(false);
 
 	scrollbar_.moved.connect([this](int32_t a) { set_scrollpos(a); });
@@ -143,7 +142,7 @@ void BaseListselect::clear() {
 	scrollbar_.set_steps(1);
 	scrollpos_ = 0;
 	selection_ = no_selection_index();
-	last_click_time_ = -10000;
+	last_click_time_ = std::numeric_limits<uint32_t>::max();
 	last_selection_ = no_selection_index();
 }
 
@@ -242,6 +241,7 @@ void BaseListselect::select(const uint32_t i) {
 		entry_records_[i]->pic = check_pic_;
 	}
 	selection_ = i;
+	scroll_to_selection();
 
 	selected(selection_);
 }
@@ -289,8 +289,13 @@ const Image* BaseListselect::get_selected_image() const {
 	return entry_records_[selection_]->pic;
 }
 
+int BaseListselect::get_lineheight_without_padding() const {
+	return std::max(lineheight_, min_lineheight_);
+}
+
 int BaseListselect::get_lineheight() const {
-	return lineheight_ + (selection_mode_ == ListselectLayout::kDropdown ? 2 * kMargin : kMargin);
+	return get_lineheight_without_padding() +
+	       (selection_mode_ == ListselectLayout::kDropdown ? 2 * kMargin : kMargin);
 }
 
 uint32_t BaseListselect::get_eff_w() const {
@@ -397,6 +402,8 @@ void BaseListselect::draw(RenderTarget& dst) {
 			break;
 		}
 
+		const int lineheight_unpadded = get_lineheight_without_padding();
+
 		Vector2i point(selection_mode_ == ListselectLayout::kDropdown ? 3 : 1, y);
 		uint32_t maxw =
 		   get_eff_w() -
@@ -404,7 +411,7 @@ void BaseListselect::draw(RenderTarget& dst) {
 
 		// Highlight the current selected entry
 		if (idx == selection_) {
-			Recti r(point, maxw, lineheight_);
+			Recti r(point, maxw, lineheight_unpadded);
 			if (r.x < 0) {
 				r.w += r.x;
 				r.x = 0;
@@ -427,15 +434,15 @@ void BaseListselect::draw(RenderTarget& dst) {
 			dst.blit(Vector2i(UI::g_fh->fontset()->is_rtl() ?
                               get_eff_w() - er.pic->width() - 1 - kIndentStrength * er.indent :
                               kIndentStrength * er.indent + 1,
-			                  y + (lineheight_ - er.pic->height()) / 2),
+			                  y + (lineheight_unpadded - er.pic->height()) / 2),
 			         er.pic);
 		}
 
 		// Fix vertical position for mixed font heights
 		if (get_lineheight() > txt_height) {
-			point.y += (lineheight_ - txt_height) / 2;
+			point.y += (lineheight_unpadded - txt_height) / 2;
 		} else {
-			point.y -= (txt_height - lineheight_) / 2;
+			point.y -= (txt_height - lineheight_unpadded) / 2;
 		}
 
 		// Don't draw over the bottom edge
@@ -490,6 +497,10 @@ void BaseListselect::draw(RenderTarget& dst) {
  * Handle mouse wheel events
  */
 bool BaseListselect::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
+	if (!select_with_wheel_) {
+		return scrollbar_.handle_mousewheel(x, y, modstate);
+	}
+
 	const uint32_t selected_idx = selection_index();
 	uint32_t max = size();
 	if (max > 0) {
@@ -497,6 +508,7 @@ bool BaseListselect::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate)
 	} else {
 		return false;
 	}
+
 	if (y != 0 && matches_keymod(modstate, KMOD_NONE)) {
 		if (selected_idx > max) {
 			select(y < 0 ? 0 : max);
@@ -505,8 +517,10 @@ bool BaseListselect::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate)
 		} else if (y < 0 && selected_idx < max) {
 			select(selected_idx + 1);
 		}
+
 		return scrollbar_.handle_mousewheel(x, y, modstate);
 	}
+
 	return false;
 }
 
@@ -540,8 +554,8 @@ bool BaseListselect::handle_mousepress(const uint8_t btn, int32_t /*x*/, int32_t
 		clicked();
 
 		if  //  check if doubleclicked
-		   (time - real_last_click_time < DOUBLE_CLICK_INTERVAL && last_selection_ == selection_ &&
-		    selection_ != no_selection_index()) {
+		   (time >= real_last_click_time && time - real_last_click_time < DOUBLE_CLICK_INTERVAL &&
+		    last_selection_ == selection_ && selection_ != no_selection_index()) {
 			double_clicked(selection_);
 		}
 
@@ -553,17 +567,17 @@ bool BaseListselect::handle_mousepress(const uint8_t btn, int32_t /*x*/, int32_t
 }
 
 bool BaseListselect::handle_mousemove(
-   uint8_t /*state*/, int32_t /*x*/, int32_t y, int32_t /*xdiff*/, int32_t /*ydiff*/) {
+   uint8_t state, int32_t x, int32_t y, int32_t xdiff, int32_t ydiff) {
 	y = (y + scrollpos_) / get_lineheight();
 	if (y < 0 || static_cast<int32_t>(entry_records_.size()) <= y) {
 		set_tooltip("");
-		return false;
+		return Panel::handle_mousemove(state, x, y, xdiff, ydiff);
 	}
 	if (selection_mode_ == ListselectLayout::kDropdown) {
 		select(y);
 	}
 	set_tooltip(entry_records_.at(y)->tooltip);
-	return true;
+	return Panel::handle_mousemove(state, x, y, xdiff, ydiff);
 }
 
 bool BaseListselect::handle_key(bool const down, SDL_Keysym const code) {
@@ -577,7 +591,7 @@ bool BaseListselect::handle_key(bool const down, SDL_Keysym const code) {
 			return UI::Panel::handle_key(down, code);
 		case SDLK_ESCAPE:
 		case SDLK_RETURN:
-			if (linked_dropdown != nullptr && (SDL_GetModState() & KMOD_CTRL) == 0) {
+			if (linked_dropdown != nullptr && (code.mod & KMOD_CTRL) == 0) {
 				return linked_dropdown->handle_key(down, code);
 			}
 			return UI::Panel::handle_key(down, code);
@@ -624,20 +638,27 @@ bool BaseListselect::handle_key(bool const down, SDL_Keysym const code) {
 		assert((selected_idx <= max) ^ (selected_idx == no_selection_index()));
 		if (handle) {
 			select(selected_idx);
-			if (selection_index() * get_lineheight() < scrollpos_) {
-				scrollpos_ = selection_index() * get_lineheight();
-				scrollbar_.set_scrollpos(scrollpos_);
-			} else if ((selected_idx + 1) * get_lineheight() - get_inner_h() > scrollpos_) {
-				int32_t scrollpos = (selection_index() + 1) * get_lineheight() - get_inner_h();
-				scrollpos_ = (scrollpos < 0) ? 0 : scrollpos;
-				scrollbar_.set_scrollpos(scrollpos_);
-			}
 			return true;
 		}
 	}
 	return UI::Panel::handle_key(down, code);
 }
 
+void BaseListselect::scroll_to_selection() {
+	if (!has_selection()) {
+		return;
+	}
+	if (selection_index() * get_lineheight() < scrollpos_) {
+		scrollpos_ = selection_index() * get_lineheight();
+		scrollbar_.set_scrollpos(scrollpos_);
+	} else if (static_cast<int>((selection_index() + 1) * get_lineheight() - get_inner_h()) >
+	           static_cast<int>(scrollpos_)) {
+		int32_t scrollpos = (selection_index() + 1) * get_lineheight() - get_inner_h();
+		scrollpos_ = (scrollpos < 0) ? 0 : scrollpos;
+		scrollbar_.set_scrollpos(scrollpos_);
+	}
+}
+
 /**
  * Remove entry
  */
@@ -692,7 +713,8 @@ Recti BaseListselect::get_highlight_rect(const std::string& text, int x, int y)
 	   UI::g_fh->render(as_richtext_paragraph(
 	      richtext_escape(text2.substr(start, filter.length())), table_style().enabled()));
 
-	Recti highlight_rect(x + rendered_start->width(), y, rendered_substring->width(), lineheight_);
+	Recti highlight_rect(x + rendered_start->width(), y, rendered_substring->width(),
+	                     get_lineheight_without_padding());
 
 	return highlight_rect;
 }
