@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -49,8 +49,11 @@ Recti Panel::tooltip_fixed_rect_ = Recti(0, 0, 0, 0);
 // events are ignored and not passed on to any widget. This is only useful
 // for scripts that want to show off functionality without the user interfering.
 bool Panel::allow_user_input_ = true;
+bool Panel::allow_fastclick_ = true;
 FxId Panel::click_fx_ = kNoSoundEffect;
 
+uint32_t Panel::time_of_last_user_activity_ = 0U;
+
 inline static bool tooltip_accessibility_mode() {
 	return get_config_bool("tooltip_accessibility_mode", false);
 }
@@ -60,34 +63,35 @@ inline static bool tooltip_accessibility_mode() {
  */
 Panel::Panel(Panel* const nparent,
              const PanelStyle s,
+             const std::string& name,
              const int nx,
              const int ny,
              const int nw,
              const int nh,
              const std::string& tooltip_text)
    : panel_style_(s),
-     initialized_(false),
+
      parent_(nparent),
-     first_child_(nullptr),
-     last_child_(nullptr),
-     mousein_child_(nullptr),
-     focus_(nullptr),
+
      flags_(pf_handle_mouse | pf_thinks | pf_visible | pf_handle_keypresses),
+     name_(name),
      x_(nx),
      y_(ny),
      w_(nw),
      h_(nh),
-     lborder_(0),
-     rborder_(0),
-     tborder_(0),
-     bborder_(0),
-     border_snap_distance_(0),
-     panel_snap_distance_(0),
+
      desired_w_(nw),
      desired_h_(nh),
-     running_(false),
+
      tooltip_(tooltip_text),
+     hyperlink_subscriber_(
+        Notifications::subscribe<NoteHyperlink>([this](const NoteHyperlink& note) {
+	        if (starts_with(name_, note.target)) {
+		        handle_hyperlink(note.action);
+	        }
+        })),
      logic_thread_locked_(LogicThreadState::kEndingConfirmed) {
+	assert(!name.empty());
 	assert(nparent != this);
 	if (parent_ != nullptr) {
 		next_ = parent_->first_child_;
@@ -169,6 +173,8 @@ void Panel::free_children() {
 }
 
 Panel::ModalGuard::ModalGuard(Panel& p) : bottom_panel_(Panel::modal_), top_panel_(p) {
+	MutexLock::push_stay_responsive_function([&p]() { p.handle_notes(); });
+
 	if (Panel::modal_ != nullptr) {
 		/* Clean up stale notes first. */
 		Panel::modal_.load()->handle_notes();
@@ -176,6 +182,8 @@ Panel::ModalGuard::ModalGuard(Panel& p) : bottom_panel_(Panel::modal_), top_pane
 	Panel::modal_ = &top_panel_;
 }
 Panel::ModalGuard::~ModalGuard() {
+	MutexLock::pop_stay_responsive_function();
+
 	Panel::modal_ = bottom_panel_;
 	if (bottom_panel_ != nullptr) {
 		bottom_panel_->become_modal_again(top_panel_);
@@ -190,9 +198,9 @@ constexpr uint32_t kGameLogicDelay = 50;
 void Panel::logic_thread() {
 	set_logic_thread();
 	logic_thread_running_ = true;
-	WLApplication* const app = WLApplication::get();
+	const WLApplication& app = WLApplication::get();
 
-	while (!app->should_die()) {
+	while (!app.should_die()) {
 		Panel* m =
 		   modal_;  // copy this because another panel may become modal during a lengthy logic frame
 
@@ -202,7 +210,16 @@ void Panel::logic_thread() {
 			       lock_if_free, LogicThreadState::kLocked)) {
 				MutexLock lock(MutexLock::ID::kLogicFrame);
 
-				m->game_logic_think();  // actual game logic
+				try {
+					m->game_logic_think();  // actual game logic
+				} catch (const WException& e) {
+					// Forward uncaught exceptions to the main thread's handler.
+					NoteThreadSafeFunction::instantiate(
+					   [&e]() { throw WException::copy(e); }, true, false);
+				} catch (const std::exception& e) {
+					NoteThreadSafeFunction::instantiate([&e]() { throw e; }, true, false);
+				}
+
 				LogicThreadState free_if_locked = LogicThreadState::kLocked;
 				m->logic_thread_locked_.compare_exchange_strong(
 				   free_if_locked, LogicThreadState::kFree);
@@ -224,11 +241,13 @@ void Panel::handle_notes() {
 			// If there are multiple modal panels, ensure each note is handled only once
 			Notifications::publish(NoteThreadSafeFunctionHandled(notes_.front().id));
 
-			notes_.front().run();
+			NoteThreadSafeFunction note = notes_.front();
+			notes_.pop_front();
+			note.run();
 		} else {
 			handled_notes_.erase(notes_.front().id);
+			notes_.pop_front();
 		}
-		notes_.pop_front();
 	}
 }
 
@@ -243,19 +262,19 @@ Panel& Panel::get_topmost_forefather() {
 void Panel::do_redraw_now(const bool handle_input, const std::string& message) {
 	assert(is_initializer_thread());
 
-	WLApplication* const app = WLApplication::get();
+	WLApplication& app = WLApplication::get();
 	static InputCallback input_callback = {Panel::ui_mousepress, Panel::ui_mouserelease,
 	                                       Panel::ui_mousemove,  Panel::ui_key,
 	                                       Panel::ui_textinput,  Panel::ui_mousewheel};
 	if (handle_input && message.empty()) {
-		app->handle_input(&input_callback);
+		app.handle_input(&input_callback);
 	}
 
 	Panel& ff = get_topmost_forefather();
 	RenderTarget& rt = *g_gr->get_render_target();
 
 	{
-		MutexLock m(MutexLock::ID::kObjects, [this]() { handle_notes(); });
+		MutexLock m(MutexLock::ID::kObjects);
 		ff.do_draw(rt);
 	}
 
@@ -264,21 +283,20 @@ void Panel::do_redraw_now(const bool handle_input, const std::string& message) {
 		// until the logic frame has ended. During this time, we no longer
 		// handle input, and we gray out the user interface to indicate this.
 
-		rt.tile(Recti(0, 0, g_gr->get_xres(), g_gr->get_yres()),
-		        &load_safe_template_image("loadscreens/ending.png"), Vector2i(0, 0));
+		rt.tile(rt.get_rect(), g_image_cache->get("loadscreens/ending.png"), Vector2i(0, 0));
 
-		draw_game_tip(rt, Recti(0, 0, g_gr->get_xres(), g_gr->get_yres()), message, 2);
+		draw_game_tip(rt, rt.get_rect(), message, 2);
 	}
 
 	if (g_mouse_cursor->is_visible()) {
-		g_mouse_cursor->change_cursor(app->is_mouse_pressed());
-		g_mouse_cursor->draw(rt, app->get_mouse_position());
+		g_mouse_cursor->change_cursor(app.is_mouse_pressed());
+		g_mouse_cursor->draw(rt, app.get_mouse_position());
 
 		// Tooltip magic. Some panels never want to show a tooltip, and if the display an
 		// overlay message we ignore user input and don't draw tooltips any more either.
 		// When deciding which panel gets to draw a tooltip (if allowed), modal panels and
 		// their children take precedence over a (potentially non-modal) toplevel panel.
-		if (message.empty() && (flags_ & pf_hide_all_overlays) == 0) {
+		if (message.empty() && !get_flag(pf_hide_all_overlays)) {
 			if (modal_ != nullptr) {
 				modal_.load()->do_tooltip();
 			} else if (is_modal()) {
@@ -342,13 +360,14 @@ int Panel::do_run() {
 	   LogicThreadState::kEndingConfirmed;  // don't start the logic thread ere we're ready
 
 	// TODO(sirver): the main loop should not be in UI, but in WLApplication.
-	WLApplication* const app = WLApplication::get();
+	WLApplication& app = WLApplication::get();
 	ModalGuard prevmodal(*this);
-	mousegrab_ = nullptr;        // good ol' paranoia
-	app->set_mouse_lock(false);  // more paranoia :-)
+	mousegrab_ = nullptr;       // good ol' paranoia
+	app.set_mouse_lock(false);  // more paranoia :-)
 
 	// With the default of 30FPS, the game will be drawn every 33ms.
-	const uint32_t draw_delay = 1000 / std::max(5, get_config_int("maxfps", 30));
+	constexpr uint32_t kMaxFPS = 30;
+	constexpr uint32_t kDrawDelay = 1000 / kMaxFPS;
 
 	static InputCallback input_callback = {Panel::ui_mousepress, Panel::ui_mouserelease,
 	                                       Panel::ui_mousemove,  Panel::ui_key,
@@ -386,18 +405,24 @@ int Panel::do_run() {
 		}
 
 		if (is_initializer) {
-			app->handle_input(&input_callback);
+			app.handle_input(&input_callback);
 		}
 
 		if (start_time >= next_time) {
-			if (app->should_die()) {
+			if (app.should_die()) {
 				end_modal<Returncodes>(Returncodes::kBack);
 				assert(!running_);
 			}
 
 			{
-				current_think_mutex_.reset(
-				   new MutexLock(MutexLock::ID::kObjects, [this]() { handle_notes(); }));
+				// TODO(tothxa): This is overbroad locking. Should be refined, but that needs
+				//               a lot of hunting down problems.
+				//               Since the addition of Lua plugin timers, which run their actions
+				//               in the UI thread, this requires that all Lua entry points are
+				//               guarded first by the kObjects, then by the kLua locks, otherwise
+				//               deadlocks may occur when Lua code calls functions that need
+				//               kObjects, due to the different locking order.
+				current_think_mutex_.reset(new MutexLock(MutexLock::ID::kObjects));
 				do_think();
 				current_think_mutex_.reset();
 			}
@@ -408,7 +433,7 @@ int Panel::do_run() {
 				do_redraw_now();
 			}
 
-			next_time = start_time + draw_delay;
+			next_time = start_time + kDrawDelay;
 		}
 
 		const int32_t delay = next_time - SDL_GetTicks();
@@ -422,7 +447,7 @@ int Panel::do_run() {
 	// so we continue refreshing the graphics while we wait.
 	if (logic_thread_locked_ != LogicThreadState::kEndingConfirmed && logic_thread_running_) {
 		logic_thread_locked_ = LogicThreadState::kEndingRequested;
-		while (((flags_ & pf_logic_think) != 0u) && logic_thread_running_ &&
+		while (get_flag(pf_logic_think) && logic_thread_running_ &&
 		       logic_thread_locked_ != LogicThreadState::kEndingConfirmed) {
 			const uint32_t start_time = SDL_GetTicks();
 
@@ -432,7 +457,7 @@ int Panel::do_run() {
 				do_redraw_now(true, _("Game ending – please wait…"));
 			}
 
-			next_time = start_time + draw_delay;
+			next_time = start_time + kDrawDelay;
 			const int32_t delay = next_time - SDL_GetTicks();
 			if (delay > 0) {
 				SDL_Delay(delay);
@@ -550,22 +575,6 @@ void Panel::set_desired_size(int w, int h) {
 void Panel::update_desired_size() {
 }
 
-/**
- * Set whether this panel acts as a layouting toplevel.
- *
- * Typically, only true for \ref Window.
- */
-void Panel::set_layout_toplevel(bool ltl) {
-	flags_ &= ~pf_layout_toplevel;
-	if (ltl) {
-		flags_ |= pf_layout_toplevel;
-	}
-}
-
-bool Panel::get_layout_toplevel() const {
-	return (flags_ & pf_layout_toplevel) != 0u;
-}
-
 /**
  * Interpret \p pt as a point in the interior of this panel,
  * and translate it into the interior coordinate system of the parent
@@ -631,10 +640,22 @@ int Panel::get_inner_h() const {
 /**
  * Make this panel the top-most panel in the parent's Z-order.
  */
-void Panel::move_to_top() {
+void Panel::move_to_top(const bool on_top_of_equal_z) {
 	if (parent_ == nullptr) {
 		return;
 	}
+	/* If all siblings above us are permanently on top, skip. */
+	bool all_on_top = true;
+	for (Panel* p = parent_->first_child_; p != this && p != nullptr && all_on_top; p = p->next_) {
+		if (on_top_of_equal_z) {
+			all_on_top &= (static_cast<uint8_t>(p->get_z()) > static_cast<uint8_t>(get_z()));
+		} else {
+			all_on_top &= (static_cast<uint8_t>(p->get_z()) >= static_cast<uint8_t>(get_z()));
+		}
+	}
+	if (all_on_top) {
+		return;
+	}
 
 	// unlink
 	if (prev_ != nullptr) {
@@ -663,14 +684,12 @@ void Panel::move_to_top() {
  * Makes the panel visible or invisible
  */
 void Panel::set_visible(bool const on) {
-	if (((flags_ & pf_visible) > 1) == on) {
+	if (get_flag(pf_visible) == on) {
 		return;
 	}
 
-	flags_ &= ~pf_visible;
-	if (on) {
-		flags_ |= pf_visible;
-	} else if ((parent_ != nullptr) && parent_->focus_ == this) {
+	set_flag(pf_visible, on);
+	if (!on && (parent_ != nullptr) && parent_->focus_ == this) {
 		parent_->focus_ = nullptr;
 	}
 	if (parent_ != nullptr) {
@@ -721,14 +740,8 @@ std::vector<Recti> Panel::focus_overlay_rects() {
  */
 void Panel::draw_overlay(RenderTarget& dst) {
 	if (has_focus()) {
-		for (Panel* p = this; p->parent_ != nullptr; p = p->parent_) {
-			if (p->parent_->focus_ != p) {
-				// doesn't have toplevel focus
-				return;
-			}
-			if (p->parent_->is_focus_toplevel()) {
-				break;
-			}
+		if (!has_top_level_focus()) {
+			return;
 		}
 		for (const Recti& r : focus_overlay_rects()) {
 			dst.fill_rect(r,
@@ -738,6 +751,18 @@ void Panel::draw_overlay(RenderTarget& dst) {
 		}
 	}
 }
+bool Panel::has_top_level_focus() {
+	for (Panel* p = this; p->parent_ != nullptr; p = p->parent_) {
+		if (p->parent_->focus_ != p) {
+			// doesn't have toplevel focus
+			return false;
+		}
+		if (p->parent_->is_focus_toplevel()) {
+			return true;
+		}
+	}
+	return true;
+}
 
 /**
  * Draw texture and color from the info if they have been specified.
@@ -779,6 +804,8 @@ void Panel::do_think() {
 		return;
 	}
 
+	move_to_top(false);
+
 	if (thinks()) {
 		think();
 
@@ -791,7 +818,7 @@ void Panel::do_think() {
 	}
 
 	// think() may have called die()
-	if ((flags_ & pf_die) != 0u) {
+	if (get_flag(pf_die)) {
 		return;
 	}
 
@@ -805,7 +832,7 @@ void Panel::do_think() {
  */
 Vector2i Panel::get_mouse_position() const {
 	return (parent_ != nullptr ? parent_->get_mouse_position() :
-                                WLApplication::get()->get_mouse_position()) -
+                                WLApplication::get().get_mouse_position()) -
 	       Vector2i(get_x() + get_lborder(), get_y() + get_tborder());
 }
 
@@ -817,7 +844,7 @@ void Panel::set_mouse_pos(const Vector2i p) {
 	if (parent_ != nullptr) {
 		parent_->set_mouse_pos(relative_p);
 	} else {
-		WLApplication::get()->warp_mouse(relative_p);
+		WLApplication::get().warp_mouse(relative_p);
 	}
 }
 
@@ -894,7 +921,7 @@ bool Panel::handle_key(bool down, SDL_Keysym code) {
 		}
 		switch (code.sym) {
 		case SDLK_TAB:
-			return handle_tab_pressed((SDL_GetModState() & KMOD_SHIFT) != 0);
+			return handle_tab_pressed((code.mod & KMOD_SHIFT) != 0);
 		case SDLK_ESCAPE:
 			if ((parent_ != nullptr) && parent_->focus_ == this && get_can_focus()) {
 				parent_->focus_ = nullptr;
@@ -980,21 +1007,6 @@ std::deque<Panel*> Panel::gather_focusable_children() {
 	return list;
 }
 
-/**
- * Enable/Disable mouse handling by this panel
- * Default is enabled. Note that when mouse handling is disabled, child panels
- * don't receive mouse events either.
- *
- * \param yes true if the panel should receive mouse events
- */
-void Panel::set_handle_mouse(bool const yes) {
-	if (yes) {
-		flags_ |= pf_handle_mouse;
-	} else {
-		flags_ &= ~pf_handle_mouse;
-	}
-}
-
 /**
  * Enable/Disable mouse grabbing. If a panel grabs the mouse, all mouse
  * related events will be sent directly to that panel.
@@ -1016,15 +1028,9 @@ void Panel::grab_mouse(bool const grab) {
  * Set if this panel can receive the keyboard focus
  */
 void Panel::set_can_focus(bool const yes) {
-
-	if (yes) {
-		flags_ |= pf_can_focus;
-	} else {
-		flags_ &= ~pf_can_focus;
-
-		if ((parent_ != nullptr) && parent_->focus_ == this) {
-			parent_->focus_ = nullptr;
-		}
+	set_flag(pf_can_focus, yes);
+	if (!yes && (parent_ != nullptr) && parent_->focus_ == this) {
+		parent_->focus_ = nullptr;
 	}
 }
 
@@ -1057,20 +1063,6 @@ void Panel::focus(const bool topcaller) {
 	parent_->focus(false);
 }
 
-/**
- * Enables/Disables calling think() during the event loop.
- * The default is enabled.
- *
- * \param yes true if the panel's think function should be called
- */
-void Panel::set_thinks(bool const yes) {
-	if (yes) {
-		flags_ |= pf_thinks;
-	} else {
-		flags_ &= ~pf_thinks;
-	}
-}
-
 /**
  * Cause this panel to be removed on the next frame.
  * Use this for a panel that needs to destroy itself after a button has
@@ -1080,11 +1072,11 @@ void Panel::set_thinks(bool const yes) {
 void Panel::die() {
 	initialized_ = false;
 
-	flags_ &= ~pf_visible;
-	flags_ |= pf_die;
+	set_flag(pf_visible, false);
+	set_flag(pf_die, true);
 
 	for (Panel* p = parent_; p != nullptr; p = p->parent_) {
-		p->flags_ |= pf_child_die;
+		p->set_flag(pf_child_die, true);
 		if (p == modal_) {
 			break;
 		}
@@ -1131,15 +1123,15 @@ void Panel::check_child_death() {
 		Panel* p = next;
 		next = p->next_;
 
-		if ((p->flags_ & pf_die) != 0u) {
+		if (p->get_flag(pf_die)) {
 			p->do_delete();
 			p = nullptr;
-		} else if ((p->flags_ & pf_child_die) != 0u) {
+		} else if (p->get_flag(pf_child_die)) {
 			p->check_child_death();
 		}
 	}
 
-	flags_ &= ~pf_child_die;
+	set_flag(pf_child_die, false);
 }
 
 /**
@@ -1180,8 +1172,12 @@ void Panel::do_draw(RenderTarget& dst) {
 	}
 
 	// Make sure the panel's size is sane. If it's bigger than 10000 it's likely a bug.
-	assert(desired_w_ <= std::max(10000, g_gr->get_xres()));
-	assert(desired_h_ <= std::max(10000, g_gr->get_yres()));
+#ifndef NDEBUG
+	if (!get_flag(pf_unlimited_size)) {
+		assert(desired_w_ <= std::max(10000, g_gr->get_xres()));
+		assert(desired_h_ <= std::max(10000, g_gr->get_yres()));
+	}
+#endif
 
 	Recti outerrc;
 	Vector2i outerofs = Vector2i::zero();
@@ -1190,6 +1186,17 @@ void Panel::do_draw(RenderTarget& dst) {
 		return;
 	}
 
+	if (modal_.load() == this) {
+		// Darken out everything behind the modal window.
+		const Recti rect = dst.get_rect();
+		const Vector2i offset = dst.get_offset();
+		dst.set_window(Recti(Vector2i::zero(), g_gr->get_xres(), g_gr->get_yres()), Vector2i::zero());
+
+		dst.tile(dst.get_rect(), g_image_cache->get("loadscreens/ending.png"), Vector2i(0, 0));
+
+		dst.set_window(rect, offset);
+	}
+
 	draw_border(dst);
 
 	Recti innerwindow(
@@ -1215,11 +1222,25 @@ void Panel::set_tooltip(const std::string& text) {
 			tooltip_fixed_pos_ = Vector2i::invalid();
 		} else if (!text.empty()) {
 			tooltip_panel_ = this;
-			tooltip_fixed_pos_ = WLApplication::get()->get_mouse_position();
+			tooltip_fixed_pos_ = WLApplication::get().get_mouse_position();
 		}
 	}
 }
 
+Panel* Panel::find_child_by_name(const std::string& name, bool recurse) {
+	for (Panel* child = first_child_; child != nullptr; child = child->next_) {
+		if (child->get_name() == name) {
+			return child;
+		}
+		if (recurse) {
+			if (Panel* np = child->find_child_by_name(name, true); np != nullptr) {
+				return np;
+			}
+		}
+	}
+	return nullptr;
+}
+
 void Panel::find_all_children_at(const int16_t x,
                                  const int16_t y,
                                  std::vector<Panel*>& result) const {
@@ -1240,13 +1261,12 @@ void Panel::find_all_children_at(const int16_t x,
 inline Panel* Panel::child_at_mouse_cursor(int32_t const x, int32_t const y, Panel* child) {
 
 	for (; child != nullptr; child = child->next_) {
-		if (!child->handles_mouse() || !child->is_visible()) {
+		if (!child->is_visible() || x >= child->x_ + static_cast<int32_t>(child->w_) ||
+		    x < child->x_ || y >= child->y_ + static_cast<int32_t>(child->h_) || y < child->y_ ||
+		    !child->check_handles_mouse(x - child->x_, y - child->y_)) {
 			continue;
 		}
-		if (x < child->x_ + static_cast<int32_t>(child->w_) && x >= child->x_ &&
-		    y < child->y_ + static_cast<int32_t>(child->h_) && y >= child->y_) {
-			break;
-		}
+		break;
 	}
 
 	if ((mousein_child_ != nullptr) && mousein_child_ != child) {
@@ -1279,7 +1299,7 @@ void Panel::do_mousein(bool const inside) {
 		    ((tooltip_panel_ == nullptr) || tooltip_fixed_pos_ == Vector2i::invalid()) &&
 		    !tooltip().empty()) {
 			tooltip_panel_ = this;
-			tooltip_fixed_pos_ = WLApplication::get()->get_mouse_position();
+			tooltip_fixed_pos_ = WLApplication::get().get_mouse_position();
 		} else if (!inside && tooltip_panel_ == this && tooltip_fixed_pos_ == Vector2i::invalid()) {
 			tooltip_panel_ = nullptr;
 		}
@@ -1303,7 +1323,7 @@ bool Panel::do_mousepress(const uint8_t btn, int32_t x, int32_t y) {
 	}
 	x -= lborder_;
 	y -= tborder_;
-	if ((flags_ & pf_top_on_click) != 0u) {
+	if (get_flag(pf_top_on_click)) {
 		move_to_top();
 	}
 
@@ -1325,13 +1345,11 @@ bool Panel::do_mousewheel(int32_t x, int32_t y, uint16_t modstate, Vector2i rel_
 
 	// Check if a child-panel is beneath the mouse and processes the event
 	for (Panel* child = first_child_; child != nullptr; child = child->next_) {
-		if (!child->handles_mouse() || !child->is_visible()) {
-			continue;
-		}
-		if (rel_mouse_pos.x >= child->x_ + static_cast<int32_t>(child->w_) ||
+		if (!child->is_visible() || rel_mouse_pos.x >= child->x_ + static_cast<int32_t>(child->w_) ||
 		    rel_mouse_pos.x < child->x_ ||
 		    rel_mouse_pos.y >= child->y_ + static_cast<int32_t>(child->h_) ||
-		    rel_mouse_pos.y < child->y_) {
+		    rel_mouse_pos.y < child->y_ ||
+		    !child->check_handles_mouse(x - child->x_, y - child->y_)) {
 			continue;
 		}
 		// Found a child at the position
@@ -1473,7 +1491,7 @@ bool Panel::do_tooltip() {
  * \return \c true if the given key is currently pressed, or \c false otherwise
  */
 bool Panel::get_key_state(const SDL_Scancode key) const {
-	return WLApplication::get()->get_key_state(key);
+	return WLApplication::get().get_key_state(key);
 }
 
 UI::Panel* Panel::get_open_dropdown() {
@@ -1537,6 +1555,10 @@ Panel* Panel::ui_trackmouse(int32_t& x, int32_t& y) {
 	return rcv;
 }
 
+inline void Panel::register_user_activity() {
+	time_of_last_user_activity_ = SDL_GetTicks();
+}
+
 /**
  * Input callback function. Pass the mouseclick event to the currently modal
  * panel.
@@ -1560,6 +1582,8 @@ bool Panel::ui_mousepress(const uint8_t button, int32_t x, int32_t y) {
 	if (p == nullptr) {
 		return false;
 	}
+
+	register_user_activity();
 	return p->do_mousepress(button, x, y);
 }
 
@@ -1572,6 +1596,8 @@ bool Panel::ui_mouserelease(const uint8_t button, int32_t x, int32_t y) {
 	if (p == nullptr) {
 		return false;
 	}
+
+	register_user_activity();
 	return p->do_mouserelease(button, x, y);
 }
 
@@ -1594,7 +1620,7 @@ bool Panel::ui_mousemove(uint8_t const state, int32_t x, int32_t y, int32_t xdif
 	}
 
 	int factor = 1;
-	if (WLApplication::get()->is_mouse_locked()) {
+	if (WLApplication::get().is_mouse_locked()) {
 		if (matches_keymod(SDL_GetModState(), KMOD_CTRL)) {
 			factor = 4;
 		} else if (!matches_keymod(SDL_GetModState(), KMOD_SHIFT)) {
@@ -1625,6 +1651,8 @@ bool Panel::ui_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
 	if (p == nullptr) {
 		return false;
 	}
+
+	register_user_activity();
 	return p->do_mousewheel(x, y, modstate, p->get_mouse_position());
 }
 
@@ -1644,6 +1672,8 @@ bool Panel::ui_key(bool const down, SDL_Keysym const code) {
 			p = dd;
 		}
 	}
+
+	register_user_activity();
 	return p->do_key(down, code);
 }
 
@@ -1657,6 +1687,8 @@ bool Panel::ui_textinput(const std::string& text) {
 	if (modal_ == nullptr) {
 		return false;
 	}
+
+	register_user_activity();
 	return modal_.load()->do_textinput(text);
 }
 
@@ -1699,7 +1731,7 @@ bool Panel::draw_tooltip(const std::string& text, const PanelStyle style, Vector
 	const uint16_t tip_height = rendered_text->height() + kPadding;
 
 	if (pos == Vector2i::invalid()) {
-		pos = WLApplication::get()->get_mouse_position();
+		pos = WLApplication::get().get_mouse_position();
 	}
 	tooltip_fixed_rect_ = Recti(pos + Vector2i(2, kCursorHeight), tip_width, tip_height);
 	const Vector2i tooltip_bottom_right = tooltip_fixed_rect_.opposite_of_origin();
@@ -1718,4 +1750,9 @@ bool Panel::draw_tooltip(const std::string& text, const PanelStyle style, Vector
 	rendered_text->draw(dst, tooltip_fixed_rect_.origin() + Vector2i(2, 2));
 	return true;
 }
+
+void Panel::handle_hyperlink(const std::string& action) {
+	throw wexception("Panel %s: Invalid hyperlink action '%s'", name_.c_str(), action.c_str());
+}
+
 }  // namespace UI
