@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2022 by the Widelands Development Team
+ * Copyright (C) 2004-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,17 +31,16 @@ namespace UI {
  */
 ProgressBar::ProgressBar(Panel* const parent,
                          PanelStyle style,
+                         const std::string& name,
                          int32_t const x,
                          int32_t const y,
                          int32_t const w,
                          int32_t const h,
                          uint32_t const orientation)
-   : Panel(parent, style, x, y, w, h),
+   : Panel(parent, style, name, x, y, w, h),
      orientation_(orientation),
-     state_(0),
-     total_(100),
-     progress_style_(style),
-     show_percent_(true) {
+
+     progress_style_(style) {
 }
 
 inline const UI::ProgressbarStyleInfo& ProgressBar::progress_style() const {
