@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,28 +41,29 @@ namespace UI {
  *       w       dimensions, in pixels, of the Table
  *       h
  */
-Table<void*>::Table(Panel* const parent,
-                    int32_t x,
-                    int32_t y,
-                    uint32_t w,
-                    uint32_t h,
-                    PanelStyle style,
-                    TableRows rowtype)
-   : Panel(parent, style, x, y, w, h),
-     total_width_(0),
+BaseTable::Table(Panel* const parent,
+                 const std::string& name,
+                 int32_t x,
+                 int32_t y,
+                 uint32_t w,
+                 uint32_t h,
+                 PanelStyle style,
+                 TableRows rowtype)
+   : Panel(parent, style, name, x, y, w, h),
+
      lineheight_(text_height(g_style_manager->table_style(style).enabled())),
      headerheight_(lineheight_ + 4),
      button_style_(style == UI::PanelStyle::kFsMenu ? UI::ButtonStyle::kFsMenuMenu :
                                                       UI::ButtonStyle::kWuiSecondary),
-     scrollbar_(nullptr),
-     scrollbar_filler_button_(
-        new Button(this, "", 0, 0, Scrollbar::kSize, headerheight_, button_style_, "")),
-     scrollpos_(0),
+
+     scrollbar_filler_button_(new Button(
+        this, "scrollbar_filler", 0, 0, Scrollbar::kSize, headerheight_, button_style_, "")),
+
      selection_(no_selection_index()),
      last_multiselect_(no_selection_index()),
-     last_click_time_(-10000),
+
      last_selection_(no_selection_index()),
-     sort_column_(0),
+
      sort_descending_(rowtype == TableRows::kSingleDescending ||
                       rowtype == TableRows::kMultiDescending),
      flexible_column_idx_(std::numeric_limits<size_t>::max()),
@@ -70,8 +71,8 @@ Table<void*>::Table(Panel* const parent,
 	set_thinks(false);
 	set_can_focus(true);
 	scrollbar_filler_button_->set_visible(false);
-	scrollbar_ = new Scrollbar(this, get_w() - Scrollbar::kSize, headerheight_, Scrollbar::kSize,
-	                           get_h() - headerheight_, style);
+	scrollbar_ = new Scrollbar(this, "scrollbar", get_w() - Scrollbar::kSize, headerheight_,
+	                           Scrollbar::kSize, get_h() - headerheight_, style);
 	scrollbar_->moved.connect([this](int32_t a) { set_scrollpos(a); });
 	scrollbar_->set_steps(1);
 	scrollbar_->set_singlestepsize(lineheight_);
@@ -82,7 +83,7 @@ Table<void*>::Table(Panel* const parent,
 /**
  * Free allocated resources
  */
-Table<void*>::~Table() {
+BaseTable::~Table() {
 	for (const EntryRecord* entry : entry_records_) {
 		delete entry;
 	}
@@ -92,16 +93,16 @@ Table<void*>::~Table() {
 	multiselect_.clear();
 }
 
-size_t Table<void*>::number_of_columns() const {
+size_t BaseTable::number_of_columns() const {
 	return columns_.size();
 }
 
 /// Add a new column to this table.
-void Table<void*>::add_column(uint32_t const width,
-                              const std::string& title,
-                              const std::string& tooltip_string,
-                              Align const alignment,
-                              TableColumnType column_type) {
+void BaseTable::add_column(uint32_t const width,
+                           const std::string& title,
+                           const std::string& tooltip_string,
+                           Align const alignment,
+                           TableColumnType column_type) {
 	//  If there would be existing entries, they would not get the new column.
 	assert(size() == 0);
 
@@ -117,8 +118,8 @@ void Table<void*>::add_column(uint32_t const width,
 		Column c;
 		// All columns have a title button that is clickable for sorting.
 		// The title text can be empty.
-		c.btn = new Button(this, title, complete_width, 0, width, headerheight_, button_style_, title,
-		                   tooltip_string);
+		c.btn =
+		   new Button(this, title, complete_width, 0, width, headerheight_, button_style_, title, "");
 		const size_t col_index = columns_.size();
 		c.btn->sigclicked.connect([this, col_index]() { header_button_clicked(col_index); });
 		c.width = width;
@@ -126,6 +127,8 @@ void Table<void*>::add_column(uint32_t const width,
 		c.alignment = alignment;
 		c.compare = [this, col_index](
 		               uint32_t a, uint32_t b) { return default_compare_string(col_index, a, b); };
+		c.user_tooltip = tooltip_string;
+		c.update_tooltip(columns_.size() == get_sort_column());
 		columns_.push_back(c);
 		if (column_type == TableColumnType::kFlexible) {
 			assert(flexible_column_idx_ == std::numeric_limits<size_t>::max());
@@ -134,30 +137,55 @@ void Table<void*>::add_column(uint32_t const width,
 	}
 }
 
-void Table<void*>::set_column_title(uint8_t const col, const std::string& title) {
+void BaseTable::set_column_title(const uint8_t col, const std::string& title) {
 	assert(col < columns_.size());
 	Column& column = columns_.at(col);
 	assert(column.btn);
 	column.btn->set_title(title);
 }
 
-void Table<void*>::set_column_tooltip(uint8_t col, const std::string& text) {
+void BaseTable::set_column_tooltip(uint8_t col, const std::string& text) {
 	assert(col < columns_.size());
 	Column& column = columns_.at(col);
 	assert(column.btn);
-	column.btn->set_tooltip(text);
+	column.user_tooltip = text;
+	column.update_tooltip(col == get_sort_column());
+}
+
+void BaseTable::Column::update_tooltip(const bool sorted) const {
+	const std::string click_tooltip =
+	   sorted ? _("Click to reverse sorting") : _("Click to sort by this column");
+	if (user_tooltip.empty()) {
+		btn->set_tooltip(click_tooltip);
+	} else {
+		/** TRANSLATORS: Tooltip format for table headers: %1 is column description, %2 is
+		    "Click to sort ..." or "Click to reverse..." */
+		btn->set_tooltip(format(_("%1$s<br>(%2$s)"), user_tooltip, click_tooltip));
+	}
 }
 
 /**
  * Set a custom comparison function for sorting of the given column.
  */
-void Table<void*>::set_column_compare(uint8_t col, const Table<void*>::CompareFn& fn) {
+void BaseTable::set_column_compare(uint8_t col, const BaseTable::CompareFn& fn) {
 	assert(col < columns_.size());
 	Column& column = columns_.at(col);
 	column.compare = fn;
 }
 
-Table<void*>::EntryRecord* Table<void*>::find(const void* const entry) const
+void BaseTable::set_sort_column(uint8_t const col) {
+	const uint8_t old = get_sort_column();
+	if (col == old) {
+		return;
+	}
+	columns_.at(old).update_tooltip(false);
+
+	assert(col < columns_.size());
+	sort_column_ = col;
+	columns_.at(col).update_tooltip(true);
+}
+
+BaseTable::EntryRecord* BaseTable::find(const void* const entry) const
 
 {
 	for (EntryRecord* temp_entry : entry_records_) {
@@ -171,22 +199,23 @@ Table<void*>::EntryRecord* Table<void*>::find(const void* const entry) const
 /**
  * A header button has been clicked
  */
-void Table<void*>::header_button_clicked(Columns::size_type const n) {
+void BaseTable::header_button_clicked(Columns::size_type const n) {
 	assert(columns_.at(n).btn);
-	if (get_sort_colum() == n) {
+	if (get_sort_column() == n) {
 		set_sort_descending(!get_sort_descending());  //  change sort direction
 		sort();
 		return;
 	}
 
 	set_sort_column(n);
+	set_sort_descending(false);
 	sort();
 }
 
 /**
  * Remove all entries from the table
  */
-void Table<void*>::clear() {
+void BaseTable::clear() {
 	for (const EntryRecord* entry : entry_records_) {
 		delete entry;
 	}
@@ -196,21 +225,21 @@ void Table<void*>::clear() {
 		scrollbar_->set_steps(1);
 	}
 	scrollpos_ = 0;
-	last_click_time_ = -10000;
+	last_click_time_ = std::numeric_limits<uint32_t>::max();
 	clear_selections();
 }
 
-void Table<void*>::clear_selections() {
+void BaseTable::clear_selections() {
 	multiselect_.clear();
 	selection_ = no_selection_index();
 	last_selection_ = no_selection_index();
 }
 
-uint32_t Table<void*>::get_eff_w() const {
+uint32_t BaseTable::get_eff_w() const {
 	return scrollbar_->is_enabled() ? get_w() - scrollbar_->get_w() : get_w();
 }
 
-void Table<void*>::fit_height(uint32_t entries) {
+void BaseTable::fit_height(uint32_t entries) {
 	if (entries == 0) {
 		entries = size();
 	}
@@ -221,7 +250,7 @@ void Table<void*>::fit_height(uint32_t entries) {
 	set_desired_size(tablewidth, tableheight);
 }
 
-std::vector<Recti> Table<void*>::focus_overlay_rects() {
+std::vector<Recti> BaseTable::focus_overlay_rects() {
 	if (!has_selection()) {
 		return Panel::focus_overlay_rects();
 	}
@@ -250,7 +279,7 @@ std::vector<Recti> Table<void*>::focus_overlay_rects() {
 /**
  * Redraw the table
  */
-void Table<void*>::draw(RenderTarget& dst) {
+void BaseTable::draw(RenderTarget& dst) {
 	//  draw text lines
 	int32_t lineheight = get_lineheight();
 	uint32_t idx = scrollpos_ / lineheight;
@@ -368,7 +397,7 @@ void Table<void*>::draw(RenderTarget& dst) {
 	}
 }
 
-bool Table<void*>::handle_tooltip() {
+bool BaseTable::handle_tooltip() {
 	int32_t lineheight = get_lineheight();
 	uint32_t idx = scrollpos_ / lineheight;
 	int32_t y = 1 + idx * lineheight - scrollpos_ + headerheight_;
@@ -381,10 +410,10 @@ bool Table<void*>::handle_tooltip() {
 			const int column_w = columns_[c].width;
 			Vector2i point(column_x, y);
 			if (is_mouse_in(cursor_pos, point, column_w)) {
-				const std::string& entry_string = er.get_string(c);
+				const std::string entry_string = richtext_escape(er.get_string(c));
 				FontStyleInfo& font_style = get_column_fontstyle(er);
 				std::shared_ptr<const UI::RenderedText> rendered_text =
-				   UI::g_fh->render(as_richtext_paragraph(richtext_escape(entry_string), font_style));
+				   UI::g_fh->render(as_richtext_paragraph(entry_string, font_style));
 
 				if (rendered_text->width() > column_w) {
 					return Panel::draw_tooltip(entry_string, panel_style_);
@@ -398,15 +427,15 @@ bool Table<void*>::handle_tooltip() {
 	return true;
 }
 
-UI::FontStyleInfo& Table<void*>::get_column_fontstyle(const Table<void*>::EntryRecord& er) {
+UI::FontStyleInfo& BaseTable::get_column_fontstyle(const BaseTable::EntryRecord& er) {
 	return const_cast<FontStyleInfo&>(
 	   er.font_style() != nullptr ? *er.font_style() :
 	   er.is_disabled()           ? g_style_manager->table_style(panel_style_).disabled() :
                                    g_style_manager->table_style(panel_style_).enabled());
 }
-bool Table<void*>::is_mouse_in(const Vector2i& cursor_pos,
-                               const Vector2i& point,
-                               const int column_width) const {
+bool BaseTable::is_mouse_in(const Vector2i& cursor_pos,
+                            const Vector2i& point,
+                            const int column_width) const {
 	const int line = get_lineheight();
 
 	return cursor_pos.x >= point.x && cursor_pos.x <= point.x + column_width &&
@@ -416,7 +445,7 @@ bool Table<void*>::is_mouse_in(const Vector2i& cursor_pos,
 /**
  * handle key presses
  */
-bool Table<void*>::handle_key(bool down, SDL_Keysym code) {
+bool BaseTable::handle_key(bool down, SDL_Keysym code) {
 	if (down) {
 		if (is_multiselect_ && !empty() &&
 		    matches_shortcut(KeyboardShortcut::kCommonSelectAll, code)) {
@@ -482,14 +511,14 @@ bool Table<void*>::handle_key(bool down, SDL_Keysym code) {
 	return UI::Panel::handle_key(down, code);
 }
 
-bool Table<void*>::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
+bool BaseTable::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
 	return scrollbar_->handle_mousewheel(x, y, modstate);
 }
 
 /**
  * Handle mouse presses: select the appropriate entry
  */
-bool Table<void*>::handle_mousepress(uint8_t const btn, int32_t /*x*/, int32_t const y) {
+bool BaseTable::handle_mousepress(uint8_t const btn, int32_t /*x*/, int32_t const y) {
 	if (get_can_focus()) {
 		focus();
 	}
@@ -512,7 +541,7 @@ bool Table<void*>::handle_mousepress(uint8_t const btn, int32_t /*x*/, int32_t c
 		}
 
 		// Check if doubleclicked
-		if (((SDL_GetModState() & (KMOD_CTRL | KMOD_SHIFT)) == 0) &&
+		if (((SDL_GetModState() & (KMOD_CTRL | KMOD_SHIFT)) == 0) && time >= real_last_click_time &&
 		    time - real_last_click_time < DOUBLE_CLICK_INTERVAL && last_selection_ == selection_ &&
 		    selection_ != no_selection_index()) {
 			double_clicked(selection_);
@@ -529,7 +558,7 @@ bool Table<void*>::handle_mousepress(uint8_t const btn, int32_t /*x*/, int32_t c
  * \param offset positive value move the selection down and
  *        negative values up.
  */
-void Table<void*>::move_selection(const int32_t offset) {
+void BaseTable::move_selection(const int32_t offset) {
 	int32_t new_selection =
 	   has_selection() ? (is_multiselect_ ? last_multiselect_ : selection_) + offset : 0;
 
@@ -549,7 +578,7 @@ void Table<void*>::move_selection(const int32_t offset) {
  *
  * Args: i  the entry to select
  */
-void Table<void*>::select(const uint32_t i) {
+void BaseTable::select(const uint32_t i) {
 	if (empty() || i == no_selection_index()) {
 		return;
 	}
@@ -566,7 +595,7 @@ void Table<void*>::select(const uint32_t i) {
 /**
  * If 'force' is true, adds the given 'row' to the selection, ignoring everything else.
  */
-void Table<void*>::multiselect(uint32_t row, bool force) {
+void BaseTable::multiselect(uint32_t row, bool force) {
 	if (force) {
 		select(row);
 		return;
@@ -601,7 +630,7 @@ void Table<void*>::multiselect(uint32_t row, bool force) {
 }
 
 // Scroll to the given item. Out of range items will be corrected automatically.
-void Table<void*>::scroll_to_item(int32_t item) {
+void BaseTable::scroll_to_item(int32_t item) {
 	if (scrollbar_ != nullptr) {
 		// Correct out of range items
 		if (item < 0) {
@@ -624,7 +653,7 @@ void Table<void*>::scroll_to_item(int32_t item) {
  * Returns the row that should be selected afterwards, or no_selection_index() if
  * the multiselect is empty.
  */
-uint32_t Table<void*>::toggle_entry(uint32_t row) {
+uint32_t BaseTable::toggle_entry(uint32_t row) {
 	assert(is_multiselect_);
 	if (multiselect_.count(row) != 0u) {
 		multiselect_.erase(row);
@@ -641,7 +670,7 @@ uint32_t Table<void*>::toggle_entry(uint32_t row) {
 /**
  * Add a new entry to the table.
  */
-Table<void*>::EntryRecord& Table<void*>::add(void* const entry, const bool do_select) {
+BaseTable::EntryRecord& BaseTable::add(void* const entry, const bool do_select) {
 	EntryRecord& result = *new EntryRecord(entry);
 	entry_records_.push_back(&result);
 	result.data_.resize(columns_.size());
@@ -657,18 +686,18 @@ Table<void*>::EntryRecord& Table<void*>::add(void* const entry, const bool do_se
 /**
  * Scroll to the given position, in pixels.
  */
-void Table<void*>::set_scrollpos(int32_t const i) {
+void BaseTable::set_scrollpos(int32_t const i) {
 	scrollpos_ = i;
 }
 
-void Table<void*>::scroll_to_top() {
+void BaseTable::scroll_to_top() {
 	scrollbar_->set_scrollpos(0);
 }
 
 /**
  * Remove the table entry at the given (zero-based) index.
  */
-void Table<void*>::remove(const uint32_t i) {
+void BaseTable::remove(const uint32_t i) {
 	assert(i < entry_records_.size());
 	multiselect_.clear();
 
@@ -692,7 +721,7 @@ void Table<void*>::remove(const uint32_t i) {
 /**
  * Remove the given table entry if it exists.
  */
-void Table<void*>::remove_entry(const void* const entry) {
+void BaseTable::remove_entry(const void* const entry) {
 	for (uint32_t i = 0; i < entry_records_.size(); ++i) {
 		if (entry_records_[i]->entry() == entry) {
 			remove(i);
@@ -701,14 +730,14 @@ void Table<void*>::remove_entry(const void* const entry) {
 	}
 }
 
-bool Table<void*>::sort_helper(uint32_t a, uint32_t b) {
+bool BaseTable::sort_helper(uint32_t a, uint32_t b) {
 	if (sort_descending_) {
 		return columns_[sort_column_].compare(b, a);
 	}
 	return columns_[sort_column_].compare(a, b);
 }
 
-void Table<void*>::layout() {
+void BaseTable::layout() {
 	if (columns_.empty() || get_h() == 0 || get_w() == 0) {
 		return;
 	}
@@ -725,13 +754,13 @@ void Table<void*>::layout() {
 	}
 }
 
-void Table<void*>::reposition_scrollbar() {
+void BaseTable::reposition_scrollbar() {
 	scrollbar_->set_pos(Vector2i(get_w() - Scrollbar::kSize, headerheight_));
 	scrollbar_->set_size(scrollbar_->get_w(), get_h() - headerheight_);
 	scrollbar_->set_pagesize(get_h() - 2 * get_lineheight() - headerheight_);
 	scrollbar_->set_steps(entry_records_.size() * get_lineheight() - (get_h() - headerheight_ - 2));
 }
-size_t Table<void*>::find_resizable_column_idx() {
+size_t BaseTable::find_resizable_column_idx() {
 
 	if (flexible_column_idx_ < columns_.size()) {
 		return flexible_column_idx_;
@@ -748,7 +777,7 @@ size_t Table<void*>::find_resizable_column_idx() {
 	return widest_column_idx;
 }
 
-int Table<void*>::total_columns_width() {
+int BaseTable::total_columns_width() {
 	int all_columns_width = scrollbar_->is_enabled() ? scrollbar_->get_w() : 0;
 	for (const auto& column : columns_) {
 		all_columns_width += column.width;
@@ -756,7 +785,7 @@ int Table<void*>::total_columns_width() {
 	return all_columns_width;
 }
 
-void Table<void*>::adjust_column_sizes(int all_columns_width, size_t resizeable_column_idx) {
+void BaseTable::adjust_column_sizes(int all_columns_width, size_t resizeable_column_idx) {
 	Column& resizable_col = columns_.at(resizeable_column_idx);
 	resizable_col.width = std::max(0, resizable_col.width + get_w() - all_columns_width);
 	resizable_col.btn->set_size(resizable_col.width, resizable_col.btn->get_h());
@@ -775,7 +804,7 @@ void Table<void*>::adjust_column_sizes(int all_columns_width, size_t resizeable_
 	}
 }
 
-void Table<void*>::update_scrollbar_filler() {
+void BaseTable::update_scrollbar_filler() {
 	if (scrollbar_->is_enabled()) {
 		const UI::Button* last_column_btn = columns_.back().btn;
 		scrollbar_filler_button_->set_pos(
@@ -795,7 +824,7 @@ void Table<void*>::update_scrollbar_filler() {
  * For example, you might want to sort directories for themselves at the
  * top of the list, and files at the bottom.
  */
-void Table<void*>::sort(const uint32_t lower_bound, uint32_t upper_bound) {
+void BaseTable::sort(const uint32_t lower_bound, uint32_t upper_bound) {
 	assert(columns_.at(sort_column_).btn);
 	assert(sort_column_ < columns_.size());
 
@@ -837,42 +866,41 @@ void Table<void*>::sort(const uint32_t lower_bound, uint32_t upper_bound) {
 	}
 }
 
-bool Table<void*>::default_compare_string(uint32_t column, uint32_t a, uint32_t b) const {
+bool BaseTable::default_compare_string(uint32_t column, uint32_t a, uint32_t b) const {
 	const EntryRecord& ea = get_record(a);
 	const EntryRecord& eb = get_record(b);
 	return ea.get_string(column) < eb.get_string(column);
 }
-bool Table<void*>::handle_mousemove(
+bool BaseTable::handle_mousemove(
    uint8_t /*state*/, int32_t /*x*/, int32_t /*y*/, int32_t /*xdiff*/, int32_t /*ydiff*/) {
 	// needed to activate tooltip rendering without providing tooltiptext to parent (panel) class
 	return true;
 }
 
-Table<void*>::EntryRecord::EntryRecord(void* const e)
-   : entry_(e), font_style_(nullptr), disabled_(false) {
+BaseTable::EntryRecord::EntryRecord(void* const e) : entry_(e), font_style_(nullptr) {
 }
 
-void Table<void*>::EntryRecord::set_picture(uint8_t const col,
-                                            const Image* pic,
-                                            const std::string& str) {
+void BaseTable::EntryRecord::set_picture(uint8_t const col,
+                                         const Image* pic,
+                                         const std::string& str) {
 	assert(col < data_.size());
 
 	data_.at(col).d_picture = pic;
 	data_.at(col).d_string = str;
 }
-void Table<void*>::EntryRecord::set_string(uint8_t const col, const std::string& str) {
+void BaseTable::EntryRecord::set_string(uint8_t const col, const std::string& str) {
 	assert(col < data_.size());
 
 	data_.at(col).d_picture = nullptr;
 	data_.at(col).d_string = str;
 }
 
-const Image* Table<void*>::EntryRecord::get_picture(uint8_t const col) const {
+const Image* BaseTable::EntryRecord::get_picture(uint8_t const col) const {
 	assert(col < data_.size());
 
 	return data_.at(col).d_picture;
 }
-const std::string& Table<void*>::EntryRecord::get_string(uint8_t const col) const {
+const std::string& BaseTable::EntryRecord::get_string(uint8_t const col) const {
 	assert(col < data_.size());
 
 	return data_.at(col).d_string;
