@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,6 +29,7 @@
 #include "graphic/style_manager.h"
 #include "graphic/text_layout.h"
 #include "io/filesystem/layered_filesystem.h"
+#include "ui_basic/toolbar_setup.h"
 
 namespace UI {
 
@@ -76,24 +77,17 @@ Window::Window(Panel* const parent,
                uint32_t const w,
                uint32_t const h,
                const std::string& title)
-   : NamedPanel(parent,
-                s == WindowStyle::kWui ? PanelStyle::kWui : PanelStyle::kFsMenu,
-                name,
-                x,
-                y,
-                w + kVerticalBorderThickness * 2,
-                kTopBorderThickness + h + kBottomBorderThickness),
+   : Panel(parent,
+           s == WindowStyle::kWui ? PanelStyle::kWui : PanelStyle::kFsMenu,
+           name,
+           x,
+           y,
+           w + kVerticalBorderThickness * 2,
+           kTopBorderThickness + h + kBottomBorderThickness),
      window_style_(s),
-     is_minimal_(false),
+
      oldh_(kTopBorderThickness + h + kBottomBorderThickness),
-     dragging_(false),
-     drag_start_win_x_(0),
-     drag_start_win_y_(0),
-     drag_start_mouse_x_(0),
-     drag_start_mouse_y_(0),
-     pinned_(false),
-     center_panel_(nullptr),
-     fastclick_panel_(nullptr),
+
      button_close_(new Button(
         this,
         "b_close",
@@ -149,6 +143,7 @@ Window::Window(Panel* const parent,
 	           kBottomBorderThickness);
 	set_top_on_click(true);
 	set_layout_toplevel(true);
+	set_snap_target(true);
 	layout();
 	focus();
 
@@ -236,23 +231,102 @@ void Window::layout() {
 }
 
 /**
- * Move the window out of the way so that the field bewlow it is visible
+ * Position the window near the clicked position, but keeping the clicked field visible
  */
 void Window::move_out_of_the_way() {
-	center_to_parent();
+	constexpr int32_t kClearance = 100;
 
-	const Vector2i mouse = get_mouse_position();
-	if (0 <= mouse.x && mouse.x < get_w() && 0 <= mouse.y && mouse.y < get_h()) {
-		set_pos(Vector2i(get_x(), get_y()) + Vector2i(0, (mouse.y < get_h() / 2 ? 1 : -1) * get_h()));
-		move_inside_parent();
+	const Panel* parent = get_parent();
+	assert(parent != nullptr);
+
+	// Actually this could be asserted to be true, because only field action-, building- and
+	// ship windows call this function currently.
+	const bool parent_is_main = parent->get_parent() == nullptr;
+
+	const bool toolbar_at_bottom = main_toolbar_at_bottom();
+	const int32_t toolbar_bottom_h =
+	   (parent_is_main && toolbar_at_bottom) ? main_toolbar_button_size() : 0;
+	const int32_t toolbar_top_h =
+	   (parent_is_main && !toolbar_at_bottom) ? main_toolbar_button_size() : 0;
+
+	// We have to do this because InfoPanel::think() pushes child windows off the toolbar, which
+	// messes up the fastclick position.
+	// We only care about the toolbar at the bottom because we prefer moving the window below
+	// the mouse pointer, so it never covers the toolbar at the top, but it may cover it at the
+	// bottom.
+	const int ph = parent->get_inner_h() - toolbar_bottom_h;
+	const int pw = parent->get_inner_w();
+
+	// Pop up messages have higher priority, so they can steal mouse clicks. We try to avoid that.
+	const int max_popup_h = UI::kMaxPopupMessages * (toolbar_bottom_h + toolbar_top_h);
+
+	const Vector2i mouse = parent->get_mouse_position();
+
+	int32_t nx = mouse.x;
+	int32_t ny = mouse.y;
+
+	// TODO(tothxa): This will have to be calculated when the field action window is converted to
+	//               use styles instead of constants
+	constexpr int kFlagActionHeight = 112;
+
+	if (get_w() < get_h() && get_h() > kFlagActionHeight) {
+		const bool fits_right = mouse.x + kClearance + get_w() < pw;
+		const bool fits_left = mouse.x - kClearance - get_w() >= 0;
+
+		bool to_right = fits_right;
+
+		if (!fits_right && !fits_left) {
+			to_right = mouse.x < pw / 2;
+		}
+
+		if (fits_right && fits_left) {
+			bool need_check_popups = false;
+			if (parent_is_main) {
+				if (toolbar_at_bottom) {
+					need_check_popups =
+					   mouse.y + get_h() / 2 - kBottomBorderThickness > ph - max_popup_h;
+				} else {
+					need_check_popups =
+					   mouse.y - get_h() / 2 + kTopBorderThickness < toolbar_top_h + max_popup_h;
+				}
+			}
+
+			if (need_check_popups) {
+				// This is much simplified, because there may be no perfect solution for some tall
+				// windows anyway
+				to_right = mouse.x >= pw / 2;
+			}
+		}
+
+		if (to_right) {
+			nx += kClearance;
+		} else {
+			nx -= get_w() + kClearance;
+		}
+		ny -= get_h() / 2;
+	} else {
+		const int max_popup_h_bottom =
+		   toolbar_bottom_h > 0 ? (max_popup_h - kBottomBorderThickness) : 0;
+		if (mouse.y + kClearance + get_h() + max_popup_h_bottom < ph || mouse.y < ph / 2) {
+			ny += kClearance;
+		} else {
+			ny -= get_h() + kClearance;
+		}
+		nx -= get_w() / 2;
 	}
+
+	// Don't use overridden functions in UniqueWindow, position is not final yet
+	Panel::set_pos(Vector2i(nx, ny));
+	Window::move_inside_parent();
+	// move_inside_parent() always calls overridden set_pos(), so position is always finalised
+	// for UniqueWindow
 }
 
 /**
  * Moves the mouse to the child panel that is activated as fast click panel
  */
 void Window::warp_mouse_to_fastclick_panel() {
-	if (fastclick_panel_ != nullptr) {
+	if (fastclick_panel_ != nullptr && Panel::allow_fastclick()) {
 		Vector2i pt(fastclick_panel_->get_w() / 2, fastclick_panel_->get_h() / 2);
 		UI::Panel* p = fastclick_panel_;
 
@@ -271,16 +345,22 @@ void Window::warp_mouse_to_fastclick_panel() {
  */
 void Window::move_inside_parent() {
 	if (Panel* const parent = get_parent()) {
+		const bool parent_is_main = parent->get_parent() == nullptr;
+		const bool toolbar_at_bottom = main_toolbar_at_bottom();
+		const int32_t toolbar_bottom_h =
+		   (parent_is_main && toolbar_at_bottom) ? main_toolbar_button_size() : 0;
+		const int32_t toolbar_top_h =
+		   (parent_is_main && !toolbar_at_bottom) ? main_toolbar_button_size() : 0;
+		const int32_t ph = parent->get_inner_h() - toolbar_top_h - toolbar_bottom_h;
+
 		int32_t px = get_x();
 		int32_t py = get_y();
-		if ((parent->get_inner_w() < get_w()) && (px + get_w() <= parent->get_inner_w() || px >= 0)) {
-			px = (parent->get_inner_w() - get_w()) / 2;
-		}
-		if ((parent->get_inner_h() < get_h()) && (py + get_h() < parent->get_inner_h() || py > 0)) {
-			py = 0;
-		}
 
-		if (parent->get_inner_w() >= get_w()) {
+		if (parent->get_inner_w() < get_w()) {
+			if ((px + get_w() < parent->get_inner_w() || px > 0)) {
+				px = (parent->get_inner_w() - get_w()) / 2;
+			}
+		} else {
 			if (px < 0) {
 				px = 0;
 				if (parent->get_dock_windows_to_edges()) {
@@ -293,16 +373,26 @@ void Window::move_inside_parent() {
 				}
 			}
 		}
-		if (parent->get_inner_h() >= get_h()) {
-			if (py < 0) {
+
+		if (ph < get_h()) {
+			if (parent_is_main) {
+				// The toolbar would push it off anyway
+				py = toolbar_at_bottom ? ph - get_h() : toolbar_top_h;
+			} else if (py + get_h() < ph || py > 0) {
 				py = 0;
-			} else if (py + get_h() > parent->get_inner_h()) {
-				py = parent->get_inner_h() - get_h();
-				if (!is_minimal_ && parent->get_dock_windows_to_edges()) {
+			}
+		} else {
+			if (py < toolbar_top_h) {
+				py = toolbar_top_h;
+			} else if (const int32_t p_bottom = parent->get_inner_h() - toolbar_bottom_h;
+			           py + get_h() > p_bottom) {
+				py = p_bottom - get_h();
+				if (!is_minimal_ && toolbar_bottom_h == 0 && parent->get_dock_windows_to_edges()) {
 					py += kBottomBorderThickness;
 				}
 			}
 		}
+
 		set_pos(Vector2i(px, py));
 	}
 }
@@ -681,7 +771,6 @@ bool Window::handle_mousemove(
 			}
 
 			{  //  Snap to other Panels.
-				const bool SOWO = parent->get_snap_windows_only_when_overlapping();
 				const int32_t right = left + w;
 				const int32_t bot = top + h;
 
@@ -694,14 +783,14 @@ bool Window::handle_mousemove(
 						int32_t const other_bot = other_top + snap_target->get_h();
 
 						if (other_top <= bot && other_bot >= top) {
-							if (!SOWO || left <= other_right) {
+							{
 								const int32_t distance = abs(left - other_right);
 								if (distance < nearest_snap_distance_x) {
 									nearest_snap_distance_x = distance;
 									new_left = other_right;
 								}
 							}
-							if (!SOWO || right >= other_left) {
+							{
 								const int32_t distance = abs(right - other_left);
 								if (distance < nearest_snap_distance_x) {
 									nearest_snap_distance_x = distance;
@@ -710,14 +799,14 @@ bool Window::handle_mousemove(
 							}
 						}
 						if (other_left <= right && other_right >= left) {
-							if (!SOWO || top <= other_bot) {
+							{
 								const int32_t distance = abs(top - other_bot);
 								if (distance < nearest_snap_distance_y) {
 									nearest_snap_distance_y = distance;
 									new_top = other_bot;
 								}
 							}
-							if (!SOWO || bot >= other_top) {
+							{
 								const int32_t distance = abs(bot - other_top);
 								if (distance < nearest_snap_distance_y) {
 									nearest_snap_distance_y = distance;
@@ -744,6 +833,7 @@ bool Window::handle_mousemove(
 			}
 		}
 		set_pos(Vector2i(new_left, new_top));
+		moved_by_user_ = true;
 	}
 	return true;
 }
