@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2022 by the Widelands Development Team
+ * Copyright (C) 2020-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,21 +24,21 @@
 #include <ostream>
 #include <sstream>
 
-#include <SDL.h>
-
 #include "base/i18n.h"
 #include "base/log.h"
+#include "base/time_string.h"
 #include "base/warning.h"
 #include "graphic/graphic.h"
 #include "graphic/image_cache.h"
 #include "graphic/style_manager.h"
+#include "graphic/text_layout.h"
 #include "io/profile.h"
 #include "logic/filesystem_constants.h"
 #include "logic/game.h"
 #include "scripting/lua_table.h"
 #include "ui_basic/color_chooser.h"
 #include "ui_basic/messagebox.h"
-#include "ui_basic/multilineeditbox.h"
+#include "ui_basic/textinput.h"
 #include "ui_fsmenu/addons/contact.h"
 #include "ui_fsmenu/addons/login_box.h"
 #include "ui_fsmenu/addons/packager.h"
@@ -48,7 +48,6 @@
 #include "wlapplication.h"
 #include "wlapplication_options.h"
 
-namespace FsMenu {
 namespace AddOnsUI {
 
 constexpr const char* const kDocumentationURL = "https://www.widelands.org/documentation/add-ons/";
@@ -58,13 +57,6 @@ constexpr const char* const kForumURL = "https://www.widelands.org/forum/forum/1
 // so we can and need to allow somewhat larger dimensions.
 constexpr int32_t kHugeSize = std::numeric_limits<int32_t>::max() / 2;
 
-std::string underline_tag(const std::string& text) {
-	std::string str = "<font underline=true>";
-	str += text;
-	str += "</font>";
-	return str;
-}
-
 std::string filesize_string(const uint32_t bytes) {
 	if (bytes > 1000 * 1000 * 1000) {
 		return format_l(_("%.2f GB"), (bytes / (1000.f * 1000.f * 1000.f)));
@@ -118,7 +110,7 @@ const std::map<unsigned, std::function<AddOnQuality()>> AddOnQuality::kQualities
 
 struct OperationCancelledByUserException : std::exception {};
 
-AddOnsCtrl::AddOnsCtrl(MainMenu& fsmm, UI::UniqueWindow::Registry& reg)
+AddOnsCtrl::AddOnsCtrl(FsMenu::MainMenu& fsmm, UI::UniqueWindow::Registry& reg)
    : UI::UniqueWindow(&fsmm,
                       UI::WindowStyle::kFsMenu,
                       "addons",
@@ -127,48 +119,93 @@ AddOnsCtrl::AddOnsCtrl(MainMenu& fsmm, UI::UniqueWindow::Registry& reg)
                       fsmm.calc_desired_window_height(UI::Window::WindowLayoutID::kFsMenuDefault),
                       _("Add-On Manager")),
      fsmm_(fsmm),
-     main_box_(this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     buttons_box_(&main_box_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
-     warn_requirements_(
-        &main_box_, 0, 0, get_w(), get_h() / 12, UI::PanelStyle::kFsMenu, "", UI::Align::kCenter),
-     tabs_placeholder_(&main_box_, UI::PanelStyle::kFsMenu, 0, 0, 0, 0),
-     tabs_(this, UI::TabPanelStyle::kFsMenu),
-     installed_addons_outer_wrapper_(&tabs_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
-     installed_addons_inner_wrapper_(
-        &installed_addons_outer_wrapper_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     installed_addons_buttons_box_(
-        &installed_addons_outer_wrapper_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
+     main_box_(this, UI::PanelStyle::kFsMenu, "main_box", 0, 0, UI::Box::Vertical),
+     buttons_box_(&main_box_, UI::PanelStyle::kFsMenu, "buttons_box", 0, 0, UI::Box::Horizontal),
+     warn_requirements_(&main_box_,
+                        "warn_requirements",
+                        0,
+                        0,
+                        get_w(),
+                        get_h() / 12,
+                        UI::PanelStyle::kFsMenu,
+                        "",
+                        UI::Align::kCenter),
+     tabs_placeholder_(&main_box_, UI::PanelStyle::kFsMenu, "tabs_placeholder", 0, 0, 0, 0),
+     tabs_(this, UI::TabPanelStyle::kFsMenu, "tabs"),
+     installed_addons_outer_wrapper_(
+        &tabs_, UI::PanelStyle::kFsMenu, "installed_outer_wrapper_box", 0, 0, UI::Box::Horizontal),
+     installed_addons_inner_wrapper_(&installed_addons_outer_wrapper_,
+                                     UI::PanelStyle::kFsMenu,
+                                     "installed_inner_wrapper_box",
+                                     0,
+                                     0,
+                                     UI::Box::Vertical),
+     installed_addons_buttons_box_(&installed_addons_outer_wrapper_,
+                                   UI::PanelStyle::kFsMenu,
+                                   "installed_buttons_box",
+                                   0,
+                                   0,
+                                   UI::Box::Vertical),
      installed_addons_box_(&installed_addons_inner_wrapper_,
                            UI::PanelStyle::kFsMenu,
+                           "installed_box",
                            0,
                            0,
                            UI::Box::Vertical,
                            kHugeSize,
                            kHugeSize),
-     browse_addons_outer_wrapper_(&tabs_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     browse_addons_inner_wrapper_(
-        &browse_addons_outer_wrapper_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     browse_addons_buttons_box_(
-        &browse_addons_outer_wrapper_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
-     browse_addons_buttons_box_lvbox_(
-        &browse_addons_buttons_box_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     browse_addons_buttons_box_rvbox_(
-        &browse_addons_buttons_box_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     browse_addons_buttons_box_category_box_(
-        &browse_addons_buttons_box_lvbox_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
-     browse_addons_buttons_box_right_hbox_(
-        &browse_addons_buttons_box_rvbox_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
+     browse_addons_outer_wrapper_(
+        &tabs_, UI::PanelStyle::kFsMenu, "browse_outer_wrapper_box", 0, 0, UI::Box::Vertical),
+     browse_addons_inner_wrapper_(&browse_addons_outer_wrapper_,
+                                  UI::PanelStyle::kFsMenu,
+                                  "browse_inner_wrapper_box",
+                                  0,
+                                  0,
+                                  UI::Box::Vertical),
+     browse_addons_buttons_box_(&browse_addons_outer_wrapper_,
+                                UI::PanelStyle::kFsMenu,
+                                "browse_buttons_box",
+                                0,
+                                0,
+                                UI::Box::Horizontal),
+     browse_addons_buttons_box_lvbox_(&browse_addons_buttons_box_,
+                                      UI::PanelStyle::kFsMenu,
+                                      "browse_buttons_left_vbox",
+                                      0,
+                                      0,
+                                      UI::Box::Vertical),
+     browse_addons_buttons_box_rvbox_(&browse_addons_buttons_box_,
+                                      UI::PanelStyle::kFsMenu,
+                                      "browse_buttons_right_vbox",
+                                      0,
+                                      0,
+                                      UI::Box::Vertical),
+     browse_addons_buttons_box_category_box_(&browse_addons_buttons_box_lvbox_,
+                                             UI::PanelStyle::kFsMenu,
+                                             "browse_buttons_category_box",
+                                             0,
+                                             0,
+                                             UI::Box::Horizontal),
+     browse_addons_buttons_box_right_hbox_(&browse_addons_buttons_box_rvbox_,
+                                           UI::PanelStyle::kFsMenu,
+                                           "browse_buttons_hbox",
+                                           0,
+                                           0,
+                                           UI::Box::Horizontal),
      browse_addons_box_(&browse_addons_inner_wrapper_,
                         UI::PanelStyle::kFsMenu,
+                        "browse_box",
                         0,
                         0,
                         UI::Box::Vertical,
                         kHugeSize,
                         kHugeSize),
-     dev_box_(&tabs_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     filter_name_(&browse_addons_buttons_box_rvbox_, 0, 0, 100, UI::PanelStyle::kFsMenu),
+     dev_box_(&tabs_, UI::PanelStyle::kFsMenu, "development_box", 0, 0, UI::Box::Vertical),
+     filter_name_(
+        &browse_addons_buttons_box_rvbox_, "filter_name", 0, 0, 100, UI::PanelStyle::kFsMenu),
      filter_verified_(&browse_addons_buttons_box_right_hbox_,
                       UI::PanelStyle::kFsMenu,
+                      "filter_verified",
                       Vector2i(0, 0),
                       _("Verified only"),
                       _("Show only verified add-ons")),
@@ -218,6 +255,7 @@ AddOnsCtrl::AddOnsCtrl(MainMenu& fsmm, UI::UniqueWindow::Registry& reg)
                         UI::ButtonStyle::kFsMenuSecondary),
      upload_addon_accept_(&dev_box_,
                           UI::PanelStyle::kFsMenu,
+                          "upload_addon_accept",
                           Vector2i(0, 0),
                           _("Understood and confirmed"),
                           _("By ticking this checkbox, you confirm that you have read and agree to "
@@ -322,18 +360,30 @@ AddOnsCtrl::AddOnsCtrl(MainMenu& fsmm, UI::UniqueWindow::Registry& reg)
               /** TRANSLATORS: This button allows the user to send a message to the Widelands
                  Development Team */
               _("Contact us…")),
-     server_name_(this, UI::PanelStyle::kFsMenu, UI::FontStyle::kWarning, "", UI::Align::kRight) {
+     server_name_(this,
+                  UI::PanelStyle::kFsMenu,
+                  "server_name",
+                  UI::FontStyle::kWarning,
+                  0,
+                  0,
+                  0,
+                  0,
+                  "",
+                  UI::Align::kRight) {
+
+	installed_addons_box_.set_flag(UI::Panel::pf_unlimited_size, true);
+	browse_addons_box_.set_flag(UI::Panel::pf_unlimited_size, true);
 
 	dev_box_.set_force_scrolling(true);
-	dev_box_.add(
-	   new UI::Textarea(&dev_box_, UI::PanelStyle::kFsMenu, UI::FontStyle::kFsMenuInfoPanelHeading,
-	                    _("Tools for Add-Ons Developers"), UI::Align::kCenter),
-	   UI::Box::Resizing::kFullSize);
+	dev_box_.add(new UI::Textarea(&dev_box_, UI::PanelStyle::kFsMenu, "label_development",
+	                              UI::FontStyle::kFsMenuInfoPanelHeading,
+	                              _("Tools for Add-Ons Developers"), UI::Align::kCenter),
+	             UI::Box::Resizing::kFullSize);
 	dev_box_.add_space(kRowButtonSize);
 	{
 		UI::MultilineTextarea* m = new UI::MultilineTextarea(
-		   &dev_box_, 0, 0, 100, 100, UI::PanelStyle::kFsMenu, "", UI::Align::kLeft,
-		   UI::MultilineTextarea::ScrollMode::kNoScrolling);
+		   &dev_box_, "message_development", 0, 0, 100, 100, UI::PanelStyle::kFsMenu, "",
+		   UI::Align::kLeft, UI::MultilineTextarea::ScrollMode::kNoScrolling);
 		m->set_style(UI::FontStyle::kFsMenuInfoPanelParagraph);
 		m->set_text(_("The interactive add-ons packager allows you to create, edit, and delete "
 		              "add-ons. You can bundle maps designed with the Widelands Map Editor as an "
@@ -346,40 +396,22 @@ AddOnsCtrl::AddOnsCtrl(MainMenu& fsmm, UI::UniqueWindow::Registry& reg)
 
 	dev_box_.add_space(kRowButtonSize);
 	dev_box_.add(new UI::MultilineTextarea(
-	                &dev_box_, 0, 0, 100, 100, UI::PanelStyle::kFsMenu,
+	                &dev_box_, "message_links", 0, 0, 100, 100, UI::PanelStyle::kFsMenu,
 	                format("<rt><p>%1$s</p></rt>",
 	                       g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
 	                          .as_font_tag(format(
 	                             _("For more information regarding how to develop and package your "
 	                               "own add-ons, please visit %s."),
-	                             underline_tag(kDocumentationURL)))),
+	                             g_style_manager->font_style(UI::FontStyle::kFsTooltip)
+	                                .as_font_tag(as_url_hyperlink(kDocumentationURL))))),
 	                UI::Align::kLeft, UI::MultilineTextarea::ScrollMode::kNoScrolling),
 	             UI::Box::Resizing::kFullSize);
-	auto add_button = [this](const std::string& url) {
-		UI::Button* b =
-		   new UI::Button(&dev_box_, "url", 0, 0, 0, 0, UI::ButtonStyle::kFsMenuSecondary,
-#if SDL_VERSION_ATLEAST(2, 0, 14)
-		                  _("Open Link")
-#else
-		                  _("Copy Link")
-#endif
-		   );
-		b->sigclicked.connect([url]() {
-#if SDL_VERSION_ATLEAST(2, 0, 14)
-			SDL_OpenURL(url.c_str());
-#else
-			SDL_SetClipboardText(url.c_str());
-#endif
-		});
-		dev_box_.add(b);
-	};
 	dev_box_.add_space(kRowButtonSpacing);
-	add_button(kDocumentationURL);
 
 	dev_box_.add_space(kRowButtonSize);
 	dev_box_.add(
 	   new UI::MultilineTextarea(
-	      &dev_box_, 0, 0, 100, 100, UI::PanelStyle::kFsMenu,
+	      &dev_box_, "message_detailed_info", 0, 0, 100, 100, UI::PanelStyle::kFsMenu,
 	      format(
 	         "<rt><p>%s</p><p>%s</p><p>%s</p><p>%s</p></rt>",
 	         g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
@@ -429,7 +461,7 @@ AddOnsCtrl::AddOnsCtrl(MainMenu& fsmm, UI::UniqueWindow::Registry& reg)
 	dev_box_.add_space(kRowButtonSize);
 	dev_box_.add(
 	   new UI::MultilineTextarea(
-	      &dev_box_, 0, 0, 100, 100, UI::PanelStyle::kFsMenu,
+	      &dev_box_, "message_contact_public", 0, 0, 100, 100, UI::PanelStyle::kFsMenu,
 	      format("<rt><p>%1$s</p></rt>",
 	             g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
 	                .as_font_tag(
@@ -437,16 +469,16 @@ AddOnsCtrl::AddOnsCtrl(MainMenu& fsmm, UI::UniqueWindow::Registry& reg)
 	                            "as deletion of an add-on or collaborating with another add-on "
 	                            "designer? Please visit our forums at %s, explain your needs, and "
 	                            "the Widelands Development Team will be happy to help."),
-	                          underline_tag(kForumURL)))),
+	                          g_style_manager->font_style(UI::FontStyle::kFsTooltip)
+	                             .as_font_tag(as_url_hyperlink(kForumURL))))),
 	      UI::Align::kLeft, UI::MultilineTextarea::ScrollMode::kNoScrolling),
 	   UI::Box::Resizing::kFullSize);
 	dev_box_.add_space(kRowButtonSpacing);
-	add_button(kForumURL);
 
 	dev_box_.add_space(kRowButtonSize);
 	dev_box_.add(
 	   new UI::MultilineTextarea(
-	      &dev_box_, 0, 0, 100, 100, UI::PanelStyle::kFsMenu,
+	      &dev_box_, "message_contact_private", 0, 0, 100, 100, UI::PanelStyle::kFsMenu,
 	      format("<rt><p>%1$s</p></rt>",
 	             g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
 	                .as_font_tag(
@@ -521,7 +553,8 @@ AddOnsCtrl::AddOnsCtrl(MainMenu& fsmm, UI::UniqueWindow::Registry& reg)
 			}
 			UI::Checkbox* c =
 			   new UI::Checkbox(&browse_addons_buttons_box_category_box_, UI::PanelStyle::kFsMenu,
-			                    Vector2i(0, 0), g_image_cache->get(pair.second.icon),
+			                    format("category_%s", pair.second.internal_name), Vector2i(0, 0),
+			                    g_image_cache->get(pair.second.icon),
 			                    format(_("Toggle category ‘%s’"), pair.second.descname()));
 			filter_category_[pair.first] = c;
 			c->set_state(true);
@@ -619,7 +652,7 @@ AddOnsCtrl::AddOnsCtrl(MainMenu& fsmm, UI::UniqueWindow::Registry& reg)
 			text += '\n';
 			for (const auto& pair : upgrades) {
 				if (pair.second) {
-					text += format(_("\n· %1$s (%2$s) by %3$s"), pair.first->descname(),
+					text += format(_("\n• %1$s (%2$s) by %3$s"), pair.first->descname(),
 					               (pair.first->verified ? _("verified") : _("NOT VERIFIED")),
 					               pair.first->author());
 				}
@@ -747,7 +780,7 @@ AddOnsCtrl::~AddOnsCtrl() {
 void AddOnsCtrl::login_button_clicked() {
 	if (username_.empty()) {
 		UI::UniqueWindow::Registry r;
-		AddOnsLoginBox b(*this);
+		AddOnsLoginBox b(get_topmost_forefather(), UI::WindowStyle::kFsMenu);
 		if (b.run<UI::Panel::Returncodes>() != UI::Panel::Returncodes::kOk) {
 			return;
 		}
@@ -920,7 +953,7 @@ void AddOnsCtrl::erase_remote(std::shared_ptr<AddOns::AddOnInfo> a) {
 
 void AddOnsCtrl::refresh_remotes(const bool showall) {
 	UI::ProgressWindow progress(this, "", "");
-	const std::string step_message = _("Fetching add-ons (%.1f%%)");
+	const std::string step_message = _("Fetching add-ons… (%.1f%%)");
 
 	try {
 		progress.step(_("Connecting to the server…"));
@@ -951,7 +984,7 @@ void AddOnsCtrl::refresh_remotes(const bool showall) {
 		std::string bug = _("a networking bug");
 		std::string err = format(_("Unable to fetch the list of available add-ons from "
 		                           "the server!<br>Error Message: %s"),
-		                         e.what());
+		                         richtext_escape(e.what()));
 		std::shared_ptr<AddOns::AddOnInfo> i = std::make_shared<AddOns::AddOnInfo>();
 		i->unlocalized_descname = title;
 		i->unlocalized_description = err;
@@ -991,19 +1024,15 @@ bool AddOnsCtrl::matches_filter(std::shared_ptr<AddOns::AddOnInfo> info) {
 		return false;
 	}
 
-	if (filter_name_.text().empty()) {
+	if (filter_name_.get_text().empty()) {
 		// no text filter given, so we accept it
 		return true;
 	}
-	for (const std::string& text : {info->descname(), info->author(), info->upload_username,
-	                                info->internal_name, info->description()}) {
-		if (text.find(filter_name_.text()) != std::string::npos) {
-			// text filter found
-			return true;
-		}
-	}
-	// doesn't match the text filter
-	return false;
+	auto array = {info->descname(), info->author(), info->upload_username, info->internal_name,
+	              info->description()};
+	return std::any_of(array.begin(), array.end(), [this](const std::string& text) {
+		return text.find(filter_name_.get_text()) != std::string::npos;
+	});
 }
 
 void AddOnsCtrl::rebuild(const bool need_to_update_dependency_errors) {
@@ -1137,7 +1166,7 @@ void AddOnsCtrl::rebuild(const bool need_to_update_dependency_errors) {
 		                          has_upgrades.size());
 		for (const std::string& name : has_upgrades) {
 			text += "<br>";
-			text += format(_("· %s"), name);
+			text += format(_("• %s"), name);
 		}
 		upgrade_all_.set_tooltip(text);
 	}
@@ -1173,18 +1202,19 @@ void AddOnsCtrl::update_dependency_errors() {
 			}
 			if (search_result == AddOns::g_addons.end()) {
 				warn_requirements.push_back(
-				   format(_("· ‘%1$s’ requires ‘%2$s’ which could not be found"),
+				   format(_("• ‘%1$s’ requires ‘%2$s’ which could not be found"),
 				          addon->first->descname(), requirement));
 			} else {
-				if (!search_result->second) {
-					warn_requirements.push_back(format(_("· ‘%1$s’ requires ‘%2$s’ which is disabled"),
+				if (!search_result->second &&
+				    AddOns::require_enabled(addon->first->category, search_result->first->category)) {
+					warn_requirements.push_back(format(_("• ‘%1$s’ requires ‘%2$s’ which is disabled"),
 					                                   addon->first->descname(),
 					                                   search_result->first->descname()));
 				}
 				if (too_late &&
 				    AddOns::order_matters(addon->first->category, search_result->first->category)) {
 					warn_requirements.push_back(
-					   format(_("· ‘%1$s’ requires ‘%2$s’ which is listed below the requiring add-on"),
+					   format(_("• ‘%1$s’ requires ‘%2$s’ which is listed below the requiring add-on"),
 					          addon->first->descname(), search_result->first->descname()));
 				}
 			}
@@ -1208,13 +1238,14 @@ void AddOnsCtrl::update_dependency_errors() {
 						too_late = true;
 					}
 				}
-				assert(prev != nullptr);
-				assert(!too_late || next != nullptr);
-				if (too_late && AddOns::order_matters(prev->category, next->category)) {
-					warn_requirements.push_back(format(
-					   _("· ‘%1$s’ requires first ‘%2$s’ and then ‘%3$s’, but they are "
-					     "listed in the wrong order"),
-					   addon->first->descname(), prev->descname(), search_result->first->descname()));
+				if (prev != nullptr) {
+					assert(!too_late || next != nullptr);
+					if (too_late && AddOns::order_matters(prev->category, next->category)) {
+						warn_requirements.push_back(format(
+						   _("• ‘%1$s’ requires first ‘%2$s’ and then ‘%3$s’, but they are "
+						     "listed in the wrong order"),
+						   addon->first->descname(), prev->descname(), search_result->first->descname()));
+					}
 				}
 			}
 		}
@@ -1277,7 +1308,6 @@ void AddOnsCtrl::layout() {
 		int w;
 		int h;
 		server_name_.get_desired_size(&w, &h);
-		server_name_.set_size(w, h);
 		server_name_.set_pos(Vector2i(login_button_.get_x() - w - kRowButtonSpacing,
 		                              login_button_.get_y() + (login_button_.get_h() - h) / 2));
 	}
@@ -1299,12 +1329,8 @@ bool AddOnsCtrl::is_remote(const std::string& name) const {
 		// No data available
 		return true;
 	}
-	for (const auto& r : remotes_) {
-		if (r->internal_name == name) {
-			return true;
-		}
-	}
-	return false;
+	return std::any_of(remotes_.begin(), remotes_.end(),
+	                   [&name](const auto& r) { return r->internal_name == name; });
 }
 
 static void install_translation(const std::string& temp_locale_path,
@@ -1396,7 +1422,8 @@ void AddOnsCtrl::upload_addon(std::shared_ptr<AddOns::AddOnInfo> addon) {
 		}
 	}
 
-	ProgressIndicatorWindow w(&get_topmost_forefather(), addon->descname());
+	ProgressIndicatorWindow w(
+	   &get_topmost_forefather(), UI::WindowStyle::kFsMenu, addon->descname());
 	w.set_message_1(format(_("Uploading ‘%s’…"), addon->descname()));
 	try {
 		int64_t nr_files = 0;
@@ -1437,11 +1464,12 @@ void AddOnsCtrl::upload_addon(std::shared_ptr<AddOns::AddOnInfo> addon) {
 // requirements
 void AddOnsCtrl::install_or_upgrade(std::shared_ptr<AddOns::AddOnInfo> remote,
                                     const bool only_translations) {
-	ProgressIndicatorWindow w(&get_topmost_forefather(), remote->descname());
+	ProgressIndicatorWindow w(
+	   &get_topmost_forefather(), UI::WindowStyle::kFsMenu, remote->descname());
 	w.set_message_1(format(_("Downloading ‘%s’…"), remote->descname()));
 
-	std::string temp_dir =
-	   kTempFileDir + FileSystem::file_separator() + remote->internal_name + kTempFileExtension;
+	std::string temp_dir = kTempFileDir + FileSystem::file_separator() + timestring() + ".addon." +
+	                       remote->internal_name + kTempFileExtension;
 	if (g_fs->file_exists(temp_dir)) {
 		g_fs->fs_unlink(temp_dir);
 	}
@@ -1652,4 +1680,3 @@ step1:
 #endif
 
 }  // namespace AddOnsUI
-}  // namespace FsMenu
