@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2022 by the Widelands Development Team
+ * Copyright (C) 2021-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,13 +33,12 @@
 #include "wlapplication.h"
 #include "wlapplication_options.h"
 
-namespace FsMenu {
 namespace AddOnsUI {
 
 constexpr int16_t kButtonSize = 32;
 constexpr int16_t kSpacing = 4;
 
-AddOnsPackager::AddOnsPackager(MainMenu& parent, AddOnsCtrl& ctrl)
+AddOnsPackager::AddOnsPackager(FsMenu::MainMenu& parent, AddOnsCtrl& ctrl)
    : UI::Window(&parent,
                 UI::WindowStyle::kFsMenu,
                 "addons_packager",
@@ -50,22 +49,47 @@ AddOnsPackager::AddOnsPackager(MainMenu& parent, AddOnsCtrl& ctrl)
                 _("Add-Ons Packager")),
      main_menu_(parent),
      ctrl_(ctrl),
-     main_box_(this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
-     box_left_(&main_box_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     box_right_(&main_box_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     box_left_buttons_(&box_left_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
-     box_right_subbox_header_hbox_(&box_right_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
-     box_right_subbox_header_box_left_(
-        &box_right_subbox_header_hbox_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     box_right_subbox_header_box_right_(
-        &box_right_subbox_header_hbox_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     box_right_addon_specific_(&box_right_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
-     box_right_bottombox_(&box_right_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
-     name_(&box_right_subbox_header_box_right_, 0, 0, 100, UI::PanelStyle::kFsMenu),
-     author_(&box_right_subbox_header_box_right_, 0, 0, 100, UI::PanelStyle::kFsMenu),
-     version_(&box_right_subbox_header_box_right_, 0, 0, 100, UI::PanelStyle::kFsMenu),
-     descr_(*new UI::MultilineEditbox(
-        &box_right_subbox_header_box_right_, 0, 0, 100, 100, UI::PanelStyle::kFsMenu)),
+     main_box_(this, UI::PanelStyle::kFsMenu, "main_box", 0, 0, UI::Box::Horizontal),
+     box_left_(&main_box_, UI::PanelStyle::kFsMenu, "left_box", 0, 0, UI::Box::Vertical),
+     box_right_(&main_box_, UI::PanelStyle::kFsMenu, "right_box", 0, 0, UI::Box::Vertical),
+     box_left_buttons_(
+        &box_left_, UI::PanelStyle::kFsMenu, "left_buttons_box", 0, 0, UI::Box::Horizontal),
+     box_right_subbox_header_hbox_(
+        &box_right_, UI::PanelStyle::kFsMenu, "right_subheader_hbox", 0, 0, UI::Box::Horizontal),
+     box_right_subbox_header_box_left_(&box_right_subbox_header_hbox_,
+                                       UI::PanelStyle::kFsMenu,
+                                       "right_subheader_left_box",
+                                       0,
+                                       0,
+                                       UI::Box::Vertical),
+     box_right_subbox_header_box_right_(&box_right_subbox_header_hbox_,
+                                        UI::PanelStyle::kFsMenu,
+                                        "right_subheader_right_box",
+                                        0,
+                                        0,
+                                        UI::Box::Vertical),
+     box_right_addon_specific_(&box_right_,
+                               UI::PanelStyle::kFsMenu,
+                               "right_addon_specific_box",
+                               0,
+                               0,
+                               UI::Box::Horizontal),
+     box_right_bottombox_(
+        &box_right_, UI::PanelStyle::kFsMenu, "right_bottom_box", 0, 0, UI::Box::Horizontal),
+     name_(&box_right_subbox_header_box_right_, "name", 0, 0, 100, UI::PanelStyle::kFsMenu),
+     author_(&box_right_subbox_header_box_right_, "author", 0, 0, 100, UI::PanelStyle::kFsMenu),
+     version_(&box_right_subbox_header_box_right_, "version", 0, 0, 100, UI::PanelStyle::kFsMenu),
+     min_wl_version_(
+        &box_right_subbox_header_box_right_, "min_wl_version", 0, 0, 100, UI::PanelStyle::kFsMenu),
+     max_wl_version_(
+        &box_right_subbox_header_box_right_, "max_wl_version", 0, 0, 100, UI::PanelStyle::kFsMenu),
+     descr_(*new UI::MultilineEditbox(&box_right_subbox_header_box_right_,
+                                      "description",
+                                      0,
+                                      0,
+                                      100,
+                                      100,
+                                      UI::PanelStyle::kFsMenu)),
      addon_new_(&box_left_buttons_,
                 "addon_new",
                 0,
@@ -108,9 +132,8 @@ AddOnsPackager::AddOnsPackager(MainMenu& parent, AddOnsCtrl& ctrl)
          kButtonSize,
          UI::ButtonStyle::kFsMenuPrimary,
          _("OK")),
-     addons_(&box_left_, 0, 0, 250, 0, UI::PanelStyle::kFsMenu),
-     update_in_progress_(false),
-     progress_window_(this, _("Writing Add-Ons…")) {
+     addons_(&box_left_, "addons", 0, 0, 250, 0, UI::PanelStyle::kFsMenu),
+     progress_window_(this, UI::WindowStyle::kFsMenu, _("Writing Add-Ons…")) {
 	progress_window_.set_visible(false);
 	progress_window_.set_message_1(_("Please be patient while your changes are written."));
 
@@ -126,28 +149,44 @@ AddOnsPackager::AddOnsPackager(MainMenu& parent, AddOnsCtrl& ctrl)
 	box_right_subbox_header_box_right_.add_space(kSpacing);
 	box_right_subbox_header_box_right_.add(&version_, UI::Box::Resizing::kFullSize);
 	box_right_subbox_header_box_right_.add_space(kSpacing);
+	box_right_subbox_header_box_right_.add(&min_wl_version_, UI::Box::Resizing::kFullSize);
+	box_right_subbox_header_box_right_.add_space(kSpacing);
+	box_right_subbox_header_box_right_.add(&max_wl_version_, UI::Box::Resizing::kFullSize);
+	box_right_subbox_header_box_right_.add_space(kSpacing);
 	box_right_subbox_header_box_right_.add(&descr_, UI::Box::Resizing::kFullSize);
 
 	box_right_subbox_header_box_left_.add_space(kSpacing);
 	box_right_subbox_header_box_left_.add(
-	   new UI::Textarea(&box_right_subbox_header_box_left_, UI::PanelStyle::kFsMenu,
+	   new UI::Textarea(&box_right_subbox_header_box_left_, UI::PanelStyle::kFsMenu, "label_name",
 	                    UI::FontStyle::kFsMenuInfoPanelHeading, _("Name:"), UI::Align::kRight),
 	   UI::Box::Resizing::kFullSize);
 	box_right_subbox_header_box_left_.add_space(3 * kSpacing);
 	box_right_subbox_header_box_left_.add(
-	   new UI::Textarea(&box_right_subbox_header_box_left_, UI::PanelStyle::kFsMenu,
+	   new UI::Textarea(&box_right_subbox_header_box_left_, UI::PanelStyle::kFsMenu, "label_author",
 	                    UI::FontStyle::kFsMenuInfoPanelHeading, _("Author:"), UI::Align::kRight),
 	   UI::Box::Resizing::kFullSize);
 	box_right_subbox_header_box_left_.add_space(3 * kSpacing);
 	box_right_subbox_header_box_left_.add(
-	   new UI::Textarea(&box_right_subbox_header_box_left_, UI::PanelStyle::kFsMenu,
+	   new UI::Textarea(&box_right_subbox_header_box_left_, UI::PanelStyle::kFsMenu, "label_version",
 	                    UI::FontStyle::kFsMenuInfoPanelHeading, _("Version:"), UI::Align::kRight),
 	   UI::Box::Resizing::kFullSize);
 	box_right_subbox_header_box_left_.add_space(3 * kSpacing);
 	box_right_subbox_header_box_left_.add(
 	   new UI::Textarea(&box_right_subbox_header_box_left_, UI::PanelStyle::kFsMenu,
-	                    UI::FontStyle::kFsMenuInfoPanelHeading, _("Description:"),
-	                    UI::Align::kRight),
+	                    "label_min_wl_version", UI::FontStyle::kFsMenuInfoPanelHeading,
+	                    _("Minimum Widelands Version:"), UI::Align::kRight),
+	   UI::Box::Resizing::kFullSize);
+	box_right_subbox_header_box_left_.add_space(3 * kSpacing);
+	box_right_subbox_header_box_left_.add(
+	   new UI::Textarea(&box_right_subbox_header_box_left_, UI::PanelStyle::kFsMenu,
+	                    "label_max_wl_version", UI::FontStyle::kFsMenuInfoPanelHeading,
+	                    _("Maximum Widelands Version:"), UI::Align::kRight),
+	   UI::Box::Resizing::kFullSize);
+	box_right_subbox_header_box_left_.add_space(3 * kSpacing);
+	box_right_subbox_header_box_left_.add(
+	   new UI::Textarea(&box_right_subbox_header_box_left_, UI::PanelStyle::kFsMenu,
+	                    "label_description", UI::FontStyle::kFsMenuInfoPanelHeading,
+	                    _("Description:"), UI::Align::kRight),
 	   UI::Box::Resizing::kFullSize);
 
 	box_right_subbox_header_hbox_.add(
@@ -195,6 +234,8 @@ AddOnsPackager::AddOnsPackager(MainMenu& parent, AddOnsCtrl& ctrl)
 	name_.changed.connect([this]() { current_addon_edited(); });
 	author_.changed.connect([this]() { current_addon_edited(); });
 	version_.changed.connect([this]() { current_addon_edited(); });
+	min_wl_version_.changed.connect([this]() { current_addon_edited(); });
+	max_wl_version_.changed.connect([this]() { current_addon_edited(); });
 	descr_.changed.connect([this]() { current_addon_edited(); });
 
 	initialize_mutable_addons();
@@ -275,24 +316,25 @@ void AddOnsPackager::addon_selected() {
 		return;
 	}
 
-	box_right_subbox_header_hbox_.set_visible(true);
-
 	update_in_progress_ = true;
+
+	box_right_subbox_header_hbox_.set_visible(true);
 	name_.set_text(selected->get_descname());
 	descr_.set_text(selected->get_description());
 	author_.set_text(selected->get_author());
 	version_.set_text(selected->get_version());
-	update_in_progress_ = false;
+	min_wl_version_.set_text(selected->get_min_wl_version());
+	max_wl_version_.set_text(selected->get_max_wl_version());
 
-	if (addon_boxes_.count(selected->get_category()) == 0) {
-		return;
+	if (auto it = addon_boxes_.find(selected->get_category()); it != addon_boxes_.end()) {
+		AddOnsPackagerBox* box = it->second.get();
+		box_right_addon_specific_.add(box, UI::Box::Resizing::kExpandBoth);
+		box->set_visible(true);
+		box->load_addon(selected);
+		layout();
 	}
 
-	AddOnsPackagerBox* box = addon_boxes_[selected->get_category()].get();
-	box_right_addon_specific_.add(box, UI::Box::Resizing::kExpandBoth);
-	box->set_visible(true);
-	box->load_addon(selected);
-	layout();
+	update_in_progress_ = false;
 }
 
 void AddOnsPackager::current_addon_edited() {
@@ -303,7 +345,8 @@ void AddOnsPackager::current_addon_edited() {
 	const std::string& sel = addons_.get_selected();
 	AddOns::MutableAddOn* m = mutable_addons_.at(sel).get();
 
-	m->update_info(name_.text(), author_.text(), descr_.get_text(), version_.text());
+	m->update_info(name_.get_text(), author_.get_text(), descr_.get_text(), version_.get_text(),
+	               min_wl_version_.get_text(), max_wl_version_.get_text());
 
 	addons_with_changes_[sel] = false;
 	check_for_unsaved_changes();
@@ -329,7 +372,7 @@ void AddOnsPackager::clicked_new_addon() {
 			return;
 		}
 
-		std::string name = n.text();
+		std::string name = n.get_text();
 
 		const std::string err = check_addon_filename_validity(name);
 		if (!err.empty()) {
@@ -364,7 +407,7 @@ void AddOnsPackager::clicked_new_addon() {
 		// These default strings are not localized because these editboxes are meant to be
 		// filled out in English. We will add localization markup to the resulting config file.
 		a.internal_name = name;
-		a.unlocalized_descname = n.text();
+		a.unlocalized_descname = n.get_text();
 		a.unlocalized_description = "No description";
 		a.unlocalized_author = get_config_string("realname", pgettext("author_name", "Unknown"));
 		a.version = {1, 0, 0};
@@ -414,7 +457,7 @@ void AddOnsPackager::die() {
 		             addons_with_changes_.size()),
 		          addons_with_changes_.size());
 		for (const auto& str : addons_with_changes_) {
-			msg = format(_("%1$s\n· %2$s"), msg, str.first);
+			msg = format(_("%1$s\n• %2$s"), msg, str.first);
 		}
 
 		UI::WLMessageBox m(get_parent(), UI::WindowStyle::kFsMenu, _("Quit Packager"), msg,
@@ -441,7 +484,7 @@ void AddOnsPackager::clicked_discard_changes() {
 		                   addons_with_changes_.size()),
 		          addons_with_changes_.size());
 		for (const auto& str : addons_with_changes_) {
-			msg = format(_("%1$s\n· %2$s"), msg, str.first);
+			msg = format(_("%1$s\n• %2$s"), msg, str.first);
 		}
 	}
 
@@ -466,7 +509,7 @@ void AddOnsPackager::clicked_write_changes() {
 		            addons_with_changes_.size()),
 		   addons_with_changes_.size());
 		for (const auto& str : addons_with_changes_) {
-			msg = format(_("%1$s\n· %2$s"), msg, str.first);
+			msg = format(_("%1$s\n• %2$s"), msg, str.first);
 		}
 	}
 	msg += "\n\n";
@@ -519,20 +562,78 @@ bool AddOnsPackager::do_write_addon_to_disk(const std::string& addon) {
 	AddOns::MutableAddOn* m = mutable_addons_.at(addon).get();
 
 	// Check that the version string is valid and beautify it
+	std::string currently_checking;
 	try {
-		AddOns::AddOnVersion v = AddOns::string_to_version(m->get_version());
-		m->set_version(AddOns::version_to_string(v, false));
+		currently_checking = m->get_version();
+		AddOns::AddOnVersion addon_v = AddOns::string_to_version(currently_checking);
+		m->set_version(AddOns::version_to_string(addon_v, false));
 		if (addons_.has_selection() && addons_.get_selected() == addon) {
 			version_.set_text(m->get_version());
 		}
+
+		currently_checking = m->get_min_wl_version();
+		AddOns::AddOnVersion min_wl_v = AddOns::string_to_version(currently_checking);
+		m->set_min_wl_version(AddOns::version_to_string(min_wl_v, false));
+		if (addons_.has_selection() && addons_.get_selected() == addon) {
+			min_wl_version_.set_text(m->get_min_wl_version());
+		}
+
+		currently_checking = m->get_max_wl_version();
+		AddOns::AddOnVersion max_wl_v = AddOns::string_to_version(currently_checking);
+		m->set_max_wl_version(AddOns::version_to_string(max_wl_v, false));
+		if (addons_.has_selection() && addons_.get_selected() == addon) {
+			max_wl_version_.set_text(m->get_max_wl_version());
+		}
+
+		if (!min_wl_v.empty() && !max_wl_v.empty() && AddOns::is_newer_version(max_wl_v, min_wl_v)) {
+			main_menu_.show_messagebox(
+			   _("Invalid Version Requirement"),
+			   format(_("The minimum Widelands version ‘%1$s’ may not be newer than the maximum "
+			            "Widelands version ‘%2$s’. The add-on ‘%3$s’ will not be saved."),
+			          m->get_min_wl_version(), m->get_max_wl_version(), addon));
+			return false;
+		}
 	} catch (...) {
 		main_menu_.show_messagebox(
 		   _("Invalid Version"),
 		   format(_("‘%1$s’ is not a valid version string. The add-on ‘%2$s’ will not be saved."),
-		          m->get_version(), addon));
+		          currently_checking, addon));
 		return false;
 	}
 
+	// Compare the version requirements of maps to the specified min version
+	if (m->get_category() == AddOns::AddOnCategory::kMaps) {
+		AddOns::AddOnVersion nominal_min = AddOns::string_to_version(m->get_min_wl_version());
+		AddOns::AddOnVersion actual_min =
+		   dynamic_cast<AddOns::MapsAddon*>(m)->detect_min_wl_version();
+		if (actual_min != AddOns::MapsAddon::kNoVersionRequirement &&
+		    AddOns::is_newer_version(nominal_min, actual_min)) {
+			UI::WLMessageBox mbox(
+			   get_parent(), UI::WindowStyle::kFsMenu, _("Version Requirement"),
+			   nominal_min.empty() ?
+               format(_("The add-on ‘%1$s’ does not specify a minimum Widelands version. "
+			               "None of the contained maps can be loaded with a Widelands version older "
+			               "than %2$s. "
+			               "The minimum version requirement will automatically be set accordingly."),
+			             addon, AddOns::version_to_string(actual_min)) :
+               format(_("The add-on ‘%1$s’ specifies a minimum Widelands version of %2$s. "
+			               "However, none of the contained maps can be loaded with a Widelands "
+			               "version older than %3$s. "
+			               "The version requirement will automatically be changed accordingly."),
+			             addon, AddOns::version_to_string(nominal_min),
+			             AddOns::version_to_string(actual_min)),
+			   UI::WLMessageBox::MBoxType::kOkCancel);
+			if (mbox.run<UI::Panel::Returncodes>() != UI::Panel::Returncodes::kOk) {
+				return false;
+			}
+
+			m->set_min_wl_version(AddOns::version_to_string(actual_min, false));
+			if (addons_.has_selection() && addons_.get_selected() == addon) {
+				min_wl_version_.set_text(m->get_min_wl_version());
+			}
+		}
+	}
+
 	try {
 		return m->write_to_disk();
 	} catch (const WLWarning& e) {
@@ -544,4 +645,3 @@ bool AddOnsPackager::do_write_addon_to_disk(const std::string& addon) {
 }
 
 }  // namespace AddOnsUI
-}  // namespace FsMenu
