@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2022 by the Widelands Development Team
+ * Copyright (C) 2021-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,6 @@
 #include "ui_fsmenu/addons/remote_interaction.h"
 #include "wlapplication_options.h"
 
-namespace FsMenu {
 namespace AddOnsUI {
 
 namespace {
@@ -123,6 +122,7 @@ InstalledAddOnRow::InstalledAddOnRow(Panel* parent,
                                      bool enabled)
    : UI::Panel(parent,
                UI::PanelStyle::kFsMenu,
+               format("installed_addon_row_%s", info->internal_name),
                0,
                0,
                3 * kRowButtonSize,
@@ -149,12 +149,14 @@ InstalledAddOnRow::InstalledAddOnRow(Panel* parent,
                                                   "images/ui_basic/checkbox_empty.png"),
                      enabled ? _("Disable") : _("Enable"),
                      UI::Button::VisualState::kFlat),
-     icon_(this, UI::PanelStyle::kFsMenu, info_->icon),
+     icon_(this, UI::PanelStyle::kFsMenu, "icon", info_->icon),
      category_(this,
                UI::PanelStyle::kFsMenu,
+               "category",
                g_image_cache->get(AddOns::kAddOnCategories.at(info->category).icon)),
      version_(this,
               UI::PanelStyle::kFsMenu,
+              "version",
               UI::FontStyle::kFsMenuInfoPanelHeading,
               0,
               0,
@@ -165,6 +167,7 @@ InstalledAddOnRow::InstalledAddOnRow(Panel* parent,
               UI::Align::kCenter),
      txt_(
         this,
+        "description",
         0,
         0,
         24,
@@ -207,6 +210,7 @@ InstalledAddOnRow::InstalledAddOnRow(Panel* parent,
 		}
 		NEVER_HERE();
 	});
+	toggle_enabled_.set_enabled(info->matches_widelands_version());
 	category_.set_handle_mouse(true);
 	category_.set_tooltip(
 	   format(_("Category: %s"), AddOns::kAddOnCategories.at(info->category).descname()));
@@ -260,7 +264,13 @@ RemoteAddOnRow::RemoteAddOnRow(Panel* parent,
                                std::shared_ptr<AddOns::AddOnInfo> info,
                                const AddOns::AddOnVersion& installed_version,
                                uint32_t installed_i18n_version)
-   : UI::Panel(parent, UI::PanelStyle::kFsMenu, 0, 0, 3 * kRowButtonSize, 4 * kRowButtonSize),
+   : UI::Panel(parent,
+               UI::PanelStyle::kFsMenu,
+               format("remote_addon_row_%s", info->internal_name),
+               0,
+               0,
+               3 * kRowButtonSize,
+               4 * kRowButtonSize),
      info_(info),
      install_(this,
               "install",
@@ -298,17 +308,23 @@ RemoteAddOnRow::RemoteAddOnRow(Panel* parent,
                UI::ButtonStyle::kFsMenuSecondary,
                "…",
                _("Comments and Votes")),
-     icon_(this, UI::PanelStyle::kFsMenu, info_->icon),
+     icon_(this, UI::PanelStyle::kFsMenu, "icon", info_->icon),
      category_(this,
                UI::PanelStyle::kFsMenu,
+               "category",
                g_image_cache->get(AddOns::kAddOnCategories.at(info->category).icon)),
      verified_(this,
                UI::PanelStyle::kFsMenu,
+               "verified",
                g_image_cache->get(info->verified ? "images/ui_basic/list_selected.png" :
                                                    "images/ui_basic/stop.png")),
-     quality_(this, UI::PanelStyle::kFsMenu, AddOnQuality::kQualities.at(info->quality)().icon),
+     quality_(this,
+              UI::PanelStyle::kFsMenu,
+              "quality",
+              AddOnQuality::kQualities.at(info->quality)().icon),
      version_(this,
               UI::PanelStyle::kFsMenu,
+              "version",
               UI::FontStyle::kFsMenuInfoPanelHeading,
               0,
               0,
@@ -319,6 +335,7 @@ RemoteAddOnRow::RemoteAddOnRow(Panel* parent,
               UI::Align::kCenter),
      bottom_row_left_(this,
                       UI::PanelStyle::kFsMenu,
+                      "timestamp",
                       UI::FontStyle::kFsTooltip,
                       0,
                       0,
@@ -329,6 +346,7 @@ RemoteAddOnRow::RemoteAddOnRow(Panel* parent,
      bottom_row_right_(
         this,
         UI::PanelStyle::kFsMenu,
+        "statistics",
         UI::FontStyle::kFsTooltip,
         0,
         0,
@@ -337,7 +355,7 @@ RemoteAddOnRow::RemoteAddOnRow(Panel* parent,
         info->internal_name.empty() ?
            "" :
            format(
-              /** TRANSLATORS: Filesize · Download count · Average rating · Number of comments ·
+              /** TRANSLATORS: Filesize • Download count • Average rating • Number of comments •
                  Number of screenshots */
               _("%1$s   ⬇ %2$u   ★ %3$s   “” %4$u   ▣ %5$u"),
               filesize_string(info->total_file_size),
@@ -348,6 +366,7 @@ RemoteAddOnRow::RemoteAddOnRow(Panel* parent,
         UI::Align::kRight),
      txt_(
         this,
+        "description",
         0,
         0,
         24,
@@ -402,6 +421,9 @@ RemoteAddOnRow::RemoteAddOnRow(Panel* parent,
 		ctrl->install_or_upgrade(info_, false);
 		ctrl->rebuild(true);
 	});
+#ifdef NDEBUG
+	install_.set_enabled(info->matches_widelands_version());
+#endif
 	upgrade_.sigclicked.connect([this, ctrl, info, installed_version]() {
 		if (!info->verified || !matches_keymod(SDL_GetModState(), KMOD_CTRL)) {
 			UI::WLMessageBox w(
@@ -428,6 +450,9 @@ RemoteAddOnRow::RemoteAddOnRow(Panel* parent,
 		ctrl->install_or_upgrade(info, !full_upgrade_possible_);
 		ctrl->rebuild(true);
 	});
+#ifdef NDEBUG
+	upgrade_.set_enabled(info->matches_widelands_version());
+#endif
 	if (info->internal_name.empty()) {
 		install_.set_enabled(false);
 		upgrade_.set_enabled(false);
@@ -527,4 +552,3 @@ bool RemoteAddOnRow::upgradeable() const {
 }
 
 }  // namespace AddOnsUI
-}  // namespace FsMenu
