@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,6 +44,7 @@ LaunchGame::LaunchGame(MenuCapsule& fsmm,
 
      configure_game_(&right_column_content_box_,
                      UI::PanelStyle::kFsMenu,
+                     "label_configure",
                      UI::FontStyle::kFsGameSetupHeadings,
                      0,
                      0,
@@ -53,6 +54,7 @@ LaunchGame::LaunchGame(MenuCapsule& fsmm,
                      UI::Align::kCenter),
      warn_desyncing_addon_(
         &right_column_content_box_,
+        "warn_desync",
         0,
         0,
         10,
@@ -75,12 +77,85 @@ LaunchGame::LaunchGame(MenuCapsule& fsmm,
                              UI::DropdownType::kTextual,
                              UI::PanelStyle::kFsMenu,
                              UI::ButtonStyle::kFsMenuMenu),
-     peaceful_(
-        &right_column_content_box_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Peaceful mode")),
-     custom_starting_positions_(&right_column_content_box_,
-                                UI::PanelStyle::kFsMenu,
-                                Vector2i::zero(),
-                                _("Custom starting positions")),
+     win_condition_duration_(&right_column_content_box_,
+                             "win_condition_duration",
+                             0,
+                             0,
+                             0,
+                             240,
+                             Widelands::kDefaultWinConditionDuration,
+                             15,        // 15 minutes minimum gametime
+                             512 * 60,  // 512 hours maximum gametime (arbitrary limit)
+                             UI::PanelStyle::kFsMenu,
+                             _("Playing time"),
+                             UI::SpinBox::Units::kMinutes,
+                             UI::SpinBox::Type::kBig,
+                             5,
+                             60),
+
+     toggle_advanced_options_(&right_column_content_box_,
+                              "toggle_advanced",
+                              0,
+                              0,
+                              0,
+                              ok_.get_h(),
+                              UI::ButtonStyle::kFsMenuSecondary,
+                              "" /* set later */,
+                              _("Show or hide additional game configuration options")),
+     advanced_options_box_(&right_column_content_box_,
+                           UI::PanelStyle::kFsMenu,
+                           "advanced_options_box",
+                           0,
+                           0,
+                           UI::Box::Vertical),
+     game_flag_checkboxes_({
+        {GameSettings::Flags::kPeaceful,
+         {new UI::Checkbox(&advanced_options_box_,
+                           UI::PanelStyle::kFsMenu,
+                           "peaceful",
+                           Vector2i::zero(),
+                           _("Peaceful mode")),
+          &LaunchGame::update_peaceful_mode}},
+
+        {GameSettings::Flags::kFogless,
+         {new UI::Checkbox(&advanced_options_box_,
+                           UI::PanelStyle::kFsMenu,
+                           "fogless",
+                           Vector2i::zero(),
+                           _("No fog of war")),
+          &LaunchGame::update_fogless}},
+
+        {GameSettings::Flags::kForbidDiplomacy,
+         {new UI::Checkbox(&advanced_options_box_,
+                           UI::PanelStyle::kFsMenu,
+                           "forbid_diplomacy",
+                           Vector2i::zero(),
+                           _("Forbid diplomacy")),
+          &LaunchGame::update_forbid_diplomacy}},
+
+        {GameSettings::Flags::kAllowNavalWarfare,
+         {new UI::Checkbox(&advanced_options_box_,
+                           UI::PanelStyle::kFsMenu,
+                           "allow_naval_warfare",
+                           Vector2i::zero(),
+                           _("Enable naval warfare (experimental feature)")),
+          &LaunchGame::update_naval_warfare}},
+
+        {GameSettings::Flags::kCustomStartingPositions,
+         {new UI::Checkbox(&advanced_options_box_,
+                           UI::PanelStyle::kFsMenu,
+                           "custom_starting_positions",
+                           Vector2i::zero(),
+                           _("Custom starting positions")),
+          &LaunchGame::update_custom_starting_positions}},
+
+     }),
+     write_replay_(&advanced_options_box_,
+                   UI::PanelStyle::kFsMenu,
+                   "write_replay",
+                   Vector2i::zero(),
+                   _("Write Replay")),
+
      choose_map_(mpg && settings.can_change_map() && !preconfigured ?
                     new UI::Button(&right_column_content_box_,
                                    "choose_map",
@@ -105,11 +180,21 @@ LaunchGame::LaunchGame(MenuCapsule& fsmm,
      // Variables and objects used in the menu
      settings_(settings),
      ctrl_(ctrl),
-     peaceful_mode_forbidden_(false) {
+     has_desyncing_addon_(false) {
 	warn_desyncing_addon_.set_visible(false);
+	write_replay_.set_state(true);
+
 	win_condition_dropdown_.selected.connect([this]() { win_condition_selected(); });
-	peaceful_.changed.connect([this]() { toggle_peaceful(); });
-	custom_starting_positions_.changed.connect([this]() { toggle_custom_starting_positions(); });
+	win_condition_duration_.changed.connect([this]() { win_condition_duration_changed(); });
+	for (auto& pair : game_flag_checkboxes_) {
+		pair.second.first->changed.connect(
+		   [this, pair]() { settings_.set_flag(pair.first, pair.second.first->get_state()); });
+	}
+	toggle_advanced_options_.sigclicked.connect([this]() {
+		toggle_advanced_options_.toggle();
+		update_advanced_options();
+	});
+
 	if (choose_map_ != nullptr) {
 		choose_map_->sigclicked.connect([this]() { clicked_select_map(); });
 	}
@@ -120,8 +205,8 @@ LaunchGame::LaunchGame(MenuCapsule& fsmm,
 
 	lua_.reset(new LuaInterface());
 	add_all_widgets();
-	add_behaviour_to_widgets();
 
+	update_advanced_options();
 	layout();
 
 	initialization_complete();
@@ -137,14 +222,24 @@ void LaunchGame::add_all_widgets() {
 	right_column_content_box_.add(&map_details_, UI::Box::Resizing::kExpandBoth);
 	right_column_content_box_.add_space(1 * kPadding);
 	right_column_content_box_.add(&warn_desyncing_addon_, UI::Box::Resizing::kFullSize);
-	right_column_content_box_.add_space(4 * kPadding);
+	right_column_content_box_.add_space(1 * kPadding);
 	right_column_content_box_.add(&configure_game_, UI::Box::Resizing::kAlign, UI::Align::kCenter);
-	right_column_content_box_.add_space(3 * kPadding);
+	right_column_content_box_.add_space(1 * kPadding);
 	right_column_content_box_.add(&win_condition_dropdown_, UI::Box::Resizing::kFullSize);
+	right_column_content_box_.add_space(1 * kPadding);
+	right_column_content_box_.add(&win_condition_duration_, UI::Box::Resizing::kFullSize);
 	right_column_content_box_.add_space(3 * kPadding);
-	right_column_content_box_.add(&peaceful_, UI::Box::Resizing::kFullSize);
-	right_column_content_box_.add_space(3 * kPadding);
-	right_column_content_box_.add(&custom_starting_positions_, UI::Box::Resizing::kFullSize);
+	right_column_content_box_.add(&toggle_advanced_options_, UI::Box::Resizing::kFullSize);
+	right_column_content_box_.add_space(1 * kPadding);
+	right_column_content_box_.add(&advanced_options_box_, UI::Box::Resizing::kExpandBoth);
+
+	for (auto& pair : game_flag_checkboxes_) {
+		advanced_options_box_.add(pair.second.first, UI::Box::Resizing::kFullSize);
+		advanced_options_box_.add_space(1 * kPadding);
+	}
+	advanced_options_box_.add(&write_replay_, UI::Box::Resizing::kFullSize);
+	advanced_options_box_.set_scrolling(true);
+
 	if (choose_map_ != nullptr) {
 		right_column_content_box_.add_space(3 * kPadding);
 		right_column_content_box_.add(choose_map_, UI::Box::Resizing::kFullSize);
@@ -155,53 +250,77 @@ void LaunchGame::add_all_widgets() {
 	}
 }
 
-void LaunchGame::add_behaviour_to_widgets() {
-	win_condition_dropdown_.selected.connect([this]() { win_condition_selected(); });
-	peaceful_.changed.connect([this]() { toggle_peaceful(); });
-}
 void LaunchGame::layout() {
 	TwoColumnsFullNavigationMenu::layout();
 	win_condition_dropdown_.set_desired_size(0, standard_height_);
 
-	map_details_.set_max_size(0, right_column_box_.get_h() / 3);
-	map_details_.force_new_dimensions(right_column_width_, standard_height_);
+	map_details_.set_max_size(0, right_column_box_.get_h() / 4);
+	advanced_options_box_.set_max_size(0, 2 * kStateboxSize + kPadding);
+
+	int w;
+	int h;
+	// The warning might overflow the available space,
+	// hide it, if it does not fit
+	warn_desyncing_addon_.set_visible(has_desyncing_addon_);
+	right_column_content_box_.get_desired_size(&w, &h);
+	int h_max = (right_column_box_.get_h() - button_box_.get_h() - 5 * kPadding);
+	if (h > h_max) {
+		warn_desyncing_addon_.set_visible(false);
+	}
 }
 
 void LaunchGame::update_warn_desyncing_addon() {
-	for (const auto& pair : AddOns::g_addons) {
-		if (pair.second && !pair.first->sync_safe) {
-			warn_desyncing_addon_.set_visible(true);
-			return;
-		}
-	}
-	warn_desyncing_addon_.set_visible(false);
+	has_desyncing_addon_ = std::any_of(
+	   AddOns::g_addons.begin(), AddOns::g_addons.end(),
+	   [](const AddOns::AddOnState& addon) { return addon.second && !addon.first->sync_safe; });
+
+	// TODO(Nordfriese): The scenario check is a quickfix for #5745 and related #4531,
+	// remove when scenario replays are functional
+	write_replay_.set_visible(!has_desyncing_addon_ && !settings_.settings().scenario);
+}
+
+bool LaunchGame::should_write_replay() const {
+	return write_replay_.is_visible() && write_replay_.get_state();
+}
+
+void LaunchGame::update_advanced_options() {
+	const bool show = toggle_advanced_options_.style() == UI::Button::VisualState::kPermpressed;
+	advanced_options_box_.set_visible(show);
+	toggle_advanced_options_.set_title(show ? _("Show fewer options") : _("Show more options"));
+	layout();
 }
 
 void LaunchGame::update_peaceful_mode() {
+	UI::Checkbox* checkbox = game_flag_checkboxes_.at(GameSettings::Flags::kPeaceful).first;
 	bool forbidden =
 	   peaceful_mode_forbidden_ || settings_.settings().scenario || settings_.settings().savegame;
-	peaceful_.set_enabled(!forbidden && settings_.can_change_map());
+
+	checkbox->set_enabled(!forbidden && settings_.can_change_map());
 	if (forbidden) {
-		peaceful_.set_state(false);
+		checkbox->set_state(false);
 	}
+
 	if (settings_.settings().scenario) {
-		peaceful_.set_tooltip(_("The relations between players are set by the scenario"));
+		checkbox->set_tooltip(_("The relations between players are set by the scenario"));
 	} else if (settings_.settings().savegame) {
-		peaceful_.set_tooltip(_("The relations between players are set by the saved game"));
+		checkbox->set_tooltip(_("The relations between players are set by the saved game"));
 	} else if (peaceful_mode_forbidden_) {
-		peaceful_.set_tooltip(_("The selected win condition does not allow peaceful matches"));
+		checkbox->set_tooltip(_("The selected win condition does not allow peaceful matches"));
 	} else {
-		peaceful_.set_tooltip(_("Forbid fighting between players"));
+		checkbox->set_tooltip(_("Forbid fighting between players"));
 	}
 }
 
 void LaunchGame::update_custom_starting_positions() {
+	UI::Checkbox* checkbox =
+	   game_flag_checkboxes_.at(GameSettings::Flags::kCustomStartingPositions).first;
 	const GameSettings& settings = settings_.settings();
 	const bool forbidden = settings.scenario || settings.savegame;
+
 	if (forbidden || !settings_.can_change_map()) {
-		custom_starting_positions_.set_enabled(false);
+		checkbox->set_enabled(false);
 		if (forbidden) {
-			custom_starting_positions_.set_state(false);
+			checkbox->set_state(false);
 		}
 	} else {
 		bool allowed = false;
@@ -214,25 +333,100 @@ void LaunchGame::update_custom_starting_positions() {
 				break;
 			}
 		}
-		custom_starting_positions_.set_enabled(allowed);
+		checkbox->set_enabled(allowed);
 		if (!allowed) {
-			custom_starting_positions_.set_state(false);
-			custom_starting_positions_.set_tooltip(
+			checkbox->set_state(false);
+			checkbox->set_tooltip(
 			   _("All selected starting conditions ignore the map’s starting positions"));
 			return;
 		}
 	}
+
 	if (settings_.settings().scenario) {
-		custom_starting_positions_.set_tooltip(_("The starting positions are set by the scenario"));
+		checkbox->set_tooltip(_("The starting positions are set by the scenario"));
 	} else if (settings_.settings().savegame) {
-		custom_starting_positions_.set_tooltip(_("The starting positions are set by the saved game"));
+		checkbox->set_tooltip(_("The starting positions are set by the saved game"));
 	} else {
-		custom_starting_positions_.set_tooltip(_(
+		checkbox->set_tooltip(_(
 		   "Allow the players to choose their own starting positions at the beginning of the game"));
 	}
 }
 
+void LaunchGame::update_fogless() {
+	UI::Checkbox* checkbox = game_flag_checkboxes_.at(GameSettings::Flags::kFogless).first;
+	const GameSettings& settings = settings_.settings();
+	const bool forbidden = settings.scenario || settings.savegame;
+
+	if (forbidden || !settings_.can_change_map()) {
+		checkbox->set_enabled(false);
+		if (forbidden) {
+			checkbox->set_state(false);
+		}
+	} else {
+		checkbox->set_enabled(true);
+	}
+
+	if (settings_.settings().scenario) {
+		checkbox->set_tooltip(_("Player vision is set by the scenario"));
+	} else if (settings_.settings().savegame) {
+		checkbox->set_tooltip(_("Player vision is set by the saved game"));
+	} else {
+		checkbox->set_tooltip(_("Give all players permanent vision of the entire map"));
+	}
+}
+
+void LaunchGame::update_naval_warfare() {
+	UI::Checkbox* checkbox = game_flag_checkboxes_.at(GameSettings::Flags::kAllowNavalWarfare).first;
+	const GameSettings& settings = settings_.settings();
+	const bool forbidden = settings.scenario || settings.savegame || !map_is_seafaring_;
+
+	if (forbidden || !settings_.can_change_map()) {
+		checkbox->set_enabled(false);
+		if (forbidden) {
+			checkbox->set_state(false);
+		}
+	} else {
+		checkbox->set_enabled(true);
+	}
+
+	checkbox->set_visible(map_is_seafaring_);
+
+	if (settings_.settings().scenario) {
+		checkbox->set_tooltip(_("Naval warfare is set by the scenario"));
+	} else if (settings_.settings().savegame) {
+		checkbox->set_tooltip(_("Naval warfare is set by the saved game"));
+	} else {
+		checkbox->set_tooltip(
+		   _("Enable coastal invasions and ship-to-ship battles. This feature is experimental and "
+		     "may undergo substantial redesign in future versions of Widelands."));
+	}
+}
+
+void LaunchGame::update_forbid_diplomacy() {
+	UI::Checkbox* checkbox = game_flag_checkboxes_.at(GameSettings::Flags::kForbidDiplomacy).first;
+	const GameSettings& settings = settings_.settings();
+	const bool forbidden = settings.scenario || settings.savegame;
+
+	if (forbidden || !settings_.can_change_map()) {
+		checkbox->set_enabled(false);
+		if (forbidden) {
+			checkbox->set_state(false);
+		}
+	} else {
+		checkbox->set_enabled(true);
+	}
+
+	if (settings_.settings().scenario) {
+		checkbox->set_tooltip(_("Player relations are set by the scenario"));
+	} else if (settings_.settings().savegame) {
+		checkbox->set_tooltip(_("Player relations are set by the saved game"));
+	} else {
+		checkbox->set_tooltip(_("Disallow players to change their alliances during the game"));
+	}
+}
+
 bool LaunchGame::init_win_condition_label() {
+	win_condition_duration_.set_visible(false);
 	if (settings_.settings().scenario) {
 		win_condition_dropdown_.set_enabled(false);
 		win_condition_dropdown_.set_label(_("Scenario"));
@@ -268,6 +462,7 @@ void LaunchGame::update_win_conditions() {
 				tags = map.get_tags();
 			}
 		}
+		map_is_seafaring_ = tags.count("seafaring") != 0;
 		load_win_conditions(tags);
 	}
 }
@@ -353,11 +548,7 @@ LaunchGame::win_condition_if_valid(const std::string& win_condition_script,
 	return t;
 }
 
-void LaunchGame::toggle_peaceful() {
-	settings_.set_peaceful_mode(peaceful_.get_state());
-}
-
-void LaunchGame::toggle_custom_starting_positions() {
-	settings_.set_custom_starting_positions(custom_starting_positions_.get_state());
+void LaunchGame::win_condition_duration_changed() {
+	settings_.set_win_condition_duration(win_condition_duration_.get_value());
 }
 }  // namespace FsMenu
