@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -48,7 +48,6 @@ LaunchMPG::LaunchMPG(MenuCapsule& fsmm,
                      GameSettingsProvider& settings,
                      GameController& ctrl,
                      ChatProvider& chat,
-                     Widelands::Game& g,
                      bool game_done_on_cancel,
                      const std::function<void()>& c)
    : LaunchGame(fsmm, settings, &ctrl, false, true),
@@ -66,8 +65,21 @@ LaunchMPG::LaunchMPG(MenuCapsule& fsmm,
      help_(nullptr),
 
      mpsg_(this, &left_column_box_, 0, 0, 0, 0, &settings, scale_factor * standard_height_),
-     chat_(new GameChatPanel(&left_column_box_, 0, 0, 0, 0, chat, UI::PanelStyle::kFsMenu)),
-     game_(g) {
+     chat_(new GameChatPanel(
+        &left_column_box_,
+        [&settings](int player_number) {
+	        const GameSettings& s = settings.settings();
+	        return (player_number > 0 && player_number <= static_cast<int>(s.players.size())) ?
+                     &s.players.at(player_number - 1).color :
+                     nullptr;
+        },
+        0,
+        0,
+        0,
+        0,
+        chat,
+        &g_chat_sent_history,
+        UI::PanelStyle::kFsMenu)) {
 
 	help_button_.sigclicked.connect([this]() { help_clicked(); });
 	chat_->aborted.connect([this]() { die(); });
@@ -81,14 +93,15 @@ LaunchMPG::LaunchMPG(MenuCapsule& fsmm,
 	ok_.set_enabled(settings_.can_launch());
 
 	left_column_box_.add(&mpsg_, UI::Box::Resizing::kExpandBoth);
-	left_column_box_.add_space(kPadding);
-	left_column_box_.add(chat_.get(), UI::Box::Resizing::kExpandBoth);
+	left_column_box_.add_space(8 * kPadding);
+	left_column_box_.add(chat_.get(), UI::Box::Resizing::kFullSize);
 
 	subscriber_ = Notifications::subscribe<NoteGameSettings>([this](const NoteGameSettings& s) {
 		if (s.action == NoteGameSettings::Action::kMap) {
 			map_changed();
 		}
 	});
+	update_warn_desyncing_addon();
 	layout();
 	initialization_complete();
 }
@@ -107,13 +120,20 @@ void LaunchMPG::layout() {
 	help_button_.set_size(standard_height_, standard_height_);
 	help_button_.set_pos(Vector2i(get_inner_w() - help_button_.get_w(), 0));
 
-	mpsg_.set_max_size(0, left_column_box_.get_h() / 2);
-
-	mpsg_.force_new_dimensions(
-	   left_column_box_.get_w(), left_column_box_.get_h() / 2, scale_factor * standard_height_);
+	// Reset size to fit left_column_box_, then relayout
+	if (chat_ != nullptr) {
+		chat_->set_desired_size(0, 0);
+	}
+	uint32_t h = left_column_box_.get_h() / 2 - 4 * kPadding;
+	// Assign heights to properly layout the scrollable box
+	mpsg_.force_new_dimensions(left_column_box_.get_w(), h, scale_factor * standard_height_);
+	if (chat_ != nullptr) {
+		chat_->set_desired_size(0, h);
+	}
+	LaunchGame::layout();
 
 	// set focus to chat input
-	if (chat_) {
+	if (chat_ != nullptr) {
 		chat_->focus_edit();
 	}
 }
@@ -125,9 +145,13 @@ void LaunchMPG::win_condition_selected() {
 
 		std::unique_ptr<LuaTable> t = lua_->run_script(last_win_condition_);
 		t->do_not_warn_about_unaccessed_keys();
-		peaceful_mode_forbidden_ = !t->get_bool("peaceful_mode_allowed");
+		win_condition_duration_.set_visible(t->has_key("configurable_time") &&
+		                                    t->get_bool("configurable_time"));
+		peaceful_mode_forbidden_ =
+		   t->has_key("peaceful_mode_allowed") && !t->get_bool("peaceful_mode_allowed");
 		update_peaceful_mode();
 		mpsg_.update_players();
+		layout();
 	}
 }
 
@@ -148,7 +172,7 @@ void LaunchMPG::clicked_select_map_callback(const MapData* map, const bool scena
 	}
 
 	settings_.set_scenario(scenario);
-	settings_.set_map(map->name, map->filename, map->theme, map->background, map->nrplayers);
+	settings_.set_map(map->name, map->filenames.at(0), map->theme, map->background, map->nrplayers);
 
 	map_changed();
 	update_win_conditions();
@@ -232,9 +256,10 @@ void LaunchMPG::clicked_ok() {
 }
 
 void LaunchMPG::think() {
-	if (ctrl_ != nullptr) {
-		ctrl_->think();
-	}
+	assert(ctrl_ != nullptr);
+	ctrl_->set_write_replay(should_write_replay());
+	ctrl_->think();
+
 	refresh();
 
 	// unfocus chat window when other UI element has focus
@@ -274,10 +299,10 @@ void LaunchMPG::refresh() {
 
 	ok_.set_enabled(settings_.can_launch());
 
-	update_peaceful_mode();
-	update_custom_starting_positions();
-	custom_starting_positions_.set_state(settings_.get_custom_starting_positions());
-	peaceful_.set_state(settings_.is_peaceful_mode());
+	for (auto& pair : game_flag_checkboxes_) {
+		(this->*pair.second.second)();
+		pair.second.first->set_state(settings_.get_flag(pair.first));
+	}
 
 	if (!settings_.can_change_map() && !init_win_condition_label()) {
 		try {
@@ -287,6 +312,10 @@ void LaunchMPG::refresh() {
 			i18n::Textdomain td("win_conditions");
 			win_condition_dropdown_.set_label(_(t->get_string("name")));
 			win_condition_dropdown_.set_tooltip(_(t->get_string("description")));
+			win_condition_duration_.set_visible(t->has_key("configurable_time") &&
+			                                    t->get_bool("configurable_time"));
+			const int32_t duration = settings_.get_win_condition_duration();
+			win_condition_duration_.set_interval(duration, duration, false);
 		} catch (LuaScriptNotExistingError&) {
 			win_condition_dropdown_.set_label(_("Error"));
 			win_condition_dropdown_.set_tooltip(
