@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -50,22 +50,27 @@ LaunchSPG::LaunchSPG(MenuCapsule& fsmm,
 	settings_.set_scenario(scenario);
 	if (!preconfigured_) {
 		assert(!settings_.settings().tribes.empty());
-		settings_.set_map(
-		   mapdata->name, mapdata->filename, mapdata->theme, mapdata->background, mapdata->nrplayers);
+		settings_.set_map(mapdata->name, mapdata->filenames.at(0), mapdata->theme,
+		                  mapdata->background, mapdata->nrplayers);
 	}
 	Notifications::publish(NoteGameSettings(NoteGameSettings::Action::kMap));
 
 	update_win_conditions();
-	update_peaceful_mode();
-	update_custom_starting_positions();
+	for (auto& pair : game_flag_checkboxes_) {
+		(this->*pair.second.second)();
+	}
 	update_warn_desyncing_addon();
+
 	update();
 	layout();
 	initialization_complete();
 }
 
 void LaunchSPG::update() {
-	peaceful_.set_state(settings_.is_peaceful_mode());
+	for (auto& pair : game_flag_checkboxes_) {
+		pair.second.first->set_state(settings_.get_flag(pair.first));
+	}
+
 	if (preconfigured_) {
 		map_details_.update(&settings_, *game_->mutable_map());
 		ok_.set_enabled(true);
@@ -113,7 +118,10 @@ void LaunchSPG::win_condition_selected() {
 
 		std::unique_ptr<LuaTable> t = lua_->run_script(last_win_condition_);
 		t->do_not_warn_about_unaccessed_keys();
-		peaceful_mode_forbidden_ = !t->get_bool("peaceful_mode_allowed");
+		win_condition_duration_.set_visible(t->has_key("configurable_time") &&
+		                                    t->get_bool("configurable_time"));
+		peaceful_mode_forbidden_ =
+		   t->has_key("peaceful_mode_allowed") && !t->get_bool("peaceful_mode_allowed");
 		update_peaceful_mode();
 		player_setup_.update();
 	}
@@ -145,6 +153,8 @@ void LaunchSPG::clicked_ok() {
 	upcast(SinglePlayerGameSettingsProvider, sp, &settings_);
 	assert(sp);
 	game_->set_ai_training_mode(get_config_bool("ai_training", false));
+	game_->set_write_replay(should_write_replay());
+	game_->logic_rand_seed(RNG::static_rand());
 	try {
 		if (sp->settings().scenario) {  // scenario
 			game_->run_splayer_scenario_direct({sp->get_map()}, "");
@@ -159,14 +169,20 @@ void LaunchSPG::clicked_ok() {
 				tipstexts.push_back(sp->get_players_tribe());
 			}
 			game_->create_loader_ui(
-			   tipstexts, true, sp->settings().map_theme, sp->settings().map_background);
+			   tipstexts, true, sp->settings().map_theme, sp->settings().map_background, true);
 
 			Notifications::publish(UI::NoteLoadingMessage(_("Preparing game…")));
 
 			game_->set_game_controller(
 			   std::make_shared<SinglePlayerGameController>(*game_, true, playernumber));
 			game_->init_newgame(sp->settings());
-			game_->run(Widelands::Game::StartGameType::kMap, "", false, "single_player");
+
+			auto custom_names = Widelands::read_custom_warehouse_ship_names();
+			Widelands::Player* player = game_->get_safe_player(playernumber);
+			player->set_shipnames(custom_names.first);
+			player->set_warehousenames(custom_names.second);
+
+			game_->run(Widelands::Game::StartGameType::kMap, "", "single_player");
 		}
 	} catch (const std::exception& e) {
 		WLApplication::emergency_save(&capsule_.menu(), *game_, e.what(), playernumber);
