@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,12 +20,15 @@
 
 #include <cstdlib>
 #include <memory>
+#include <optional>
 
 #include <SDL_timer.h>
 
 #include "base/i18n.h"
 #include "base/log.h"
 #include "base/random.h"
+#include "base/time_string.h"
+#include "base/warning.h"
 #include "build_info.h"
 #include "editor/editorinteractive.h"
 #include "graphic/graphic.h"
@@ -34,7 +37,6 @@
 #include "logic/filesystem_constants.h"
 #include "logic/game.h"
 #include "logic/single_player_game_settings_provider.h"
-#include "map_io/widelands_map_loader.h"
 #include "network/internet_gaming.h"
 #include "network/internet_gaming_protocol.h"
 #include "sound/sound_handler.h"
@@ -52,6 +54,7 @@
 #include "ui_fsmenu/scenario_select.h"
 #include "wlapplication.h"
 #include "wlapplication_options.h"
+#include "wui/maptable.h"
 #include "wui/savegameloader.h"
 
 namespace FsMenu {
@@ -86,7 +89,7 @@ int16_t MainMenu::calc_desired_window_height(const UI::Window::WindowLayoutID id
 	case UI::Window::WindowLayoutID::kFsMenuOptions:
 		return std::max(450, get_h() / 2);
 	case UI::Window::WindowLayoutID::kFsMenuKeyboardOptions:
-		return std::max(360, get_h() * 3 / 8);
+		return std::max(400, get_h() / 2);
 	default:
 		NEVER_HERE();
 	}
@@ -101,13 +104,19 @@ int16_t MainMenu::calc_desired_window_y(const UI::Window::WindowLayoutID id) {
 }
 
 MainMenu::MainMenu(const bool skip_init)
-   : UI::Panel(nullptr, UI::PanelStyle::kFsMenu, 0, 0, g_gr->get_xres(), g_gr->get_yres()),
+   : UI::Panel(nullptr,
+               UI::PanelStyle::kFsMenu,
+               "widelands_main_menu",
+               0,
+               0,
+               g_gr->get_xres(),
+               g_gr->get_yres()),
      box_rect_(0, 0, 0, 0),
      butw_(get_w() * 7 / 20),
      buth_(get_h() * 9 / 200),
      padding_(buth_ / 3),
-     vbox1_(this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, padding_),
-     vbox2_(this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, padding_),
+     vbox1_(this, UI::PanelStyle::kFsMenu, "vbox_1", 0, 0, UI::Box::Vertical, 0, 0, padding_),
+     vbox2_(this, UI::PanelStyle::kFsMenu, "vbox_2", 0, 0, UI::Box::Vertical, 0, 0, padding_),
      singleplayer_(&vbox1_,
                    "singleplayer",
                    0,
@@ -132,7 +141,18 @@ MainMenu::MainMenu(const bool skip_init)
                   UI::PanelStyle::kFsMenu,
                   UI::ButtonStyle::kFsMenuMenu,
                   [this](MenuTarget t) { action(t); }),
-     replay_(&vbox1_, "replay", 0, 0, butw_, buth_, UI::ButtonStyle::kFsMenuMenu, ""),
+     replay_(&vbox1_,
+             "replay",
+             0,
+             0,
+             butw_,
+             6,
+             buth_,
+             "",
+             UI::DropdownType::kTextualMenu,
+             UI::PanelStyle::kFsMenu,
+             UI::ButtonStyle::kFsMenuMenu,
+             [this](MenuTarget t) { action(t); }),
      editor_(&vbox1_,
              "editor",
              0,
@@ -149,8 +169,19 @@ MainMenu::MainMenu(const bool skip_init)
      options_(&vbox2_, "options", 0, 0, butw_, buth_, UI::ButtonStyle::kFsMenuMenu, ""),
      about_(&vbox2_, "about", 0, 0, butw_, buth_, UI::ButtonStyle::kFsMenuMenu, ""),
      exit_(&vbox2_, "exit", 0, 0, butw_, buth_, UI::ButtonStyle::kFsMenuMenu, ""),
+     clock_(this,
+            UI::PanelStyle::kFsMenu,
+            "clock",
+            UI::FontStyle::kFsMenuLabel,
+            0,
+            0,
+            0,
+            0,
+            "",
+            UI::Align::kLeft),
      version_(this,
               UI::PanelStyle::kFsMenu,
+              "version",
               UI::FontStyle::kFsMenuInfoPanelParagraph,
               0,
               0,
@@ -160,6 +191,7 @@ MainMenu::MainMenu(const bool skip_init)
               UI::Align::kCenter),
      copyright_(this,
                 UI::PanelStyle::kFsMenu,
+                "copyright",
                 UI::FontStyle::kFsMenuInfoPanelParagraph,
                 0,
                 0,
@@ -168,12 +200,8 @@ MainMenu::MainMenu(const bool skip_init)
                 "",
                 UI::Align::kCenter),
      init_time_(kNoSplash),
-     last_image_exchange_time_(0),
-     draw_image_(0),
-     last_image_(0),
-     visible_(true),
-     menu_capsule_(*this),
-     auto_log_(false) {
+
+     menu_capsule_(*this) {
 	graphic_resolution_changed_subscriber_ = Notifications::subscribe<GraphicResolutionChanged>(
 	   [this](const GraphicResolutionChanged& message) {
 		   set_size(message.new_width, message.new_height);
@@ -186,7 +214,7 @@ MainMenu::MainMenu(const bool skip_init)
 	singleplayer_.selected.connect([this]() { action(singleplayer_.get_selected()); });
 	multiplayer_.selected.connect([this]() { action(multiplayer_.get_selected()); });
 	editor_.selected.connect([this]() { action(editor_.get_selected()); });
-	replay_.sigclicked.connect([this]() { action(MenuTarget::kReplay); });
+	replay_.selected.connect([this]() { action(replay_.get_selected()); });
 	addons_.sigclicked.connect([this]() { action(MenuTarget::kAddOns); });
 	options_.sigclicked.connect([this]() { action(MenuTarget::kOptions); });
 	about_.sigclicked.connect([this]() { action(MenuTarget::kAbout); });
@@ -224,6 +252,29 @@ MainMenu::MainMenu(const bool skip_init)
 	initialization_complete();
 }
 
+void MainMenu::main_loop() {
+	for (;;) {
+		try {
+			run<int>();
+			return;  // We only get here though normal termination by the user.
+		} catch (const WLWarning& e) {
+			// WLWarning is reserved for bad circumstances that are (most likely) not a bug.
+			show_messagebox(e.title(), e.what());
+		} catch (const std::exception& e) {
+			// This is the outermost wrapper within the GUI and should only very rarely be reached.
+			// Most likely we got here through a bug in Widelands.
+			show_messagebox(
+			   _("Error!"),
+			   format(
+			      _("An error has occured. The error message is:\n\n%1$s\n\nPlease report "
+			        "this problem to help us improve Widelands. You will find related messages in the "
+			        "standard output (stdout.txt on Windows). You are using version %2$s.\n"
+			        "Please add this information to your report."),
+			      e.what(), build_ver_details()));
+		}
+	}
+}
+
 Widelands::Game* MainMenu::create_safe_game(const bool show_error) {
 	try {
 		return new Widelands::Game;
@@ -243,15 +294,29 @@ Widelands::Game* MainMenu::create_safe_game(const bool show_error) {
 void MainMenu::update_template() {
 	UI::Panel::update_template();
 
-	splashscreen_ = &load_safe_template_image("loadscreens/splash.jpg");
-	title_image_ = &load_safe_template_image("loadscreens/logo.png");
+	int dropdowns_lineheight =
+	   g_style_manager->styled_size(UI::StyledSize::kFsMainMenuDropdownHeight);
+	singleplayer_.set_min_lineheight(dropdowns_lineheight);
+	multiplayer_.set_min_lineheight(dropdowns_lineheight);
+	editor_.set_min_lineheight(dropdowns_lineheight);
+	replay_.set_min_lineheight(dropdowns_lineheight);
+
+	splashscreen_ = g_image_cache->get("loadscreens/splash.jpg");
+	title_image_ = g_image_cache->get("loadscreens/logo.png");
 
 	images_.clear();
 	for (const std::string& img : g_fs->list_directory(template_dir() + "loadscreens/mainmenu")) {
 		images_.push_back(img);
 	}
+	if (images_.empty() && !is_using_default_theme()) {
+		log_warn("No main menu backgrounds found, using fallback images");
+		for (const std::string& img :
+		     g_fs->list_directory(kDefaultTemplate + "loadscreens/mainmenu")) {
+			images_.push_back(img);
+		}
+	}
 	if (images_.empty()) {
-		log_warn("No main menu backgrounds found, using fallback image");
+		log_warn("No main menu backgrounds found in default theme, using fallback image");
 		images_.emplace_back("images/logos/wl-ico-128.png");
 	}
 
@@ -278,44 +343,20 @@ void MainMenu::become_modal_again(UI::Panel& prevmodal) {
 	}
 }
 
-void MainMenu::find_maps(const std::string& directory, std::vector<MapEntry>& results) {
-	Widelands::Map map;
-	for (const std::string& file : g_fs->list_directory(directory)) {
-		std::unique_ptr<Widelands::MapLoader> ml = map.get_correct_loader(file);
-		if (ml) {
-			try {
-				map.set_filename(file);
-				ml->preload_map(true, nullptr);
-				if (map.version().map_version_timestamp > 0) {
-					MapData::MapType type = map.scenario_types() == Map::SP_SCENARIO ?
-                                          MapData::MapType::kScenario :
-                                          MapData::MapType::kNormal;
-					results.emplace_back(
-					   MapData(map, file, type, MapData::DisplayType::kFilenames), map.version());
-				}
-			} catch (...) {
-				// invalid file – silently ignore
-			}
-		} else if (g_fs->is_directory(file)) {
-			find_maps(file, results);
-		}
-	}
-}
-
 void MainMenu::set_labels() {
 	{
 		// TODO(Nordfriese): Code duplication, the same code is used in InteractiveBase
 		Section& global_s = get_config_section();
 		set_border_snap_distance(global_s.get_int("border_snap_distance", 0));
 		set_panel_snap_distance(global_s.get_int("panel_snap_distance", 10));
-		set_snap_windows_only_when_overlapping(
-		   global_s.get_bool("snap_windows_only_when_overlapping", false));
 		set_dock_windows_to_edges(global_s.get_bool("dock_windows_to_edges", false));
+		system_clock_ = get_config_bool("game_clock", true);
 	}
 
 	singleplayer_.clear();
 	multiplayer_.clear();
 	editor_.clear();
+	replay_.clear();
 
 	singleplayer_.add(_("New Game"), MenuTarget::kNewGame, nullptr, false, _("Begin a new game"),
 	                  shortcut_string_for(KeyboardShortcut::kMainMenuNew, false));
@@ -335,49 +376,33 @@ void MainMenu::set_labels() {
 	// every language switch because it contains localized strings.
 	{
 		filename_for_continue_playing_ = "";
-		std::unique_ptr<Widelands::Game> game(create_safe_game(false));
-		if (game != nullptr) {
-			SinglePlayerLoader loader(*game);
-			std::vector<SavegameData> games = loader.load_files(kSaveDir);
-			SavegameData* newest_singleplayer = nullptr;
-			for (SavegameData& data : games) {
-				if (!data.is_directory() && data.is_singleplayer() &&
-				    (newest_singleplayer == nullptr || newest_singleplayer->compare_save_time(data))) {
-					newest_singleplayer = &data;
-				}
-			}
-			if (newest_singleplayer != nullptr) {
-				filename_for_continue_playing_ = newest_singleplayer->filename;
-				singleplayer_.add(
-				   _("Continue Playing"), MenuTarget::kContinueLastsave, nullptr, false,
-				   format("%s<br>%s<br>%s<br>%s<br>%s<br>%s",
-				          g_style_manager->font_style(UI::FontStyle::kFsTooltipHeader)
-				             .as_font_tag(
-				                /* strip leading "save/" and trailing ".wgf" */
-				                filename_for_continue_playing_.substr(
-				                   kSaveDir.length() + 1, filename_for_continue_playing_.length() -
-				                                             kSaveDir.length() -
-				                                             kSavegameExtension.length() - 1)),
-				          format(_("Map: %s"),
-				                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-				                    .as_font_tag(newest_singleplayer->mapname)),
-				          format(_("Win Condition: %s"),
-				                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-				                    .as_font_tag(newest_singleplayer->wincondition)),
-				          format(_("Players: %s"),
-				                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-				                    .as_font_tag(newest_singleplayer->nrplayers)),
-				          format(_("Gametime: %s"),
-				                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-				                    .as_font_tag(newest_singleplayer->gametime)),
-				          /** TRANSLATORS: Information about when a game was saved, e.g. 'Saved: Today,
-				           * 10:30'
-				           */
-				          format(_("Saved: %s"),
-				                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-				                    .as_font_tag(newest_singleplayer->savedatestring))),
-				   shortcut_string_for(KeyboardShortcut::kMainMenuContinuePlaying, false));
-			}
+		std::optional<SavegameData> newest_singleplayer = newest_saved_game_or_replay();
+		if (newest_singleplayer.has_value()) {
+			filename_for_continue_playing_ = newest_singleplayer->filename;
+			singleplayer_.add(
+			   _("Continue Playing"), MenuTarget::kContinueLastsave, nullptr, false,
+			   format(
+			      "%s<br>%s<br>%s<br>%s<br>%s<br>%s",
+			      as_font_tag(UI::FontStyle::kFsTooltipHeader,
+			                  /* strip leading "save/" and trailing ".wgf" */
+			                  filename_for_continue_playing_.substr(
+			                     kSaveDir.length() + 1, filename_for_continue_playing_.length() -
+			                                               kSaveDir.length() -
+			                                               kSavegameExtension.length() - 1)),
+			      format(_("Map: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                       newest_singleplayer->mapname)),
+			      format(_("Win Condition: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                                 newest_singleplayer->wincondition)),
+			      format(_("Players: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                           newest_singleplayer->nrplayers)),
+			      format(_("Gametime: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                            newest_singleplayer->gametime)),
+			      /** TRANSLATORS: Information about when a game was saved, e.g. 'Saved: Today,
+			       * 10:30'
+			       */
+			      format(_("Saved: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                         newest_singleplayer->savedatestring))),
+			   shortcut_string_for(KeyboardShortcut::kMainMenuContinuePlaying, false));
 		}
 	}
 
@@ -401,45 +426,71 @@ void MainMenu::set_labels() {
 
 	{
 		filename_for_continue_editing_ = "";
-		std::vector<MapEntry> v;
-		find_maps("maps/My_Maps", v);
-		MapEntry* last_edited = nullptr;
-		for (MapEntry& m : v) {
-			if (last_edited == nullptr ||
-			    m.second.map_version_timestamp > last_edited->second.map_version_timestamp) {
-				last_edited = &m;
-			}
-		}
-		if (last_edited != nullptr) {
-			filename_for_continue_editing_ = last_edited->first.filename;
+		std::optional<MapData> last_edited = newest_edited_map();
+		if (last_edited.has_value()) {
+			filename_for_continue_editing_ = last_edited->filenames.at(0);
 			editor_.add(
 			   _("Continue Editing"), MenuTarget::kEditorContinue, nullptr, false,
-			   format("%s<br>%s<br>%s<br>%s<br>%s",
-			          g_style_manager->font_style(UI::FontStyle::kFsTooltipHeader)
-			             .as_font_tag(
-			                /* strip leading "maps/My_Maps/" and trailing ".wgf" */
-			                filename_for_continue_editing_.substr(
-			                   13, filename_for_continue_editing_.length() - 17)),
-			          format(_("Name: %s"),
-			                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			                    .as_font_tag(last_edited->first.localized_name)),
-			          format(_("Size: %s"),
-			                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			                    .as_font_tag(format(_("%1$u×%2$u"), last_edited->first.width,
-			                                        last_edited->first.height))),
-			          format(_("Players: %s"),
-			                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			                    .as_font_tag(std::to_string(last_edited->first.nrplayers))),
-			          format(_("Description: %s"),
-			                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			                    .as_font_tag(last_edited->first.description))),
+			   format(
+			      "%s<br>%s<br>%s<br>%s<br>%s",
+			      as_font_tag(UI::FontStyle::kFsTooltipHeader,
+			                  /* strip leading "maps/My_Maps/" and trailing ".wmf" */
+			                  filename_for_continue_editing_.substr(
+			                     kMyMapsDirFull.length() + 1, filename_for_continue_editing_.length() -
+			                                                     kMyMapsDirFull.length() -
+			                                                     kWidelandsMapExtension.length() - 1)),
+			      format(_("Name: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                        last_edited->localized_name)),
+			      format(_("Size: %s"),
+			             as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                         format(_("%1$u×%2$u"), last_edited->width, last_edited->height))),
+			      format(_("Players: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                           std::to_string(last_edited->nrplayers))),
+			      format(_("Description: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                               last_edited->description))),
 			   shortcut_string_for(KeyboardShortcut::kMainMenuContinueEditing, false));
 		}
 	}
 
+	replay_.add(_("Load Replay"), MenuTarget::kReplay, nullptr, false,
+	            _("Watch the replay of an old game"),
+	            shortcut_string_for(KeyboardShortcut::kMainMenuLoadReplay, false));
+	{
+		filename_for_last_replay_ = "";
+		std::optional<SavegameData> newest_replay = newest_saved_game_or_replay(true);
+		if (newest_replay.has_value()) {
+			filename_for_last_replay_ = newest_replay->filename;
+			replay_.add(
+			   _("Watch last saved replay"), MenuTarget::kReplayLast, nullptr, false,
+			   format(
+			      "%s<br>%s<br>%s<br>%s<br>%s<br>%s",
+			      as_font_tag(UI::FontStyle::kFsTooltipHeader,
+			                  /* strip leading "replays/" and trailing ".wry" */
+			                  filename_for_last_replay_.substr(
+			                     kReplayDir.length() + 1, filename_for_last_replay_.length() -
+			                                                 kReplayDir.length() -
+			                                                 kReplayExtension.length() - 1)),
+			      format(_("Map: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                       newest_replay->mapname)),
+			      format(_("Win Condition: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                                 newest_replay->wincondition)),
+			      format(_("Players: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                           newest_replay->nrplayers)),
+			      format(_("Gametime: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                            newest_replay->gametime)),
+			      /** TRANSLATORS: Information about when a game was saved, e.g. 'Saved: Today,
+			       * 10:30'
+			       */
+			      format(_("Saved: %s"), as_font_tag(UI::FontStyle::kFsMenuInfoPanelParagraph,
+			                                         newest_replay->savedatestring))),
+			   shortcut_string_for(KeyboardShortcut::kMainMenuReplayLast, false));
+		}
+	}
+
 	singleplayer_.set_label(_("Single Player…"));
 	multiplayer_.set_label(_("Multiplayer…"));
 	editor_.set_label(_("Editor…"));
+	replay_.set_label(_("Watch Replay…"));
 	singleplayer_.set_tooltip(as_tooltip_text_with_hotkey(
 	   _("Begin or load a single-player campaign or free game"),
 	   shortcut_string_for(KeyboardShortcut::kMainMenuSP, true), UI::PanelStyle::kFsMenu));
@@ -449,10 +500,8 @@ void MainMenu::set_labels() {
 	editor_.set_tooltip(as_tooltip_text_with_hotkey(
 	   _("Launch the map editor"), shortcut_string_for(KeyboardShortcut::kMainMenuE, true),
 	   UI::PanelStyle::kFsMenu));
-
-	replay_.set_title(_("Watch Replay"));
 	replay_.set_tooltip(as_tooltip_text_with_hotkey(
-	   _("Watch the replay of an old game"),
+	   _("Watch again a recorded earlier game"),
 	   shortcut_string_for(KeyboardShortcut::kMainMenuReplay, true), UI::PanelStyle::kFsMenu));
 
 	addons_.set_title(_("Add-Ons"));
@@ -477,7 +526,7 @@ void MainMenu::set_labels() {
 	   format(_("Version %1$s"), build_ver_details()));
 	copyright_.set_text(
 	   /** TRANSLATORS: Placeholders are the copyright years */
-	   format(_("(C) %1%-%2% by the Widelands Development Team · Licensed under "
+	   format(_("(C) %1%-%2% by the Widelands Development Team • Licensed under "
 	            "the GNU General Public License V2.0"),
 	          kWidelandsCopyrightStart, kWidelandsCopyrightEnd));
 }
@@ -491,6 +540,11 @@ void MainMenu::set_button_visibility(const bool v) {
 	vbox2_.set_visible(v);
 	copyright_.set_visible(v);
 	version_.set_visible(v);
+	clock_.set_visible(v);
+}
+
+void MainMenu::abort_splashscreen() {
+	init_time_ = kNoSplash;
 }
 
 bool MainMenu::handle_mousepress(uint8_t /*btn*/, int32_t /*x*/, int32_t /*y*/) {
@@ -543,7 +597,10 @@ bool MainMenu::handle_key(const bool down, const SDL_Keysym code) {
 		if (check_match_shortcut(KeyboardShortcut::kMainMenuLoad, MenuTarget::kLoadGame)) {
 			return true;
 		}
-		if (check_match_shortcut(KeyboardShortcut::kMainMenuReplay, MenuTarget::kReplay)) {
+		if (check_match_shortcut(KeyboardShortcut::kMainMenuLoadReplay, MenuTarget::kReplay)) {
+			return true;
+		}
+		if (check_match_shortcut(KeyboardShortcut::kMainMenuReplayLast, MenuTarget::kReplayLast)) {
 			return true;
 		}
 		if (check_match_shortcut(KeyboardShortcut::kMainMenuTutorial, MenuTarget::kTutorial)) {
@@ -592,9 +649,13 @@ bool MainMenu::handle_key(const bool down, const SDL_Keysym code) {
 			editor_.toggle();
 			return true;
 		}
+		if (matches_shortcut(KeyboardShortcut::kMainMenuReplay, code)) {
+			replay_.toggle();
+			return true;
+		}
 		if (matches_shortcut(KeyboardShortcut::kMainMenuQuit, code)) {
 			if (!fell_through) {
-				end_modal<MenuTarget>(MenuTarget::kBack);
+				exit();
 				return true;
 			}
 		}
@@ -710,6 +771,14 @@ void MainMenu::draw(RenderTarget& r) {
 		r.fill_rect(Recti((get_w() - max_w - padding_) / 2, version_.get_y() - padding_ / 2,
 		                  max_w + padding_, get_h() - version_.get_y() + padding_ / 2),
 		            bg, BlendMode::Default);
+
+		if (system_clock_) {
+			clock_.set_text(realtimestring());
+			r.fill_rect(Recti(0, 0, clock_.get_w() + 2 * padding_, clock_.get_h() + 2 * padding_), bg,
+			            BlendMode::Default);
+		} else {
+			clock_.set_text(std::string());
+		}
 	}
 
 	// Widelands logo
@@ -741,7 +810,7 @@ void MainMenu::draw_overlay(RenderTarget& r) {
 }
 
 inline Rectf MainMenu::title_pos() {
-	const float imgh = box_rect_.y / 3.f;
+	const float imgh = box_rect_.y / 2.5f;
 	const float imgw = imgh * title_image_->width() / title_image_->height();
 	return Rectf((get_w() - imgw) / 2.f, buth_, imgw, imgh);
 }
@@ -755,9 +824,10 @@ void MainMenu::layout() {
 	   (get_inner_w() - copyright_.get_w()) / 2, get_inner_h() - copyright_.get_h() - padding_ / 2));
 	version_.set_pos(Vector2i((get_inner_w() - version_.get_w()) / 2,
 	                          copyright_.get_y() - version_.get_h() - padding_ / 2));
+	clock_.set_pos(Vector2i(padding_, padding_));
 
 	box_rect_ = Recti((get_inner_w() - padding_) / 2 - butw_,
-	                  version_.get_y() - padding_ * 5 / 2 - get_inner_h() / 4, 2 * butw_ + padding_,
+	                  version_.get_y() - padding_ * 4 - get_inner_h() * 3 / 10, 2 * butw_ + padding_,
 	                  get_inner_h() / 4);
 
 	singleplayer_.set_desired_size(butw_, buth_);
@@ -826,7 +896,7 @@ void MainMenu::action(const MenuTarget t) {
 	switch (t) {
 
 	case MenuTarget::kExit:
-		end_modal<MenuTarget>(MenuTarget::kBack);
+		exit((SDL_GetModState() & KMOD_CTRL) != 0);
 		break;
 
 	case MenuTarget::kOptions: {
@@ -844,11 +914,19 @@ void MainMenu::action(const MenuTarget t) {
 		break;
 
 	case MenuTarget::kReplay:
-		if (Widelands::Game* g = create_safe_game()) {
+		if (Widelands::Game* g = create_safe_game(); g != nullptr) {
 			menu_capsule_.clear_content();
 			new LoadGame(menu_capsule_, *g, *new SinglePlayerGameSettingsProvider(), true, true);
 		}
 		break;
+	case MenuTarget::kReplayLast:
+		if (!filename_for_last_replay_.empty()) {
+			if (Widelands::Game* g = create_safe_game(); g != nullptr) {
+				menu_capsule_.clear_content();
+				g->run_replay(filename_for_last_replay_, "");
+			}
+		}
+		break;
 	case MenuTarget::kLoadGame:
 		if (Widelands::Game* g = create_safe_game()) {
 			menu_capsule_.clear_content();
@@ -955,6 +1033,18 @@ void MainMenu::action(const MenuTarget t) {
 	}
 }
 
+void MainMenu::exit(const bool force) {
+	if (!force) {
+		UI::WLMessageBox confirmbox(this, UI::WindowStyle::kFsMenu, _("Exit Confirmation"),
+		                            _("Are you sure you wish to exit Widelands?"),
+		                            UI::WLMessageBox::MBoxType::kOkCancel);
+		if (confirmbox.run<UI::Panel::Returncodes>() == UI::Panel::Returncodes::kBack) {
+			return;
+		}
+	}
+	end_modal<MenuTarget>(MenuTarget::kBack);
+}
+
 /// called if the user is not registered
 void MainMenu::show_internet_login(const bool modal) {
 	r_login_.create();
