@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2022 by the Widelands Development Team
+ * Copyright (C) 2021-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -82,9 +82,11 @@ void MousewheelConfigSettings::read() {
 	map_scroll_mod_ = READ_MOD(kMapScroll);
 	speed_mod_ = READ_MOD(kGameSpeed);
 	toolsize_mod_ = READ_MOD(kEditorToolsize);
+	toolgap_mod_ = READ_MOD(kEditorToolgap);
 	zoom_dir_ = READ_DIR(kMapZoom);
 	speed_dir_ = READ_DIR(kGameSpeed);
 	toolsize_dir_ = READ_DIR(kEditorToolsize);
+	toolgap_dir_ = READ_DIR(kEditorToolgap);
 	value_invert_ = READ_DIR(kUIChangeValueInvert);
 	tab_invert_ = READ_DIR(kUITabInvert);
 	zoom_invert_ = READ_DIR(kMapZoomInvert);
@@ -111,9 +113,11 @@ void MousewheelConfigSettings::apply() const {
 	set_mousewheel_keymod(MousewheelOptionID::kMapScrollMod, map_scroll_mod_);
 	set_mousewheel_keymod(MousewheelOptionID::kGameSpeedMod, speed_mod_);
 	set_mousewheel_keymod(MousewheelOptionID::kEditorToolsizeMod, toolsize_mod_);
+	set_mousewheel_keymod(MousewheelOptionID::kEditorToolgapMod, toolgap_mod_);
 	APPLY_DIR(kMapZoom, zoom_dir_)
 	APPLY_DIR(kGameSpeed, speed_dir_)
 	APPLY_DIR(kEditorToolsize, toolsize_dir_)
+	APPLY_DIR(kEditorToolgap, toolgap_dir_)
 	APPLY_DIR(kUIChangeValueInvert, value_invert_)
 	APPLY_DIR(kUITabInvert, tab_invert_)
 	APPLY_DIR(kMapZoomInvert, zoom_invert_)
@@ -128,7 +132,7 @@ void MousewheelConfigSettings::apply() const {
 
 KeymodDropdown::KeymodDropdown(UI::Panel* parent)
    : UI::Dropdown<uint16_t>(parent,
-                            std::string(),
+                            "keymod_dropdown",
                             0,
                             0,
                             kModDirDropdownMaxWidth,
@@ -165,7 +169,7 @@ KeymodDropdown::KeymodDropdown(UI::Panel* parent)
 
 DirDropdown::DirDropdown(UI::Panel* parent, bool two_d)
    : UI::Dropdown<uint8_t>(parent,
-                           std::string(),
+                           "dir_dropdown",
                            0,
                            0,
                            kModDirDropdownMaxWidth,
@@ -187,7 +191,7 @@ DirDropdown::DirDropdown(UI::Panel* parent, bool two_d)
 
 InvertDirDropdown::InvertDirDropdown(UI::Panel* parent)
    : UI::Dropdown<uint8_t>(parent,
-                           std::string(),
+                           "invert_dir_dropdown",
                            0,
                            0,
                            kModDirDropdownMaxWidth,
@@ -215,9 +219,20 @@ KeymodAndDirBox::KeymodAndDirBox(UI::Panel* parent,
                                  uint16_t* keymod,
                                  uint8_t* dir,
                                  bool two_d)
-   : UI::Box(parent, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal, 0, kButtonSize, kPadding),
-     title_area_(
-        this, UI::PanelStyle::kFsMenu, UI::FontStyle::kFsMenuLabel, format(_("%1%:"), title)),
+   : UI::Box(parent,
+             UI::PanelStyle::kFsMenu,
+             "key_mod_and_dir_box",
+             0,
+             0,
+             UI::Box::Horizontal,
+             0,
+             kButtonSize,
+             kPadding),
+     title_area_(this,
+                 UI::PanelStyle::kFsMenu,
+                 "title",
+                 UI::FontStyle::kFsMenuLabel,
+                 format(_("%1%:"), title)),
      keymod_dropdown_(this),
      dir_dropdown_(this, two_d),
      title_(title),
@@ -302,8 +317,16 @@ void KeymodAndDirBox::set_width(int w) {
 /***** End of KeymodAndDirBox members *****/
 
 InvertDirBox::InvertDirBox(UI::Panel* parent, const std::string& title, uint8_t* dir)
-   : UI::Box(parent, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal, 0, kButtonSize, kPadding),
-     title_area_(this, UI::PanelStyle::kFsMenu, UI::FontStyle::kFsMenuLabel, title),
+   : UI::Box(parent,
+             UI::PanelStyle::kFsMenu,
+             "invert_dir_box",
+             0,
+             0,
+             UI::Box::Horizontal,
+             0,
+             kButtonSize,
+             kPadding),
+     title_area_(this, UI::PanelStyle::kFsMenu, "title", UI::FontStyle::kFsMenuLabel, title),
      dir_dropdown_(this),
      dir_(dir) {
 	if (UI::g_fh->fontset()->is_rtl()) {
@@ -329,9 +352,17 @@ void InvertDirBox::set_width(int w) {
 }
 
 ScrollOptionsButtonBox::ScrollOptionsButtonBox(MousewheelOptionsDialog* parent)
-   : UI::Box(parent, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal, 0, kButtonSize, kPadding),
+   : UI::Box(parent,
+             UI::PanelStyle::kFsMenu,
+             "scroll_options_box",
+             0,
+             0,
+             UI::Box::Horizontal,
+             0,
+             kButtonSize,
+             kPadding),
      reset_button_(this,
-                   std::string(),
+                   "reset",
                    0,
                    0,
                    0,
@@ -341,7 +372,7 @@ ScrollOptionsButtonBox::ScrollOptionsButtonBox(MousewheelOptionsDialog* parent)
                    _("Reset scroll settings to the defaults recommended for a standard single "
                      "wheel mouse")),
      touchpad_button_(this,
-                      std::string(),
+                      "touchpad",
                       0,
                       0,
                       0,
@@ -352,7 +383,7 @@ ScrollOptionsButtonBox::ScrollOptionsButtonBox(MousewheelOptionsDialog* parent)
                         "touchpad or other pointing device that can scroll horizontally as well "
                         "as vertically.")),
      apply_button_(this,
-                   std::string(),
+                   "apply",
                    0,
                    0,
                    0,
@@ -377,13 +408,21 @@ ScrollOptionsButtonBox::ScrollOptionsButtonBox(MousewheelOptionsDialog* parent)
 
 // The main scrolling options dialog box
 MousewheelOptionsDialog::MousewheelOptionsDialog(UI::Panel* parent)
-   : UI::Box(parent, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, kPadding),
+   : UI::Box(parent,
+             UI::PanelStyle::kFsMenu,
+             "mousewheel_options_box",
+             0,
+             0,
+             UI::Box::Vertical,
+             0,
+             0,
+             kPadding),
      zoom_box_(this,
                /** TRANSLATORS: Name of a function for the scroll wheel.
                    Used as e.g.: "Zoom Map: Ctrl+Any scroll"
                    The ':' will be added by another format string. */
                _("Zoom Map"),
-               {&mapscroll_box_, &speed_box_, &toolsize_box_},
+               {&mapscroll_box_, &speed_box_, &toolsize_box_, &toolgap_box_},
                &(settings_.zoom_mod_),
                &(settings_.zoom_dir_)),
      mapscroll_box_(this,
@@ -391,7 +430,7 @@ MousewheelOptionsDialog::MousewheelOptionsDialog(UI::Panel* parent)
                         Used as e.g.: "Scroll Map: Ctrl+Any scroll"
                         The ':' will be added by another format string. */
                     _("Scroll Map"),
-                    {&zoom_box_, &speed_box_, &toolsize_box_},
+                    {&zoom_box_, &speed_box_, &toolsize_box_, &toolgap_box_},
                     &(settings_.map_scroll_mod_),
                     &(settings_.enable_map_scroll_),
                     true),
@@ -408,9 +447,17 @@ MousewheelOptionsDialog::MousewheelOptionsDialog(UI::Panel* parent)
                        Used as e.g.: "Change Editor Toolsize: Ctrl+Any scroll"
                        The ':' will be added by another format string. */
                    _("Change Editor Toolsize"),
-                   {&zoom_box_, &mapscroll_box_},
+                   {&zoom_box_, &mapscroll_box_, &toolgap_box_},
                    &(settings_.toolsize_mod_),
                    &(settings_.toolsize_dir_)),
+     toolgap_box_(this,
+                  /** TRANSLATORS: Name of a function for the scroll wheel.
+                      Used as e.g.: "Change Editor Tool Gap: Ctrl+Any scroll"
+                      The ':' will be added by another format string. */
+                  _("Change Editor Tool Gap"),
+                  {&zoom_box_, &mapscroll_box_, &toolsize_box_},
+                  &(settings_.toolgap_mod_),
+                  &(settings_.toolgap_dir_)),
      zoom_invert_box_(
         this,
         /** TRANSLATORS: Used as e.g. "Invert scroll direction for map zooming: Vertical" */
@@ -426,11 +473,19 @@ MousewheelOptionsDialog::MousewheelOptionsDialog(UI::Panel* parent)
         /** TRANSLATORS: Used as e.g. "Invert scroll direction for increase/decrease: Vertical" */
         _("Invert scroll direction for increase/decrease:"),
         &(settings_.value_invert_)),
-     horiz_override_box_(
-        this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal, 0, kButtonSize, kPadding),
+     horiz_override_box_(this,
+                         UI::PanelStyle::kFsMenu,
+                         "fix_inverted_scrolling_box",
+                         0,
+                         0,
+                         UI::Box::Horizontal,
+                         0,
+                         kButtonSize,
+                         kPadding),
      inverted_x_checkbox_(
         &horiz_override_box_,
         UI::PanelStyle::kFsMenu,
+        "fix_inverted_scrolling",
         Vector2i::zero(),
         _("Fix inverted horizontal scrolling"),
         _("An SDL bug in some configurations causes horizontal scroll to be inverted. "
@@ -438,7 +493,7 @@ MousewheelOptionsDialog::MousewheelOptionsDialog(UI::Panel* parent)
           "Please report if you need to turn this on, so that we can improve the detection."),
         0),
      feedback_button_(&horiz_override_box_,
-                      std::string(),
+                      "feedback",
                       0,
                       0,
                       0,
@@ -465,6 +520,7 @@ MousewheelOptionsDialog::MousewheelOptionsDialog(UI::Panel* parent)
 	add(&mapscroll_box_);
 	add(&speed_box_);
 	add(&toolsize_box_);
+	add(&toolgap_box_);
 	add_space(kDividerSpace);
 	add(&zoom_invert_box_);
 	add(&tab_invert_box_);
@@ -482,6 +538,7 @@ void MousewheelOptionsDialog::update_settings() {
 	mapscroll_box_.update_sel();
 	speed_box_.update_sel();
 	toolsize_box_.update_sel();
+	toolgap_box_.update_sel();
 	zoom_invert_box_.update_sel();
 	tab_invert_box_.update_sel();
 	value_invert_box_.update_sel();
@@ -501,13 +558,16 @@ void MousewheelOptionsDialog::set_touchpad() {
 	   speed_box_.conflicts(KMOD_NONE, SD::kAny) || speed_box_.conflicts(KMOD_CTRL, SD::kVertical);
 	const bool conflict_toolsize = toolsize_box_.conflicts(KMOD_NONE, SD::kAny) ||
 	                               toolsize_box_.conflicts(KMOD_CTRL, SD::kVertical);
-	if (conflict_speed || conflict_toolsize) {
+	const bool conflict_toolgap = toolgap_box_.conflicts(KMOD_NONE, SD::kAny) ||
+	                              toolgap_box_.conflicts(KMOD_CTRL, SD::kVertical);
+	if (conflict_speed || conflict_toolsize || conflict_toolgap) {
 		UI::WLMessageBox warning(
 		   &get_topmost_forefather(), UI::WindowStyle::kFsMenu, _("Scroll Settings Conflict"),
-		   as_richtext_paragraph(format(_("‘%1$s’ or ‘%2$s’ conflicts with the recommended "
-		                                  "settings. Change the conflicting setting(s) too?"),
-		                                speed_box_.get_title(), toolsize_box_.get_title()),
-		                         UI::FontStyle::kFsMenuLabel, UI::Align::kCenter),
+		   as_richtext_paragraph(
+		      format(_("‘%1$s’, ‘%2$s’, or ‘%3$s’ conflicts with the recommended "
+		               "settings. Change the conflicting setting(s) too?"),
+		             speed_box_.get_title(), toolsize_box_.get_title(), toolgap_box_.get_title()),
+		      UI::FontStyle::kFsMenuLabel, UI::Align::kCenter),
 		   UI::WLMessageBox::MBoxType::kOkCancel);
 		if (warning.run<UI::Panel::Returncodes>() != UI::Panel::Returncodes::kOk) {
 			return;
@@ -520,6 +580,10 @@ void MousewheelOptionsDialog::set_touchpad() {
 			settings_.toolsize_mod_ = KMOD_ALT;
 			toolsize_box_.update_sel();
 		}
+		if (conflict_toolgap) {
+			settings_.toolgap_mod_ = KMOD_SHIFT;
+			toolgap_box_.update_sel();
+		}
 	}
 
 	// Set recommended values
@@ -528,7 +592,8 @@ void MousewheelOptionsDialog::set_touchpad() {
 	mapscroll_box_.update_sel();
 	settings_.zoom_mod_ = KMOD_CTRL;
 	settings_.zoom_dir_ = (speed_box_.conflicts(KMOD_CTRL, SD::kHorizontal) ||
-	                       toolsize_box_.conflicts(KMOD_CTRL, SD::kHorizontal)) ?
+	                       toolsize_box_.conflicts(KMOD_CTRL, SD::kHorizontal) ||
+	                       toolgap_box_.conflicts(KMOD_CTRL, SD::kHorizontal)) ?
                             SD::kVertical :
                             SD::kAny;
 	zoom_box_.update_sel();
@@ -550,6 +615,7 @@ void MousewheelOptionsDialog::set_size(int w, int h) {
 		mapscroll_box_.set_width(w_hbox);
 		speed_box_.set_width(w_hbox);
 		toolsize_box_.set_width(w_hbox);
+		toolgap_box_.set_width(w_hbox);
 		zoom_invert_box_.set_width(w_hbox);
 		tab_invert_box_.set_width(w_hbox);
 		value_invert_box_.set_width(w_hbox);
