@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2022 by the Widelands Development Team
+ * Copyright (C) 2010-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -45,7 +45,15 @@ struct MultiPlayerClientGroup : public UI::Box {
 	                       int32_t const h,
 	                       PlayerSlot id,
 	                       GameSettingsProvider* const settings)
-	   : UI::Box(parent, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal, 0, 0, kPadding),
+	   : UI::Box(parent,
+	             UI::PanelStyle::kFsMenu,
+	             format("multi_player_client_group_%u", id),
+	             0,
+	             0,
+	             UI::Box::Horizontal,
+	             0,
+	             0,
+	             kPadding),
 	     slot_dropdown_(this,
 	                    format("dropdown_slot%d", static_cast<unsigned int>(id)),
 	                    0,
@@ -59,10 +67,11 @@ struct MultiPlayerClientGroup : public UI::Box {
 	                    UI::ButtonStyle::kFsMenuSecondary),
 	     // Name needs to be initialized after the dropdown, otherwise the layout function will
 	     // crash.
-	     name(this, UI::PanelStyle::kFsMenu, UI::FontStyle::kFsMenuLabel, 0, 0, 0, 0),
+	     name(
+	        this, UI::PanelStyle::kFsMenu, format("label_name_%u", id), UI::FontStyle::kFsMenuLabel),
 	     settings_(settings),
 	     id_(id),
-	     slot_selection_locked_(false),
+
 	     menu_parent_(grand_parent) {
 		add(&slot_dropdown_);
 		add(&name, UI::Box::Resizing::kAlign, UI::Align::kCenter);
@@ -156,8 +165,8 @@ struct MultiPlayerClientGroup : public UI::Box {
 	UI::Dropdown<uintptr_t> slot_dropdown_;  /// Select the player slot.
 	UI::Textarea name;                       /// Client nick name
 	GameSettingsProvider* const settings_;
-	uint8_t const id_;            /// User number
-	bool slot_selection_locked_;  // Ensure that dropdowns will close on selection.
+	uint8_t const id_;                   /// User number
+	bool slot_selection_locked_{false};  // Ensure that dropdowns will close on selection.
 	UI::Panel* menu_parent_;
 };
 
@@ -169,10 +178,24 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 	                       PlayerSlot id,
 	                       GameSettingsProvider* const settings,
 	                       NetworkPlayerSettingsBackend* const npsb)
-	   : UI::Box(parent, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
+	   : UI::Box(parent,
+	             UI::PanelStyle::kFsMenu,
+	             format("multi_player_setup_group_box_%d", id),
+	             0,
+	             0,
+	             UI::Box::Horizontal),
 	     settings_(settings),
 	     n(npsb),
 	     id_(id),
+	     number_(this,
+	             "player",
+	             0,
+	             0,
+	             h,
+	             h,
+	             UI::ButtonStyle::kFsMenuSecondary,
+	             std::to_string(id_ + 1),
+	             _("Player Number")),
 	     player_(this,
 	             "player",
 	             0,
@@ -226,18 +249,15 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 	                    _("Team"),
 	                    UI::DropdownType::kPictorial,
 	                    UI::PanelStyle::kFsMenu,
-	                    UI::ButtonStyle::kFsMenuSecondary),
-	     last_state_(PlayerSettings::State::kClosed),
-	     type_selection_locked_(false),
-	     tribe_selection_locked_(false),
-	     init_selection_locked_(false),
-	     team_selection_locked_(false) {
+	                    UI::ButtonStyle::kFsMenuSecondary) {
 
 		type_dropdown_.set_disable_style(UI::ButtonDisableStyle::kFlat);
 		tribes_dropdown_.set_disable_style(UI::ButtonDisableStyle::kFlat);
 		init_dropdown_.set_disable_style(UI::ButtonDisableStyle::kFlat);
 		team_dropdown_.set_disable_style(UI::ButtonDisableStyle::kFlat);
 		player_.set_disable_style(UI::ButtonDisableStyle::kFlat);
+		number_.set_disable_style(UI::ButtonDisableStyle::kFlat);
+		number_.set_enabled(false);
 
 		type_dropdown_.selected.connect([this]() { set_type(); });
 		tribes_dropdown_.selected.connect([this]() { set_tribe_or_shared_in(); });
@@ -245,6 +265,7 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 		team_dropdown_.selected.connect([this]() { set_team(); });
 		player_.sigclicked.connect([this]() { set_color(); });
 
+		add(&number_);
 		add(&player_);
 		add(&type_dropdown_);
 		add(&tribes_dropdown_);
@@ -614,6 +635,7 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 	}
 
 	void force_new_dimensions(uint32_t height) {
+		number_.set_desired_size(height, height);
 		player_.set_desired_size(height, height);
 		type_dropdown_.set_desired_size(height, height);
 		tribes_dropdown_.set_desired_size(height, height);
@@ -625,6 +647,7 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 	NetworkPlayerSettingsBackend* const n;
 	PlayerSlot const id_;
 
+	UI::Button number_;
 	UI::Button player_;
 	UI::Dropdown<std::string>
 	   type_dropdown_;  /// Select who owns the slot (human, AI, open, closed, shared-in).
@@ -632,13 +655,14 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 	UI::Dropdown<uintptr_t>
 	   init_dropdown_;  /// Select the initialization (Headquarters, Fortified Village etc.)
 	UI::Dropdown<uintptr_t> team_dropdown_;  /// Select the team number
-	PlayerSettings::State
-	   last_state_;  /// The dropdowns for the other slots need updating if this changes
+	PlayerSettings::State last_state_{
+	   PlayerSettings::State::kClosed};  /// The dropdowns for the other slots need updating if this
+	                                     /// changes
 	/// Lock rebuilding dropdowns so that they can close on selection
-	bool type_selection_locked_;
-	bool tribe_selection_locked_;
-	bool init_selection_locked_;
-	bool team_selection_locked_;
+	bool type_selection_locked_{false};
+	bool tribe_selection_locked_{false};
+	bool init_selection_locked_{false};
+	bool team_selection_locked_{false};
 
 	std::unique_ptr<Notifications::Subscriber<NoteGameSettings>> subscriber_;
 };
@@ -651,15 +675,21 @@ MultiPlayerSetupGroup::MultiPlayerSetupGroup(UI::Panel* const launchgame,
                                              int32_t const /* h */,
                                              GameSettingsProvider* const settings,
                                              uint32_t buth)
-   : UI::Box(parent, UI::PanelStyle::kFsMenu, x, y, UI::Box::Horizontal),
+   : UI::Box(
+        parent, UI::PanelStyle::kFsMenu, "multi_player_setup_group", x, y, UI::Box::Horizontal),
      settings_(settings),
      npsb_(new NetworkPlayerSettingsBackend(settings_)),
      launchgame_(launchgame),
-     clientbox_(this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     playerbox_(this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, kPadding),
-     scrollable_playerbox_(&playerbox_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
+     clientbox_(this, UI::PanelStyle::kFsMenu, "client_box", 0, 0, UI::Box::Vertical),
+     scrollable_clientbox_(
+        &clientbox_, UI::PanelStyle::kFsMenu, "scrollable_clientbox", 0, 0, UI::Box::Vertical),
+     playerbox_(
+        this, UI::PanelStyle::kFsMenu, "player_box", 0, 0, UI::Box::Vertical, 0, 0, kPadding),
+     scrollable_playerbox_(
+        &playerbox_, UI::PanelStyle::kFsMenu, "scrollable_playerbox", 0, 0, UI::Box::Vertical),
      clients_(&clientbox_,
               UI::PanelStyle::kFsMenu,
+              "label_clients",
               UI::FontStyle::kFsGameSetupHeadings,
               0,
               0,
@@ -669,6 +699,7 @@ MultiPlayerSetupGroup::MultiPlayerSetupGroup(UI::Panel* const launchgame,
               UI::Align::kCenter),
      players_(&playerbox_,
               UI::PanelStyle::kFsMenu,
+              "label_players",
               UI::FontStyle::kFsGameSetupHeadings,
               0,
               0,
@@ -677,18 +708,19 @@ MultiPlayerSetupGroup::MultiPlayerSetupGroup(UI::Panel* const launchgame,
               _("Players"),
               UI::Align::kCenter),
      buth_(buth) {
+	scrollable_clientbox_.set_scrolling(true);
 	clientbox_.add(&clients_, Resizing::kAlign, UI::Align::kCenter);
 	clientbox_.add_space(3 * kPadding);
-	clientbox_.set_scrolling(true);
+	clientbox_.add(&scrollable_clientbox_, Resizing::kExpandBoth);
 
-	add(&clientbox_);
+	scrollable_playerbox_.set_scrolling(true);
+	playerbox_.add(&players_, Resizing::kAlign, UI::Align::kCenter);
+	playerbox_.add_space(kPadding);
+	playerbox_.add(&scrollable_playerbox_, Resizing::kExpandBoth);
+
+	add(&clientbox_, Resizing::kFullSize);
 	add_space(8 * kPadding);
 	add(&playerbox_, Resizing::kExpandBoth);
-	playerbox_.add(&players_, Resizing::kAlign, UI::Align::kCenter);
-	scrollable_playerbox_.set_scrolling(true);
-	playerbox_.add_space(kPadding);
-
-	playerbox_.add(&scrollable_playerbox_, Resizing::kExpandBoth);
 
 	subscriber_ = Notifications::subscribe<NoteGameSettings>([this](const NoteGameSettings& n) {
 		if (n.action == NoteGameSettings::Action::kMap) {
@@ -717,9 +749,8 @@ void MultiPlayerSetupGroup::update_players() {
 	const size_t number_of_players = settings.players.size();
 
 	for (PlayerSlot i = multi_player_player_groups_.size(); i < number_of_players; ++i) {
-		multi_player_player_groups_.push_back(new MultiPlayerPlayerGroup(
-		   &scrollable_playerbox_, playerbox_.get_w() - UI::Scrollbar::kSize, buth_, i, settings_,
-		   npsb_.get()));
+		multi_player_player_groups_.push_back(
+		   new MultiPlayerPlayerGroup(&scrollable_playerbox_, 0, buth_, i, settings_, npsb_.get()));
 		scrollable_playerbox_.add(multi_player_player_groups_.at(i), Resizing::kFullSize);
 	}
 	for (auto& p : multi_player_player_groups_) {
@@ -733,8 +764,8 @@ void MultiPlayerSetupGroup::update_clients() {
 	if (number_of_users > multi_player_client_groups_.size()) {
 		for (uint32_t i = multi_player_client_groups_.size(); i < number_of_users; ++i) {
 			multi_player_client_groups_.push_back(new MultiPlayerClientGroup(
-			   launchgame_, &clientbox_, clientbox_.get_w(), buth_, i, settings_));
-			clientbox_.add(multi_player_client_groups_.at(i), Resizing::kFullSize);
+			   launchgame_, &scrollable_clientbox_, 0, buth_, i, settings_));
+			scrollable_clientbox_.add(multi_player_client_groups_.at(i), Resizing::kFullSize);
 		}
 	}
 	for (auto& c : multi_player_client_groups_) {
@@ -767,10 +798,14 @@ void MultiPlayerSetupGroup::force_new_dimensions(uint32_t max_width,
                                                  uint32_t max_height,
                                                  uint32_t standard_element_height) {
 	buth_ = standard_element_height;
-	clientbox_.set_min_desired_breadth(max_width / 3);
-	clientbox_.set_max_size(max_width / 3, max_height);
-	playerbox_.set_max_size(max_width / 2, max_height);
-	scrollable_playerbox_.set_max_size(max_width / 2, max_height - players_.get_h() - 4 * kPadding);
+	uint32_t scroll_height = max_height - players_.get_h() - 4 * kPadding;
+	scrollable_playerbox_.set_max_size(0, scroll_height);
+	// Reset desired size to properly fit into scroll box
+	scrollable_playerbox_.set_desired_size(0, 0);
+
+	clients_.set_desired_size(max_width / 3, clients_.get_h());
+	scrollable_clientbox_.set_max_size(max_width / 3, scroll_height);
+	scrollable_clientbox_.set_desired_size(0, 0);
 
 	for (auto& multiPlayerClientGroup : multi_player_client_groups_) {
 		multiPlayerClientGroup->force_new_dimensions(standard_element_height);
