@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
 
 #include "ui_fsmenu/options.h"
 
+#include <algorithm>
 #include <memory>
 
 #include "base/i18n.h"
@@ -86,23 +87,42 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
                 _("Options")),
 
      // Buttons
-     button_box_(this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
+     button_box_(this, UI::PanelStyle::kFsMenu, "buttons_box", 0, 0, UI::Box::Horizontal),
      cancel_(&button_box_, "cancel", 0, 0, 0, 0, UI::ButtonStyle::kFsMenuSecondary, _("Cancel")),
      apply_(&button_box_, "apply", 0, 0, 0, 0, UI::ButtonStyle::kFsMenuSecondary, _("Apply")),
      ok_(&button_box_, "ok", 0, 0, 0, 0, UI::ButtonStyle::kFsMenuPrimary, _("OK")),
 
      // Tabs
-     tabs_(this, UI::TabPanelStyle::kFsMenu),
+     tabs_(this, UI::TabPanelStyle::kFsMenu, "tabs"),
 
-     box_interface_(&tabs_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, kPadding),
-     box_interface_hbox_(
-        &box_interface_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal, 0, 0, kPadding),
-     box_interface_vbox_(
-        &box_interface_hbox_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, kPadding),
-     box_sound_(&tabs_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, kPadding),
-     box_saving_(&tabs_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, kPadding),
-     box_newgame_(&tabs_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, kPadding),
-     box_ingame_(&tabs_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, kPadding),
+     box_interface_(
+        &tabs_, UI::PanelStyle::kFsMenu, "box_interface", 0, 0, UI::Box::Vertical, 0, 0, kPadding),
+     box_interface_hbox_(&box_interface_,
+                         UI::PanelStyle::kFsMenu,
+                         "hbox_interface",
+                         0,
+                         0,
+                         UI::Box::Horizontal,
+                         0,
+                         0,
+                         kPadding),
+     box_interface_vbox_(&box_interface_hbox_,
+                         UI::PanelStyle::kFsMenu,
+                         "vbox_interface",
+                         0,
+                         0,
+                         UI::Box::Vertical,
+                         0,
+                         0,
+                         kPadding),
+     box_sound_(
+        &tabs_, UI::PanelStyle::kFsMenu, "box_sound", 0, 0, UI::Box::Vertical, 0, 0, kPadding),
+     box_saving_(
+        &tabs_, UI::PanelStyle::kFsMenu, "box_saving", 0, 0, UI::Box::Vertical, 0, 0, kPadding),
+     box_newgame_(
+        &tabs_, UI::PanelStyle::kFsMenu, "box_newgame", 0, 0, UI::Box::Vertical, 0, 0, kPadding),
+     box_ingame_(
+        &tabs_, UI::PanelStyle::kFsMenu, "box_ingame", 0, 0, UI::Box::Vertical, 0, 0, kPadding),
 
      // Interface options
      language_dropdown_(&box_interface_vbox_,
@@ -127,43 +147,32 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
                           UI::DropdownType::kTextual,
                           UI::PanelStyle::kFsMenu,
                           UI::ButtonStyle::kFsMenuMenu),
-
-     inputgrab_(&box_interface_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Grab Input"), "", 0),
      sdl_cursor_(&box_interface_,
                  UI::PanelStyle::kFsMenu,
+                 "sdl_cursor",
                  Vector2i::zero(),
                  _("Use system mouse cursor"),
                  "",
                  0),
-     sb_maxfps_(
-        &box_interface_, 0, 0, 0, 0, opt.maxfps, 0, 99, UI::PanelStyle::kFsMenu, _("Maximum FPS:")),
      tooltip_accessibility_mode_(&box_interface_,
                                  UI::PanelStyle::kFsMenu,
+                                 "tooltip_accessibility_mode",
                                  Vector2i::zero(),
                                  _("Accessibility mode for tooltips")),
-     translation_info_(&box_interface_hbox_, 0, 0, 100, 20, UI::PanelStyle::kFsMenu),
+     translation_info_(
+        &box_interface_hbox_, "translation_info", 0, 0, 100, 20, UI::PanelStyle::kFsMenu),
+     translation_padding_(&box_interface_vbox_, UI::PanelStyle::kFsMenu, "padding", 0, 0, 0, 0),
 
-     // Windows options
-     snap_win_overlap_only_(&box_interface_,
-                            UI::PanelStyle::kFsMenu,
-                            Vector2i::zero(),
-                            _("Snap windows only when overlapping"),
-                            "",
-                            0),
+     // Window options
      dock_windows_to_edges_(&box_interface_,
                             UI::PanelStyle::kFsMenu,
+                            "dock_to_edges",
                             Vector2i::zero(),
                             _("Dock windows to edges"),
                             "",
                             0),
-     animate_map_panning_(&box_interface_,
-                          UI::PanelStyle::kFsMenu,
-                          Vector2i::zero(),
-                          _("Animate automatic map movements"),
-                          "",
-                          0),
-
      sb_dis_panel_(&box_interface_,
+                   "panel_snap_distance",
                    0,
                    0,
                    0,
@@ -176,6 +185,7 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
                    UI::SpinBox::Units::kPixels),
 
      sb_dis_border_(&box_interface_,
+                    "border_snap_distance",
                     0,
                     0,
                     0,
@@ -201,6 +211,7 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 
      // Saving options
      sb_autosave_(&box_saving_,
+                  "autosave_interval",
                   0,
                   0,
                   0,
@@ -214,6 +225,7 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
                   UI::SpinBox::Type::kBig),
 
      sb_rolling_autosave_(&box_saving_,
+                          "rolling_autosave",
                           0,
                           0,
                           0,
@@ -224,38 +236,97 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
                           UI::PanelStyle::kFsMenu,
                           _("Maximum number of autosave files:"),
                           UI::SpinBox::Units::kNone,
-                          UI::SpinBox::Type::kBig),
+                          UI::SpinBox::Type::kBig,
+                          1,
+                          ChangeBigStep::kMediumRange),
 
+     sb_replay_lifetime_(&box_saving_,
+                         "replay_lifetime",
+                         0,
+                         0,
+                         0,
+                         0,
+                         opt.replay_lifetime,
+                         0,
+                         52,
+                         UI::PanelStyle::kFsMenu,
+                         _("Delete replays after:"),
+                         UI::SpinBox::Units::kWeeks,
+                         UI::SpinBox::Type::kBig,
+                         1,
+                         4),
+
+     skip_autosave_on_inactivity_(&box_saving_,
+                                  UI::PanelStyle::kFsMenu,
+                                  "skip_autosave_on_inactivity",
+                                  Vector2i::zero(),
+                                  _("Skip autosaves while inactive"),
+                                  "",
+                                  0),
      zip_(&box_saving_,
           UI::PanelStyle::kFsMenu,
+          "compress",
           Vector2i::zero(),
           _("Compress Widelands data files (maps, replays, and savegames)"),
           "",
           0),
-     write_syncstreams_(&box_saving_,
+
+     save_chat_history_(&box_saving_,
                         UI::PanelStyle::kFsMenu,
+                        "save_chat_history",
                         Vector2i::zero(),
-                        _("Write syncstreams in network games to debug desyncs"),
+                        _("Keep history of sent chat messages in a file"),
                         "",
                         0),
+
      // New Game options
-     show_buildhelp_(
-        &box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Building Spaces")),
-     show_census_(&box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Census")),
-     show_statistics_(&box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Status")),
-     show_soldier_levels_(
-        &box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Soldier Levels")),
-     show_buildings_(&box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Buildings")),
-     show_workarea_overlap_(
-        &box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Workarea Overlaps")),
+     show_buildhelp_(&box_newgame_,
+                     UI::PanelStyle::kFsMenu,
+                     "buildhelp",
+                     Vector2i::zero(),
+                     _("Show Building Spaces")),
+     show_census_(
+        &box_newgame_, UI::PanelStyle::kFsMenu, "census", Vector2i::zero(), _("Show Census")),
+     show_statistics_(
+        &box_newgame_, UI::PanelStyle::kFsMenu, "statistics", Vector2i::zero(), _("Show Status")),
+     show_soldier_levels_(&box_newgame_,
+                          UI::PanelStyle::kFsMenu,
+                          "soldier_levels",
+                          Vector2i::zero(),
+                          _("Show Soldier Levels")),
+     show_buildings_(
+        &box_newgame_, UI::PanelStyle::kFsMenu, "buildings", Vector2i::zero(), _("Show Buildings")),
+     show_workarea_overlap_(&box_newgame_,
+                            UI::PanelStyle::kFsMenu,
+                            "workarea_overlap",
+                            Vector2i::zero(),
+                            _("Show Workarea Overlaps")),
 
      // In-Game options
+     sb_pause_game_on_inactivity_(&box_ingame_,
+                                  "pause_game_on_inactivity",
+                                  0,
+                                  0,
+                                  0,
+                                  0,
+                                  opt.pause_game_on_inactivity,
+                                  0,
+                                  120,
+                                  UI::PanelStyle::kFsMenu,
+                                  _("Pause game when inactive for:"),
+                                  UI::SpinBox::Units::kMinutes,
+                                  UI::SpinBox::Type::kBig,
+                                  1,
+                                  10),
+
      auto_roadbuild_mode_(&box_ingame_,
                           UI::PanelStyle::kFsMenu,
+                          "auto_roadbuild",
                           Vector2i::zero(),
                           _("Start building road after placing a flag")),
      transparent_chat_(&box_ingame_,
                        UI::PanelStyle::kFsMenu,
+                       "transparent_chat",
                        Vector2i::zero(),
                        _("Show in-game chat with transparent background"),
                        "",
@@ -263,6 +334,7 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 
      single_watchwin_(&box_ingame_,
                       UI::PanelStyle::kFsMenu,
+                      "single_watchwindow",
                       Vector2i::zero(),
                       /** TRANSLATORS: A watchwindow is a window where you keep watching an object
                          or a map region,*/
@@ -270,20 +342,32 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
                       _("Use single watchwindow mode")),
      game_clock_(&box_ingame_,
                  UI::PanelStyle::kFsMenu,
+                 "system_clock",
                  Vector2i::zero(),
                  _("Display system time in the info panel")),
      numpad_diagonalscrolling_(&box_ingame_,
                                UI::PanelStyle::kFsMenu,
+                               "numpad_diagonal_scrolling",
                                Vector2i::zero(),
                                _("Allow diagonal scrolling with the numeric keypad")),
      edge_scrolling_(&box_ingame_,
                      UI::PanelStyle::kFsMenu,
+                     "edge_scrolling",
                      Vector2i::zero(),
                      _("Scroll when the mouse cursor is near the screen edge")),
      invert_movement_(&box_ingame_,
                       UI::PanelStyle::kFsMenu,
+                      "invert_movement",
                       Vector2i::zero(),
                       _("Invert click-and-drag map movement direction")),
+     animate_map_panning_(&box_ingame_,
+                          UI::PanelStyle::kFsMenu,
+                          "animate_map_panning",
+                          Vector2i::zero(),
+                          _("Animate automatic map movements"),
+                          "",
+                          0),
+
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
      training_wheels_box_(&box_ingame_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
      training_wheels_(&training_wheels_box_,
@@ -321,28 +405,26 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 	tabs_.add("options_newgame", _("New Games"), &box_newgame_, "");
 	tabs_.add("options_ingame", _("In-Game"), &box_ingame_, "");
 
-	// We want the last active tab when "Apply" was clicked.
-	if (os_.active_tab < tabs_.tabs().size()) {
-		tabs_.activate(os_.active_tab);
-	}
-
 	// Interface
 	box_interface_vbox_.add(&language_dropdown_, UI::Box::Resizing::kFullSize);
 	box_interface_vbox_.add(&resolution_dropdown_, UI::Box::Resizing::kFullSize);
+	// TODO(tothxa): Replace with infinite space if box layouting quirks get fixed
+	box_interface_vbox_.add(&translation_padding_, UI::Box::Resizing::kFullSize);
+	// box_interface_vbox_.add_inf_space();
+
 	box_interface_hbox_.add(&box_interface_vbox_, UI::Box::Resizing::kExpandBoth);
 	box_interface_hbox_.add(&translation_info_, UI::Box::Resizing::kExpandBoth);
 
 	box_interface_.add(&box_interface_hbox_, UI::Box::Resizing::kFullSize);
-	box_interface_.add(&inputgrab_, UI::Box::Resizing::kFullSize);
+
 	box_interface_.add(&sdl_cursor_, UI::Box::Resizing::kFullSize);
-	box_interface_.add(&sb_maxfps_);
 	box_interface_.add(&tooltip_accessibility_mode_, UI::Box::Resizing::kFullSize);
 
-	box_interface_.add(&snap_win_overlap_only_, UI::Box::Resizing::kFullSize);
 	box_interface_.add(&dock_windows_to_edges_, UI::Box::Resizing::kFullSize);
-	box_interface_.add(&animate_map_panning_, UI::Box::Resizing::kFullSize);
 	box_interface_.add(&sb_dis_panel_);
 	box_interface_.add(&sb_dis_border_);
+
+	box_interface_.add_space(kPadding);
 	box_interface_.add(&configure_keyboard_);
 
 	// Sound
@@ -351,8 +433,10 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 	// Saving
 	box_saving_.add(&sb_autosave_, UI::Box::Resizing::kFullSize);
 	box_saving_.add(&sb_rolling_autosave_, UI::Box::Resizing::kFullSize);
+	box_saving_.add(&sb_replay_lifetime_, UI::Box::Resizing::kFullSize);
+	box_saving_.add(&skip_autosave_on_inactivity_, UI::Box::Resizing::kFullSize);
 	box_saving_.add(&zip_, UI::Box::Resizing::kFullSize);
-	box_saving_.add(&write_syncstreams_, UI::Box::Resizing::kFullSize);
+	box_saving_.add(&save_chat_history_, UI::Box::Resizing::kFullSize);
 
 	// New Games
 	box_newgame_.add(&show_buildhelp_, UI::Box::Resizing::kFullSize);
@@ -363,6 +447,7 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 	box_newgame_.add(&show_workarea_overlap_, UI::Box::Resizing::kFullSize);
 
 	// In-Game
+	box_ingame_.add(&sb_pause_game_on_inactivity_, UI::Box::Resizing::kFullSize);
 	box_ingame_.add(&auto_roadbuild_mode_, UI::Box::Resizing::kFullSize);
 	box_ingame_.add(&transparent_chat_, UI::Box::Resizing::kFullSize);
 	box_ingame_.add(&single_watchwin_, UI::Box::Resizing::kFullSize);
@@ -370,8 +455,9 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 	box_ingame_.add(&numpad_diagonalscrolling_, UI::Box::Resizing::kFullSize);
 	box_ingame_.add(&edge_scrolling_, UI::Box::Resizing::kFullSize);
 	box_ingame_.add(&invert_movement_, UI::Box::Resizing::kFullSize);
-	box_ingame_.add_space(kPadding);
+	box_ingame_.add(&animate_map_panning_, UI::Box::Resizing::kFullSize);
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
+	box_ingame_.add_space(kPadding);
 	box_ingame_.add(&training_wheels_box_, UI::Box::Resizing::kFullSize);
 	training_wheels_box_.add(&training_wheels_, UI::Box::Resizing::kFullSize);
 	training_wheels_box_.add_inf_space();
@@ -418,23 +504,24 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 
 	/** TRANSLATORS: Options: Save game automatically every: */
 	sb_autosave_.add_replacement(0, _("Off"));
+	sb_pause_game_on_inactivity_.add_replacement(0, _("Off"));
+	/** TRANSLATORS: Options: Delete replays after: */
+	sb_replay_lifetime_.add_replacement(0, _("Never"));
 
 	// Fill in data
 	// Interface options
 	add_screen_resolutions(opt);
 
-	inputgrab_.set_state(opt.inputgrab);
 	sdl_cursor_.set_state(opt.sdl_cursor);
 	tooltip_accessibility_mode_.set_state(opt.tooltip_accessibility_mode);
 
-	// Windows options
-	snap_win_overlap_only_.set_state(opt.snap_win_overlap_only);
+	// Window options
 	dock_windows_to_edges_.set_state(opt.dock_windows_to_edges);
-	animate_map_panning_.set_state(opt.animate_map_panning);
 
 	// Saving options
+	skip_autosave_on_inactivity_.set_state(opt.skip_autosave_on_inactivity);
 	zip_.set_state(opt.zip);
-	write_syncstreams_.set_state(opt.write_syncstreams);
+	save_chat_history_.set_state(opt.save_chat_history);
 
 	// Game options
 	auto_roadbuild_mode_.set_state(opt.auto_roadbuild_mode);
@@ -444,6 +531,7 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 	numpad_diagonalscrolling_.set_state(opt.numpad_diagonalscrolling);
 	edge_scrolling_.set_state(opt.edge_scrolling);
 	invert_movement_.set_state(opt.invert_movement);
+	animate_map_panning_.set_state(opt.animate_map_panning);
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 	training_wheels_.set_state(opt.training_wheels);
 #endif
@@ -462,16 +550,20 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 	update_language_stats();
 	layout();
 
+	// We want the last active tab when "Apply" was clicked.
+	if (os_.active_tab < tabs_.tabs().size()) {
+		tabs_.activate(os_.active_tab);
+	}
 	initialization_complete();
 }
 void Options::add_screen_resolutions(const OptionsCtrl::OptionsStruct& opt) {
 	ScreenResolution current_res = {g_gr->get_window_mode_xres(), g_gr->get_window_mode_yres()};
 
 	resolution_dropdown_.add(
-	   /** TRANSLATORS: Entry in the window size dropdown*/
+	   /** TRANSLATORS: Entry in the window size dropdown */
 	   _("Fullscreen"), {kDropdownFullscreen, kDropdownFullscreen}, nullptr, opt.fullscreen);
 #ifdef RESIZABLE_WINDOW
-	/** TRANSLATORS: Entry in the window size dropdown*/
+	/** TRANSLATORS: Entry in the window size dropdown */
 	resolution_dropdown_.add(_("Maximized"), {kDropdownMaximized, kDropdownMaximized}, nullptr,
 	                         !resolution_dropdown_.has_selection() && opt.maximized);
 #endif
@@ -487,7 +579,7 @@ void Options::add_screen_resolutions(const OptionsCtrl::OptionsStruct& opt) {
 				previous = this_res;
 				const bool selected = !resolution_dropdown_.has_selection() && this_res == current_res;
 				resolution_dropdown_.add(
-				   /** TRANSLATORS: Screen resolution, e.g. 800 × 600*/
+				   /** TRANSLATORS: Screen resolution, e.g. 800 × 600 */
 				   format(_("%1% × %2%"), this_res.xres, this_res.yres), this_res, nullptr, selected);
 			}
 		}
@@ -495,7 +587,7 @@ void Options::add_screen_resolutions(const OptionsCtrl::OptionsStruct& opt) {
 
 	if (!resolution_dropdown_.has_selection()) {
 		resolution_dropdown_.add(
-		   /** TRANSLATORS: Screen resolution, e.g. 800 × 600*/
+		   /** TRANSLATORS: Screen resolution, e.g. 800 × 600 */
 		   format(_("%1% × %2%"), current_res.xres, current_res.yres), current_res, nullptr, true);
 	}
 }
@@ -504,6 +596,7 @@ void Options::layout() {
 	if (!is_minimal()) {
 		const int16_t butw = get_inner_w() / 5;
 		const int16_t buth = get_inner_h() / 16;
+		constexpr int16_t kSpinboxW = 250;
 
 		// Buttons
 		cancel_.set_desired_size(butw, buth);
@@ -521,22 +614,40 @@ void Options::layout() {
 		// Interface
 		language_dropdown_.set_height(tabs_.get_h() - language_dropdown_.get_y() - buth -
 		                              3 * kPadding);
-		translation_info_.set_size(
-		   language_dropdown_.get_w(),
-		   language_dropdown_.get_h() + resolution_dropdown_.get_h() + kPadding);
-		sb_maxfps_.set_unit_width(unit_w);
-		sb_maxfps_.set_desired_size(tab_panel_width, sb_maxfps_.get_h());
 
-		sb_dis_panel_.set_unit_width(unit_w);
-		sb_dis_panel_.set_desired_size(tab_panel_width, sb_dis_panel_.get_h());
-		sb_dis_border_.set_unit_width(unit_w);
-		sb_dis_border_.set_desired_size(tab_panel_width, sb_dis_border_.get_h());
+		const int min_h = language_dropdown_.get_h() + resolution_dropdown_.get_h() + 2 * kPadding;
+		const int half_w = (tab_panel_width - 3 * kPadding) / 2;
+
+		// Make initial value big enough to avoid needing a scrollbar
+		int translation_h = 3 * min_h;
+
+		// Find out the required height
+		translation_info_.set_desired_size(half_w, translation_h);
+		translation_info_.set_size(half_w, translation_h);
+		int tmp_w = 0;
+		translation_info_.get_text_size(&tmp_w, &translation_h);
+
+		// Now set the final height
+		translation_h = std::max(translation_h, min_h);
+		translation_info_.set_desired_size(half_w, translation_h);
+		translation_info_.set_size(half_w, translation_h);
+		// TODO(tothxa): Remove if box layouting quirks get fixed
+		const int translation_pad_h = translation_h - min_h;
+		translation_padding_.set_desired_size(half_w, translation_pad_h);
+		translation_padding_.set_size(half_w, translation_pad_h);
+
+		// Interface tab spinboxes
+		for (UI::SpinBox* sb : {&sb_dis_panel_, &sb_dis_border_}) {
+			sb->set_unit_width(unit_w);
+			sb->set_desired_size(tab_panel_width, sb->get_h());
+		}
 
 		// Saving options
-		sb_autosave_.set_unit_width(250);
-		sb_autosave_.set_desired_size(tab_panel_width, sb_autosave_.get_h());
-		sb_rolling_autosave_.set_unit_width(250);
-		sb_rolling_autosave_.set_desired_size(tab_panel_width, sb_rolling_autosave_.get_h());
+		for (UI::SpinBox* sb : {&sb_autosave_, &sb_rolling_autosave_, &sb_replay_lifetime_,
+		                        &sb_pause_game_on_inactivity_}) {
+			sb->set_unit_width(kSpinboxW);
+			sb->set_desired_size(tab_panel_width, sb->get_h());
+		}
 	}
 	UI::Window::layout();
 }
@@ -673,9 +784,10 @@ void Options::update_language_stats() {
 		          format(_("If you wish to help us translate, please visit %s"),
 		                 "<font underline=1>widelands.org/wiki/TranslatingWidelands</font>");
 	}
-	// Make font a bit smaller so the link will fit at 800x600 resolution.
+
 	translation_info_.set_text(
 	   as_richtext_paragraph(message, UI::FontStyle::kFsMenuTranslationInfo));
+	layout();
 }
 
 void Options::clicked_ok() {
@@ -722,23 +834,21 @@ OptionsCtrl::OptionsStruct Options::get_values() {
 			os_.yres = res.yres;
 		}
 	}
-	os_.inputgrab = inputgrab_.get_state();
 	os_.sdl_cursor = sdl_cursor_.get_state();
-	os_.maxfps = sb_maxfps_.get_value();
 	os_.tooltip_accessibility_mode = tooltip_accessibility_mode_.get_state();
 
-	// Windows options
-	os_.snap_win_overlap_only = snap_win_overlap_only_.get_state();
+	// Window options
 	os_.dock_windows_to_edges = dock_windows_to_edges_.get_state();
-	os_.animate_map_panning = animate_map_panning_.get_state();
 	os_.panel_snap_distance = sb_dis_panel_.get_value();
 	os_.border_snap_distance = sb_dis_border_.get_value();
 
 	// Saving options
 	os_.autosave = sb_autosave_.get_value();
 	os_.rolling_autosave = sb_rolling_autosave_.get_value();
+	os_.replay_lifetime = sb_replay_lifetime_.get_value();
+	os_.skip_autosave_on_inactivity = skip_autosave_on_inactivity_.get_state();
 	os_.zip = zip_.get_state();
-	os_.write_syncstreams = write_syncstreams_.get_state();
+	os_.save_chat_history = save_chat_history_.get_state();
 
 	// Game options
 	os_.auto_roadbuild_mode = auto_roadbuild_mode_.get_state();
@@ -748,11 +858,13 @@ OptionsCtrl::OptionsStruct Options::get_values() {
 	os_.numpad_diagonalscrolling = numpad_diagonalscrolling_.get_state();
 	os_.edge_scrolling = edge_scrolling_.get_state();
 	os_.invert_movement = invert_movement_.get_state();
+	os_.animate_map_panning = animate_map_panning_.get_state();
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 	os_.training_wheels = training_wheels_.get_state();
 #endif
 
 	// New Game options
+	os_.pause_game_on_inactivity = sb_pause_game_on_inactivity_.get_value();
 	int32_t flags = show_buildhelp_.get_state() ? InteractiveBase::dfShowBuildhelp : 0;
 	flags |= show_census_.get_state() ? InteractiveBase::dfShowCensus : 0;
 	flags |= show_statistics_.get_state() ? InteractiveBase::dfShowStatistics : 0;
@@ -803,25 +915,24 @@ OptionsCtrl::OptionsStruct OptionsCtrl::options_struct(uint32_t active_tab) {
 	opt.yres = opt_section_.get_int("yres", kDefaultResolutionH);
 	opt.maximized = opt_section_.get_bool("maximized", false);
 	opt.fullscreen = opt_section_.get_bool("fullscreen", false);
-	opt.inputgrab = opt_section_.get_bool("inputgrab", false);
-	opt.maxfps = opt_section_.get_int("maxfps", 25);
 	opt.sdl_cursor = opt_section_.get_bool("sdl_cursor", true);
 	opt.tooltip_accessibility_mode = opt_section_.get_bool("tooltip_accessibility_mode", false);
 
-	// Windows options
-	opt.snap_win_overlap_only = opt_section_.get_bool("snap_windows_only_when_overlapping", false);
+	// Window options
 	opt.dock_windows_to_edges = opt_section_.get_bool("dock_windows_to_edges", false);
-	opt.animate_map_panning = opt_section_.get_bool("animate_map_panning", true);
 	opt.panel_snap_distance = opt_section_.get_int("panel_snap_distance", 0);
 	opt.border_snap_distance = opt_section_.get_int("border_snap_distance", 0);
 
 	// Saving options
 	opt.autosave = opt_section_.get_int("autosave", kDefaultAutosaveInterval * 60);
 	opt.rolling_autosave = opt_section_.get_int("rolling_autosave", 5);
+	opt.replay_lifetime = opt_section_.get_int("replay_lifetime", 0);
+	opt.skip_autosave_on_inactivity = opt_section_.get_bool("skip_autosave_on_inactivity", true);
 	opt.zip = !opt_section_.get_bool("nozip", false);
-	opt.write_syncstreams = opt_section_.get_bool("write_syncstreams", true);
+	opt.save_chat_history = opt_section_.get_bool("save_chat_history", false);
 
 	// Game options
+	opt.pause_game_on_inactivity = opt_section_.get_int("pause_game_on_inactivity", 0);
 	opt.auto_roadbuild_mode = opt_section_.get_bool("auto_roadbuild_mode", true);
 	opt.transparent_chat = opt_section_.get_bool("transparent_chat", true);
 	opt.single_watchwin = opt_section_.get_bool("single_watchwin", false);
@@ -829,6 +940,7 @@ OptionsCtrl::OptionsStruct OptionsCtrl::options_struct(uint32_t active_tab) {
 	opt.numpad_diagonalscrolling = opt_section_.get_bool("numpad_diagonalscrolling", false);
 	opt.edge_scrolling = opt_section_.get_bool("edge_scrolling", false);
 	opt.invert_movement = opt_section_.get_bool("invert_movement", false);
+	opt.animate_map_panning = opt_section_.get_bool("animate_map_panning", true);
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 	opt.training_wheels = opt_section_.get_bool("training_wheels", true);
 #endif
@@ -852,25 +964,24 @@ void OptionsCtrl::save_options() {
 	opt_section_.set_int("yres", opt.yres);
 	opt_section_.set_bool("maximized", opt.maximized);
 	opt_section_.set_bool("fullscreen", opt.fullscreen);
-	opt_section_.set_bool("inputgrab", opt.inputgrab);
-	opt_section_.set_int("maxfps", opt.maxfps);
 	opt_section_.set_bool("sdl_cursor", opt.sdl_cursor);
 	opt_section_.set_bool("tooltip_accessibility_mode", opt.tooltip_accessibility_mode);
 
-	// Windows options
-	opt_section_.set_bool("snap_windows_only_when_overlapping", opt.snap_win_overlap_only);
+	// Window options
 	opt_section_.set_bool("dock_windows_to_edges", opt.dock_windows_to_edges);
-	opt_section_.set_bool("animate_map_panning", opt.animate_map_panning);
 	opt_section_.set_int("panel_snap_distance", opt.panel_snap_distance);
 	opt_section_.set_int("border_snap_distance", opt.border_snap_distance);
 
 	// Saving options
 	opt_section_.set_int("autosave", opt.autosave * 60);
 	opt_section_.set_int("rolling_autosave", opt.rolling_autosave);
+	opt_section_.set_int("replay_lifetime", opt.replay_lifetime);
+	opt_section_.set_bool("skip_autosave_on_inactivity", opt.skip_autosave_on_inactivity);
 	opt_section_.set_bool("nozip", !opt.zip);
-	opt_section_.set_bool("write_syncstreams", opt.write_syncstreams);
+	opt_section_.set_bool("save_chat_history", opt.save_chat_history);
 
 	// Game options
+	opt_section_.set_int("pause_game_on_inactivity", opt.pause_game_on_inactivity);
 	opt_section_.set_bool("auto_roadbuild_mode", opt.auto_roadbuild_mode);
 	opt_section_.set_bool("transparent_chat", opt.transparent_chat);
 	opt_section_.set_bool("single_watchwin", opt.single_watchwin);
@@ -878,6 +989,7 @@ void OptionsCtrl::save_options() {
 	opt_section_.set_bool("numpad_diagonalscrolling", opt.numpad_diagonalscrolling);
 	opt_section_.set_bool("edge_scrolling", opt.edge_scrolling);
 	opt_section_.set_bool("invert_movement", opt.invert_movement);
+	opt_section_.set_bool("animate_map_panning", opt.animate_map_panning);
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 	opt_section_.set_bool("training_wheels", opt.training_wheels);
 #endif
@@ -888,7 +1000,6 @@ void OptionsCtrl::save_options() {
 	// Language options
 	opt_section_.set_string("language", opt.language);
 
-	WLApplication::get()->set_input_grab(opt.inputgrab);
 	g_mouse_cursor->set_use_sdl(opt_dialog_->get_values().sdl_cursor);
 	i18n::set_locale(opt.language);
 	UI::g_fh->reinitialize_fontset(i18n::get_locale());
