@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2022 by the Widelands Development Team
+ * Copyright (C) 2006-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,6 +44,7 @@
 #include "base/random.h"
 #include "base/string.h"
 #include "base/time_string.h"
+#include "base/warning.h"
 #include "base/wexception.h"
 #include "build_info.h"
 #include "config.h"
@@ -53,6 +54,7 @@
 #include "graphic/font_handler.h"
 #include "graphic/graphic.h"
 #include "graphic/mouse_cursor.h"
+#include "graphic/style_manager.h"
 #include "graphic/text/font_set.h"
 #include "io/filesystem/disk_filesystem.h"
 #include "io/filesystem/filesystem_exceptions.h"
@@ -64,7 +66,6 @@
 #include "logic/game_settings.h"
 #include "logic/map.h"
 #include "logic/replay.h"
-#include "logic/replay_game_controller.h"
 #include "logic/single_player_game_controller.h"
 #include "logic/single_player_game_settings_provider.h"
 #include "map_io/map_loader.h"
@@ -73,9 +74,11 @@
 #include "network/host_game_settings_provider.h"
 #include "network/internet_gaming.h"
 #include "sound/sound_handler.h"
+#include "ui_basic/color_chooser.h"
 #include "ui_basic/messagebox.h"
 #include "ui_basic/progresswindow.h"
 #include "ui_fsmenu/about.h"
+#include "ui_fsmenu/crash_report.h"
 #include "ui_fsmenu/launch_spg.h"
 #include "ui_fsmenu/loadgame.h"
 #include "ui_fsmenu/main.h"
@@ -84,8 +87,10 @@
 #include "wlapplication_messages.h"
 #include "wlapplication_mousewheel_options.h"
 #include "wlapplication_options.h"
+#include "wui/game_chat_panel.h"
 #include "wui/interactive_player.h"
 #include "wui/interactive_spectator.h"
+#include "wui/maptable.h"
 
 std::string get_executable_directory(const bool logdir) {
 	std::string executabledir;
@@ -136,6 +141,12 @@ void terminate(int /*unused*/) {
 }
 #endif
 
+#ifdef SIGUSR1
+void toggle_verbose(int /*unused*/) {
+	g_verbose = !g_verbose;
+}
+#endif
+
 bool is_absolute_path(const std::string& path) {
 	std::regex re("^/|\\w:");
 	return std::regex_search(path.c_str(), re);
@@ -206,8 +217,7 @@ bool extract_creation_day(const std::string& path, tm* tfile) {
 
 // Returns true if 'filename' was autogenerated, i.e. if 'extract_creation_day'
 // can return a date and it is old enough to be deleted.
-bool is_autogenerated_and_expired(const std::string& filename,
-                                  const double keep_time = kReplayKeepAroundTime) {
+bool is_autogenerated_and_expired(const std::string& filename, const double keep_time) {
 	tm tfile;
 	if (!extract_creation_day(filename, &tfile)) {
 		return false;
@@ -243,6 +253,8 @@ private:
 
 }  // namespace
 
+std::string WLApplication::segfault_backtrace_dir;
+
 // Set up the homedir. Exit 1 if the homedir is illegal or the logger couldn't be initialized for
 // Windows.
 // Also sets the config directory. This defaults to $XDG_CONFIG_HOME/widelands/config on Unix.
@@ -297,8 +309,13 @@ void WLApplication::setup_homedir() {
 		// Create directory structure
 		g_fs->ensure_directory_exists("save");
 		g_fs->ensure_directory_exists("replays");
-		g_fs->ensure_directory_exists("maps/My_Maps");
-		g_fs->ensure_directory_exists("maps/Downloaded");
+		g_fs->ensure_directory_exists(kMyMapsDirFull);
+		g_fs->ensure_directory_exists(kDownloadedMapsDirFull);
+
+		g_fs->ensure_directory_exists(kCrashDir);
+		segfault_backtrace_dir = homedir_;
+		segfault_backtrace_dir += "/";
+		segfault_backtrace_dir += kCrashDir;
 	}
 
 #ifdef USE_XDG
@@ -310,8 +327,6 @@ void WLApplication::setup_homedir() {
 	i18n::set_homedir(homedir_);
 }
 
-WLApplication* WLApplication::the_singleton = nullptr;
-
 /**
  * The main entry point for the WLApplication singleton.
  *
@@ -324,13 +339,10 @@ WLApplication* WLApplication::the_singleton = nullptr;
  *
  * \param argc The number of command line arguments
  * \param argv Array of command line arguments
- * \return An (always valid!) pointer to the WLApplication singleton
+ * \return A reference to the WLApplication singleton
  */
-// TODO(unknown): Return a reference - the return value is always valid anyway
-WLApplication* WLApplication::get(int const argc, char const** argv) {
-	if (the_singleton == nullptr) {
-		the_singleton = new WLApplication(argc, argv);
-	}
+WLApplication& WLApplication::get(int const argc, char const** argv) {
+	static WLApplication the_singleton{argc, argv};
 	return the_singleton;
 }
 
@@ -350,25 +362,18 @@ WLApplication* WLApplication::get(int const argc, char const** argv) {
  * \param argv Array of command line arguments
  */
 WLApplication::WLApplication(int const argc, char const* const* const argv)
-   : game_type_(GameType::kNone),
-     mouse_swapped_(false),
-     faking_middle_mouse_button_(false),
-     mouse_position_(Vector2i::zero()),
-     mouse_locked_(false),
-     handle_key_enabled_(true),
-     should_die_(false),
+   :
 #ifdef _WIN32
-     homedir_(FileSystem::get_homedir() + "\\.widelands"),
+     homedir_(FileSystem::get_homedir() + "\\.widelands")
 #elif defined USE_XDG
      // To enable backwards compatibility, the program name is passed with the
      // path.
      homedir_(FileSystem::get_userdatadir()),
-     userconfigdir_(FileSystem::get_userconfigdir()),
+     userconfigdir_(FileSystem::get_userconfigdir())
 #else
-     homedir_(FileSystem::get_homedir() + "/.widelands"),
+     homedir_(FileSystem::get_homedir() + "/.widelands")
 #endif
-     redirected_stdio_(false),
-     last_resolution_change_(0) {
+{
 	g_fs = new LayeredFileSystem();
 
 	parse_commandline(argc, argv);  // throws ParameterError, handled by main.cc
@@ -380,6 +385,10 @@ WLApplication::WLApplication(int const argc, char const* const* const argv)
 
 	set_initializer_thread();
 
+#ifdef SIGUSR1
+	signal(SIGUSR1, toggle_verbose);
+#endif
+
 	log_info("Adding directory: %s\n", datadir_.c_str());
 	g_fs->add_file_system(&FileSystem::create(datadir_));
 
@@ -460,6 +469,9 @@ WLApplication::WLApplication(int const argc, char const* const* const argv)
 	g_sh->register_songs("music", Songset::kIngame);
 	g_sh->register_songs("music", Songset::kCustom);
 
+	UI::ColorChooser::read_favorites_settings();
+
+	set_template_dir("");
 	initialize_g_addons();
 
 	// Register the click sound for UI::Panel.
@@ -467,14 +479,20 @@ WLApplication::WLApplication(int const argc, char const* const* const argv)
 	// register it once.
 	UI::Panel::register_click();
 
-	set_input_grab(get_config_bool("inputgrab", false));
-
 	// Make sure we didn't forget to read any global option
 	check_config_used();
 
 	// Save configuration now. Otherwise, the UUID and sound options
 	// are not saved, when the game crashes
 	write_config();
+
+	if (get_config_bool("save_chat_history", false)) {
+		g_chat_sent_history.load(kChatSentHistoryFile);
+	}
+	if (g_allow_script_console) {
+		log_info("Developer tools are enabled.");
+		g_script_console_history.load(kScriptConsoleHistoryFile);
+	}
 }
 
 /**
@@ -493,6 +511,13 @@ WLApplication::~WLApplication() {
 	shutdown_hardware();
 	shutdown_settings();
 
+	if (get_config_bool("save_chat_history", false)) {
+		g_chat_sent_history.save(kChatSentHistoryFile);
+	}
+	if (g_allow_script_console) {
+		g_script_console_history.save(kScriptConsoleHistoryFile);
+	}
+
 	assert(UI::g_fh);
 	delete UI::g_fh;
 	UI::g_fh = nullptr;
@@ -560,7 +585,11 @@ void WLApplication::initialize_g_addons() {
 			}
 		}
 	}
-	AddOns::update_ui_theme(AddOns::UpdateThemeAction::kLoadFromConfig);
+	try {
+		AddOns::update_ui_theme(AddOns::UpdateThemeAction::kLoadFromConfig);
+	} catch (const std::exception& e) {
+		log_err("Failed to load add-on theme: %s", e.what());
+	}
 }
 
 static void init_one_player_from_template(unsigned p,
@@ -631,7 +660,7 @@ static void init_one_player_from_template(unsigned p,
 	}
 }
 
-void WLApplication::init_and_run_game_from_template() {
+void WLApplication::init_and_run_game_from_template(FsMenu::MainMenu& mainmenu) {
 	AddOns::AddOnsGuard ag;
 
 	Profile profile(filename_.c_str());
@@ -673,8 +702,9 @@ void WLApplication::init_and_run_game_from_template() {
 	std::unique_ptr<GameSettingsProvider> settings;
 	std::shared_ptr<GameController> ctrl;
 	GameHost* host = nullptr;  // will be deleted by ctrl
+	FsMenu::MenuCapsule capsule(mainmenu);
 	if (multiplayer) {
-		host = new GameHost(nullptr, ctrl, get_config_string("nickname", _("nobody")),
+		host = new GameHost(&capsule, ctrl, get_config_string("nickname", _("nobody")),
 		                    Widelands::get_all_tribeinfos(nullptr), false);
 		ctrl.reset(host);
 		settings.reset(new HostGameSettingsProvider(host));
@@ -683,8 +713,14 @@ void WLApplication::init_and_run_game_from_template() {
 		settings.reset(new SinglePlayerGameSettingsProvider());
 	}
 
-	settings->set_peaceful_mode(section.get_bool("peaceful", false));
-	settings->set_custom_starting_positions(section.get_bool("custom_starting_positions", false));
+	settings->set_flag(GameSettings::Flags::kPeaceful, section.get_bool("peaceful", false));
+	settings->set_flag(GameSettings::Flags::kFogless, section.get_bool("fogless", false));
+	settings->set_flag(GameSettings::Flags::kCustomStartingPositions,
+	                   section.get_bool("custom_starting_positions", false));
+	settings->set_flag(
+	   GameSettings::Flags::kForbidDiplomacy, section.get_bool("forbid_diplomacy", false));
+	settings->set_flag(
+	   GameSettings::Flags::kAllowNavalWarfare, section.get_bool("allow_naval_warfare", false));
 
 	{
 		std::string wc_name = section.get_string("win_condition", "endless_game.lua");
@@ -700,6 +736,8 @@ void WLApplication::init_and_run_game_from_template() {
 			script += wc_name;
 		}
 		settings->set_win_condition_script(script);
+		settings->set_win_condition_duration(
+		   section.get_int("win_condition_duration", Widelands::kDefaultWinConditionDuration));
 	}
 
 	{
@@ -746,20 +784,23 @@ void WLApplication::init_and_run_game_from_template() {
 		tipstexts.push_back(settings->get_players_tribe());
 	}
 	game.create_loader_ui(
-	   tipstexts, true, settings->settings().map_theme, settings->settings().map_background);
+	   tipstexts, true, settings->settings().map_theme, settings->settings().map_background, true);
 	Notifications::publish(UI::NoteLoadingMessage(_("Preparing game…")));
 
 	game.set_ibase(new InteractivePlayer(game, get_config_section(), playernumber, false));
 
 	game.set_game_controller(std::make_shared<SinglePlayerGameController>(game, true, playernumber));
 	game.init_newgame(settings->settings());
+
+	auto custom_names = Widelands::read_custom_warehouse_ship_names();
+	Widelands::Player* player = game.get_safe_player(playernumber);
+	player->set_shipnames(custom_names.first);
+	player->set_warehousenames(custom_names.second);
+
 	try {
-		game.run(Widelands::Game::StartGameType::kMap, script_to_run_, false, "single_player");
-	} catch (const Widelands::GameDataError& e) {
-		log_err("Game not started: Game data error: %s\n", e.what());
+		game.run(Widelands::Game::StartGameType::kMap, script_to_run_, "single_player");
 	} catch (const std::exception& e) {
-		log_err("Fatal exception: %s\n", e.what());
-		emergency_save(nullptr, game, e.what());
+		emergency_save(&mainmenu, game, e.what());
 	}
 }
 
@@ -772,38 +813,68 @@ void WLApplication::init_and_run_game_from_template() {
 void WLApplication::run() {
 	GameLogicThread game_logic_thread(&should_die_);
 
-	if (game_type_ == GameType::kEditor) {
+	FsMenu::MainMenu menu(game_type_ != GameType::kNone);
+
+	check_crash_reports(menu);
+
+	switch (game_type_) {
+	case GameType::kEditor: {
+		bool success = false;
 		g_sh->change_music(Songset::kIngame);
 		if (filename_.empty()) {
-			EditorInteractive::run_editor(nullptr, EditorInteractive::Init::kDefault);
+			success = EditorInteractive::run_editor(&menu, EditorInteractive::Init::kDefault);
 		} else {
-			EditorInteractive::run_editor(
-			   nullptr, EditorInteractive::Init::kLoadMapDirectly, filename_, script_to_run_);
+			bool have_filename = true;
+			if (use_last(filename_)) {
+				if (std::optional<MapData> map = newest_edited_map(); map.has_value()) {
+					filename_ = map->filenames.at(0);
+				} else {
+					const std::string message = _("Widelands could not find the last edited map.");
+					log_err("%s\n", message.c_str());
+
+					g_sh->change_music(Songset::kMenu);
+					menu.show_messagebox(_("No Last Edited Map"), message);
+					have_filename = false;
+				}
+			}
+
+			if (have_filename) {
+				success = EditorInteractive::run_editor(
+				   &menu, EditorInteractive::Init::kLoadMapDirectly, filename_, script_to_run_);
+			}
 		}
-	} else if (game_type_ == GameType::kReplay || game_type_ == GameType::kLoadGame) {
+
+		if (!success) {
+			menu.main_loop();
+		}
+	} break;
+
+	case GameType::kReplay:
+	case GameType::kLoadGame: {
 		Widelands::Game game;
 		std::string title;
 		std::string message;
+		g_sh->change_music(Songset::kIngame);
 		try {
-			if (game_type_ == GameType::kReplay) {
-				std::string map_theme;
-				std::string map_bg;
-				game.create_loader_ui({"general_game"}, true, map_theme, map_bg);
-				game.set_ibase(new InteractiveSpectator(game, get_config_section()));
-				game.set_write_replay(false);
-				new ReplayGameController(game, filename_);
-				game.save_handler().set_allow_saving(false);
-				game.run(Widelands::Game::StartGameType::kSaveGame, "", true, "replay");
+			bool start_replay = (game_type_ == GameType::kReplay);
+			if (use_last(filename_)) {
+				std::optional<SavegameData> data = newest_saved_game_or_replay(start_replay);
+				if (data.has_value()) {
+					filename_ = data->filename;
+				} else {
+					// Parameters will be reordered by FileNotFoundError::what()
+					if (start_replay) {
+						throw FileNotFoundError("--replay", _("No last saved replay."), filename_);
+					}
+					throw FileNotFoundError("--loadgame", _("No last saved game."), filename_);
+				}
+			}
+			if (start_replay) {
+				game.run_replay(filename_, "");
 			} else {
 				game.set_ai_training_mode(get_config_bool("ai_training", false));
 				game.run_load_game(filename_, script_to_run_);
 			}
-		} catch (const Widelands::GameDataError& e) {
-			message = format(_("Widelands could not load the file \"%s\". The file format "
-			                   "seems to be incompatible."),
-			                 filename_.c_str());
-			message = message + "\n\n" + _("Error message:") + "\n" + e.what();
-			title = _("Game data error");
 		} catch (const FileNotFoundError& e) {
 			message = format(_("Widelands could not find the file \"%s\"."), filename_.c_str());
 			message = message + "\n\n" + _("Error message:") + "\n" + e.what();
@@ -814,31 +885,37 @@ void WLApplication::run() {
 			title = _("Error message:");
 		}
 		if (!message.empty()) {
-			g_sh->change_music(Songset::kMenu);
-			FsMenu::MainMenu m(true);
-			m.show_messagebox(title, message);
 			log_err("%s\n", message.c_str());
-			m.run<int>();
+			game.full_cleanup();
+			g_sh->change_music(Songset::kMenu);
+			menu.show_messagebox(title, message);
+			menu.main_loop();
 		}
-	} else if (game_type_ == GameType::kScenario) {
+	} break;
+
+	case GameType::kScenario: {
+		g_sh->change_music(Songset::kIngame);
 		Widelands::Game game;
 		try {
 			game.run_splayer_scenario_direct({filename_}, script_to_run_);
-		} catch (const Widelands::GameDataError& e) {
-			log_err("Scenario not started: Game data error: %s\n", e.what());
 		} catch (const std::exception& e) {
-			log_err("Fatal exception: %s\n", e.what());
-			emergency_save(nullptr, game, e.what());
+			emergency_save(&menu, game, e.what());
 		}
-	} else if (game_type_ == GameType::kFromTemplate) {
-		init_and_run_game_from_template();
-	} else {
+	} break;
+
+	case GameType::kFromTemplate: {
+		g_sh->change_music(Songset::kIngame);
+		init_and_run_game_from_template(menu);
+	} break;
+
+	default:
+		// TODO(tothxa): This is actually never heard due to the next one replacing it immediately.
+		//               Can we delete this, and the song file too?
 		g_sh->change_music(Songset::kIntro);
 
 		g_sh->change_music(Songset::kMenu, 1000);
 
-		FsMenu::MainMenu m;
-		m.run<int>();
+		menu.main_loop();
 	}
 
 	g_sh->stop_music(500);
@@ -905,7 +982,7 @@ bool WLApplication::handle_key(bool down, const SDL_Keycode& keycode, const int
 	if (matches_shortcut(KeyboardShortcut::kCommonScreenshot, keycode, modifiers)) {
 		if (g_fs->disk_space() < kMinimumDiskSpace) {
 			log_warn("Omitting screenshot because diskspace is lower than %lluMiB\n",
-			         kMinimumDiskSpace / (1024 * 1024));
+			         kMinimumDiskSpace / (1024ULL * 1024));
 		} else {
 			g_fs->ensure_directory_exists(kScreenshotsDir);
 			for (uint32_t nr = 0; nr < 10000; ++nr) {
@@ -1080,41 +1157,20 @@ void WLApplication::warp_mouse(const Vector2i position) {
 		SDL_Window* sdl_window = g_gr->get_sdlwindow();
 		if (sdl_window != nullptr) {
 			if (!mouse_locked_) {
-				SDL_PumpEvents();
-				SDL_FlushEvent(SDL_MOUSEMOTION);
-				SDL_WarpMouseInWindow(sdl_window, position.x, position.y);
+				// Fix for #5655 needed for macOS
+				NoteThreadSafeFunction::instantiate(
+				   [sdl_window, position]() {
+					   SDL_PumpEvents();
+					   SDL_FlushEvent(SDL_MOUSEMOTION);
+					   SDL_WarpMouseInWindow(sdl_window, position.x, position.y);
+				   },
+				   true);
 				return;
 			}
 		}
 	}
 }
 
-/**
- * Changes input grab mode.
- *
- * This makes sure that the mouse cannot leave our window (and also that we get
- * mouse/keyboard input nearly unmodified, but we don't really care about that).
- *
- * \note This also cuts out any mouse-speed modifications that a generous window
- * manager might be doing.
- */
-void WLApplication::set_input_grab(bool grab) {
-	if (g_gr == nullptr) {
-		return;
-	}
-	SDL_Window* sdl_window = g_gr->get_sdlwindow();
-	if (grab) {
-		if (sdl_window != nullptr) {
-			SDL_SetWindowGrab(sdl_window, SDL_TRUE);
-		}
-	} else {
-		if (sdl_window != nullptr) {
-			SDL_SetWindowGrab(sdl_window, SDL_FALSE);
-		}
-		warp_mouse(mouse_position_);  // TODO(unknown): is this redundant?
-	}
-}
-
 void WLApplication::set_mouse_lock(const bool locked) {
 	mouse_locked_ = locked;
 	if (mouse_locked_) {
@@ -1160,7 +1216,7 @@ bool WLApplication::init_settings() {
 
 	int64_t last_start = get_config_int("last_start", 0);
 	int64_t now = time(nullptr);
-	if (last_start + 12 * 60 * 60 < now || get_config_string("uuid", "").empty()) {
+	if (last_start + 12LL * 60LL * 60LL < now || get_config_string("uuid", "").empty()) {
 		// First start of the game or not started for 12 hours. Create a (new) UUID.
 		// For the use of the UUID, see network/internet_gaming_protocol.h
 		set_config_string("uuid", generate_random_uuid());
@@ -1297,45 +1353,85 @@ void WLApplication::parse_commandline(int const argc, char const* const* const a
 	}
 }
 
+namespace {
+
+void throw_extra_value(const std::string& opt) {
+	throw ParameterError(
+	   CmdLineVerbosity::None, format(_("Command line parameter --%s can not use a value"), opt));
+}
+
+void throw_empty_value(const std::string& opt) {
+	throw ParameterError(
+	   CmdLineVerbosity::None, format(_("Empty value of command line parameter --%s"), opt));
+}
+
+void throw_exclusive(const std::string& opt, const std::string& other) {
+	throw ParameterError(
+	   CmdLineVerbosity::None,
+	   format(_("Command line parameters --%1$s and --%2$s can not be combined"), opt, other));
+}
+
+}  // namespace
+
+// Checks and returns whether `param` was set, but throws ParameterError if it also had a value.
+bool WLApplication::check_commandline_flag(const std::string& opt) {
+	auto found = commandline_.find(opt);
+	if (found == commandline_.end()) {
+		return false;
+	}
+	if (!found->second.empty()) {
+		throw_extra_value(opt);
+	}
+	commandline_.erase(found);
+	return true;
+}
+
+// Returns the value of `opt`. Only returns std::nullopt if `opt` was not used.
+// If `opt` was used without a value, then returns an empty string if `allow_empty` is true,
+// otherwise throws ParameterError.
+OptionalParameter WLApplication::get_commandline_option_value(const std::string& opt,
+                                                              const bool allow_empty) {
+	auto found = commandline_.find(opt);
+	if (found == commandline_.end()) {
+		return std::nullopt;
+	}
+	// need to copy before deletion for returning later
+	std::string rv = found->second;
+	if (!allow_empty && rv.empty()) {
+		throw_empty_value(opt);
+	}
+	commandline_.erase(found);
+
+	// Fix warning in old clang versions
+#ifdef __clang__
+#if __clang_major__ < 13
+	return std::move(rv);
+#else
+	return rv;
+#endif
+#else
+	return rv;
+#endif
+}
+
 /**
  * Parse the command line given in commandline_
  *
  * \throw a ParameterError if there were errors during parsing \e or if "--help"
  */
 void WLApplication::handle_commandline_parameters() {
-	auto throw_empty_value = [](const std::string& opt) {
-		throw ParameterError(
-		   CmdLineVerbosity::None, format(_("Empty value of command line parameter: %s"), opt));
-	};
 
-	auto throw_exclusive = [](const std::string& opt) {
-		throw ParameterError(
-		   CmdLineVerbosity::None, format(_("%s can not be combined with other actions"), opt));
-	};
-
-	if (commandline_.count("nosound") != 0u) {
-		SoundHandler::disable_backend();
-		commandline_.erase("nosound");
-	}
-	if (commandline_.count("verbose-i18n") != 0u) {
+	if (check_commandline_flag("verbose-i18n")) {
 		i18n::enable_verbose_i18n();
-		commandline_.erase("verbose-i18n");
-	}
-	if (commandline_.count("fail-on-lua-error") != 0u) {
-		g_fail_on_lua_error = true;
-		commandline_.erase("fail-on-lua-error");
-	}
-	if (commandline_.count("nozip") != 0u) {
-		set_config_bool("nozip", true);
-		commandline_.erase("nozip");
-	}
-	if (commandline_.count("localedir") != 0u) {
-		localedir_ = commandline_["localedir"];
-		commandline_.erase("localedir");
 	}
 
-	const bool skip_check_datadir_version = commandline_.count("skip_check_datadir_version") != 0u;
-	commandline_.erase("skip_check_datadir_version");
+	if (OptionalParameter localedir_option = get_commandline_option_value("localedir");
+	    localedir_option.has_value()) {
+		localedir_ = *localedir_option;
+	}
+
+	const bool skip_check_datadir_version = check_commandline_flag("skip_check_datadir_version");
+
 	auto checkdatadirversion = [skip_check_datadir_version](const std::string& dd) {
 		if (skip_check_datadir_version) {
 			return std::string();
@@ -1375,9 +1471,9 @@ void WLApplication::handle_commandline_parameters() {
 		return std::string();
 	};
 	bool found = false;
-	if (commandline_.count("datadir") != 0u) {
-		datadir_ = commandline_["datadir"];
-		commandline_.erase("datadir");
+	if (OptionalParameter datadir_option = get_commandline_option_value("datadir");
+	    datadir_option.has_value()) {
+		datadir_ = *datadir_option;
 
 		const std::string err = checkdatadirversion(datadir_);
 		found = err.empty();
@@ -1433,7 +1529,7 @@ void WLApplication::handle_commandline_parameters() {
 			        "with the --datadir command line option. Tried the following %d path(s):",
 			        static_cast<int>(wrong_candidates.size()));
 			for (const auto& pair : wrong_candidates) {
-				log_err(" · '%s': %s", pair.first.c_str(), pair.second.c_str());
+				log_err(" • '%s': %s", pair.first.c_str(), pair.second.c_str());
 			}
 		}
 	}
@@ -1447,150 +1543,159 @@ void WLApplication::handle_commandline_parameters() {
 		}
 	}
 
-	if (commandline_.count("language") != 0u) {
-		const std::string& lang = commandline_["language"];
-		if (!lang.empty()) {
-			set_config_string("language", lang);
-		} else {
-			if (found) {
-				init_language();
-			}
-			throw_empty_value("--language");
-		}
+	if (OptionalParameter lang = get_commandline_option_value("language"); lang.has_value()) {
+		set_config_string("language", *lang);
 	}
 	if (found) {
 		init_language();  // do this now to have translated command line help
 	}
+	// Set up list of valid command line options and their translated help texts
 	fill_parameter_vector();
 
-	if (commandline_.count("error") != 0u) {
-		throw ParameterError(CmdLineVerbosity::Normal,
-		                     format(_("Unknown command line parameter: %s\nMaybe a '=' is missing?"),
-		                            commandline_["error"]));
+	// This is used by the parser to report an error
+	if (OptionalParameter err = get_commandline_option_value("error"); err.has_value()) {
+		throw ParameterError(
+		   CmdLineVerbosity::Normal,
+		   format(_("Unknown command line parameter: %s\nMaybe a '=' is missing?"), *err));
 	}
 
-	if (commandline_.count("datadir_for_testing") != 0u) {
-		datadir_for_testing_ = commandline_["datadir_for_testing"];
-		commandline_.erase("datadir_for_testing");
+	if (OptionalParameter testdir = get_commandline_option_value("datadir_for_testing");
+	    testdir.has_value()) {
+		datadir_for_testing_ = *testdir;
 	}
 
-	if (commandline_.count("verbose") != 0u) {
+	// TODO(tothxa): These were checked before datadir and locale were set up, but don't seem to be
+	//               used during detecting and setting them up. Let's see if anything breaks if we
+	//               move them here.
+	if (check_commandline_flag("nosound")) {
+		SoundHandler::disable_backend();
+	}
+	if (check_commandline_flag("fail-on-lua-error")) {
+		g_fail_on_lua_error = true;
+	}
+
+	// Mutually exclusive options.
+	// These would be better as e.g. "--use_zip=[true|false]", but then we'd have to negate the
+	// boolean value stored in the string, but trueWords and falseWords are hidden in io/profile.cc.
+	// The obvious "--nozip=[true|false]" would be confusing, or at least hard to write a helptext
+	// for.
+	const bool has_nozip = check_commandline_flag("nozip");
+	const bool has_zip = check_commandline_flag("zip");
+	if (has_nozip && has_zip) {
+		throw_exclusive("nozip", "zip");
+	}
+	// Only override config file if we have a command line parameter
+	if (has_nozip || has_zip) {
+		set_config_bool("nozip", has_nozip);
+	}
+
+	// *** End of moved checks ***
+
+	if (check_commandline_flag("verbose")) {
 		g_verbose = true;
-		commandline_.erase("verbose");
 	}
 
-	if (commandline_.count("editor") != 0u) {
-		filename_ = commandline_["editor"];
-		if (!filename_.empty() && *filename_.rbegin() == '/') {
+	static const std::map<GameType, std::string> game_type_options = {
+	   {GameType::kEditor, "editor"},
+	   {GameType::kReplay, "replay"},
+	   {GameType::kFromTemplate, "new_game_from_template"},
+	   {GameType::kLoadGame, "loadgame"},
+	   {GameType::kScenario, "scenario"}};
+
+	for (const auto& pair : game_type_options) {
+		const std::string& opt = pair.second;
+		const bool allow_empty = opt == "editor";
+		OptionalParameter val = get_commandline_option_value(opt, allow_empty);
+		if (!val.has_value()) {
+			continue;
+		}
+
+		if (game_type_ != GameType::kNone) {
+			throw_exclusive(opt, game_type_options.at(game_type_));
+		}
+		game_type_ = pair.first;
+
+		filename_ = *val;
+		if (filename_.back() == '/') {
+			// Strip trailing directory separator
 			filename_.erase(filename_.size() - 1);
 		}
-		game_type_ = GameType::kEditor;
-		commandline_.erase("editor");
 	}
 
-	if (commandline_.count("replay") != 0u) {
-		if (game_type_ != GameType::kNone) {
-			throw_exclusive("replay");
-		}
-		filename_ = commandline_["replay"];
-		if (filename_.empty()) {
-			throw_empty_value("--replay");
-		}
-		if (*filename_.rbegin() == '/') {
-			filename_.erase(filename_.size() - 1);
-		}
-		game_type_ = GameType::kReplay;
-		commandline_.erase("replay");
-	}
-
-	if (commandline_.count("new_game_from_template") != 0u) {
-		if (game_type_ != GameType::kNone) {
-		}
-		filename_ = commandline_["new_game_from_template"];
-		if (filename_.empty()) {
-			throw_empty_value("--new_game_from_template");
-		}
-		game_type_ = GameType::kFromTemplate;
-		commandline_.erase("new_game_from_template");
-	}
-
-	if (commandline_.count("loadgame") != 0u) {
-		if (game_type_ != GameType::kNone) {
-			throw_exclusive("loadgame");
-		}
-		filename_ = commandline_["loadgame"];
-		if (filename_.empty()) {
-			throw_empty_value("--loadgame");
-		}
-		if (*filename_.rbegin() == '/') {
-			filename_.erase(filename_.size() - 1);
-		}
-		game_type_ = GameType::kLoadGame;
-		commandline_.erase("loadgame");
-	}
-
-	if (commandline_.count("scenario") != 0u) {
-		if (game_type_ != GameType::kNone) {
-			throw_exclusive("scenario");
-		}
-		filename_ = commandline_["scenario"];
-		if (filename_.empty()) {
-			throw_empty_value("--scenario");
-		}
-		if (*filename_.rbegin() == '/') {
-			filename_.erase(filename_.size() - 1);
-		}
-		game_type_ = GameType::kScenario;
-		commandline_.erase("scenario");
-	}
-	if (commandline_.count("script") != 0u) {
-		script_to_run_ = commandline_["script"];
-		if (script_to_run_.empty()) {
-			throw_empty_value("--script");
-		}
-		if (*script_to_run_.rbegin() == '/') {
+	if (OptionalParameter val = get_commandline_option_value("script"); val.has_value()) {
+		script_to_run_ = *val;
+		if (script_to_run_.back() == '/') {
+			// Strip trailing directory separator
 			script_to_run_.erase(script_to_run_.size() - 1);
 		}
-		commandline_.erase("script");
 	}
 
 	// Following is used for training of AI
-	if (commandline_.count("ai_training") != 0u) {
-		set_config_bool("ai_training", true);
-		commandline_.erase("ai_training");
-	} else {
-		set_config_bool("ai_training", false);
+	set_config_bool("ai_training", check_commandline_flag("ai_training"));
+
+	set_config_bool("auto_speed", check_commandline_flag("auto_speed"));
+
+	if (check_commandline_flag("enable_development_testing_tools")) {
+		g_allow_script_console = true;
+	}
+#ifndef NDEBUG
+	// Always enable in debug builds
+	g_allow_script_console = true;
+#endif
+
+	if (check_commandline_flag("write_syncstreams")) {
+		g_write_syncstreams = true;
 	}
 
-	if (commandline_.count("auto_speed") != 0u) {
-		set_config_bool("auto_speed", true);
-		commandline_.erase("auto_speed");
-	} else {
-		set_config_bool("auto_speed", false);
-	}
-
-	if (commandline_.count("version") != 0u) {
+	if (check_commandline_flag("version")) {
 		throw ParameterError(CmdLineVerbosity::None);  // No message on purpose
 	}
 
-	if (commandline_.count("help-all") != 0u) {
+	if (check_commandline_flag("help-all")) {
 		throw ParameterError(CmdLineVerbosity::All);  // No message on purpose
 	}
 
-	if (commandline_.count("help") != 0u) {
+	if (check_commandline_flag("help")) {
 		throw ParameterError(CmdLineVerbosity::Normal);  // No message on purpose
 	}
 
-	// Override maximized and fullscreen settings for window options
-	uint8_t exclusives = commandline_.count("xres") + commandline_.count("yres") +
-	                     2 * commandline_.count("maximized") + 2 * commandline_.count("fullscreen");
-	if (exclusives > 2) {
-		throw ParameterError(CmdLineVerbosity::None,
-		                     _("--xres/--yres, --maximized and --fullscreen can not be combined"));
-	}
-	if (exclusives > 0) {
-		set_config_bool("maximized", false);
-		set_config_bool("fullscreen", false);
+	// Window size: three mutually exclusive possibilities
+	// TODO(tothxa): Move to a function, but I don't want to add another member.
+	//               The whole commandline parsing and handling together with reading the config file
+	//               should be moved out of WLApplication, but it's a mess with scattered global
+	//               variables and variables that need passing back to WLApplication.
+	{
+		const bool display_fullscreen = check_commandline_flag("fullscreen");
+		const bool display_maximized = check_commandline_flag("maximized");
+		if (display_maximized && display_fullscreen) {
+			throw_exclusive("fullscreen", "maximized");
+		}
+
+		const OptionalParameter xres = get_commandline_option_value("xres");
+		const OptionalParameter yres = get_commandline_option_value("yres");
+		if ((xres.has_value() || yres.has_value()) && (display_fullscreen || display_maximized)) {
+			std::string which_res;
+			if (xres.has_value() && yres.has_value()) {
+				// "--" will be prepended... ugly here but convenient everywhere else
+				which_res = "xres/--yres";
+			} else {
+				// Exactly one of them
+				which_res = xres.has_value() ? "xres" : "yres";
+			}
+			throw_exclusive(display_fullscreen ? "fullscreen" : "maximized", which_res);
+		}
+
+		// Only override config file if we have a command line parameter
+		if (xres.has_value() || yres.has_value() || display_fullscreen || display_maximized) {
+			set_config_bool("fullscreen", display_fullscreen);
+			set_config_bool("maximized", display_maximized);
+			if (xres.has_value()) {
+				set_config_string("xres", *xres);
+			}
+			if (yres.has_value()) {
+				set_config_string("yres", *yres);
+			}
+		}
 	}
 
 	// If it hasn't been handled yet it's probably an attempt to
@@ -1622,10 +1727,26 @@ void WLApplication::emergency_save(UI::Panel* panel,
                                    const uint8_t playernumber,
                                    const bool replace_ctrl,
                                    const bool ask_for_bug_report) {
+	if (!is_initializer_thread()) {
+		// We're already handling a bad situation... Let it go as far as it can, but UI calls
+		// may violate assertions and segfault.
+		log_err("WLApplication::emergency_save() is running in the logic thread!\n");
+	}
 	log_err("##############################\n"
 	        "  FATAL EXCEPTION: %s\n"
 	        "##############################\n",
 	        error.c_str());
+
+	if (Widelands::UnhandledVersionError::is_unhandled_version_error(error)) {
+		// It's an incompatible savegame. Don't ask for a bug report, don't bother trying to save.
+		if (panel != nullptr) {
+			UI::WLMessageBox m(panel, UI::WindowStyle::kFsMenu, _("Incompatible"), error,
+			                   UI::WLMessageBox::MBoxType::kOk);
+			m.run<UI::Panel::Returncodes>();
+		}
+		return;
+	}
+
 	if (ask_for_bug_report) {
 		log_err("  Please report this problem to help us improve Widelands.\n"
 		        "  You will find related messages in the standard output (stdout.txt on Windows).\n"
@@ -1637,7 +1758,7 @@ void WLApplication::emergency_save(UI::Panel* panel,
 	        "  It is often – though not always – possible to load it and continue playing.\n"
 	        "##############################");
 	if (!game.is_loaded()) {
-		if (!ask_for_bug_report) {
+		if (!ask_for_bug_report || panel == nullptr) {
 			return;
 		}
 		UI::WLMessageBox m(
@@ -1678,7 +1799,15 @@ void WLApplication::emergency_save(UI::Panel* panel,
 		}
 	}
 
+	bool added_loader = false;
 	try {
+		if (!game.has_loader_ui()) {
+			// Shouldn't have one yet, but in an emergency situation, don't make any assumptions.
+			game.create_loader_ui(
+			   {"crash"}, true, game.map().get_background_theme(), game.map().get_background(), true);
+			added_loader = true;
+		}
+
 		if (replace_ctrl) {
 			game.set_game_controller(
 			   std::make_shared<SinglePlayerGameController>(game, true, playernumber));
@@ -1687,7 +1816,7 @@ void WLApplication::emergency_save(UI::Panel* panel,
 		SaveHandler& save_handler = game.save_handler();
 		std::string e;
 		if (!save_handler.save_game(
-		       game, save_handler.create_file_name(kSaveDir, timestring()), &e)) {
+		       game, save_handler.create_file_name(kSaveDir, timestring()), FileSystem::ZIP, &e)) {
 			throw wexception("Save handler returned error: %s", e.c_str());
 		}
 	} catch (const std::exception& e) {
@@ -1702,6 +1831,10 @@ void WLApplication::emergency_save(UI::Panel* panel,
 			m.run<UI::Panel::Returncodes>();
 		}
 	}
+
+	if (added_loader) {
+		game.remove_loader_ui();
+	}
 }
 
 /**
@@ -1709,11 +1842,19 @@ void WLApplication::emergency_save(UI::Panel* panel,
  * Delete old replay files on startup
  */
 void WLApplication::cleanup_replays() {
+	const int64_t keep_seconds =
+	   7LL * 24 * 60 * 60 * get_config_section().get_int("replay_lifetime", 0);
+	if (keep_seconds <= 0) {
+		return;
+	}
 	for (const std::string& filename : g_fs->filter_directory(kReplayDir, [](const std::string& fn) {
-		     return ends_with(fn, format("%s%s", kReplayExtension, kSyncstreamExtension));
+		     return ends_with(fn, kReplayExtension) ||
+		            ends_with(fn, kSyncstreamExtension)
+		            // TODO(Nordfriese): Remove the legacy extensions after v1.2
+		            || ends_with(fn, ".wrpl") || ends_with(fn, ".wrpl.wgf");
 	     })) {
-		if (is_autogenerated_and_expired(filename, kReplayKeepAroundTime)) {
-			log_info("Delete syncstream or replay %s\n", filename.c_str());
+		if (is_autogenerated_and_expired(filename, keep_seconds)) {
+			log_info("Deleting syncstream or replay %s", filename.c_str());
 			try {
 				g_fs->fs_unlink(filename);
 			} catch (const FileError& e) {
@@ -1747,8 +1888,7 @@ void WLApplication::cleanup_ai_files() {
  * Delete old temp files that might still lurk around (game crashes etc.)
  */
 void WLApplication::cleanup_temp_files() {
-	for (const std::string& filename : g_fs->filter_directory(
-	        kTempFileDir, [](const std::string& fn) { return ends_with(fn, kTempFileExtension); })) {
+	for (const std::string& filename : g_fs->list_directory(kTempFileDir)) {
 		if (is_autogenerated_and_expired(filename, kTempFilesKeepAroundTime)) {
 			log_info("Deleting old temp file: %s\n", filename.c_str());
 			try {
@@ -1834,3 +1974,38 @@ bool WLApplication::redirect_output(std::string path) {
 	redirected_stdio_ = true;
 	return true;
 }
+
+void WLApplication::check_crash_reports(FsMenu::MainMenu& menu) {
+	// First, delete very old crash reports
+	for (const std::string& filename : g_fs->filter_directory(
+	        kCrashDir, [](const std::string& fn) { return ends_with(fn, kOldCrashExtension); })) {
+		if (is_autogenerated_and_expired(filename, kCrashFilesKeepAroundTime)) {
+			log_info("Deleting stale crash file: %s\n", filename.c_str());
+			try {
+				g_fs->fs_unlink(filename);
+			} catch (const FileError& e) {
+				log_warn("WLApplication::check_crash_reports: File %s couldn't be deleted: %s\n",
+				         filename.c_str(), e.what());
+			}
+		}
+	}
+
+	// Now look for new, unreported crashes
+	FilenameSet crashes = g_fs->filter_directory(
+	   kCrashDir, [](const std::string& fn) { return ends_with(fn, kCrashExtension); });
+	if (crashes.empty()) {
+		return;
+	}
+
+	log_info("Found %" PRIuS " unsent crash reports.\nPlease consider submitting them to the "
+	         "Widelands Development Team under %s",
+	         crashes.size(), FsMenu::CrashReportWindow::kReportBugsURL);
+	for (const std::string& filename : crashes) {
+		log_info("- %s", filename.c_str());
+	}
+
+	g_sh->change_music(Songset::kMenu);
+	menu.abort_splashscreen();
+	FsMenu::CrashReportWindow reporter(menu, crashes);
+	reporter.run<UI::Panel::Returncodes>();
+}
