@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2022 by the Widelands Development Team
+ * Copyright (C) 2012-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -47,9 +47,24 @@ static const std::string kDefaultHomedir = "%USERPROFILE%\\.widelands";
 static std::vector<Parameter> parameters;
 void fill_parameter_vector() {
 	i18n::Textdomain textdomain("widelands_console");
+
+	/** TRANSLATORS: Separator for alternative values for command line parameters */
+	const std::string alternatives_format = _("%1$s|%2$s");
+
+	// TODO(tothxa): Replace all uses and do this for true|false and others as well
+	const std::string filename_placeholder = _("FILENAME");
+
+	/** TRANSLATORS: Used instead of a file name indicating last savegame, replay or map.
+	    Use '_' instead of spaces if you need multiple words and don't use punctuation marks
+	 */
+	const std::string lastused_word = _("last");
+
+	const std::string file_or_last_placeholder =
+	   format(alternatives_format, filename_placeholder, lastused_word);
+
 	parameters =
 	{ {_("Usage:"), _("widelands <option0>=<value0> ... <optionN>=<valueN>"), "--", "", false},
-	  {"", _("widelands <save.wgf>/<replay.wrpl>"), "--", "", false},
+	  {"", _("widelands <save.wgf>/<replay.wry>"), "--", "", false},
 	  /// Paths
 	  {_("Options:"), "datadir", _("DIRNAME"),
 		_("Use the specified directory for the Widelands data files."), false},
@@ -76,19 +91,33 @@ void fill_parameter_vector() {
 		false},
 	  {"", "scenario", _("FILENAME"),
 		_("Start the map `FILENAME` directly as a singleplayer scenario."), false},
-	  {"", "loadgame", _("FILENAME"), _("Load the savegame `FILENAME` directly."), false},
-	  {"", "replay", _("FILENAME"), _("Load the replay `FILENAME` directly."), false},
+	  {"", "loadgame", file_or_last_placeholder,
+		/** TRANSLATORS: %1 is translation for FILENAME,
+		                 %2 is translation for "last" for last used file */
+		format(_("Load the savegame `%1$s` directly or the last saved game if `=%2$s` is used."),
+		       filename_placeholder, lastused_word),
+		false},
+	  {"", "replay", file_or_last_placeholder,
+		/** TRANSLATORS: %1 is translation for FILENAME,
+		                 %2 is translation for "last" for last used file */
+		format(_("Load the replay `%1$s` directly or the last saved replay if `=%2$s` is used."),
+		       filename_placeholder, lastused_word),
+		false},
+	  {"", "editor", "",
+		/** TRANSLATORS: %1 is translation for FILENAME,
+		                 %2 is translation for "last" for last used file */
+		format(_("Start the Widelands map editor directly. You can add `=%1$s` to directly load the "
+		         "map `FILENAME` in the editor or `=%2$s` to load the last edited map."),
+		       filename_placeholder, lastused_word),
+		false},
 	  {"", "script", _("FILENAME"),
 		_("Run the given Lua script after initialization. Only valid with --scenario, --loadgame, or "
 		  "--editor."),
 		false},
-	  {"", "editor", "",
-		_("Start the Widelands map editor directly. You can add `=FILENAME` to directly load the map "
-		  "`FILENAME` in the editor."),
-		false},
 	  /// Misc
 	  {"", "nosound", "", _("Start the game with sound disabled."), false},
 	  {"", "fail-on-lua-error", "", _("Force Widelands to crash when a Lua error occurs."), false},
+	  {"", "replay_lifetime", _("n"), _("Delete replays automatically after `n` weeks."), true},
 	  {"", "ai_training", "",
 		_("Enable AI training mode. See https://www.widelands.org/wiki/Ai%20Training/ for a full "
 		  "description of the AI training logic."),
@@ -97,6 +126,9 @@ void fill_parameter_vector() {
 		_("Constantly adjust the game speed automatically depending on AI delay. "
 		  "Only to be used for AI testing or training (in conjunction with --ai_training)."),
 		true},
+	  /* This is deliberately so long to discourage overusage */
+	  {"", "enable_development_testing_tools", "", _("Enable the Script Console and Cheating Mode."),
+		true},
 	  /// Saving options
 	  {_("Game options:"), _("Note: New values will be written to the config file."), "--", "",
 		false},
@@ -108,12 +140,28 @@ void fill_parameter_vector() {
 	  {"", "rolling_autosave", _("n"),
 		/** TRANSLATORS: `n` references a numerical placeholder */
 		_("Use `n` files for rolling autosaves."), true},
+	  /** TRANSLATORS: You may translate true/false, also as on/off or yes/no, but */
+	  /** TRANSLATORS: it HAS TO BE CONSISTENT with the translation in the widelands textdomain. */
+	  /** TRANSLATORS: * marks the default value */
+	  {"", "skip_autosave_on_inactivity", _("[true*|false]"),
+		_("Do not create an autosave when the user has been inactive since the last autosave."),
+		true},
 	  {"", "nozip", "", _("Do not save files as binary zip archives."), false},
+	  {"", "zip", "", _("Save files as binary zip archives."), false},
+	  // The below comment is duplicated from above for the other case, when false is the default.
+	  /** TRANSLATORS: You may translate true/false, also as on/off or yes/no, but */
+	  /** TRANSLATORS: it HAS TO BE CONSISTENT with the translation in the widelands textdomain. */
+	  /** TRANSLATORS: * marks the default value */
+	  {"", "save_chat_history", _("[true|false*]"),
+		_("Whether to save the history of sent chat messages to a file."), true},
 	  {"", "display_replay_filenames", _("[true*|false]"), _("Show filenames in the replay screen."),
 		true},
 	  {"", "editor_player_menu_warn_too_many_players", _("[true*|false]"),
 		_("Whether a warning should be shown in the editor if there are too many players."), true},
 	  /// Game options
+	  {"", "pause_game_on_inactivity", _("n"),
+		/** TRANSLATORS: `n` references a numerical placeholder */
+		_("Pause the game after `n` minutes of user inactivity."), true},
 	  {"", "auto_roadbuild_mode", _("[true*|false]"),
 		_("Start building a road after placing a flag."), true},
 	  {"", "display_flags",
@@ -128,11 +176,8 @@ void fill_parameter_vector() {
 	 true
 	},
 #endif
-	  {"", "edge_scrolling",
-		/** TRANSLATORS: You may translate true/false, also as on/off or yes/no, but */
-		/** TRANSLATORS: it HAS TO BE CONSISTENT with the translation in the widelands textdomain. */
-		/** TRANSLATORS: * marks the default value */
-		_("[true|false*]"), _("Scroll when the mouse cursor is near the screen edge."), true},
+	  {"", "edge_scrolling", _("[true|false*]"),
+		_("Scroll when the mouse cursor is near the screen edge."), true},
 	  {"", "invert_movement", _("[true|false*]"), _("Invert click-and-drag map movement direction."),
 		true},
 	  {"", "numpad_diagonalscrolling", _("[true|false*]"),
@@ -143,17 +188,8 @@ void fill_parameter_vector() {
 		_("Show in-game chat with transparent background."), true},
 	  {"", "toolbar_pos", _("[...]"), _("Bitmask to set the toolbar location and mode."), true},
 	  /// Networking
-	  {_("Networking:"), "write_syncstreams",
-		/** TRANSLATORS: You may translate true/false, also as on/off or yes/no, but */
-		/** TRANSLATORS: it HAS TO BE CONSISTENT with the translation in the widelands textdomain. */
-		/** TRANSLATORS: * marks the default value */
-		_("[true*|false]"),
-		/** TRANSLATORS: A syncstream is a synchronization stream. Syncstreams are used in multiplayer
-		 */
-		/** TRANSLATORS: games to make sure that there is no mismatch between the players. */
-		_("Create syncstream dump files to help debug network games."), false},
-	  {"", "metaserver", _("URI"), _("Connect to a different metaserver for internet gaming."),
-		false},
+	  {_("Networking:"), "metaserver", _("URI"),
+		_("Connect to a different metaserver for internet gaming."), false},
 	  {"", "metaserverport", _("n"),
 		/** TRANSLATORS: `n` references a numerical placeholder */
 		_("Port number `n` of the metaserver for internet gaming."), false},
@@ -168,12 +204,16 @@ void fill_parameter_vector() {
 		_("Connect to a different server address from the add-ons manager."), false},
 	  {"", "addon_server_port", _("n"),
 		_("Connect to a different server port from the add-ons manager."), false},
+	  {"", "write_syncstreams", "",
+		/** TRANSLATORS: A syncstream is a synchronization stream. Syncstreams are used in multiplayer
+		 */
+		/** TRANSLATORS: games to make sure that there is no mismatch between the players. */
+		_("Create syncstream dump files to help debug network games."), true},
 
 	  /// Interface options
-	  {_("Graphic options:"), "fullscreen", _("[true|false*]"),
-		_("Whether to use the whole display for the game screen."), false},
-	  {"", "maximized", _("[true|false*]"), _("Whether to start the game in a maximized window."),
+	  {_("Graphic options:"), "fullscreen", "", _("Use the whole display for the game screen."),
 		false},
+	  {"", "maximized", "", _("Start the game in a maximized window."), false},
 	  {"", "xres",
 		/** TRANSLATORS: A placeholder for window width */
 		_("x"),
@@ -184,14 +224,10 @@ void fill_parameter_vector() {
 		_("y"),
 		/** TRANSLATORS: `y` references a window height placeholder */
 		_("Height `y` of the window in pixel."), false},
-	  {"", "inputgrab", _("[true|false*]"), _("Whether to grab the mouse input."), true},
 	  {"", "sdl_cursor", _("[true*|false]"), _("Whether to use the mouse cursor provided by SDL."),
 		true},
 	  {"", "tooltip_accessibility_mode", _("[true|false*]"), _("Whether to use sticky tooltips."),
 		true},
-	  {"", "maxfps", _("n"),
-		/** TRANSLATORS: `n` references a numerical placeholder */
-		_("Maximal optical framerate `n` of the game."), true},
 	  {"", "theme", _("DIRNAME"),
 		_("The path to the active UI theme, relative to the Widelands home directory."), false},
 	  /// Window options
@@ -211,9 +247,6 @@ void fill_parameter_vector() {
 		_("Move a window to the edge of the panel when the edge of the window comes within "
 		  "a distance of `n` from the edge of the panel."),
 		true},
-	  {"", "snap_windows_only_when_overlapping", _("[true|false*]"),
-		_("Only move a window to the edge of a panel if the window is overlapping with the panel."),
-		true},
 	  /// Others
 	  {_("Others:"), "verbose", "", _("Enable verbose debug messages"), false},
 	  {"", "verbose-i18n", "",
@@ -223,23 +256,31 @@ void fill_parameter_vector() {
 	  {"", "version", "", _("Only print version and exit."), false},
 	  {"", "help", "", _("Show this help."), false},
 	  {"", "help-all", "", _("Show this help with all available config options."), false},
-	  {"", _("<save.wgf>/<replay.wrpl>"), "--",
-		_("Load the given savegame or replay directly. Useful for .wgf/.wrpl file extension "
+	  {"", _("<save.wgf>/<replay.wry>"), "--",
+		_("Load the given savegame or replay directly. Useful for .wgf/.wry file extension "
 		  "association. Does not work with other options. Also see --loadgame/--replay."),
 		false} };
 }
 
 const std::vector<std::string> get_all_parameters() {
-	std::vector<std::string> result(parameters.size());
-	std::transform(parameters.begin(), parameters.end(), result.begin(),
-	               [](const Parameter& p) { return p.key_; });
+	std::vector<std::string> result;
+	for (const Parameter& param : parameters) {
+		// Filter out special entries
+		if (param.hint_ != "--") {
+			result.emplace_back(param.key_);
+		}
+	}
 	return result;
 }
 
 bool is_parameter(const std::string& name) {
 	auto result = std::find_if(
 	   parameters.begin(), parameters.end(), [name](const Parameter& p) { return p.key_ == name; });
-	return result != parameters.end();
+	return result != parameters.end() && result->hint_ != "--";
+}
+
+bool use_last(const std::string& filename_arg) {
+	return i18n::is_translation_of(filename_arg, "last", "widelands_console");
 }
 
 /**
