@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,8 @@
 
 #include "base/macros.h"
 #include "base/multithreading.h"
+#include "economy/ferry_fleet.h"
+#include "economy/ship_fleet.h"
 #include "graphic/image.h"
 #include "graphic/rendertarget.h"
 #include "graphic/style_manager.h"
@@ -49,12 +51,17 @@ constexpr const char* const kImgMuteAll = "images/wui/buildings/menu_mute_all.pn
 constexpr const char* const kImgUnmuteAll = "images/wui/buildings/menu_unmute_all.png";
 
 BuildingWindow::BuildingWindow(InteractiveBase& parent,
-                               UI::UniqueWindow::Registry& reg,
+                               BuildingWindow::Registry& reg,
                                Widelands::Building& b,
                                const Widelands::BuildingDescr& descr,
                                bool avoid_fastclick)
-   : UI::UniqueWindow(
-        &parent, UI::WindowStyle::kWui, "building_window", &reg, Width, 0, b.descr().descname()),
+   : UI::UniqueWindow(&parent,
+                      UI::WindowStyle::kWui,
+                      format("building_window_%u", b.serial()),
+                      &reg,
+                      Width,
+                      0,
+                      b.descr().descname()),
      game_(parent.get_game()),
      is_dying_(false),
      parent_(&parent),
@@ -63,6 +70,7 @@ BuildingWindow::BuildingWindow(InteractiveBase& parent,
      building_position_(b.get_position()),
      showing_workarea_(false),
      avoid_fastclick_(avoid_fastclick),
+     priority_collapsed_(&reg.priority_collapsed),
      expeditionbtn_(nullptr),
      mute_this_(nullptr),
      mute_all_(nullptr) {
@@ -71,7 +79,7 @@ BuildingWindow::BuildingWindow(InteractiveBase& parent,
 }
 
 BuildingWindow::BuildingWindow(InteractiveBase& parent,
-                               UI::UniqueWindow::Registry& reg,
+                               BuildingWindow::Registry& reg,
                                Widelands::Building& b,
                                bool avoid_fastclick)
    : BuildingWindow(parent, reg, b, b.descr(), avoid_fastclick) {
@@ -89,6 +97,7 @@ void BuildingWindow::on_building_note(const Widelands::NoteBuilding& note) {
 		case Widelands::NoteBuilding::Action::kChanged:
 			if (!is_dying_) {
 				const std::string active_tab = tabs_->tabs()[tabs_->active()]->get_name();
+				save_position();
 				NoteThreadSafeFunction::instantiate(
 				   [this, active_tab]() {
 					   init(true, showing_workarea_);
@@ -117,18 +126,21 @@ void BuildingWindow::init(bool avoid_fastclick, bool workarea_preview_wanted) {
 	watch_button_ = nullptr;
 	avoid_fastclick_ = avoid_fastclick;
 
-	vbox_.reset(new UI::Box(this, UI::PanelStyle::kWui, 0, 0, UI::Box::Vertical));
+	vbox_.reset(new UI::Box(this, UI::PanelStyle::kWui, "vbox", 0, 0, UI::Box::Vertical));
+	set_center_panel(vbox_.get());  // Must be set immediately after deleting the old vbox, if any
 
-	tabs_ = new UI::TabPanel(vbox_.get(), UI::TabPanelStyle::kWuiLight);
+	tabs_ = new UI::TabPanel(vbox_.get(), UI::TabPanelStyle::kWuiLight, "tabs");
 	vbox_->add(tabs_, UI::Box::Resizing::kFullSize);
 
-	capsbuttons_ = new UI::Box(vbox_.get(), UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal);
+	setup_name_field_editbox(*vbox_);
+
+	capsbuttons_ =
+	   new UI::Box(vbox_.get(), UI::PanelStyle::kWui, "caps_buttons_box", 0, 0, UI::Box::Horizontal);
 	vbox_->add(capsbuttons_, UI::Box::Resizing::kFullSize);
 
 	// actually create buttons on the first call to think(),
 	// so that overriding create_capsbuttons() works
 
-	set_center_panel(vbox_.get());
 	set_thinks(true);
 	set_fastclick_panel(this);
 	if (workarea_preview_wanted) {
@@ -277,11 +289,29 @@ void BuildingWindow::create_capsbuttons(UI::Box* capsbuttons, Widelands::Buildin
 			stopbtn->sigclicked.connect([this]() { act_start_stop(); });
 			capsbuttons->add(stopbtn);
 
+			if (productionsite->descr().is_infinite_production_useful()) {
+				UI::Button* infbtn = new UI::Button(
+				   capsbuttons,
+				   productionsite->infinite_production() ? "end_produce_infinite" : "produce_infinite",
+				   0, 0, 34, 34, UI::ButtonStyle::kWuiMenu,
+				   g_image_cache->get((productionsite->infinite_production() ?
+                                      "images/wui/menus/end_infinity.png" :
+                                      "images/wui/menus/infinity.png")),
+				   productionsite->infinite_production() ?
+                  /** TRANSLATORS: Infinite Production toggle button for production sites. */
+                  _("Stop producing indefinitely") :
+                  /** TRANSLATORS: Infinite Production toggle button for production sites. */
+                  _("Produce indefinitely regardless of the economy’s needs"));
+				infbtn->sigclicked.connect([this]() { act_produce_infinite(); });
+				capsbuttons->add(infbtn);
+			}
+
 			// Add a fixed width separator rather than infinite space so the
 			// enhance/destroy/dismantle buttons are fixed in their position
 			// and not subject to the number of buttons on the right of the
 			// panel.
-			UI::Panel* spacer = new UI::Panel(capsbuttons, UI::PanelStyle::kWui, 0, 0, 17, 34);
+			UI::Panel* spacer =
+			   new UI::Panel(capsbuttons, UI::PanelStyle::kWui, "spacer_1", 0, 0, 17, 34);
 			capsbuttons->add(spacer);
 		}  // upcast to productionsite
 
@@ -346,7 +376,8 @@ void BuildingWindow::create_capsbuttons(UI::Box* capsbuttons, Widelands::Buildin
 		if (requires_destruction_separator && can_see) {
 			// Need this as well as the infinite space from the can_see section
 			// to ensure there is a separation.
-			UI::Panel* spacer = new UI::Panel(capsbuttons, UI::PanelStyle::kWui, 0, 0, 17, 34);
+			UI::Panel* spacer =
+			   new UI::Panel(capsbuttons, UI::PanelStyle::kWui, "spacer_2", 0, 0, 17, 34);
 			capsbuttons->add(spacer);
 			capsbuttons->add_inf_space();
 		}
@@ -500,6 +531,27 @@ void BuildingWindow::act_start_stop() {
 	}
 }
 
+/**
+===============
+Callback for toggling the production site's infinite production mode
+===============
+*/
+void BuildingWindow::act_produce_infinite() {
+	Widelands::Building* building = building_.get(parent_->egbase());
+	if (building == nullptr) {
+		return;
+	}
+
+	if (building->descr().type() >= Widelands::MapObjectType::PRODUCTIONSITE) {
+		if (game_ != nullptr) {
+			game_->send_player_toggle_infinite_production(*building);
+		} else {
+			Widelands::ProductionSite& ps = dynamic_cast<Widelands::ProductionSite&>(*building);
+			ps.set_infinite_production(!ps.infinite_production());
+		}
+	}
+}
+
 /**
 ===============
 Callback for starting an expedition request
@@ -585,6 +637,7 @@ Callback for debug window
 ===============
 */
 void BuildingWindow::act_debug() {
+	ibase()->broadcast_cheating_message();
 	show_field_debug(*ibase(), ibase()->egbase().map().get_fcoords(building_position_));
 }
 
@@ -609,7 +662,17 @@ void BuildingWindow::show_workarea() {
 	if (workarea_info->empty()) {
 		return;
 	}
-	ibase()->show_workarea(*workarea_info, building_position_);
+
+	std::set<Widelands::Coords> special_coords;
+	if (upcast(const Widelands::ProductionSite, ps, building); ps != nullptr) {
+		for (const Widelands::ShipFleetYardInterface* interface : ps->get_ship_fleet_interfaces()) {
+			special_coords.insert(interface->get_position());
+		}
+		for (const Widelands::FerryFleetYardInterface* interface : ps->get_ferry_fleet_interfaces()) {
+			special_coords.insert(interface->get_position());
+		}
+	}
+	ibase()->show_workarea(*workarea_info, building_position_, special_coords);
 	showing_workarea_ = true;
 
 	configure_workarea_button();
@@ -675,17 +738,24 @@ void BuildingWindow::update_expedition_button(bool expedition_was_canceled) {
 	expeditionbtn_->set_enabled(true);
 }
 
-constexpr uint16_t kCurrentPacketVersion = 1;
+constexpr uint16_t kCurrentPacketVersion = 2;
 UI::Window& BuildingWindow::load(FileRead& fr, InteractiveBase& ib) {
 	try {
 		const uint16_t packet_version = fr.unsigned_16();
-		if (packet_version == kCurrentPacketVersion) {
+		if (packet_version >= 1 && packet_version <= kCurrentPacketVersion) {
 			const int32_t x = fr.signed_32();
 			const int32_t y = fr.signed_32();
 			const bool workarea = fr.unsigned_8() != 0u;
 			BuildingWindow& bw = dynamic_cast<BuildingWindow&>(
 			   *ib.show_building_window(Widelands::Coords(x, y), true, workarea));
 			bw.tabs_->activate(fr.unsigned_8());
+			// TODO(tothxa): Savegame compatibility v1.2
+			if (packet_version >= 2) {
+				bw.set_priority_collapsed(static_cast<BuildingWindow::CollapsedState>(fr.unsigned_8()));
+			} else {
+				// Shouldn't be necessary, but making it explicit only makes it safer.
+				bw.set_priority_collapsed(BuildingWindow::CollapsedState::kExpanded);
+			}
 			return bw;
 		}
 		throw Widelands::UnhandledVersionError(
@@ -701,4 +771,5 @@ void BuildingWindow::save(FileWrite& fw, Widelands::MapObjectSaver& /* mos */) c
 	fw.signed_32(building_position_.y);
 	fw.unsigned_8(showing_workarea_ ? 1 : 0);
 	fw.unsigned_8(tabs_->active());
+	fw.unsigned_8(static_cast<uint8_t>(*priority_collapsed_));
 }
