@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +27,8 @@
 #include "graphic/style_manager.h"
 #include "logic/cmd_queue.h"
 #include "logic/map_objects/checkstep.h"
+#include "logic/map_objects/pinned_note.h"
+#include "logic/map_objects/tribes/ship.h"
 #include "logic/map_objects/tribes/soldier.h"
 #include "logic/map_objects/tribes/tribe_descr.h"
 #include "logic/map_objects/tribes/warehouse.h"
@@ -36,10 +38,12 @@
 #include "ui_basic/box.h"
 #include "ui_basic/button.h"
 #include "ui_basic/icongrid.h"
+#include "ui_basic/spinbox.h"
 #include "ui_basic/tabpanel.h"
 #include "ui_basic/unique_window.h"
 #include "wui/actionconfirm.h"
 #include "wui/economy_options_window.h"
+#include "wui/fleet_options_window.h"
 #include "wui/game_debug_ui.h"
 #include "wui/interactive_player.h"
 #include "wui/waresdisplay.h"
@@ -49,7 +53,6 @@ namespace Widelands {
 class BuildingDescr;
 }  // namespace Widelands
 using Widelands::Building;
-using Widelands::EditorGameBase;
 using Widelands::Game;
 
 constexpr int kBuildGridCellSize = 50;
@@ -73,7 +76,14 @@ private:
 };
 
 BuildGrid::BuildGrid(UI::Panel* parent, Widelands::Player* plr, int32_t x, int32_t y, int32_t cols)
-   : UI::IconGrid(parent, UI::PanelStyle::kWui, x, y, kBuildGridCellSize, kBuildGridCellSize, cols),
+   : UI::IconGrid(parent,
+                  UI::PanelStyle::kWui,
+                  "build_grid",
+                  x,
+                  y,
+                  kBuildGridCellSize,
+                  kBuildGridCellSize,
+                  cols),
      plr_(plr) {
 	icon_clicked.connect([this](Widelands::DescriptionIndex i) { click_slot(i); });
 	mouseout.connect([this](Widelands::DescriptionIndex i) { mouseout_slot(i); });
@@ -164,6 +174,8 @@ public:
 	void act_debug();
 	void act_buildflag();
 	void act_configure_economy();
+	void act_configure_ship_fleet();
+	void act_configure_ferry_fleet();
 	void act_ripflag();
 	void act_buildroad();
 	void act_abort_buildroad();
@@ -172,6 +184,7 @@ public:
 	void act_buildwaterway();
 	void act_abort_buildwaterway();
 	void act_abort_buildwaterway_and_start_buildroad();
+	void act_goto_buildroadstart();
 	void act_removewaterway();
 	void act_build(Widelands::DescriptionIndex /* idx */);
 	void building_icon_mouse_out(Widelands::DescriptionIndex /* idx */);
@@ -180,6 +193,7 @@ public:
 	void act_scout();
 	void act_mark_removal();
 	void act_unmark_removal();
+	void act_pinned_note();
 
 private:
 	uint32_t add_tab(const std::string& name,
@@ -203,9 +217,9 @@ private:
 	Widelands::FCoords node_;
 
 	UI::TabPanel tabpanel_;
-	bool fastclick_;  // if true, put the mouse over first button in first tab
-	uint32_t best_tab_;
-	bool showing_workarea_preview_;
+	bool fastclick_{true};  // if true, put the mouse over first button in first tab
+	uint32_t best_tab_{0U};
+	bool showing_workarea_preview_{false};
 	std::set<Widelands::Coords> overlapping_workareas_;
 	bool is_showing_workarea_overlaps_;
 	Widelands::DescriptionIndex building_under_mouse_;
@@ -235,11 +249,13 @@ constexpr const char* const kImgButtonRipFlag = "images/wui/fieldaction/menu_rip
 constexpr const char* const kImgButtonWatchField = "images/wui/fieldaction/menu_watch_field.png";
 constexpr const char* const kImgDebug = "images/wui/fieldaction/menu_debug.png";
 constexpr const char* const kImgButtonAbort = "images/wui/menu_abort.png";
+constexpr const char* const kImgButtonGoto = "images/wui/menus/goto.png";
 constexpr const char* const kImgButtonGeologist = "images/wui/fieldaction/menu_geologist.png";
 constexpr const char* const kImgButtonScout = "images/wui/menus/watch_follow.png";
 constexpr const char* const kImgButtonMarkRemoval = "images/wui/fieldaction/menu_mark_removal.png";
 constexpr const char* const kImgButtonUnmarkRemoval =
    "images/wui/fieldaction/menu_unmark_removal.png";
+constexpr const char* const kImgButtonPinnedNote = "images/wui/fieldaction/pinned_note.png";
 
 constexpr const char* const kImgTabTarget = "images/wui/fieldaction/menu_tab_target.png";
 
@@ -255,10 +271,8 @@ FieldActionWindow::FieldActionWindow(InteractiveBase* const ib,
      player_(plr),
      map_(ib->egbase().map()),
      node_(ib->get_sel_pos().node, &map_[ib->get_sel_pos().node]),
-     tabpanel_(this, UI::TabPanelStyle::kWuiDark),
-     fastclick_(true),
-     best_tab_(0),
-     showing_workarea_preview_(false),
+     tabpanel_(this, UI::TabPanelStyle::kWuiDark, "tabs"),
+
      is_showing_workarea_overlaps_(ib->get_display_flag(InteractiveBase::dfShowWorkareaOverlap)),
      building_under_mouse_(Widelands::INVALID_INDEX) {
 	ib->set_sel_freeze(true);
@@ -365,23 +379,25 @@ Add the buttons you normally get when clicking on a field.
 */
 void FieldActionWindow::add_buttons_auto() {
 	UI::Box* buildbox = nullptr;
-	UI::Box& watchbox = *new UI::Box(&tabpanel_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal);
+	UI::Box& watchbox =
+	   *new UI::Box(&tabpanel_, UI::PanelStyle::kWui, "watch_box", 0, 0, UI::Box::Horizontal);
 
 	upcast(InteractiveGameBase, igbase, &ibase());
+	upcast(InteractivePlayer, ipl, igbase);
 
-	if (upcast(InteractivePlayer, ipl, igbase)) {
+	if (ipl != nullptr) {
 		// Target immovables for removal by workers
 		if (upcast(const Widelands::Immovable, mo, map_.get_immovable(node_))) {
 			if (mo->is_marked_for_removal(ipl->player_number())) {
-				UI::Box& box =
-				   *new UI::Box(&tabpanel_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal);
+				UI::Box& box = *new UI::Box(&tabpanel_, UI::PanelStyle::kWui, "immovable_actions_box",
+				                            0, 0, UI::Box::Horizontal);
 				add_button(&box, "unmark_for_removal", kImgButtonUnmarkRemoval,
 				           &FieldActionWindow::act_unmark_removal,
 				           _("Marked for removal by a worker – click to unmark"));
 				add_tab("target", kImgTabTarget, &box, _("Immovable Actions"));
 			} else if (suited_for_targeting(ipl->player_number(), ipl->egbase(), *mo)) {
-				UI::Box& box =
-				   *new UI::Box(&tabpanel_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal);
+				UI::Box& box = *new UI::Box(&tabpanel_, UI::PanelStyle::kWui, "immovable_actions_box",
+				                            0, 0, UI::Box::Horizontal);
 				add_button(&box, "mark_for_removal", kImgButtonMarkRemoval,
 				           &FieldActionWindow::act_mark_removal,
 				           _("Mark this immovable for timely removal by a suited worker"));
@@ -399,7 +415,8 @@ void FieldActionWindow::add_buttons_auto() {
 		const bool can_act = igbase != nullptr ? igbase->can_act(owner) : true;
 
 		// The box with road-building buttons
-		buildbox = new UI::Box(&tabpanel_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal);
+		buildbox =
+		   new UI::Box(&tabpanel_, UI::PanelStyle::kWui, "build_box", 0, 0, UI::Box::Horizontal);
 
 		if (upcast(Widelands::Flag, flag, imm)) {
 			// Add flag actions
@@ -488,15 +505,50 @@ void FieldActionWindow::add_buttons_auto() {
 		}
 	}
 
+	// Fleet target settings
+	bool has_ship_fleet = false;
+	bool has_ferry_fleet = false;
+	for (Widelands::Bob* bob = node_.field->get_first_bob(); bob != nullptr;
+	     bob = bob->get_next_bob()) {
+		if (bob->descr().type() == Widelands::MapObjectType::SHIP_FLEET_YARD_INTERFACE &&
+		    !has_ship_fleet &&
+		    (ipl == nullptr || bob->owner().player_number() == ipl->player_number())) {
+			if (buildbox == nullptr) {
+				buildbox = new UI::Box(
+				   &tabpanel_, UI::PanelStyle::kWui, "build_box", 0, 0, UI::Box::Horizontal);
+			}
+			has_ship_fleet = true;
+			add_button(
+			   buildbox, "configure_ship_fleet", "images/wui/fieldaction/menu_tab_ship_targets.png",
+			   &FieldActionWindow::act_configure_ship_fleet, _("Configure this ocean’s ship fleet"));
+		} else if (bob->descr().type() == Widelands::MapObjectType::FERRY_FLEET_YARD_INTERFACE &&
+		           !has_ferry_fleet &&
+		           (ipl == nullptr || bob->owner().player_number() == ipl->player_number())) {
+			if (buildbox == nullptr) {
+				buildbox = new UI::Box(
+				   &tabpanel_, UI::PanelStyle::kWui, "build_box", 0, 0, UI::Box::Horizontal);
+			}
+			has_ferry_fleet = true;
+			add_button(
+			   buildbox, "configure_ferry_fleet", "images/wui/fieldaction/menu_tab_ferry_targets.png",
+			   &FieldActionWindow::act_configure_ferry_fleet, _("Configure this ocean’s ferry fleet"));
+		}
+	}
+
 	//  Watch actions, only when in game (no use in editor).
 	if (ibase().egbase().is_game()) {
 		add_button(&watchbox, "watch", kImgButtonWatchField, &FieldActionWindow::act_watch,
 		           _("Watch field in a separate window"));
 	}
 
+	if (ipl != nullptr) {
+		add_button(&watchbox, "pinned_note", kImgButtonPinnedNote,
+		           &FieldActionWindow::act_pinned_note, _("Pinned note"));
+	}
+
 	if (ibase().get_display_flag(InteractiveBase::dfDebug)) {
 		add_button(
-		   &watchbox, "debug", kImgDebug, &FieldActionWindow::act_debug, _("Show Debug Window"));
+		   &watchbox, "debug", kImgDebug, &FieldActionWindow::act_debug, _("Show debug window"));
 	}
 
 	// Add tabs
@@ -631,7 +683,8 @@ Buttons used during road building: Set flag here and Abort
 ===============
 */
 void FieldActionWindow::add_buttons_road(bool flag) {
-	UI::Box& buildbox = *new UI::Box(&tabpanel_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal);
+	UI::Box& buildbox =
+	   *new UI::Box(&tabpanel_, UI::PanelStyle::kWui, "build_box", 0, 0, UI::Box::Horizontal);
 
 	if (flag) {
 		add_button(&buildbox, "build_flag", kImgButtonBuildFlag, &FieldActionWindow::act_buildflag,
@@ -640,6 +693,9 @@ void FieldActionWindow::add_buttons_road(bool flag) {
 
 	add_button(&buildbox, "cancel_road", kImgButtonAbort, &FieldActionWindow::act_abort_buildroad,
 	           _("Cancel road"));
+	add_button(&buildbox, "goto_buildroadstart", kImgButtonGoto,
+	           &FieldActionWindow::act_goto_buildroadstart, _("Go to road"));
+
 	const Widelands::Map& map = ibase().egbase().map();
 	if (map.get_waterway_max_length() >= 2 &&
 	    Widelands::CheckStepFerry(ibase().egbase())
@@ -654,7 +710,8 @@ void FieldActionWindow::add_buttons_road(bool flag) {
 }
 
 void FieldActionWindow::add_buttons_waterway(bool flag) {
-	UI::Box& buildbox = *new UI::Box(&tabpanel_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal);
+	UI::Box& buildbox =
+	   *new UI::Box(&tabpanel_, UI::PanelStyle::kWui, "build_box", 0, 0, UI::Box::Horizontal);
 
 	if (flag) {
 		add_button(&buildbox, "build_flag", kImgButtonBuildFlag, &FieldActionWindow::act_buildflag,
@@ -663,6 +720,8 @@ void FieldActionWindow::add_buttons_waterway(bool flag) {
 
 	add_button(&buildbox, "cancel_waterway", kImgButtonAbort,
 	           &FieldActionWindow::act_abort_buildwaterway, _("Cancel waterway"));
+	add_button(&buildbox, "goto_buildwaterwaystart", kImgButtonGoto,
+	           &FieldActionWindow::act_goto_buildroadstart, _("Go to waterway"));
 	add_button(&buildbox, "cancel_waterway_build_road", kImgButtonBuildRoad,
 	           &FieldActionWindow::act_abort_buildwaterway_and_start_buildroad,
 	           _("Cancel waterway and start building road"));
@@ -722,12 +781,23 @@ void FieldActionWindow::act_watch() {
 	reset_mouse_and_die();
 }
 
+/*
+===============
+Open the notes editor for the given field and delete self.
+===============
+*/
+void FieldActionWindow::act_pinned_note() {
+	dynamic_cast<InteractivePlayer&>(ibase()).edit_pinned_note(node_);
+	reset_mouse_and_die();
+}
+
 /*
 ===============
 Show a debug widow for this field.
 ===============
 */
 void FieldActionWindow::act_debug() {
+	ibase().broadcast_cheating_message();
 	show_field_debug(ibase(), node_);
 }
 
@@ -766,6 +836,32 @@ void FieldActionWindow::act_configure_economy() {
 	die();
 }
 
+void FieldActionWindow::act_configure_ship_fleet() {
+	upcast(InteractivePlayer, ipl, &ibase());
+	for (Widelands::Bob* bob = node_.field->get_first_bob(); bob != nullptr;
+	     bob = bob->get_next_bob()) {
+		if (bob->descr().type() == Widelands::MapObjectType::SHIP_FLEET_YARD_INTERFACE &&
+		    (ipl == nullptr || bob->owner().player_number() == ipl->player_number())) {
+			FleetOptionsWindow::create(get_parent(), ibase(), bob);
+			break;
+		}
+	}
+	die();
+}
+
+void FieldActionWindow::act_configure_ferry_fleet() {
+	upcast(InteractivePlayer, ipl, &ibase());
+	for (Widelands::Bob* bob = node_.field->get_first_bob(); bob != nullptr;
+	     bob = bob->get_next_bob()) {
+		if (bob->descr().type() == Widelands::MapObjectType::FERRY_FLEET_YARD_INTERFACE &&
+		    (ipl == nullptr || bob->owner().player_number() == ipl->player_number())) {
+			FleetOptionsWindow::create(get_parent(), ibase(), bob);
+			break;
+		}
+	}
+	die();
+}
+
 /*
 ===============
 Remove the flag at this field
@@ -837,6 +933,17 @@ void FieldActionWindow::act_abort_buildwaterway() {
 	reset_mouse_and_die();
 }
 
+void FieldActionWindow::act_goto_buildroadstart() {
+	if (!ibase().in_road_building_mode()) {
+		return;
+	}
+	ibase().map_view()->scroll_to_field((SDL_GetModState() & KMOD_CTRL) != 0 ?
+                                          ibase().get_build_road_end() :
+                                          ibase().get_build_road_start(),
+	                                    MapView::Transition::Smooth);
+	reset_mouse_and_die();
+}
+
 void FieldActionWindow::act_abort_buildroad_and_start_buildwaterway() {
 	if (!ibase().in_road_building_mode(RoadBuildingType::kRoad) ||
 	    ibase().in_road_building_mode(RoadBuildingType::kWaterway)) {
@@ -916,7 +1023,7 @@ void FieldActionWindow::building_icon_mouse_in(const Widelands::DescriptionIndex
 		building_under_mouse_ = idx;
 		const Widelands::BuildingDescr& descr = *player_->tribe().get_building_descr(idx);
 		const WorkareaInfo& workarea_info = descr.workarea_info();
-		ibase().show_workarea(workarea_info, node_);
+		ibase().show_workarea(workarea_info, node_, {});
 		showing_workarea_preview_ = true;
 		if (!is_showing_workarea_overlaps_) {
 			return;
@@ -1007,7 +1114,7 @@ void FieldActionWindow::building_icon_mouse_in(const Widelands::DescriptionIndex
 								colors[t] = kOverlapColorPale;
 							}
 						}
-						ibase().show_workarea(wa, mr.location(), colors);
+						ibase().show_workarea(wa, mr.location(), colors, {});
 						overlapping_workareas_.insert(mr.location());
 					}
 				}
@@ -1085,7 +1192,7 @@ void show_field_action(InteractiveBase* const ibase,
 		}
 
 		// append or take away from the road
-		if (!ibase->append_build_road(target) ||
+		if (!ibase->append_build_road(target, false) ||
 		    (ibase->in_road_building_mode(RoadBuildingType::kWaterway) &&
 		     target != ibase->get_build_road_end())) {
 			FieldActionWindow& w = *new FieldActionWindow(ibase, player, registry);
@@ -1132,3 +1239,143 @@ void show_field_action(InteractiveBase* const ibase,
 		return w.init();
 	}
 }
+
+// Ship selection window implementation
+
+class ShipSelectionWindow : public UI::UniqueWindow {
+	static constexpr int kPadding = 4;
+	static constexpr int kListHeight = 200;
+
+	static inline std::string ship_description(const Widelands::Ship* ship, bool hostile) {
+		std::string status;
+		if (ship->get_ship_type() == Widelands::ShipType::kWarship) {
+			status = format(
+			   /** TRANSLATORS: Placeholders are the hitpoints and the attack bonus of the warship. */
+			   pgettext("ship_state", "Warship · HP %1$u%% · AT +%2$u%%"),
+			   100U * ship->get_hitpoints() / ship->descr().max_hitpoints_,
+			   ship->get_sea_attack_soldier_bonus(ship->owner().egbase()));
+		} else if (hostile) {
+			status = pgettext("ship_state", "Enemy Transport Ship");
+		} else if (ship->state_is_expedition()) {
+			status = pgettext("ship_state", "Expedition");
+		} else {
+			status = pgettext("ship_state", "Transport");
+		}
+		return format(_("%1$s (%2$s)"), ship->get_shipname(), status);
+	}
+
+public:
+	ShipSelectionWindow(InteractiveBase* ibase,
+	                    UI::UniqueWindow::Registry* registry,
+	                    const Widelands::Coords& node,
+	                    const std::vector<Widelands::Ship*>& manageable,
+	                    const std::vector<Widelands::Ship*>& attackable)
+	   : UI::UniqueWindow(
+	        ibase, UI::WindowStyle::kWui, "ship_selection", registry, 0, 0, _("Select Ship")),
+	     ibase_(*ibase),
+	     node_(node),
+	     hbox_(this, UI::PanelStyle::kWui, "hbox", 0, 0, UI::Box::Horizontal),
+	     box_manageable_(&hbox_, UI::PanelStyle::kWui, "manageable_box", 0, 0, UI::Box::Vertical),
+	     box_attackable_(&hbox_, UI::PanelStyle::kWui, "attackable_box", 0, 0, UI::Box::Vertical),
+	     list_manageable_(&box_manageable_, "manageable_list", 0, 0, 0, 0, UI::PanelStyle::kWui),
+	     list_attackable_(&box_attackable_, "attackable_list", 0, 0, 0, 0, UI::PanelStyle::kWui) {
+		for (Widelands::Ship* ship : manageable) {
+			list_manageable_.add(ship_description(ship, false), ship, ship->descr().icon());
+		}
+		for (Widelands::Ship* ship : attackable) {
+			list_attackable_.add(ship_description(ship, true), ship, ship->descr().icon());
+		}
+
+		const int listw = std::max(
+		   list_manageable_.calculate_desired_width(), list_attackable_.calculate_desired_width());
+		list_manageable_.set_desired_size(listw, kListHeight);
+		list_attackable_.set_desired_size(listw, kListHeight);
+
+		if (ibase_.get_player() != nullptr) {
+			box_manageable_.add(
+			   new UI::Textarea(&box_manageable_, UI::PanelStyle::kWui, "label_manageable",
+			                    UI::FontStyle::kWuiInfoPanelHeading, _("Own Ships"),
+			                    UI::Align::kCenter),
+			   UI::Box::Resizing::kFullSize);
+			box_manageable_.add_space(kPadding);
+		}
+#ifndef NDEBUG
+		else {
+			assert(attackable.empty());
+		}
+#endif
+		box_manageable_.add(&list_manageable_, UI::Box::Resizing::kExpandBoth);
+
+		box_attackable_.add(new UI::Textarea(&box_attackable_, UI::PanelStyle::kWui,
+		                                     "label_attackable", UI::FontStyle::kWuiInfoPanelHeading,
+		                                     _("Attackable Ships"), UI::Align::kCenter),
+		                    UI::Box::Resizing::kFullSize);
+		box_attackable_.add_space(kPadding);
+		box_attackable_.add(&list_attackable_, UI::Box::Resizing::kExpandBoth);
+
+		hbox_.add(&box_manageable_, UI::Box::Resizing::kFullSize);
+		if (manageable.empty()) {
+			box_manageable_.set_visible(false);
+		} else if (attackable.empty()) {
+			box_attackable_.set_visible(false);
+		} else {
+			hbox_.add_space(kPadding);
+		}
+		hbox_.add(&box_attackable_, UI::Box::Resizing::kFullSize);
+
+		list_manageable_.set_select_with_wheel(false);
+		list_attackable_.set_select_with_wheel(false);
+
+		list_manageable_.selected.connect([this](uint32_t index) { clicked_manageable(index); });
+		list_attackable_.selected.connect([this](uint32_t index) { clicked_attackable(index); });
+		list_manageable_.double_clicked.connect(
+		   [this](uint32_t index) { clicked_manageable(index); });
+		list_attackable_.double_clicked.connect(
+		   [this](uint32_t index) { clicked_attackable(index); });
+
+		set_center_panel(&hbox_);
+		set_fastclick_panel(manageable.empty() ? &list_attackable_ : &list_manageable_);
+		center_to_parent();
+		move_out_of_the_way();
+		warp_mouse_to_fastclick_panel();
+		initialization_complete();
+	}
+
+	void clicked_manageable(uint32_t index) {
+		Widelands::Ship* ship = list_manageable_[index].get(ibase_.egbase());
+		if (ship != nullptr) {
+			ibase_.show_ship_window(ship);
+		}
+		if ((SDL_GetModState() & KMOD_CTRL) == 0) {
+			die();
+		}
+	}
+
+	void clicked_attackable(uint32_t index) {
+		Widelands::Ship* ship = list_attackable_[index].get(ibase_.egbase());
+		if (ship != nullptr) {
+			ibase_.show_attack_window(node_, ship, true);
+		}
+		if ((SDL_GetModState() & KMOD_CTRL) == 0) {
+			die();
+		}
+	}
+
+private:
+	InteractiveBase& ibase_;
+	Widelands::Coords node_;
+
+	UI::Box hbox_;
+	UI::Box box_manageable_;
+	UI::Box box_attackable_;
+	UI::Listselect<Widelands::OPtr<Widelands::Ship>> list_manageable_;
+	UI::Listselect<Widelands::OPtr<Widelands::Ship>> list_attackable_;
+};
+
+void show_ship_selection_window(InteractiveBase* ibase,
+                                UI::UniqueWindow::Registry* registry,
+                                const Widelands::Coords& node,
+                                const std::vector<Widelands::Ship*>& manageable,
+                                const std::vector<Widelands::Ship*>& attackable) {
+	new ShipSelectionWindow(ibase, registry, node, manageable, attackable);
+}
