@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,8 +33,8 @@
 #include "ui_basic/slider.h"
 #include "wui/interactive_player.h"
 
-#define PLOT_HEIGHT 145
-#define NR_BASE_DATASETS 11
+constexpr int kPlotHeight = 145;
+constexpr int kNrBaseDatasets = 15;
 
 GeneralStatisticsMenu::GeneralStatisticsMenu(InteractiveGameBase& parent,
                                              GeneralStatisticsMenu::Registry& registry)
@@ -46,16 +46,18 @@ GeneralStatisticsMenu::GeneralStatisticsMenu(InteractiveGameBase& parent,
                       400,
                       _("General Statistics")),
      my_registry_(&registry),
-     box_(this, UI::PanelStyle::kWui, 0, 0, UI::Box::Vertical, 0, 0, 5),
-     player_buttons_box_(&box_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal, 0, 0, 1),
+     box_(this, UI::PanelStyle::kWui, "main_box", 0, 0, UI::Box::Vertical, 0, 0, 5),
+     player_buttons_box_(
+        &box_, UI::PanelStyle::kWui, "buttons_box", 0, 0, UI::Box::Horizontal, 0, 0, 1),
      plot_(&box_,
+           "plot",
            0,
            0,
            430,
-           PLOT_HEIGHT,
+           kPlotHeight,
            Widelands::kStatisticsSampleTime.get(),
            WuiPlotArea::Plotmode::kAbsolute),
-     selected_information_(0),
+
      game_(*parent.get_game()),
      subscriber_(Notifications::subscribe<Widelands::NotePlayerDetailsEvent>(
         [this](const Widelands::NotePlayerDetailsEvent& note) {
@@ -83,7 +85,7 @@ GeneralStatisticsMenu::GeneralStatisticsMenu(InteractiveGameBase& parent,
 	}
 
 	// Is there a hook dataset?
-	ndatasets_ = NR_BASE_DATASETS;
+	ndatasets_ = kNrBaseDatasets;
 	std::unique_ptr<LuaTable> hook = game_.lua().get_hook("custom_statistic");
 	std::string cs_name;
 	std::string cs_pic;
@@ -101,20 +103,29 @@ GeneralStatisticsMenu::GeneralStatisticsMenu(InteractiveGameBase& parent,
                                              // The plot is always invisible if this player doesn't
                                              // exist, but we need to assign a color anyway
                                              kPlayerColors[i];
-		plot_.register_plot_data(i * ndatasets_ + 0, &genstats[i].land_size, color);
-		plot_.register_plot_data(i * ndatasets_ + 1, &genstats[i].nr_workers, color);
-		plot_.register_plot_data(i * ndatasets_ + 2, &genstats[i].nr_buildings, color);
-		plot_.register_plot_data(i * ndatasets_ + 3, &genstats[i].nr_wares, color);
-		plot_.register_plot_data(i * ndatasets_ + 4, &genstats[i].productivity, color);
-		plot_.register_plot_data(i * ndatasets_ + 5, &genstats[i].nr_casualties, color);
-		plot_.register_plot_data(i * ndatasets_ + 6, &genstats[i].nr_kills, color);
-		plot_.register_plot_data(i * ndatasets_ + 7, &genstats[i].nr_msites_lost, color);
-		plot_.register_plot_data(i * ndatasets_ + 8, &genstats[i].nr_msites_defeated, color);
-		plot_.register_plot_data(i * ndatasets_ + 9, &genstats[i].nr_civil_blds_lost, color);
-		plot_.register_plot_data(i * ndatasets_ + 10, &genstats[i].miltary_strength, color);
+
+		unsigned o = 0;
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].land_size, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_workers, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_buildings, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_wares, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].productivity, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_ships, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_naval_losses, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_naval_victories, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_casualties, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_kills, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_msites_lost, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_msites_defeated, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_civil_blds_lost, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].nr_civil_blds_defeated, color);
+		plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].miltary_strength, color);
 		if (hook) {
-			plot_.register_plot_data(i * ndatasets_ + 11, &genstats[i].custom_statistic, color);
+			plot_.register_plot_data(i * ndatasets_ + o++, &genstats[i].custom_statistic, color);
 		}
+
+		assert(o == ndatasets_);
+
 		if (game_.get_player(i + 1) != nullptr) {  // Show area plot
 			plot_.show_plot(i * ndatasets_ + selected_information_, my_registry_->selected_players[i]);
 		}
@@ -128,70 +139,103 @@ GeneralStatisticsMenu::GeneralStatisticsMenu(InteractiveGameBase& parent,
 	create_player_buttons();
 	box_.add(&player_buttons_box_, UI::Box::Resizing::kFullSize);
 
-	UI::Box* hbox2 = new UI::Box(&box_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal, 0, 0, 1);
+	UI::Box* hbox2 =
+	   new UI::Box(&box_, UI::PanelStyle::kWui, "hbox", 0, 0, UI::Box::Horizontal, 0, 0, 1);
 
 	UI::Radiobutton* btn;
 
 	const Vector2i zero = Vector2i::zero();
 
-	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, zero,
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "landsize", zero,
 	                       g_image_cache->get("images/wui/stats/genstats_landsize.png"), _("Land"),
 	                       &btn);
 	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 
-	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, zero,
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "workers", zero,
 	                       g_image_cache->get("images/wui/stats/genstats_nrworkers.png"),
 	                       _("Workers"), &btn);
 	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 
-	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, zero,
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "buildings", zero,
 	                       g_image_cache->get("images/wui/stats/genstats_nrbuildings.png"),
 	                       _("Buildings"), &btn);
 	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 
-	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, zero,
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "wares", zero,
 	                       g_image_cache->get("images/wui/stats/genstats_nrwares.png"), _("Wares"),
 	                       &btn);
 	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 
-	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, zero,
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "productivity", zero,
 	                       g_image_cache->get("images/wui/stats/genstats_productivity.png"),
 	                       _("Productivity"), &btn);
 	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 
-	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, zero,
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "ships", zero,
+	                       g_image_cache->get("images/wui/stats/genstats_ships.png"), _("Ships"),
+	                       &btn);
+	if (parent.egbase().map().allows_seafaring()) {
+		hbox2->add(btn, UI::Box::Resizing::kFillSpace);
+	} else {
+		btn->set_visible(false);
+	}
+
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "naval_losses", zero,
+	                       g_image_cache->get("images/wui/stats/genstats_naval_losses.png"),
+	                       _("Warships lost"), &btn);
+	if (parent.egbase().map().allows_seafaring()) {
+		hbox2->add(btn, UI::Box::Resizing::kFillSpace);
+	} else {
+		btn->set_visible(false);
+	}
+
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "naval_victories", zero,
+	                       g_image_cache->get("images/wui/stats/genstats_naval_victories.png"),
+	                       _("Warships defeated"), &btn);
+	if (parent.egbase().map().allows_seafaring()) {
+		hbox2->add(btn, UI::Box::Resizing::kFillSpace);
+	} else {
+		btn->set_visible(false);
+	}
+
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "casualties", zero,
 	                       g_image_cache->get("images/wui/stats/genstats_casualties.png"),
 	                       _("Casualties"), &btn);
 	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 
-	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, zero,
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "kills", zero,
 	                       g_image_cache->get("images/wui/stats/genstats_kills.png"), _("Kills"),
 	                       &btn);
 	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 
-	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, zero,
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "military_lost", zero,
 	                       g_image_cache->get("images/wui/stats/genstats_msites_lost.png"),
 	                       _("Military buildings lost"), &btn);
 	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 
-	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, zero,
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "military_defeated", zero,
 	                       g_image_cache->get("images/wui/stats/genstats_msites_defeated.png"),
 	                       _("Military buildings defeated"), &btn);
 	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 
-	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, zero,
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "civil_lost", zero,
 	                       g_image_cache->get("images/wui/stats/genstats_civil_blds_lost.png"),
 	                       _("Civilian buildings lost"), &btn);
 	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 
-	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, zero,
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "civil_defeated", zero,
+	                       g_image_cache->get("images/wui/stats/genstats_civil_blds_defeated.png"),
+	                       _("Civilian buildings destroyed"), &btn);
+	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
+
+	radiogroup_.add_button(hbox2, UI::PanelStyle::kWui, "strength", zero,
 	                       g_image_cache->get("images/wui/stats/genstats_militarystrength.png"),
 	                       _("Military"), &btn);
 	hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 
 	if (hook) {
 		radiogroup_.add_button(
-		   hbox2, UI::PanelStyle::kWui, zero, g_image_cache->get(cs_pic), cs_name, &btn);
+		   hbox2, UI::PanelStyle::kWui, "hook", zero, g_image_cache->get(cs_pic), cs_name, &btn);
 		hbox2->add(btn, UI::Box::Resizing::kFillSpace);
 	}
 
@@ -200,7 +244,7 @@ GeneralStatisticsMenu::GeneralStatisticsMenu(InteractiveGameBase& parent,
 
 	box_.add(hbox2, UI::Box::Resizing::kFullSize);
 
-	slider_ = new WuiPlotAreaSlider(&box_, plot_, 0, 0, 100, 45);
+	slider_ = new WuiPlotAreaSlider(&box_, "slider", plot_, 0, 0, 100, 45);
 	slider_->changedto.connect([this](int32_t i) { plot_.set_time_id(i); });
 	box_.add(slider_, UI::Box::Resizing::kFullSize);
 
