@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2022 by the Widelands Development Team
+ * Copyright (C) 2020-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,14 +18,12 @@
 
 #include "wui/info_panel.h"
 
-#include <iomanip>
 #include <memory>
-#include <sstream>
 
 #include <SDL_timer.h>
 
+#include "base/time_string.h"
 #include "graphic/font_handler.h"
-#include "graphic/style_manager.h"
 #include "graphic/text_layout.h"
 #include "logic/message_queue.h"
 #include "wlapplication_options.h"
@@ -34,13 +32,12 @@
 
 constexpr int8_t kSpacing = 4;
 
-constexpr uint8_t kMaxMessages = 4;
-
 constexpr int16_t kMessagePreviewMaxLifetime = 15 * 1000;  // show messages for 15 seconds
 
 MessagePreview::MessagePreview(InfoPanel* i, const std::string& text, const std::string& tooltip)
    : UI::Textarea(i,
                   UI::PanelStyle::kWui,
+                  "message_preview",
                   UI::FontStyle::kWuiGameSpeedAndCoordinates,
                   0,
                   0,
@@ -50,8 +47,7 @@ MessagePreview::MessagePreview(InfoPanel* i, const std::string& text, const std:
                   std::string("   ") + text + "   ",
                   UI::g_fh->fontset()->is_rtl() ? UI::Align::kRight : UI::Align::kLeft),
      owner_(*i),
-     creation_time_(SDL_GetTicks()),
-     message_(nullptr) {
+     creation_time_(SDL_GetTicks()) {
 	set_thinks(true);
 	set_handle_mouse(true);
 	set_tooltip(tooltip);
@@ -70,17 +66,21 @@ inline bool MessagePreview::message_still_exists() const {
 }
 
 void MessagePreview::think() {
+	MutexLock m(MutexLock::ID::kMessages);
+
 	if (!message_still_exists() || SDL_GetTicks() - creation_time_ > kMessagePreviewMaxLifetime) {
 		owner_.pop_message(this);
 	}
 }
 
 void MessagePreview::draw(RenderTarget& r) {
+	MutexLock m(MutexLock::ID::kMessages);
+
 	if (!message_still_exists()) {
 		return;
 	}
 
-	r.tile(Recti(0, 0, get_w(), get_h()), &load_safe_template_image("wui/windows/background.png"),
+	r.tile(Recti(0, 0, get_w(), get_h()), g_image_cache->get("wui/windows/background.png"),
 	       Vector2i(0, 0));
 
 	// every second message is highlighted
@@ -129,26 +129,23 @@ bool MessagePreview::handle_mousepress(const uint8_t button, int32_t /* x */, in
 }
 
 InfoPanel::InfoPanel(InteractiveBase& ib)
-   : UI::Panel(&ib, UI::PanelStyle::kWui, 0, 0, 0, 0),
+   : UI::Panel(&ib, UI::PanelStyle::kWui, "info_panel", 0, 0, 0, 0),
      ibase_(ib),
-     iplayer_(nullptr),  // this function is called from IBase ctor so we can't upcast yet
-     on_top_(false),
-     display_mode_(DisplayMode::kPinned),
-     last_mouse_pos_(Vector2i(-1, -1)),
-     toolbar_(nullptr),  // will be set later by the IBase
+     snap_target_panel_(&ibase_, UI::PanelStyle::kWui, "snap_target", 0, 0, 0, 0),
      toggle_mode_(this,
                   "mode",
                   0,
                   0,
-                  MainToolbar::kButtonSize,
+                  UI::main_toolbar_button_size(),
                   8,
-                  MainToolbar::kButtonSize,
+                  UI::main_toolbar_button_size(),
                   _("Info Panel Visibility"),
                   UI::DropdownType::kPictorial,
                   UI::PanelStyle::kWui,
                   UI::ButtonStyle::kWuiMenu),
      text_time_speed_(this,
                       UI::PanelStyle::kWui,
+                      "label_time_speed",
                       UI::FontStyle::kWuiGameSpeedAndCoordinates,
                       0,
                       0,
@@ -158,6 +155,7 @@ InfoPanel::InfoPanel(InteractiveBase& ib)
                       UI::Align::kLeft),
      text_fps_(this,
                UI::PanelStyle::kWui,
+               "label_fps",
                UI::FontStyle::kWuiGameSpeedAndCoordinates,
                0,
                0,
@@ -167,6 +165,7 @@ InfoPanel::InfoPanel(InteractiveBase& ib)
                UI::Align::kLeft),
      text_coords_(this,
                   UI::PanelStyle::kWui,
+                  "label_coords",
                   UI::FontStyle::kWuiGameSpeedAndCoordinates,
                   0,
                   0,
@@ -174,18 +173,20 @@ InfoPanel::InfoPanel(InteractiveBase& ib)
                   0,
                   "",
                   UI::Align::kRight),
-     message_queue_(nullptr),
-     last_message_id_(nullptr),
+
      draw_real_time_(get_config_bool("game_clock", true)) {
+	set_z(UI::Panel::ZOrder::kInfoPanel);
 	text_fps_.set_handle_mouse(true);
+	snap_target_panel_.set_snap_target(true);
 	int mode = get_config_int("toolbar_pos", 0);
-	on_top_ = ((mode & DisplayMode::kCmdSwap) != 0);
-	if ((mode & (DisplayMode::kOnMouse_Visible | DisplayMode::kOnMouse_Hidden)) != 0) {
-		display_mode_ = DisplayMode::kOnMouse_Visible;
-	} else if ((mode & DisplayMode::kMinimized) != 0) {
-		display_mode_ = DisplayMode::kMinimized;
+	on_top_ = !UI::main_toolbar_at_bottom();
+	if ((mode & (UI::ToolbarDisplayMode::kOnMouse_Visible |
+	             UI::ToolbarDisplayMode::kOnMouse_Hidden)) != 0) {
+		display_mode_ = UI::ToolbarDisplayMode::kOnMouse_Visible;
+	} else if ((mode & UI::ToolbarDisplayMode::kMinimized) != 0) {
+		display_mode_ = UI::ToolbarDisplayMode::kMinimized;
 	} else {
-		display_mode_ = DisplayMode::kPinned;
+		display_mode_ = UI::ToolbarDisplayMode::kPinned;
 	}
 	rebuild_dropdown();
 	update_mode();
@@ -194,34 +195,35 @@ InfoPanel::InfoPanel(InteractiveBase& ib)
 void InfoPanel::rebuild_dropdown() {
 	toggle_mode_.clear();
 
-	toggle_mode_.add(_("Pin"), DisplayMode::kPinned,
-	                 &load_safe_template_image("wui/windows/pin.png"),
-	                 display_mode_ == DisplayMode::kPinned);
-	toggle_mode_.add(_("Follow mouse"), DisplayMode::kOnMouse_Visible,
+	toggle_mode_.add(_("Pin"), UI::ToolbarDisplayMode::kPinned,
+	                 g_image_cache->get("wui/windows/pin.png"),
+	                 display_mode_ == UI::ToolbarDisplayMode::kPinned);
+	toggle_mode_.add(_("Follow mouse"), UI::ToolbarDisplayMode::kOnMouse_Visible,
 	                 g_image_cache->get("images/ui_basic/fsel.png"),
-	                 display_mode_ == DisplayMode::kOnMouse_Visible ||
-	                    display_mode_ == DisplayMode::kOnMouse_Hidden);
+	                 display_mode_ == UI::ToolbarDisplayMode::kOnMouse_Visible ||
+	                    display_mode_ == UI::ToolbarDisplayMode::kOnMouse_Hidden);
 	toggle_mode_.add(
-	   _("Hide"), DisplayMode::kMinimized,
-	   &load_safe_template_image(on_top_ ? "wui/windows/minimize.png" : "wui/windows/maximize.png"),
-	   display_mode_ == DisplayMode::kMinimized);
+	   _("Hide"), UI::ToolbarDisplayMode::kMinimized,
+	   g_image_cache->get(on_top_ ? "wui/windows/minimize.png" : "wui/windows/maximize.png"),
+	   display_mode_ == UI::ToolbarDisplayMode::kMinimized);
 
 	toggle_mode_.add(
-	   on_top_ ? _("Move panel to bottom") : _("Move panel to top"), DisplayMode::kCmdSwap,
-	   &load_safe_template_image(on_top_ ? "wui/windows/maximize.png" : "wui/windows/minimize.png"));
+	   on_top_ ? _("Move panel to bottom") : _("Move panel to top"),
+	   UI::ToolbarDisplayMode::kCmdSwap,
+	   g_image_cache->get(on_top_ ? "wui/windows/maximize.png" : "wui/windows/minimize.png"));
 
 	toggle_mode_.selected.connect([this]() {
 		update_mode();
-		set_config_int(
-		   "toolbar_pos", on_top_ ? (display_mode_ | DisplayMode::kCmdSwap) : display_mode_);
+		set_config_int("toolbar_pos",
+		               on_top_ ? (display_mode_ | UI::ToolbarDisplayMode::kCmdSwap) : display_mode_);
 	});
 
 	layout();
 }
 
 void InfoPanel::update_mode() {
-	const DisplayMode d = toggle_mode_.get_selected();
-	if (d == DisplayMode::kCmdSwap) {
+	const UI::ToolbarDisplayMode d = toggle_mode_.get_selected();
+	if (d == UI::ToolbarDisplayMode::kCmdSwap) {
 		on_top_ = !on_top_;
 		toolbar_->on_top = on_top_;
 		rebuild_dropdown();
@@ -231,17 +233,17 @@ void InfoPanel::update_mode() {
 	display_mode_ = d;
 
 	switch (display_mode_) {
-	case DisplayMode::kPinned:
+	case UI::ToolbarDisplayMode::kPinned:
 		set_textareas_visibility(true);
 		break;
-	case DisplayMode::kMinimized:
+	case UI::ToolbarDisplayMode::kMinimized:
 		set_textareas_visibility(false);
 		break;
-	case DisplayMode::kOnMouse_Visible:
+	case UI::ToolbarDisplayMode::kOnMouse_Visible:
 		if (is_mouse_over_panel()) {
 			set_textareas_visibility(true);
 		} else {
-			display_mode_ = DisplayMode::kOnMouse_Hidden;
+			display_mode_ = UI::ToolbarDisplayMode::kOnMouse_Hidden;
 			set_textareas_visibility(false);
 		}
 		break;
@@ -260,21 +262,20 @@ void InfoPanel::set_toolbar(MainToolbar& t) {
 	layout();
 }
 
-inline bool InfoPanel::is_mouse_over_panel() const {
+inline bool InfoPanel::is_mouse_over_panel(int32_t x, int32_t y) const {
 	uint8_t h;
 	switch (display_mode_) {
-	case DisplayMode::kMinimized:
+	case UI::ToolbarDisplayMode::kMinimized:
 		return false;
-	case DisplayMode::kOnMouse_Hidden:
+	case UI::ToolbarDisplayMode::kOnMouse_Hidden:
 		h = kSpacing;
 		break;
 	default:
-		h = MainToolbar::kButtonSize;
+		h = UI::main_toolbar_button_size();
 		break;
 	}
-	return last_mouse_pos_.x >= 0 && last_mouse_pos_.x <= get_w() &&
-	       (on_top_ ? (last_mouse_pos_.y >= 0 && last_mouse_pos_.y <= h) :
-                     (last_mouse_pos_.y <= get_h() && last_mouse_pos_.y >= get_h() - h));
+	return x >= 0 && x <= get_w() &&
+	       (on_top_ ? (y >= 0 && y <= h) : (y <= get_h() && y >= get_h() - h));
 }
 
 void InfoPanel::set_textareas_visibility(bool v) {
@@ -291,21 +292,21 @@ bool InfoPanel::handle_mousemove(
 	last_mouse_pos_ = Vector2i(x, y);
 
 	switch (display_mode_) {
-	case DisplayMode::kMinimized:
+	case UI::ToolbarDisplayMode::kMinimized:
 		return false;
-	case DisplayMode::kPinned:
+	case UI::ToolbarDisplayMode::kPinned:
 		return is_mouse_over_panel();
-	case DisplayMode::kOnMouse_Visible:
+	case UI::ToolbarDisplayMode::kOnMouse_Visible:
 		if (!is_mouse_over_panel()) {
-			display_mode_ = DisplayMode::kOnMouse_Hidden;
+			display_mode_ = UI::ToolbarDisplayMode::kOnMouse_Hidden;
 			set_textareas_visibility(false);
 			layout();
 			return false;
 		}
 		return is_mouse_over_panel();
-	case DisplayMode::kOnMouse_Hidden:
+	case UI::ToolbarDisplayMode::kOnMouse_Hidden:
 		if (is_mouse_over_panel()) {
-			display_mode_ = DisplayMode::kOnMouse_Visible;
+			display_mode_ = UI::ToolbarDisplayMode::kOnMouse_Visible;
 			set_textareas_visibility(true);
 			layout();
 			return true;
@@ -326,6 +327,22 @@ bool InfoPanel::handle_mouserelease(uint8_t /*btn*/, int32_t x, int32_t y) {
 	return is_mouse_over_panel();
 }
 
+bool InfoPanel::check_handles_mouse(int32_t x, int32_t y) {
+	last_mouse_pos_ = Vector2i(x, y);
+	if (is_mouse_over_panel() || display_mode_ == UI::ToolbarDisplayMode::kOnMouse_Visible) {
+		return true;
+	}
+
+	for (Panel* p = get_first_child(); p != nullptr; p = p->get_next_sibling()) {
+		if (x >= p->get_x() && y >= p->get_y() && x < p->get_x() + p->get_w() &&
+		    y < p->get_y() + p->get_h() && p->check_handles_mouse(x - p->get_x(), y - p->get_y())) {
+			return true;
+		}
+	}
+
+	return false;
+}
+
 size_t InfoPanel::index_of(const MessagePreview* mp) const {
 	size_t i = 0;
 	for (const MessagePreview* m : messages_) {
@@ -340,7 +357,7 @@ size_t InfoPanel::index_of(const MessagePreview* mp) const {
 	return 0;
 }
 
-void InfoPanel::log_message(const std::string& message) {
+void InfoPanel::log_message(const std::string& message, const std::string& tooltip) {
 	// There is never more than 1 system message visible
 	for (MessagePreview* m : messages_) {
 		if (m->is_system_message()) {
@@ -349,7 +366,7 @@ void InfoPanel::log_message(const std::string& message) {
 		}
 	}
 
-	push_message(new MessagePreview(this, message, ""));
+	push_message(new MessagePreview(this, message, tooltip));
 }
 
 void InfoPanel::pop_message(MessagePreview* m) {
@@ -368,7 +385,7 @@ void InfoPanel::pop_message(MessagePreview* m) {
 void InfoPanel::push_message(MessagePreview* message) {
 	messages_.push_back(message);
 
-	if (messages_.size() > kMaxMessages) {
+	if (messages_.size() > UI::kMaxPopupMessages) {
 		pop_message(messages_.front());
 	}
 
@@ -383,7 +400,7 @@ void InfoPanel::set_fps_string(const bool show,
 		text_fps_.set_text("");
 		text_fps_.set_tooltip("");
 	} else {
-		const std::string text = format("%5.1f fps (avg: %5.1f fps)", fps, average);
+		const std::string text = format_l("%5.1f fps (avg: %5.1f fps)", fps, average);
 		// The FPS string overlaps with the coords string at low resolution.
 		// Therefore abbreviate it if the available width is less than an arbitrary threshold.
 		if (cheating) {
@@ -413,11 +430,7 @@ void InfoPanel::set_speed_string(const std::string& t) {
 void InfoPanel::update_time_speed_string() {
 	std::string realtime;
 	if (draw_real_time_) {
-		std::time_t t = std::time(nullptr);
-		std::tm tm = *std::localtime(&t);
-		std::ostringstream oss("");
-		oss << std::put_time(&tm, "%X");
-		realtime = oss.str();
+		realtime = realtimestring();
 	}
 
 	std::vector<std::string*> non_empty;
@@ -436,14 +449,14 @@ void InfoPanel::update_time_speed_string() {
 		break;
 	case 2:
 		text_time_speed_.set_text(format(
-		   /** TRANSLATORS: (Gametime · Realtime) or (Gametime · Gamespeed) or (Realtime · Gamespeed)
+		   /** TRANSLATORS: (Gametime • Realtime) or (Gametime • Gamespeed) or (Realtime • Gamespeed)
 		    */
-		   _("%1$s · %2$s"), *non_empty[0], *non_empty[1]));
+		   _("%1$s • %2$s"), *non_empty[0], *non_empty[1]));
 		break;
 	case 3:
 		text_time_speed_.set_text(format(
-		   /** TRANSLATORS: Gametime · Realtime · Gamespeed */
-		   _("%1$s · %2$s · %3$s"), *non_empty[0], *non_empty[1], *non_empty[2]));
+		   /** TRANSLATORS: Gametime • Realtime • Gamespeed */
+		   _("%1$s • %2$s • %3$s"), *non_empty[0], *non_empty[1], *non_empty[2]));
 		break;
 	default:
 		NEVER_HERE();
@@ -479,6 +492,19 @@ void InfoPanel::think() {
 		// Refresh real time on every tick
 		update_time_speed_string();
 	}
+
+	for (UI::Panel* p = ibase_.get_first_child(); p != nullptr; p = p->get_next_sibling()) {
+		if (static_cast<uint8_t>(p->get_z()) < static_cast<uint8_t>(UI::Panel::ZOrder::kInfoPanel) &&
+		    p->get_x() < snap_target_panel_.get_w() &&
+		    (on_top_ ? (p->get_y() < snap_target_panel_.get_y() + snap_target_panel_.get_h()) :
+                     (p->get_y() + p->get_h() > snap_target_panel_.get_y()))) {
+			if (UI::Window* w = dynamic_cast<UI::Window*>(p); w != nullptr && !w->moved_by_user()) {
+				w->set_pos(Vector2i(
+				   w->get_x(), on_top_ ? snap_target_panel_.get_y() + snap_target_panel_.get_h() :
+                                     snap_target_panel_.get_y() - w->get_h()));
+			}
+		}
+	}
 }
 
 void InfoPanel::layout() {
@@ -501,17 +527,17 @@ void InfoPanel::layout() {
 	toolbar_->box.set_pos(Vector2i((toolbar_->get_w() - toolbar_->box.get_w()) / 2,
 	                               on_top_ ? 0 : toolbar_->get_h() - toolbar_->box.get_h()));
 
-	const int16_t offset_y = (MainToolbar::kButtonSize - 20 /* font size estimate */) / 4 +
-	                         kSpacing + (on_top_ ? 0 : get_h() - MainToolbar::kButtonSize);
+	const int16_t offset_y = (UI::main_toolbar_button_size() - 20 /* font size estimate */) / 4 +
+	                         kSpacing + (on_top_ ? 0 : get_h() - UI::main_toolbar_button_size());
 
-	text_coords_.set_size(w / 3, MainToolbar::kButtonSize);
+	text_coords_.set_size(w / 3, UI::main_toolbar_button_size());
 	text_coords_.set_pos(Vector2i(w - text_coords_.get_w() - kSpacing, offset_y));
 
-	text_time_speed_.set_size(w / 3, MainToolbar::kButtonSize);
+	text_time_speed_.set_size(w / 3, UI::main_toolbar_button_size());
 	text_time_speed_.set_pos(
 	   Vector2i(toggle_mode_.get_x() + toggle_mode_.get_w() + kSpacing, offset_y));
 
-	text_fps_.set_size(w / 3, MainToolbar::kButtonSize);
+	text_fps_.set_size(w / 3, UI::main_toolbar_button_size());
 	text_fps_.set_pos(Vector2i(toolbar_->get_x() + toolbar_->get_w() + kSpacing, offset_y));
 
 	toolbar_->move_to_top();
@@ -520,7 +546,7 @@ void InfoPanel::layout() {
 		int16_t message_offset = toolbar_->box.get_h();
 		for (MessagePreview* m : messages_) {
 			m->move_to_top();
-			m->set_size(toolbar_->box.get_w(), MainToolbar::kButtonSize);
+			m->set_size(toolbar_->box.get_w(), UI::main_toolbar_button_size());
 			m->set_pos(Vector2i((get_w() - m->get_w()) / 2, message_offset));
 			message_offset += m->get_h();
 		}
@@ -528,20 +554,28 @@ void InfoPanel::layout() {
 		int16_t message_offset = get_h() - toolbar_->box.get_h();
 		for (MessagePreview* m : messages_) {
 			m->move_to_top();
-			m->set_size(toolbar_->box.get_w(), MainToolbar::kButtonSize);
+			m->set_size(toolbar_->box.get_w(), UI::main_toolbar_button_size());
 			message_offset -= m->get_h();
 			m->set_pos(Vector2i((get_w() - m->get_w()) / 2, message_offset));
 		}
 	}
+
+	snap_target_panel_.set_pos(Vector2i(0, toggle_mode_.get_y()));
+	snap_target_panel_.set_size((display_mode_ == UI::ToolbarDisplayMode::kMinimized ||
+	                             display_mode_ == UI::ToolbarDisplayMode::kOnMouse_Hidden) ?
+                                  toggle_mode_.get_w() :
+                                  w,
+	                            toggle_mode_.get_h());
 }
 
 void InfoPanel::draw(RenderTarget& r) {
-	if (display_mode_ == DisplayMode::kMinimized) {
+	if (display_mode_ == UI::ToolbarDisplayMode::kMinimized) {
 		return;
 	}
 
-	const int h =
-	   display_mode_ == DisplayMode::kOnMouse_Hidden ? kSpacing : MainToolbar::kButtonSize;
+	const int h = display_mode_ == UI::ToolbarDisplayMode::kOnMouse_Hidden ?
+                    kSpacing :
+                    UI::main_toolbar_button_size();
 	r.brighten_rect(Recti(0, on_top_ ? 0 : get_h() - h, get_w(), h), -100);
 
 	r.draw_rect(Recti(0, on_top_ ? h : get_h() - h - 1, get_w(), 1), RGBColor(0, 0, 0));
