@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2022 by the Widelands Development Team
+ * Copyright (C) 2010-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,8 +44,7 @@ static size_t priority_to_index(const Widelands::WarePriority& priority) {
 	if (priority == Widelands::WarePriority::kVeryHigh) {
 		return 4;
 	}
-	// TODO(Nordfriese): For savegame compatibility. Replace with NEVER_HERE() after v1.0
-	return 1;
+	NEVER_HERE();
 }
 static const Widelands::WarePriority& index_to_priority(const size_t priority) {
 	switch (priority) {
@@ -91,28 +90,40 @@ void ensure_box_can_hold_input_queues(UI::Box& b) {
 }
 
 InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
-                                     InteractiveBase& ib,
-                                     Widelands::Building& bld,
-                                     Widelands::InputQueue& q,
+                                     InteractiveBase& interactive_base,
+                                     Widelands::Building& building,
+                                     Widelands::InputQueue& queue,
                                      bool show_only,
-                                     bool has_priority)
-   : InputQueueDisplay(
-        parent, ib, bld, q.get_type(), q.get_index(), &q, nullptr, show_only, has_priority) {
+                                     bool has_priority,
+                                     BuildingWindow::CollapsedState* collapsed)
+   : InputQueueDisplay(parent,
+                       interactive_base,
+                       building,
+                       queue.get_type(),
+                       queue.get_index(),
+                       &queue,
+                       nullptr,
+                       show_only,
+                       has_priority,
+                       collapsed) {
 }
 InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
-                                     InteractiveBase& ib,
-                                     Widelands::ConstructionSite& csite,
-                                     Widelands::WareWorker ww,
-                                     Widelands::DescriptionIndex di)
-   : InputQueueDisplay(parent,
-                       ib,
-                       csite,
-                       ww,
-                       di,
-                       nullptr,
-                       dynamic_cast<Widelands::ProductionsiteSettings*>(csite.get_settings()),
-                       false,
-                       true) {
+                                     InteractiveBase& interactive_base,
+                                     Widelands::ConstructionSite& constructionsite,
+                                     Widelands::WareWorker type,
+                                     Widelands::DescriptionIndex ware_or_worker_index,
+                                     BuildingWindow::CollapsedState* collapsed)
+   : InputQueueDisplay(
+        parent,
+        interactive_base,
+        constructionsite,
+        type,
+        ware_or_worker_index,
+        nullptr,
+        dynamic_cast<Widelands::ProductionsiteSettings*>(constructionsite.get_settings()),
+        false,
+        true,
+        collapsed) {
 }
 
 static inline std::string create_tooltip(const bool increase) {
@@ -143,27 +154,33 @@ static inline std::string create_tooltip(const bool increase) {
 }
 
 InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
-                                     InteractiveBase& ib,
-                                     Widelands::Building& bld,
-                                     Widelands::WareWorker ww,
-                                     Widelands::DescriptionIndex idx,
-                                     Widelands::InputQueue* q,
-                                     Widelands::ProductionsiteSettings* s,
+                                     InteractiveBase& interactive_base,
+                                     Widelands::Building& building,
+                                     Widelands::WareWorker type,
+                                     Widelands::DescriptionIndex ware_or_worker_index,
+                                     Widelands::InputQueue* queue,
+                                     Widelands::ProductionsiteSettings* settings,
                                      bool show_only,
-                                     bool has_priority)
-   : UI::Box(parent, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal),
-     ibase_(ib),
-     can_act_(!show_only && ibase_.can_act(bld.owner().player_number())),
+                                     bool has_priority,
+                                     BuildingWindow::CollapsedState* collapsed)
+   : UI::Box(parent,
+             UI::PanelStyle::kWui,
+             format("inputqueuedisplay_%u_%u", static_cast<unsigned>(type), ware_or_worker_index),
+             0,
+             0,
+             UI::Box::Horizontal),
+     ibase_(interactive_base),
+     can_act_(!show_only && ibase_.can_act(building.owner().player_number())),
      show_only_(show_only),
-     has_priority_(has_priority && ww == Widelands::wwWARE),
-     building_(&bld),
-     type_(ww),
-     index_(idx),
-     queue_(q),
-     settings_(s),
+     has_priority_(has_priority && type == Widelands::wwWARE),
+     building_(&building),
+     type_(type),
+     index_(ware_or_worker_index),
+     queue_(queue),
+     settings_(settings),
      max_fill_indicator_(*g_image_cache->get("images/wui/buildings/max_fill_indicator.png")),
-     vbox_(this, UI::PanelStyle::kWui, 0, 0, UI::Box::Vertical),
-     hbox_(&vbox_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal),
+     vbox_(this, UI::PanelStyle::kWui, "vbox", 0, 0, UI::Box::Vertical),
+     hbox_(&vbox_, UI::PanelStyle::kWui, "hbox", 0, 0, UI::Box::Horizontal),
      b_decrease_desired_fill_(&hbox_,
                               "decrease_desired",
                               0,
@@ -219,15 +236,17 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
                4,
                has_priority_ ? priority_to_index(settings_ != nullptr ?
                                                     settings_->ware_queues.at(index_).priority :
-                                                    bld.get_priority(type_, index_)) :
+                                                    building.get_priority(type_, index_)) :
                                2,
                UI::SliderStyle::kWuiLight,
                "",
                kButtonSize,
                can_act_ && has_priority_),
-     spacer_(&hbox_, UI::PanelStyle::kWui, 0, 0, priority_.get_w(), priority_.get_h()),
+     spacer_(&hbox_, UI::PanelStyle::kWui, "spacer", 0, 0, priority_.get_w(), priority_.get_h()),
+     priority_indicator_(
+        &hbox_, UI::PanelStyle::kWui, "priority_indicator", 0, 0, kButtonSize / 5, kButtonSize),
      slider_was_moved_(nullptr),
-     collapsed_(false),
+     collapsed_(collapsed),
      nr_icons_(queue_ != nullptr            ? queue_->get_max_size() :
                type_ == Widelands::wwWORKER ? settings_->worker_queues.at(index_).max_fill :
                                               settings_->ware_queues.at(index_).max_fill),
@@ -255,10 +274,10 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
 	hbox_.add(&b_decrease_desired_fill_);
 
 	for (size_t i = 0; i < nr_icons_; ++i) {
-		icons_[i] = new UI::Icon(&hbox_, UI::PanelStyle::kWui, 0, 0, kButtonSize, kButtonSize,
-		                         type_ == Widelands::wwWARE ?
-                                  bld.owner().tribe().get_ware_descr(index_)->icon() :
-                                  bld.owner().tribe().get_worker_descr(index_)->icon());
+		icons_[i] = new UI::Icon(
+		   &hbox_, UI::PanelStyle::kWui, format("icon_%u", i), 0, 0, kButtonSize, kButtonSize,
+		   type_ == Widelands::wwWARE ? building.owner().tribe().get_ware_descr(index_)->icon() :
+                                      building.owner().tribe().get_worker_descr(index_)->icon());
 		hbox_.add(icons_[i]);
 	}
 
@@ -270,7 +289,7 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
 
 	// To make sure the fill buttons are aligned even when some queues
 	// have priority buttons and some don't (e.g. in barracks)
-	hbox_.add_space(kButtonSize / 4);
+	hbox_.add(&priority_indicator_);
 	priority_.set_visible(has_priority_);
 	spacer_.set_visible(!has_priority_);
 	hbox_.add(&priority_, UI::Box::Resizing::kAlign, UI::Align::kCenter);
@@ -285,12 +304,8 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
 
 	if (can_act_) {
 		collapse_.sigclicked.connect([this]() {
-			const bool c = !collapsed_;
-			recurse([c](InputQueueDisplay& i) {
-				if (i.can_act_) {
-					i.set_collapsed(c);
-				}
-			});
+			*collapsed_ = is_collapsed() ? BuildingWindow::CollapsedState::kExpanded :
+                                        BuildingWindow::CollapsedState::kCollapsed;
 		});
 
 		b_decrease_desired_fill_.sigclicked.connect([this]() {
@@ -332,12 +347,14 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
 		});
 	} else {
 		collapse_.set_visible(false);
-		set_collapsed(true);
+		*collapsed_ = BuildingWindow::CollapsedState::kCollapsed;
 	}
 
+	set_collapsed();
+
 	set_tooltip(type_ == Widelands::wwWARE ?
-                  bld.owner().tribe().get_ware_descr(index_)->descname() :
-                  bld.owner().tribe().get_worker_descr(index_)->descname());
+                  building.owner().tribe().get_ware_descr(index_)->descname() :
+                  building.owner().tribe().get_worker_descr(index_)->descname());
 
 	if (nr_icons_ == 0) {
 		// Can happen when this is a dropout queue that has already been emptied.
@@ -394,33 +411,63 @@ bool InputQueueDisplay::handle_mousewheel(int32_t x, int32_t y, uint16_t modstat
 	if (show_only_ || !can_act_) {
 		return false;
 	}
+	bool big_step = false;
 	int32_t change = get_mousewheel_change(MousewheelHandlerConfigID::kChangeValue, x, y,
 	                                       // shift has special meaning, prevent it to work
 	                                       // as part of modifier
 	                                       modstate & ~KMOD_SHIFT);
-	if (change != 0) {
-		if (get_mouse_position().x < priority_.get_x() - kButtonSize / 4) {
-			// Mouse is over desired fill
-			if ((modstate & KMOD_SHIFT) != 0) {
-				recurse([change](InputQueueDisplay& i) { i.change_desired_fill(change); });
-			} else {
-				change_desired_fill(change);
-			}
-			return true;
-		}
-		if (has_priority_) {
-			// Mouse is over priority or collapse button
-			// Can't just use method from Slider, because of the special
-			// meaning of shift to change all input priorities together.
-
-			// KMOD_SHIFT + changedto is already connected to recurse.
-			priority_.change_value_by(change);
-			return true;
+	if (change == 0) {
+		// Try big step
+		change = get_mousewheel_change(
+		   MousewheelHandlerConfigID::kChangeValueBig, x, y, modstate & ~KMOD_SHIFT);
+		if (change == 0) {
+			return false;
 		}
+		big_step = true;
 	}
+
+	if (get_mouse_position().x < priority_indicator_.get_x()) {
+		// Mouse is over desired fill
+
+		if (big_step) {
+			change *= ChangeBigStep::kSmallRange;
+		}
+
+		if ((modstate & KMOD_SHIFT) != 0) {
+			recurse([change](InputQueueDisplay& i) { i.change_desired_fill(change); });
+		} else {
+			change_desired_fill(change);
+		}
+		return true;
+	}
+	if (has_priority_) {
+		// Mouse is over priority or collapse button
+		// Can't just use method from Slider, because of the special
+		// meaning of shift to change all input priorities together.
+
+		if (big_step) {
+			// This makes the steps: Very Low -> Normal -> Very High
+			change *= 2;
+		}
+
+		// KMOD_SHIFT + changedto is already connected to recurse.
+		priority_.change_value_by(change);
+		return true;
+	}
+
 	return false;
 }
 
+bool UI::PrioritySlider::handle_key(bool down, SDL_Keysym code) {
+	// shift has special meaning, prevent it to work as part of modifier
+	const uint16_t filtered_keymod = code.mod & ~KMOD_SHIFT;
+	SDL_Keysym filtered_code = {code.scancode, code.sym, filtered_keymod, code.unused};
+
+	// If this changes the slider's value while Shift is pressed, then KMOD_SHIFT + changedto
+	// takes care of changing the other ones too.
+	return Slider::handle_key(down, filtered_code);
+}
+
 void InputQueueDisplay::set_priority(const Widelands::WarePriority& priority) {
 	MutexLock m(MutexLock::ID::kObjects);
 	Widelands::Building* b = building_.get(ibase_.egbase());
@@ -563,15 +610,16 @@ void InputQueueDisplay::clicked_real_fill(const int8_t delta) {
 	queue_->set_filled(new_fill);
 }
 
-void InputQueueDisplay::set_collapsed(const bool c) {
-	assert(collapsed_ != c);
-	collapsed_ = c;
-	priority_.set_visible(has_priority_ && !collapsed_);
-	spacer_.set_visible(!has_priority_ && !collapsed_);
-	b_decrease_desired_fill_.set_visible(!collapsed_ && !show_only_);
-	b_increase_desired_fill_.set_visible(!collapsed_ && !show_only_);
-	b_decrease_real_fill_.set_visible(!collapsed_ && ibase_.omnipotent());
-	b_increase_real_fill_.set_visible(!collapsed_ && ibase_.omnipotent());
+void InputQueueDisplay::set_collapsed() {
+	priority_.set_visible(has_priority_ && !is_collapsed());
+	spacer_.set_visible(!has_priority_ && !is_collapsed());
+	b_decrease_desired_fill_.set_visible(!is_collapsed() && !show_only_);
+	b_increase_desired_fill_.set_visible(!is_collapsed() && !show_only_);
+	b_decrease_real_fill_.set_visible(!is_collapsed() && ibase_.omnipotent());
+	b_increase_real_fill_.set_visible(!is_collapsed() && ibase_.omnipotent());
+	collapse_.set_tooltip(is_collapsed() ? _("Show controls") : _("Hide controls"));
+	collapse_.set_pic(g_image_cache->get(is_collapsed() ? "images/ui_basic/scrollbar_right.png" :
+                                                         "images/ui_basic/scrollbar_left.png"));
 }
 
 inline Widelands::ProductionsiteSettings::InputQueueSetting*
@@ -634,9 +682,7 @@ void InputQueueDisplay::think() {
 		priority_.set_tooltip(priority_tooltip(priority_.get_value()));
 	}
 
-	collapse_.set_tooltip(collapsed_ ? _("Show controls") : _("Hide controls"));
-	collapse_.set_pic(g_image_cache->get(collapsed_ ? "images/ui_basic/scrollbar_right.png" :
-                                                     "images/ui_basic/scrollbar_left.png"));
+	set_collapsed();
 }
 
 static const RGBAColor kPriorityColors[] = {RGBAColor(0, 0, 255, 127), RGBAColor(63, 127, 255, 127),
@@ -645,7 +691,7 @@ static const RGBAColor kPriorityColors[] = {RGBAColor(0, 0, 255, 127), RGBAColor
 
 void InputQueueDisplay::draw(RenderTarget& r) {
 	// Draw priority indicator
-	if (has_priority_ && !collapsed_) {
+	if (has_priority_ && !is_collapsed()) {
 		const int x = hbox_.get_x() + priority_.get_x();
 		for (size_t i = 0; i < 5; ++i) {
 			r.fill_rect(Recti(x + i * kButtonSize, hbox_.get_y() + kButtonSize * 2 / 5, kButtonSize,
@@ -695,14 +741,14 @@ void InputQueueDisplay::draw_overlay(RenderTarget& r) {
 	}
 
 	// Draw priority indicator
-	if (has_priority_ && collapsed_) {
+	if (has_priority_ && is_collapsed()) {
 		const size_t p = priority_to_index(queue_ != nullptr ? b->get_priority(type_, index_) :
                                                              get_setting()->priority);
-		const int w = kButtonSize / 5;
-		const int x = hbox_.get_x() + collapse_.get_x() - w;
+		const int w = priority_indicator_.get_w();
+		// Add kButtonSize / 4 to the position to align it against the collapse button
+		const int x = hbox_.get_x() + priority_indicator_.get_x() + kButtonSize / 4;
 		r.brighten_rect(Recti(x, hbox_.get_y(), w, kButtonSize), -32);
-		r.fill_rect(Recti(x, hbox_.get_y() + (4 - p) * kButtonSize / 5, w, kButtonSize / 5),
-		            kPriorityColors[p], BlendMode::Copy);
+		r.fill_rect(Recti(x, hbox_.get_y() + (4 - p) * w, w, w), kPriorityColors[p], BlendMode::Copy);
 	}
 
 	UI::Box::draw_overlay(r);
