@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2022 by the Widelands Development Team
+ * Copyright (C) 2007-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
 
 #include "wui/interactive_gamebase.h"
 
+#include <cstdlib>
 #include <memory>
 
 #include "base/macros.h"
@@ -32,10 +33,13 @@
 #include "logic/map_objects/tribes/ship.h"
 #include "logic/player.h"
 #include "network/gamehost.h"
+#include "ui_basic/toolbar_setup.h"
 #include "wlapplication_mousewheel_options.h"
 #include "wlapplication_options.h"
+#include "wui/fieldaction.h"
 #include "wui/game_chat_menu.h"
 #include "wui/game_client_disconnected.h"
+#include "wui/game_diplomacy_menu.h"
 #include "wui/game_exit_confirm_box.h"
 #include "wui/game_main_menu_save_game.h"
 #include "wui/game_options_sound_menu.h"
@@ -43,6 +47,7 @@
 #include "wui/info_panel.h"
 #include "wui/interactive_player.h"
 #include "wui/toolbar.h"
+#include "wui/waresdisplay.h"
 #include "wui/watchwindow.h"
 
 namespace {
@@ -70,9 +75,9 @@ InteractiveGameBase::InteractiveGameBase(Widelands::Game& g,
                    "dropdown_menu_showhide",
                    0,
                    0,
-                   MainToolbar::kButtonSize,
+                   UI::main_toolbar_button_size(),
                    10,
-                   MainToolbar::kButtonSize,
+                   UI::main_toolbar_button_size(),
                    /** TRANSLATORS: Title for a menu button in the game. This menu will show/hide
                       building spaces, census, status, etc. */
                    _("Show / Hide"),
@@ -80,13 +85,17 @@ InteractiveGameBase::InteractiveGameBase(Widelands::Game& g,
                    UI::PanelStyle::kWui,
                    UI::ButtonStyle::kWuiPrimary,
                    [this](ShowHideEntry t) { showhide_menu_selected(t); }),
+     grid_marker_pic_(g_image_cache->get("images/wui/overlays/grid_marker.png")),
+     special_coords_marker_pic_(g_image_cache->get("images/wui/overlays/special.png")),
+     pause_on_inactivity_(get_config_int("pause_game_on_inactivity", 0) * 60 * 1000),
+     can_restart_(g.is_replay() || !g.list_of_scenarios().empty()),
      mainmenu_(toolbar(),
                "dropdown_menu_main",
                0,
                0,
-               MainToolbar::kButtonSize,
+               UI::main_toolbar_button_size(),
                10,
-               MainToolbar::kButtonSize,
+               UI::main_toolbar_button_size(),
                as_tooltip_text_with_hotkey(
                   /** TRANSLATORS: Title for the main menu button in the game */
                   _("Main Menu"),
@@ -100,9 +109,9 @@ InteractiveGameBase::InteractiveGameBase(Widelands::Game& g,
                     "dropdown_menu_gamespeed",
                     0,
                     0,
-                    MainToolbar::kButtonSize,
+                    UI::main_toolbar_button_size(),
                     10,
-                    MainToolbar::kButtonSize,
+                    UI::main_toolbar_button_size(),
                     /** TRANSLATORS: Title for a menu button in the game. This menu will show
                        options o increase/decrease the gamespeed, and to pause the game */
                     _("Game Speed"),
@@ -124,13 +133,12 @@ void InteractiveGameBase::add_main_menu() {
 
 void InteractiveGameBase::rebuild_main_menu() {
 	mainmenu_.clear();
-#ifndef NDEBUG  //  only in debug builds
-	/** TRANSLATORS: An entry in the game's main menu */
-	mainmenu_.add(_("Script Console"), MainMenuEntry::kScriptConsole,
-	              g_image_cache->get("images/wui/menus/lua.png"), false,
-	              /** TRANSLATORS: Tooltip for Script Console in the game's main menu */
-	              "", pgettext("hotkey", "Ctrl+Shift+Space"));
-#endif
+	if (g_allow_script_console) {
+		/** TRANSLATORS: An entry in the game's main menu */
+		mainmenu_.add(_("Script Console"), MainMenuEntry::kScriptConsole,
+		              g_image_cache->get("images/wui/menus/lua.png"), false, "",
+		              shortcut_string_for(KeyboardShortcut::kCommonDebugConsole, false));
+	}
 
 	menu_windows_.sound_options.open_window = [this] {
 		new GameOptionsSoundMenu(*this, menu_windows_.sound_options);
@@ -139,7 +147,8 @@ void InteractiveGameBase::rebuild_main_menu() {
 	mainmenu_.add(_("Sound Options"), MainMenuEntry::kOptions,
 	              g_image_cache->get("images/wui/menus/options.png"), false,
 	              /** TRANSLATORS: Tooltip for Sound Options in the game's main menu */
-	              _("Set sound effect and music options"));
+	              _("Set sound effect and music options"),
+	              shortcut_string_for(KeyboardShortcut::kInGameSoundOptions, false));
 
 	menu_windows_.savegame.open_window = [this] {
 		new GameMainMenuSaveGame(*this, menu_windows_.savegame, GameMainMenuSaveGame::Type::kSave);
@@ -147,36 +156,53 @@ void InteractiveGameBase::rebuild_main_menu() {
 	/** TRANSLATORS: An entry in the game's main menu */
 	mainmenu_.add(_("Save Game"), MainMenuEntry::kSaveMap,
 	              g_image_cache->get("images/wui/menus/save_game.png"), false, "",
-	              shortcut_string_for(KeyboardShortcut::kInGameSave, false));
+	              shortcut_string_for(KeyboardShortcut::kCommonSave, false));
 
-	if (!is_multiplayer() && !game().is_replay()) {
+	if (game().is_replay()) {
 		menu_windows_.loadgame.open_window = [this] {
-			new GameMainMenuSaveGame(*this, menu_windows_.loadgame, GameMainMenuSaveGame::Type::kLoad);
+			new GameMainMenuSaveGame(
+			   *this, menu_windows_.loadgame, GameMainMenuSaveGame::Type::kLoadReplay);
+		};
+		/** TRANSLATORS: An entry in the game's main menu */
+		mainmenu_.add(_("Load Replay"), MainMenuEntry::kLoadMap,
+		              g_image_cache->get("images/wui/menus/load_game.png"), false, "",
+		              shortcut_string_for(KeyboardShortcut::kCommonLoad, false));
+
+		/** TRANSLATORS: An entry in the game's main menu */
+		mainmenu_.add(_("Restart Replay"), MainMenuEntry::kRestartScenario,
+		              g_image_cache->get("images/wui/menus/restart_scenario.png"), false, "",
+		              shortcut_string_for(KeyboardShortcut::kInGameRestart, false));
+	} else if (!is_multiplayer()) {
+		menu_windows_.loadgame.open_window = [this] {
+			new GameMainMenuSaveGame(
+			   *this, menu_windows_.loadgame, GameMainMenuSaveGame::Type::kLoadSavegame);
 		};
 		/** TRANSLATORS: An entry in the game's main menu */
 		mainmenu_.add(_("Load Game"), MainMenuEntry::kLoadMap,
 		              g_image_cache->get("images/wui/menus/load_game.png"), false, "",
-		              shortcut_string_for(KeyboardShortcut::kInGameLoad, false));
+		              shortcut_string_for(KeyboardShortcut::kCommonLoad, false));
 	}
 
 	if (!game().list_of_scenarios().empty()) {
 		/** TRANSLATORS: An entry in the game's main menu */
 		mainmenu_.add(_("Restart Scenario"), MainMenuEntry::kRestartScenario,
-		              g_image_cache->get("images/wui/menus/restart_scenario.png"));
+		              g_image_cache->get("images/wui/menus/restart_scenario.png"), false, "",
+		              shortcut_string_for(KeyboardShortcut::kInGameRestart, false));
 	}
 
-	mainmenu_.add(
-	   /** TRANSLATORS: An entry in the game's main menu */
-	   _("Exit Game"), MainMenuEntry::kExitGame, g_image_cache->get("images/wui/menus/exit.png"));
+	/** TRANSLATORS: An entry in the game's main menu */
+	mainmenu_.add(_("Exit Game"), MainMenuEntry::kExitGame,
+	              g_image_cache->get("images/wui/menus/exit.png"), false, "",
+	              shortcut_string_for(KeyboardShortcut::kCommonExit, false));
 }
 
 void InteractiveGameBase::main_menu_selected(MainMenuEntry entry) {
 	switch (entry) {
-#ifndef NDEBUG  //  only in debug builds
 	case MainMenuEntry::kScriptConsole: {
-		GameChatMenu::create_script_console(this, debugconsole_, *DebugConsole::get_chat_provider());
+		assert(g_allow_script_console);
+		GameChatMenu::create_script_console(
+		   this, color_functor(), debugconsole_, *DebugConsole::get_chat_provider());
 	} break;
-#endif
 	case MainMenuEntry::kOptions: {
 		menu_windows_.sound_options.toggle();
 	} break;
@@ -184,19 +210,10 @@ void InteractiveGameBase::main_menu_selected(MainMenuEntry entry) {
 		menu_windows_.savegame.toggle();
 	} break;
 	case MainMenuEntry::kRestartScenario: {
-		if ((SDL_GetModState() & KMOD_CTRL) != 0) {
-			game().set_next_game_to_load(game().list_of_scenarios().front());
-			end_modal<UI::Panel::Returncodes>(UI::Panel::Returncodes::kBack);
-		} else {
-			GameExitConfirmBox* gecb =
-			   new GameExitConfirmBox(*this, *this, _("Restart Scenario"),
-			                          _("Are you sure you wish to restart this scenario?"));
-			gecb->ok.connect(
-			   [this] { game().set_next_game_to_load(game().list_of_scenarios().front()); });
-		}
+		handle_restart((SDL_GetModState() & KMOD_CTRL) != 0);
 	} break;
 	case MainMenuEntry::kLoadMap:
-		if (!is_multiplayer() && !game().is_replay()) {
+		if (!is_multiplayer()) {
 			menu_windows_.loadgame.toggle();
 		}
 		break;
@@ -210,6 +227,31 @@ void InteractiveGameBase::main_menu_selected(MainMenuEntry entry) {
 	}
 }
 
+void InteractiveGameBase::handle_restart(const bool force) {
+	const bool r = game().is_replay();
+	const std::string& next = r ? game().replay_filename() : game().list_of_scenarios().front();
+	if (force) {
+		game().set_next_game_to_load(next);
+		end_modal<UI::Panel::Returncodes>(UI::Panel::Returncodes::kBack);
+	} else {
+		GameExitConfirmBox* gecb =
+		   new GameExitConfirmBox(*this, *this, r ? _("Restart Replay") : _("Restart Scenario"),
+		                          r ? _("Are you sure you wish to restart this replay?") :
+                                    _("Are you sure you wish to restart this scenario?"));
+		gecb->ok.connect([this, next] { game().set_next_game_to_load(next); });
+	}
+}
+
+void InteractiveGameBase::add_diplomacy_menu() {
+	add_toolbar_button(
+	   "wui/menus/diplomacy", "diplomacy",
+	   as_tooltip_text_with_hotkey(_("Diplomacy"),
+	                               shortcut_string_for(KeyboardShortcut::kInGameDiplomacy, true),
+	                               UI::PanelStyle::kWui),
+	   &diplomacy_, true);
+	diplomacy_.open_window = [this] { new GameDiplomacyMenu(*this, diplomacy_); };
+}
+
 void InteractiveGameBase::add_showhide_menu() {
 	showhidemenu_.set_image(g_image_cache->get("images/wui/menus/showhide.png"));
 	toolbar()->add(&showhidemenu_);
@@ -369,7 +411,7 @@ void InteractiveGameBase::add_chat_ui() {
 	   &chat_, true);
 	chat_.open_window = [this] {
 		if (chat_provider_ != nullptr) {
-			GameChatMenu::create_chat_console(this, chat_, *chat_provider_);
+			GameChatMenu::create_chat_console(this, color_functor(), chat_, *chat_provider_);
 		}
 	};
 }
@@ -464,18 +506,37 @@ bool InteractiveGameBase::handle_key(bool down, SDL_Keysym code) {
 		menu_windows_.stats_general.toggle();
 		return true;
 	}
-	if (matches_shortcut(KeyboardShortcut::kInGameSave, code)) {
+	if (matches_shortcut(KeyboardShortcut::kInGameDiplomacy, code)) {
+		diplomacy_.toggle();
+		return true;
+	}
+	if (matches_shortcut(KeyboardShortcut::kCommonSave, code)) {
 		new GameMainMenuSaveGame(*this, menu_windows_.savegame, GameMainMenuSaveGame::Type::kSave);
 		return true;
 	}
-	if (!is_multiplayer() && !game().is_replay() &&
-	    matches_shortcut(KeyboardShortcut::kInGameLoad, code)) {
-		new GameMainMenuSaveGame(*this, menu_windows_.loadgame, GameMainMenuSaveGame::Type::kLoad);
+	if (!is_multiplayer()) {
+		if (matches_shortcut(KeyboardShortcut::kCommonLoad, code)) {
+			new GameMainMenuSaveGame(*this, menu_windows_.loadgame,
+			                         game().is_replay() ? GameMainMenuSaveGame::Type::kLoadReplay :
+                                                       GameMainMenuSaveGame::Type::kLoadSavegame);
+			return true;
+		}
+		if (can_restart_ && matches_shortcut(KeyboardShortcut::kInGameRestart, code)) {
+			handle_restart();
+			return true;
+		}
+	}
+	if (matches_shortcut(KeyboardShortcut::kCommonExit, code)) {
+		new GameExitConfirmBox(*this, *this);
+		return true;
+	}
+	if (matches_shortcut(KeyboardShortcut::kInGameSoundOptions, code)) {
+		menu_windows_.sound_options.toggle();
 		return true;
 	}
 	if ((chat_provider_ != nullptr) && matches_shortcut(KeyboardShortcut::kInGameChat, code)) {
 		if (chat_.window == nullptr) {
-			GameChatMenu::create_chat_console(this, chat_, *chat_provider_);
+			GameChatMenu::create_chat_console(this, color_functor(), chat_, *chat_provider_);
 		}
 		return dynamic_cast<GameChatMenu*>(chat_.window)->enter_chat_message();
 	}
@@ -501,6 +562,19 @@ bool InteractiveGameBase::handle_mousewheel(int32_t x, int32_t y, uint16_t modst
 	return true;
 }
 
+void InteractiveGameBase::think() {
+	InteractiveBase::think();
+
+	if (pause_on_inactivity_ != 0 &&
+	    static_cast<int>(SDL_GetTicks() - UI::Panel::time_of_last_user_activity()) >
+	       pause_on_inactivity_) {
+		Widelands::Game& g = game();
+		if (g.game_controller() != nullptr && !g.game_controller()->is_paused()) {
+			toggle_game_paused();
+		}
+	}
+}
+
 /// \return a pointer to the running \ref Game instance.
 Widelands::Game* InteractiveGameBase::get_game() const {
 	return dynamic_cast<Widelands::Game*>(&egbase());
@@ -522,16 +596,27 @@ void InteractiveGameBase::draw_overlay(RenderTarget& dst) {
 	// Display the gamespeed.
 	if (game_controller != nullptr) {
 		std::string game_speed;
-		uint32_t const real = game_controller->real_speed();
-		uint32_t const desired = game_controller->desired_speed();
-		if (real == desired) {
-			if (real != 1000) {
-				game_speed = speed_string(real);
+		const int64_t computed_target = game_controller->real_speed();
+		const int64_t desired = game_controller->desired_speed();
+		int64_t actual = average_real_gamespeed();
+		constexpr int64_t kFluctuationTolerance = 1000;  // Arbitrary value.
+		if (abs(actual - computed_target) < kFluctuationTolerance) {
+			actual = computed_target;  // Ignore minor fluctuations.
+		}
+
+		if (desired == computed_target && actual == computed_target) {
+			if (actual != 1000) {
+				game_speed = speed_string(actual);
 			}
-		} else {
+		} else if (desired == computed_target || actual == computed_target) {
 			game_speed = format
 			   /** TRANSLATORS: actual_speed (desired_speed) */
-			   (_("%1$s (%2$s)"), speed_string(real), speed_string(desired));
+			   (_("%1$s (%2$s)"), speed_string(actual), speed_string(desired));
+		} else {
+			game_speed = format
+			   /** TRANSLATORS: actual_speed (target_speed) (desired_speed) */
+			   (_("%1$s (%2$s) (%3$s)"), speed_string(actual), speed_string(computed_target),
+			    speed_string(desired));
 		}
 
 		info_panel_.set_speed_string(game_speed);
@@ -561,7 +646,16 @@ void InteractiveGameBase::set_sel_pos(Widelands::NodeAndTriangle<> const center)
 
 	if (imm->descr().type() == Widelands::MapObjectType::IMMOVABLE) {
 		// Trees, Resource Indicators, fields ...
-		return set_tooltip(imm->descr().descname());
+		Widelands::Buildcost cost;
+		dynamic_cast<Widelands::Immovable&>(*imm).construct_remaining_buildcost(&cost);
+		if (cost.empty()) {
+			return set_tooltip(imm->descr().descname());
+		}
+		return set_tooltip(imm->descr().descname() + "<br>" +
+		                   g_style_manager->ware_info_style(UI::WareInfoStyle::kNormal)
+		                      .header_font()
+		                      .as_font_tag(_("Remaining construction costs:")) +
+		                   "<br>" + waremap_to_richtext(imm->owner().tribe(), cost));
 	}
 	if (upcast(Widelands::ProductionSite, productionsite, imm)) {
 		// No productionsite tips for hostile players
@@ -599,6 +693,11 @@ void InteractiveGameBase::start() {
 		if (game().get_player(pln) != nullptr) {
 			map_view()->scroll_to_field(game().map().get_starting_pos(pln), MapView::Transition::Jump);
 		}
+
+		if (g_allow_script_console) {
+			// Let's warn all users once again
+			broadcast_cheating_message("CAN_CHEAT");
+		}
 	}
 }
 
@@ -606,26 +705,54 @@ void InteractiveGameBase::start() {
  * See if we can reasonably open a ship window at the current selection position.
  * If so, do it and return true; otherwise, return false.
  */
-bool InteractiveGameBase::try_show_ship_window() {
+bool InteractiveGameBase::try_show_ship_windows() {
 	const Widelands::Map& map = game().map();
 	Widelands::Area<Widelands::FCoords> area(map.get_fcoords(get_sel_pos().node), 1);
 
-	if ((area.field->nodecaps() & Widelands::MOVECAPS_SWIM) == 0) {
-		return false;
-	}
-
-	std::vector<Widelands::Bob*> ships;
-	if (map.find_bobs(egbase(), area, &ships, Widelands::FindBobShip()) != 0u) {
-		for (Widelands::Bob* ship : ships) {
+	std::vector<Widelands::Bob*> all_ships;
+	std::vector<Widelands::Ship*> manageable;
+	std::vector<Widelands::Ship*> attackable;
+	if (map.find_bobs(egbase(), area, &all_ships, Widelands::FindBobShip()) != 0u) {
+		for (Widelands::Bob* bob : all_ships) {
+			upcast(Widelands::Ship, ship, bob);
+			assert(ship != nullptr);  // FindBobShip should have returned only ships
 			if (can_see(ship->owner().player_number())) {
-				// FindBobShip should have returned only ships
-				assert(ship->descr().type() == Widelands::MapObjectType::SHIP);
-				show_ship_window(dynamic_cast<Widelands::Ship*>(ship));
-				return true;
+				manageable.push_back(ship);
+			} else if (get_player() != nullptr && get_player()->is_hostile(ship->owner()) &&
+			           ship->can_be_attacked()) {
+				attackable.push_back(ship);
 			}
 		}
 	}
-	return false;
+
+	if (manageable.empty() && attackable.empty()) {
+		return false;  // No ships nearby.
+	}
+
+	if (manageable.size() + attackable.size() == 1) {
+		// Exactly one ship nearby, open its appropriate window.
+		if (attackable.empty()) {
+			show_ship_window(manageable.front());
+		} else {
+			show_attack_window(get_sel_pos().node, attackable.front(), true);
+		}
+		return true;
+	}
+
+	if ((SDL_GetModState() & KMOD_CTRL) != 0) {
+		// Open all applicable windows if Ctrl is held.
+		for (Widelands::Ship* ship : manageable) {
+			show_ship_window(ship);
+		}
+		for (Widelands::Ship* ship : attackable) {
+			show_attack_window(get_sel_pos().node, ship, true);
+		}
+		return true;
+	}
+
+	// Show a selection dialog.
+	show_ship_selection_window(this, &fieldaction_, get_sel_pos().node, manageable, attackable);
+	return true;
 }
 
 void InteractiveGameBase::show_game_summary() {
