@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,9 +20,10 @@
 
 #include "base/i18n.h"
 #include "base/macros.h"
-#include "base/multithreading.h"
+#include "base/mutex.h"
 #include "economy/flag.h"
 #include "game_io/game_loader.h"
+#include "graphic/color.h"
 #include "graphic/game_renderer.h"
 #include "graphic/mouse_cursor.h"
 #include "graphic/text_layout.h"
@@ -32,20 +33,22 @@
 #include "logic/map_objects/tribes/building.h"
 #include "logic/map_objects/tribes/constructionsite.h"
 #include "logic/map_objects/tribes/productionsite.h"
+#include "logic/map_objects/tribes/ship.h"
 #include "logic/map_objects/tribes/soldier.h"
 #include "logic/map_objects/tribes/tribe_descr.h"
 #include "logic/message_queue.h"
 #include "logic/player.h"
+#include "ui_basic/toolbar_setup.h"
 #include "ui_basic/unique_window.h"
 #include "wlapplication_options.h"
 #include "wui/attack_window.h"
 #include "wui/building_statistics_menu.h"
 #include "wui/debugconsole.h"
 #include "wui/fieldaction.h"
-#include "wui/game_diplomacy_menu.h"
 #include "wui/game_message_menu.h"
 #include "wui/game_objectives_menu.h"
 #include "wui/general_statistics_menu.h"
+#include "wui/pinned_note.h"
 #include "wui/seafaring_statistics_menu.h"
 #include "wui/soldier_statistics_menu.h"
 #include "wui/stock_menu.h"
@@ -89,7 +92,8 @@ InfoToDraw filter_info_to_draw(InfoToDraw info_to_draw,
                                const Widelands::Player& player) {
 	InfoToDraw result = info_to_draw;
 	const Widelands::Player* owner = object->get_owner();
-	if (owner != nullptr && !player.see_all() && player.is_hostile(*owner)) {
+	if (owner != nullptr && !player.see_all() && player.is_hostile(*owner) &&
+	    object->descr().type() != Widelands::MapObjectType::WAREHOUSE) {
 		result = static_cast<InfoToDraw>(result & ~InfoToDraw::kStatistics);
 	}
 	return result;
@@ -101,6 +105,9 @@ void draw_bobs_for_visible_field(const Widelands::EditorGameBase& egbase,
                                  const InfoToDraw info_to_draw,
                                  const Widelands::Player& player,
                                  RenderTarget* dst) {
+	if (field.obscured_by_slope) {
+		return;
+	}
 	MutexLock m(MutexLock::ID::kObjects);
 	for (Widelands::Bob* bob = field.fcoords.field->get_first_bob(); bob != nullptr;
 	     bob = bob->get_next_bob()) {
@@ -114,7 +121,7 @@ void draw_immovable_for_formerly_visible_field(const FieldsToDraw::Field& field,
                                                const Widelands::Player::Field& player_field,
                                                const float scale,
                                                RenderTarget* dst) {
-	if (player_field.map_object_descr == nullptr) {
+	if (player_field.map_object_descr == nullptr || field.obscured_by_slope) {
 		return;
 	}
 
@@ -174,16 +181,15 @@ InteractivePlayer::InteractivePlayer(Widelands::Game& g,
                      "dropdown_menu_statistics",
                      0,
                      0,
-                     MainToolbar::kButtonSize,
+                     UI::main_toolbar_button_size(),
                      10,
-                     MainToolbar::kButtonSize,
+                     UI::main_toolbar_button_size(),
                      /** TRANSLATORS: Title for the statistics menu button in the game */
                      _("Statistics"),
                      UI::DropdownType::kPictorialMenu,
                      UI::PanelStyle::kWui,
                      UI::ButtonStyle::kWuiPrimary,
                      [this](StatisticsMenuEntry t) { statistics_menu_selected(t); }),
-     grid_marker_pic_(g_image_cache->get("images/wui/overlays/grid_marker.png")),
      portspace_hint_pic_(g_image_cache->get("images/wui/overlays/port_hint.png"))
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
      , training_wheel_indicator_pic_(g_image_cache->get("images/wui/training_wheels_arrow.png")),
@@ -214,13 +220,7 @@ InteractivePlayer::InteractivePlayer(Widelands::Game& g,
 	   &objectives_, true);
 	objectives_.open_window = [this] { new GameObjectivesMenu(*this, objectives_); };
 
-	add_toolbar_button(
-	   "wui/menus/diplomacy", "diplomacy",
-	   as_tooltip_text_with_hotkey(_("Diplomacy"),
-	                               shortcut_string_for(KeyboardShortcut::kInGameDiplomacy, true),
-	                               UI::PanelStyle::kWui),
-	   &diplomacy_, true);
-	diplomacy_.open_window = [this] { new GameDiplomacyMenu(*this, diplomacy_); };
+	add_diplomacy_menu();
 
 	toggle_message_menu_ = add_toolbar_button(
 	   "wui/menus/message_old", "messages",
@@ -247,24 +247,17 @@ InteractivePlayer::InteractivePlayer(Widelands::Game& g,
 		node_action(node_and_triangle);
 	});
 
+	add_plugin_menu();
+
 	finalize_toolbar();
 
-#ifndef NDEBUG  //  only in debug builds
-	addCommand(
-	   "switchplayer", [this](const std::vector<std::string>& str) { cmdSwitchPlayer(str); });
-#endif
+	if (g_allow_script_console) {
+		addCommand(
+		   "switchplayer", [this](const std::vector<std::string>& str) { cmdSwitchPlayer(str); });
+	}
 
 	map_options_subscriber_ = Notifications::subscribe<NoteMapOptions>(
 	   [this](const NoteMapOptions& /* note */) { rebuild_statistics_menu(); });
-	shipnotes_subscriber_ =
-	   Notifications::subscribe<Widelands::NoteShip>([this](const Widelands::NoteShip& note) {
-		   if (note.ship->owner().player_number() == player_number() &&
-		       note.action == Widelands::NoteShip::Action::kWaitingForCommand &&
-		       note.ship->get_ship_state() ==
-		          Widelands::Ship::ShipStates::kExpeditionPortspaceFound) {
-			   expedition_port_spaces_.emplace(note.ship, note.ship->exp_port_spaces().front());
-		   }
-	   });
 
 	initialization_complete();
 }
@@ -392,13 +385,32 @@ void InteractivePlayer::rebuild_showhide_menu() {
 	showhidemenu_.select(last_selection);
 }
 
-bool InteractivePlayer::has_expedition_port_space(const Widelands::Coords& coords) const {
-	for (const auto& pair : expedition_port_spaces_) {
-		if (pair.second == coords) {
-			return true;
+InteractivePlayer::HasExpeditionPortSpace
+InteractivePlayer::has_expedition_port_space(const Widelands::Coords& coords) const {
+	HasExpeditionPortSpace tmp_rv = HasExpeditionPortSpace::kNone;
+	for (auto serial : player().ships()) {
+		Widelands::Ship* ship = dynamic_cast<Widelands::Ship*>(egbase().objects().get_object(serial));
+		if (ship == nullptr) {
+			continue;
+		}
+		const Widelands::Coords portspace = ship->current_portspace();
+		if (!static_cast<bool>(portspace)) {
+			continue;
+		}
+		// Expeditions can only use the primary port space, show the others faded.
+		// Warships show all seen port spaces the same: either as usable if they are not engaged
+		// or faded otherwise.
+		if (portspace == coords && !ship->has_battle()) {
+			return HasExpeditionPortSpace::kPrimary;
+		}
+		if (ship->sees_portspace(coords)) {
+			if (ship->can_attack()) {
+				return HasExpeditionPortSpace::kPrimary;
+			}
+			tmp_rv = HasExpeditionPortSpace::kOther;
 		}
 	}
-	return false;
+	return tmp_rv;
 }
 
 void InteractivePlayer::draw_immovables_for_visible_field(
@@ -409,6 +421,9 @@ void InteractivePlayer::draw_immovables_for_visible_field(
    const Widelands::Player& player,
    RenderTarget* dst,
    std::set<Widelands::Coords>& deferred_coords) {
+	if (field.obscured_by_slope) {
+		return;
+	}
 	MutexLock m(MutexLock::ID::kObjects);
 
 	Widelands::BaseImmovable* const imm = field.fcoords.field->get_immovable();
@@ -432,9 +447,10 @@ void InteractivePlayer::draw_immovables_for_visible_field(
 }
 
 void InteractivePlayer::think() {
-	InteractiveBase::think();
+	InteractiveGameBase::think();
 
-	if (player().is_picking_custom_starting_position()) {
+	if (player().is_picking_custom_starting_position() &&
+	    !player().local_player_starting_position_is_pending()) {
 		set_sel_picture(
 		   playercolor_image(player().get_playercolor(), "images/players/player_position_menu.png"));
 	}
@@ -475,25 +491,6 @@ void InteractivePlayer::think() {
 		   msg_tooltip, shortcut_string_for(KeyboardShortcut::kInGameMessages, true),
 		   UI::PanelStyle::kWui));
 	}
-
-	// Cleanup found port spaces if the ship sailed on or was destroyed
-	for (auto it = expedition_port_spaces_.begin(); it != expedition_port_spaces_.end(); ++it) {
-		Widelands::Ship* ship = it->first.get(egbase());
-		if ((ship == nullptr) ||
-		    ship->get_ship_state() != Widelands::Ship::ShipStates::kExpeditionPortspaceFound) {
-			expedition_port_spaces_.erase(it);
-			// If another port space also needs removing, we'll take care of it in the next frame
-			return;
-		}
-	}
-
-	// Pop up diplomacy confirmation windows for new actions affecting us
-	for (const Widelands::Game::PendingDiplomacyAction& pda : game().pending_diplomacy_actions()) {
-		if (pda.other == player_number() && handled_diplomacy_actions_.count(&pda) == 0) {
-			handled_diplomacy_actions_.insert(&pda);
-			new DiplomacyConfirmWindow(*this, pda);
-		}
-	}
 }
 
 void InteractivePlayer::draw(RenderTarget& dst) {
@@ -506,7 +503,8 @@ void InteractivePlayer::draw(RenderTarget& dst) {
 	draw_map_view(map_view(), &dst);
 }
 
-constexpr float kBuildhelpOpacity = 0.3f;
+constexpr float kBuildhelpOpacityMedium = 0.6f;
+constexpr float kBuildhelpOpacityWeak = 0.3f;
 
 void InteractivePlayer::draw_map_view(MapView* given_map_view, RenderTarget* dst) {
 	// In-game, selection can never be on triangles or have a radius.
@@ -550,7 +548,7 @@ void InteractivePlayer::draw_map_view(MapView* given_map_view, RenderTarget* dst
 
 		// Add road building overlays if applicable.
 		if (f->seeing != Widelands::VisibleState::kUnexplored) {
-			draw_road_building(*f);
+			draw_road_building(dst, *f, gametime, scale);
 
 			draw_bridges(
 			   dst, f, f->seeing == Widelands::VisibleState::kVisible ? gametime : Time(0), scale);
@@ -575,19 +573,44 @@ void InteractivePlayer::draw_map_view(MapView* given_map_view, RenderTarget* dst
 		}
 
 		// Draw the player starting position overlays.
-		const bool suited_as_starting_pos =
-		   picking_starting_pos && plr.get_starting_position_suitability(f->fcoords);
-		if (suited_as_starting_pos) {
-			for (unsigned p = map.get_nrplayers(); p != 0u; --p) {
-				if (map.get_starting_pos(p) == f->fcoords) {
-					const Image* player_image =
-					   playercolor_image(p - 1, "images/players/player_position.png");
-					static constexpr int kStartingPosHotspotY = 55;
-					blit_field_overlay(dst, *f, player_image,
-					                   Vector2i(player_image->width() / 2, kStartingPosHotspotY), scale);
+		bool suited_as_starting_pos = false;
+		if (picking_starting_pos) {
+			static const std::string icon_filename = "images/players/player_position.png";
+			static constexpr int kStartingPosHotspotY = 55;
+
+			const Image* player_image = nullptr;
+			float icon_scale = 0.7f;
+			float icon_opacity = 1.0f;
+
+			// Not all map starting positions pass the suitability test.
+			// TODO(tothxa): Make the editor at least use the same test. But manual changes would still
+			//               be possible.
+			for (unsigned pn = map.get_nrplayers(); pn != 0u; --pn) {
+				if (map.get_starting_pos(pn) == f->fcoords) {
+					Widelands::Player* p = gbase.get_player(pn);
+					if (p == nullptr || !p->is_picking_custom_starting_position()) {
+						// Should have a HQ if finished picking, no need for the overlay
+						continue;
+					}
+					player_image = playercolor_image(p->get_playercolor(), icon_filename);
+					icon_scale = 1.0f;
+					icon_opacity = p->get_starting_position_suitability(f->fcoords) ?
+                                 kBuildhelpOpacityMedium :
+                                 kBuildhelpOpacityWeak;
 					break;
 				}
 			}
+
+			if (player_image == nullptr && plr.get_starting_position_suitability(f->fcoords)) {
+				player_image = g_image_cache->get(icon_filename);
+			}
+
+			if (player_image != nullptr) {
+				suited_as_starting_pos = true;
+				blit_field_overlay(dst, *f, player_image,
+				                   Vector2i(player_image->width() / 2, kStartingPosHotspotY),
+				                   scale * icon_scale, icon_opacity);
+			}
 		}
 
 		// Draw work area markers.
@@ -596,20 +619,32 @@ void InteractivePlayer::draw_map_view(MapView* given_map_view, RenderTarget* dst
 			                   Vector2i(grid_marker_pic_->width() / 2, grid_marker_pic_->height() / 2),
 			                   scale);
 		}
+		if (has_workarea_special_coords(f->fcoords)) {
+			blit_field_overlay(dst, *f, special_coords_marker_pic_,
+			                   Vector2i(special_coords_marker_pic_->width() / 2,
+			                            special_coords_marker_pic_->height() / 2),
+			                   scale);
+		}
 
 		if (f->seeing != Widelands::VisibleState::kUnexplored) {
 			// Draw build help.
-			const bool show_port_space = has_expedition_port_space(f->fcoords);
-			if (show_port_space || suited_as_starting_pos || buildhelp()) {
+			const HasExpeditionPortSpace show_port_space = map.is_port_space(f->fcoords) ?
+                                                           has_expedition_port_space(f->fcoords) :
+                                                           HasExpeditionPortSpace::kNone;
+			if (show_port_space != HasExpeditionPortSpace::kNone || buildhelp()) {
 				Widelands::NodeCaps caps;
 				Widelands::NodeCaps maxcaps = f->fcoords.field->maxcaps();
 				float opacity =
-				   f->seeing == Widelands::VisibleState::kVisible ? 1.f : kBuildhelpOpacity;
+				   f->seeing == Widelands::VisibleState::kVisible ? 1.f : kBuildhelpOpacityWeak;
 				if (picking_starting_pos) {
-					caps = suited_as_starting_pos || buildhelp() ? f->fcoords.field->nodecaps() :
-                                                              Widelands::CAPS_NONE;
-				} else if (show_port_space) {
+					caps = (show_port_space != HasExpeditionPortSpace::kNone || buildhelp()) ?
+                         f->fcoords.field->nodecaps() :
+                         Widelands::CAPS_NONE;
+				} else if (show_port_space != HasExpeditionPortSpace::kNone) {
 					caps = maxcaps;
+					if (show_port_space == HasExpeditionPortSpace::kOther) {
+						opacity *= kBuildhelpOpacityMedium;
+					}
 				} else {
 					caps = plr.get_buildcaps(f->fcoords);
 					if ((caps & Widelands::BUILDCAPS_SIZEMASK) == 0) {
@@ -617,16 +652,19 @@ void InteractivePlayer::draw_map_view(MapView* given_map_view, RenderTarget* dst
 						     plr.tribe().buildings_built_over_immovables()) {
 							if (plr.check_can_build(*b, f->fcoords)) {
 								caps = maxcaps;
-								opacity *= 2 * kBuildhelpOpacity;
+								opacity *= kBuildhelpOpacityMedium;
 								break;
 							}
 						}
 					}
 				}
 
-				const auto* overlay = get_buildhelp_overlay(caps);
-				if (overlay != nullptr) {
-					blit_field_overlay(dst, *f, overlay->pic, overlay->hotspot, scale, opacity);
+				const auto* overlay = get_buildhelp_overlay(caps, scale);
+				if (overlay != nullptr &&
+				    (!suited_as_starting_pos || (caps & Widelands::BUILDCAPS_PORT) != 0)) {
+					// draw overlay if not a starting pos, but draw port space anyway
+					blit_field_overlay(
+					   dst, *f, overlay->pic, overlay->hotspot, scale / overlay->scale, opacity);
 				}
 
 				// Draw port space hint if a port could be built here, but current situation doesn't
@@ -739,14 +777,12 @@ void InteractivePlayer::node_action(const Widelands::NodeAndTriangle<>& node_and
 				return;
 			}
 		}
-		if (show_attack_window(node_and_triangle.node, true) != nullptr) {
+
+		if (!in_road_building_mode() && try_show_ship_windows()) {
 			return;
 		}
-
-		if (!in_road_building_mode()) {
-			if (try_show_ship_window()) {
-				return;
-			}
+		if (show_attack_window(node_and_triangle.node, nullptr, true) != nullptr) {
+			return;
 		}
 
 		// everything else can bring up the temporary dialog
@@ -754,27 +790,84 @@ void InteractivePlayer::node_action(const Widelands::NodeAndTriangle<>& node_and
 	}
 }
 
-UI::Window* InteractivePlayer::show_attack_window(const Widelands::Coords& c,
-                                                  const bool fastclick) {
-	const Map& map = egbase().map();
-	if (Widelands::BaseImmovable* immo = map.get_immovable(c)) {
-		if (immo->descr().type() >= Widelands::MapObjectType::BUILDING) {
-			upcast(Building, building, immo);
-			assert(building != nullptr);
-			if (const Widelands::AttackTarget* attack_target = building->attack_target()) {
-				if (player().is_hostile(building->owner()) && attack_target->can_be_attacked()) {
-					UI::UniqueWindow::Registry& registry =
-					   unique_windows().get_registry(format("attack_%d", building->serial()));
-					registry.open_window = [this, &registry, building, &c, fastclick]() {
-						new AttackWindow(*this, registry, *building, c, fastclick);
-					};
-					registry.create();
-					return registry.window;
+UI::Window* InteractivePlayer::show_attack_window(const Widelands::Coords& coords,
+                                                  Widelands::MapObject* object,
+                                                  bool fastclick) {
+	if (object == nullptr) {
+		// Auto-detect what kind of attack window we need to show.
+		const Map& map = egbase().map();
+
+		for (auto serial : player().ships()) {
+			Widelands::Ship* ship =
+			   dynamic_cast<Widelands::Ship*>(egbase().objects().get_object(serial));
+			if (ship != nullptr && ship->get_ship_type() == Widelands::ShipType::kWarship &&
+			    ship->sees_portspace(coords)) {
+				UI::UniqueWindow::Registry& registry =
+				   unique_windows().get_registry(format("attack_coords_%d_%d", coords.x, coords.y));
+				registry.open_window = [this, &registry, &coords, fastclick]() {
+					new AttackWindow(*this, registry, nullptr, coords, fastclick);
+				};
+				registry.create();
+				return registry.window;
+			}
+		}
+
+		if (Widelands::BaseImmovable* immo = map.get_immovable(coords)) {
+			if (immo->descr().type() >= Widelands::MapObjectType::BUILDING) {
+				upcast(Building, building, immo);
+				assert(building != nullptr);
+				if (const Widelands::AttackTarget* attack_target = building->attack_target();
+				    attack_target != nullptr) {
+					if (player().is_hostile(building->owner()) && attack_target->can_be_attacked()) {
+						object = building;
+					}
 				}
 			}
 		}
+
+		for (Widelands::Bob* bob = map[coords].get_first_bob(); bob != nullptr && object == nullptr;
+		     bob = bob->get_next_bob()) {
+			if (bob->descr().type() == Widelands::MapObjectType::SHIP &&
+			    player().is_hostile(bob->owner()) &&
+			    dynamic_cast<Widelands::Ship*>(bob)->can_be_attacked()) {
+				object = bob;
+			}
+		}
+
+		if (object == nullptr) {
+			return nullptr;
+		}
 	}
-	return nullptr;
+
+	if (object->descr().type() >= Widelands::MapObjectType::BUILDING) {
+		upcast(Widelands::Building, building, object);
+		assert(building != nullptr);
+		assert(player().is_hostile(building->owner()));
+		assert(building->attack_target() != nullptr);
+		assert(building->attack_target()->can_be_attacked());
+
+		UI::UniqueWindow::Registry& registry =
+		   unique_windows().get_registry(format("attack_building_%u", building->serial()));
+		registry.open_window = [this, &registry, building, &coords, fastclick]() {
+			new AttackWindow(*this, registry, building, coords, fastclick);
+		};
+		registry.create();
+		return registry.window;
+	}
+
+	if (object->descr().type() == Widelands::MapObjectType::SHIP) {
+		assert(player().is_hostile(object->owner()));
+		assert(dynamic_cast<Widelands::Ship*>(object)->can_be_attacked());
+		UI::UniqueWindow::Registry& registry =
+		   unique_windows().get_registry(format("attack_ship_%u", object->serial()));
+		registry.open_window = [this, &registry, object, &coords, fastclick]() {
+			new AttackWindow(*this, registry, object, coords, fastclick);
+		};
+		registry.create();
+		return registry.window;
+	}
+
+	throw wexception("Attempting to show attack window for a %s", object->descr().name().c_str());
 }
 
 /**
@@ -812,10 +905,6 @@ bool InteractivePlayer::handle_key(bool const down, SDL_Keysym const code) {
 			objectives_.toggle();
 			return true;
 		}
-		if (matches_shortcut(KeyboardShortcut::kInGameDiplomacy, code)) {
-			diplomacy_.toggle();
-			return true;
-		}
 		if (matches_shortcut(KeyboardShortcut::kInGameStatsBuildings, code)) {
 			if (menu_windows_.stats_buildings.window == nullptr) {
 				new BuildingStatisticsMenu(*this, menu_windows_.stats_buildings);
@@ -852,6 +941,10 @@ bool InteractivePlayer::handle_key(bool const down, SDL_Keysym const code) {
 			   game().map().get_starting_pos(player_number_), MapView::Transition::Smooth);
 			return true;
 		}
+		if (matches_shortcut(KeyboardShortcut::kInGamePinnedNote, code)) {
+			edit_pinned_note(egbase().map().get_fcoords(get_sel_pos().node));
+			return true;
+		}
 
 		const Widelands::DescriptionIndex fastplace = egbase().descriptions().building_index(
 		   matching_fastplace_shortcut(code, player().tribe().name()));
@@ -864,6 +957,55 @@ bool InteractivePlayer::handle_key(bool const down, SDL_Keysym const code) {
 	return InteractiveGameBase::handle_key(down, code);
 }
 
+std::string InteractivePlayer::get_fastplace_help() const {
+	const Widelands::TribeDescr& tribe = player().tribe();
+	std::vector<FastplaceShortcut> fp_sc_v = get_active_fastplace_shortcuts(tribe.name());
+	if (fp_sc_v.empty()) {
+		return "";
+	}
+
+	std::string rv;
+	for (const FastplaceShortcut& fp_sc : fp_sc_v) {
+		const Widelands::DescriptionIndex bi = egbase().descriptions().building_index(fp_sc.building);
+		if (tribe.has_building(bi)) {
+			rv += as_definition_line(fp_sc.hotkey, tribe.get_building_descr(bi)->descname());
+		}
+	}
+
+	if (rv.empty()) {
+		return "";
+	}
+	return as_paragraph_style(UI::ParagraphStyle::kWuiHeading2, _("Fastplace Shortcuts")) + rv;
+}
+
+void InteractivePlayer::edit_pinned_note(const Widelands::FCoords& c) {
+	std::string text;
+	const RGBColor* rgb = &player().get_playercolor();
+	bool exists = false;
+
+	for (Widelands::Bob* b = c.field->get_first_bob(); b != nullptr; b = b->get_next_bob()) {
+		if (b->descr().type() == Widelands::MapObjectType::PINNED_NOTE &&
+		    b->owner().player_number() == player_number()) {
+			exists = true;
+			const Widelands::PinnedNote& pn = dynamic_cast<Widelands::PinnedNote&>(*b);
+			text = pn.get_text();
+			rgb = &pn.get_rgb();
+			break;
+		}
+	}
+
+	UI::UniqueWindow::Registry& r =
+	   unique_windows().get_registry(format("pinned_note_%d_%d", c.x, c.y));
+	r.open_window = [this, c, &r, text, rgb, exists] {
+		new PinnedNoteEditor(*this, r, c, text, *rgb, !exists);
+	};
+	r.create();
+
+	if (!exists) {  // Already create the note if it did not exist yet.
+		game().send_player_pinned_note(player_number(), c, text, *rgb, false);
+	}
+}
+
 /**
  * Set the player and the visibility to this
  * player
@@ -893,6 +1035,10 @@ bool InteractivePlayer::player_hears_field(const Widelands::Coords& coords) cons
 }
 
 void InteractivePlayer::cmdSwitchPlayer(const std::vector<std::string>& args) {
+	if (!g_allow_script_console) {
+		throw wexception("Trying to switch player when the Script Console is disabled.");
+	}
+
 	if (args.size() != 2) {
 		DebugConsole::write("Usage: switchplayer <nr>");
 		return;
@@ -900,13 +1046,17 @@ void InteractivePlayer::cmdSwitchPlayer(const std::vector<std::string>& args) {
 
 	int const n = stoi(args[1]);
 	if (n < 1 || n > kMaxPlayers || (game().get_player(n) == nullptr)) {
+		broadcast_cheating_message();
 		DebugConsole::write(format("Player #%d does not exist.", n));
 		return;
 	}
 
 	DebugConsole::write(format("Switching from #%d to #%d.", static_cast<int>(player_number_), n));
+	broadcast_cheating_message("SWITCHED_PLAYER", game().get_player(n)->get_name());
+
 	player_number_ = n;
 
+	// TODO(tothxa): All statistics windows need updates, not just these 2
 	if (UI::UniqueWindow* const building_statistics_window = menu_windows_.stats_buildings.window) {
 		dynamic_cast<BuildingStatisticsMenu&>(*building_statistics_window).update();
 	}
