@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,7 +20,9 @@
 
 #include <memory>
 
+#include "build_info.h"
 #include "logic/filesystem_constants.h"
+#include "ui_basic/messagebox.h"
 
 LoadOrSaveGame::LoadOrSaveGame(UI::Panel* parent,
                                Widelands::Game& g,
@@ -30,16 +32,20 @@ LoadOrSaveGame::LoadOrSaveGame(UI::Panel* parent,
                                bool localize_autosave,
                                UI::Panel* table_parent,
                                UI::Panel* delete_button_parent)
-   : table_box_(new UI::Box(
-        table_parent != nullptr ? table_parent : parent, style, 0, 0, UI::Box::Vertical)),
+   : window_style_(ws),
+     table_box_(new UI::Box(table_parent != nullptr ? table_parent : parent,
+                            style,
+                            "table_box",
+                            0,
+                            0,
+                            UI::Box::Vertical)),
      filetype_(filetype),
 
      // Savegame description
      game_details_(
         parent,
         style,
-        filetype == FileType::kReplay ? GameDetails::Mode::kReplay : GameDetails::Mode::kSavegame,
-        g),
+        filetype == FileType::kReplay ? GameDetails::Mode::kReplay : GameDetails::Mode::kSavegame),
      delete_(new UI::Button(
         delete_button_parent != nullptr ? delete_button_parent : game_details()->button_box(),
         "delete",
@@ -97,13 +103,8 @@ LoadOrSaveGame::LoadOrSaveGame(UI::Panel* parent,
 
 bool LoadOrSaveGame::selection_contains_directory() const {
 	const std::set<uint32_t>& selections = table_->selections();
-	for (const uint32_t index : selections) {
-		const SavegameData& gamedata = get_savegame(index);
-		if (gamedata.is_directory()) {
-			return true;
-		}
-	}
-	return false;
+	return std::any_of(selections.begin(), selections.end(),
+	                   [this](const uint32_t index) { return get_savegame(index).is_directory(); });
 }
 
 const SavegameData& LoadOrSaveGame::get_savegame(uint32_t index) const {
@@ -129,6 +130,28 @@ bool LoadOrSaveGame::compare_map_name(uint32_t rowa, uint32_t rowb) const {
 	return get_savegame(rowa).compare_map_name(get_savegame(rowb));
 }
 
+/**
+ * Check whether the given file is a replay that is potentially incompatible with the
+ * current Widelands version. If so, show a message box to ask the user how to proceed.
+ * @param sd File data to investigate.
+ * @return \c false if the user has aborted loading the incompatible replay; \c true if it is not
+ * a replay at all or the replay is compatible or the user wishes to ignore the incompatibility.
+ */
+bool LoadOrSaveGame::check_replay_compatibility(const SavegameData& sd) {
+	if (filetype_ != FileType::kReplay || sd.is_directory() || !sd.errormessage.empty() ||
+	    (SDL_GetModState() & KMOD_CTRL) != 0 || sd.version == build_id()) {
+		return true;
+	}
+
+	UI::WLMessageBox w(&game_details_.get_topmost_forefather(), window_style_, _("Version Mismatch"),
+	                   _("This replay was created with a different Widelands version. It "
+	                     "might be compatible, but will more likely desync or even fail to "
+	                     "load.\n\nPlease do not report any bugs that occur while watching "
+	                     "this replay.\n\nDo you want to load the replay anyway?"),
+	                   UI::WLMessageBox::MBoxType::kOkCancel);
+	return w.run<UI::Panel::Returncodes>() == UI::Panel::Returncodes::kOk;
+}
+
 std::unique_ptr<SavegameData> LoadOrSaveGame::entry_selected() {
 	std::unique_ptr<SavegameData> result(new SavegameData());
 
