@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,13 +40,14 @@ MapDetails::MapDetails(Panel* parent,
                        int32_t h,
                        UI::PanelStyle style,
                        Widelands::EditorGameBase& egbase)
-   : UI::Panel(parent, style, x, y, w, h),
+   : UI::Panel(parent, style, "map_details", x, y, w, h),
 
      style_(style),
-     padding_(4),
-     main_box_(this, style, 0, 0, UI::Box::Vertical, 0, 0, 0),
-     descr_box_(&main_box_, style, 0, 0, UI::Box::Vertical, 0, 0, 0),
+
+     main_box_(this, style, "main_box", 0, 0, UI::Box::Vertical, 0, 0, 0),
+     descr_box_(&main_box_, style, "description_box", 0, 0, UI::Box::Vertical, 0, 0, 0),
      name_label_(&main_box_,
+                 "label_name",
                  0,
                  0,
                  UI::Scrollbar::kSize,
@@ -56,6 +57,7 @@ MapDetails::MapDetails(Panel* parent,
                  UI::Align::kLeft,
                  UI::MultilineTextarea::ScrollMode::kNoScrolling),
      descr_(&descr_box_,
+            "description",
             0,
             0,
             UI::Scrollbar::kSize,
@@ -64,7 +66,7 @@ MapDetails::MapDetails(Panel* parent,
             "",
             UI::Align::kLeft,
             UI::MultilineTextarea::ScrollMode::kNoScrolling),
-     minimap_icon_(&descr_box_, style, 0, 0, 0, 0, nullptr),
+     minimap_icon_(&descr_box_, style, "minimap", 0, 0, 0, 0, nullptr),
      suggested_teams_box_(
         new UI::SuggestedTeamsBox(this, style, 0, 0, UI::Box::Vertical, padding_, 0)),
      egbase_(egbase) {
@@ -116,13 +118,13 @@ void MapDetails::layout() {
 	const int full_height = descr_.get_h() + minimap_icon_.get_h();
 	const int descr_height = main_box_.get_h() - name_label_.get_h() - padding_;
 	descr_box_.set_force_scrolling(full_height > descr_height);
-	descr_box_.set_size(main_box_.get_w(), descr_height);
+	descr_box_.set_desired_size(0, 0);
 }
 
 bool MapDetails::update(const MapData& mapdata, bool localize_mapname, bool render_minimap) {
 	clear();
 	name_ = mapdata.name;
-	last_map_ = mapdata.filename;
+	last_map_ = mapdata.filenames.at(0);
 	bool loadable = true;
 	// Show directory information
 	if (mapdata.maptype == MapData::MapType::kDirectory) {
@@ -164,32 +166,53 @@ bool MapDetails::update(const MapData& mapdata, bool localize_mapname, bool rend
                forms here, please let us know. */
                _("Authors");
 		std::string description = as_heading(authors_heading, style_);
-		description = format("%s%s", description, as_content(mapdata.authors.get_names(), style_));
+		description += as_content(mapdata.authors.get_names(), style_);
 
 		std::vector<std::string> tags;
 		for (const auto& tag : mapdata.tags) {
-			tags.push_back(localize_tag(tag));
+			tags.push_back(localize_tag(tag).displayname);
 		}
 		std::sort(tags.begin(), tags.end());
-		description = format("%s%s", description, as_heading(_("Tags"), style_));
-		description =
-		   format("%s%s", description,
-		          as_content(i18n::localize_list(tags, i18n::ConcatenateWith::COMMA), style_));
+		description += as_heading(_("Tags"), style_);
+		description += as_content(i18n::localize_list(tags, i18n::ConcatenateWith::COMMA), style_);
 
 		AddOns::AddOnConflict addons = AddOns::check_requirements(mapdata.required_addons);
 		loadable = !addons.second;
 
-		description =
-		   format("%s%s", description,
-		          as_heading_with_content(_("Add-Ons:"), addons.first, style_, false, true));
+		description += as_heading_with_content(_("Add-Ons:"), addons.first, style_, false, true);
 
-		description = format("%s%s", description, as_heading(_("Description"), style_));
-		description = format("%s%s", description, as_content(mapdata.description, style_));
+		if (!mapdata.minimum_required_widelands_version.empty()) {
+			bool compatible;
+			try {
+				compatible = AddOns::matches_widelands_version(
+				   mapdata.minimum_required_widelands_version, std::string());
+			} catch (const std::exception& e) {
+				compatible = false;
+				log_warn("Could not parse map version requirement '%s': %s",
+				         mapdata.minimum_required_widelands_version.c_str(), e.what());
+			}
+
+			loadable &= compatible;
+
+			if (compatible) {
+				description += as_heading_with_content(_("Minimum Widelands Version:"),
+				                                       mapdata.minimum_required_widelands_version,
+				                                       style_, false, true);
+			} else {
+				description += as_heading_with_content(
+				   _("Minimum Widelands Version:"),
+				   as_font_tag(UI::FontStyle::kWarning, mapdata.minimum_required_widelands_version),
+				   style_, false, true);
+			}
+		}
+
+		description += as_heading(_("Description"), style_);
+		description += as_content(mapdata.description, style_);
 
 		if (!mapdata.hint.empty()) {
 			/** TRANSLATORS: Map hint header when selecting a map. */
-			description = format("%s%s", description, as_heading(_("Hint"), style_));
-			description = format("%s%s", description, as_content(mapdata.hint, style_));
+			description += as_heading(_("Hint"), style_);
+			description += as_content(mapdata.hint, style_);
 		}
 
 		// Render minimap
@@ -201,13 +224,13 @@ bool MapDetails::update(const MapData& mapdata, bool localize_mapname, bool rend
 			} else {
 				egbase_.cleanup_for_load();
 				std::unique_ptr<Widelands::MapLoader> ml(
-				   egbase_.mutable_map()->get_correct_loader(mapdata.filename));
+				   egbase_.mutable_map()->get_correct_loader(mapdata.filenames.at(0)));
 				try {
 					if (ml != nullptr &&
 					    0 == ml->load_map_for_render(egbase_, &egbase_.enabled_addons())) {
-						minimap_cache_[last_map_] = draw_minimap(
-						   egbase_, nullptr, Rectf(), MiniMapType::kStaticMap,
-						   MiniMapLayer::Terrain | MiniMapLayer::StartingPositions | MiniMapLayer::Owner);
+						minimap_cache_[last_map_] =
+						   draw_minimap(egbase_, nullptr, Rectf(), MiniMapType::kStaticMap,
+						                MiniMapLayer::Terrain | MiniMapLayer::StartingPositions);
 						minimap_icon_.set_icon(minimap_cache_.at(last_map_).get());
 						minimap_icon_.set_visible(true);
 					}
