@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -81,7 +81,7 @@ public:
 	   : start_(start), end_(end), dt_(dt) {
 	}
 
-	T value(const float time_ms) const {
+	[[nodiscard]] T value(const float time_ms) const {
 		const float t = math::clamp(time_ms / dt_, 0.f, 1.f);
 		return mix(math::sqr(t) * (3.f - 2.f * t), start_, end_);
 	}
@@ -101,7 +101,7 @@ public:
 	   : first_(start, middle, dt / 2.f), second_(middle, end, dt / 2.f), dt_(dt) {
 	}
 
-	T value(const float time_ms) const {
+	[[nodiscard]] T value(const float time_ms) const {
 		const float t = math::clamp(time_ms / dt_, 0.f, 1.f);
 		if (t < 0.5f) {
 			return first_.value(t * dt_);
@@ -315,17 +315,15 @@ bool MapView::View::view_roughly_near(const View& other) const {
 
 MapView::MapView(
    UI::Panel* parent, const Widelands::Map& map, int32_t x, int32_t y, uint32_t w, uint32_t h)
-   : UI::Panel(parent, UI::PanelStyle::kWui, x, y, w, h),
+   : UI::Panel(parent, UI::PanelStyle::kWui, "mapview", x, y, w, h),
      animate_map_panning_(get_config_bool("animate_map_panning", true)),
      map_(map),
      last_mouse_pos_(Vector2i::zero()),
-     dragging_(false),
+
      edge_scrolling_((parent != nullptr) &&
                      (parent->get_parent() == nullptr) /* not in watch windows */ &&
                      get_config_bool("edge_scrolling", false)),
-     invert_movement_(get_config_bool("invert_movement", false)),
-     is_scrolling_x_(0),
-     is_scrolling_y_(0) {
+     invert_movement_(get_config_bool("invert_movement", false)) {
 }
 
 Vector2f MapView::to_panel(const Vector2f& map_pixel) const {
@@ -455,6 +453,12 @@ void MapView::set_view(const View& target_view, const Transition& passed_transit
 	}
 }
 
+void MapView::set_centered_view(const View& centered_view, const Transition& transition) {
+	const Vector2f viewpoint = MapviewPixelFunctions::panel_to_map(
+	   centered_view.viewpoint, centered_view.zoom, Vector2f(-get_w() / 2.f, -get_h() / 2.f));
+	set_view(View(viewpoint, centered_view.zoom), transition);
+}
+
 void MapView::scroll_to_field(const Widelands::Coords& c, const Transition& transition) {
 	assert(0 <= c.x);
 	assert(c.x < map_.get_width());
@@ -481,6 +485,12 @@ const MapView::View& MapView::view() const {
 	return view_;
 }
 
+MapView::View MapView::get_centered_view() const {
+	const Vector2f center = MapviewPixelFunctions::panel_to_map(
+	   view_.viewpoint, view_.zoom, Vector2f(get_w() / 2.f, get_h() / 2.f));
+	return View(center, view_.zoom);
+}
+
 void MapView::pan_by(Vector2i delta_pixels, const Transition& transition) {
 	if (is_animating() || map_.get_width() == 0 || map_.get_height() == 0) {
 		return;
@@ -489,9 +499,11 @@ void MapView::pan_by(Vector2i delta_pixels, const Transition& transition) {
 }
 
 void MapView::stop_dragging() {
-	WLApplication::get()->set_mouse_lock(false);
-	grab_mouse(false);
-	dragging_ = false;
+	if (dragging_) {
+		WLApplication::get().set_mouse_lock(false);
+		grab_mouse(false);
+		dragging_ = false;
+	}
 }
 
 bool MapView::handle_mousepress(uint8_t const btn, int32_t const x, int32_t const y) {
@@ -500,13 +512,13 @@ bool MapView::handle_mousepress(uint8_t const btn, int32_t const x, int32_t cons
 		const auto node_and_triangle = track_sel(Vector2i(x, y));
 		field_clicked(node_and_triangle);
 		// Do not return true, because we want to give our parent a chance to
-		// also handle the click.
+		// also handle the click. Painting mode in the editor needs it.
 	}
 	if (btn == SDL_BUTTON_RIGHT) {
 		jump();
 		dragging_ = true;
 		grab_mouse(true);
-		WLApplication::get()->set_mouse_lock(true);
+		WLApplication::get().set_mouse_lock(true);
 		return true;
 	}
 	return false;
@@ -584,8 +596,8 @@ bool MapView::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
 		if (is_animating()) {
 			return true;
 		}
-		const uint16_t scroll_distance_y = g_gr->get_yres() / 20;
-		const uint16_t scroll_distance_x = g_gr->get_xres() / 20;
+		const uint16_t scroll_distance_y = get_h() / 20;
+		const uint16_t scroll_distance_x = get_w() / 20;
 		pan_by(Vector2i(change_2d.x * scroll_distance_x, change_2d.y * scroll_distance_y),
 		       Transition::Jump);
 		return true;
@@ -695,8 +707,8 @@ bool MapView::scroll_map() {
 #undef kNP
 
 	   // set the scrolling distance
-	   const uint16_t xres = g_gr->get_xres();
-	const uint16_t yres = g_gr->get_yres();
+	   const uint16_t xres = get_w();
+	const uint16_t yres = get_h();
 
 	uint16_t scroll_distance_x = xres / 8;
 	uint16_t scroll_distance_y = yres / 8;
