@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,12 +31,13 @@ class MiniMap : public UI::UniqueWindow {
 public:
 	struct Registry : public UI::UniqueWindow::Registry {
 		MiniMapLayer minimap_layers;
-		MiniMapType minimap_type;
+		MiniMapType minimap_type{MiniMapType::kStaticViewWindow};
 
-		Registry()
-		   : minimap_layers(MiniMapLayer::Terrain | MiniMapLayer::Owner | MiniMapLayer::Flag |
-		                    MiniMapLayer::Road | MiniMapLayer::Building),
-		     minimap_type(MiniMapType::kStaticViewWindow) {
+		explicit Registry(bool is_game)
+		   : minimap_layers(MiniMapLayer::Terrain | MiniMapLayer::Flag | MiniMapLayer::Road |
+		                    MiniMapLayer::Building | MiniMapLayer::Ship | MiniMapLayer::Attack |
+		                    (is_game ? MiniMapLayer::Owner | MiniMapLayer::Traffic :
+                                     MiniMapLayer::StartingPositions | MiniMapLayer::Artifacts)) {
 		}
 
 		MiniMap* get_window() const {
@@ -104,7 +105,7 @@ private:
 
 		// Intermediate texture, cached between frames.
 		std::unique_ptr<Texture> minimap_image_static_;
-		uint16_t rows_drawn_;
+		uint16_t rows_drawn_{0U};
 
 		// This needs to be owned since it will be rendered by the RenderQueue
 		// later, so it must be valid for the whole frame.
@@ -121,13 +122,17 @@ private:
 	uint32_t but_h() const;
 
 	InteractiveBase& ibase_;
+	MiniMapLayer owner_button_impl_;
+	MiniMapLayer additional_button_impl_;
 	View view_;
-	UI::Button button_terrn;
+	UI::Button button_terrain;
 	UI::Button button_owner;
 	UI::Button button_flags;
 	UI::Button button_roads;
-	UI::Button button_bldns;
+	UI::Button button_buildings;
 	UI::Button button_ships;
+	UI::Button button_traffic_artifacts;
+	UI::Button button_attack;
 	UI::Button button_zoom;
 };
 
