@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -317,24 +317,19 @@ void draw_diagram(uint32_t time_ms,
 }  // namespace
 
 WuiPlotArea::WuiPlotArea(UI::Panel* const parent,
+                         const std::string& name,
                          int32_t const x,
                          int32_t const y,
                          int32_t const w,
                          int32_t const h,
                          uint32_t sample_rate,
                          Plotmode plotmode)
-   : UI::Panel(parent, UI::PanelStyle::kWui, x, y, w, h),
+   : UI::Panel(parent, UI::PanelStyle::kWui, name, x, y, w, h),
      plotmode_(plotmode),
      sample_rate_(sample_rate),
-     needs_update_(true),
-     lastupdate_(0),
+
      xline_length_(get_inner_w() - kSpaceRight - kSpacing),
-     yline_length_(get_inner_h() - kSpaceBottom - kSpacing * 3),
-     time_ms_(0),
-     highest_scale_(0),
-     sub_(0),
-     time_(TIME_GAME),
-     game_time_id_(0) {
+     yline_length_(get_inner_h() - kSpaceBottom - kSpacing * 3) {
 	update();
 }
 
@@ -632,13 +627,14 @@ void WuiPlotAreaSlider::draw(RenderTarget& dst) {
 }
 
 DifferentialPlotArea::DifferentialPlotArea(UI::Panel* const parent,
+                                           const std::string& name,
                                            int32_t const x,
                                            int32_t const y,
                                            int32_t const w,
                                            int32_t const h,
                                            uint32_t sample_rate,
                                            Plotmode plotmode)
-   : WuiPlotArea(parent, x, y, w, h, sample_rate, plotmode) {
+   : WuiPlotArea(parent, name, x, y, w, h, sample_rate, plotmode) {
 	update();
 }
 
