@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,40 +38,35 @@ Create the window and its panels, add it to the registry.
 ===============
 */
 ProductionSiteWindow::ProductionSiteWindow(InteractiveBase& parent,
-                                           UI::UniqueWindow::Registry& reg,
+                                           BuildingWindow::Registry& reg,
                                            Widelands::ProductionSite& ps,
                                            bool avoid_fastclick,
                                            bool workarea_preview_wanted)
-   : BuildingWindow(parent, reg, ps, avoid_fastclick),
-     production_site_(&ps),
-     worker_table_(nullptr),
-     worker_caps_(nullptr),
-     worker_type_(nullptr),
-     worker_xp_decrease_(nullptr),
-     worker_xp_increase_(nullptr) {
-	productionsitenotes_subscriber_ = Notifications::subscribe<Widelands::NoteBuilding>(
-	   [this](const Widelands::NoteBuilding& note) {
-		   if (is_dying_) {
-			   return;
-		   }
-		   Widelands::ProductionSite* production_site = production_site_.get(ibase()->egbase());
-		   if (production_site == nullptr) {
-			   return;
-		   }
-		   if (note.serial == production_site->serial()) {
-			   switch (note.action) {
-			   case Widelands::NoteBuilding::Action::kWorkersChanged:
-				   update_worker_table(production_site);
-				   worker_table_selection_changed();
-				   break;
-			   default:
-				   break;
-			   }
-		   }
-	   });
+   : BuildingWindow(parent, reg, ps, avoid_fastclick), production_site_(&ps) {
 	init(avoid_fastclick, workarea_preview_wanted);
 }
 
+void ProductionSiteWindow::on_building_note(const Widelands::NoteBuilding& note) {
+	if (is_dying_) {
+		return;
+	}
+	Widelands::ProductionSite* production_site = production_site_.get(ibase()->egbase());
+	if (production_site == nullptr || note.serial != production_site->serial()) {
+		return;
+	}
+
+	switch (note.action) {
+	case Widelands::NoteBuilding::Action::kWorkersChanged:
+		update_worker_table(production_site);
+		worker_table_selection_changed();
+		break;
+	default:
+		break;
+	}
+
+	BuildingWindow::on_building_note(note);
+}
+
 void ProductionSiteWindow::init(bool avoid_fastclick, bool workarea_preview_wanted) {
 	Widelands::ProductionSite* production_site = production_site_.get(ibase()->egbase());
 	assert(production_site != nullptr);
@@ -81,14 +76,15 @@ void ProductionSiteWindow::init(bool avoid_fastclick, bool workarea_preview_want
 
 	if (!inputqueues.empty()) {
 		// Add the wares tab
-		UI::Box* prod_box = new UI::Box(get_tabs(), UI::PanelStyle::kWui, 0, 0, UI::Box::Vertical,
-		                                g_gr->get_xres() - 80, g_gr->get_yres() - 80);
+		UI::Box* prod_box =
+		   new UI::Box(get_tabs(), UI::PanelStyle::kWui, "inputs_box", 0, 0, UI::Box::Vertical,
+		               g_gr->get_xres() - 80, g_gr->get_yres() - 80);
 		ensure_box_can_hold_input_queues(*prod_box);
 
 		for (Widelands::InputQueue* queue : inputqueues) {
-			prod_box->add(
-			   new InputQueueDisplay(prod_box, *ibase(), *production_site, *queue, false, true),
-			   UI::Box::Resizing::kFullSize);
+			prod_box->add(new InputQueueDisplay(prod_box, *ibase(), *production_site, *queue, false,
+			                                    true, priority_collapsed()),
+			              UI::Box::Resizing::kFullSize);
 		}
 
 		get_tabs()->add("wares", g_image_cache->get(pic_tab_wares), prod_box, _("Wares"));
@@ -98,9 +94,12 @@ void ProductionSiteWindow::init(bool avoid_fastclick, bool workarea_preview_want
 	if (production_site->descr().nr_working_positions() == 0u) {
 		worker_table_ = nullptr;
 	} else {
-		UI::Box* worker_box = new UI::Box(get_tabs(), UI::PanelStyle::kWui, 0, 0, UI::Box::Vertical);
-		worker_table_ = new UI::Table<uintptr_t>(worker_box, 0, 0, 0, 100, UI::PanelStyle::kWui);
-		worker_caps_ = new UI::Box(worker_box, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal);
+		UI::Box* worker_box =
+		   new UI::Box(get_tabs(), UI::PanelStyle::kWui, "workers_box", 0, 0, UI::Box::Vertical);
+		worker_table_ =
+		   new UI::Table<uintptr_t>(worker_box, "workers_table", 0, 0, 0, 100, UI::PanelStyle::kWui);
+		worker_caps_ = new UI::Box(
+		   worker_box, UI::PanelStyle::kWui, "worker_caps_box", 0, 0, UI::Box::Horizontal);
 
 		const std::string workers_heading =
 		   (production_site->descr().nr_working_positions() == 1) ?
@@ -243,7 +242,9 @@ void ProductionSiteWindow::update_worker_table(Widelands::ProductionSite* produc
 		} else if (request != nullptr) {
 			const Widelands::WorkerDescr* desc =
 			   production_site->owner().tribe().get_worker_descr(request->get_index());
-			er.set_picture(0, desc->icon(), request->is_open() ? _("(vacant)") : _("(coming)"));
+			er.set_picture(0, desc->icon(),
+			               format(_("%1$s (%2$s)"), request->is_open() ? _("vacant") : _("coming"),
+			                      desc->descname()));
 
 			er.set_string(1, "");
 			er.set_string(2, "");
@@ -260,16 +261,18 @@ void ProductionSiteWindow::evict_worker() {
 		return;
 	}
 
-	if (worker_table_->has_selection()) {
-		Widelands::Worker* worker = production_site->working_positions()
-		                               ->at(worker_table_->get_selected())
-		                               .worker.get(ibase()->egbase());
-		if (worker != nullptr) {
-			if (game_ != nullptr) {
-				game_->send_player_evict_worker(*worker);
-			} else {
-				NEVER_HERE();  // TODO(Nordfriese / Scenario Editor): implement
-			}
+	if (!worker_table_->has_selection()) {
+		worker_table_->select(0);
+	}
+
+	Widelands::Worker* worker = production_site->working_positions()
+	                               ->at(worker_table_->get_selected())
+	                               .worker.get(ibase()->egbase());
+	if (worker != nullptr) {
+		if (game_ != nullptr) {
+			game_->send_player_evict_worker(*worker);
+		} else {
+			NEVER_HERE();  // TODO(Nordfriese / Scenario Editor): implement
 		}
 	}
 }
