@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2024 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,7 +26,8 @@
 #include "logic/filesystem_constants.h"
 
 SavegameTable::SavegameTable(UI::Panel* parent, UI::PanelStyle style, bool localize_autosave)
-   : UI::Table<uintptr_t>(parent, 0, 0, 0, 0, style, UI::TableRows::kMultiDescending),
+   : UI::Table<uintptr_t>(
+        parent, "savegame_table", 0, 0, 0, 0, style, UI::TableRows::kMultiDescending),
      tooltip_style_(style == UI::PanelStyle::kWui ? UI::FontStyle::kWuiTooltip :
                                                     UI::FontStyle::kFsTooltip),
      tooltip_header_style_(style == UI::PanelStyle::kWui ? UI::FontStyle::kWuiTooltipHeader :
@@ -126,7 +127,7 @@ void SavegameTable::create_directory_entry(UI::Table<const uintptr_t>::EntryReco
 		               format("<%s>", _("parent")));
 	} else if (savegame.is_sub_directory()) {
 		te.set_picture(last_column_index, g_image_cache->get("images/ui_basic/ls_dir.png"),
-		               FileSystem::filename_without_ext(savegame.filename.c_str()));
+		               FileSystem::fs_filename(savegame.filename.c_str()));
 	}
 }
 
@@ -166,16 +167,16 @@ void SavegameTableSinglePlayer::create_valid_entry(UI::Table<uintptr_t const>::E
 SavegameTableReplay::SavegameTableReplay(UI::Panel* parent,
                                          UI::PanelStyle style,
                                          bool localize_autosave)
-   : SavegameTable(parent, style, localize_autosave), show_filenames_(false) {
+   : SavegameTable(parent, style, localize_autosave) {
 	add_columns();
 }
 
 void SavegameTableReplay::add_columns() {
 	add_column(130, _("Save Date"), _("The date this game was saved"), UI::Align::kLeft);
-	std::string game_mode_tooltip;
-	/** TRANSLATORS: Tooltip header for the "Mode" column when choosing a game/replay to
-	load.*/
-	g_style_manager->font_style(tooltip_header_style_).as_font_tag(_("Game Mode"));
+	std::string game_mode_tooltip =
+	   /** TRANSLATORS: Tooltip header for the "Mode" column when choosing a game/replay to
+	       load. */
+	   g_style_manager->font_style(tooltip_header_style_).as_font_tag(_("Game Mode"));
 
 	/** TRANSLATORS: Tooltip for the "Mode" column when choosing a game/replay to load. */
 	/** TRANSLATORS: Make sure that you keep consistency in your translation. */
@@ -235,10 +236,10 @@ SavegameTableMultiplayer::SavegameTableMultiplayer(UI::Panel* parent,
 
 void SavegameTableMultiplayer::add_columns() {
 	add_column(130, _("Save Date"), _("The date this game was saved"), UI::Align::kLeft);
-	std::string game_mode_tooltip;
-	/** TRANSLATORS: Tooltip header for the "Mode" column when choosing a game/replay to
-	load.*/
-	g_style_manager->font_style(tooltip_header_style_).as_font_tag(_("Game Mode"));
+	std::string game_mode_tooltip =
+	   /** TRANSLATORS: Tooltip header for the "Mode" column when choosing a game/replay to
+	       load. */
+	   g_style_manager->font_style(tooltip_header_style_).as_font_tag(_("Game Mode"));
 
 	/** TRANSLATORS: Tooltip for the "Mode" column when choosing a game/replay to load. */
 	/** TRANSLATORS: Make sure that you keep consistency in your translation. */
